//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_Object.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Objects/Tools_Mode_Object.h"

// asura
#ifdef ASURA_USE_AXON
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#endif
#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Asura_StaticMessage_System.h"

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Asura_ServerEntity_Container.h"
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#include "Entity/EffectVolume/Tools_Entity_EffectVolume.h"
#include "Entity/Triggers/Asura_ServerEntity_WaypointBlockerVolume.h"
#include "Entity/Triggers/Asura_ServerEntity_ProxyTrigger.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Interface.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Text_Output.h"
#include "Shapes/Asura_Shape_System.h"

// tools
#include "Tools_Overlays.h"
#ifdef ASURA_USE_AXON
#include "AnimTemplates/Tools_Mode_AnimTemplates.h"
#endif
#ifdef ASURA_USE_ATTRACTORS
#include "Attractors/Tools_Object_Attractor_Node.h"
#endif
#include "Cutscene/Tools_Mode_Cutscene.h"
#if defined(ASURA_USE_AFFECTORS)
#include "Entity/Tools_Entity_Affector.h"
#include "Entity/Tools_Entity_Affector_Wind.h"
#endif	// defined(ASURA_USE_AFFECTORS)
#include "Entity/Tools_Entity_Data.h"
#include "Entity/CameraVolume/Tools_Entity_CameraVolume.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Intangible/Tools_Entity_ConsoleVar.h"
#endif //ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Sound/Tools_Entity_DialogueController.h"
#include "Entity/Tools_Entity_CountedVolumeTrigger.h"
#include "Entity/Tools_Entity_EnvTextureAnimControl.h"
#include "Entity/Tools_Entity_Simulation.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Entity/Tools_Entity_MusicTrigger_Object.h"
#include "Entity/Tools_Entity_MetaMusicTrigger_Object.h"

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
#include "Entity/Tools_Entity_FMVController.h"
#include "Entity/Tools_Entity_FMVSpeaker.h"
#endif

#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
#include "Entity/Tools_MessageEditor.h"
#endif

#include "Entity/Tools_Entity_FMVTrigger_Object.h"
#include "Entity/Tools_Entity_AdvancedVolumeTrigger.h"
#include "Entity/Tools_Entity_ClockTrigger_Object.h"
#include "Entity/Tools_Entity_CountedTrigger_Object.h"
#ifndef ASURA_TARGET_SM3
#include "Entity/Tools_Entity_AdvancedLight.h"
#include "Entity/Tools_Entity_DestructibleLight.h"
#endif // #ifndef ASURA_TARGET_SM3
#include "Entity/Tools_Entity_ForceField_Conveyor.h"
#include "Entity/Tools_Entity_Glass_Object.h"
#include "Entity/Tools_Entity_Lift.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Entity_Objective.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Entity/Tools_Entity_SplitterBlock_Object.h"
#include "Entity/Tools_Entity_LogicTrigger_Object.h"
#include "Entity/Tools_Entity_Trigger_Object.h"
#include "Entity/Tools_Entity_TimelineTrigger_Object.h"
#include "Entity/Tools_Entity_ClientVolumeTrigger_Object.h"
#include "Entity/Tools_Entity_StartPoint.h"
#include "Entity/Tools_Entity_MultiDamageVolume.h"
#include "Entity/Tools_Entity_ProxyTrigger_Object.h"
#ifdef ASURA_TARGET_SM3
#include "Entity/Tools_Entity_Projector.h"
#include "Entity/Tools_Entity_DynamicLight.h"
#include "Entity/Tools_Entity_LightShaft.h"
#endif
#include "Entity/Tools_Entity_WaypointBlockerVolume.h"
#if !defined(ASURA_DONT_USE_TELEPORTERS)
#include "Entity/Tools_Entity_Teleporter.h"
#include "Entity/Tools_Entity_TeleportDestination.h"
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)
#include "Entity/Tools_Message_Editing.h"
#include "Objects/Tools_StartPosition_Object.h"
#include "EntityTemplate/Tools_Entity_Template_Object.h"
#include "Entity/Tools_Entity_PFX_Effect.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Entity/Sound/Tools_Entity_DialogueTrigger.h"
#endif
#include "Entity/Tools_ReceivedMessages.h"

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Tools_Entity_Container.h"
#include "Entity/Container/Tools_Container_System.h"
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#include "Entity/Locations/Tools_Entity_Node.h"
#include "Entity/Locations/OrientedNode/Tools_EntityNode_OrientedNode.h"
#include "Entity/Locations/Spline/Tools_EntityNode_Spline.h"
#include "Entity/Locations/OrientedNode/LiftNode/Tools_EntityNode_LiftNode.h"
#ifdef ASURA_USE_AXON
#include "Entity/Locations/GuardZone/Tools_EntityNode_GuardZone.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/Tools_EntityNode_CoverpointNode.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AtttactorNode/Tools_EntityNode_GS2Attractor.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"
#endif //ASURA_USE_AXON

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
#include "Entity/Tools_Entity_DebugWaypointTrigger_Object.h"
#endif // #ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER

#include "HelperObject/Tools_HelperObject_Object.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Lighting/Tools_Corona_Object.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "MeshEdit/Tools_Mode_MeshEdit.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Render_Options.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_Mode_Sound.h"
#include "Sound/Tools_StreamingBackgroundSound_Object.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Tools_Dialogue_System.h"
#endif
#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
#include "Sound/Tools_Overlay_DynamicMusic.h"
#endif
#include "Texturing/Tools_Texture_Management.h"


//////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_Overlay_ObjectReport **
**----------------------------*/

class Tools_Overlay_ObjectReport : public Asura_Overlay
{
public :
	Tools_Overlay_ObjectReport(const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
	:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
	{}

	~Tools_Overlay_ObjectReport()
	{
		if(IsClosingByCloseGadget()) Tools_Mode_Object::ReportOverlayOpen = false;
	}
};


//////////////////////////////////////////////////////////////////////////////////////////
/*-------------------**
** Tools_Mode_Object **
**-------------------*/

static const char* g_aszObjectModeHelpStrings [] =
{
	"Ctrl+E :      Open/close entity manager overlay",
	"Ctrl+T :      Create object of the type selected in the object mode overlay",

	#ifdef ASURA_USE_AXON
	"",
	"Ctrl+N :      Create gamescene node",
	"Ctrl+L :      Create gamescenelink",
	"Ctrl+K :      Create linked gamescene node",
	#endif
	NULL
};


// public static member variables
Asura_Vector_2					Tools_Mode_Object::ObjectModeOverlayPosition = Asura_Vector_2(3.0f,28.0f);
bool							Tools_Mode_Object::ObjectModeOverlayOpen = true;
Asura_Handle					Tools_Mode_Object::MenuHandle;
Asura_Vector_2					Tools_Mode_Object::ReportOverlayPosition = Asura_Vector_2(3.0f, 14.0f);
bool							Tools_Mode_Object::ReportOverlayOpen = false;
Asura_Vector_2					Tools_Mode_Object::GlassOverlayPosition = Asura_Vector_2(3.0f, 14.0f);
bool							Tools_Mode_Object::s_bSelectingVertices = false;
int								Tools_Mode_Object::s_iNumSelectedVertices;
int								Tools_Mode_Object::s_aiSelectedModule[4];
int								Tools_Mode_Object::s_aiSelectedVertices[4];

List<ToolsObjectCreationInfo*>	Tools_Mode_Object::s_xObjectCreationList;
int								Tools_Mode_Object::s_iNumObjectCategories = 0;
const char**					Tools_Mode_Object::s_ppszObjectCategoryNames = 0;
int								Tools_Mode_Object::s_iNumObjectTypes = 0;
const char**					Tools_Mode_Object::s_ppszObjectTypeNames = 0;
const char**					Tools_Mode_Object::s_ppszObjectHelpText = 0;
ToolsObjectCreationInfo**		Tools_Mode_Object::s_ppxObjectCreationArray = 0;

int								Tools_Mode_Object::s_iSelectedObjectCategory;
int								Tools_Mode_Object::s_iSelectedObjectType;
bool							Tools_Mode_Object::s_bSelectedCategoryChanged;
Asura_List_Dialog*				Tools_Mode_Object::s_pxObjectListDialog;


// private static member variables
Tools_Mode_Object*              Tools_Mode_Object::s_pxThis = NULL;

// (MH - 28.10.2002) was called Initialise(), but that was a virtual function called
// by the constructor, which is no good as only the current scope gets called.
void Tools_Mode_Object::InitialiseMode()
{
	Tools_ReceivedMessages::Initialise();

#ifdef ASURA_USE_AXON
    Tools_Object_Axon_Interface::Initialise();
#endif

	//create the menubar
	MenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* pxMenubarTitle;

	pxMenubarTitle = Asura_Menubar_System::AddTitle(MenuHandle,"Object Mode");

	pxMenubarTitle->AddItem("Animation Templates",		Tools_AnimTemplates::ToggleOverlays,							Tools_AnimTemplates::MainOverlayIsOpen);
	pxMenubarTitle->AddItem("Entity Manager [Ctrl+E]",	Tools_SecondaryEntityFile_System::ToggleOverlay,				Tools_SecondaryEntityFile_System::OverlayIsOn);
    pxMenubarTitle->AddItem("Object Overlay",			Tools_Mode_Object::ToggleObjectModeOverlay,						Tools_Mode_Object::ObjectModeOverlayIsOn);
    pxMenubarTitle->AddItem("Received Messages",		Tools_ReceivedMessages::ToggleOverlay,							Tools_ReceivedMessages::OverlayIsOn);
    pxMenubarTitle->AddItem("Report Overlay",			Tools_Mode_Object::ToggleReportOverlay,							Tools_Mode_Object::ReportOverlayIsOn);
    pxMenubarTitle->AddItem("Simulation",				Tools_Entity_Simulation::ToggleOverlay,							Tools_Entity_Simulation::OverlayIsOn);
	pxMenubarTitle->AddItem("Templates",				Tools_Entity_TemplateInstance_Object::ToggleTemplateOverlay,	Tools_Entity_TemplateInstance_Object::TemplateOverlayIsOn);
	pxMenubarTitle->AddItem("Visible Object Types",		ObjectHideByType_Overlay::ToggleOverlay,						ObjectHideByType_Overlay::IsOverlayOn);

	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);
	
	BuildObjectCreationList();
	BuildObjectCategoryStrings();


	//create overlays
    CreateObjectModeOverlay();
	CreateReportOverlay();
	Tools_Entity_TemplateInstance_Object::CreateTemplateOverlay();
	Tools_Entity_Simulation::CreateOverlay();
	Tools_ReceivedMessages::CreateOverlay();
#ifdef ASURA_USE_DIALOGUE_SYSTEM
	if(Tools_Dialogue_System::OverlayShouldBeOpen()) Tools_Dialogue_System::CreateOverlay();
#endif
#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	if( Tools_Overlay_DynamicMusic::OverlayShouldBeOpen() )
	{
		Tools_Overlay_DynamicMusic::OpenOverlay();
	}
#endif
	Tools_Object_System::EnableAllControls(true);

	//create tools objects for coronas...need to do this before the static objects to avoid
	//problems with template updated
	for(int i=0;i<Asura_StaticCorona_System::GetArraySize();i++)
	{
		if(Asura_StaticCorona_System::IsValidCorona(i))
		{
			new Tools_Corona_Object(i);
		}
	}

    Asura_Entity_System::InitialiseServerEntitiesFromBlueprints();

	//create tools objects for everything
#if defined(ASURA_USE_TOOLS_START_POSITION_OBJECTS) || defined(ASURA_USE_BOTH_START_POSITION_TYPES)
	Tools_StartPosition_Object::CreateAllObjects();
#endif //defined(ASURA_USE_TOOLS_START_POSITION_OBJECTS) || defined(ASURA_USE_BOTH_START_POSITION_TYPES)
#ifdef ASURA_USE_ATTRACTORS
	Tools_Object_Attractor_Node::CreateAttractorObjects();
#endif
Tools_Entity_Object::CreateAllObjects();
	Tools_Object_System::CheckForMessageBlockOwnershipProblems();
    Tools_Mode_Sound::SetShowRadius(false);
	ClearVertexSelection();

	//Entity File overlay creation
	Tools_SecondaryEntityFile_System::CreateOverlay();

	// Check physical objects to make sure the blocked waypoints are okay.
    bool bLinkBlocksChanged = false;
    Asura_ServerEntity_PhysicalObject::InitialiseWaypointLinkBlockCalculations();
    Asura_Hashed_List_It< Asura_ServerEntity_PhysicalObject > xPhyObjectIter( Asura_ServerEntity_PhysicalObject::GetList() );
    while ( !xPhyObjectIter.Done() )
    {
        // Remove the old link blocks.
        xPhyObjectIter.GetCurrent()->UpdateWaypointLinkBlocks( true );
        // Calculate the new link blocks.
        bLinkBlocksChanged |= xPhyObjectIter.GetCurrent()->CalculateWaypointLinkBlocks();
        // Apply the new link blocks.
        xPhyObjectIter.GetCurrent()->UpdateWaypointLinkBlocks();

        xPhyObjectIter.Next();
    }
    Asura_ServerEntity_PhysicalObject::ShutdownWaypointLinkBlockCalculations();
    // (TI - 11.10.2007) - Do the same for any waypoint blocker volumes we may have
	Asura_ServerEntity_WaypointBlockerVolume::InitialiseWaypointLinkBlockCalculations();
	Asura_Hashed_List_It< Asura_ServerEntity_WaypointBlockerVolume > xWBVIter( Asura_ServerEntity_WaypointBlockerVolume::GetList() );
	while ( !xWBVIter.Done() )
	{
		// Calculate the new link blocks.
		bLinkBlocksChanged |= xWBVIter.GetCurrent()->CalculateWaypointLinkBlocks();

		// Apply the new link blocks (only applied if WBV is active)
		xWBVIter.GetCurrent()->UpdateWaypointLinkBlocks();

		xWBVIter.Next();
	}
	Asura_ServerEntity_WaypointBlockerVolume::ShutdownWaypointLinkBlockCalculations();

#ifdef ASURA_USE_CONTAINER_SYSTEM
	// MGH 18/11/2008: And repeat the process for containers.
	Asura_ServerEntity_Container::InitialiseWaypointLinkBlockCalculations();
	Asura_Hashed_List_It< Asura_ServerEntity_Container > xContIter( Asura_ServerEntity_Container::GetList() );
	for ( ; !xContIter.Done() ; xContIter.Next() )
	{
		Asura_ServerEntity_Container* pxServerEntity = xContIter.GetCurrent();

		// Remove the old link blocks.
		pxServerEntity->UpdateWaypointLinkBlocks( true );

		// Calculate the new link blocks.
        bLinkBlocksChanged |= pxServerEntity->CalculateWaypointLinkBlocks();

        // Apply the new link blocks.
        pxServerEntity->UpdateWaypointLinkBlocks();
	}
    Asura_ServerEntity_Container::ShutdownWaypointLinkBlockCalculations();
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

    if ( bLinkBlocksChanged )
    {
        Tools_SaveAsura::SetEntityModified();
    }

	//Don't want object polygons that have been merged to environment to be hidden while doing object placement
	Tools_Render_Options::SetDisableHideObjectPolygonFlag(true);

	Tools_SecondaryEntityFile_System::SelectInitialEntityFile();

#ifdef ASURA_USE_AXON
	// Must call this after the objects have been created.
	Tools_AnimTemplates::Initialise();

	Tools_Object_Axon_Interface::GetInstance()->EstablishGS2ObjectOwnerData();
	Tools_Object_Axon_Interface::GetInstance()->UpdateBehaviourTrees();
#endif

	VerifyObjects( 1 );

	Tools_Help::AddHelpTopic("Object Mode",g_aszObjectModeHelpStrings);
}
 


// (MH - 28.10.2002) was called Shutdown(), but that was a virtual function called
// by the constructor, which is no good as only the current scope gets called.
void Tools_Mode_Object::ShutdownMode(void)
{
	Tools_Entity_Simulation::End();

	Tools_Help::RemoveHelpTopic("Object Mode");

#ifdef ASURA_USE_AXON
	Tools_Object_Axon_Interface::StopPreview();
#endif

#ifdef ASURA_USE_ATTRACTORS
	Tools_Object_Attractor_Node::DeleteAttractorObjects();
#endif

	//delete all the objects
	Tools_Object_System::DeleteAllModeSpecific();
	Tools_Object::SetOverrideAnimTimers(false);

	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);
	
	//shutdown the menu
	if(MenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(MenuHandle);
		MenuHandle = ASURA_INVALID_HANDLE;
	}

	//clear up list of object creation functions
	while(s_xObjectCreationList.size())
	{
		delete s_xObjectCreationList.first_entry();
		s_xObjectCreationList.delete_first_entry();
	}
	s_iNumObjectCategories = 0;
	delete [] s_ppszObjectCategoryNames;
	s_ppszObjectCategoryNames = 0;

	s_iNumObjectTypes = 0;
	delete [] s_ppszObjectTypeNames;
	s_ppszObjectTypeNames = 0;
	delete [] s_ppszObjectHelpText;
	s_ppszObjectHelpText = 0;
	delete [] s_ppxObjectCreationArray;
	s_ppxObjectCreationArray = 0;
	s_pxObjectListDialog = 0;

	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_OBJECTREPORT);
	CloseGlassInitOverlay();
	Tools_Entity_TemplateInstance_Object::CloseTemplateOverlay();
	Tools_Entity_Simulation::CloseOverlay();
	Tools_SecondaryEntityFile_System::CloseOverlay();

	Tools_Object_System::EnableAllControls(false);

	Tools_Render_Options::SetDisableHideObjectPolygonFlag(false);

#ifdef ASURA_USE_AXON
	Tools_AnimTemplates::Shutdown();
    Tools_Object_Axon_Interface::Shutdown();
#endif

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Tools_Dialogue_System::CloseOverlays();
	#endif

	#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	Tools_Overlay_DynamicMusic::CloseOverlay();
	#endif

    Tools_ReceivedMessages::Shutdown();
}

void Tools_Mode_Object::PreShutdown( void )
{
	Tools_Entity_Simulation::End();
  
    // Deselect the currently selected object(s), if necessary. This allows
    // objects to fix up or record any temporary global changes they've made
    // while selected, etc.
    Tools_Object_System::DeselectAllObjects();

	//make sure all objects flagged for merging into the environment have been
	Tools_Entity_PhysicalObject::MergeObjectsIntoEnvironmentIfNeeded(false);
}

void Tools_Mode_Object::TransferModeData(int iMode)
{
	//open the file
	Tools_ModeDataFile xFile("ObjectMode.dat",iMode,6);

	//transfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(ObjectModeOverlayPosition);
	xFile.Transfer(ObjectModeOverlayOpen);
	xFile.Transfer(ReportOverlayPosition);
	xFile.Transfer(ReportOverlayOpen);
	xFile.Transfer(GlassOverlayPosition);
	xFile.Transfer(Tools_ReceivedMessages::s_xOverlayPosition);

	//Going to need to start doing some versioning , since I can't be sure that nothing is going to be inserted in the middle
	//of the file with extra stuff being added by Tools_Object_Axon_Interface::TransferModeData()
	if(xFile.GetVersion() >= 1)
	{
		xFile.Transfer(Tools_ReceivedMessages::s_bOverlayShouldBeOpen);

		xFile.Transfer(Tools_SecondaryEntityFile_System::s_xOverlayPosition);
		xFile.Transfer(Tools_SecondaryEntityFile_System::s_bOverlayIsOn);

		xFile.Transfer(Tools_Entity_TemplateInstance_Object::s_xTemplateOverlayPosition);
		xFile.Transfer(Tools_Entity_TemplateInstance_Object::s_bTemplateOverlayOpen);
	}

	if(xFile.GetVersion() >= 2)
	{
		xFile.Transfer(Tools_Entity_TemplateInstance_Object::s_bHideTriggers);
	}

	if(xFile.GetVersion() >= 3)
	{
		xFile.Transfer(Tools_SecondaryEntityFile_System::s_xTreeDialogSize);
		xFile.Transfer(Tools_SecondaryEntityFile_System::s_xFilterListDialogSize);
	}

	if(xFile.GetVersion() >= 4)
	{
		xFile.Transfer(Tools_Entity_TemplateInstance_Object::s_xTemplateListSize);
		xFile.Transfer(Tools_Entity_TemplateInstance_Object::s_bShowTemplatesInSubDirs);
	}

	if(xFile.GetVersion() >= 5)
	{
		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		xFile.Transfer(Tools_MessageEditor::s_xOverlayPosition);
		xFile.Transfer(Tools_MessageEditor::s_xMessageAttributeOverlayPosition);
		#else
		Asura_Vector_2 xDummyPosition(1.0f, 1.0f);
		xFile.Transfer(xDummyPosition);
		#endif
	}

	if(xFile.GetVersion() >= 6)
	{
		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		xFile.Transfer(Tools_MessageEditor::s_xDialogSize);
		#else
		Asura_Vector_2 xDummySize(0.0f, 0.0f);
		xFile.Transfer(xDummySize);
		#endif
	}

#ifdef ASURA_USE_AXON
    Tools_Object_Axon_Interface::GetInstance()->TransferModeData( xFile );
#endif
}


void Tools_Mode_Object::Update(void)
{
	//act upon keyboard input
    Asura_Assert(s_pxThis, "");
    if (s_pxThis)
    {
        s_pxThis->HandleKeyboard();
    }

	if(s_bSelectedCategoryChanged)
	{
		//object category changed , so need to update the list of object types shown
		UpdateObjectTypeList();
		s_bSelectedCategoryChanged = false;
	}

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Tools_Dialogue_System::Update();
	#endif


	Tools_Entity_Simulation::Update();
	Tools_SecondaryEntityFile_System::Update();
	Tools_ReceivedMessages::Update();

#ifdef ASURA_USE_AXON
    Tools_Object_Axon_Interface::GetInstance()->Update();
	Tools_AnimTemplates::Update();
#endif
}

void Tools_Mode_Object::RenderWorld_PostToneMap(void)
{
	Tools_Entity_Simulation::Render();

#ifdef ASURA_USE_AXON
    Tools_Object_Axon_Interface::GetInstance()->Render();
	Tools_AnimTemplates::Render();
#endif

	if (s_bSelectingVertices)
	{
		if (s_iNumSelectedVertices)
		{
			//show the selected vertices
			for(int i=0;i<s_iNumSelectedVertices;i++)
			{
				Asura_Vector_3 xPosition;
				Asura_Vector_3 xPoint;
				Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(s_aiSelectedModule[i]);
				Asura_Module_System::GetPositionOfModule(s_aiSelectedModule[i],xPosition);
				xPoint = xPosition + xMesh->VertexDataPtr[s_aiSelectedVertices[i]].Position;
				Tools_Mode_MeshEdit::RenderVertexSelection(xPoint);
			}
		}
	}
}

void Tools_Mode_Object::RenderUI(void)
{
	if (s_bSelectingVertices)
	{	
		// Also, render the help text.
		float x = 0.0f;
		float y = 4.0f;
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Select vertices by clicking on them.");
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"De-Select vertices by holding shift while clicking on them.");
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Press 'L' to de-select all vertices.");
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Press 'F1' for wireframe mode to make selection easier.");
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Choose vertices that make up a right-angled triangle,");
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"or a rectangle to create a pane of glass.");
	}
}

void Tools_Mode_Object::PreSave()
{
	Tools_Entity_Simulation::End();

	//make sure all templates are up to date before save
	Tools_Entity_TemplateInstance_Object::UpdateAllTemplates();

	//make sure all objects flagged for merging into the environment have been
	Tools_Entity_PhysicalObject::MergeObjectsIntoEnvironmentIfNeeded(false);
}

void Tools_Mode_Object::QuickSave(void)
{
	Tools_Entity_Simulation::End();

	//make sure all templates are up to date first
	Tools_Entity_TemplateInstance_Object::UpdateAllTemplates();
	//make sure all objects flagged for merging into the environment have been
	Tools_Entity_PhysicalObject::MergeObjectsIntoEnvironmentIfNeeded(false);

	if(Tools_SaveAsura::IsMainFileModified()) Tools_SaveAsura::QuickSaveFile();
	Tools_SaveAsura::SaveEntities();

	//Some of the other files may have been modified by creation/updates of template
	//instance objects. So save them as well if required. RAR 19/08/04
	if(Tools_SaveAsura::IsSoundsModified()) Tools_SaveAsura::SaveSounds();
	if(Tools_SaveAsura::IsGlobalIlluminationModified()) Tools_SaveAsura::SaveGlobalIllumination();
	if(Tools_SaveAsura::IsLightingModified()) Tools_SaveAsura::SaveLights();
	if(Tools_SaveAsura::IsEnvironmentalLightTransportModified()) Tools_SaveAsura::SaveEnvironmentalLightTransport();
	#ifdef ASURA_USE_ATTRACTORS
	if(Tools_SaveAsura::IsAttractorsModified()) Tools_SaveAsura::SaveAttractors();
	#endif
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Tools_Dialogue_System::SaveModified();
	#endif
	#ifdef ASURA_USE_AXON
	if(Tools_SaveAsura::AreAnimTemplatesModified()) Tools_SaveAsura::SaveAnimTemplates();
	#endif
}

bool Tools_Mode_Object::IsModified()
{
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	if(Tools_Dialogue_System::IsModified()) return true;
	#endif
	#ifdef ASURA_USE_ATTRACTORS
	if(Tools_SaveAsura::IsAttractorsModified()) return true;
	#endif
	return Tools_SaveAsura::IsMainFileModified()||Tools_SaveAsura::IsEntityModified();
}


// virtual function - returns whether a keypress was handled, or explicitly ignored
bool Tools_Mode_Object::HandleKeyboard()
{
	if (Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return true;
	if (Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return true;
 
	if (Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{

#ifdef ASURA_USE_AXON
		if (Asura_UserInput::DebouncedKeyPressed('n'))
		{
			Tools_Object_Axon_Interface::GetInstance()->CreateGS2Node();
		}

		if (Asura_UserInput::DebouncedKeyPressed('l') && Tools_Object_Axon_Interface::CanCreateGS2LinkBetweenNodes())
		{
			Tools_Object_Axon_Interface::GetInstance()->CreateGS2LinkBetweenNodes();
		}

		if (Asura_UserInput::DebouncedKeyPressed('k') && Tools_Object_Axon_Interface::CanCreateGS2LinkedNode())
		{
			Tools_Object_Axon_Interface::GetInstance()->CreateGS2LinkedNode();
		}
#endif // ASURA_USE_AXON

		if (Asura_UserInput::DebouncedKeyPressed('t'))
		{
			CreateObject_SnapToVertex();
            return true;
		}
		if(Asura_UserInput::DebouncedKeyPressed('e'))
		{
			Tools_SecondaryEntityFile_System::ToggleOverlay();
			return true;
		}
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
	}
	else
	{
	}
	if (s_bSelectingVertices)
	{
		HandleVertexSelection();
	}

    return false;
}

void Tools_Mode_Object::HandleVertexSelection()
{
	Tools_Object_System::DeselectAllObjects();

	if (Asura_UserInput::DebouncedKeyPressed('l'))
	{
		ClearVertexSelection();
		return;
	}

	//don't do vertex selection if using menus etc.
	if (Asura_Interface::UsingMenusOrOverlays()) return;
	if(!Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1)) return;
	
	//find the vertex selected
	int iModuleIndex,iVertIndex;
	if(!Tools_LOS::FindVertUnderCursor(iVertIndex,iModuleIndex)) return;
	Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);
	Asura_Vector_3 xPosition;
	Asura_Module_System::GetPositionOfModule(iModuleIndex,xPosition);
	Asura_Vector_3 xSelectedPos = xPosition + xMesh->VertexDataPtr[iVertIndex].Position;
	//selecting or deselecting?
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		//deselect this vertex  , if it was selected
		for(int i=0;i<s_iNumSelectedVertices;i++)
		{
			if(s_aiSelectedModule[i] == iModuleIndex &&
				s_aiSelectedVertices[i] == iVertIndex)
			{
				//deselect this one
				s_iNumSelectedVertices--;
				s_aiSelectedVertices[i] = s_aiSelectedVertices[s_iNumSelectedVertices];
				s_aiSelectedModule[i] = s_aiSelectedModule[s_iNumSelectedVertices];
				break;
			}
		}
	}
	else
	{
		//selection
		if(s_iNumSelectedVertices==0)
		{
			s_aiSelectedModule[s_iNumSelectedVertices] = iModuleIndex;
			s_aiSelectedVertices[s_iNumSelectedVertices++] = iVertIndex;
		}
		else if(s_iNumSelectedVertices<4)
		{
			//check this vertex isn't already selected
			//must check by position, not by index, because the same vertex might exist in 
			//different modules.
			int i;
			for(i=0;i<s_iNumSelectedVertices;i++)
			{
				Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(s_aiSelectedModule[i]);
				Asura_Vector_3 xPosition;
				Asura_Module_System::GetPositionOfModule(s_aiSelectedModule[i],xPosition);
				Asura_Vector_3 xPos = xPosition + xMesh->VertexDataPtr[s_aiSelectedVertices[i]].Position;
				if (xPos.ApproximatelyEquals(xSelectedPos))
					return;
			}
			if(i==s_iNumSelectedVertices)
			{
				s_aiSelectedModule[s_iNumSelectedVertices] = iModuleIndex;
				s_aiSelectedVertices[s_iNumSelectedVertices++] = iVertIndex;
			}
			if (s_iNumSelectedVertices == 3)
			{
				if (!CanCreateGlassFromVertices())
				{
					//cancel the last selection then
					s_iNumSelectedVertices--;
					Tools_Timed_Message_System::AddMessage(0xffff0000, "Third vertex must create a right-angled triangle");
				}
			}

			if(s_iNumSelectedVertices==4)
			{
				//see if the vertices will make a quad
				//get the 4 points
				Asura_Vector_3 xPoints[4];
				for(int i=0;i<4;i++)
				{
					Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(s_aiSelectedModule[i]);
					Asura_Vector_3 xPosition;
					Asura_Module_System::GetPositionOfModule(s_aiSelectedModule[i],xPosition);
					xPoints[i] = xPosition + xMesh->VertexDataPtr[s_aiSelectedVertices[i]].Position;
				}
				if(!Tools_Mode_MeshEdit::WillSelectedVerticesMakeAQuad(xPoints) ||
					!CanCreateGlassFromVertices())
				{
					//cancel the last selection then
					s_iNumSelectedVertices--;
					Tools_Timed_Message_System::AddMessage(0xffff0000, "Fourth vertex must complete a rectangle");
				}
			}
		}
	}
}

void Tools_Mode_Object::ToggleObjectModeOverlay(void)
{
	ObjectModeOverlayOpen = !ObjectModeOverlayOpen;

	if (ObjectModeOverlayOpen)
	{
        CreateObjectModeOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);
	}
}
bool Tools_Mode_Object::ObjectModeOverlayIsOn(void)
{
	return ObjectModeOverlayOpen;
}


void Tools_Mode_Object::CreateObjectModeOverlay(void)
{
	if(!ObjectModeOverlayOpen) return;

	Asura_Overlay* pxNewOverlay = new Asura_Overlay("Object Mode", false, &ObjectModeOverlayPosition);
	
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f, 3.0f);

    Asura_Assert(s_pxThis, "");
    if (s_pxThis)
    {
        s_pxThis->AddItemsToObjectModeOverlay(pxNewOverlay, xOffset);
    }
	pxNewOverlay->m_uID = TOOLS_OVERLAY_MODE;

	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay, false);
}

void Tools_Mode_Object::AddItemsToObjectModeOverlay(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset)
{

	Asura_Dialog* pxNewDialog;

	//update list of object types to be shown for creation
	s_pxObjectListDialog = 0;
	UpdateObjectTypeList();

	//add title for categories
	pxNewDialog = new Asura_Dialog("Category :", xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//add drop down list for catgories
	pxNewDialog = new Asura_DropDown_List_Dialog(xOffset,s_ppszObjectCategoryNames,&s_iSelectedObjectCategory,10,s_iNumObjectCategories,NULL,&s_bSelectedCategoryChanged);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//add title for object
	pxNewDialog = new Asura_Dialog("Object Type :", xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//add object list
	pxNewDialog = s_pxObjectListDialog = new Asura_List_Dialog(xOffset,s_ppszObjectTypeNames,&s_iSelectedObjectType,10,s_iNumObjectTypes,NULL);
	s_pxObjectListDialog->SetListHelpStrings(s_ppszObjectHelpText);
	s_pxObjectListDialog->SetDoubleClickFunction(CreateObject_DoubleClick,0);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y = pxNewDialog->m_xMaxCoord.y + 1.0f;

	//add creation option
	pxNewDialog = new Asura_Function_Dialog("Create at camera", xOffset, CreateObject_AtCamera);
	pxNewDialog->SetHelpText("Create the object selected from the list above.\nIn most cases the object will be left at the camera.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Create", xOffset, CreateObject_SnapToVertex);
	pxNewDialog->SetHelpText("Create the object selected from the list above, and start placing via snap to vertex.\nShortcut - Ctrl + T.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//add copy object controls
	Tools_Object_System::AddCopyOverlayControls(pxNewOverlay,xOffset);

    pxNewDialog = new Asura_Function_Dialog("Find Object by Guid", xOffset, Tools_Object_System::FindObjectByGuid);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Function_Dialog("Hide By Type", xOffset, Tools_Object_System::ShowUserHideByTypeOverlay);
	pxNewDialog->SetHelpText("Set object visibility based on type.\nShortcut - Alt + V.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//anim timer overide
	Tools_Object::AddAnimTimerOverrideOverlayControls(pxNewOverlay,xOffset);
	xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Function_Dialog("Merge with environment", xOffset, Tools_Object_System::MergeSelectedIntoModules);
	pxNewDialog->SetHelpText("This merges the meshes of all the selected objects into the environment (modules).\nThese objects will be deleted afterwards.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

    pxNewDialog = new Asura_Function_Dialog("Fix Object Name Prefixes", xOffset, FixNamePrefixes );
	pxNewDialog->SetHelpText( "Adds the appropriate prefix to all object names." );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Verify Objects", xOffset, VerifyObjects, 0 ); // 0 => manual check
	pxNewDialog->SetHelpText( "Verifies object properties/set-ups and other data to help detect errors." );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}


void Tools_Mode_Object::ToggleReportOverlay(void)
{
	if (!ReportOverlayIsOn())
	{
		ReportOverlayOpen = true;
		CreateReportOverlay();
	}
	else
	{
		ReportOverlayOpen = false;
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_OBJECTREPORT);
	}
}

bool Tools_Mode_Object::ReportOverlayIsOn(void)
{
	return Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_OBJECTREPORT);
}

void Tools_Mode_Object::CreateReportOverlay(void)
{
	if(!ReportOverlayOpen) return;

	Asura_Overlay* pxNewOverlay = new Tools_Overlay_ObjectReport("Object Reports", false, &ReportOverlayPosition);
	
	Asura_Dialog* pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f, 3.0f);

	pxNewDialog = new Asura_Function_Dialog("Create Static Object Report", xOffset, Tools_Mode_Object::CreateObjectReport);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewOverlay->m_uID = TOOLS_OVERLAY_OBJECTREPORT;

	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay);
}

void Tools_Mode_Object::CreateGlassInitOverlay(void)
{
	Asura_Overlay* pxNewOverlay = new Asura_Overlay("Create glass", false, &GlassOverlayPosition);
	Asura_Dialog* pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f, 3.0f);

	pxNewDialog = new Asura_Function_Dialog("Create default glass at camera position", xOffset, CreateDefaultGlass);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Create glass from selected vertices", xOffset, CreateGlassFromVertices, NULL, CanCreateGlassFromVertices);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Cancel", xOffset, CloseGlassInitOverlay);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewOverlay->m_uID = TOOLS_OVERLAY_GLASSCREATION;
	s_bSelectingVertices = true;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay, false);
}

void Tools_Mode_Object::CloseGlassInitOverlay(void)
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GLASSCREATION);
	s_bSelectingVertices = false;
	ClearVertexSelection();
}

void Tools_Mode_Object::CreateDefaultGlass(void)
{
	Tools_Entity_Glass_Object::Project_Create(*Asura_Camera::GetPosition(), *Asura_Camera::GetOrientation(), Asura_Vector_2(1.0f, 1.0f));
	CloseGlassInitOverlay();

	//add glass to seconary file if needed
	
	if(Tools_Object_System::SelectedObject && Tools_Object_System::SelectedObject->Type == TOT_Entity_Glass)
	{
		Tools_SecondaryEntityFile_System::AddToCurrentFile(Tools_Object_System::SelectedObject);
	}
}

void Tools_Mode_Object::CreateGlassFromVertices(void)
{
	Asura_Vector_3 xPoints[3];
	for(int i=0;i<3;i++)
	{
		Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(s_aiSelectedModule[i]);
		Asura_Vector_3 xPosition;
		Asura_Module_System::GetPositionOfModule(s_aiSelectedModule[i],xPosition);
		xPoints[i] = xPosition + xMesh->VertexDataPtr[s_aiSelectedVertices[i]].Position;
	} 
	Asura_Vector_3 xCornerPosition = xPoints[0];
	Asura_Vector_3 xEdge1 = (xPoints[1] - xPoints[0]);
	Asura_Vector_3 xEdge2 = (xPoints[2] - xPoints[0]);
	Asura_Vector_2 xEdgeLengths;
	xEdgeLengths.x = xEdge1.Magnitude();
	xEdgeLengths.y = xEdge2.Magnitude();

	xEdge1.Normalise();
	xEdge2.Normalise();
	Asura_Matrix_3x3 xMat;
	xMat.e11 = xEdge1.x;
	xMat.e21 = xEdge1.y;
	xMat.e31 = xEdge1.z;
	xMat.e12 = xEdge2.x;
	xMat.e22 = xEdge2.y;
	xMat.e32 = xEdge2.z;
	Asura_Vector_3 xCross = xEdge1 ^ xEdge2;
	xMat.e13 = xCross.x;
	xMat.e23 = xCross.y;
	xMat.e33 = xCross.z;
	
	Tools_Entity_Glass_Object::Project_Create(xCornerPosition, xMat, xEdgeLengths);
	CloseGlassInitOverlay();

	//add glass to seconary file if needed
	
	if(Tools_Object_System::SelectedObject && Tools_Object_System::SelectedObject->Type == TOT_Entity_Glass)
	{
		Tools_SecondaryEntityFile_System::AddToCurrentFile(Tools_Object_System::SelectedObject);
	}
	

}

bool Tools_Mode_Object::CanCreateGlassFromVertices(void)
{
	// To create glass from the vertices then we must either have a right-angled triangle or a rectangle.
	if (s_iNumSelectedVertices == 3)
	{
		float fThreshold = 0.01f;
		// Check that it's a right angled triangle.
		Asura_Vector_3 xPoints[3];
		for(int i=0;i<3;i++)
		{
			Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(s_aiSelectedModule[i]);
			Asura_Vector_3 xPosition;
			Asura_Module_System::GetPositionOfModule(s_aiSelectedModule[i],xPosition);
			xPoints[i] = xPosition + xMesh->VertexDataPtr[s_aiSelectedVertices[i]].Position;
		}
		float afLengthsSquared[3];
		for (int i=0;i<3;i++)
		{
			int iOtherPoint = (i == 2) ? 0 : i+1;
			Asura_Vector_3 xDiff = (xPoints[i] - xPoints[iOtherPoint]);
			afLengthsSquared[i] = xDiff * xDiff;
		}
		// Now, find out if the square on the hypoteneuse is equal to the sum of the squares
		// of the other 2 sides.aw.
		for (int i=0;i<3;i++)
		{
			float fTotal = 0.0f;
			for (int j=0;j<3;j++)
			{
				if (j==i)
					continue;
				fTotal += afLengthsSquared[j];
			}
			if (Asura_Maths::Abs(afLengthsSquared[i] - fTotal) < fThreshold)
			{
				// Re-order the selection so that the right-angle point is first.
				int iRightAngle = i - 1;
				if (iRightAngle < 0)
					iRightAngle = 2;
				if (iRightAngle != 0)
				{
					int iSwapModule = s_aiSelectedModule[0];
					int iSwapVertex = s_aiSelectedVertices[0];
					s_aiSelectedModule[0] = s_aiSelectedModule[iRightAngle];
					s_aiSelectedVertices[0] = s_aiSelectedVertices[iRightAngle];
					s_aiSelectedModule[iRightAngle] = iSwapModule;
					s_aiSelectedVertices[iRightAngle] = iSwapVertex;
				}
				return true;
			}
		}
		return false;
	}
	else if (s_iNumSelectedVertices == 4)
	{
		float fThreshold = 0.75f;
		// Check that this is a rectangle.
		// We already know that this is planar, it doesn't let you select the vertices if it isn't.
		Asura_Vector_3 xPoints[4];
		for(int i=0;i<4;i++)
		{
			Asura_Mesh* xMesh = Asura_Module_System::GetMeshForModule(s_aiSelectedModule[i]);
			Asura_Vector_3 xPosition;
			Asura_Module_System::GetPositionOfModule(s_aiSelectedModule[i],xPosition);
			xPoints[i] = xPosition + xMesh->VertexDataPtr[s_aiSelectedVertices[i]].Position;
		}
		float aafLengthsSquared[4][4];
		for (int i=0;i<4;i++)
		{
			for (int j=0;j<4;j++)
			{
				if (i==j)
					continue;
				Asura_Vector_3 xDiff = xPoints[i] - xPoints[j];
				aafLengthsSquared[i][j] = xDiff * xDiff;
			}
		}
		// There should only be a max of 3 different distances if this is a rectangle.
		float afDistSquared[3];
		int iNumDiffDistances = 0;
		for (int i=0;i<4;i++)
		{
			for (int j=0;j<4;j++)
			{
				if (i==j)
					continue;
				if (iNumDiffDistances == 0)
				{
					afDistSquared[iNumDiffDistances++] = aafLengthsSquared[i][j];
				}
				else
				{	
					bool bMatches = false;
					for (int k=0;k<iNumDiffDistances;k++)
					{
						if (Asura_Maths::Abs(afDistSquared[k] - aafLengthsSquared[i][j]) < fThreshold)
							bMatches = true;
					}
					if (!bMatches)
					{
						if (iNumDiffDistances == 3)
							return false;
						afDistSquared[iNumDiffDistances++] = aafLengthsSquared[i][j];
					}
				}
			}
		}
		if (iNumDiffDistances <= 3)
		{
			// Re-order the selection so that points 1 and 2 are adjacent to point 0.
			for (int i=1;i<3;i++)
			{
				bool bIsOpposite = true;
				for (int j=0;j<iNumDiffDistances;j++)
				{
					if (aafLengthsSquared[0][i] < afDistSquared[j])
						bIsOpposite = false;
				}
				if (bIsOpposite)
				{
					// We need to swap selection i with selection 3.
					int iSwapModule = s_aiSelectedModule[3];
					int iSwapVertex = s_aiSelectedVertices[3];
					s_aiSelectedModule[3] = s_aiSelectedModule[i];
					s_aiSelectedVertices[3] = s_aiSelectedVertices[i];
					s_aiSelectedModule[i] = iSwapModule;
					s_aiSelectedVertices[i] = iSwapVertex;
				}
			}
			return true;
		}
	}
	return false;
}

void Tools_Mode_Object::ClearVertexSelection()
{
	s_iNumSelectedVertices = 0;
}

bool Tools_Mode_Object::EntityReceivesActivateMessage( Asura_Guid uGuid ) const
{
	// loop over static messages
	for(Tools_StaticMessageIterator xIterator(true);!xIterator.Done();xIterator.Next())
	{
		const Asura_StaticMessage* pxStaticMessage = xIterator.GetCurrent();
		if(pxStaticMessage->IsDisabled()) continue;

		if (pxStaticMessage->m_usMsgID == ASURA_MSG_ACTIVATE)
		{
			if (pxStaticMessage->m_uSendTo == uGuid)
			{
				return true;
			}
			else
			{
				// check if this is sent to a proxy trigger which targets the entity we're looking for.
				const Asura_ServerEntity_ProxyTrigger* pxProxy = Asura_ServerEntity_ProxyTrigger::FindEntity(pxStaticMessage->m_uSendTo);
				if (pxProxy)
				{
					if (pxProxy->IsEntityTargeted(uGuid))
					{
						return true;
					}
				}
			}
		}

	}

	return false;
}

void Tools_Mode_Object::VerifyObjects( int iAutomatic )
{
	Tools_MessageLog::DeleteMessages( Tools_MessageLog::VERIFICATION, NULL );

	const bool bAutomatic = ( iAutomatic != 0 );
	if ( s_pxThis->DoVerificationTests( bAutomatic ) )
	{
		if ( !bAutomatic )
		{
			Message_Overlay xMessage( "No entity errors found." );
			xMessage.DoModal();
		}
	}
	else
	{
		// Show if manual, or if tickbox is on.
		if ( !bAutomatic || Tools_MessageLog::ShouldShowForEntities() )
		{
			Tools_MessageLog::ShowOverlay( Tools_MessageLog::VERIFICATION, NULL );
		}
	}
}

bool Tools_Mode_Object::DoVerificationTests( const bool bAutomatic )
{
	bool bOk = true;

	bOk &= VerifyToolsObjects( bAutomatic );
	bOk &= VerifyEntityMessages( bAutomatic );
	bOk &= VerifyProjectors();

	#ifdef ASURA_USE_AXON
	if ( !bAutomatic )
	{
		bOk &= VerifyAnimTemplates();
		Tools_EntityNode_CoverpointNode::CheckCoverpointsForValidity();
	}
	#endif

	#ifdef ASURA_USE_CONTAINER_SYSTEM
	bOk &= Tools_Container_System::VerifyContainerMessages();
	#endif //ASURA_USE_CONTAINER_SYSTEM

	if ( !bAutomatic )
	{
		OutputWaypointBlockingStats();
	}

	return bOk;
}

bool Tools_Mode_Object::VerifyToolsObjects( const bool bAutomatic )
{
	bool bOk = true;
	for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList);!olif.done();olif.next())
	{
		bOk &= olif()->Verify( bAutomatic );
	}
	return bOk;
}

bool Tools_Mode_Object::VerifyEntityMessages( const bool bAutomatic )
{
	bool bOk = true;

	//get a list of entities used by cutscenes to help with determining those that never become active
	List<Asura_Guid> xCutsceneAttachmentList;
	Tools_Mode_Cutscene::ListGuidsUsedByEntityAttachments(xCutsceneAttachmentList);

	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		// If entity doesn't start active, check it receives at least one activate message.
		// We're not concerned if the entity is inactive because its template instance is currently inactive,
		// as long as the template does get activated (which will be checked for by this routine as well).
		// Therefore use the entity's copy of activity rather than the combined (entity && template).
		if (!pxEntity->GetEntityActiveIgnoringTemplate())
		{
			if (!s_pxThis->EntityReceivesActivateMessage(pxEntity->GetGuid()))
			{
				//ignore entities used by cutscenes ,since they will be activated by the cutscene attachment
				if(!xCutsceneAttachmentList.contains(pxEntity->GetGuid()))
				{
				
					// log message
					Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Messages", new Tools_MessageLogAction_GotoEntity( pxEntity->GetGuid() ),
						"Activate message never sent for inactive entity with GUID %u (classification %s).",
						pxEntity->GetGuid(), Asura_Network::GetEntityClassificationDesc(pxEntity->GetClassification()));
					bOk = false;
				}
			}
		}

		// Check for containing events which the entity doesn't support.
		int iNumEvents = pxEntity->GetNumEvents();
		if (iNumEvents)
		{
			Tools_Entity_Object* pxToolsObject = Tools_Entity_Object::FindEntity(pxEntity->GetGuid());
			Asura_Assert(pxToolsObject, "failed to find tools object for entity");
			if (pxToolsObject)
			{
				for (int i = 0; i < iNumEvents; ++i)
				{
					if (!pxToolsObject->HandlesThisEvent(pxEntity->GetEvents()[i].m_usEventID))
					{
						Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Messages", 
							new Tools_MessageLogAction_GotoEntity( pxEntity->GetGuid() ),
							"Entity %u has messages to send on events which it doesn't support (classification %s),",
							pxEntity->GetGuid(), Asura_Network::GetEntityClassificationDesc(pxEntity->GetClassification()));
						bOk = false;
					}
				}
			}
		}
	}

	// Check that all static messages have valid targets.
	for ( Tools_StaticMessageIterator xMsgIt; !xMsgIt.Done(); xMsgIt.Next() )
	{
		const Asura_StaticMessage* pxMsg = xMsgIt.GetCurrent();
		if ( ASURA_GUID_TOOLSCREATEDENTITY_START <= pxMsg->m_uSendTo && pxMsg->m_uSendTo <= ASURA_GUID_TOOLSCREATEDENTITY_END )
		{
			Tools_Object* pxFromObject = Tools_Object_System::GetOwnerOfMessageBlock( xMsgIt.GetCurrentMessageBlockID() );
			if ( pxFromObject )
			{
				Tools_Object* pxToObject = Tools_Object_System::GetObjectFromGuid( pxMsg->m_uSendTo );
				
				if(pxToObject)
				{
					//Make sure the message target is safe
					if(!Tools_SecondaryEntityFile_System::IsAllowableTarget(pxFromObject, pxToObject, false))
					{
						if(pxToObject->IsEntityObject())
						{
							if(((Tools_Entity_Object*)pxToObject)->IsMemberOfTemplateInstance())
							{
								//Members of a template instance get generated within the main file , even if the
								//template isn't. Which could result in a false error about a non-allowable target , if
								//the template is part of a common file.
								//So ignore such errors. RAR 23/02/2009
								continue;
							}
						}
						
						Asura_String xErrorText( "Object '" );
						xErrorText += pxFromObject->GetName();
						xErrorText += "' has static message ";
						xErrorText += Asura_Network::GetMsgIDDesc( pxMsg->m_usMsgID );
						xErrorText += " with unsafe target (";
						xErrorText += pxToObject->GetName();
						xErrorText += ").";

						Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Messages",
							new Tools_MessageLogAction_GotoEntity( pxFromObject->GetGuid() ),
							xErrorText.GetString() );
						bOk = false;
					}
				}
				else
				{
					//Target of message is missing
					Asura_String xErrorText( "Object '" );
					xErrorText += pxFromObject->GetName();
					xErrorText += "' has static message ";
					xErrorText += Asura_Network::GetMsgIDDesc( pxMsg->m_usMsgID );
					xErrorText += " with non-existent target (";
					char acNum[ 12 ];
					Asura_PrintfToBfr( acNum, "%u", 12, true, pxMsg->m_uSendTo );
					xErrorText += acNum;
					xErrorText += ").";

					Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Messages",
						new Tools_MessageLogAction_GotoEntity( pxFromObject->GetGuid() ),
						xErrorText.GetString() );
					bOk = false;
				}
			}
		}
	}

	// Check that all static messages are still handled by their targets.
	for ( Tools_StaticMessageIterator xMsgIt; !xMsgIt.Done(); xMsgIt.Next() )
	{
		const Asura_StaticMessage* pxMsg = xMsgIt.GetCurrent();
		if ( ASURA_GUID_TOOLSCREATEDENTITY_START <= pxMsg->m_uSendTo && pxMsg->m_uSendTo <= ASURA_GUID_TOOLSCREATEDENTITY_END )
		{
			Tools_Object* pxToObject = Tools_Object_System::GetObjectFromGuid( pxMsg->m_uSendTo );
			if ( pxToObject && !pxToObject->HandlesThisMessage( pxMsg->m_usMsgID ) )
			{
				Tools_Object* pxFromObject = Tools_Object_System::GetOwnerOfMessageBlock( xMsgIt.GetCurrentMessageBlockID() );
				if ( pxFromObject )
				{
					Asura_String xErrorText( "Object '" );
					xErrorText += pxFromObject->GetName();
					xErrorText += "' is sending unhandled static message ";
					xErrorText += Asura_Network::GetMsgIDDesc( pxMsg->m_usMsgID );
					xErrorText += " to object '";
					xErrorText += pxToObject->GetName();
					xErrorText += "'.";

					Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Messages",
						new Tools_MessageLogAction_GotoEntity( pxFromObject->GetGuid() ),
						xErrorText.GetString() );
					bOk = false;
				}
			}
		}
	}

	// Check if there are any old gamescene messages.
	for ( Tools_StaticMessageIterator xMsgIt; !xMsgIt.Done(); xMsgIt.Next() )
	{
		const Asura_StaticMessage* pxMsg = xMsgIt.GetCurrent();
		switch ( pxMsg->m_usMsgID )
		{
			case ASURA_MSG_AXON_GAMESCENE_LEAVE_GAMESCENE_AI:
			case ASURA_MSG_AXON_GAMESCENE_NEVER_STOP_AT_NODE:
			case ASURA_MSG_AXON_GAMESCENE_LEAVE_NODE_IF_THERE:
			case ASURA_MSG_AXON_GAMESCENE_TELEPORT_TO_NODE:
			case ASURA_MSG_AXON_GAMESCENE_LEAVE_NODE:
			case ASURA_MSG_AXON_GAMESCENE_ATTRACT_TO_NODE:
			case ASURA_MSG_AXON_GAMESCENE_ALWAYS_STOP_AT_NODE:
			case ASURA_MSG_AXON_GAMESCENE_ACTOR_ATTRACT_TO_NODE:
			case ASURA_MSG_AXON_GAMESCENE_SET_ATTRACTOR_FLAGS:
			{
				Tools_Object* pxFromObject = Tools_Object_System::GetOwnerOfMessageBlock( xMsgIt.GetCurrentMessageBlockID() );
				if ( pxFromObject )
				{
					Asura_String xErrorText( "Object '" );
					xErrorText += pxFromObject->GetName();
					xErrorText += "' is sending old gamescene message ";
					xErrorText += Asura_Network::GetMsgIDDesc( pxMsg->m_usMsgID );
					xErrorText += ". Please remove this message.";

					Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Messages",
						new Tools_MessageLogAction_GotoEntity( pxFromObject->GetGuid() ),
						xErrorText.GetString() );
				}

				break;
			}
		}
	}

	return bOk;
}

bool Tools_Mode_Object::VerifyProjectors()
{
#ifdef ASURA_TARGET_SM3
	bool bOk = true;

	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		// check for broken projector data
		if (pxEntity->GetClassification() == AsuraEntityClass_Projector)
		{
			Asura_ServerEntity_Projector* pxProjector = static_cast<Asura_ServerEntity_Projector*>(pxEntity);
			int iFOV = *static_cast<const int*>(static_cast<const void*>(&pxProjector->GetParams().m_fFoV));
			if (iFOV == 0 || iFOV == 1)
			{
				Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Projectors", new Tools_MessageLogAction_GotoEntity( pxEntity->GetGuid() ),
					"Projector with GUID %u is probably corrupt - select 'Reset Damaged Data' from its overlay if so.", 
					pxEntity->GetGuid());
				bOk = false;
			}
		}
	}

	return bOk;
#else
	return true;
#endif // ASURA_TARGET_SM3
}

#ifdef ASURA_USE_AXON
bool Tools_Mode_Object::VerifyAnimTemplates()
{
	bool bOk = true;
	for ( Axon_GS2_AnimTemplate_Iterator xTemplateIter; !xTemplateIter.Done(); xTemplateIter.Next() )
	{
		Axon_Gamescene_AnimTemplate* pxTemplate = xTemplateIter.GetCurrent();
		if ( !pxTemplate ) { continue; }
		bOk &= pxTemplate->Verify();
	}
	return bOk;
}
#endif // #ifdef ASURA_USE_AXON

struct Tools_PtrAndCount
{
	const Asura_Entity_Server* m_pxLB;
	u_int                      m_uNumLinksBlocked;
};

static int ComparePtrAndCounts( const Tools_PtrAndCount& xOne, const Tools_PtrAndCount& xTwo )
{
	return static_cast< int >( xTwo.m_uNumLinksBlocked ) - static_cast< int >( xOne.m_uNumLinksBlocked );
}

void Tools_Mode_Object::OutputWaypointBlockingStats()
{
	static const char* s_pszWBSubCat = "Waypoint Blockers";

	Tools_MessageLog::DeleteMessages( Tools_MessageLog::ENTITIES, s_pszWBSubCat );

	u_int uTotalBlockers     = 0;
	u_int uTotalLinksBlocked = 0;

	static const u_int uLARGEST_BLOCKERS_CAPACITY = 5;
	Asura_LinkList< Tools_PtrAndCount > xLBList;

	for ( Asura_Hashed_List_ConstIt< Asura_ServerEntity_WaypointBlockerVolume > xWBVIter( Asura_ServerEntity_WaypointBlockerVolume::GetList() );
		!xWBVIter.Done();
		xWBVIter.Next() )
	{
		const Asura_ServerEntity_WaypointBlockerVolume* pxWBV = xWBVIter.GetCurrent();

		++uTotalBlockers;

		// Update the stats.
		const u_int uNumLinksBlocked = pxWBV->GetNumLinksToBlock();
		uTotalLinksBlocked += uNumLinksBlocked;

		// Store this if it's one of the uLARGEST_BLOCKERS_CAPACITY largest.
		if ( xLBList.Size() < uLARGEST_BLOCKERS_CAPACITY )
		{
			Tools_PtrAndCount xPnC;
            xPnC.m_pxLB             = pxWBV;
			xPnC.m_uNumLinksBlocked = uNumLinksBlocked;
			xLBList.Insert( xPnC, ComparePtrAndCounts );
		}
		else if ( uNumLinksBlocked > xLBList.Back()->Data().m_uNumLinksBlocked )
		{
			xLBList.PopBack();
			Tools_PtrAndCount xPnC;
			xPnC.m_pxLB             = pxWBV;
			xPnC.m_uNumLinksBlocked = uNumLinksBlocked;
			xLBList.Insert( xPnC, ComparePtrAndCounts );
		}
	}

	for ( Asura_Hashed_List_ConstIt< Asura_ServerEntity_PhysicalObject > xPOIter( Asura_ServerEntity_PhysicalObject::GetList() );
		!xPOIter.Done();
		xPOIter.Next() )
	{
		const Asura_ServerEntity_PhysicalObject* pxPO = xPOIter.GetCurrent();
		if ( pxPO->HasBlockWaypointLinksFlag() )
		{
			++uTotalBlockers;

			// Update the stats.
			const u_int uNumLinksBlocked = pxPO->GetNumLinksToBlock();
			uTotalLinksBlocked += uNumLinksBlocked;

			// Store this if it's one of the uLARGEST_BLOCKERS_CAPACITY largest.
			if ( xLBList.Size() < uLARGEST_BLOCKERS_CAPACITY )
			{
				Tools_PtrAndCount xPnC;
				xPnC.m_pxLB             = pxPO;
				xPnC.m_uNumLinksBlocked = uNumLinksBlocked;
				xLBList.Insert( xPnC, ComparePtrAndCounts );
			}
			else if ( uNumLinksBlocked > xLBList.Back()->Data().m_uNumLinksBlocked )
			{
				xLBList.PopBack();
				Tools_PtrAndCount xPnC;
				xPnC.m_pxLB             = pxPO;
				xPnC.m_uNumLinksBlocked = uNumLinksBlocked;
				xLBList.Insert( xPnC, ComparePtrAndCounts );
			}
		}
	}

	// Output our stats to the message log.
	Tools_MessageLog::AddMessage( Tools_MessageLog::ENTITIES, s_pszWBSubCat, "Waypoint blocker stats:" );
	Tools_MessageLog::AddMessage( Tools_MessageLog::ENTITIES, s_pszWBSubCat, "%u blockers, blocking a total of %u links.",
		uTotalBlockers, uTotalLinksBlocked );
	Tools_MessageLog::AddMessage( Tools_MessageLog::ENTITIES, s_pszWBSubCat, "This adds %0.3fK to the entity file.",
		uTotalLinksBlocked / 1024.0f );

	if ( xLBList.Size() )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::ENTITIES, s_pszWBSubCat, "The %u largest blockers are:", xLBList.Size() );
	}
	for ( Asura_LinkList_ConstIt< Tools_PtrAndCount > xLBIter( xLBList ); !xLBIter.Done(); xLBIter.Next() )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::ENTITIES, s_pszWBSubCat,
			new Tools_MessageLogAction_GotoEntity( xLBIter.GetCurrent()->m_pxLB->GetGuid() ),
			"    '%s': blocking %u links.", xLBIter.GetCurrent()->m_pxLB->ToolsData->Name.GetString(), xLBIter.GetCurrent()->m_uNumLinksBlocked );
	}
}

void Tools_Mode_Object::FixNamePrefixes()
{
	int iRenameCount = 0;
	for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList);!olif.done();olif.next())
	{
		Asura_String xName = olif()->GetName();
		if( !xName.StartsWith( olif()->GetMyNamePrefix() ) )
		{
			// Rename functionality is only present one level down, in Tools_BaseEntity_Object.
			// This also means that Tools objects that don't inherit from Tools_BaseEntity_Object won't be renamed. (AN 30/03/09)
			Tools_BaseEntity_Object* const pxBaseObject = static_cast<Tools_BaseEntity_Object*>( olif() );
			Asura_Assert( pxBaseObject, "Found an object that doesn't inherit from Tools_BaseEntity_Object, can't add prefix." );
			if( pxBaseObject )
			{
				// Store old name for display
				const Asura_String xOldName = pxBaseObject->GetName();

				// Add prefix
				xName.Prepend( pxBaseObject->GetMyNamePrefix() );
				pxBaseObject->Rename( xName.GetString() );
				iRenameCount++;

				Tools_MessageLog::AddMessage( Tools_MessageLog::ENTITIES, "Renaming", new Tools_MessageLogAction_GotoEntity( olif()->GetGuid() ),
					"Changed object name from '%s' to '%s'.", xOldName.GetString(), xName.GetString() );
			}
		}
	}

	if( iRenameCount )
	{
		Tools_Timed_Message_System::AddMessage( 0x00ff00 ,"Renamed %d object%c", iRenameCount, ( iRenameCount > 1 ) ? 's' : ' ' );
	}
	else
	{
		Tools_Timed_Message_System::AddMessage( 0x00ff00 ,"No objects renamed" );
	}
}


struct ObjectReportShapeInfo
{
	ObjectReportShapeInfo()
	{
		uShapeFileID = 0;
		uSkinFileID = 0;
		uSkinID = 0;
		szName[0] = 0;
		szFilename[0] = 0;
		uPolys = 0;
		uNumberOfInstances = 0;
		for(int i=0;i<10;i++) uLODPolys[i] = 0;
		pxShape = 0;
		pxSkin = 0;
	}
	
	Asura_Hash_ID uShapeFileID;
	Asura_Hash_ID uSkinFileID;
	Asura_Hash_ID uSkinID;
	char szName[255];
	char szFilename[MAX_PATH];
	unsigned uPolys;
	unsigned uNumberOfInstances;
	unsigned uLODPolys[10];
	Asura_Shape* pxShape;
	Asura_Hierarchy_Skin* pxSkin;
	List<Asura_Mesh*> xMeshList; //list of all meshes used across all LODs
};

static int ObjectReportSort(const void* p1,const void* p2)
{
	const ObjectReportShapeInfo* pxShape1 = *((const ObjectReportShapeInfo**)p1);
	const ObjectReportShapeInfo* pxShape2 = *((const ObjectReportShapeInfo**)p2);

	return stricmp(pxShape1->szName,pxShape2->szName);
}

struct ObjectReportTextureFlags
{
	int iValue;
	const char* szShortName;
	const char* szLongName;
};

static ObjectReportTextureFlags g_axObjectReportTextureFlags[] =
{
	AsuraTextureFlag_8BitAlpha,"8 Alpha","8 Bit Alpha",
	AsuraTextureFlag_16ColourPalette,"16 Col","16 Colour Palette",
	AsuraTextureFlag_PS2MoireFix,"Moire","PS2 Moire Fix",
	AsuraTextureFlag_NoMipMaps,"No Mip","No Mip Maps",
	AsuraTextureFlag_WIIOnly_NoMipMaps,"No WiiMip","Wii No Mip Maps",
	AsuraTextureFlag_PS2DistantTexture,"Distant","PS2 Distant Texture",
	AsuraTextureFlag_FMV,"FMV","Full Motion Video",
	AsuraTextureFlag_PS2Allow512,"Allow 512","PS2 Allow 512",
	AsuraTextureFlag_PS2AllowMipMapsOn512,"512 Mip","PS2 Allow Mipmaps on 512",
	AsuraTextureFlag_PS2DontHalveSize,"No 1/2","PS2 Don't Halve Size",
	AsuraTextureFlag_PS2DontHalveExtraForMultiplayer,"No MP 1/2","PS2 Don't Halve Extra For Multiplayer",
    AsuraTextureFlag_ReduceExtraForConsoles,"1/2 +","Reduce extra level for consoles",
	AsuraTextureFlag_HalfForXBox,"1/2 XB","Xbox Half Size",
	AsuraTextureFlag_Sharpen,"Sharp","Ps2 Sharpen",
	AsuraTextureFlag_SharpenMore,"Sharp+","PS2 Sharpen More",
	0,0,0,//terminator
};


void Tools_Mode_Object::CreateObjectReport(void)
{
	//Get a list of all the objects we are going to be dealing with
	List<Asura_ServerEntity_PhysicalObject*> xObjectList;

	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		//just interested in static objects	with shapes
		//extend to all physical objects , and check for skins as well. RAR 29/05/2008
		{
			Asura_ServerEntity_PhysicalObject* pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity(pxEntity->GetGuid());
			if(pxPhysicalObject && (pxPhysicalObject->GetShapeFileID() || pxPhysicalObject->GetSkinID()))
			{
				xObjectList.add_entry(pxPhysicalObject);
			}
		}
	}
	if(!xObjectList.size())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"There aren't any physical objects with shapes or skins");
		return;
	}

	//allocate shape info array , will need at most one shape per object
	ObjectReportShapeInfo** ppxShapeInfo = new ObjectReportShapeInfo*[xObjectList.size()];

	//Go over the objects ,and get details of the shapes/skins used
	int iNumberOfShapes = 0;
	for(LIF<Asura_ServerEntity_PhysicalObject*> xObjectLIF(&xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Asura_Hash_ID uShapeFileID = xObjectLIF()->GetShapeFileID();
		Asura_Hash_ID uSkinFileID = xObjectLIF()->GetSkinFileID();
		Asura_Hash_ID uSkinID = xObjectLIF()->GetSkinID();
		
		//got an entry for this shape/skin yet?
		bool bFound = false;
		for(int i=0;i<iNumberOfShapes;i++)
		{
			if(ppxShapeInfo[i]->uShapeFileID == uShapeFileID 
				&& ppxShapeInfo[i]->uSkinFileID == uSkinFileID
				&& ppxShapeInfo[i]->uSkinID == uSkinID)
			{
				bFound = true;
				ppxShapeInfo[i]->uNumberOfInstances++;
				break;
			}
		}	

		if(!bFound)
		{
			//a new shape then, so allocate a new entry
			ObjectReportShapeInfo* pxShapeInfo = new ObjectReportShapeInfo;
			ppxShapeInfo[iNumberOfShapes++] = pxShapeInfo;
			
			pxShapeInfo->uShapeFileID = uShapeFileID;
			pxShapeInfo->uSkinFileID = uSkinFileID;
			pxShapeInfo->uSkinID = uSkinID;
			pxShapeInfo->uNumberOfInstances = 1;

			if(uShapeFileID)
			{
				pxShapeInfo->pxShape = Asura_Shape_System::GetShape(uShapeFileID);
				if(!pxShapeInfo->pxShape)
				{
					//missing shape... oh well.
					Asura_PrintfToBfr(pxShapeInfo->szName,"Unknown Shape 0x%x",255,true,uShapeFileID);
					continue;
				}
			
				pxShapeInfo->pxShape->GetShapeName(pxShapeInfo->szName,254);
				pxShapeInfo->pxShape->GetShapeFilename(pxShapeInfo->szFilename,MAX_PATH);


				//get the mesh for poly counts
				Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pxShapeInfo->szName);
				if(!pxMesh) continue;
				
				pxShapeInfo->uPolys = pxMesh->NumberOfPolygons;
				pxShapeInfo->xMeshList.add_entry(pxMesh);

				int iCount = 0;
				while (pxMesh->m_pxNextLowerMesh &&
						iCount <= 9)
				{
					pxMesh = pxMesh->m_pxNextLowerMesh;
					pxShapeInfo->uLODPolys[iCount] = pxMesh->NumberOfPolygons;
					pxShapeInfo->xMeshList.add_entry(pxMesh);
					iCount++;
				}

			}
			else
			{
				pxShapeInfo->pxSkin = Tools_Animation_Files::GetSkinFromID(uSkinID, uSkinFileID, false);
				if(!pxShapeInfo->pxSkin)
				{
					//missing skin
					Asura_PrintfToBfr(pxShapeInfo->szName,"Unknown Skin 0x%x",255,true,uSkinID);
					continue;
				}

				Asura_PrintfToBfr(pxShapeInfo->szName, "%s", 255, true, pxShapeInfo->pxSkin->Name.GetString());
				
				Asura_Hash_ID uSkinFileID = pxShapeInfo->uSkinFileID ? pxShapeInfo->uSkinFileID : pxShapeInfo->uSkinID;
				const char* szFilename = Asura_HashedFilename_System::GetFullFilename(uSkinFileID, false);
				if(szFilename)
				{
					Asura_PrintfToBfr(pxShapeInfo->szFilename, "%s", MAX_PATH, true, szFilename);
				}


				//get the mesh(es) for poly counts
				Asura_Hierarchy_Skin* pxLODSkin = pxShapeInfo->pxSkin;

				for(int iCount=0; pxLODSkin && iCount<=10; iCount++, pxLODSkin = pxLODSkin->m_pxNextLowerSkin)
				{
					u_int uNumPolys = 0;
					if(pxLODSkin->IsSkinnedModel())
					{
						Asura_Mesh* pxMesh = pxLODSkin->GetMesh();
						if(pxMesh)
						{
							pxShapeInfo->xMeshList.add_entry(pxMesh);
							uNumPolys = pxMesh->NumberOfPolygons;
						}
					}
					else
					{
						for(int i=0;i<pxLODSkin->NumberOfBones;i++)
						{
							Asura_Mesh* pxMesh = pxLODSkin->SectionData[i].Mesh;
							if(pxMesh)
							{
								pxShapeInfo->xMeshList.add_entry(pxMesh);
								uNumPolys += pxMesh->NumberOfPolygons;
							}
						}
					}
					if(iCount == 0)
					{
						//main skin
						pxShapeInfo->uPolys = uNumPolys;
					}
					else
					{
						//LOD skin
						pxShapeInfo->uLODPolys[iCount-1] = uNumPolys;
					}
				}

			}
		}
	}

	//sort the shapes
	qsort(ppxShapeInfo,iNumberOfShapes,sizeof(ObjectReportShapeInfo*),ObjectReportSort);

	// Begin output to text file:

	//Build the filename for the report by appending _InstanceReport.txt onto the
	//name of the asura file
	Asura_String	xEnvPath;
	Asura_String	xEnvFilename;
	Asura_String 	xFullEnvironmentFilename = Tools_SaveAsura::IsEditingMission() ? Tools_SaveAsura::GetMissionFilenameWithoutExtension() : Tools_SaveAsura::GetMainFilename();

	xFullEnvironmentFilename.SplitFilename(&xEnvPath, &xEnvFilename, NULL);
	xEnvPath += "\\Logfiles\\";
	Asura_String xLogFilename = xEnvPath;
	xLogFilename +=  xEnvFilename;
	xLogFilename +=  "_ShapeInstanceReport.csv";

	//First do report on shapes and polygon counts
	Tools_FileSystem::EnsureDirectoryExists(xLogFilename.GetString());
	FILE* pxFile = fopen(xLogFilename.GetString(),"wt");
	if (pxFile)
	{
	
		
		fprintf(pxFile,"Shape/Skin instances used in %s:\n\n",xFullEnvironmentFilename.GetString());		
		fprintf(pxFile,"%s%s,,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n\n","Name","Type","Filename","Num.Inst","Tot.Polys","Polygons","LOD#1","LOD#2","LOD#3","LOD#4","LOD#5","LOD#6","LOD#7","LOD#8","LOD#9","LOD#10");
		unsigned uSummaryTotalShapes = 0;
		unsigned uSummaryTotalPolys = 0;

		for (int i=0;i<iNumberOfShapes;i++)
		{
			ObjectReportShapeInfo* pxShapeInfo = ppxShapeInfo[i];

			const char* szType = pxShapeInfo->uShapeFileID ? "Shape" : "Skin";

			unsigned uTotalPolys = pxShapeInfo->uNumberOfInstances * pxShapeInfo->uPolys;
			uSummaryTotalPolys += uTotalPolys;
			uSummaryTotalShapes += pxShapeInfo->uNumberOfInstances;
			fprintf(pxFile,"%s,%s,%s,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",pxShapeInfo->szName, szType, pxShapeInfo->szFilename, pxShapeInfo->uNumberOfInstances, uTotalPolys, pxShapeInfo->uPolys,
				pxShapeInfo->uLODPolys[0], pxShapeInfo->uLODPolys[1], pxShapeInfo->uLODPolys[2], pxShapeInfo->uLODPolys[3], pxShapeInfo->uLODPolys[4], 
				pxShapeInfo->uLODPolys[5], pxShapeInfo->uLODPolys[6], pxShapeInfo->uLODPolys[7], pxShapeInfo->uLODPolys[8], pxShapeInfo->uLODPolys[9]);
		}
		fprintf(pxFile, "\n");
		fprintf(pxFile, "%s,,,%d,%d", "Total", uSummaryTotalShapes, uSummaryTotalPolys);
	}
  	fclose(pxFile);
	Tools_Timed_Message_System::AddMessage(0x00ff00,"%s written",xLogFilename.GetString());

	//And now , a logfile of textures used by the shapes
	xLogFilename = xEnvPath;
	xLogFilename +=  xEnvFilename;
	xLogFilename +=  "_ShapeTextureReport.csv";

	Tools_FileSystem::EnsureDirectoryExists(xLogFilename.GetString());
	pxFile = fopen(xLogFilename.GetString(),"wt");
	if (pxFile)
	{
		fprintf(pxFile,"Shape/Skin textures used in %s:\n\n",xFullEnvironmentFilename.GetString());		


		//write heading
		fprintf(pxFile,"Shape,Filename,Texture,Filename,Width,Height");
		for(ObjectReportTextureFlags* pxFlagData = g_axObjectReportTextureFlags;pxFlagData->iValue;pxFlagData++)
		{
			fprintf(pxFile,",%s",pxFlagData->szShortName);
		}
		fprintf(pxFile,"\n\n");

		//output textures for each shape/skin
		for (int i=0;i<iNumberOfShapes;i++)
		{
			ObjectReportShapeInfo* pxShapeInfo = ppxShapeInfo[i];

			fprintf(pxFile,"%s,%s",pxShapeInfo->szName,pxShapeInfo->szFilename);
	
			if(!pxShapeInfo->xMeshList.size())
			{
				if(pxShapeInfo->pxSkin)
				{
					//a mesh-less skin
					fprintf(pxFile,",None\n");
				}
				else
				{
					fprintf(pxFile,",Unknown\n");
				}
				continue;
			}

			//find all the textures used
			List<int> xTextureList;
			for(LIF<Asura_Mesh*> xMeshLIF(&pxShapeInfo->xMeshList); !xMeshLIF.done(); xMeshLIF.next())
			{
				Asura_Mesh* pxMesh = xMeshLIF();
				for(int j=0;j<pxMesh->NumberOfPolygons;j++)
				{
					int iMaterialID = pxMesh->TextureDataPtr[j].m_iMaterialID;
					if(iMaterialID == ASURA_MATERIAL_NONE) continue;

					for(int k=0;k<ASURA_MATERIAL_MAXTEXTURES;k++)
					{
						int iTextureID = Asura_Material_System::GetTextureID(iMaterialID,k);
						if(iTextureID != ASURA_TEXTURE_NONE && !xTextureList.contains(iTextureID))
						{
							xTextureList.add_entry(iTextureID);
						}
						Asura_Material_System::GetNormalMapID(iMaterialID,k);
						if(iTextureID != ASURA_TEXTURE_NONE && !xTextureList.contains(iTextureID))
						{
							xTextureList.add_entry(iTextureID);
						}
					}

				}
			}
			if(!xTextureList.size())
			{
				fprintf(pxFile,",None\n");
				continue;
			}

			bool bFirstTexture = true;
			for(LIF<int> xTextureLIF(&xTextureList);!xTextureLIF.done();xTextureLIF.next())
			{
				if(!bFirstTexture)
				{
					//need to skip a column
					fprintf(pxFile,",");
				}
				bFirstTexture = false;

				
				int iTextureID = xTextureLIF();

				//texture name and path
				const Asura_String* pxName = Asura_Texture_Management::GetTextureName(iTextureID);
				if(!pxName || pxName->IsBlank())
				{
					fprintf(pxFile,",Unknown Texture\n");
					continue;
				}
				fprintf(pxFile,",%s,%s",Tools_FileSystem::StripPath(pxName->GetString()),pxName->GetString());

				//dimensions
				int iWidth = 0;
				int iHeight = 0;
				Tools_Texture_Management::GetTextureSize(iTextureID,iWidth,iHeight);
				fprintf(pxFile,",%d,%d",iWidth,iHeight);

				//texture flags
				int iFlags = Asura_Texture_Management::GetTextureFlags(iTextureID);
				for(ObjectReportTextureFlags* pxFlagData = g_axObjectReportTextureFlags;pxFlagData->iValue;pxFlagData++)
				{
					if(pxFlagData->iValue & iFlags)
						fprintf(pxFile,",X");
					else
						fprintf(pxFile,",");
				}
				fprintf(pxFile,"\n");
			}
		}

		//write texture flag key
		fprintf(pxFile,"\nFlags\n");
		for(ObjectReportTextureFlags* pxFlagData = g_axObjectReportTextureFlags;pxFlagData->iValue;pxFlagData++)
		{
			fprintf(pxFile,"%s,%s\n",pxFlagData->szShortName,pxFlagData->szLongName);
		}
	}

  	fclose(pxFile);
	Tools_Timed_Message_System::AddMessage(0x00ff00,"%s written",xLogFilename.GetString());



	for(int i=0;i<iNumberOfShapes;i++)
	{
		delete ppxShapeInfo[i];
	}
	delete [] ppxShapeInfo;
}



void Tools_Mode_Object::BuildObjectCreationList()
{
#ifdef ASURA_USE_BOTH_START_POSITION_TYPES
	AddObjectCreationFunction("Start Pos (Single Player)", "Misc", "", Tools_Entity_StartPoint::Project_Create, TOT_Entity_StartPoint);
	AddObjectCreationFunction("Start Pos (Multiplayer)","Misc","",Tools_StartPosition_Object::CreateStartPosition,TOT_StartPosition);
#else//ASURA_USE_BOTH_START_POSITION_TYPES
#ifndef ASURA_USE_TOOLS_START_POSITION_OBJECTS
	AddObjectCreationFunction("Start Point", "Misc", "", Tools_Entity_StartPoint::Project_Create, TOT_Entity_StartPoint);
#else
	AddObjectCreationFunction("Start Position","Misc","",Tools_StartPosition_Object::CreateStartPosition,TOT_StartPosition);
#endif //ASURA_USE_TOOLS_START_POSITION_OBJECTS
#endif // !ASURA_USE_BOTH_START_POSITION_TYPES
	AddObjectCreationFunction("Time Trigger","Trigger","",Tools_Entity_TimeTrigger_Object::Project_Create,TOT_Entity_TimeTrigger);
	AddObjectCreationFunction("Timeline Trigger","Trigger","",Tools_Entity_TimelineTrigger_Object::Create,TOT_Entity_TimelineTrigger);
	AddObjectCreationFunction("Splitter Block","Trigger","",Tools_Entity_SplitterBlock_Object::Create,TOT_Entity_SplitterBlock);
	AddObjectCreationFunction("Counted Trigger","Trigger","",Tools_Entity_CountedTrigger_Object::Create,TOT_Entity_CountedTrigger);
	AddObjectCreationFunction("Glass","Physical Object","",CreateGlassInitOverlay,TOT_Entity_Glass);
	AddObjectCreationFunction( "Static object", "Physical Object", "", Tools_Entity_PhysicsObject::Project_CreateStatic, TOT_Entity_PhysicsObject );
	AddObjectCreationFunction( "Physics object", "Physical Object", "", Tools_Entity_PhysicsObject::Project_Create, TOT_Entity_PhysicsObject );
#ifndef ASURA_TARGET_SM3
	AddObjectCreationFunction("Destructible Light","Light","",Tools_Entity_DestructibleLight::Project_Create,TOT_Entity_DestructibleLight);
	AddObjectCreationFunction("Advanced Light","Light","",Tools_Entity_AdvancedLight::Project_Create,TOT_Entity_AdvancedLight);
#endif
	AddObjectCreationFunction("Lift","Physical Object","",Tools_Entity_Lift::Project_Create,TOT_Entity_Lift);
	AddObjectCreationFunction("Advanced Volume Trigger","Trigger","aka Advanced Volume Trigger",Tools_Entity_AdvancedVolumeTrigger::Project_Create,TOT_Entity_AdvancedVolumeTrigger);
#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
	AddObjectCreationFunction("Meta Music Trigger","Sound & Music","",Tools_Entity_MetaMusicTrigger_Object::Create,TOT_Entity_MetaMusicTrigger);
#endif
	AddObjectCreationFunction("FMV Trigger","Sound & Music","",Tools_Entity_FMVTrigger_Object::Create,TOT_Entity_FMVTrigger);
	AddObjectCreationFunction("Look-at Trigger","Trigger","",Tools_Entity_LookAtTrigger_Object::Create,TOT_Entity_LookAtTrigger);
    AddObjectCreationFunction("Clock Trigger","Trigger","",Tools_Entity_ClockTrigger_Object::Create,TOT_Entity_ClockTrigger);
	AddObjectCreationFunction("Logic Trigger","Trigger","",Tools_Entity_LogicTrigger_Object::Create,TOT_Entity_LogicTrigger);
	AddObjectCreationFunction("Client Volume Trigger","Trigger","",Tools_Entity_ClientVolumeTrigger_Object::Create,TOT_Entity_ClientVolumeTrigger);
	AddObjectCreationFunction("Env Texture Anim Controller","Animation","Entity that forwards texture animation messages to its assigned animation",Tools_Entity_EnvTextureAnimControl::Create,TOT_Entity_EnvTextureAnimControl);
	AddObjectCreationFunction("Counted Volume Trigger","Trigger","",Tools_Entity_CountedVolumeTrigger::Project_Create,TOT_Entity_CountedVolumeTrigger);
	AddObjectCreationFunction("Multi Damage Volume","Trigger","",Tools_Entity_MultiDamageVolume::Project_Create,TOT_Entity_MultiDamageVolume);
	AddObjectCreationFunction("Proxy Trigger","Trigger","",Tools_Entity_ProxyTrigger_Object::Create,TOT_Entity_ProxyTrigger);

	AddObjectCreationFunction("Effect Volume","Trigger","",Tools_Entity_EffectVolume::Create,TOT_Entity_EffectVolume);

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
	AddObjectCreationFunction("Debug Message Trigger","Trigger","Entity which will display a text message on screen when triggered. Useful for debugging purposes.",Tools_Entity_DebugMessageTrigger_Object::Create,TOT_Entity_DebugMessageTrigger);
	AddObjectCreationFunction("Debug Waypoint Trigger","Trigger","Entity which will display a text message and draw a shape on screen when triggered. Useful for design prototyping.",Tools_Entity_DebugWaypointTrigger_Object::Create,TOT_Entity_DebugWaypointTrigger);
#endif

	AddObjectCreationFunction("PFX","Special FX","",Tools_Entity_PFX_Effect::Create,TOT_Entity_PFX_Effect);

#ifdef ASURA_USE_DIALOGUE_SYSTEM
	AddObjectCreationFunction("Dialogue Trigger","Sound & Music","",Tools_Entity_DialogueTrigger::Create,TOT_Entity_DialogueTrigger);
	AddObjectCreationFunction("Dialogue Controller","Sound & Music","",Tools_Entity_DialogueController::Create,TOT_Entity_DialogueController);
#endif


	AddObjectCreationFunction("Tools Helper","Tools","",Tools_HelperObject_Object::Create,TOT_HelperObject);

#ifdef ASURA_USE_CAMERA_VOLUME
	AddObjectCreationFunction("Camera Volume", "Misc", "", Tools_Entity_CameraVolume::Project_Create, TOT_Entity_CameraVolume);
#endif //ASURA_USE_CAMERA_VOLUME

	AddObjectCreationFunction("Lift Node", "Misc", "", Tools_EntityNode_LiftNode::Create, TOT_Entity_LiftNode);

#ifdef ASURA_USE_AXON
	AddObjectCreationFunction("Coverpoint Node", "Axon", "", Tools_EntityNode_CoverpointNode::Create, TOT_Entity_CoverpointNode);
	AddObjectCreationFunction("Guard Zone", "Axon", "", Tools_EntityNode_GuardZone::Create, TOT_Entity_GuardZone);
	AddObjectCreationFunction("GS2 Node", "Axon", "", Tools_EntityNode_GS2Node::Create, TOT_Entity_GS2Node);
	AddObjectCreationFunction("GS2 Attractor", "Axon", "", Tools_EntityNode_GS2Attractor::Create, TOT_Entity_GS2Attractor);
#endif // ASURA_USE_AXON

#if defined(ASURA_USE_AFFECTORS)
	AddObjectCreationFunction("Affector","Affector","",Tools_Entity_Affector::Create,TOT_Entity_Affector);
	AddObjectCreationFunction("Wind Affector","Affector","",Tools_Entity_Affector_Wind::Create,TOT_Entity_Affector_Wind);
#endif	// defined(ASURA_USE_AFFECTORS)

#ifdef ASURA_TARGET_SM3
	AddObjectCreationFunction("Projector","Special FX","",Tools_Entity_Projector::Create,TOT_Entity_Projector);
	AddObjectCreationFunction("Dynamic Light","Light","",Tools_Entity_DynamicLight::Create,TOT_Entity_DynamicLight);
	AddObjectCreationFunction("Light Shaft","Special FX","",Tools_Entity_LightShaft::Create,TOT_Entity_LightShaft);
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
	AddObjectCreationFunction("Teleporter","Teleporter","",Tools_Entity_Teleporter::Create,TOT_Entity_Teleporter);
	AddObjectCreationFunction("Teleport Destination","Teleporter","",Tools_Entity_TeleportDestination::Create,TOT_Entity_TeleportDestination);
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

	AddObjectCreationFunction("Force Field - Conveyor","ForceField-Conveyor","",Tools_Entity_ForceField_Conveyor::Create,TOT_Entity_ForceField_Conveyor);

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	AddObjectCreationFunction("Console Variable", "Misc", "", Tools_Entity_ConsoleVar::Create, TOT_Entity_ConsoleVar);
#endif //ASURA_SUPPORT_COMMAND_CONSOLE

	AddObjectCreationFunction("Waypoint Blocker Volume", "Axon", "", Tools_Entity_WaypointBlockerVolume::Create,TOT_Entity_WaypointBlockerVolume);

	AddObjectCreationFunction("Objective", "Misc", "", Tools_Entity_Objective::Create,TOT_Entity_Objective);

#ifdef ASURA_USE_CONTAINER_SYSTEM
	AddObjectCreationFunction("Container", "Misc", "", Tools_Entity_Container::Create, TOT_Entity_Container);			
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	AddObjectCreationFunction("FMV Controller","Sound & Music","",Tools_Entity_FMVController::Create,TOT_Entity_FMVController);
	AddObjectCreationFunction("FMV Speaker","Sound & Music","", Tools_Entity_FMVSpeaker::Create, TOT_Entity_FMVSpeaker);
#endif
}

void Tools_Mode_Object::AddObjectCreationFunction(const char* pszName,const char* pszCategory,const char* pszHelpText,void (*pfnObjectCreate)(),int iToolsObjectType)
{
	Asura_Assert(pfnObjectCreate,"");
	ToolsObjectCreationInfo* pxInfo = new ToolsObjectCreationInfo;
	pxInfo->m_xCategory = pszCategory;
	pxInfo->m_xName = pszName;
	pxInfo->m_pfnObjectCreate = pfnObjectCreate;
	pxInfo->m_pfnObjectCreateWithData = NULL;
	pxInfo->m_pData = NULL;

	return AddObjectCreationFunctionMain(pxInfo, pszName, pszCategory, pszHelpText, iToolsObjectType);
}

void Tools_Mode_Object::AddObjectCreationFunctionWithData(const char* pszName,const char* pszCategory,const char* pszHelpText,void (*pfnObjectCreate)(void* pSetupData),int iToolsObjectType, void* pData)
{
	Asura_Assert(pfnObjectCreate,"");
	ToolsObjectCreationInfo* pxInfo = new ToolsObjectCreationInfo;
	pxInfo->m_xCategory = pszCategory;
	pxInfo->m_xName = pszName;
	pxInfo->m_pfnObjectCreateWithData = pfnObjectCreate;
	pxInfo->m_pfnObjectCreate = NULL;
	pxInfo->m_pData = pData;

	return AddObjectCreationFunctionMain(pxInfo, pszName, pszCategory, pszHelpText, iToolsObjectType);
}

void Tools_Mode_Object::AddObjectCreationFunctionMain(	ToolsObjectCreationInfo* pxInfo, const char* pszName,const char* pszCategory,const char* pszHelpText, int iToolsObjectType)
{
	Asura_Assert(pszCategory,"");
	Asura_Assert(pszName,"");

	pxInfo->m_xHelpText = pszHelpText ? pszHelpText : "";
	pxInfo->m_iToolsObjectType = iToolsObjectType;

	//add this to the list , sorting by name
	LIF<ToolsObjectCreationInfo*> xObjLIF(&s_xObjectCreationList);

	for(;!xObjLIF.done();xObjLIF.next())
	{
		if( pxInfo->m_xName.Compare(xObjLIF()->m_xName, true) <= 0 )
		{
			s_xObjectCreationList.add_entry_before( pxInfo, xObjLIF() );
			break;
		}
	}
	if(xObjLIF.done())
	{
		s_xObjectCreationList.add_entry(pxInfo);
	}
}


void Tools_Mode_Object::RemoveObjectCreationFunction( int iToolsObjectType )
{
    while( true )
    {
        ToolsObjectCreationInfo* pxInfo = GetObjectCreationFunction( iToolsObjectType );
        if( pxInfo )
        {
            s_xObjectCreationList.delete_entry( pxInfo );
            delete pxInfo;
        }
        else
        {
            break;
        }
    }
}


ToolsObjectCreationInfo *Tools_Mode_Object::GetObjectCreationFunction( int iToolsObjectType )
{
	for(LIF<ToolsObjectCreationInfo*> xObjLIF(&s_xObjectCreationList);!xObjLIF.done();xObjLIF.next())
	{
		if (xObjLIF()->m_iToolsObjectType == iToolsObjectType)
			return xObjLIF();
	}

	return NULL;
}

void Tools_Mode_Object::BuildObjectCategoryStrings()
{
	List<const char*> xCatList;

	//search for distinct category names , and add them to a sorted list

	for(LIF<ToolsObjectCreationInfo*> xObjLIF(&s_xObjectCreationList);!xObjLIF.done();xObjLIF.next())
	{
		const char* pszCatName = xObjLIF()->m_xCategory.GetString();

		LIF<const char*> xCatLIF(&xCatList);
		for(;!xCatLIF.done();xCatLIF.next())
		{
			int iCompare = stricmp(pszCatName,xCatLIF());
			if(iCompare == 0)
			{
				break;
			}
			else if(iCompare < 0)
			{
				xCatList.add_entry_before(pszCatName,xCatLIF());
				break;
			}
		}
		if(xCatLIF.done())
		{
			xCatList.add_entry(pszCatName);
		}
	}

	//entry to show all object types
	xCatList.add_entry_start("*All*");


	//copy names into an array
	s_iNumObjectCategories = xCatList.size();
	s_ppszObjectCategoryNames = new const char*[s_iNumObjectCategories];

	int iIndex = 0;
	for(LIF<const char*> xCatLIF(&xCatList);!xCatLIF.done();xCatLIF.next())
	{
		s_ppszObjectCategoryNames[iIndex++] = xCatLIF();		
	}

	s_iSelectedObjectCategory = 0;
}

void Tools_Mode_Object::UpdateObjectTypeList()
{
	//get the current selection (if any) before deleting the list
	const char* pszSelection = 0;
	if(s_iSelectedObjectType < s_iNumObjectTypes)
	{
		pszSelection = s_ppszObjectTypeNames[s_iSelectedObjectType];
	}

	//delete old strings
	s_iNumObjectTypes = 0;
	delete [] s_ppszObjectTypeNames;
	s_ppszObjectTypeNames = 0;
	delete [] s_ppszObjectHelpText;
	s_ppszObjectHelpText = 0;
	delete [] s_ppxObjectCreationArray ;
	s_ppxObjectCreationArray = 0;


	//get a list of all object types in the current category
	const char* pszCatName = s_ppszObjectCategoryNames[s_iSelectedObjectCategory];

	List<ToolsObjectCreationInfo*> xObjList;

	for(LIF<ToolsObjectCreationInfo*> xObjLIF(&s_xObjectCreationList);!xObjLIF.done();xObjLIF.next())
	{
		bool bAdd = false;
		if(s_iSelectedObjectCategory == 0)
		{
			//first entry is 'all' categories , so everything should be added
			bAdd = true;
		}
		else if(!stricmp(pszCatName,xObjLIF()->m_xCategory.GetString()))
		{
			bAdd = true;
		}
		if(!bAdd) continue;

		//should probably add this one , but make sure an equivalent entry hasn't already been added
		for(LIF<ToolsObjectCreationInfo*> xObjLIF2(&xObjList);!xObjLIF2.done();xObjLIF2.next())
		{
			if(xObjLIF()->m_pfnObjectCreate == xObjLIF2()->m_pfnObjectCreate &&
				!stricmp(xObjLIF()->m_xName.GetString(),xObjLIF2()->m_xName.GetString()))
			{
				//same name , and creation function ,so skip this after all
				bAdd = false;
				break;
			}

		}
		if(!bAdd) continue;

		xObjList.add_entry(xObjLIF());
	}

	//should always find at least one object , unless there are no object types at all.. which shouldn't be happening
	Asura_Assert(xObjList.size(),"");

	//build the various string arrays etc
	s_iNumObjectTypes = xObjList.size();
	s_ppszObjectTypeNames = new const char*[s_iNumObjectTypes];
	s_ppszObjectHelpText = new const char*[s_iNumObjectTypes];
	s_ppxObjectCreationArray = new ToolsObjectCreationInfo*[s_iNumObjectTypes];

	//also check for index of previous selection while building the list
	s_iSelectedObjectType = 0;
		
	int iIndex = 0;
	for(LIF<ToolsObjectCreationInfo*> xObjLIF(&xObjList);!xObjLIF.done();xObjLIF.next(),iIndex++)
	{
		s_ppszObjectTypeNames[iIndex] = xObjLIF()->m_xName.GetString();
		s_ppszObjectHelpText[iIndex] = xObjLIF()->m_xHelpText.GetString();
		s_ppxObjectCreationArray[iIndex] = xObjLIF();

		if(pszSelection && !stricmp(pszSelection,s_ppszObjectTypeNames[iIndex]))
		{
			s_iSelectedObjectType = iIndex;
		}
	}

	//update the strings in list dialog...
	if(s_pxObjectListDialog)
	{
		s_pxObjectListDialog->ReplaceStrings(s_iNumObjectTypes,s_ppszObjectTypeNames);
		s_pxObjectListDialog->SetListHelpStrings(s_ppszObjectHelpText);
		s_pxObjectListDialog->SetSelection(s_iSelectedObjectType);
	}

	
}


void Tools_Mode_Object::CreateObject(bool bSnapToVertex)
{
	//various arrays etc may not exist if overlay isn't open
	if(!ObjectModeOverlayOpen) return;

	//call the object creation function for the item currently selected in the list
	Asura_Assert(s_ppxObjectCreationArray,"");
	Asura_Assert(s_iSelectedObjectType<s_iNumObjectTypes,"");

	Tools_SecondaryEntityFile_System::PreCreateObject();

	if(s_ppxObjectCreationArray[s_iSelectedObjectType]->m_pData == NULL)
	{
		s_ppxObjectCreationArray[s_iSelectedObjectType]->m_pfnObjectCreate();
	}
	else
	{
		s_ppxObjectCreationArray[s_iSelectedObjectType]->m_pfnObjectCreateWithData(s_ppxObjectCreationArray[s_iSelectedObjectType]->m_pData);
	}

	//check for bugged object constructors...
	Tools_Object_System::CheckForMessageBlockOwnershipProblems();

	Tools_SecondaryEntityFile_System::PostCreateObject();

	if(bSnapToVertex) Tools_Object_System::StartSnapToVertex();
}
