#include "Core/Asura.h"
#include "Objects/Tools_Object.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Env_System.h"
#include "Mesh/Asura_Env.h"
#include "Physics/Asura_Physics.h"
#include "Physics/Asura_Physics_MovementQuery.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Module.h"


#include "Rendering/Asura_PlatformRenderObject.h"

#include "Tools_System.h"
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_Entity_Trigger_Object.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Render_Options.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Object_Gizmo.h"
#include "Objects/Tools_Object_System.h"
#include "Entity/Tools_MessageEditor.h"


//extern Asura_Dynamic_Object* PlayerDynamicObject;

#ifndef ABS
#define ABS(x) ((x) > 0 ? (x) : -(x))
#endif

#define TOOLS_USE_PHYSICS


///////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Tools_Object **
**--------------*/
int Tools_Object::s_iNextID = 0;

bool Tools_Object::s_bAllowChildObjectSelection = false;
bool Tools_Object::s_bOverrideAnimTimers = false;
float Tools_Object::s_fAnimTimerOverride = 0.0f;
Asura_Vector_2 Tools_Object::s_xOverlayPosition(10,10);

Tools_Object::Tools_Object(int type /*= TOT_Anon*/)
{
	Tools_Object_System::ObjectList.add_entry(this);
	Tools_Object_System::EndMessageBlockEditing();

	RelativePosition.Zero();
	RelativeOrientation.Identity();

	Position.Zero();
	Orientation.Identity();

	Mesh = 0;
	m_iDistantIconTexture = ASURA_TEXTURE_NONE;
	m_xDistantIconOffset.Zero();
	m_bInitialisedDistantIconTexture = false;

	m_pxAnimation = 0;
	m_bUseAnimMovement = false;
	Type = type;
	HiddenFlags = 0;
	m_bSelectable = true;
	m_fAlpha = 1.0f;
	m_bBeingDestroyed = false;
	m_fScaleWhenCloseDist = 0.0f;
	SetScale(1.0f);
	m_iTextureID = ASURA_TEXTURE_NONE;
	m_uColour = 0xffffff;

	m_iID = s_iNextID++;
	m_bPreviousPositionValid = false;
	m_fStaticTimer = Asura_Maths::LargeFloat;
	m_pxObjectToSelectInstead = NULL;

#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	m_iLastKnownModule = -1;

	m_uVisCacheFrameStamp = 0xffffffff;
	m_xVisCacheBB.Zero();
	m_bCachedIsVisible = true;
#endif

	m_fVerticalSpeed = 0.0f;


	m_bGroupSelected = false;
	m_bSelectedByChooseObject = false;
	m_bShouldBeRenderedThisFrame = false;
}

Tools_Object::~Tools_Object()
{
	Asura_Assert(!IsSelected() && !IsGroupSelected(),"");//(should be deselected before being deleted)

	while(m_xReferenceList.size())
	{
		Tools_Object* pxObject = m_xReferenceList.first_entry();
		m_xReferenceList.delete_first_entry();

		pxObject->ReferencedObjectIsBeingDeleted(this);
	}

	Tools_Object_System::ObjectDeleted(this);

	delete m_pxAnimation;
}

void Tools_Object::RenderMeshOrAnimation(Asura_Mesh *pxMesh, Asura_Mesh *pxMeshToRender, Asura_Animation *pxAnimation)
{
	u_int uOutlineColour = 0;
	Tools_Object* pxRootObject = GetObjectToSelect();
	if(pxRootObject->ShouldRenderSelectionOutline())
	{
		if(pxRootObject->RenderAsSelected())
		{
			//indicate the selectedness by drawing red polygon outlines
			uOutlineColour = 0xffff0000;
		}
		else if(pxRootObject->IsGroupSelected())
		{
			uOutlineColour = 0xff0000ff;
		}
	}

	if(pxAnimation)
	{

		u_int uRenderFlags = 0;
		if(!ShouldRenderShadow())
		{
			uRenderFlags |= ASURA_HIERARCHY_RENDER_NO_SHADOW;
		}
		if(!UseModuleLinking())
		{
			uRenderFlags |= ASURA_HIERARCHY_RENDER_NO_MODULELINK_TEST;
		}

		pxAnimation->SetOutlineColour(uOutlineColour);

		if(ShouldRenderSelectionOutlineOnly() && !uOutlineColour)
		{
			//Haven't got the means of just doing the outline for animations at the moment ,but if
			//we are supposed to be doing the outline only and there is no outline , then we can
			//skip the rendering part.
			//RAR 09/08/2007
			uRenderFlags |= ASURA_HIERARCHY_RENDER_DONTRENDER;
		}

		Asura_Vector_3 xPos;
		Asura_Matrix_3x3 xMat;
		GetAnimatedPosition(&xPos,&xMat);

		pxAnimation->SetAnimTexID(m_iTextureID);
		if(s_bOverrideAnimTimers)
		{
			pxAnimation->m_xPrimaryWrapper.SetTimer(s_fAnimTimerOverride);
			pxAnimation->Update(xPos, xMat,0.0f,uRenderFlags);
		}
		else
		{
			pxAnimation->Update(xPos, xMat,ANIMTIME_USEGAMETIME,uRenderFlags);
		}

		if(!ShouldRenderSelectionOutlineOnly())
		{
			// render the shadow too (which will have different culling, potentially)
			pxAnimation->RenderShadow( xPos, xMat, uRenderFlags );
		}
	}
	else if (pxMesh && pxMeshToRender)
	{
		//Check to see if the mesh is used by a skin.
		//If it is , make sure that the skin's normals have been calculated.
		//Doing this will have the side-effect of putting the mesh into the rest pose. RAR 17/03/03
		Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(pxMeshToRender->Name.GetString());
		if(pxSkin && !pxSkin->NormalData)
		{
			pxSkin->CalculateSkinNormalData();
		}

		if(!pxMesh->BoundingBoxDataValid) pxMesh->CalculateBoundingBox();

		//check that the shape is on screen.
		Asura_Bounding_Box xScaledBB = pxMesh->BoundingBox;
		if(IsScaled())
		{
			xScaledBB.MaxX *= m_xScale.x;
			xScaledBB.MinX *= m_xScale.x;
			xScaledBB.MaxY *= m_xScale.y;
			xScaledBB.MinY *= m_xScale.y;
			xScaledBB.MaxZ *= m_xScale.z;
			xScaledBB.MinZ *= m_xScale.z;
		}

		float fRadius = xScaledBB.CalculateRadius();

		Asura_Vector_3 xCentre(Position + (xScaledBB.CalculateCentre() * Orientation));
		if(!Asura_Camera::IsSphereInFrustum(xCentre,fRadius)) return;


		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);

		if(!ShouldRenderSelectionOutlineOnly())
		{
			Asura_Direct3D_ObjectRenderer::ToolsRender( pxMeshToRender, Position, Orientation, UseLighting(), m_fAlpha, m_xScale, m_iTextureID, m_uColour );
		}

		if(uOutlineColour)
		{
			//indicate the selectedness by drawing red polygon outlines
			Tools_Render::RenderMeshOutline(pxMeshToRender,&Position,&Orientation,uOutlineColour, true, m_xScale);
			
			// (AMM 7/11/08) check type of tools objects that we are rendering and render any type specific information
			switch (Type)
			{
				case TOT_Entity_LookAtTrigger:
				{
					Tools_Entity_LookAtTrigger_Object* pxTrigger = static_cast<Tools_Entity_LookAtTrigger_Object*>(this);
					
					if (Tools_Entity_LookAtTrigger_Object::ShowCone())
					{
						float fAngle = pxTrigger->GetAngle();
						bool bWireframe = Tools_Entity_LookAtTrigger_Object::WireframeCone();
					
						// render cone which represents the look-at angle
						Tools_Render::RenderCone(Position,Orientation,2.0f,fAngle,uOutlineColour,bWireframe);
					}
					break;
				}
				default: // default is to do no additional rendering
					break;
			}
		}
	}
}

void Tools_Object::Render()
{
	if(!m_bInitialisedDistantIconTexture)
	{
		InitialiseDistantIconTexture();
	}


	//check for rendering as an icon
	if(IsBeingRenderedAsIcon())
	{
		if(IsSelected())
		{
			//indicate selected phonon by flashing it on/off
			static float s_fFlashTimer = 0.0f;
			s_fFlashTimer += Asura_Timers::GetFrameTime();
			while(s_fFlashTimer>0.8f) s_fFlashTimer-=0.8f;
			if(s_fFlashTimer<0.15f) return;//currently in off state
		}
		Asura_Vector_3 xIconPos = Position + m_xDistantIconOffset*Orientation;
		Tools_Render::AddIconToBatch( xIconPos , m_iDistantIconTexture, Tools_Render_Options::GetDistantIconSize());
	
		return;	
	}

	if (!m_pxAnimation)
	{
		if(!Mesh)
		{
			Asura_Assertf(0,("Tools object %s of type %s hasn't been assigned any shape or animation.\nAssigning default shape",GetName(),GetTypeDescription()));
			Mesh = Asura_Mesh_System::GetMesh("ToolsShape_CutsceneEvent");
			if(!Mesh)
			{
				Asura_Assert(0,"Failed to find default shape either.\nToo bad.");
				return;
			}
		}
		RenderMeshOrAnimation(Mesh, GetLODMesh(), NULL);
	} else {
		RenderMeshOrAnimation(NULL, NULL, m_pxAnimation);
	}
}


void Tools_Object::UpdateMovement(float* pMovementScale,float* pRotationScale)
{
	//make sure our position/orientation is up to date
	UpdatePositionFromAsuraData();
	
	if(!Tools_Movement::IsUsingDynamics())
	{
		m_fVerticalSpeed = 0.0f;
	}
	
	
	//get the rotation for this frame	
	Asura_Euler_3 EulerRotate = GetRotationForFrame();
	EulerRotate.x *= *pRotationScale;
	EulerRotate.y *= *pRotationScale;
	EulerRotate.z *= *pRotationScale;

	bool movedThisFrame = false;

	//determine if orbit movement should be done.
	//(ignore whether any rotation is actually being done here , so that the centre
	//remains locked in if the mouse stands still for a few frames)
	bool bOrbitMovement = false;
	bool bMouseOrbitMovement = false;
	Asura_Vector_3 xOrbitCentre;
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPADDIVIDE) || Asura_UserInput::KeyPressed('/')
		||(Tools_Movement::IsUsingMouseRotation() && Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL)))
	{
		bOrbitMovement = true;
		bMouseOrbitMovement = (Tools_Movement::IsUsingMouseRotation() && Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL));
		xOrbitCentre = Tools_Movement::GetOrbitCentre();
	}
	
	
	if(EulerRotate.x || EulerRotate.y || EulerRotate.z)
	{
		if(bOrbitMovement)
		{
			Asura_Matrix_3x3 xRotation;
			//move the camera so that it orbits around a focus point
			
			if(!Tools_Movement::UseRelativeRotation)
			{
				//If using world relative rotation , we need to apply the rotation to the individual
				//euler angles , so that the camera will typically stay upright
				Asura_Matrix_3x3 xInitialOrient = Orientation;

				Asura_Euler_3 EulerOrient;
				EulerOrient.FromMatrix(Orientation);
			
			
				EulerOrient.x -= EulerRotate.x;
				EulerOrient.y -= EulerRotate.y;
				EulerOrient.z -= EulerRotate.z;

				if(EulerOrient.x<-Asura_Maths::Deg90) EulerOrient.x = -Asura_Maths::Deg90;
				else if (EulerOrient.x>Asura_Maths::Deg90) EulerOrient.x = Asura_Maths::Deg90;

				EulerOrient.ToMatrix(Orientation);

				//determine the actual matrix rotation used		
				Asura_Matrix_3x3 xInitialOrientInv = xInitialOrient;
				xInitialOrientInv.Transpose();
				xRotation = Orientation * xInitialOrientInv;
			}
			else
			{
				EulerRotate.ToMatrix(xRotation);
				xRotation.Transpose();

				if(Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPADDIVIDE) || bMouseOrbitMovement)
				{
					//change rotation matrix so it is relative to the 'player's ' axes.
					Asura_Matrix_3x3 OrientationInverse = Orientation;
					OrientationInverse.Transpose();
					xRotation = Orientation*xRotation;
					xRotation = xRotation*OrientationInverse;
				}
				Orientation = xRotation*Orientation;
			}

			//renormalise orientation
			Asura_Euler_3 xEuler;
			xEuler.FromMatrix(Orientation);
			xEuler.ToMatrix(Orientation);


			//move the position around the orbit centre using this rotation
			Asura_Vector_3 xOffset = Position - xOrbitCentre;
			xOffset = xOffset * xRotation;
			Position = xOffset + xOrbitCentre;

		}
		else if(Tools_Movement::UseRelativeRotation)
		{
			//get the rotation as a matrix
			Asura_Matrix_3x3 Rotation;
			EulerRotate.ToMatrix(Rotation);

			Orientation = Orientation*Rotation;

			//renormalise orientation
			Asura_Euler_3 euler;
			euler.FromMatrix(Orientation);
			euler.ToMatrix(Orientation);
		}
		else
		{
			Asura_Euler_3 EulerOrient;
			EulerOrient.FromMatrix(Orientation);
			
			//just add on the euler angles
			EulerOrient.x += EulerRotate.x;
			EulerOrient.y += EulerRotate.y;
			EulerOrient.z += EulerRotate.z;

			if(EulerOrient.x<-Asura_Maths::Deg90) EulerOrient.x = -Asura_Maths::Deg90;
			else if (EulerOrient.x>Asura_Maths::Deg90) EulerOrient.x = Asura_Maths::Deg90;


			EulerOrient.ToMatrix(Orientation);

		}
	}

	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPADMULTIPLY))
		{
			bool bAxisSnap = false;
			
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
			{
				//remove roll component
				Asura_Euler_3 xEulerOrient;
				xEulerOrient.FromMatrix(Orientation);
				xEulerOrient.z = 0;
				xEulerOrient.ToMatrix(Orientation);
			}
			else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				SnapToNearestAxes(false);
				bAxisSnap = true;
			}
			else if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
			{
				SnapTo45DegreesFromAxes(false);
				bAxisSnap = true;
			}
			else
			{
				//snap to level (remove roll and pitch)
				SnapToLevel(false);
			}

			movedThisFrame = true;

			if(bAxisSnap)
			{
				static float s_fLastAxisSnapTime = 0.0;
				//if axis snap is done repeatedly in a short time , rotate by 90 degrees
				float fTimer = Asura_Timers::GetLocalUnscaledTime();

				if(fTimer - s_fLastAxisSnapTime < 1.0f)
				{
					Asura_Matrix_3x3 xMat90(0,0,1,
											0,1,0,
											-1,0,0);

					Orientation = Orientation * xMat90;
				}
				s_fLastAxisSnapTime = fTimer;
			}
			UpdatePositionInAsuraData();

		}
	}


	Asura_Vector_3 delta,deltah,deltav;

	if(Tools_Movement::IsUsingWorldRelativeMovement())
	{
		delta = Asura_Vector_3(0,0,1);
		deltah = Asura_Vector_3(1,0,0);
		deltav = Asura_Vector_3(0,1,0);
	}
	else
	{
		delta = Asura_Vector_3(Orientation.e13,
							  Orientation.e23,
							  Orientation.e33);

		deltah = Asura_Vector_3(Orientation.e11,
							   Orientation.e21,
							   Orientation.e31);

		deltav = Asura_Vector_3(Orientation.e12,
							   Orientation.e22,
							   Orientation.e32);
	}

	Asura_Vector_3 LinVelocity;
	LinVelocity.Zero();


	if (Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON4))
	{
			delta *= 20.0f;
			deltah *= 20.0f;
			deltav *= 20.0f;
	}

	delta *= *pMovementScale;
	deltah *= *pMovementScale;
	deltav *= *pMovementScale;

	float fVerticalSpeed = 0;

	
	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
/*
			if (Asura_UserInput::KeyPressed('w'))
			{
				LinVelocity += delta*4.0f;
			}
*/
			if (Asura_UserInput::KeyPressed('s'))
			{
				LinVelocity -= delta*4.0f;
			}
			if (Asura_UserInput::KeyPressed('a'))
			{
				LinVelocity -= deltah*4.0f;
			}
			if (Asura_UserInput::KeyPressed('d'))
			{
				LinVelocity += deltah*4.0f;
			}
/*
			if (Asura_UserInput::KeyPressed('q'))
			{
				LinVelocity += delta*8.0f;//double speed
			}
*/
			if(Tools_Movement::IsUsingDynamics())
			{
				if (Asura_UserInput::KeyPressed('e'))
				{
					fVerticalSpeed -= 4.0f;
				}
			}
			
			if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				if (Asura_UserInput::KeyPressed('w'))
				{
					LinVelocity += delta*4.0f;
				}
				if (Asura_UserInput::KeyPressed('q'))
				{
					LinVelocity += delta*8.0f;//double speed
				}
			}
		
			if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
			{
				if(!Tools_Help::IsDisplayingHelp())
				{
					//cursor key controls
					if (Asura_UserInput::KeyPressed(ASURA_KEYID_UP))
					{
						fVerticalSpeed -= 4.0f;
					}
					if (Asura_UserInput::KeyPressed(ASURA_KEYID_DOWN))
					{
						fVerticalSpeed += 4.0f;
					}
					if (Asura_UserInput::KeyPressed(ASURA_KEYID_LEFT))
					{
						LinVelocity -= deltah*4.0f;
					}
					if (Asura_UserInput::KeyPressed(ASURA_KEYID_RIGHT))
					{
						LinVelocity += deltah*4.0f;
					}
				}
			}
		}
		if(!Tools_Movement::DisableNumberPadControls)
		{
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD1))
			{
				LinVelocity -= deltah*4.0f;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD3))
			{
				LinVelocity += deltah*4.0f;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPADSUB))
			{
				fVerticalSpeed -= 4.0f;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPADADD))
			{
				fVerticalSpeed += 4.0f;
			}
		}
	}
	
	Asura_Vector_3 xInitialPosition = Position;
	if(Tools_Movement::IsUsingDynamics())
	{

		if(fVerticalSpeed)
		{
			m_fVerticalSpeed = LinVelocity.y = fVerticalSpeed;
		}
		else
		{
			m_fVerticalSpeed += 10.0f * Asura_Timers::GetFrameTime();
			m_fVerticalSpeed = Asura_Maths::Min(m_fVerticalSpeed, 10.0f);
			LinVelocity.y = m_fVerticalSpeed;
		}
		
		
		//Fill in everything for the movement query
		Asura_Vector_3 xDisplacement = LinVelocity*Asura_Timers::GetFrameTime();
		Asura_Bounding_Box xBB;
		GetBoundingBox(xBB);
		Asura_OBB xOBB = Asura_OBB(xBB, Asura_AMatrix3x3(Orientation));

		Asura_Physics_MovementQuery xInput(Asura_AVector3(Position), Asura_AVector3(xDisplacement), xOBB);
		xInput.SetStopAtFirstCollision(false);
		xInput.SetCollideWithStaticObjects(true);
		xInput.SetCollideWithExternalControlObjects(true);
		xInput.SetCollideWithActorObjects(true);
		xInput.SetCollideWithDynamicObjects(true);
		xInput.SetHandleToIgnore(GetEntityPhysicsObjectHandle());

		//And get the restult
		Asura_Physics_MovementQuery_Results xResults(xInput);

		//get the new position
		Position = xResults.GetEndPosition();

		Asura_AVector3 xCollisionNormal = xResults.GetLastCollisionNormal();
		if(xCollisionNormal.m_fY < -0.5f)
		{
			m_fVerticalSpeed = 0.0f;
		}
		
	}
	else
	{
		LinVelocity += deltav*fVerticalSpeed;

		//add on the movement for this frame
		Position += LinVelocity*Asura_Timers::GetFrameTime();
	}



	//if any movement has happened , need to update the position in whatever is being moved
	if(EulerRotate.x || 
	   EulerRotate.y ||
	   EulerRotate.z ||
	   xInitialPosition != Position)
	{
		UpdatePositionInAsuraData();
		movedThisFrame = true;
	}

	if(movedThisFrame && IsSelected())
	{
		Tools_Object_System::UpdateGroupMovement();
	}
}	



Asura_Euler_3 Tools_Object::GetRotationForFrame()
{
	Asura_Euler_3 EulerRotate(0,0,0);
	//handle facing changes using mouse..
	if (Tools_Movement::IsUsingMouseRotation())
	{
		EulerRotate.y += Asura_UserInput::GetAxisValue(ASURA_AXIS_MOUSE_H) * 2.0f * Asura_Timers::GetFrameTime() * Tools_Movement::MouseRotationMultiplier;
		EulerRotate.x -= Asura_UserInput::GetAxisValue(ASURA_AXIS_MOUSE_V) * 2.0f * Asura_Timers::GetFrameTime() * Tools_Movement::MouseRotationMultiplier;
	}

	#if 1
	static bool s_bAxisRestrict = false;
	static bool s_bAxisRestrict_AxisLocked = false;
	static int s_iAxisRestrict_Axis;
	static Asura_Euler_3 s_xAxisRestrict_TotalMove;
	static Asura_Euler_3 s_xAxisRestrict_AbsMove;
	static float s_fAxisRestrict_Timer;
	if(Tools_Movement::IsUsingMouseRotation() && Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		if(!s_bAxisRestrict)
		{
			s_bAxisRestrict = true;
			s_bAxisRestrict_AxisLocked = false;
			s_xAxisRestrict_TotalMove.Zero();
			s_xAxisRestrict_AbsMove.Zero();
		}
		if(!s_bAxisRestrict_AxisLocked)
		{
			s_xAxisRestrict_TotalMove += EulerRotate;
			s_xAxisRestrict_AbsMove.x += Asura_Maths::Abs(EulerRotate.x);
			s_xAxisRestrict_AbsMove.y += Asura_Maths::Abs(EulerRotate.y);


			if(s_xAxisRestrict_AbsMove.x > s_xAxisRestrict_AbsMove.y)
			{
				if(s_iAxisRestrict_Axis == 0)
				{
					EulerRotate.y = 0;
					if(s_xAxisRestrict_AbsMove.x - s_xAxisRestrict_AbsMove.y > 0.2f)
					{
						s_bAxisRestrict_AxisLocked = true;
					}
				}
				else
				{
					s_iAxisRestrict_Axis = 0;
					EulerRotate.x = s_xAxisRestrict_TotalMove.x;
					EulerRotate.y -= s_xAxisRestrict_TotalMove.y;
						
				}
			}
			else
			{
				if(s_iAxisRestrict_Axis == 1)
				{
					EulerRotate.x = 0;
					if(s_xAxisRestrict_AbsMove.y - s_xAxisRestrict_AbsMove.x > 0.2f)
					{
						s_bAxisRestrict_AxisLocked = true;
					}
				}
				else
				{
					s_iAxisRestrict_Axis = 1;
					EulerRotate.x -= s_xAxisRestrict_TotalMove.x;
					EulerRotate.y = s_xAxisRestrict_TotalMove.y;
				}
			}

			
		}
		else
		{
			if(s_iAxisRestrict_Axis == 0)
			{
				EulerRotate.y = 0;
			}
			else
			{
				EulerRotate.x = 0;
			}
		}

	}
	else
	{
		s_bAxisRestrict = false;
	}
	#endif

	//check for keyboard rotation changes
	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus() && !Tools_Movement::DisableNumberPadControls)
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			Asura_Angle RotationAngle = Asura_Timers::GetFrameTime() * Asura_Maths::Deg90;
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD4)) //yaw left
			{
				EulerRotate.y -= RotationAngle;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD6)) //yaw right
			{
				EulerRotate.y += RotationAngle;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD2)) //pitch down
			{
				EulerRotate.x -= RotationAngle;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD8)) //pitch up
			{
				EulerRotate.x += RotationAngle;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD7)) //roll left
			{
				EulerRotate.z -= RotationAngle;
			}
			if (Asura_UserInput::KeyPressed(ASURA_KEYID_NUMPAD9)) //roll right
			{
				EulerRotate.z += RotationAngle;
			}
		}
	}
	if(Tools_Movement::InvertYAxis)
	{
		//invert pitch
		EulerRotate.x = -EulerRotate.x;
	}

	return EulerRotate;
	
}

void Tools_Object::SnapToNearestAxes(bool bUpdateAsura)
{
	Orientation = Orientation.GetAxiallyAlignedMatrix();
	
	if(bUpdateAsura)
	{
		UpdatePositionInAsuraData();
	}

}

void Tools_Object::SnapTo45DegreesFromAxes(bool bUpdateAsura /*= true*/)
{
	Asura_Matrix_3x3 xMat;
	Asura_Euler_3 xEuler45(0,Asura_Maths::Pi/4.0f,0);
	xEuler45.ToMatrix(xMat);

	//rotate orientation to be relative to axes 45 degrees off around vertical axis
	Orientation = xMat * Orientation;

	//snap to nearest axes in this space
	SnapToNearestAxes(false);

	//and rotate back to world space
	xMat.Transpose();
	Orientation = xMat * Orientation;

	if(bUpdateAsura)
	{	
		UpdatePositionInAsuraData();
	}
}

void Tools_Object::SnapToLevel(bool bUpdateAsura /*= true*/)
{
	Asura_Euler_3 xEuler;
	xEuler.FromMatrix(Orientation);
	//remove roll and pitch , but leave yaw alone
	xEuler.x = 0;
	xEuler.z = 0;
	xEuler.ToMatrix(Orientation);
	
	if(bUpdateAsura)
	{
		UpdatePositionInAsuraData();
	}

}

void Tools_Object::CalculateRelativePosition(Tools_Object* object)
{
	Asura_Matrix_3x3 inverseMat = object->Orientation;
	inverseMat.Transpose();

	RelativeOrientation = inverseMat * Orientation;
	RelativePosition = (Position-object->Position)*inverseMat;
	
		
}

void Tools_Object::UpdateRelative(Tools_Object* object)
{
	if(object==this) return;

	Orientation = object->Orientation * RelativeOrientation;
	Position = object->Position + RelativePosition*object->Orientation;

	//renormalise orientation
	Asura_Euler_3 xEuler;
	xEuler.FromMatrix(Orientation);
	xEuler.ToMatrix(Orientation);

	UpdatePositionInAsuraData(true);
}

bool Tools_Object::ObjectIsInLOS(const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDir, float& fLowest)
{
	//check for objects that are being shown as icons
	if(m_iDistantIconTexture != ASURA_TEXTURE_NONE && Tools_Render_Options::IsUseRenderAsIconDistance())
	{
		float fDistance = (Position - *Asura_Camera::GetPosition()).Magnitude();

		if(fDistance>Tools_Render_Options::GetRenderAsIconDistance())
		{
			if(fDistance>fLowest) return false;
			float fSize = Tools_Render_Options::GetDistantIconSize();
			if(fDistance>20.0f) fSize *= fDistance/20.0f;

			Asura_Vector_3 xIconPos = Position + m_xDistantIconOffset*Orientation;

			if (Asura_Maths::RayIntersectsSphere(xOrigin, xDir, xIconPos, fSize))
			{
				fLowest = fDistance;
				return true;
			}
			return false;
		}
	}
	

	if(m_pxAnimation)
	{
        Asura_Vector_3 xCentre;
        if (m_pxAnimation->GetOffsetFromCompassPoint(xCentre))
        {
            Asura_Bounding_Box xBB;
			GetBoundingBox(xBB);
			float fRadius = xBB.CalculateExtents().Magnitude();

			Asura_Vector_3 xAnimPos;
			Asura_Matrix_3x3 xAnimMat;
			GetAnimatedPosition(&xAnimPos,&xAnimMat);
            
            xCentre += xAnimPos;
            if (Asura_Maths::RayIntersectsSphere(xOrigin, xDir, xCentre, fRadius))
            {
				Asura_LOS_InputData xLOSInput(xOrigin, xDir, ASURA_GUID_UNREGISTERED, fLowest);
				xLOSInput.m_uPolyFlagsToIgnore = 0;//(default would ignore polygons flagged ignore-for-los... which would be bad)
				
                Asura_LOS_OutputData xLOSOutput;
				xLOSOutput.m_fDistanceToIntersection = fLowest;

				if (m_pxAnimation->LOSTest( xAnimPos, 
											xAnimMat,
											& xLOSInput,
											& xLOSOutput) )
				{
					// A hit, write back the distance.
					fLowest = xLOSOutput.m_fDistanceToIntersection;
					return true;
				}
				else
				{
					return false;
				}
            }
        }
        // probably offscreen
        return false;
	}
	else
	{
		int iPolyIndex;	
		return LOSTest(xOrigin,xDir,true,fLowest,iPolyIndex);
	}
}

// DF 17/11/08
// Duplicate function to ObjectIsInLOS but passes in BB only check flag to the LOSInput
bool Tools_Object::ObjectBBIsInLOS(const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDir, float& fLowest)
{
	if(m_pxAnimation)
	{
		Asura_Vector_3 xCentre;
		if (m_pxAnimation->GetOffsetFromCompassPoint(xCentre))
		{
			Asura_Bounding_Box xBB;
			GetBoundingBox(xBB);
			float fRadius = xBB.CalculateExtents().Magnitude();

			Asura_Vector_3 xAnimPos;
			Asura_Matrix_3x3 xAnimMat;
			GetAnimatedPosition(&xAnimPos,&xAnimMat);

			xCentre += xAnimPos;
			if (Asura_Maths::RayIntersectsSphere(xOrigin, xDir, xCentre, fRadius))
			{
				Asura_LOS_InputData xLOSInput(xOrigin, xDir, ASURA_GUID_UNREGISTERED, fLowest);
				xLOSInput.m_uPolyFlagsToIgnore = 0;//(default would ignore polygons flagged ignore-for-los... which would be bad)
				xLOSInput.m_bBoundingBoxOnly = true;

				Asura_LOS_OutputData xLOSOutput;
				xLOSOutput.m_fDistanceToIntersection = fLowest;

				if (m_pxAnimation->LOSTest( xAnimPos, 
					xAnimMat,
					& xLOSInput,
					& xLOSOutput) )
				{
					// A hit, write back the distance.
					fLowest = xLOSOutput.m_fDistanceToIntersection;
					return true;
				}
				else
				{
					return false;
				}
			}
		}
		// probably offscreen
		return false;
	}
	else
	{
		int iPolyIndex;	
		return LOSTest(xOrigin,xDir,true,fLowest,iPolyIndex);
	}
}


bool Tools_Object::ShouldTestForLOS()
{
	//ignore objects that aren't being rendered
	if(IsHidden()) return false;
	if(!Tools_Render_Options::IsPointWithinViewDistance(Position)) return false;

	return true;
}


bool Tools_Object::GetPolygonAndPositionUnderCursor(int& iPolygonIndex,Asura_Vector_3& xPosition)
{
	const Asura_Vector_3* cameraPosition = Asura_Camera::GetPosition();
	const Asura_Matrix_3x3* cameraOrientation = Asura_Camera::GetOrientation();


	//get the z direction for the mouse
	Asura_Vector_3 zvec;
	Asura_Cursor::GetLOSDirection(zvec);

	// Kevin Lea - 22/06/2001 12:49:20 - and transform this direction into worldspace
	zvec = zvec * (*cameraOrientation);

	float fDistance = ASURA_LOS_MAX_DISTANCE;
	if(m_pxAnimation)
	{
		//if using an animation , don't bother with getting a polygon.
		//just get a position.
		iPolygonIndex = -1;
		if(!ObjectIsInLOS(*cameraPosition,zvec,fDistance))
		{
			return false;
		}
	}
	else if(!LOSTest(*cameraPosition,zvec,false,fDistance,iPolygonIndex))
	{
		return false;
	}

	xPosition = *cameraPosition + (zvec * fDistance);
	return true;
}

bool Tools_Object::GetPolygonNormalAndPositionUnderCursor(Asura_Vector_3& xNormal,Asura_Vector_3& xPosition)
{
	const Asura_Vector_3* cameraPosition = Asura_Camera::GetPosition();
	const Asura_Matrix_3x3* cameraOrientation = Asura_Camera::GetOrientation();


	//get the z direction for the mouse
	Asura_Vector_3 zvec;
	Asura_Cursor::GetLOSDirection(zvec);

	// Kevin Lea - 22/06/2001 12:49:20 - and transform this direction into worldspace
	zvec = zvec * (*cameraOrientation);

	float fDistance = ASURA_LOS_MAX_DISTANCE;
	if(m_pxAnimation)
	{
		
		//Do standard LOS test first to see if we are in with a chance.
		if(!ObjectIsInLOS(*cameraPosition,zvec,fDistance))
		{
			return false;
		}

		//Now need to do more detailed tests in order to get a face normal out of it.

		Asura_Hierarchy_Skin* pxSkin = m_pxAnimation->GetSkin();

		Asura_Hierarchy_Tween* pxTweenStorage = m_pxAnimation->GetTweenStore();
		if(!pxTweenStorage->IsPositionDataValid()) return false;

		//calculate the world positions of all the bones
		Asura_Vector_3 axBonePosition[ASURA_MAX_NO_OF_BONES_PER_MODEL];
		Asura_Matrix_3x3 axBoneOrientation[ASURA_MAX_NO_OF_BONES_PER_MODEL];
		
    	for(int i = 0; i < pxSkin->NumberOfBones; i++)
		{
			Asura_Matrix_3x3	xBoneOri;
			Asura_Vector_3		xBonePos;

			pxTweenStorage->GetBoneData(i).GetOrientation(xBoneOri);
			pxTweenStorage->GetBoneData(i).GetPosition(xBonePos);

			if (i == 0)
			{
				// Root
				axBonePosition[0] = xBonePos * Orientation + Position;
				axBoneOrientation[0] = Orientation * xBoneOri;
			}
			else
			{
				// Get parent
				int iParentIndex = pxSkin->ParentBone[i];
				Asura_Assert(iParentIndex >= 0 && iParentIndex < i, "");

				axBonePosition[i] = xBonePos * axBoneOrientation[iParentIndex] + axBonePosition[iParentIndex];
				axBoneOrientation[i] = axBoneOrientation[iParentIndex] * xBoneOri;
			}
		}

		if(pxSkin->IsSkinnedModel())
		{
			Asura_Mesh* pxSkinMesh = pxSkin->GetMesh();
			if(!pxSkinMesh) return false;
			
			//calculate position of each vertex in skin
			Asura_Vector_3* pxVerts = new Asura_Vector_3[pxSkin->NumberOfVertices];
			for(int i=0;i<pxSkin->NumberOfVertices;i++)
			{
				pxVerts[i].Zero();

				for(int b=0;b<4;b++)
				{
					float fWeight = pxSkin->VertexData[i].Weight[b];
					if (fWeight==-1.0f) break;
					
					int iBone = pxSkin->VertexData[i].BoneID[b];

					Asura_Vector_3 xBoneVertex = (pxSkin->VertexData[i].Offset[b]*axBoneOrientation[iBone]) + axBonePosition[iBone];
		
					pxVerts[i] += xBoneVertex*fWeight;
				}
			}

			//do the LOS test against these verts
			float fLowest = ASURA_LOS_MAX_DISTANCE;
			Asura_Vector_3 axTriVerts[3];

			
			for (int i=0; i<pxSkinMesh->NumberOfPolygons; i++)
			{
			
				int iNumTri = pxSkinMesh->PolygonDataPtr[i].IsQuad() ? 2 : 1;
				
				for(int iTri=0;iTri<iNumTri;iTri++)
				{
					bool bFacingToward;
					Asura_Vector_3 v0 = pxVerts[pxSkinMesh->PolygonDataPtr[i].VertexID[0+iTri*2]];
					Asura_Vector_3 v1 = pxVerts[pxSkinMesh->PolygonDataPtr[i].VertexID[1+iTri*2]];
					Asura_Vector_3 v2 = pxVerts[pxSkinMesh->PolygonDataPtr[i].VertexID[(2+iTri*2)%4]];

					float d = Asura_Maths::RayIntersectsTri(*cameraPosition,zvec,v0,v1,v2,bFacingToward);

					if (d>=0.0f && d<fLowest)
					{
					
						if (bFacingToward)
						{
							fLowest=d;
							axTriVerts[0] = v0;
							axTriVerts[1] = v1;
							axTriVerts[2] = v2;

						}
 					}
				}
			}
			
			delete [] pxVerts;

			if(fLowest == ASURA_LOS_MAX_DISTANCE)
			{
				//failed to get intersection after all
				return false;
			}

			//get intersection point and normal
			xPosition = *cameraPosition + (zvec * fLowest);

			Asura_Vector_3 xNormal = (axTriVerts[1]-axTriVerts[0])^(axTriVerts[2]-axTriVerts[1]);
	        xNormal.Normalise();

	        if( xNormal.MagnitudeSquared() < 0.0001f )
	        {
	            xNormal.x = 1.0f;
	            xNormal.y = 0.0f;
	            xNormal.z = 0.0f;
	        }

			return true;

		}
		else
		{
			//los test against an object hierarchy
			float fLowest =	ASURA_LOS_MAX_DISTANCE;
			
			Asura_Assert(pxSkin->SectionData,"");
			for(int iBone=0;iBone<pxSkin->NumberOfBones;iBone++)
			{
				Asura_Mesh* pxSectionMesh = pxSkin->SectionData[iBone].Mesh;
				if(!pxSectionMesh) continue;

				const Asura_Vector_3& xBonePos = axBonePosition[iBone];
				const Asura_Matrix_3x3& xBoneOri = axBoneOrientation[iBone];

				for (int i=0; i<pxSectionMesh->NumberOfPolygons; i++)
				{
			
					int iNumTri = pxSectionMesh->PolygonDataPtr[i].IsQuad() ? 2 : 1;
					
					for(int iTri=0;iTri<iNumTri;iTri++)
					{
						bool bFacingToward;
						Asura_Vector_3 v0 = pxSectionMesh->VertexDataPtr[pxSectionMesh->PolygonDataPtr[i].VertexID[0+iTri*2]].Position;
						Asura_Vector_3 v1 = pxSectionMesh->VertexDataPtr[pxSectionMesh->PolygonDataPtr[i].VertexID[1+iTri*2]].Position;
						Asura_Vector_3 v2 = pxSectionMesh->VertexDataPtr[pxSectionMesh->PolygonDataPtr[i].VertexID[(2+iTri*2)%4]].Position;

						v0 = v0*xBoneOri + xBonePos;
						v1 = v1*xBoneOri + xBonePos;
						v2 = v2*xBoneOri + xBonePos;

						float d = Asura_Maths::RayIntersectsTri(*cameraPosition,zvec,v0,v1,v2,bFacingToward);

						if (d>=0.0f && d<fLowest)
						{
						
							if (bFacingToward)
							{
								fLowest=d;
								if(!pxSectionMesh->AreNormalsValid()) pxSectionMesh->CreateNormals();

								xNormal = pxSectionMesh->NormalsDataPtr[i].FaceNormal;
								xNormal = xNormal * xBoneOri;

							}
 						}
					}
				}
			}

			if(fLowest == ASURA_LOS_MAX_DISTANCE)
			{
				//failed to get intersection after all
				return false;
			}

			//get intersection point. normal has been got earlier
			xPosition = *cameraPosition + (zvec * fLowest);
			return true;

		}

	}
	else 
	{
		int iPolygonIndex;
		if(!LOSTest(*cameraPosition,zvec,false,fDistance,iPolygonIndex))
		{
			return false;
		}

		//get the position
		xPosition = *cameraPosition + (zvec * fDistance);
		
		//and the face normal in world space
		if(!Mesh->AreNormalsValid()) Mesh->CreateNormals();
		xNormal = Mesh->NormalsDataPtr[iPolygonIndex].FaceNormal;
		xNormal = xNormal * Orientation;

		return true;

	}

}

bool Tools_Object::LOSTest(const Asura_Vector_3& xLOSOrigin, const Asura_Vector_3& xLOSDirection, bool bUseLOD, float& fNearest, int& iPolygonIndex)
{
	if(!Mesh) return false;

	Asura_Mesh* pxMesh = bUseLOD ? GetLODMesh() : Mesh;

	return pxMesh->LOSTest(xLOSOrigin, xLOSDirection, Position, Orientation, fNearest, iPolygonIndex, IsScaled() ? &m_xScale : NULL);
}


bool Tools_Object::IsSelected() const
{
	return (this==Tools_Object_System::SelectedObject);
}


bool Tools_Object::IsRootObjectSelected() const
{
	return GetObjectToSelect()->IsSelected();	
}

bool Tools_Object::IsRootObjectGroupSelected() const
{
	return GetObjectToSelect()->IsGroupSelected();	
}


bool Tools_Object::RenderAsSelected()
{
	if(Tools_Object_System::IsChoosingObject())
	{
		return m_bSelectedByChooseObject;
	}
	else
	{
		return IsSelected();
	}

}

void Tools_Object::PrintInfo(float& x,float& y)
{
	//show the object type and object name
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Type : %s",GetTypeDescription());
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Name : %s",GetName());

	Asura_Guid uGuid = GetGuid();
	if(uGuid != ASURA_GUID_UNREGISTERED)
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Guid : %d",uGuid);
	}
}

void Tools_Object::Destroy()
{
	Tools_Object_System::DeleteObject(this);
}

bool Tools_Object::IsHidden()
{
	if(HiddenFlags) return true;
	if(Tools_Object_System::AllHidden()) return true;
	if(Tools_Object_System::TypeIsHidden(Type)) return true;
	return false;
}

void Tools_Object::SetHiddenFlag(int iHiddenFlag,bool bHide)
{
	if(bHide)
		HiddenFlags |= iHiddenFlag;
	else
		HiddenFlags &=~ iHiddenFlag;
}


void Tools_Object::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	char szText[200];

	Tools_ContextMenu_Item* pxItem;
	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	pxSection->AddSeparator();


	//select/deselect single object
	if(xContext.m_bShiftHeld)
	{
		Asura_PrintfToBfr(szText,"Deselect : %s",sizeof(szText),true,GetName());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_DeselectObject, this);
		pxItem->SetID("Select");
		pxSection->AddItem(pxItem);
	}
	else if(xContext.m_bControlHeld)
	{
		Asura_PrintfToBfr(szText,"Select : %s",sizeof(szText),true,GetName());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectObject_Add, this);
		pxItem->SetID("Select");
		pxSection->AddItem(pxItem);
	}
	else
	{
		Asura_PrintfToBfr(szText,"Select : %s",sizeof(szText),true,GetName());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectObject_Replace, this);
		pxItem->SetID("Select");
		pxSection->AddItem(pxItem);
	}

	//select/deselect all of type
	if(Type != TOT_Anon)
	{
		if(xContext.m_bShiftHeld)
		{
			Asura_PrintfToBfr(szText,"Deselect type : %s",sizeof(szText),true,GetTypeDescription());
			pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_DeselectByType, this);
			pxItem->SetID("Select by type");
			pxSection->AddItem(pxItem);
		}
		else if(xContext.m_bControlHeld)
		{
			Asura_PrintfToBfr(szText,"Select type : %s",sizeof(szText),true,GetTypeDescription());
			pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectByType_Add, this);
			pxItem->SetID("Select by type");
			pxSection->AddItem(pxItem);
		}
		else
		{
			Asura_PrintfToBfr(szText,"Select type : %s",sizeof(szText),true,GetTypeDescription());
			pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectByType_Replace, this);
			pxItem->SetID("Select by type");
			pxSection->AddItem(pxItem);
		}

		//select/deselect all with same skin
		if(!GetShapeOrSkinName().IsBlank())
		{
			if(xContext.m_bShiftHeld)
			{
				Asura_PrintfToBfr(szText,"Deselect skin : %s",sizeof(szText),true,GetShapeOrSkinName());
				pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_DeselectBySkin, this);
			}
			else if(xContext.m_bControlHeld)
			{
				Asura_PrintfToBfr(szText,"Select skin : %s",sizeof(szText),true,GetShapeOrSkinName());
				pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectBySkin_Add, this);
			}
			else
			{
				Asura_PrintfToBfr(szText,"Select skin : %s",sizeof(szText),true,GetShapeOrSkinName());
				pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectBySkin_Replace, this);
			}

			pxItem->SetID("Select by skin");
			pxSection->AddItem(pxItem);
		}
	}

	pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_DISPLAY);
	pxSection->AddSeparator();

	//hide object
	Asura_PrintfToBfr(szText,"Hide : %s",sizeof(szText),true,GetName());
	pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_HideObject, this);
	pxItem->SetID("Hide");
	pxSection->AddItem(pxItem);


	if(Type != TOT_Anon)
	{
		//hide all of one type
		Asura_PrintfToBfr(szText,"Hide type : %s",sizeof(szText),true,GetTypeDescription());
		pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_HideByType, this);
		pxItem->SetID("Hide by type");
		pxSection->AddItem(pxItem);

		//hide all other types
		pxItem = new Tools_ContextMenu_Item_Function("Hide all other types", ContextMenu_HideOtherTypes, this);
		pxItem->SetHelpText("Hide all object types other than the type used by this object");
		pxSection->AddItem(pxItem);

	}

	pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_OBJECT);
	pxSection->AddSeparator();

	//For showing movement gizmo
	if(!Tools_Object_System::IsChoosingObject() && !Tools_Object_System::IsMessageEditing())
	{
		pxItem = new Tools_ContextMenu_Item_Function("Move", ContextMenu_Move, this);
		pxItem->SetHelpText("Toggles movement gizmos for this object");
		pxSection->AddItem(pxItem);
	}

}

void Tools_Object::ContextMenu_DeselectObject(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;
	Tools_Object_System::DeselectObject(pxObject);
}

void Tools_Object::ContextMenu_SelectObject_Add(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;
	Tools_Object_System::SelectObject(pxObject, true);
}

void Tools_Object::ContextMenu_SelectObject_Replace(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;
	Tools_Object_System::SelectObject(pxObject, false);
}

void Tools_Object::ContextMenu_HideObject(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;
	
	Asura_Assert(pxObject,"");
	pxObject->SetHiddenFlag(ObjectHideFlag_User,true);

}

void Tools_Object::ContextMenu_DeselectByType(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type!=TOT_Anon,"");

	//get all objects of this type
	List<Tools_Object*> xSelectionList;
	Tools_Object_System::ListObjectsOfType(pxObject->Type,xSelectionList);

	//And deselect them
	Tools_Object_System::DeselectObjects(xSelectionList);

}

void Tools_Object::ContextMenu_SelectByType_Add(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type!=TOT_Anon,"");

	//get all objects of this type
	List<Tools_Object*> xSelectionList;
	Tools_Object_System::ListObjectsOfType(pxObject->Type,xSelectionList);

	//move this object to the end of the list , so it becomes the primary selection
	if(xSelectionList.contains(pxObject))
	{
		xSelectionList.delete_entry(pxObject);
		xSelectionList.add_entry(pxObject);
	}

	//And select them
	Tools_Object_System::SelectObjects(xSelectionList, true);
}

void Tools_Object::ContextMenu_SelectByType_Replace(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type!=TOT_Anon,"");

	//get all objects of this type
	List<Tools_Object*> xSelectionList;
	Tools_Object_System::ListObjectsOfType(pxObject->Type,xSelectionList);

	//move this object to the end of the list , so it becomes the primary selection
	if(xSelectionList.contains(pxObject))
	{
		xSelectionList.delete_entry(pxObject);
		xSelectionList.add_entry(pxObject);
	}

	//And select them
	Tools_Object_System::SelectObjects(xSelectionList, false);
}

void Tools_Object::ContextMenu_SelectBySkin_Helper(void* pObject, int iSelectionType)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type!=TOT_Anon,"");

	if(pxObject->GetShapeOrSkinName().IsBlank())
	{
		// Object has no skin set, nothing to do.
		return;
	}

	List<Tools_Object*> xSelectionList;

	// Selection types: 0: replace current selection
	//					1: add to current selection
	//					2: remove from current selection

	if(iSelectionType == 1)
	{
		xSelectionList = Tools_Object_System::SelectedObjectList;
	}

	// Find all objects with the same skin/anim
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		Tools_Object* const pxOtherObject = xObjectLIF();
		if(pxOtherObject && (pxObject != pxOtherObject) && (pxObject->GetShapeOrSkinName() == pxOtherObject->GetShapeOrSkinName()))
		{
			xSelectionList.add_entry(pxOtherObject);
		}
	}

	//add this object to the end of the list, so it becomes the primary selection
	xSelectionList.add_entry(pxObject);

	if(iSelectionType == 2)
	{
		Tools_Object_System::DeselectObjects(xSelectionList);
	}
	else
	{
		// Deselect all selected objects first, if replacing
		if(iSelectionType == 0)
		{
			Tools_Object_System::DeselectAllObjects();
		}

		Tools_Object_System::SelectObjects(xSelectionList, true);
	}
}

void Tools_Object::ContextMenu_HideByType(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type!=TOT_Anon,"");

	//Hide this object type , sharing the same the flag as the hide by type overlay
	Tools_Object_System::SetTypeHidden(pxObject->Type,ObjectHideFlag_HideByTypeOverlay,true);
}

void Tools_Object::ContextMenu_HideOtherTypes(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject && pxObject->Type!=TOT_Anon,"");

	//Hide other types used by currently existing objects , sharing the same the flag as the hide by type overlay
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		int iType = xObjectLIF()->Type;
		if( iType != TOT_Anon && iType != pxObject->Type)
		{
			Tools_Object_System::SetTypeHidden(iType,ObjectHideFlag_HideByTypeOverlay,true);
		}
	}


}


void Tools_Object::ContextMenu_Move(void* pObject)
{
	Tools_Object* pxObject = (Tools_Object*) pObject;

	Asura_Assert(pxObject ,"");

	Tools_Gizmo_Handler::ToggleGizmos(pxObject);
}



Asura_Mesh* Tools_Object::GetLODMesh()
{
	if(!Mesh) return 0;

	if(Mesh->m_pxNextLowerMesh)
	{
		float fDistance = (*Asura_Camera::GetPosition() - Position).Magnitude();
		return Mesh->GetDetailMeshFromDistance(fDistance);
	}
	return Mesh;
}


bool Tools_Object::HandlesThisMessage(Asura_Guid uGuid,int iMessageID)
{
	if(uGuid == GetGuid())
	{
		return HandlesThisMessage(iMessageID);
	}
	else
	{
		return SubEntityHandlesThisMessage(uGuid,iMessageID);
	}
}

Asura_Guid Tools_Object::SelectSubEntity()
{
	Tools_Timed_Message_System::AddMessage(0xff0000,"This object doesn't have any sub-entites");

	return ASURA_GUID_UNREGISTERED;
}

void Tools_Object::RenderMessageBlockLinks()
{
	Asura_Guid uGuid = GetGuid();
	if(uGuid == ASURA_GUID_UNREGISTERED) return;

	for(int iSet = 0;iSet<ASURA_STATICMESSAGE_NUMSETS;iSet++)
	{
		Asura_StaticMessageSetIndex eSet=(Asura_StaticMessageSetIndex)iSet;
		int iNumberOfMessageBlocks = Asura_StaticMessage_System::GetNumberOfMessageBlocks(eSet);

		for(int iBlockIndex = 0;iBlockIndex<iNumberOfMessageBlocks;iBlockIndex++)
		{
			unsigned short usBlockID = Asura_StaticMessage_System::ConvertIndexToID(eSet,iBlockIndex);
			int iNumberOfMessages = Asura_StaticMessage_System::GetNumberOfMessagesInBlock(usBlockID);

			//does the block belong to this entity?
			if(OwnsThisMessageBlock(usBlockID))
			{
				int iColour = GetColourForMessageBlock(usBlockID);
				Tools_Object* pxTargetOfEditedMessage = 0;
				//render links from this to all targets
				for(int i=0;i<iNumberOfMessages;i++)
				{
					const Asura_StaticMessage* pxMessage = Asura_StaticMessage_System::GetMessagePointer(eSet,iBlockIndex,i);

					//find the destination object
					Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(pxMessage->m_uSendTo,true);
					if(!pxObject) continue;

					#if ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
					Asura_StaticMessage* pxSelectedMessageInME = Tools_MessageEditor::GetSelectedStaticMessage();

					if(pxSelectedMessageInME == pxMessage)
					#else
					if(Tools_Message_Editing::IsEditedMessage(usBlockID,i))
					#endif
					{
						//This is the message that is currently being edited.
						//take note of it , and draw the line at the end in a different colour
						pxTargetOfEditedMessage = pxObject;
					}
					else
					{
						if( !pxObject->IsHidden() )
						{
							Asura_Render::RenderLine(Position,pxObject->Position,iColour);
						}
					}
					if(pxTargetOfEditedMessage)
					{
						if( !pxObject->IsHidden() )
						{
							Asura_Render::RenderLine(Position,pxTargetOfEditedMessage->Position,0xffffffff);
						}
					}
				}
			}
			else
			{
				//check to see if any of the messages target this
				for(int i=0;i<iNumberOfMessages;i++)
				{
					const Asura_StaticMessage* pxMessage = Asura_StaticMessage_System::GetMessagePointer(eSet,iBlockIndex,i);
					if(pxMessage->m_uSendTo == uGuid)
					{
						//find the owner of the message block
						Tools_Object* pxObject = Tools_Object_System::GetOwnerOfMessageBlock(usBlockID);
						if(pxObject)
						{
							if( !pxObject->IsHidden() )
							{
								Asura_Render::RenderLine(pxObject->Position,Position,0xff0000ff);
							}
						}
					}
				}
			}
		}
	}
}


void Tools_Object::ScalePosition(float fScale)
{
	Position = Position*fScale;

	//Not updating positions of linked objects , since they may well get scaled at the
	//same time anyway , and this could result in objects being moved twice. 
	//(if this causes problems , then it may be necessary to store positions of all objects before
	//starting with the scale) RAR 10/10/2003
	UpdatePositionInAsuraData(false);
}

void Tools_Object::UpdateMovementUndo()
{
	if(m_bPreviousPositionValid)
	{
		if(Position != m_xPreviousPosition || Orientation != m_xPreviousOrientation)
		{
			//if this is the first frame in a batch of movement , log for undo
			if((m_fStaticTimer > 0.5f)&& AllowMovementUndo())
			{
				Tools_Object_System::GetUndo()->Log_Object_Moved(this);
			}
			m_fStaticTimer = 0;

		}
		else
		{
			m_fStaticTimer += Asura_Timers::GetFrameTime();
		}
	}

	m_xPreviousPosition = Position;
	m_xPreviousOrientation = Orientation;
	m_bPreviousPositionValid = true;
}

Asura_Vector_3 Tools_Object::GetSnapPoint(const Asura_Vector_3& xPos) const
{
	Asura_Vector_3 xSnapPos = Position;

	float fMinDist = ASURA_LOS_MAX_DISTANCE;

	if(m_pxAnimation)
	{
		Asura_Hierarchy_Skin* pxSkin = m_pxAnimation->GetSkin();

		Asura_Hierarchy_Tween* pxTweenStorage = m_pxAnimation->GetTweenStore();
		if(!pxTweenStorage->IsPositionDataValid()) return xSnapPos;

		//calculate the world positions of all the bones
		Asura_Vector_3 axBonePosition[ASURA_MAX_NO_OF_BONES_PER_MODEL];
		Asura_Matrix_3x3 axBoneOrientation[ASURA_MAX_NO_OF_BONES_PER_MODEL];
		
    	for(int i = 0; i < pxSkin->NumberOfBones; i++)
		{
			Asura_Matrix_3x3	xBoneOri;
			Asura_Vector_3		xBonePos;

			pxTweenStorage->GetBoneData(i).GetOrientation(xBoneOri);
			pxTweenStorage->GetBoneData(i).GetPosition(xBonePos);

			if (i == 0)
			{
				// Root
				axBonePosition[0] = xBonePos * Orientation + Position;
				axBoneOrientation[0] = Orientation * xBoneOri;
			}
			else
			{
				// Get parent
				int iParentIndex = pxSkin->ParentBone[i];
				Asura_Assert(iParentIndex >= 0 && iParentIndex < i, "");

				axBonePosition[i] = xBonePos * axBoneOrientation[iParentIndex] + axBonePosition[iParentIndex];
				axBoneOrientation[i] = axBoneOrientation[iParentIndex] * xBoneOri;
			}
		}

		if(pxSkin->IsSkinnedModel())
		{
			//calculate position of each vertex in skin
			for(int i=0;i<pxSkin->NumberOfVertices;i++)
			{
				Asura_Vector_3 xVertPos(0,0,0);
				xVertPos.Zero();

				for(int b=0;b<4;b++)
				{
					float fWeight = pxSkin->VertexData[i].Weight[b];
					if (fWeight==-1.0f) break;
					
					int iBone = pxSkin->VertexData[i].BoneID[b];

					Asura_Vector_3 xBoneVertex = (pxSkin->VertexData[i].Offset[b]*axBoneOrientation[iBone]) + axBonePosition[iBone];
		
					xVertPos += xBoneVertex*fWeight;
				}

				float fDist = (xVertPos-xPos).Magnitude();
				if(fDist<fMinDist)
				{
					fMinDist = fDist;
					xSnapPos = xVertPos;
				}

			}
		}
		else
		{
			Asura_Assert(pxSkin->SectionData,"");
			for(int iBone=0;iBone<pxSkin->NumberOfBones;iBone++)
			{
				Asura_Mesh* pxSectionMesh = pxSkin->SectionData[iBone].Mesh;
				if(!pxSectionMesh) continue;

				//search the section mesh for the nearest point
				for(int i=0;i<pxSectionMesh->NumberOfVertices;i++)
				{
					Asura_Vector_3 v = pxSectionMesh->VertexDataPtr[i].Position;

					//transform the point into world space
					v = v*axBoneOrientation[iBone];
					v += axBonePosition[iBone];

					float fDist = (v-xPos).Magnitude();
					if(fDist<fMinDist)
					{
						fMinDist = fDist;
						xSnapPos = v;
					}
				}
			}
		}
	}
	else if(Mesh)
	{
		//search for the nearest vertex in the object being moved
		for(int i=0;i<Mesh->NumberOfVertices;i++)
		{
			Asura_Vector_3 v = Mesh->VertexDataPtr[i].Position;
			if(IsScaled())
			{
				v.x *= m_xScale.x;
				v.y *= m_xScale.y;
				v.z *= m_xScale.z;
			}

			//transform the point into world space
			v = v*Orientation;
			v += Position;

			float fDist = (v-xPos).Magnitude();
			if(fDist<fMinDist)
			{
				fMinDist = fDist;
				xSnapPos = v;
			}
		}
	}
	
	return xSnapPos;
}

void Tools_Object::GetBoundingBox(Asura_Bounding_Box& xOutputBB) const
{
	if(m_pxAnimation)
	{
		//Got an animation , so try to get a BB from that
		const Asura_Hierarchy_Skin* pxSkin = m_pxAnimation->GetSkin();
		if(pxSkin)
		{
			xOutputBB = pxSkin->GetBB();
			return;			
		}
	}
	else if(Mesh)
	{
		if(!Mesh->BoundingBoxDataValid) Mesh->CalculateBoundingBox();
		xOutputBB =  Mesh->BoundingBox;
		xOutputBB.MinX *= m_xScale.x;
		xOutputBB.MinY *= m_xScale.y;
		xOutputBB.MinZ *= m_xScale.z;
		xOutputBB.MaxX *= m_xScale.x;
		xOutputBB.MaxY *= m_xScale.y;
		xOutputBB.MaxZ *= m_xScale.z;

	}
	else
	{
		xOutputBB.Reset();
		xOutputBB.Set(Asura_Maths::ZeroVector3);
	}
}

void Tools_Object::GetAccurateBoundingBox(Asura_Bounding_Box& xOutputBB) const
{
	if(!m_pxAnimation)
	{
		GetBoundingBox(xOutputBB);
		return;
	}
	xOutputBB.Reset();

	Asura_Hierarchy_Skin* pxSkin = m_pxAnimation->GetSkin();

	Asura_Hierarchy_Tween* pxTweenStorage = m_pxAnimation->GetTweenStore();
	if(!pxTweenStorage->IsPositionDataValid())
	{
		//force an animation update to update the tween storage
		//this may or may not screw up the timings for the animation on this frame
		//but it shouldn't matter a great deal for tools objects.
		//RAR 14//11/2007
		Asura_Vector_3 xPos;
		Asura_Matrix_3x3 xMat;
		GetAnimatedPosition(&xPos,&xMat);

		m_pxAnimation->Update(xPos,xMat,ANIMTIME_USEGAMETIME,ASURA_HIERARCHY_RENDER_UPDATEBONESALWAYS|ASURA_HIERARCHY_RENDER_DONTRENDER);
		if(!pxTweenStorage->IsPositionDataValid())
		{
			Asura_Assert(0,"Position data still not valid in tween store");		
			GetBoundingBox(xOutputBB);
			return;
		}
	}


	//calculate the relative positions of all the bones
	Asura_Vector_3 axBonePosition[ASURA_MAX_NO_OF_BONES_PER_MODEL];
	Asura_Matrix_3x3 axBoneOrientation[ASURA_MAX_NO_OF_BONES_PER_MODEL];
	
   	for(int i = 0; i < pxSkin->NumberOfBones; i++)
	{
		Asura_Matrix_3x3	xBoneOri;
		Asura_Vector_3		xBonePos;

		pxTweenStorage->GetBoneData(i).GetOrientation(xBoneOri);
		pxTweenStorage->GetBoneData(i).GetPosition(xBonePos);

		if (i == 0)
		{
			// Root
			axBonePosition[0] = xBonePos;
			axBoneOrientation[0] = xBoneOri;
		}
		else
		{
			// Get parent
			int iParentIndex = pxSkin->ParentBone[i];
			Asura_Assert(iParentIndex >= 0 && iParentIndex < i, "");

			axBonePosition[i] = xBonePos * axBoneOrientation[iParentIndex] + axBonePosition[iParentIndex];
			axBoneOrientation[i] = axBoneOrientation[iParentIndex] * xBoneOri;
		}
	}

	if(pxSkin->IsSkinnedModel())
	{
		//calculate position of each vertex in skin
		for(int i=0;i<pxSkin->NumberOfVertices;i++)
		{
			Asura_Vector_3 xVertPos(0,0,0);
			xVertPos.Zero();

			for(int b=0;b<4;b++)
			{
				float fWeight = pxSkin->VertexData[i].Weight[b];
				if (fWeight==-1.0f) break;
				
				int iBone = pxSkin->VertexData[i].BoneID[b];

				Asura_Vector_3 xBoneVertex = (pxSkin->VertexData[i].Offset[b]*axBoneOrientation[iBone]) + axBonePosition[iBone];
	
				xVertPos += xBoneVertex*fWeight;
			}

			xOutputBB.Set(xVertPos);
			
		}
	}
	else
	{
		Asura_Assert(pxSkin->SectionData,"");
		for(int iBone=0;iBone<pxSkin->NumberOfBones;iBone++)
		{
			Asura_Mesh* pxSectionMesh = pxSkin->SectionData[iBone].Mesh;
			if(!pxSectionMesh) continue;

			//search the section mesh for the nearest point
			for(int i=0;i<pxSectionMesh->NumberOfVertices;i++)
			{
				Asura_Vector_3 v = pxSectionMesh->VertexDataPtr[i].Position;

				//transform the point into world space
				v = v*axBoneOrientation[iBone];
				v += axBonePosition[iBone];
	
				xOutputBB.Set(v);
			}
		}
	}
}

bool Tools_Object::ShouldBeRendered()
{
	if(IsHidden()) return false;
    if(Tools_Object_System::SelectedObject == this) return true;
	if(!Tools_Render_Options::IsPointWithinViewDistance(Position)) return false;
	#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	// KJFL - 21 February 2008 - move over to visibility query. However, the caching
	// stuff used with the old module linking is interesting - takes advantage of the
	// objects (and camera) not changing position often. However it needs a cache of a
	// BB. Still, a similar mechanism can be put around the visiblity query below if
	// required. Will profile.
	Asura_Bounding_Box xAABB;
	GetBoundingBoxForModLinkTest( xAABB );
	if ( !m_xVisibilityQuery.IsAABBVisibleFromCurrentCamera( xAABB ) ) return false;
	#else
	if(UseModuleLinking() && !IsInVisibleModule()) return false;
	#endif

	return true;
}

#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)

static bool g_bInAnyModule;
static int g_iVisibleModule;
static bool g_bBBInNonExteriorModule;
void Tools_Object::ObjectInVisibleModuleCallback( const u_int uModuleIdx )
{
	if(Asura_Environment_System::GetEnvironment()->IsSceneryModule(uModuleIdx))
	{
		return;//ignore scenery modules RAR 26/03/03
	}

	bool bNonExterior = !Asura_Module_System::GetCurrentHandler().IsExteriorModule(uModuleIdx);
	if(bNonExterior && !g_bBBInNonExteriorModule)
	{
		//This is the first non exterior module found ,so reset the results up until this point
		//RAR 10/04/2007
		g_bBBInNonExteriorModule = true;
		g_iVisibleModule = -1;

	}

	g_bInAnyModule = true;
	if(bNonExterior == g_bBBInNonExteriorModule)
	{
		if( Asura_Module_System::IsModuleVisible(uModuleIdx) )
    	{
    	    g_iVisibleModule = static_cast<int>(uModuleIdx);
    	}
	}
}

static bool g_bTestVisCache = false;
static bool g_bEnableVisCache = true;

bool Tools_Object::IsInVisibleModule()
{
	Asura_Bounding_Box xBB;
	GetBoundingBoxForModLinkTest(xBB);
	
	u_int uRenderListChangedFrameStamp = Asura_Module_System::GetCurrentHandler().GetRenderListChangedFrameStamp();
	bool bCanUseCachedResult = (uRenderListChangedFrameStamp == m_uVisCacheFrameStamp) && 
								(xBB == m_xVisCacheBB);
	if (g_bEnableVisCache && (!g_bTestVisCache) && bCanUseCachedResult)
	{
		return m_bCachedIsVisible;
	}

	//check last known module
	if(m_iLastKnownModule != -1)
	{
		if(m_iLastKnownModule < Asura_Module_System::GetNumberOfModules())
		{
			//Make sure the last known module wasn't an exterior module.
			//If it was an exterior module then knowing whether the point is still within it won't help ,since
			//we still need to check for being in non-exterior modules.
			//RAR 10/04/2007
			if(!Asura_Module_System::GetCurrentHandler().IsExteriorModule(m_iLastKnownModule))
			{
				if(Asura_Module_System::IsModuleVisible(m_iLastKnownModule))
				{
					Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(m_iLastKnownModule);
					if(!pxMesh->BoundingBoxDataValid) pxMesh->CalculateBoundingBox();
            	    
            	    if( xBB.Intersects(pxMesh->BoundingBox, Asura_Module_System::GetPositionOfModule(m_iLastKnownModule)))
		    	    {
						// test code
						if (g_bTestVisCache && bCanUseCachedResult)
						{
							Asura_Assert(m_bCachedIsVisible, "MH test failure");
						}
						else
						{
							m_bCachedIsVisible = true;
							m_xVisCacheBB = xBB;
							m_uVisCacheFrameStamp = uRenderListChangedFrameStamp;
						}
						return true;
		    	    }
				}
			}
		}
	}
	
	g_iVisibleModule = -1;
	g_bInAnyModule = false;
	g_bBBInNonExteriorModule = false;
	Asura_Module_System::TestModuleTree(xBB,ObjectInVisibleModuleCallback);

	if(g_iVisibleModule != -1)
	{
		//in visible module
		m_iLastKnownModule = g_iVisibleModule;
		// test code
		if (g_bTestVisCache && bCanUseCachedResult)
		{
			Asura_Assert(m_bCachedIsVisible, "MH test failure");
		}
		else
		{
			m_bCachedIsVisible = true;
			m_xVisCacheBB = xBB;
			m_uVisCacheFrameStamp = uRenderListChangedFrameStamp;
		}
		return true;
	}
	else
	{
		//not in a visible module , but render if in no module
		bool bIsVisible = !g_bInAnyModule;
		// test code
		if (g_bTestVisCache && bCanUseCachedResult)
		{
			Asura_Assertf(m_bCachedIsVisible == bIsVisible, ("MH test failure TOT = %s, GUID = %u", 
				GetTypeDescription(), GetGuid()));
		}
		else
		{
			m_bCachedIsVisible = bIsVisible;
			m_xVisCacheBB = xBB;
			m_uVisCacheFrameStamp = uRenderListChangedFrameStamp;
		}
		return bIsVisible;
	}
}
#endif

void Tools_Object::SetMesh(Asura_Mesh* pxMesh)
{
	Mesh = pxMesh;
	delete m_pxAnimation;
	m_pxAnimation = 0;

}

void Tools_Object::SetMesh(const char* szMeshName)
{
	SetMesh(Asura_Mesh_System::GetMesh(szMeshName));
}


void Tools_Object::SetSkinAndAnim(const Asura_Hierarchy_Skin* pxSkin,const Asura_Hierarchy_Anim* pxAnim)
{
	if(!pxSkin)
	{
		return;
	}
	//clear existing mesh/anim
	Mesh = 0;
	delete m_pxAnimation;

	m_pxAnimation = new Asura_Animation(pxSkin->Name.GetString());
	Asura_Hash_ID uAnimHash = pxAnim ? pxAnim->m_uHashID : 0;
	m_pxAnimation->m_xPrimaryWrapper.StartAnim2(uAnimHash,AsuraAnimationFlag_Loop);

	if(s_bOverrideAnimTimers)
	{
		m_pxAnimation->m_xPrimaryWrapper.SetTimer(s_fAnimTimerOverride);
	}
}

void Tools_Object::SetSkinAndAnim(Asura_Hash_ID uSkinID,Asura_Hash_ID uAnimID)
{
	Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID(uSkinID);
	if(!pxSkin) return;

	Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::GetAnimFromID(uAnimID);

	SetSkinAndAnim(pxSkin,pxAnim);
}

void Tools_Object::SetAnim(const Asura_Hierarchy_Anim* pxAnim, const bool bLoop/*=true*/)
{
	if(!m_pxAnimation) return;
	if(!pxAnim) return;

	m_pxAnimation->m_xPrimaryWrapper.StartAnim(pxAnim->Name.GetHash(),bLoop,0.0f);

	if(s_bOverrideAnimTimers)
	{
		m_pxAnimation->m_xPrimaryWrapper.SetTimer(s_fAnimTimerOverride);
	}

}

void Tools_Object::SetAnim(Asura_Hash_ID uAnimID, const bool bLoop/*=true*/)
{
	Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::GetAnimFromID(uAnimID);
	if(!pxAnim) return;

	SetAnim(pxAnim,bLoop);
}

void Tools_Object::NoLongerFlying()
{
}


bool Tools_Object::ResolveOutOfFloor(const float fAllowedMovement,bool bUpdateAsuraData/*=true*/)
{
	//Get a collision object to use
	const Asura_Collision_Object* pxCollisionObject = 0;
	Asura_Collision_Box* pxCollisionBox = 0;
	
	if(m_pxAnimation && m_pxAnimation->GetSkin())
	{
		//get bb from anim
		pxCollisionObject = m_pxAnimation->GetSkin()->GetCollisionObject();
	}
	else if(Mesh)
	{
		//get from Mesh
		if(!Mesh->BoundingBoxDataValid) Mesh->CalculateBoundingBox();
		pxCollisionObject = pxCollisionBox = new Asura_Collision_Box(Mesh->BoundingBox);
	}
	

	if(!pxCollisionObject)
	{
		//Nothing to do if there isn't any collision object to use
		return true;
	}

	Asura_Vector_3 xNewPosition = Position;
	int iHandleToIgnore = GetEntityPhysicsObjectHandle();

	//search up for a place where the object can be put
	bool bOkay = false;
	for(float fDisp = 0.0f; fDisp>=-fAllowedMovement && !bOkay; fDisp -= 0.1f)
	{
		xNewPosition.y = Position.y+fDisp;
	
		bOkay = Asura_Physics_System::CanPlaceObjectAt(xNewPosition, 
														Orientation, 
														*pxCollisionObject, 
														0, //poly flags to ifnore
														iHandleToIgnore,
														0xffff, //poly flags to test for
														true,//consider static objects
														true);//consider dynamic objects
	}
	delete pxCollisionBox;

	if(bOkay)
	{
		if(Position != xNewPosition)
		{
			Position = xNewPosition;
			if(bUpdateAsuraData) UpdatePositionInAsuraData();
		}
	}
	
	return bOkay;
}

Asura_String Tools_Object::GetShapeOrSkinName() const
{
	//get name from anim if it has one
	if(m_pxAnimation)
	{
		if(m_pxAnimation->GetSkin())
		{
			return m_pxAnimation->GetSkin()->Name.GetString();
		}
	}
	//otherwise get from the mesh
	else if(Mesh && !strstr(Mesh->Name.GetString(),"ToolsShape"))
	{
		return Mesh->Name.GetString();
	}

	return "";
}

void Tools_Object::SetScale(const Asura_Vector_3& xScale)
{
	m_xSetScale = xScale;
	m_bScaled = m_xSetScale.x != 1.0f || m_xSetScale.y != 1.0f || m_xSetScale.z != 1.0f;
	UpdateScale();
}

void Tools_Object::SetScale(float fScale)
{
	m_xSetScale.x = m_xSetScale.y = m_xSetScale.z = fScale;
	m_bScaled = fScale != 1.0f;
	UpdateScale();
}

void Tools_Object::SetScaleWhenCloseDist(float fDist)
{
	if(fDist != m_fScaleWhenCloseDist)
	{
		m_fScaleWhenCloseDist = fDist;
		UpdateScale();
	}
}

void Tools_Object::UpdateScale()
{
	m_xScale = m_xSetScale;
	if(m_fScaleWhenCloseDist > 0.0f)
	{
		float fDistance = (Position - *Asura_Camera::GetPosition()).Magnitude();
		if(fDistance < m_fScaleWhenCloseDist)
		{
			m_xScale *= fDistance/m_fScaleWhenCloseDist;
		}
	}
}

void Tools_Object::GetAnimatedPosition(Asura_Vector_3* pxOutputPos,Asura_Matrix_3x3* pxOutputMat) const
{
	if(m_pxAnimation && m_bUseAnimMovement)
	{
		//Apply offset from the animation. RAR 28/06/2006
		Asura_Vector_3 xPos = Position;
		Asura_Matrix_3x3 xMat = Orientation;

		Asura_Vector_3 xPosOffset;
		Asura_Quat xQuatOffset;
		if(m_pxAnimation->GetMovementOffset(&xPosOffset,&xQuatOffset))
		{
			Asura_Matrix_3x3 xMatOffset;
			xQuatOffset.ToMatrix(xMatOffset);

			xPos = xPos + xPosOffset*xMat;
			xMat = xMat * xMatOffset;
		}

		if(pxOutputPos) *pxOutputPos = xPos;
		if(pxOutputMat) *pxOutputMat = xMat;
	}
	else
	{
		if(pxOutputPos) *pxOutputPos = Position;
		if(pxOutputMat) *pxOutputMat = Orientation;
	}
}

void Tools_Object::SetAsDestroyed()
{
	Tools_Object_System::AddToDestroyList( this );
}

void Tools_Object::AddReference(Tools_Object* pxReferer)
{
	Asura_Assert(pxReferer,"");
	m_xReferenceList.add_entry(pxReferer);
}

void Tools_Object::RemoveReference(Tools_Object* pxReferer)
{
	if(m_xReferenceList.contains(pxReferer))
	{
		m_xReferenceList.delete_entry(pxReferer);
	}
}


bool Tools_Object::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
    //Changed assert to only fire for the primary selection ,since if the user selects multiple types of objects
	//then it is quite likely that the other objects won't handle some of the parameters.
	//RAR 10/08/2007
	Asura_Assertf( !IsSelected(), ("Shared param with ID 0x%08X not handled.", uParamID) );
    return false;
}


bool Tools_Object::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
    //Changed assert to only fire for the primary selection ,since if the user selects multiple types of objects
	//then it is quite likely that the other objects won't handle some of the parameters.
	//RAR 10/08/2007
	Asura_Assertf( !IsSelected(), ("Shared param with ID 0x%08X not handled.", uParamID) );
    return false;
}


bool Tools_Object::AddSharedParamDialogToOverlay( Asura_Overlay* pxOverlay, const Asura_Vector_2& xOffset, Asura_Hash_ID uParamID )
{
    Asura_Assertf( false, ("Shared param with ID 0x%08X not handled.", uParamID) );
    return false;
}


void Tools_Object::AddAnimTimerOverrideOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;
	
	pxDialog = new Asura_Tickbox_Dialog("Override Anim Timers",xOffset,&s_bOverrideAnimTimers);
	pxDialog->SetHelpText("Forces most animations to be displayed using the timer below.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Timer",xOffset,&s_fAnimTimerOverride,0.0f,1.0f,0.01f,"%.4f");
	pxDialog->SetHelpText("The timer to use for any objects that have an animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	
}

// Overlay creation
void Tools_Object::OnSelection()
{
	if(ShouldCreateObjectOverlay())
	{
		CreateObjectOverlay();
	}
}

void Tools_Object::OnDeselection()
{
	if(ShouldCreateObjectOverlay())
	{
		DeleteObjectOverlay();
	}
}

void Tools_Object::CreateObjectOverlay()
{
	DeleteObjectOverlay();

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(GetTypeDescription(),false,&s_xOverlayPosition);

	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	AddOverlayControls(pxOverlay,xOffset);

	pxOverlay->m_uID = GetOverlayID();

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Object::DeleteObjectOverlay()
{
	Asura_Overlay_System::CloseOverlays(GetOverlayID());
}

void Tools_Object::SetDistantIconTexture(const char* szTextureName)
{
	int iTextureIndex = Asura_Texture_Management::GetTextureIndex(szTextureName);
	if(iTextureIndex == ASURA_TEXTURE_NONE)
	{
		//add and attempt to load the texture
		iTextureIndex = Asura_Texture_Management::AddTexture(szTextureName);
		Asura_Texture_Management::LoadTextures();
	}

	//set the texture if it has been sucessfully loaded
	if(Asura_Texture_Management::IsTextureLoaded(iTextureIndex))
	{
		SetDistantIconTexture(iTextureIndex);
	}
}

void Tools_Object::SetDistantIconTexture(int iTextureID)
{
	m_iDistantIconTexture = iTextureID;

	//Get the centre of the current BB to use as the position to place the icon at.
	Asura_Bounding_Box xBB;
	GetBoundingBox(xBB);
	m_xDistantIconOffset = xBB.CalculateCentre();
}

bool Tools_Object::IsBeingRenderedAsIcon() const
{
	if(m_iDistantIconTexture != ASURA_TEXTURE_NONE && Tools_Render_Options::IsUseRenderAsIconDistance())
	{
		float fDistance = (Position - *Asura_Camera::GetPosition()).Magnitude();
		if(fDistance > Tools_Render_Options::GetRenderAsIconDistance())
		{
			return true;
		}
	}
	return false;
}

void Tools_Object::InitialiseDistantIconTexture()
{
	m_bInitialisedDistantIconTexture = true;

	//has a texture already been set?
	if(GetDistantIconTexture() != ASURA_TEXTURE_NONE) return;

	//try for a texture based on the type
	int iTextureID = Tools_Object_System::FindDistantIconTexture(GetTypeDescription());
	if(iTextureID != ASURA_TEXTURE_NONE)
	{
		SetDistantIconTexture(iTextureID);
	}
	else if(IsEntityObject())
	{
		//go for the default entity texture
		SetDistantIconTexture("\\tools\\ObjectIcons\\Entity.tga");
	}

}


//////////////////////////////////////////////////////////////////////////////////////////////////
void ObjectSystemUndo::Log_Object_Moved(const Tools_Object* pxObject)
{
	RootUndoData** ppxUndo = AllocateNextUndoData();
	if(!ppxUndo) return;
	
	*ppxUndo = new ObjectUndoData_Object_Moved(pxObject);
}

ObjectUndoData_Object_Moved::ObjectUndoData_Object_Moved(const Tools_Object* pxObject)
{
	Asura_Assert(pxObject,"");
	
	m_iObjectID = pxObject->m_iID;
	m_xPosition = pxObject->m_xPreviousPosition;
	m_xOrientation = pxObject->m_xPreviousOrientation;	
}

RootUndoData* ObjectUndoData_Object_Moved::ProcessUndoData()
{
	//find the object that this is undo data for
	Tools_Object* pxObject = Tools_Object_System::GetObjectFromToolsID(m_iObjectID);
	if(!pxObject)
	{
		//object probably deleted , so ignore
		return this;
	}
	
	//swap data between undo and object
	Asura_Vector_3 xTempPosition = m_xPosition;
	m_xPosition = pxObject->Position;
	pxObject->Position = xTempPosition;
	pxObject->m_xPreviousPosition = xTempPosition;

	Asura_Matrix_3x3 xTempOrientation = m_xOrientation;
	m_xOrientation = pxObject->Orientation;
	pxObject->Orientation = xTempOrientation;
	pxObject->m_xPreviousOrientation = xTempOrientation;

	pxObject->m_fStaticTimer = Asura_Maths::LargeFloat;

	//get asura data updated as well.
	//updateLinkedObjects is false , since any linked objects should have their own undo
	//data as well that will be processed seperately
	pxObject->UpdatePositionInAsuraData(false);

	return this;
}
