#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_OBJECT_H_
#define __TOOLS_OBJECT_H_

// Asura
#include "Core/Asura_Handle.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
#include "Visibility/Asura_Visibility_System.h"
#endif

// Tools
#include "Misc/list_tem.hpp"
#include "Tools_Overlays.h"
#include "Tools_Undo.h"

class Asura_Animation;
class Asura_Collision_Object;
class Asura_Mesh;
class Asura_Overlay;
class Tools_Parameter;

enum Tools_Object_Type
{
	TOT_Anon,//default for those types that don't need to be identifiable
	TOT_Camera,
	TOT_Cutscene,
	TOT_CutsceneActor,
	TOT_CutsceneTrackPoint,
	TOT_CutsceneEvent,
	TOT_Light,
	TOT_Corona,
	TOT_LightingTest,
	TOT_Phonon,
	TOT_StartPosition,
	TOT_VolumeEdit,
	TOT_PositionEdit,
	TOT_MarkerPoint,
	TOT_SectionBoundingBox,
	TOT_Joint,
	TOT_ToolsPhoton,
	TOT_LevelOfDetail,
	TOT_Entity_VolumeTrigger,
	TOT_Entity_TimeTrigger,
	TOT_Entity_CutsceneController,
	TOT_WorldSpaceRectangle,
	TOT_SpecialGuid,
	TOT_Entity_Glass,
    TOT_Entity_PhysicsObject,
	TOT_Entity_DestructibleLight,
	TOT_Entity_ParticleGenerator,
	TOT_Entity_SplitterBlock,
    TOT_Entity_CountedTrigger,
	TOT_Entity_AdvancedLight,
	TOT_Billboard_Plant,
	TOT_Weather_RainShelter,
	TOT_Entity_AdvancedVolumeTrigger,
	TOT_Entity_Lift,
    TOT_Entity_DamageVolume,
	TOT_Entity_MusicTrigger,
	TOT_Entity_MetaMusicTrigger,
	TOT_Entity_ExtendedParticleGenerator,
	TOT_Entity_FMVTrigger,
	TOT_Entity_ProxyTrigger,
	TOT_OrientedVolumeEdit,
	TOT_Extended_Rain_Shelter,
	TOT_PhotonMapping_LightSource,
	TOT_PhotonMapping_PolygonLightSource,
	TOT_StreamingBackgroundSound,
	TOT_Entity_MultiDamageVolume,
	TOT_VertexMove,
	TOT_Entity_PFX_Effect,
	TOT_Entity_TemplateMaster,
	TOT_Entity_TemplateInstance,
	TOT_Entity_LookAtTrigger,
    TOT_Entity_ClockTrigger,
	TOT_Entity_LogicTrigger,
	TOT_Entity_DialogueTrigger,
	TOT_Entity_ClientVolumeTrigger,
	TOT_Entity_StartPoint,
	TOT_Entity_TimelineTrigger,
	TOT_Entity_EnvTextureAnimControl,
    TOT_Entity_CountedVolumeTrigger,
	TOT_Botanicals,
	TOT_UserDefinedWaypointPosition,
	TOT_InterZoneWaypoint,
	TOT_Entity_DebugMessageTrigger,
	TOT_AttractorNode,
	TOT_HelperObject,
	TOT_Entity_CameraVolume,
	TOT_LineEdit,
    TOT_Entity_Node,
    TOT_Entity_Spline,
    TOT_Spline_Point,
    TOT_Entity_GamesceneBase,
    TOT_Entity_OrientedNode,
    TOT_Entity_CoverpointNode,
    TOT_Entity_EmergepointNode,
    TOT_Entity_GuardZone,
    TOT_Entity_GamesceneNode,
    TOT_Entity_AttractorNode,
    TOT_Entity_GamesceneSpline,
    TOT_Entity_GamesceneSpline_Point,
    TOT_Entity_LiftNode,
    TOT_Entity_LiftSpline,
    TOT_Entity_LiftSpline_Point,
	TOT_Entity_Affector,
	TOT_Entity_Affector_Wind,
	TOT_Entity_Projector,
	TOT_Entity_Teleporter,
	TOT_Entity_TeleportDestination,
	TOT_Entity_DebugWaypointTrigger,
	TOT_Entity_DynamicLight,
	TOT_Entity_ForceField_Conveyor,
	TOT_MovementGizmo,
	TOT_Entity_ConsoleVar,
	TOT_Entity_LightShaft,
	TOT_StaticDecal,
	TOT_Entity_WaypointBlockerVolume,
	TOT_VolumeGroupEdit,
	TOT_Entity_Objective,
	TOT_Entity_GS2Node,
	TOT_Entity_GS2Spline,
	TOT_Entity_GS2SplinePoint,
	TOT_Entity_GS2Attractor,
	TOT_Entity_Container,
	TOT_Container_Object_Attachment,
	TOT_Container_Object_Collection,
	TOT_Container_Object_Misc,
	TOT_Container_Object_Origin,
	TOT_Entity_DynamicMusicTrigger,
	TOT_HierarchyNode,
	TOT_RagdollCollision,
	TOT_RagdollJoint,
	TOT_Entity_DialogueController,
	TOT_Entity_FMVController,
	TOT_Entity_FMVSpeaker,
	TOT_Entity_EffectVolume,

	TOT_NumCommonTypes,
	//Project specific types should start at this point.
	//Values shouldn't be made too large since an array of size equal to the highest type is made
	//for keeping track of hidden types
	TOT_FirstProjectSpecific = TOT_NumCommonTypes
};

//flags for different reasons that an object may have been hidden.
//if any are set then the object is hidden
#define ObjectHideFlag_User					0x00000001
#define ObjectHideFlag_Mode					0x00000002
#define ObjectHideFlag_Cutscene				0x00000004
#define ObjectHideFlag_HideByTypeOverlay	0x00000008
#define ObjectHideFlag_EntityFile			0x00000010
#define ObjectHideFlag_ReplacedByCutscene	0x00000020
#define ObjectHideFlag_TemplateTriggers		0x00000040

#define ObjectHideFlag_ChooseObject			0x20000000
#define ObjectHideFlag_ObjectSystem			0x40000000
#define ObjectHideFlag_MovementSystem		0x80000000

class Asura_Entity_Server;

class Tools_Object
{
	public :
		Tools_Object(int type = TOT_Anon);
		virtual ~Tools_Object();

		//New method of duplicating tools objects makes use of existing serialisation functionality
		//Just override this method to return true and the object (and derived objects) will be duplicatable!
		virtual bool UseStreamCopy() const { return false; }

		virtual void RenderPreFSFX() {};
		virtual void Render();
		virtual bool ShouldBeRendered();//does some culling to determine if Render() should be called
		//Should return true if the meshes should be rendered using the entity lighting
		virtual bool UseLighting(){return false;}
		//Determines whether module linking is used for culling rendering of the tools object
		virtual bool UseModuleLinking(){return true;}

		//Calls ShouldBeRendered() or gets a cached value
		bool GetShouldBeRendered(bool bCalculate) {if(bCalculate) m_bShouldBeRenderedThisFrame = ShouldBeRendered(); return m_bShouldBeRenderedThisFrame;}

		//for LOS test. lowest is used as both input and output here
		virtual bool ObjectIsInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta, float& lowest);
		virtual bool ObjectBBIsInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta, float& lowest);
		virtual bool GetPolygonAndPositionUnderCursor(int& iPolygonIndex,Asura_Vector_3& xPosition);
		virtual bool GetPolygonNormalAndPositionUnderCursor(Asura_Vector_3& xNormal,Asura_Vector_3& xPosition);
		bool LOSTest(const Asura_Vector_3& xLOSOrigin, const Asura_Vector_3& xLOSDirection, bool bUseLOD, float& fNearest, int& iPolygonIndex);

		//Does some early rejection for LOS tests based on hidden flags etc
		virtual bool ShouldTestForLOS();


		//per frame update of the flying
		virtual void UpdateMovement(float* pMovementScale,float* pRotationScale);

		//per frame update while selected
		virtual void UpdateWhileSelected(){}

		//per frame update called regardless of whether the objecct is selected
		virtual void Update(){}

		//used to update the position/orientation in the Tools_Object 
		//before doing the movement
		virtual void UpdatePositionFromAsuraData(){};

		//called whenever the object has been moved ,to allow the derived
		//class to update data from the new position
		void UpdatePositionInAsuraData(){UpdatePositionInAsuraData(true);}
		virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/){};

		//called by Tools_Movement when this object is being replaced by another object
		//as the current flying object
		virtual void NoLongerFlying();

		//can this be selected via the Tools_Object_System selection methods
		virtual bool IsSelectable(){return m_bSelectable;}
		
		//Enable/disable ability to select this object
		void EnableSelection(bool bSelectable){m_bSelectable = bSelectable;}

		//called when this becomes the primary selection
		virtual void OnSelection();

		//called when this stops being the primary selection
		virtual void OnDeselection();

		//called when this gets removed from the group selection list
		virtual void OnGroupDeselection(){}

		//called if object is 'selected' while already selected
		virtual void OnReselection(){}

		//called by template objects on each child when deselected
		//(currently just for welding in child objects marked as merge with environment)
		virtual void OnParentDeselection(){}

		//called if this object has just been created from a template update
		virtual void OnUpdateFromTemplate(){};

		virtual void Edit(){};

		// Returns whether the object's associated game data is valid.
		// bAutomatic signifies whether the verification has been automatically or manually requested.
		virtual bool Verify( const bool bAutomatic ) { return true; }

		//gets the name of the object
		virtual const char* GetName(){return "<Unnamed>";}

		//gives a description of the type of Tools_Object
		virtual const char* GetTypeDescription(){return "";}

		//Somewhat clumsy name, A few classes return a different value for GetTypeDescription() for
		//different subtypes of their class. For those classes this function should be overridden so that
		//it returns a single string for all objects of their object type ,regardless of any subtypes.
		virtual const char* GetTypeDescriptionForObjectType() {return GetTypeDescription();}

		static const char* GetNamePrefix(){return "";}
		virtual const char* GetMyNamePrefix() const {return Tools_Object::GetNamePrefix();}

		//called to display information about this object
		virtual void PrintInfo(float& x,float& y);
		
		//Removes whatver this object represents from the Asura data , as well as deleting the Tools_Object
		virtual void Destroy();

		//returns false if the object shouldn't be destroyed as a result of the user pressing delete		
		virtual bool CanBeDestroyed(){return true;}

		//returns true if the user should be ask for confirm deletion of an object
		virtual bool ConfirmDestruction(){return true;}

		//returns true if extra confirmation should be made for this object specifically
		//when part of a multiple object selection that is being destroyed.
		virtual bool ExtraConfirmationForMultiDestruction(){return false;}

		//returns true if this object may be moved by Tools_Object_System (flying commands , snap to vertex , etc..)
		virtual bool CanBeMoved(){return true;}

		//returns the guid of the asura entity that this is attached to , if it is such a beast
		virtual Asura_Guid GetGuid() const {return ASURA_GUID_UNREGISTERED;}
        
        // (MH - 15.11.2002) 
        //if this object can create an entity when it gets targeted by a message (e.g. sounds) do it here
        virtual bool AcquireGuid() { return false; }

		//returns true for those derived from Tools_Entity_Object
		virtual bool IsEntityObject(){return false;}
	
		virtual bool SupportsMessageEditor() { return IsEntityObject(); }

		//flags the entity as modified in the ToolsData , and flags the entity file as modified
		virtual void SetEntityModified(){}
		//moved from Tools_Entity_Object for now to allow cutscene actors to take messages.
		//May need to fiddle around with the derivation later. RAR 16/08/02
		bool HandlesThisMessage(Asura_Guid uGuid,int messageID);
		virtual bool HandlesThisMessage(int messageID){return false;}
		virtual bool SubEntityHandlesThisMessage(Asura_Guid uGuid,int iMessageID){return false;}

		//Returns the colour to use for output lines for this message block
		virtual int GetColourForMessageBlock(unsigned short usMessageBlockID){return 0xffff0000;}
		
		//Returns true if the message block belongs to this entity
		virtual bool OwnsThisMessageBlock(unsigned short usMessageBlockID){return false;}

		//Gets description of what causes message block to be sent , returns false if not implemented ,or message isn't found
		virtual bool GetEventDescForMessageBlock(unsigned short usMessageBlockID,Asura_String& xOutputDesc) {return false;}

		//render all links to and from this object
		void RenderMessageBlockLinks();

		//Called before checking message block ownership problems.
		//(Allows cutscenes to refresh message block lists)
		virtual void PreMessageBlockOwnershipCheck(){};


		//If the object has several entities associated with it (such as cutscenes) , this can be used to choose one
		virtual Asura_Guid SelectSubEntity();

		//Returns true if the guid matches a sub entity of this object
		virtual bool IsSubEntity(Asura_Guid uGuid){return false;}

		virtual const char* GetSubEntityName(Asura_Guid uGuid){return "<Unknown>";}

		virtual bool IsHidden();
		void SetHiddenFlag(int iHideFlag,bool bHide);
		//Used to set the hide flag on any child objects at the same time. Currently used
		//to hide all objects in a template instance.
		virtual void SetHiddenFlagIncludingChildren(int iHideFlag,bool bHide){SetHiddenFlag(iHideFlag,bHide);}

		bool IsHiddenFlagSet(int iHideFlag) const{return (HiddenFlags & iHideFlag) ? true : false;}

		//Adds options to the right-click context menu overlay
		virtual void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);


		virtual bool UseObjectInModuleLinking() {return false;}

		//should return true if the object makes use of the left mouse button while selected
		//(to tell the object system to ignore it)
		virtual bool UsesLeftButton(){return false;}

		//Pretty much a one-off for the volume editing objects. Intended to prevent
		//deselection when shift clicking on the faces.
		virtual bool AllowSelectionUsingLMB() const {return true;}

		// Another one for volume group editing.
		virtual bool DisallowSelectionOfOtherTypes() const { return false; }

		//Is this the main selected object?
		bool IsSelected() const;
		//Is this in the multiple selection list?
		bool IsGroupSelected() const {return m_bGroupSelected;}

		//used by object system to flag objects that are in the selected object list
		void SetGroupSelected(bool bSelected) { m_bGroupSelected = bSelected;}

		//similary for flagging objects selectd by the 'choose object' process
		void SetSelectedByChooseObject(bool bSelected){m_bSelectedByChooseObject = bSelected;}
		bool IsSelectedByChooseObject() const {return m_bSelectedByChooseObject;}

		//If this object has had a parent object set that should be selected instead of it , then
		//this will indicate whether that parent (or the parent's parent etc) has been selected.
		//If no such object has been set , then this will just return true if this object is itself
		//selected
		bool IsRootObjectSelected() const;
		//Similary for group selected
		bool IsRootObjectGroupSelected() const;

		bool RenderAsSelected();

		//If true then the main mesh won't be rendered , but selection wil be shown
		virtual bool ShouldRenderSelectionOutlineOnly(){return false;}

		//return false if no selection outline should be used
		virtual bool ShouldRenderSelectionOutline(){return true;}

		virtual bool ShouldRenderShadow(){return false;}

		//Should this type of object be visible during cutscene playback?
		virtual bool ShouldHideDuringCutscene(){return true;}

		virtual Asura_Mesh* GetLODMesh();

		//Scales the position of the object (adding as virtual , so other positions related to the
		//object can be scaled as well , if desired)
		virtual void ScalePosition(float fScale);

		virtual Asura_Entity_Server* GetServerEntity() { return NULL; }

		//functions used by the movement system
		Asura_Euler_3 GetRotationForFrame();
		void SnapToNearestAxes(bool bUpdateAsura = true);
		void SnapTo45DegreesFromAxes(bool bUpdateAsura = true);
		void SnapToLevel(bool bUpdateAsura = true);

		virtual void CalculateRelativePosition(Tools_Object* object);
		virtual void UpdateRelative(Tools_Object* object);

		//called by tools_object_system each frame to update undo
		void UpdateMovementUndo();

		int GetToolsID() const {return m_iID;}
		static int GetNextToolsID(){return s_iNextID;}

		virtual bool AllowMovementUndo(){return true;}

		//copy from the passed object
		virtual void Copy(const Tools_Object* pxObject){}
		//Does this object support copying?
		virtual bool CanBeCopied() const {return false;}
		//Does this object support pasting itself on top of a fresh object?
		virtual bool CanBePasted() const {return CanBeCopied();}
		//Does this object support pasting itself over an already existing object?
		virtual bool CanBePastedOverAnother() const {return CanBeCopied();}
		//Creates a new object of the same type as this.
		virtual Tools_Object* CreateObjectOfThisType() const {return 0;}

		//Can this object be renamed?
		virtual bool CanBeRenamed(){return false;}

		//get the nearest snap point in world space for this object , for use in snap to vertex	
		virtual Asura_Vector_3 GetSnapPoint(const Asura_Vector_3& xPos) const;

		//Gets the bounding box relative to the objects position/orientation.
		//Used for snapping objects that don't have a normal mesh
		virtual void GetBoundingBox(Asura_Bounding_Box& xOutputBB) const;
		//Gets the bounding box for mod-link test. Allows for derived entities that draw
		//other things (like paths) while drawing the mesh
		virtual void GetBoundingBoxForModLinkTest(Asura_Bounding_Box& xOutputBB) const
		{
			GetBoundingBox(xOutputBB);
			xOutputBB.Displace(Position);
		}
		//bounding box to use when snapping this object to environment.
		virtual void GetBoundingBoxForSnapToVertex(Asura_Bounding_Box& xOutputBB) const
		{
			GetAccurateBoundingBox(xOutputBB);
		}
		//As GetBoundingBox but calculates a more accurate bounding box for objects
		//with animations , rather than relying on the skins BB
		void GetAccurateBoundingBox(Asura_Bounding_Box& xOutputBB) const;

		//If this returns true then the object will default to using its pivot for snapping , instead
		//of using the bounding box.
		virtual bool UsePivotForSnapToVertex() const {return false;}

		// alpha values for rendering
		virtual float GetAlpha(void) const { return m_fAlpha; }
		virtual void  SetAlpha( const float fAlpha ) { m_fAlpha = fAlpha; }

		//Used for redirecting mouse selection , so that a parent object will get selected instead
		void SetObjectToSelectInstead(Tools_Object* pxObject){m_pxObjectToSelectInstead = pxObject;}
		Tools_Object* GetObjectToSelect() {return (m_pxObjectToSelectInstead && !s_bAllowChildObjectSelection)? m_pxObjectToSelectInstead->GetObjectToSelect() : this;}
		const Tools_Object* GetObjectToSelect() const {return (m_pxObjectToSelectInstead && !s_bAllowChildObjectSelection)? m_pxObjectToSelectInstead->GetObjectToSelect() : this;}

		//Used when a template instance is finding its child objects , allows these objects to add any extra
		//objects that it might not know about (if they aren't entities or the like).
		//Overrides of this should use the AddChild() function in Tools_Entity_TemplateInstance_Object for each
		//child as required
		virtual void AddChildObjectsToTemplate(class Tools_Entity_TemplateInstance_Object* pxTemplateInstance){};

		//For asking template objects about which objects they own
		virtual void ListChildObjects(List<Tools_Object*>& xObjectList,bool bRecurse) const {};

		//If this returns true then then ExportSelectedAsTemplate() won't bother warning about
		//this object not being acceptable for templates. This is mainly there to prevent
		//slightly misleading warning messages caused by selecting some helper objects
		//that aren't entities , when the data they represent is in an entity that could be exported.
		virtual bool SuppressCantExportToTemplateWarning() const {return false;}

		// accessors for previous position and orientation
		const Asura_Vector_3&   GetPreviousPosition(void)    const { return m_xPreviousPosition; }
		const Asura_Matrix_3x3& GetPreviousOrientation(void) const { return m_xPreviousOrientation; }

		//Get the position and orientation including the effects of any movement from an animation
		void GetAnimatedPosition(Asura_Vector_3* pxOutputPos,Asura_Matrix_3x3* pxOutputMat) const;

		// This is analogous to the SetAsDestroyed method in Asura_Entity.
		// Objects set as destroyed are cleaned up at the end of Tools_Object_System::Update().
		// Note that this is separate from the SetBeingDestroyed stuff.
		void SetAsDestroyed();

		// Informs object that it is just about to be destroyed. Being used to prevent some
		// unneeded processing when the destroyed object gets deselected.
		// Note that this is separate from the SetAsDestroyed stuff.
		virtual void SetBeingDestroyed() { m_bBeingDestroyed = true; }
		bool IsBeingDestroyed() const { return m_bBeingDestroyed; }

		//tests if object is in visibe module (or if in no module at all)
		bool IsInVisibleModule();
		static void ObjectInVisibleModuleCallback(const u_int uIndex);
		
		static void SetAllowChildObjectSelection(bool bAllow){s_bAllowChildObjectSelection = bAllow;}
		static bool GetAllowChildObjectSelection(){return s_bAllowChildObjectSelection;}

		//Sets the mesh to use for rendering , deleting any animation for this object
		void SetMesh(Asura_Mesh* pxMesh);
		void SetMesh(const char* szMeshName);

		//Sets a texture to use for rendering this when far away
		void SetDistantIconTexture(int iTextureID);
		//Similarly , but doesn't do anything if the texture fails to load
		void SetDistantIconTexture(const char* szTextureName);
		int GetDistantIconTexture() const {return m_iDistantIconTexture;}
		//Returns true if an icon texture has been set and the object is far enough away
		bool IsBeingRenderedAsIcon() const;

		//Sets the distant icon texture based on the object type , if one hasn't already been set.
		void InitialiseDistantIconTexture();
		
		//Sets the skin and anim to use , clearing the mesh
		void SetSkinAndAnim(const class Asura_Hierarchy_Skin* pxSkin,const class Asura_Hierarchy_Anim* pxAnim);
		void SetSkinAndAnim(Asura_Hash_ID uSkinID,Asura_Hash_ID uAnimID);

		//Sets the anim to use , if currently using an Asura_Animation
		void SetAnim(const class Asura_Hierarchy_Anim* pxAnim,const bool bLoop = true);
		void SetAnim(Asura_Hash_ID uAnimID,const bool bLoop = true);

		Asura_Animation* GetAnimation(){return m_pxAnimation;}
		const Asura_Animation* GetAnimation() const {return m_pxAnimation;}

		//Causes the animation's compass movement to be used for rendering and LOS
		void SetUseAnimMovement(bool bUse){m_bUseAnimMovement = bUse;}
		bool GetUseAnimMovement() const {return m_bUseAnimMovement;}

		//Get/Set x,y and z scale values for objects that have an Asura_Mesh
		const Asura_Vector_3& GetScale() const {return m_xScale;}
		void SetScale(const Asura_Vector_3& xScale);
		//Sets uniform scale in each axis
		void SetScale(float fScale);

		//Causes the object to be scaled down if closer than the specified distance (to maintain the same size on screen)
		void SetScaleWhenCloseDist(float fDist);
		bool IsScaledWhenClose() {return m_fScaleWhenCloseDist != 0.0f;}

		//get the highest scale component
		float GetMaxScale() const {return Asura_Maths::Max(m_xScale.x,Asura_Maths::Max(m_xScale.y,m_xScale.z));}

		bool IsScaled() const {return m_bScaled || (m_fScaleWhenCloseDist != 0.0f);}

		//Recalculates the current scale
		void UpdateScale();

		//Should return true if the object can be scaled by the user
		virtual bool CanBeScaled() const {return false;}
		virtual void UpdateScaleInAsuraData(){};

		//Get/Set texture override (ASURA_TEXTURE_NONE to use default)
		int GetTextureID() const {return m_iTextureID;}
		void SetTextureID(int iTextureID) {m_iTextureID = iTextureID;}

		//Sets colour for rendering. This doesn't work for objects that use lighting.
		void SetColour(u_int uColour) {m_uColour = uColour;}
        u_int GetColour() const { return m_uColour; }

		//If object is embedded in floor , moves up to amount allowed in an attempt
		//to un-embed it.
		bool ResolveOutOfFloor(const float fAllowedMovement,bool bUpdateAsuraData=true);

		//Used to find out if object is associated with a dynamic entity that shouldn't
		//be allowed to be embedded in the ground.
		virtual bool DoesObjectNeedToBeAbleToMove() const {return false;}

		//Functions for use with the overlay controls that can affect all selected objects
        virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
        virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );
        virtual bool AddSharedParamDialogToOverlay( Asura_Overlay* pxOverlay, const Asura_Vector_2& xOffset, Asura_Hash_ID uParamID );

		//Indicates temporary objects that are used for editing , and so should
		//in general not be hidden
		virtual bool IsTemporaryEditObject() const {return false;}

		//get name of shape/skin that an object uses (for finding objects by shape)
		virtual Asura_String GetShapeOrSkinName() const;

        // Get the name of the blueprint (type) that an object uses.
        virtual Asura_String GetBlueprintTypeName() const { return ""; }
        virtual Asura_String GetBlueprintName()     const { return ""; }

		//Should true if the object shouldn't be deleted by Tools_Object_System::DeleteAllModeSpecific
		virtual bool ShouldPersistBetweenModes() const {return false;}

		//The drag functions should be overridden if the object does stuff when the cursor is clicked on it and dragged
		virtual bool IsDraggable() const {return false;}
		virtual void StartDrag(const Asura_Vector_2& xCursorPos){}
		virtual void UpdateDrag(const Asura_Vector_2& xCursorPos){}
		virtual void EndDrag(const Asura_Vector_2& xCursorPos){}

		//Adds a reference to this object. This won't prevent it from being deleted as such , but when
		//it is deleted ReferencedObjectDeleted() will be called to let the referer know that the object is no more.
		//Obviously any objects doing this must make sure that they remove their references before being deleted.
		void AddReference(Tools_Object* pxReferer);
		//Removes a reference to this object
		void RemoveReference(Tools_Object* pxReferer);
		//Informs this object that the object that it was referring to is being deleted.
		virtual void ReferencedObjectIsBeingDeleted(Tools_Object* pxObject){};

		//Causes the object to be rendered during the UI rendering phase , so that will be on top
		//of evrything else in the world.
		virtual bool ShouldRenderAsUI() const {return false;}

		//Determines whether this object should be hidden by default when a member of a template
		virtual bool ShouldHideWithinTemplate() const {return true;}

		//Adds controls for overriding the animation timers on all tools objects
		static void AddAnimTimerOverrideOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
		static void SetOverrideAnimTimers(bool bSet) {s_bOverrideAnimTimers = bSet;}

		// Determines whether the object can be replaced by a cutscene actor.
		// Physical objects and special GUIDs are automatically allowed, and the result from this function is ignored.
		virtual bool CanBeReplacedByCutsceneActor() const { return false; }
		// If the above returns true, you can use this function to specify a different guid to replace, for example
		// the static guid of an object spawned by this object.
		virtual Asura_Guid GetGUIDToReplaceByCutsceneActor() const { return GetGuid(); }

		// Create Tools_Object overlay, using AddOverlayControls.
		// Deliberately named to avoid clashing with pre-existing CreateOverlay functions.
		void CreateObjectOverlay();
		void DeleteObjectOverlay();
		virtual void AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset) {}
		// Most existing Tools_Objects have their own overlay implementation.
		// If you want to use AddOverlayControls, over-ride this and return true.
		virtual bool ShouldCreateObjectOverlay() { return false; }
		// KOR - 18/05/2006 - use the pointer to generate an overlay ID instead of TOOLS_OVERLAY_SELECTEDOBJECT
		// This should avoid problems with different objects accessing the same overlay
		TOOLS_OVERLAY_ID GetOverlayID() { return static_cast<TOOLS_OVERLAY_ID>((int)this); }

		// Used to provide more useful information on the type of an object in the inheritance hierarchy.
		// Note that this is definitely implemented for classes inheriting from Tools_Entity_Node, but not necessarily for other classes.
		virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const { return ( Type == iToolsObjectType ); }

	protected :

		//get the in-game handle for the entities physics object
		virtual Asura_Handle GetEntityPhysicsObjectHandle() const {return ASURA_INVALID_HANDLE;}

		void RenderMeshOrAnimation(Asura_Mesh *pxMesh, Asura_Mesh *pxMeshToRender, Asura_Animation *pxAnimation);

	private :

		static void ContextMenu_DeselectObject(void* pObject);
		static void ContextMenu_SelectObject_Add(void* pObject);
		static void ContextMenu_SelectObject_Replace(void* pObject);
		static void ContextMenu_DeselectByType(void* pObject);
		static void ContextMenu_SelectByType_Add(void* pObject);
		static void ContextMenu_SelectByType_Replace(void* pObject);
		static void ContextMenu_DeselectBySkin(void* pObject) { ContextMenu_SelectBySkin_Helper(pObject, 2); }
		static void ContextMenu_SelectBySkin_Add(void* pObject) { ContextMenu_SelectBySkin_Helper(pObject, 1); }
		static void ContextMenu_SelectBySkin_Replace(void* pObject) { ContextMenu_SelectBySkin_Helper(pObject, 0); }
		static void ContextMenu_HideObject(void* pObject);
		static void ContextMenu_HideByType(void* pObject);
		static void ContextMenu_HideOtherTypes(void* pObject);
		static void ContextMenu_Move(void* pObject);

		static void ContextMenu_SelectBySkin_Helper(void* pObject, int iSelectionType);
	
	public :					

		Asura_Mesh* Mesh;//can be null
		Asura_Vector_3 Position;
		Asura_Matrix_3x3 Orientation;
		int Type;//taken from Tools_Object_Type enumeration above

		//for group flying	
		Asura_Vector_3 RelativePosition;
		Asura_Matrix_3x3 RelativeOrientation;		

		int HiddenFlags;

	protected :
		//for tracking falling speed while using dynamics
		float m_fVerticalSpeed;

	private :

		Asura_Animation* m_pxAnimation;
		bool m_bUseAnimMovement;
		Asura_Vector_3 m_xSetScale;//scale set using SetScale()
		Asura_Vector_3 m_xScale;//scaling factor to use with Mesh objects
		bool m_bScaled;
		float m_fScaleWhenCloseDist;//distance at which object will start being scaled down

		int m_iTextureID;
		u_int m_uColour;

		int m_iDistantIconTexture;//icon to use for rendering at a distance
		Asura_Vector_3 m_xDistantIconOffset;//offset from position for rendering the icon
		bool m_bInitialisedDistantIconTexture;

		
		int m_iID;//unique ID for this tools_object

		//previous position data for undo
		bool m_bPreviousPositionValid;
		float m_fStaticTimer;
		Asura_Vector_3 m_xPreviousPosition;
		Asura_Matrix_3x3 m_xPreviousOrientation;

		bool m_bSelectable;
		float m_fAlpha;
		bool m_bBeingDestroyed;

	#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
		Asura_Visibility_Query m_xVisibilityQuery;
	#else
		int m_iLastKnownModule;

		// cache for visibility
		u_int m_uVisCacheFrameStamp;
		Asura_Bounding_Box m_xVisCacheBB;
		bool m_bCachedIsVisible;
	#endif

		//pointer to parent object that should get selected when this gets clicked on
		Tools_Object* m_pxObjectToSelectInstead;

		//list of objects that refer to this object
		List<Tools_Object*> m_xReferenceList;

		//true if in the SelectedObjectList
		bool m_bGroupSelected;
		bool m_bSelectedByChooseObject;
		bool m_bShouldBeRenderedThisFrame;

		static int s_iNextID;
		static bool s_bAllowChildObjectSelection;

		//for forcing the timers on all objects to a specified value
		static bool s_bOverrideAnimTimers;
		static float s_fAnimTimerOverride;


		static Asura_Vector_2 s_xOverlayPosition;
	
		friend class ObjectUndoData_Object_Moved;
};


class ObjectUndoData_Object_Moved : public RootUndoData
{
public :
	ObjectUndoData_Object_Moved(const Tools_Object* pxObject);
	virtual RootUndoData* ProcessUndoData();
	
	int m_iObjectID;
	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrientation;
};


#endif //__TOOLS_OBJECT_H_
