//
// Tools_ObjectSystem_Param.h
//
// Created by Richard Rice - 05/05/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_OBJECTSYSTEM_PARAM_H__
#define __TOOLS_OBJECTSYSTEM_PARAM_H__

#include "Interface/Asura_Dialog.h"

#include "Core/Asura_HashTree.h"
#include "Objects/Tools_Object_System.h"

///////////////////////////////////////////////////////////////////////////////

//base class for parameter type that could be one of several types.
//To allow a single pointer type to be sent through the get/set param methods for the object system
class Tools_Parameter
{
public :
	Tools_Parameter(){m_bChanged = false;}
	virtual ~Tools_Parameter(){}

	//get/set functions..which will assert if not the correct for the type.
	virtual bool GetBool() const = 0;
	virtual void SetBool(const bool&) = 0;
	virtual float GetFloat() const = 0;
	virtual void SetFloat(const float&) = 0;
	virtual int GetInt() const = 0;
	virtual void SetInt(const int&) = 0;
	virtual u_int GetUInt() const = 0;
	virtual void SetUInt(const u_int&) = 0;
	virtual Asura_String GetString() const = 0;
	virtual void SetString(const Asura_String&) = 0; 

	Asura_Hash_ID GetHash() const {return GetUInt();}
	void SetHash(Asura_Hash_ID uHash) {SetUInt(uHash);}

	bool m_bChanged;
};

template <class T> class Tools_ParameterDerived : public Tools_Parameter
{
public :
	Tools_ParameterDerived(){}
	Tools_ParameterDerived(T xValue){m_xValue = xValue;}

	T GetValue() const {return m_xValue;}
	void SetValue(const T& xValue){if(m_xValue != xValue){m_bChanged=true;m_xValue = xValue;}}

	virtual bool GetBool() const
	{
		Asura_Assert(0,"GetBool() called on non-bool parameter");
		return false;
	}

	virtual void SetBool(const bool&)
	{
		Asura_Assert(0,"SetBool() called on non-bool parameter");
	}

	virtual float GetFloat() const
	{
		Asura_Assert(0,"GetFloat() called on non-float parameter");
		return 0;
	}

		virtual void SetFloat(const float&)
	{
		Asura_Assert(0,"SetFloat() called on non-float parameter");
	}

	virtual int GetInt() const
	{
		Asura_Assert(0,"GetInt() called on non-int parameter");
		return 0;
	}

	virtual void SetInt(const int&)
	{
		Asura_Assert(0,"SetInt() called on non-int parameter");
	}

	virtual u_int GetUInt() const
	{
		Asura_Assert(0,"GetUInt() called on non-u_int parameter");
		return 0;
	}

	virtual void SetUInt(const u_int&)
	{
		Asura_Assert(0,"SetUInt() called on non-u_int parameter");
	}

	virtual Asura_String GetString() const
	{
		Asura_Assert(0,"GetString() called on non-Asura_String parameter");
		return "";
	}

	virtual void SetString(const Asura_String&)
	{
		Asura_Assert(0,"SetString() called on non-Asura_String parameter");
	}




private :
	T m_xValue;
};

///////////////////////
Tools_ParameterDerived<bool>::Tools_ParameterDerived()
{
	m_xValue = false;
}

bool Tools_ParameterDerived<bool>::GetBool() const
{
	return GetValue();
}

void Tools_ParameterDerived<bool>::SetBool(const bool& xValue)
{
	SetValue(xValue);
}

///////////////////////
Tools_ParameterDerived<float>::Tools_ParameterDerived()
{
	m_xValue = 0.0f;
}

float Tools_ParameterDerived<float>::GetFloat() const
{
	return GetValue();
}

void Tools_ParameterDerived<float>::SetFloat(const float& xValue)
{
	SetValue(xValue);
}

///////////////////////
Tools_ParameterDerived<int>::Tools_ParameterDerived()
{
	m_xValue = 0;
}

int Tools_ParameterDerived<int>::GetInt() const
{
	return GetValue();
}

void Tools_ParameterDerived<int>::SetInt(const int& xValue)
{
	SetValue(xValue);
}

///////////////////////
Tools_ParameterDerived<u_int>::Tools_ParameterDerived()
{
	m_xValue = 0;
}

u_int Tools_ParameterDerived<u_int>::GetUInt() const
{
	return GetValue();
}

void Tools_ParameterDerived<u_int>::SetUInt(const u_int& xValue)
{
	SetValue(xValue);
}

///////////////////////
Tools_ParameterDerived<Asura_String>::Tools_ParameterDerived()
{
	m_xValue = 0;
}

Asura_String Tools_ParameterDerived<Asura_String>::GetString() const
{
	return GetValue();
}

void Tools_ParameterDerived<Asura_String>::SetString(const Asura_String& xValue)
{
	SetValue(xValue);
}

///////////////////////////////////////////////////////////////////////////////
enum
{
	TOOLS_OBJECTSYSTEM_PARAM_BOOL,
	TOOLS_OBJECTSYSTEM_PARAM_INT,
	TOOLS_OBJECTSYSTEM_PARAM_UINT,
	TOOLS_OBJECTSYSTEM_PARAM_FLOAT,
	TOOLS_OBJECTSYSTEM_PARAM_STRING,
};

class Tools_ObjectSystem_Param
{
public :
	Asura_Hash_ID m_uParamID;//Hash value associated with the parameter
	Asura_Hash_ID m_uHashID;//Hash of the name. In practice this is generally the same as m_uParamID
	const char* m_szName;
	int m_iType;//type from the enumeration above.
};

//Stores list of parameters that have been declared for use by various tools objects.
class Tools_ObjectSystem_ParamRegistry
{
public :
	static void RegisterParam(Asura_Hash_ID uParamID, const char* szName, int iType);

	static u_int GetNumParam(){return s_pxTree ? s_pxTree->GetNumberOfNodes() : 0;}
	static const Tools_ObjectSystem_Param* GetParamByHash(Asura_Hash_ID uHash){return s_pxTree ? s_pxTree->GetDataByHash(uHash) : 0;}
	static const Tools_ObjectSystem_Param* GetParamByIndex(int iIndex){return s_pxTree ? s_pxTree->GetDataByIndex(iIndex) : 0;}

private :
	static Asura_TemplateHashTree<Tools_ObjectSystem_Param>* s_pxTree;
};




///////////////////////////////////////////////////////////////////////////////

//Derived class from the shared paramater class , for dialog values
//that are shared across multiple selected objects

template <class T> class Tools_ObjectSystem_SharedValue : public Asura_Dialog_SharedValue<T>
{
public :
	Tools_ObjectSystem_SharedValue()
	{
		m_uParamID = 0;
	}
	
	
	Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID)
	{
		m_uParamID = uParamID;
	}	

	Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID, const char* szName)
	{
		m_uParamID = uParamID;
	}


	void SetParamID(Asura_Hash_ID uParamID)
	{
		m_uParamID = uParamID;
	}

	
	virtual void GetValue(T* pxValue,bool* pbMixed) const
	{
		if(pxValue || pbMixed)
		{
			Tools_ParameterDerived<T> xParam;
			Tools_Object_System::GetParameterFromSelectedObjects(m_uParamID,&xParam,pbMixed);

			if(pxValue) *pxValue = xParam.GetValue();
		}
	}

	virtual void SetValue(const T& xValue)
	{
		Tools_ParameterDerived<T> xParam(xValue);
		Tools_Object_System::SetParameterForSelectedObjects(m_uParamID,&xParam);
	}


private :
	Asura_Hash_ID m_uParamID;
};

Tools_ObjectSystem_SharedValue<bool>::Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID, const char* szName)
{
	m_uParamID = uParamID;
	Tools_ObjectSystem_ParamRegistry::RegisterParam(uParamID, szName, TOOLS_OBJECTSYSTEM_PARAM_BOOL);
}

Tools_ObjectSystem_SharedValue<int>::Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID, const char* szName)
{
	m_uParamID = uParamID;
	Tools_ObjectSystem_ParamRegistry::RegisterParam(uParamID, szName, TOOLS_OBJECTSYSTEM_PARAM_INT);
}

Tools_ObjectSystem_SharedValue<u_int>::Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID, const char* szName)
{
	m_uParamID = uParamID;
	Tools_ObjectSystem_ParamRegistry::RegisterParam(uParamID, szName, TOOLS_OBJECTSYSTEM_PARAM_UINT);
}

Tools_ObjectSystem_SharedValue<float>::Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID, const char* szName)
{
	m_uParamID = uParamID;
	Tools_ObjectSystem_ParamRegistry::RegisterParam(uParamID, szName, TOOLS_OBJECTSYSTEM_PARAM_FLOAT);
}

Tools_ObjectSystem_SharedValue<Asura_String>::Tools_ObjectSystem_SharedValue(Asura_Hash_ID uParamID, const char* szName)
{
	m_uParamID = uParamID;
	Tools_ObjectSystem_ParamRegistry::RegisterParam(uParamID, szName, TOOLS_OBJECTSYSTEM_PARAM_STRING);
}






//Macro for declaring the hash id value and Tools_ObjectSystem_SharedValue for use
//with the shared paramater dialog controls.
//Of the form :
//g_xParam_Name
//g_uHash_Name
//
//Ideally __hash should equal Asura_GetHashID(__name). It doesn't matter at the moment,
//but may be convenient should they be used for other stuff later.
#define TOOLS_DECLARE_SHARED_PARAM(__type,__hash,__name)\
	static const Asura_Hash_ID g_uHash_ ## __name = __hash;\
	static Tools_ObjectSystem_SharedValue<__type> g_xParam_ ## __name (g_uHash_ ## __name, #__name);



#endif //__TOOLS_OBJECTSYSTEM_PARAM_H__
