#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_OBJECT_SYSTEM_H_
#define __TOOLS_OBJECT_SYSTEM_H_

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_List.h"
#include "Interface/Asura_Overlay.h"
#include "Network/Asura_Guid.h"

#include "Tools_Undo.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_Modal_Overlay.h"

class Tools_Object;

typedef void (*TOOLS_OBJECT_CONTEXT_FUNCTION)(Tools_Object* pxObject,const class Tools_Object_System_ContextMenu* pxMenuOverlay);

class Tools_ChooseObject;

//enum of current copy type being done ,to get around the pain of adding extra parameters
//to the virtual Copy functions
enum ToolsCopyObjectType
{
	TOOLS_COPYOBJECT_NONE,
	TOOLS_COPYOBJECT_DUPLICATE,
	TOOLS_COPYOBJECT_PASTENEW,
	TOOLS_COPYOBJECT_PASTEOVER,
};



class Tools_Object_System
{
	public :
		static void Initialise();
		static void Shutdown();
		static void Update();
		static void RenderPreFSFX();
		static void Render();
		static void RenderAsUI();

		
		//basically deletes all but the camera object
		static void DeleteAllModeSpecific();

		static void DeleteObject(Tools_Object* object);

		static void DestroySelected();
		static void DestroyAllSelected();

		//Destroys the listed objects , possibly asking for confirmation (depending on type and quantity being deleted)
		static void DestroyObjects(const List<Tools_Object*>& xObjectList, const bool bAskForConfirmation = true);

		//change the currently selected object
		static bool SelectObject(Tools_Object*,bool multipleSelect = false, bool bChangePrimarySelection = true);
		static void SelectObjects(const List<Tools_Object*>& objectList,bool multipleSelect);
		static void DeselectObject(Tools_Object*);
		static void DeselectObjects(const List<Tools_Object*>& objectList);
		static void DeselectAllObjects();

		//These functions get the 'choose object' selection if in the choose object submode , or
		//the standard selection if not.
		static void GetSelectionListForCurrentSelectionMode(List<Tools_Object*>& xOutputList);
		static Tools_Object* GetMainSelectionForCurrentSelectionMode();


		static Tools_Object* FindObjectUnderCursor(Tools_Object* pxObjectToIgnore = 0);
		static Tools_Object* FindObjectUnderCursor(const List<Tools_Object*>& xObjectsToIgnore);
		static Tools_Object* FindObjectInLOS(const Asura_Vector_3& alpha, const Asura_Vector_3& beta);

		static class Tools_Object* GetObjectFromGuid(Asura_Guid guid,bool bIncludeSubEntites = false);
		static class Tools_Object* GetOwnerOfMessageBlock(unsigned short usBlockID);
		static class Tools_Object* GetObjectFromToolsID(int iID);
				
		static void EnableAllControls(bool enable);
		static void ToggleObjectControls();
		static bool AreControlsEnabled(){return ControlsEnabled;}
		static bool AreControlsDisabled(){return !ControlsEnabled;} // For disabling overlay dialogs as appropriate

		static bool TypeIsHidden(int type);
		static bool GetTypeHiddenFlag(int iType,int iFlags);
		static void SetTypeHidden(int type,int flags,bool hide);

		static u_int GetVisiblePlatforms(){return s_uVisiblePlatforms;}
		static void SetVisiblePlatforms(u_int uPlatforms);
		static bool IsPlatformVisible(int iPlatform) {return (s_uVisiblePlatforms & (1<<iPlatform))!=0;}
		static void SetPlatformVisible(int iPlatform,bool bVisible);

		static bool IsHideByPlatformEnabled(){return s_bHideByPlatformEnabled;}
		static void SetHideByPlatformEnabled(bool bEnable){s_bHideByPlatformEnabled = bEnable;}
		
		static u_int GetVisibleLightSets(){return s_uVisibleLightSets;}
		static void SetVisibleLightSets(u_int uLightSets);
		static bool IsLightSetVisible(int iLightSet) {return (s_uVisibleLightSets & (1<<iLightSet))!=0;}
		static void SetLightSetVisible(int iLightSet,bool bVisible);

		static bool IsHideByLightSetEnabled(){return s_bHideByLightSetEnabled;}
		static void SetHideByLightSetEnabled(bool bEnable){s_bHideByLightSetEnabled = bEnable;}


		static void ShowUserHideByTypeOverlay();
		
		static bool AllHidden(){return s_iHideAllFlags!=0;}
		static void SetHideAll(int iFlags,bool bHide);

		//get the user to pick an item from the list
		static Tools_Object* ChooseObjectFromList(const char* message,bool includeHiddenObjects=false);

		static void FindObject();
		static void FindObjectByGuid();
		static void GotoAndSelect(Tools_Object* object);
		static void GotoObject(Tools_Object* pxObject);
		static void GotoSelected();

		static void UpdateGroupMovement();
		static void UpdateRelativePositions();

		static void ListObjectsOfType(int type,List<Tools_Object*>& outputList);
		static void ListSelectedObjectsOfType(int iType,List<Tools_Object*>& xOutputList);

		//Message block editing
		static void EditMessageBlock(const char* prompt,unsigned short* messageBlock,int iMessageSet,bool bClientMessages = false);
		static void EditMessageBlock(const char* prompt,unsigned short* messageBlock,bool bClientMessages = false);
		//variation that allows editing of a message block , without having the pointer to it. In this
		//case it isn't allowable to delete all messages from it
		static void EditMessageBlock(const char* szPrompt,unsigned short usMessageBlockID,int iMessageIndex,Tools_Object* pxSenderObject);
		//For editing a Asura_StaticMessageBlock
		static void EditMessageBlock(const char* szPrompt,class Asura_StaticMessageBlock* pxMessageBlock,Tools_Object* pxSenderObject,bool* pbModifiedFlag);
		static void EndMessageBlockEditing();
		static Tools_Object* GetMessageEditObject(){return MessageBlockObject;}
		static bool IsMessageEditing(){return MessageBlockEditing;}

		//warn of message block problems
		static void CheckForMessageBlockOwnershipProblems(){CheckForMessageBlockOwnershipProblems(false);}
		//delete message blocks that noone is claiming ownership for
		static void DeleteOrphanedMessages(){CheckForMessageBlockOwnershipProblems(true);}

		//ask for a name for a new object,or for renaming an object
		static bool AskForName(Asura_String& name,const char* title,const Tools_Object* objectToIgnore = 0,bool bAllowBlank = false,const char* szForcePrefix = 0);
		static bool IsNameInUse(const Asura_String& xName, const Tools_Object* objectToIgnore = 0);

		//Returns true if the object was under the cursor in the most recent update
		static bool IsObjectUnderCursor(const Tools_Object* pxObject){return ObjectUnderCursorList.contains(const_cast<Tools_Object*>(pxObject));}
		
		//objects are added/removed in this list by the Tools_Object constructor/destructor
		static List<Tools_Object*> ObjectList;
		static List<Tools_Object*> ObjectUnderCursorList;
		
		static Tools_Object* SelectedObject;//main object selected for editing etc
		static List<Tools_Object*> SelectedObjectList;//additional selected objects , for group movement

		// selection filtering
		static void SetSelectionFilter( int iToolsObjectType );
		static void ClearSelectionFilter( void );

		//Modeless object selection , calling callback function when done
		static void ChooseObject(const char* pszMessage,Tools_ChooseObject* pxChooseObject,Tools_Object* pxInitialSelection = 0);
		static void ChooseObject_Finish();

		static bool IsChoosingObject(){return s_pxChooseObject!=0;}
		static Tools_Object* GetChosenObject(){return s_pxChooseObject ? s_pxChosenObject : NULL;}

		//For setting a list of found objects , that can be iterated through using Alt+<,Alt+>
		static void SetFindObjectList(const List<Tools_Object*>& xObjectList);

		static void SelectNextFoundObject();
		static void SelectPreviousFoundObject();

		//scales positions of all selected objects
		static void ScaleSelectedPositions(float fScale);
		//as above , but prompt for scale value
		static void ScaleSelectedPositions();

		static class ObjectSystemUndo* GetUndo(){return s_pxUndo;}
		static void EnableUndo();
		static void DisableUndo();

		//Add controls for handling object copying
		static void AddCopyOverlayControls(class Asura_Overlay* pxObject,Asura_Vector_2& xOffset);
		
		//'copies' object for later pasting
		static void CopyToClipboard();
		static void CopyToClipboard(Tools_Object* pxObject, bool bClearClipboard = true);
		//pastes object from clipboard over selected object
		static void PasteOver();
		//Makes a copy of the selected object , slightly offset from the original positions
		static void DuplicateSelected();
		//Pastes the clipboard object to the current position
		static void Paste();

		//Gets the current type of copy in progress
		static enum ToolsCopyObjectType GetCopyType(){return s_eCopyType;}
		static bool IsCopying() {return s_eCopyType != TOOLS_COPYOBJECT_NONE;}

		//functions for enabled status of overlay controls
		static bool IsSelectedCopiable();
		static bool CanPasteOver();
		static bool CanPasteOver(Tools_Object* pxObject);
		static bool CanPaste();

		static void ToggleFlying();

		//merges the meshes of the selected objects into the environment , and then deletes them
		static void MergeSelectedIntoModules();

		//tells the object system where to display the status text for the selected object etc
		static void SetTextRenderPos(float x,float y);

		//resets the status text position to default (top-left)
		static void ResetTextRenderPos();

		//Gets the next line immediately after the last text rendered this frame.
		static float GetMaxTextY() {return s_fMaxTextY;}

		//set the comment to be displayed in top right
		static void SetCommentText(const char* pszText,void* pOwner);
		
		//Removes the comment text if pOwner matches the value used to create the text.
		//If pOwner is 0 , then it clears regardless
		static void ClearCommentText(void* pOwner);

        static void SetPreserveMessageEditing( const bool bKeep ) { s_bPreserveMessageEditing = bKeep; }
        static bool GetPreserveMessageEditing() { return s_bPreserveMessageEditing; }

		static bool IsDoingSnapToVertex(){return DoingSnapToVertex;}

		//Functions for use with the overlay controls that can affect all selected objects
		static bool GetParameterFromSelectedObjects(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam,bool* pbMixed);
		static bool SetParameterForSelectedObjects(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

		//Start doing snap to vertex for selected object
		static void StartSnapToVertex();

		//Duplicate the given tools object (and actual entity) and return the duplicate
		static Tools_Object* DuplicateToolsObject(Tools_Object* pxToolsObject);

		//Determines the object clicked on and fills in the context menu based on this
		static void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);
		//The function that does the work , once the current object has been determined
		static void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext, Tools_Object* pxObject);

		//Overlay callback for populating menu when an object's overlay is clicked on
		static void AddContextMenuItemsFromOverlay(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);


		//Determines whether new lights etc should be created within the mission or the common file.
		//Doesn't entirely belong here , but can live here until a better home appears.
		static bool ShouldCreateNewObjectsInMissionFile();
		static void AddMissionFileTickbox(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);

		static bool ShouldShowPhysicalObjectsInOtherModes() {return s_bShowPhysicalObjectsInOtherModes;}
		static void SetShowPhysicalObjectsInOtherModes(bool bSet) {s_bShowPhysicalObjectsInOtherModes = bSet;}

		//Gets a value that changes whenever the selection is changed.
		//Can be used to detect if any change has happened since last checking
		static u_int GetSelectionChangedStamp(){return s_uSelectionChangedStamp;}

		static void AddToDestroyList(Tools_Object* pxObject) {s_xDestroyObjectList.add_entry(pxObject);}

		//Searches for a texture in graphics\tools\objecticons , and returns a texture index for it if it exists
		static int FindDistantIconTexture(const char* szName);

	private :
		//determines whether the keys/mouse controls for the object system are in use
		static bool ControlsEnabled;

		static void HandleKeyboard();

		
		static void CancelSnapToVertex();
		static void HandleSnapToVertex();

		static void UpdateObjectUnderCursorList();

		static void MergeSelectedIntoModules_ModuleSelected(int iSelectedModule);

		static void ContextMenu_SelectAll(void* pObject);
		static void ContextMenu_SelectAllVisible(void* pObject);
		static void ContextMenu_DeselectAllVisible();
		static void ContextMenu_HideSelected();
		static void ContextMenu_HideUnselected();
		static void ContextMenu_UnhideAll();
		static void ContextMenu_UnhideAllByType();
		

		
		static bool IsFilteredOut( const int iType );

		//update mouse selection for the frame
		static void HandleMouseSelection();
		//deal with non-drag selection when button has been released
		static void HandleNonDragSelect();
		//deal with drag selection when button has been released
		static void HandleDragSelect();

		static int GetCurrentIndexInFindList();

		static void RenderComment();

		static void TransferModeData(int iMode);

		static void ObjectDeleted(Tools_Object* pxObject);

		static void CheckForMessageBlockOwnershipProblems(bool bDeleteOrphans);


		//for snap to vertex
		static bool DoingSnapToVertex;
		static Asura_Vector_3 SnapToVertexPositionStore;
		static Asura_Matrix_3x3 SnapToVertexOrientationStore;
		static bool s_bSnapToVertex_SourceVertexLocked;
		static bool s_bSnapToVertex_PickingSourceVertex;
		static Asura_Vector_3 s_xSnapToVertex_SourceVertex;
		static Asura_Vector_3 s_xSnapToVertex_HighlightPosition;
		static int s_iSnapToVertex_Rotate;
		static bool s_bSnapToVertex_UseCollisionMesh;
		static Asura_Bounding_Box s_xSnapToVertex_SourceBB;
		static int s_iSnapToVertex_SetOrientState;
		static int s_iSnapToVertex_DragStartX;
		static int s_iSnapToVertex_DragStartY;
		static bool s_bSnapToVertex_AlignMode_AlignYAxis;


		//for message block editing
		static bool MessageBlockEditing;
		static Tools_Object* MessageBlockObject;//object owning the message block


		static int s_iHideAllFlags;

		//For filtering visibility by platform and light set type
		static bool s_bHideByPlatformEnabled;
		static u_int s_uVisiblePlatforms;
		static bool s_bHideByLightSetEnabled;
		static u_int s_uVisibleLightSets;

		// selection filter data
		static int	s_iNumSelectionFilters;
		static int*	s_piSelectionFilters;

		static void ChooseObject_Accept();
		
		static Tools_ChooseObject* s_pxChooseObject;
		static Tools_Object* s_pxChosenObject;
		static Asura_String s_xChooseObjectMessage;

		static struct SelectionState
		{
			void Reset(){m_uSelectionFlags = 0;m_pxDragObject = 0;}
			
			union
			{
				struct
				{
					u_int m_bLeftClick:1;
					u_int m_bDragSelect:1;
					u_int m_bSelect:1;//false means deselection
					u_int m_bMultiSelect:1;

				};
				u_int m_uSelectionFlags;
			};

			Asura_Vector_2 m_xDragStart;
			Asura_Vector_2 m_xDragCurrent;
			Tools_Object* m_pxDragObject;
		}s_xSelectionState;

		static Tools_Object* s_pxLastFoundObject;//Last object selected from find list. May not be valid if the object isn't in the list anymore
		static List<Tools_Object*> s_xFindObjectList;

		static class ObjectSystemUndo* s_pxUndo;
		static class ObjectSelectionUndo* s_pxSelectionUndo;

		static Asura_List <Tools_Object> s_xClipboardList;

		static enum ToolsCopyObjectType s_eCopyType;

		// List of objects to be destroyed next update.
		static List<Tools_Object*> s_xDestroyObjectList;

		static Asura_Vector_2 s_xTextPos;
		static float s_fMaxTextY;

		static Asura_UnicodeChar* s_pszCommentText;
		static void* s_pCommentOwner;

        static bool s_bPreserveMessageEditing;

		static Asura_Guid s_uFindObjectGuid;

		friend class Tools_Object;
		friend class ObjectSystemUndo;

		static bool s_bCreateNewObjectsInMissionFile;

		//causes tools objects for physical entities to be created outside of object mode.
		static bool s_bShowPhysicalObjectsInOtherModes;

		static u_int s_uSelectionChangedStamp;

		static Asura_TemplateHashTree<int> s_xDistantIconTree;

	public :
		static bool s_bShowComment;


};



//class to store the callback functions for ChooseObject()
class Tools_ChooseObject
{
public :
	virtual ~Tools_ChooseObject(){};

	//Override and return true to allow selection of more than one object
	virtual bool SupportMultipleSelection() {return false;}	

	//Optional function for doing stuff at the start of the selection process.
	virtual void OnStart(){};
	//For filtering out objects that shouldn't be selectable
	virtual bool CanBeSelected(Tools_Object* pxObject){return true;}
	//Optional function called at the end of the selection process
	//Typically used to delete the instance of Tools_ChooseObject
	virtual void OnFinish(){delete this;}

	//Called when the user has accepted the current selection
	//If using multiple selection then the object provided will be one
	//chosen arbitrarily from the list
	virtual void ObjectSelected(Tools_Object* pxObject) = 0;

	// For every specific Tools_ChooseObject, there might be different keys to 
	// handle in different ways. So put handling of these keys to this function.
	virtual void HandleKeyboard() {}

	//Gets the list of all selected objects , if multiple selection is allowed
	//(well , even if it isn't , but there is less point in that case)	
	const List<Tools_Object*>* GetSelectionList() const {return &s_xSelectedObjectList;}

	//Updates the hidden state of overlays during choose object
	void HideOverlays(bool bHide);


	//////////////////////////////////////////////////////////////////////////////////////
	//Functions used by Tools_Object_System for managing the selection

	Tools_Object* GetMainSelection(){return s_xSelectedObjectList.size() ? s_xSelectedObjectList.last_entry() : 0;}

	bool SelectObject(Tools_Object* pxObject, bool bMultipleSelect);
	void DeselectObject(Tools_Object* pxObject);
	void DeselectAllObjects();

	void ClearList();
	void ClearSelectionFlags();


private :
	List<Tools_Object*> s_xSelectedObjectList;
};

class Tools_ChooseObject_ByType : public Tools_ChooseObject
{
public:
	// Version taking tools object type, pointer of guid to set, and optional pointer to bool to set if changed.
	Tools_ChooseObject_ByType( const int iToolsObjectType, Asura_Guid* puGuidToSet, bool* pbChanged = NULL )
	: m_xTypes(), m_xGuidsToIgnore(), m_puGuidToSet( puGuidToSet ), m_pfnOnChosen( NULL ), m_pbChanged( pbChanged )
	{
		AddType( iToolsObjectType );
	}
	// Version taking tools object type, guid to ignore, pointer of guid to set, and optional pointer to bool to set if changed.
	Tools_ChooseObject_ByType( const int iToolsObjectType, const Asura_Guid uGuidToIgnore, Asura_Guid* puGuidToSet, bool* pbChanged = NULL )
	: m_xTypes(), m_xGuidsToIgnore(), m_puGuidToSet( puGuidToSet ), m_pfnOnChosen( NULL ), m_pbChanged( pbChanged )
	{
		AddType( iToolsObjectType );
		AddGuidToIgnore( uGuidToIgnore );
	}
	// Version taking tools object type, callback function pointer, and optional pointer to bool to set if changed.
	Tools_ChooseObject_ByType( const int iToolsObjectType, void( *pfnOnChosen )( Asura_Guid ), bool* pbChanged = NULL )
	: m_xTypes(), m_xGuidsToIgnore(), m_puGuidToSet( NULL ), m_pfnOnChosen( pfnOnChosen ), m_pbChanged( pbChanged )
	{
		AddType( iToolsObjectType );
	}
	// Version taking tools object type, guid to ignore, callback function pointer, and optional pointer to bool to set if changed.
	Tools_ChooseObject_ByType( const int iToolsObjectType, const Asura_Guid uGuidToIgnore, void( *pfnOnChosen )( Asura_Guid ), bool* pbChanged = NULL )
	: m_xTypes(), m_xGuidsToIgnore(), m_puGuidToSet( NULL ), m_pfnOnChosen( pfnOnChosen ), m_pbChanged( pbChanged )
	{
		AddType( iToolsObjectType );
		AddGuidToIgnore( uGuidToIgnore );
	}

	void AddType( const int iType )				   { m_xTypes.Push_Front( iType ); }
	void AddGuidToIgnore( const Asura_Guid uGuid ) { m_xGuidsToIgnore.Push_Front( uGuid ); }

	virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject );

private:
	Asura_LinkList< int > m_xTypes;
	Asura_LinkList< Asura_Guid > m_xGuidsToIgnore;
	Asura_Guid* m_puGuidToSet;
	void		( *m_pfnOnChosen )( Asura_Guid );
	bool*		m_pbChanged;
};


class ObjectSystemUndo : public VariableSizeUndo
{
public :
	ObjectSystemUndo()
	:VariableSizeUndo(10000){}

	void PreUndoProcessing();


	void Log_Object_Moved(const Tools_Object* pxObject);

};


struct ObjectSelectionUndoData
{
	enum UndoType
	{
		UndoType_SelectedPrimary,
		UndoType_DeselectedPrimary,
		UndoType_Selected,
		UndoType_Deselected,
		UndoType_ForceInt = 0xffffffff,
	};
	enum UndoType m_eUndoType;
	int m_iObjectID;
};


class ObjectSelectionUndo : public FixedSizeUndo
{
public :
	ObjectSelectionUndo()
	:FixedSizeUndo(10000,sizeof(ObjectSelectionUndoData)){}
	
	void ProcessUndoData(void*);

	void LogObjectSelectedPrimary(const Tools_Object* pxObject);
	void LogObjectDeselectedPrimary(const Tools_Object* pxObject);
	void LogObjectSelected(const Tools_Object* pxObject);
	void LogObjectDeselected(const Tools_Object* pxObject);


};


//Version of ListSelection_Overlay for objects that allows filtering by object type
class ObjectSelection_Overlay : public ListSelection_Overlay
{
	public :
		ObjectSelection_Overlay(Asura_String xTitle,const List<Tools_Object*>* pxObjectList,bool bIncludeHidden);
		~ObjectSelection_Overlay();


	protected :
		virtual void AddExtraOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

		virtual void Update();

		virtual bool ShouldItemBeHidden(const char* pszString,int iData,bool bWasHidden);

		void SetOnlyShowSelectable(bool bSet);

	private :
		enum
		{
            FILTER_OBJECTTYPE,      //filter by object ype
            FILTER_SHAPE,           //filter by shape/skin
            FILTER_BLUEPRINTTYPE,   //filter by blueprint type
            FILTER_BLUEPRINT,       //filter by blueprint

			FILTER_NUMTYPES,
		};

		void DeleteFilterList();
		void UpdateFilterList();
		
		void AddToFilterList(const char* pszName,int iData,bool bMatchByName);

		static bool IsNotFilterParamBool(int iJunk);
		static bool IsNotFilterParamInt(int iJunk);
		static bool IsNotFilterParamUInt(int iJunk);
		static bool IsNotFilterParamFloat(int iJunk);
		static bool IsNotFilterParamString(int iJunk);

		List<ListSelectionItem*> m_xFilterList;

		char** m_ppszFilterStrings;

		const List<Tools_Object*>* m_pxObjectList;
		bool m_bIncludeHidden;

		//the filter type in use
		int m_iFilterType;
		bool m_bFilterTypeChanged;

		bool m_bOnlyShowSelectable;

		//the index into the list of options for given filter type
		int m_iFilterIndex;
		bool m_bFilterIndexChanged;

		class Asura_List_Dialog* m_pxFilterDialog;

		//for remembering selection in 'type' list box in between uses of this overlay
		static int s_iLastFilterType;
		static int s_iLastTypeSelected;
		static Asura_String s_xLastShapeSelected;
        static Asura_String s_xLastBlueprintTypeSelected;
        static Asura_String s_xLastBlueprintSelected;

		bool m_bFilterParamChanged;
		bool m_bFilterParamMinChanged;
		bool m_bFilterParamMaxChanged;
	
		static u_int s_uFilterParam;
		static const class Tools_ObjectSystem_Param* s_pxFilterParam;
		static bool s_bFilterParam_Bool;
		static int s_iFilterParam_MinInt;
		static int s_iFilterParam_MaxInt;
		static u_int s_uFilterParam_MinUInt;
		static u_int s_uFilterParam_MaxUInt;
		static float s_fFilterParam_MinFloat;
		static float s_fFilterParam_MaxFloat;
		static Asura_String s_xFilterParam_String;
};

class ObjectFind_Overlay : public ObjectSelection_Overlay
{
	public :
		ObjectFind_Overlay(Asura_String xTitle,const List<Tools_Object*>* pxObjectList,bool bIncludeHidden)
		:ObjectSelection_Overlay(xTitle,pxObjectList,bIncludeHidden)
		{}

		bool ShouldGotoSelected() const {return s_bGotoSelected;}


	protected :
		virtual void AddExtraOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

		virtual void Update();

	private :

		static bool s_bGotoSelected;
		static bool s_bOnlyShowSelectable;
};


class ObjectHideByType_Overlay : public Asura_Overlay
{
	public :
		static void Create(const List<Tools_Object*>* pxObjectList);
		static void Close();
		
		static void ToggleOverlay( void );
		static bool IsOverlayOn( void );

	private :
		ObjectHideByType_Overlay(const List<Tools_Object*>* pxObjectList);
		~ObjectHideByType_Overlay();

		void ShowOverlay();
		virtual void PerFrameUpdate();


		static bool _IsVisible(int iIndex,int iThis)
		{return ((ObjectHideByType_Overlay*)iThis)->IsVisible(iIndex);}

		static void _ToggleVisible(int iIndex,int iThis)
		{((ObjectHideByType_Overlay*)iThis)->ToggleVisible(iIndex);}

		static void _HideAll(int iThis)
		{((ObjectHideByType_Overlay*)iThis)->HideAll();}

		static void _ShowAll(int iThis)
		{((ObjectHideByType_Overlay*)iThis)->ShowAll();}

		static void _InvertVisible(int iThis)
		{((ObjectHideByType_Overlay*)iThis)->InvertVisible();}

		static void OkaySelected(){}

		bool IsVisible(int iIndex);
		void ToggleVisible(int iIndex);
		void HideAll();
		void ShowAll();
		void InvertVisible();
		
		void AddToTypeList(const char* pszTypeName,int iType);

		enum
		{
			SELECTEDTAB_TYPE,
			SELECTEDTAB_PLATFORM,
			SELECTEDTAB_LIGHTSET,
		};

		List<ListSelectionItem*> m_xTypeList;
		int m_iTypeSelectionIndex;
		char** m_ppszTypeStrings;

		bool m_bPlatformEnabled;
		List<ListSelectionItem*> m_xPlatformList;
		int m_iPlatformSelectionIndex;
		char** m_ppszPlatformStrings;

		bool m_bLightSetEnabled;
		List<ListSelectionItem*> m_xLightSetList;
		int m_iLightSetSelectionIndex;
		char** m_ppszLightSetStrings;

		static u_int s_uSelectedTab;
		static Asura_Vector_2 s_xPosition;

};


#endif // __TOOLS_OBJECT_SYSTEM_H_
