//
// Tools_PositionEdit_Object.cpp
//
// Created by Richard Rice - 29/01/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Objects/Tools_PositionEdit_Object.h"
#include "Debugging/Asura_Assert.h"

#include "Rendering/Asura_Text_Output.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"

#include "Objects/Tools_Object_System.h"


unsigned Tools_PositionEdit_Object::s_uNextID = 1;

Tools_PositionEdit_Object::Tools_PositionEdit_Object(Asura_Vector_3* pxPosition,Asura_Matrix_3x3* pxOrientation /*= NULL*/,const char* pszName /*= NULL*/,const char* pszMessage /*= NULL*/,bool* pbModifiedFlag /*=NULL*/)
:Tools_Object(TOT_PositionEdit)
{
	Asura_Assert(pxPosition,"This object must have a valid position pointer");
	m_uID = s_uNextID++;

	Asura_String xMeshName = "ToolsShape_CutsceneTrackPoint";
	Mesh = Asura_Mesh_System::GetMesh(xMeshName);
	
	m_xName = pszName ? pszName : "Position Edit Object";
	if(pszMessage) m_xMessage = pszMessage;
	m_pxPosition = pxPosition;
	m_pxOrientation = pxOrientation;
	m_pxDirection = NULL;
	m_pxOriQuat = NULL;
	m_pbModifiedFlag = pbModifiedFlag;

	//initialise position and orientation from values passed
	Position = *m_pxPosition;
 	if(m_pxOrientation)
 	{
 		Orientation = *m_pxOrientation;
	}
	else
	{
		Orientation.Identity();
	}
}

void Tools_PositionEdit_Object::SetDirectionToEdit(Asura_Vector_3* pxDirection)
{
	m_pxDirection = pxDirection;

	if(pxDirection)
	{
		Asura_Maths::OrientationFromDirection(Orientation,*pxDirection);
	}
}

void Tools_PositionEdit_Object::SetQuatToEdit(Asura_Quat* pxOriQuat)
{
	m_pxOriQuat = pxOriQuat;

	if(pxOriQuat)
	{
		pxOriQuat->ToMatrix(Orientation);
	}
}


void Tools_PositionEdit_Object::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
	if(m_pxPosition) *m_pxPosition = Position;
	if(m_pxOrientation) *m_pxOrientation = Orientation;
	if(m_pxDirection) *m_pxDirection = Asura_Vector_3(Orientation.e13,Orientation.e23,Orientation.e33);
	if(m_pxOriQuat) m_pxOriQuat->FromMatrix(Orientation);
	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;

}

void Tools_PositionEdit_Object::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);
	if(!m_xMessage.IsBlank())
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"%s",m_xMessage.GetString());
	}
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Delete this object when finished");
}

Tools_PositionEdit_Object* Tools_PositionEdit_Object::Find(unsigned uID)
{
	Asura_Assert(uID<s_uNextID,"Unexpected ID");

	if(!uID) return 0;

	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->Type != TOT_PositionEdit) continue;

		Tools_PositionEdit_Object* pxObject = (Tools_PositionEdit_Object*) xObjectLIF();

		if(pxObject->GetID() == uID) return pxObject;
	}

	return 0;
}

void Tools_PositionEdit_Object::Delete(unsigned uID)
{
	//find the object
	Tools_PositionEdit_Object* pxObject = Find(uID);
	//and kill it
	if(pxObject)
	{
		Tools_Object_System::DeleteObject(pxObject);
	}
}


