//////////////////////////////////////////////////////////////////////
// Tools_VolumeBaseEdit_Object.cpp							//
// SJL 14/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Objects/Tools_VolumeBaseEdit_Object.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_Los.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Cursor.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"

#include "Tools_Overlays.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeGroupEdit_Controller.h"

Tools_VolumeBaseEdit_Object* Tools_VolumeBaseEdit_Object::s_pxSelectedVolume = NULL;
Asura_Vector_2 Tools_VolumeBaseEdit_Object::s_xOverlayPosition(10,10);
u_int Tools_VolumeBaseEdit_Object::s_uNextID = 1;
bool Tools_VolumeBaseEdit_Object::s_bWireframe = false;
bool Tools_VolumeBaseEdit_Object::s_bIsDoingSnapToVertex = false;

Tools_VolumeBaseEdit_Object::Tools_VolumeBaseEdit_Object(const char* name, Asura_Volume* pxVolume,  bool* pbModifiedFlag /*=0*/)
:Tools_Object(TOT_VolumeGroupEdit)
{
	m_xName = name;
	m_bChanged = false;
	m_uID = s_uNextID++;
	m_pxParentObject = NULL;
	m_pxOrientQuat = NULL;
	m_pxOrientMat = NULL;
	m_bUseOrientation = false;
	m_bUseOrientationChanged = false;
	
	SetController(NULL);

	m_pszHelpTitle = NULL;
	m_apszHelpTexts = NULL;

	Asura_String mesh_name = "ToolsShape_CutsceneTrackPoint";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	m_pbModifiedFlag = pbModifiedFlag;
}


Tools_VolumeBaseEdit_Object::~Tools_VolumeBaseEdit_Object()
{
	
}

void Tools_VolumeBaseEdit_Object::SetVolumeOrientation(Asura_Quat* pxOrientQuat)
{
	m_pxOrientQuat = pxOrientQuat;
	m_pxOrientMat = NULL;

	m_bUseOrientation = m_pxOrientQuat && !(*m_pxOrientQuat == Asura_Maths::IdentityQuat);

	SetDefaultPositionFromVolume();

}

void Tools_VolumeBaseEdit_Object::SetVolumeOrientation(Asura_Matrix_3x3* pxOrientMat)
{
	m_pxOrientQuat = NULL;
	m_pxOrientMat = pxOrientMat;

	m_bUseOrientation = m_pxOrientMat && (*m_pxOrientMat != Asura_Maths::IdentityMatrix3x3);

	SetDefaultPositionFromVolume();

}


void Tools_VolumeBaseEdit_Object::OnSelection()
{
	if(GetController()) GetController()->VolumeSelected(this);

	SetSelectedVolume(this);
	CreateOverlay();

	AddVolumeEditHelp();	
}


void Tools_VolumeBaseEdit_Object::OnDeselection()
{
	DeleteOverlay();
	SetSelectedVolume(NULL);

	RemoveVolumeEditHelp();
}

void Tools_VolumeBaseEdit_Object::OnReselection()
{
	PARENT::OnReselection();
	CreateOverlay();
}

void Tools_VolumeBaseEdit_Object::AddVolumeEditHelp()
{
	Tools_Help::AddHelpTopic(m_pszHelpTitle, m_apszHelpTexts);
}

void Tools_VolumeBaseEdit_Object::RemoveVolumeEditHelp()
{
	Tools_Help::RemoveHelpTopic(m_pszHelpTitle);
}

void Tools_VolumeBaseEdit_Object::ShowVolumeEditHelp()
{
	Tools_Help::ShowHelpTopic(m_pszHelpTitle);
}



void Tools_VolumeBaseEdit_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(GetTypeDescription(),false,GetOverlayPosition());

	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	AddOverlayControls(pxOverlay,xOffset);


	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

	pxOverlay->m_xOrigin = *GetOverlayPosition();
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}



void Tools_VolumeBaseEdit_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}




void Tools_VolumeBaseEdit_Object::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);
	if(GetController()) GetController()->PrintInfo(x,y);
}


Tools_VolumeBaseEdit_Object* Tools_VolumeBaseEdit_Object::Find(u_int uID)
{
	Asura_Assert(uID<s_uNextID,"Unexpected ID");

	if(!uID) return 0;

	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->Type != TOT_VolumeGroupEdit) continue;

		Tools_VolumeBaseEdit_Object* pxObject = (Tools_VolumeBaseEdit_Object*) xObjectLIF();

		if(pxObject->GetID() == uID) return pxObject;
	}

	return 0;
}

void Tools_VolumeBaseEdit_Object::Delete(u_int uID)
{
	//find the object
	Tools_VolumeBaseEdit_Object* pxObject = Find(uID);
	//and kill it
	if(pxObject)
	{
		Tools_Object_System::DeleteObject(pxObject);
	}
}

void Tools_VolumeBaseEdit_Object::Done()
{
	if (GetController())
	{
		//if there is a controller , it will clear up everything
		GetController()->EndVolumeEditing();
		return;
	}

	//reselect the owning object , if there was one
	if(m_pxParentObject)
	{
		//Make sure it still exists.
		//(In practice the volume edit object should already have been deleted before
		//the owner anyway).
		if(Tools_Object_System::ObjectList.contains(m_pxParentObject))
		{
			Tools_Object_System::SelectObject(m_pxParentObject);
		}
	}


	//and delete this object
	Tools_Object_System::DeleteObject(this);
}

bool Tools_VolumeBaseEdit_Object::UsesLeftButton()
{
	// Capture shift-clicks so we don't get deselected.
	return Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT);
}

void Tools_VolumeBaseEdit_Object::UpdateWhileSelected()
{
	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		//Escape is equivalent to pressing the Done button.
		if (Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ESCAPE )
			&& AllowEndVolumeEditing()
			&& !Tools_Help::IsDisplayingHelp()
			&& !s_bIsDoingSnapToVertex)
		{
			Done();
			return;
		}
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			if (!Asura_Interface::UsingMenusOrOverlays())
			{
				if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) )
				{
					SelectFaceUnderCursor( Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) );
				}
			}

			if(Asura_UserInput::KeyPressed('b'))
			{
				SnapFaceToCursor(false);
			}
			if(Asura_UserInput::KeyPressed('g'))
			{
				SnapFaceToCursor(true);
			}
		}

	}

	s_bIsDoingSnapToVertex = Tools_Object_System::IsDoingSnapToVertex();

	if(m_bUseOrientationChanged)
	{
		m_bUseOrientationChanged = false;
		UpdatePositionInAsuraData(true);
	}

	if (m_bChanged)
	{
		m_bChanged = false;
		UpdatePositionInAsuraData(true);
	}

	if(GetController())	GetController()->Update();

	PARENT::UpdateWhileSelected();
}

void Tools_VolumeBaseEdit_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxDialog;

	if(AllowEndVolumeEditing())
	{
		pxDialog = new Asura_Function_Dialog("End volume editing",xOffset,Done,(int) this);
		pxDialog->SetHelpText("Deletes all the volume editing objects and reselects the owner of this volume.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	pxDialog = new Asura_Function_Dialog( "Show Keys", xOffset, ShowVolumeEditHelp, (int)this );
	pxDialog->SetHelpText("Shows all the shortcut keys");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Wireframe",xOffset,&s_bWireframe);
	pxDialog->SetHelpText("Draw the volume as a wireframe model.\nThis is a global setting for all volume objects.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	if(GetController())
	{
		GetController()->AddOverlayControls(pxOverlay,xOffset);
	}

	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dialog("Volume properties", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Volume specific options.
	if(OrientationEditingAllowed())
	{
		pxDialog = new Asura_Tickbox_Dialog("Use Orientation",xOffset,&m_bUseOrientation,&m_bUseOrientationChanged);
		pxDialog->SetHelpText("Volume will take its orientation from the volume marker object.\nBy default the volume is aligned to the world axes.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
}

void Tools_VolumeBaseEdit_Object::Destroy()
{
	// Remove ourselves from the volume group.
	if(m_pxController)
	{
		m_pxController->DeleteVolume(m_uID);
	}
	// Don't actually destroy here - this will be destroyed by the controller.
	//PARENT::Destroy();
}