//////////////////////////////////////////////////////////////////////
// Tools_VolumeCuboidEdit_Object.h							//
// SJL 14/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef _TOOLS_VOLUMECUBOIDEDIT_OBJECT_H_
#define _TOOLS_VOLUMECUBOIDEDIT_OBJECT_H_

#include "Core/Asura_String.h"
#include "Objects/Tools_Object.h"
#include "Volume/Asura_Volume.h"
#include "Volume/Asura_Volume_Cuboid.h"
#include "Objects/Tools_VolumeBaseEdit_Object.h"

class Tools_VolumeCuboidEdit_Object : public Tools_VolumeBaseEdit_Object
{
private:
	typedef Tools_VolumeBaseEdit_Object PARENT;
public :

	Tools_VolumeCuboidEdit_Object(const char* name, Asura_Volume_Cuboid* pxVolume, bool* pbModifiedFlag = 0);
	~Tools_VolumeCuboidEdit_Object();

	virtual bool ObjectIsInLOS(const Asura_Vector_3& xStart, const Asura_Vector_3& xDir, float& fLowest);
	virtual void Displace(const Asura_Vector_3& xOffset);
	virtual void RotateAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri);

	//Set the min/max extents vectors that restrict editing size.
	void SetMaxExtents( const Asura_Vector_3& xMax ) { m_xMaxExtents = xMax; }
	void SetMinExtents( const Asura_Vector_3& xMin ) { m_xMinExtents = xMin; }
	virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);	// Update game entities position from tools objects position.

protected :
	// Always prefer postion at base of volume
	virtual bool PreferPositionAtBaseOfVolume() {return true;}

	virtual void SetDefaultPositionFromVolume();

	//updates everything from the relative volume and position and orient
	virtual void UpdateFromRelativeVolume();
	//updates the volume from the relative volume
	virtual void UpdateWorldVolumeFromRelative();

	Asura_Vector_3 GetRelativePosition(const Asura_Vector_3& xPosition) const;

	virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);

	virtual Asura_Vector_3 GetRelativePivot() const {return m_xRelativeVolume.CalculateCentre();}
	virtual Asura_Vector_3 GetWorldPivot() const {return m_xWorldVolume.CalculateCentre();}

	virtual void Render();

	virtual void UpdateWhileSelected();
	void ExpandToIncludeModule();
	void SetToModule();
	virtual void SnapFaceToCursor(const bool bAlsoExpandOppositeFace);
	virtual void SelectFaceUnderCursor( const bool bGetFurthest );
	enum VolumeFace
	{
		FACE_NONE = -1,
		FACE_MIN_X,
		FACE_MAX_X,
		FACE_MIN_Y,
		FACE_MAX_Y,
		FACE_MIN_Z,
		FACE_MAX_Z,
		FACE_END_ENUM
	};
	void SetSelectedFace(enum VolumeFace eFace);
	void GetFaceVertices( enum VolumeFace eFace, Asura_Vector_3 axVertices[] );

	enum VolumeFace				m_eSelectedFace;
	Asura_Bounding_Box*			m_pxVolume;
	Asura_Bounding_Box			m_xRelativeVolume;
	Asura_Bounding_Box			m_xWorldVolume;
	Asura_Bounding_Box			m_xOverlayVolume;//values shown on overlay

	Asura_Vector_3				m_xMaxExtents;
	Asura_Vector_3				m_xMinExtents;

	bool						m_bOverlayVolumeChanged;
	bool						m_bMinOverlayVolumeChanged;
	bool						m_bMaxOverlayVolumeChanged;
	bool						m_bEditingRelativeVolume;

};

#endif _TOOLS_VOLUMECUBOIDEDIT_OBJECT_H_