//////////////////////////////////////////////////////////////////////
// Tools_VolumeGroupEdit_Controller.h						//
// SJL 13/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef _TOOLS_VOLUMEGROUPEDIT_CONTROLLER_H_
#define _TOOLS_VOLUMEGROUPEDIT_CONTROLLER_H_

class Asura_Volume;
class Asura_VolumeGroup;
class Tools_Object;
class Tools_VolumeBaseEdit_Object;

class Tools_VolumeGroupEdit_Controller
{
public :
	Tools_VolumeGroupEdit_Controller(bool* pbModified);
	virtual ~Tools_VolumeGroupEdit_Controller();

	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();

	//updates the currently selected index number		
	void VolumeSelected(const Tools_VolumeBaseEdit_Object* pxVolume);
	//add overlay controls to volume's overlay
	virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	//update while one of the volumes is selected
	void Update();

	void PrintInfo(float& x,float& y);

	//shift the positions of all the volume objects
	void MoveVolumeObjects(const Asura_Vector_3& xOffset);

	//rotates the volume objects by xOri around xCentre.
	void RotateVolumeObjectsAbout(const Asura_Vector_3& xCenter, const Asura_Matrix_3x3& xOri);

	//creates/deletes the volume edit objects without selecting them
	void ShowVolumes(bool bShow);

	//Returns true if one of the volumes is selected
	bool IsSelected() const;

	// Should highlight the volumes when rendering them?
	virtual bool IsHighlighted() const { return false; }

	void DeleteVolume(u_int uVolumeID = 0);

	const Tools_VolumeBaseEdit_Object* GetToolsVolume(u_int uIndex) const;
protected :
	//Create and destroy the objects for editing the volumes
	void CreateVolumeObjects(bool bSelect = true);
	void DeleteVolumeObjects();

	virtual bool CanCreateVolume(){return false;}
	virtual bool CanDeleteVolume(){return false;}

	//Add/delete volumes if multiple volumes supported
	//static void CreateVolume(int iThis){((Tools_VolumeGroupEdit_Controller*)iThis)->CreateVolume();}

	static void CreateVolumeCuboid(int iThis){((Tools_VolumeGroupEdit_Controller*)iThis)->CreateVolumeCuboid();}
	static void CreateVolumeSphere(int iThis){((Tools_VolumeGroupEdit_Controller*)iThis)->CreateVolumeSphere();}
	static void CreateVolumeCylinder(int iThis){((Tools_VolumeGroupEdit_Controller*)iThis)->CreateVolumeCylinder();}

	//void CreateVolume();
	void CreateVolumeCuboid();
	void CreateVolumeSphere();
	void CreateVolumeCylinder();

	static void DeleteVolume(int iThis){((Tools_VolumeGroupEdit_Controller*)iThis)->DeleteVolume();}

	//makes sure the volume object for this index exists
	void CreateVolumeObject(int iIndex);

	virtual Asura_Volume* GetVolume(u_int uIndex) = 0;
	//Gets the current number of volumes
	virtual int GetNumberOfVolumes() = 0;

	//Gets a pointer to the bounding box for a given volume
//	virtual Asura_Bounding_Box* GetVolumeBox(int iIndex) = 0;


	//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
	virtual Asura_Quat* GetVolumeOrient(int iIndex) = 0;

	//Gets the name of the owner , to use for part of the volume names
	virtual const char* GetName() = 0;

	//Creates the new volume in the asura data
	virtual void CreateVolumeCuboidMain(){};
	virtual void CreateVolumeSphereMain(){};
	virtual void CreateVolumeCylinderMain(){};

	//Deletes the volume from the asura data
	virtual void DeleteVolumeMain(int iIndex){};

	void SelectVolume(int iIndex);

	//overide for overlay position
	virtual Asura_Vector_2* GetOverlayPosition(){return 0;}

	int m_iNumberOfVolumes;
	u_int* m_puVolumeIDs;

	int m_iSelectedIndex;	
	bool m_bSelectedIndexChanged;

	bool* m_pbModified;
};

class Tools_Default_VolumeGroupEdit_Controller : public Tools_VolumeGroupEdit_Controller
{
public:
	Tools_Default_VolumeGroupEdit_Controller(const char* szName, Asura_VolumeGroup* pxVolumeGroup, bool* pbChangedFlag);

	virtual void StartVolumeEditing();

protected :

	virtual bool CanCreateVolume()	{ return true; }
	virtual bool CanDeleteVolume()  { return GetNumberOfVolumes() != 0; }

	virtual Asura_Volume* GetVolume(u_int uIndex);

	//Gets the current number of volumes
	virtual int GetNumberOfVolumes();

	//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
	virtual Asura_Quat* GetVolumeOrient(int iIndex);

	//Creates the new volume in the asura data
	virtual void CreateVolumeCuboidMain();
	//Creates the new volume in the asura data
	virtual void CreateVolumeSphereMain();
	//Creates the new volume in the asura data
	virtual void CreateVolumeCylinderMain();

	//Deletes the volume from the asura data
	virtual void DeleteVolumeMain(int iIndex);

	virtual const Asura_Vector_3& GetVolumeCreationPosition() const;

	//Gets the name of the owner , to use for part of the volume names
	virtual const char* GetName() { return m_szName; }
private :
	Asura_VolumeGroup* m_pxVolumeGroup;
	const char* m_szName;
};

//Default implementation of controller that just deals with the Asura_VolumeGroup directly
//RAR 24/01/2008
class Tools_Object_VolumeGroupEdit_Controller : public Tools_Default_VolumeGroupEdit_Controller
{
public :

	Tools_Object_VolumeGroupEdit_Controller(Tools_Object* pxOwnerObject, Asura_VolumeGroup* pxVolumeGroup, bool* pbChangedFlag);

	virtual void EndVolumeEditing();

protected :
	virtual const Asura_Vector_3& GetVolumeCreationPosition() const;

private :
	Tools_Object* m_pxOwnerObject;
};



#endif //_TOOLS_VOLUMGROUPEEDIT_CONTROLLER_H_

