//////////////////////////////////////////////////////////////////////
// Tools_VolumeSphereEdit_Object.cpp								//
// SJL 19/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Objects/Tools_VolumeSphereEdit_Object.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_Los.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Cursor.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"

#include "Tools_Overlays.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeGroupEdit_Controller.h"

static const char* s_apszVolumeEditHelpStrings [] =
{
	"Esc :           End volume editing",
	"",
	"9 :             Decrease Radius",
	"Shift + 9 :     Decrease Radius (faster)",
	"0 :             Increase Radius",
	"Shift + 0 :     Increase Radius (faster)",
	"B :             Snap selected face to point under cursor",
	0
};

Tools_VolumeSphereEdit_Object::Tools_VolumeSphereEdit_Object(const char* name, Asura_Volume_Sphere* pxVolume,  bool* pbModifiedFlag /*=0*/)
:Tools_VolumeBaseEdit_Object(name, pxVolume, pbModifiedFlag)
{
	m_pxPosition = pxVolume->GetPositionPtr();
	m_pfRadius = pxVolume->GetRadiusPtr();

	m_bOverlayPositionChanged = false;
	m_bOverlayRadiusChanged = false;

	SetToolsDataFromVolumeData();
	m_pszHelpTitle = "Sphere Volume";
	m_apszHelpTexts = s_apszVolumeEditHelpStrings;

}


Tools_VolumeSphereEdit_Object::~Tools_VolumeSphereEdit_Object()
{

}

void Tools_VolumeSphereEdit_Object::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
	SetVolumeDataFromToolsData();
}

// setdefaultpostionfromvolume
void Tools_VolumeSphereEdit_Object::SetToolsDataFromVolumeData()
{
	Position = *m_pxPosition;
	
	if(m_pxOrientMat)
	{
		Orientation = *m_pxOrientMat;
	}
	else if(m_pxOrientQuat)
	{
		m_pxOrientQuat->ToMatrix(Orientation);	
	}
	else
	{
		Orientation.Identity();
	}
}

// updatefromrelativevolume
void Tools_VolumeSphereEdit_Object::SetVolumeDataFromToolsData()
{

	*m_pxPosition = Position;

	if(m_bUseOrientation)
	{
		if(m_pxOrientQuat)
		{
			m_pxOrientQuat->FromMatrix(Orientation);
		}
		else if(m_pxOrientMat)
		{
			*m_pxOrientMat = Orientation;
		}
	}
	else
	{
		if(m_pxOrientQuat)
		{
			*m_pxOrientQuat = Asura_Maths::IdentityQuat;
		}
		else if(m_pxOrientMat)
		{
			*m_pxOrientMat = Asura_Maths::IdentityMatrix3x3;
		}
	}

	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;
}


void Tools_VolumeSphereEdit_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	Asura_Dialog* pxDialog;
	
	const char* pszExtentText = "Volume parameters (metres)";

	pxDialog = new Asura_Numerical_Dialog("Radius",xOffset,m_pfRadius,0.f,10000.f,0.1f,"%.3f",&m_bOverlayRadiusChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	// Padding so  dialog boxes match up with other types.
	//xOffset.y+=Asura_Dialog::DefaultSpacing();
	//xOffset.y+=Asura_Dialog::DefaultSpacing();
	//xOffset.y+=Asura_Dialog::DefaultSpacing();
	//xOffset.y+=Asura_Dialog::DefaultSpacing();
	//xOffset.y+=Asura_Dialog::DefaultSpacing();
}


void Tools_VolumeSphereEdit_Object::Render()
{
	//render the flyable bit
	Tools_Object::Render();

	Asura_Render::CurrentStates.RequestWireFrameMode(false);

	u_int uColour;
	u_int uSelectedFaceColour;
	u_int uWireframeColour;	//colour to use as additional highlight when doing solid volumes
	if(IsSelected())
	{
		uColour = 0xafafff;
		uSelectedFaceColour = 0xff0000;
		uWireframeColour = 0x0000ff;
	}
	else if(GetController() && (GetController()->IsSelected() || GetController()->IsHighlighted()))
	{
		uColour = 0xafffaf;
		uSelectedFaceColour = uColour;
		uWireframeColour = 0x00ff00;
	}
	else
	{
		uColour = 0xafafaf;
		uSelectedFaceColour = uColour;
		uWireframeColour = 0xffffff;
	}
	if(s_bWireframe)
	{
		uColour |= 0xff000000;
		uSelectedFaceColour |= 0xff000000;
	}
	else
	{
		uColour |= 0x70000000;
		uSelectedFaceColour |= 0x70000000;
	}

	Asura_Matrix_3x3 xOrient = m_bUseOrientation ? Orientation : Asura_Maths::IdentityMatrix3x3;

	Asura_Vector_3 xExtents(*m_pfRadius, *m_pfRadius, *m_pfRadius);

	Tools_Render::RenderSpheroid(Position, xOrient, xExtents, uColour, s_bWireframe, true);

	if(!s_bWireframe)
	{
		//do a wireframe outline as well
		Tools_Render::RenderSpheroid(Position, xOrient, xExtents, uWireframeColour|0xff000000, true, true);

	}


	Asura_Render::CurrentStates.RequestWireFrameMode(Asura_Render::GetWireframe());

}




void Tools_VolumeSphereEdit_Object::UpdateWhileSelected()
{
	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			const float fMultiplier = Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) ? 10.f : 1.f;
			const float fInc = fMultiplier * Asura_Timers::GetFrameTime();
			
			if(Asura_UserInput::KeyPressed('9'))
			{
				*m_pfRadius -= fInc;
				if (*m_pfRadius < 0.f) *m_pfRadius = 0.f;
			}
			if(Asura_UserInput::KeyPressed('0'))
			{
				*m_pfRadius += fInc;
			}
		}
	}
	PARENT::UpdateWhileSelected();
}


bool Tools_VolumeSphereEdit_Object::ObjectIsInLOS(const Asura_Vector_3& xStart, const Asura_Vector_3& xDir, float& fLowest)
{
	bool bInLOS = Tools_Object::ObjectIsInLOS(xStart,xDir,fLowest);
	//TODO test against the volume as well.
	return bInLOS;
}

void Tools_VolumeSphereEdit_Object::Displace(const Asura_Vector_3& xOffset)
{
	Position += xOffset;
	
}

void  Tools_VolumeSphereEdit_Object::RotateAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri)
{
	m_bUseOrientation = true;
	Orientation = xOri * Orientation;
	Position -= xCentre;
	Position = Position * xOri;
	Position += xCentre;
}


void Tools_VolumeSphereEdit_Object::SelectFaceUnderCursor( const bool bGetFurthest )
{

}

void Tools_VolumeSphereEdit_Object::SnapFaceToCursor(const bool bAlsoExpandOppositeFace)
{
	int iPolyIndex;
	int iModuleIndex;
	Asura_Vector_3 xPosition;

	//find the location of the cursor
	if(!Tools_LOS::FindPolyAndPositionUnderCursor(iPolyIndex,iModuleIndex,xPosition)) return;

	float fD = (xPosition - *m_pxPosition).Magnitude();
	*m_pfRadius = fD;
}