//
// Tools_WorldSpaceRectangle_Object.h
//
// Created by Andrew Weinkove - 18/07/02
// Based on Tools_VolumeEdit_Object by Richard Rice.
//
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_WORLDSPACERECTANGLE_OBJECT_H_
#define _TOOLS_WORLDSPACERECTANGLE_OBJECT_H_

#include "Objects/Tools_Object.h"
#include "Entity/Glass/Asura_WorldSpace_Rectangle.h"

// This tools object was created for making panes of glass. However,
// I have tried to keep it as generic as possible, so that it may be
// used for any rectangle in world space.aw.
class Tools_WorldSpaceRectangle_Object : public Tools_Object
{
public:
	Tools_WorldSpaceRectangle_Object(const char* name, Asura_WorldSpace_Rectangle* pane, Tools_WorldSpaceRectangle_Object** referenceToThis);
	~Tools_WorldSpaceRectangle_Object();

	void UpdatePositionInAsuraData(bool);
	const char* GetName() {return Name.GetString();}
	const char* GetTypeDescription(){return "World-space rectangle";}

	bool CanBeDestroyed(){return false;}	// Must delete its parent to delete this.
	void OnSelection();
	void OnDeselection();
	void UpdateWhileSelected();
	void Render();
	void PrintInfo(float& x, float &y);
	bool ConfirmDestruction() {return false;}

	void SetObjectToPosition(Tools_Object *tools_Object_To_Position);

	void CreateOverlay();
	void DeleteOverlay();

	//data
	Asura_String Name;
	Asura_WorldSpace_Rectangle *Pane;
	Asura_Euler_3 EulerOrientation;
	Tools_WorldSpaceRectangle_Object **ReferenceToThis;

	bool ChangedEulerByHand;
	bool Changed;
	bool Wireframe;
	bool LockXOrient;
	bool LockYOrient;
	bool LockZOrient;

	Tools_Object *Tools_Object_To_Position;

	//static data for overlay
	static Asura_Vector_2 OverlayPosition;
	static Tools_WorldSpaceRectangle_Object* ThisVolume;
};


#endif //_TOOLS_WORLDSPACERECTANGLE_OBJECT_H_

