// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___DEBUG_H___
#define ___DEBUG_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "PS2/PS2Strip/DMADebug/DMADebug.h"
#include "PS2/PS2Strip/DMADebug/RTF.h"

// M A C R O S ///////////////////////////////////////////////////////////////

// For debugging the debugger!
#define DEBUG_DMADEBUG (0)

#if DEBUG_DMADEBUG
    #define DMADebug_Printf( str ) { printf( "%s : ", __FILE__ ); printf str ; }
    #define DMADebug_Puts( str ) { printf( "%s : ", __FILE__ ); puts( str ); }
#else
    #define DMADebug_Printf( str ) { }
    #define DMADebug_Puts( str ) { }
#endif

// Output formatting.
#define DMADEBUG_FMT_DATA_FLOAT      DMADEBUG_RTF_COLOUR_DATA "%s" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_DEC        DMADEBUG_RTF_COLOUR_DATA "%d" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_HEX        DMADEBUG_RTF_COLOUR_DATA "0x%x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_2HEX       DMADEBUG_RTF_COLOUR_DATA "0x%02x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_4HEX       DMADEBUG_RTF_COLOUR_DATA "0x%04x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_6HEX       DMADEBUG_RTF_COLOUR_DATA "0x%06x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_8HEX       DMADEBUG_RTF_COLOUR_DATA "0x%08x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_DATA_STR        DMADEBUG_RTF_COLOUR_DATA "%s" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_VERBOSE_STR     DMADEBUG_RTF_COLOUR_VERBOSE "%s" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_VERBOSEA        "("DMADEBUG_FMT_VERBOSE_STR")"
#define DMADEBUG_FMT_MEMCONTENTS_32  DMADEBUG_RTF_COLOUR_MEMCONTENTS "%08x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_MEMCONTENTS_64  DMADEBUG_RTF_COLOUR_MEMCONTENTS "%08x.%08x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_MEMCONTENTS_128 DMADEBUG_RTF_COLOUR_MEMCONTENTS "%08x.%08x.%08x.%08x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_ADDRESS_HEX     DMADEBUG_RTF_COLOUR_FG_YELLOW "%08x" DMADEBUG_RTF_COLOUR_NORMAL
#define DMADEBUG_FMT_NAME_DATA_FLOAT DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_FLOAT
#define DMADEBUG_FMT_NAME_DATA_DEC   DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_DEC
#define DMADEBUG_FMT_NAME_DATA_HEX   DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_HEX
#define DMADEBUG_FMT_NAME_DATA_2HEX  DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_2HEX
#define DMADEBUG_FMT_NAME_DATA_4HEX  DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_4HEX
#define DMADEBUG_FMT_NAME_DATA_6HEX  DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_6HEX
#define DMADEBUG_FMT_NAME_DATA_8HEX  DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_8HEX
#define DMADEBUG_FMT_NAME_DATA_STR   DMADEBUG_RTF_COLOUR_FIELDNAME "%s" DMADEBUG_RTF_COLOUR_NORMAL "=" DMADEBUG_FMT_DATA_STR
#define DMADEBUG_FMT_DATASRCADDR     "[" DMADEBUG_RTF_COLOUR_FG_CYAN "%08x" DMADEBUG_RTF_COLOUR_NORMAL "]"


// The indent used for lines that continue onto the next one
// 24 spaces
// Note, the last char MUST not be a space
#define DMADEBUG_FMT_INDENT "                        :"
#define DMADEBUG_INDENT_SIZE (25)

// B E G I N   N A M E S P A C E /////////////////////////////////////////////

namespace DMADebug
{

// T Y P E S /////////////////////////////////////////////////////////////////

enum DMA_ERROR
{
    ERR__start                     = 0,
    ERR_DEBUG_TOO_MANY_WARNINGS    = 5,   // too many warnings to continue
    ERR_NULL_NEXTTAG               = 10,  // attempted to obtain 'nextTag' but it was unexpectedly NULL.
    ERR_RESERVED_PCE               = 20,  // PCE field is set to 'reserved' value.
    ERR_DMA_ALIGNMENT              = 30,  // dma tag is not quad/word aligned.
    ERR_DMA_INVALID_ADDRESS        = 40,  // dma tag does not lie within main memory or scratch pad!
    ERR_CALLSTACK_OVERFLOW         = 60,  // CALL tag encountered, but callstack is full!
    ERR_INVALID_TAG_FOR_CHANNEL    = 70,  // This tag cannot be used on this DMA channel
    ERR_CANT_FIND_TAG_WITH_DATA    = 80,  // Cannot find any DMA tag in the entire list which has data to transfer
    ERR_INSUFFICIENT_DATA          = 90,  // Need more data, but the DMA chain has terminated or an error occurred
    ERR_DMA_INVALID_DIR            = 100, // Dn_CHCR.DIR mode is not handled
    ERR_DMA_INVALID_MOD            = 110, // Dn_CHCR.MOD > 1 is not handled
    ERR_DMA_OPENFILE               = 120, // Could not open a DMA virtual file

        // VIF related errors
    ERR_VIF_CURSOR_AT_END          = 300, // VIF cursor tried to move, but couldn't because there was insufficient data
    ERR_VIF_UNKNOWN_UNPACK_FORMAT  = 310, // Unknown UNPACK format in VIF code.
    ERR_VIF_MPG_ALIGNMENT          = 320, // MPG statement is not properly aligned.
    ERR_VIF_LIST_ALIGNMENT         = 330, // VIF list is not aligned on a 4 byte boundary
    ERR_VIF_UNKNOWN_VIFCODE        = 340, // An unknown VIF code was encountered
    ERR_VIF_INVALID_VIFCODE        = 350, // An valid VIF code was found on a VIF channel (i.e. 0) that does not support it.
    ERR_VIF_INVALID_CHANNEL        = 360, // You are trying to decode via a channel which is not SCE_DMA_VIF[0/1]
    ERR_VIF_INVALID_MODE           = 380, // MODE register is set to 3, which is 'undefined'.
    ERR_VIF_INVALID_CYCLE          = 390, // The CYCLE register has an invalid setting.
    ERR_VIF_UNPACK_TOO_SHORT       = 400, // The UNPACK function completed, but the internal VIFCODE.length was not 0.
                                          // A Logic error most likely, or the DMA ended prematurely.
    ERR_VIF_MPG_INCOMPLETE         = 410, // Error obtaining more data for a disassembly of an MPG vifcode
    ERR_VIF_STROWCOL_INCOMPLETE    = 420, // Error obtaining more data for a disassembly of an STROW/STCOL vifcode
    ERR_VIF_STMASK_INCOMPLETE      = 430, // Error obtaining more data for a disassembly of an STMASK vifcode

    ERR_GIF_INVALID_PRIM           = 700, // An invalid primitive type was specified in the PRIM field.
    ERR_GIF_INVALID_REG            = 710, // An invalid register number was specified in a REGS field.
    ERR_GIF_INSUFFICIENT_DATA      = 720, // Tried to examine a tag or register setting, but the DMA
                                          // ended prematurely.
    ERR_GIF_INVALID_REGISTER       = 730, // The disassembler encountered a reference to an invalid GS register.
    ERR_GIF_LIST_ALIGNMENT         = 740, // GIF list is not aligned on a 4 byte boundary
    ERR_GIF_INVALID_DIR            = 750, // DIR value is not handled
    ERR_GIF_INVALID_ADDR           = 760, // GIFTAG or register is at an invalid address

    ERR___end
};

enum DMA_WARNING
{
    WARN__start = 1000,
    WARN_UNUSED_TAG_BITS_NON_ZERO  = 1010, // Unused bits in the DMA tag are not zero - possibly
                                           // because >= 1MB is trying to be transferred.
    WARN_ADDR_ALIGNMENT            = 1020, // 'ADDR' field in tag is not aligned to quadword boundary
    WARN_PREMATURE_END             = 1030, // tag encountered specifying the end of the
                                           // DMA transfer, but call stack not empty
    WARN_DMA_LOOKS_SPR             = 1040, // DMA address looks like a SPR address, but bit 31 is not set
    WARN_DMA_NORMAL_QWC_ZERO       = 1050, // 'Normal' DMA started, but QWC == 0
    WARN_DMA_ASP_NOT_ZERO          = 1060, // Source Chain started with CHCR.ASP != 0
    WARN_DMA_TTE_IN_NORM_MODE      = 1070, // TTE is set to 1 in Normal transfer mode
    WARN_DMA_TTE_IN_NONVIF_CHANNEL = 1080, // TTE is set to 1 for Source Chain transfer to non VIF channel
    WARN_DMA_QWC_NOT_ZERO          = 1090, // Source Chain mode transfer started, but QWC != 0.
    WARN_DMA_NULL_REFE             = 1100, // Whole DMA tag appears to be null, which could be user error or clever way to end transfer
                                           // (i.e. REFE + QWC=0 + ADDR=0)

    WARN_VIF_INDETERMINATE_WRITE   = 3000, // Indeterminate word is being written by UNPACK
    WARN_VIF_DIRECT_INCOMPLETE     = 3010, // GIF is awaiting more data when DIRECT/DIRECTHL ends
    WARN_VIF_INVALID_ADDRESS       = 3020, // You are trying to Unpack to an invalid VU memory address (will wrap around)

    WARN_GIF_PRE_DISABLED          = 5000, // PRE bit is set in a GIFTAG, but mode is not PACKED.
    WARN_GIF_SMALL_NREG            = 5010, // Registers are set, but NREG is not high enough to use them
    WARN_GIF_INV_FIELD             = 5030, // An invalid field has been found in a GS Register
    WARN_GIF_TTE_ON                = 5040, // TTE bit is enabled for a transfer thru PATH3
    WARN_GIF_LAST_EOP_ZERO         = 5050, // EOP bit was not set in the last GIF tag.
    WARN_GIF_REGLIST_A_D           = 5060, // A+D field is in one of the REGS fields in REGLIST mode
    WARN_GIF_EXPECTING_DATA        = 5070, // GIF_disassemble ended, but LOOPCNT was not 0 (GIF was expecting more data)
    WARN__end 
}; 
 
// G L O B A L S /////////////////////////////////////////////////////////////
 
extern char  gTempString[];
extern char* gOutputBuffer;
 
// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////
 
bool DEBUG_parseConfig( const char* szInitString );
void DEBUG_reset( void );
int DEBUG_addError( int iErrNum, int iSeverity, const char* szErrString );
int DEBUG_addToOutputBuffer( const char* szToAdd );
int DEBUG_strlen( const char* szStr  );
void DEBUG_addField( DBGMSG eType, char* final, const char* szFormat, ...);
void DEBUG_saveBuffers( const char* szFilename, bool bAppend );
void DEBUG_init( const char* szInitString );
void DEBUG_FreeAll( void );
 
// E N D   N A M E S P A C E /////////////////////////////////////////////////
 
} 
 
// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___DEBUG_H___
