//////////////////////////////////////////////////////////////////////////////
//
// Tools_PS3_Export.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_PS3_EXPORT_H___
#define ___TOOLS_PS3_EXPORT_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Stdlib platform-specific.
#include <windows.h>

// Engine independent.
#include "Core/Asura_Hash.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Hash.h"
#include "Tools_PS3_ShaderExport.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Mesh;
class Asura_Hierarchy_Skin;
struct TextureConversionData;
class Asura_SaveBlock;

// C L A S S E S /////////////////////////////////////////////////////////////
struct Asura_Botanicals_Template;
class Asura_Chunk_Stream;
template< class T > class Asura_LinkList;
class Asura_Mesh;
class Asura_Hierarchy_Skin;
struct TextureConversionData;
class Tools_BGR_Type_BoneWeightCalc;
class Tools_BotanicalElementLOD;
class Tools_BotanicalShape;
class Asura_SaveBlock;

class Tools_PS3_Export_Botanical_Template;

class Tools_PS3_Export
{

public:
    
	// Texture exporting function...
	static bool ExportTexture( const char* szInputName, const char* szOutputName, const int iTextureIndex, const TextureConversionData& xConvertData );

	// Mesh exporting functions...
	static void ExportEnvironment( HANDLE hOutput );
    static bool ExportCharacter( HANDLE hOutput, Asura_Hierarchy_Skin* pxSkin );
    static bool ExportObjectHierarchy( HANDLE hOutput, const Asura_Hierarchy_Skin* pxSkin );

#if defined ASURA_USE_BOTANICALS
	static bool ExportBotanicals( const HANDLE hOutput, const int* const piMaterialConversionList );
#endif

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	static bool ExportMaterialResponses( Asura_SaveBlock& xSaveBlock, Asura_HashTree* const pxMaterialsToExport, Asura_HashTree* const pxSlaveMaterialsToExport );
#endif

#if defined ASURA_USE_STATIC_DECAL_SYSTEM
	static bool ExportStaticDecals( const HANDLE hOutput );
#endif

protected:
	class EnvironmentVertex
	{
	public:
		Asura_Vector_3 m_xPosition;

		u_int m_uDiffuse;

		Asura_HalfFloat m_xU;
		Asura_HalfFloat m_xV;
		Asura_HalfFloat m_xU2;
		Asura_HalfFloat m_xV2;

		u_int m_uNormal;
		u_int m_uTangent;
		u_int m_uBinormal;
		u_int m_uTangent2;
		u_int m_uBinormal2;

		Asura_HalfFloat m_axSHRedCoefficients[4];
		Asura_HalfFloat m_axSHGreenCoefficients[4];
		Asura_HalfFloat m_axSHBlueCoefficients[4];

		void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	};

	class ObjectVertex
	{
	public:

		Asura_Vector_3 m_xPosition;
		float m_xAlpha;

		Asura_HalfFloat m_xU;
		Asura_HalfFloat m_xV;
		Asura_HalfFloat m_xU2;
		Asura_HalfFloat m_xV2;


		u_int m_uNormal;
		u_int m_uTangent;
		u_int m_uBinormal;

		void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	};

#if defined ASURA_USE_BOTANICALS

	// Botanicals export helper functions
	static void FindUsedBotanicalsTemplatesAndShapes( u_int& uNumUsedTemplates, Asura_LinkList< Tools_PS3_Export_Botanical_Template >& xUsedTemplateList,
		u_int& uNumUsedShapes, Asura_LinkList< const Tools_BotanicalShape* >& xUsedShapeList);

	static void ExportBotanicalsShapes( Asura_Chunk_Stream& xStream, const Asura_LinkList< const Tools_BotanicalShape* >& xShapeList, const int* const piMaterialConversionList );
	static void ExportBotanicalsTemplates( Asura_Chunk_Stream& xStream, const Asura_LinkList< Tools_PS3_Export_Botanical_Template >& xTemplateList,
		const Asura_LinkList< const Tools_BotanicalShape* >& xShapeList ); // Temp shape list to find index

	static void IncrementExportSize_Geometry( const Tools_BotanicalElementLOD& xElementLOD );
	static void IncrementExportSize_Billboards( const Tools_BotanicalElementLOD& xElementLOD );
	static void IncrementExportSize_Imposter( const Tools_BotanicalElementLOD& xElementLOD );

	static void ExportBotanicalsElementLOD_Geometry( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData);
	static void ExportBotanicalsElementLOD_Billboards( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData);
	static void ExportBotanicalsElementLOD_Imposter( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData);
#endif // defined ASURA_USE_BOTANICALS

	static int FindAntiAliasEdges( const Asura_Mesh* pxMesh, uint16** ppuOutput );

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___TOOLS_PS3_EXPORT_H___
