//////////////////////////////////////////////////////////////////////////////
//
// Tools_PS3_ShaderExport.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_PS3_SHADER_EXPORT_H___
#define ___TOOLS_PS3_SHADER_EXPORT_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

// Stdlib platform-specific.
#include <windows.h>

// Engine independent.
#include "Core/Asura_FixedArray.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_List.h"
#include "Core/Asura_Maths.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Asura PC
#include "Core/Asura_PlatformBasics.h"

class Tools_MaterialResponse;
class Asura_MaterialResponse;

class Asura_PS3_Shader
{
public:
	Asura_PS3_Shader( Asura_Hash_ID uHashID )
	{
		m_uShaderHashID	= uHashID;
		m_pcShaderData	= NULL;
		m_uShaderSize	= 0;
	}

	~Asura_PS3_Shader()
	{
		CleanUp();
	}

	void CleanUp( void )
	{
		delete[] m_pcShaderData;
		m_pcShaderData	= NULL;
		m_uShaderHashID	= ASURA_HASH_ID_UNSET;
		m_uShaderSize	= 0;
	}

	Asura_Hash_ID	GetShaderID( void )					{ return m_uShaderHashID; }
	void			SetShaderID( Asura_Hash_ID uHash )	{ m_uShaderHashID = uHash; }
	u_int			GetShaderSize( void )				{ return m_uShaderSize; }
	void			SetShaderSize( u_int uSize )		{ m_uShaderSize = uSize; }

	char*			GetShaderData( void )				{ return m_pcShaderData; }

	bool AllocateShaderData( void )
	{
		if( m_uShaderSize >= 0x2000 )
		{
			Asura_Assert( false, "Shader size to big. Probably wrong compilation." );
			m_uShaderSize = 0 ;
			return false;
		}

		delete m_pcShaderData;
		m_pcShaderData = new char[ m_uShaderSize ];
		if( m_pcShaderData == NULL )
		{
			Asura_Assert( false, "Out of memory." );
			return false;
		}
		return true;
	}

private:
	Asura_Hash_ID	m_uShaderHashID;
	u_int			m_uShaderSize;
	char*			m_pcShaderData;
};

typedef Asura_TemplateHashTree<Asura_PS3_Shader*> Asura_PS3_FPTree;

class Tools_PS3_ShaderExport
{
public:
	static bool BeginShaderExport	( void );
	static bool EndShaderExport		( const HANDLE xExportHandle );

	static Asura_Hash_ID AddBotanicalsShader	( const Asura_Hash_ID uMaterial1 );
	static Asura_Hash_ID AddEnvironmentShader	( const Asura_Hash_ID uMaterial1, const Asura_Hash_ID uMaterial2 );
	static Asura_Hash_ID AddObjectShader		( const Asura_Hash_ID uMaterial1, const Asura_Hash_ID uMaterial2 );
	static Asura_Hash_ID AddSmoothSkinShader	( const Asura_Hash_ID uMaterial1, const Asura_Hash_ID uMaterial2 );
	static Asura_Hash_ID AddSkyboxShader		( void );

	static Asura_Hash_ID CreatePartialHashID( const Tools_MaterialResponse* pxMaterial, const Asura_MaterialResponse* const pxAsuraMaterial );

private:
	static void			 SetDefinesFromHash( Asura_Hash_ID uHash );
	static Asura_Hash_ID CombineHashIDs( Asura_Hash_ID uShaderHash1, Asura_Hash_ID uShaderHash2, bool bUseFog = false, bool bUseDamageMaps = false );
	static bool			 AddToTree( Asura_PS3_FPTree& xTree, Asura_Hash_ID uHashID, char* szResourceName );
	static bool			 CompileShader( Asura_PS3_Shader* pxShader, char* szResourceName );

	static void			 Cleanup( void );
	static void			 ExportShaders( const HANDLE hOutput );
	static void			 ExportShaderTree( Asura_PS3_FPTree& xTree, Asura_Chunk_Memory_Stream& xStream );

	static Asura_FixedArray< const char*, 40 >	s_aszTEXTURE_DEFINES;
	static Asura_FixedArray< const char*, 40 >	s_axUsedDefines;
	static Asura_String							s_szCommandLine;

	static u_int								s_uNumUsedDefines;

	static Asura_PS3_FPTree						s_xBotanicalsShaderTree;
	static Asura_PS3_FPTree						s_xEnvironmentShaderTree;
	static Asura_PS3_FPTree						s_xObjectShaderTree;
	static Asura_PS3_FPTree						s_xSmoothSkinShaderTree;
	static Asura_PS3_FPTree						s_xOthersShadersTree;

	static bool									s_bInitialized;
};

#endif //#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#endif // #ifndef ___TOOLS_PS3_SHADER_EXPORT_H___
