//////////////////////////////////////////////////////////////////////////////
//
// Tools_UI_Overrides_Filters.cpp
//
// Created 18 December 2008 by Andrew Southgate
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Tools_UI_Overrides_Filters.h"
#include "Tools_UI_Overrides.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

const char* const Tools_UI_Overrides_Filter::s_pszDefaultList[] =
{
	"All",
};
const int Tools_UI_Overrides_Filter::s_iDefaultListSize = sizeof( s_pszDefaultList ) / sizeof( s_pszDefaultList [ 0 ]);

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Tools_UI_Overrides_Filter::FilterStringArray::FilterStringArray()
: m_iSize( 0 )
, m_iIndex( 0 )
, m_pszStrings( NULL )
, m_bValid( false )
{
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::FilterStringArray'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::FilterStringArray::~FilterStringArray
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::~FilterStringArray'. ///
Tools_UI_Overrides_Filter::FilterStringArray::~FilterStringArray()
{
	Free();
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::~FilterStringArray'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::FilterStringArray::Add
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_UI_Overrides_Filter::FilterStringArray::Add( const char* const szString, const bool bCheckAlreadyAdded /* = true  */ )
{
	if( !m_bValid )
	{
		return( false );
	}
	if( ( szString == NULL ) || ( szString[ 0 ] == 0 ) )
	{
		//nothing to add
		return( true );
	}
	if( bCheckAlreadyAdded )
	{
		for( int i = 0; i < m_iIndex; i++ )
		{
			if( stricmp( szString, m_pszStrings[ i ] ) == 0 )
			{
				//already added
				return( true );
			}
		}
	}
	m_pszStrings[ m_iIndex ] = new char[ strlen( szString ) + 1 ];
	if( m_pszStrings[ m_iIndex ] == NULL )
	{
		m_bValid = false;
		return( false );
	}
	strcpy( m_pszStrings[ m_iIndex ], szString );
	m_iIndex++;
	return( true );
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::Add'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::FilterStringArray::Allocate
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_UI_Overrides_Filter::FilterStringArray::Allocate( const int iSize )
{
	Free();
	if( iSize <= 0 )
	{
		return( false );
	}
	m_pszStrings = new char*[ iSize ];
	if( m_pszStrings == NULL )
	{
		return( false );
	}
	m_iSize		= iSize;
	m_iIndex	= 0;
	m_bValid	= true;
	return( true );
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::Allocate'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::FilterStringArray::Find
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
int Tools_UI_Overrides_Filter::FilterStringArray::Find( const char* const szString, const int iStartIndex /*= 0*/ ) const
{
	if( ( szString == NULL ) || ( szString[ 0 ] == 0 ) )
	{
		return( -1 );
	}
	for( int i = iStartIndex; i < m_iIndex; i++ )
	{
		if( stricmp( szString, m_pszStrings[ i ] ) == 0 )
		{
			//already added
			return( i );
		}
	}
	return( -1 );
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::Find'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::FilterStringArray::Free
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides_Filter::FilterStringArray::Free( void )
{
	if( m_pszStrings != NULL )
	{
		for( int i = 0; i < m_iIndex; i++ )
		{
			delete [] m_pszStrings[ i ];
		}
		delete [] m_pszStrings;
		m_pszStrings = NULL;
	}
	m_iSize		= 0;
	m_iIndex	= 0;
	m_bValid	= false;
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::Free'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::FilterStringArray::GetEntry
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
const char* Tools_UI_Overrides_Filter::FilterStringArray::GetEntry( const int iEntry, const char* const szInvalid /* = "" */ )
{
	if( ( iEntry < 0 ) || ( iEntry >= m_iIndex ) )
	{
		return( szInvalid );
	}
	return( m_pszStrings[ iEntry ] );
}
/// End of function 'Tools_UI_Overrides_Filter::FilterStringArray::GetEntry'. ///

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::Tools_UI_Overrides_Filter
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
Tools_UI_Overrides_Filter::Tools_UI_Overrides_Filter()
: m_pxDropDownList( NULL )
, m_iIndex( 0 )
, m_iLastIndex( -1 )
{
}
/// End of function 'Tools_UI_Overrides_Filter::Tools_UI_Overrides_Filter'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::Clear
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides_Filter::Clear( void )
{
	m_pxDropDownList->SelectEntry( 0 );
	m_pxDropDownList->ReplaceStrings( s_iDefaultListSize, s_pszDefaultList );
	m_xFiltersStringArray.Free();
}
/// End of function 'Tools_UI_Overrides_Filter::Clear'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::GetFilter
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
int Tools_UI_Overrides_Filter::FindFilter( const char* const szFilters ) const
{
	if( ( szFilters == NULL ) || ( szFilters[ 0 ] == 0 ) )
	{
		return( 0 );
	}
	return( m_xFiltersStringArray.Find( szFilters, 1 ) );
}
/// End of function 'Tools_UI_Overrides_Filter::GetFilter'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::GetFilter
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
const char* Tools_UI_Overrides_Filter::GetFilter( void )
{
	if( m_iIndex == 0 )
	{
		return( "" );
	}
	return( m_xFiltersStringArray.GetEntry( m_iIndex, "" ) );
}
/// End of function 'Tools_UI_Overrides_Filter::GetFilter'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::Init
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_UI_Overrides_Filter::Init( Asura_Overlay* const pxOverlay, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2 &xOffset, void (*pfnChanged)( int iUserData ), const int iUserData, bool( *pfnIsEnabled )( int ) )
{
	//add the filter drop-downs
	m_pxDropDownList = new Asura_DropDown_List_Dialog( xOffset, s_pszDefaultList, &m_iIndex, 10, s_iDefaultListSize, NULL, NULL, NULL, true );
	if( m_pxDropDownList == NULL )
	{
		return( false );
	}
	xOffset.y += Asura_Dialog::DefaultSpacing();
	if( pxOverlay != NULL )
	{
		pxOverlay->AddDialog( m_pxDropDownList );
	}
	if( pxTabsDialog != NULL )
	{
		pxTabsDialog->AddDialog( m_pxDropDownList );
	}
	m_pxDropDownList->SetOnSelectCallback( pfnChanged, iUserData );
	m_pxDropDownList->SetIsEnabledFunction( pfnIsEnabled,  iUserData);
	return( true );
}
/// End of function 'Tools_UI_Overrides_Filter::Init'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::Populate
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_UI_Overrides_Filter::Populate( const int iNumEntries, const Asura_String * pxNameArray, const char* const szPrefix, const char* const szPrefix2 )
{
	Clear();

	int i;
	FilterStringArray xFiltersStringArray;
	if( ( iNumEntries > 0 ) && ( pxNameArray != NULL ) )
	{
		xFiltersStringArray.Allocate( iNumEntries );
		for( i = 0; i < iNumEntries; i++ )
		{
			//copy up to the first slash
			const char* szName = pxNameArray[ i ].GetString();

			//does the material name have to have a specific prefix?
			if( ( szPrefix != NULL) && ( szPrefix[ 0 ] != 0 ) )
			{
				if( !Asura_StringUtil::StartsWith( szName, szPrefix, true ) )
				{	
					//ignore this material, the prefix does not match
					continue;
				}
				szName += strlen( szPrefix );
				if( ( szName[ 0 ] != '\\' ) && ( szName[ 0 ] != '/' ) )
				{
					continue;
				}
				while( ( szName[ 0 ] == '\\' ) || ( szName[ 0 ] == '/' ) )
				{
					szName++;
				}
			}
			//does the material name have to have a specific prefix?
			if( ( szPrefix2 != NULL) && ( szPrefix2[ 0 ] != 0 ) )
			{
				if( !Asura_StringUtil::StartsWith( szName, szPrefix2, true ) )
				{	
					//ignore this material, the prefix does not match
					continue;
				}
				szName += strlen( szPrefix2 );
				if( ( szName[ 0 ] != '\\' ) && ( szName[ 0 ] != '/' ) )
				{
					continue;
				}
				while( ( szName[ 0 ] == '\\' ) || ( szName[ 0 ] == '/' ) )
				{
					szName++;
				}
			}


			Asura_String xFilter;
			while( ( szName[ 0 ] != 0 ) && ( szName[ 0 ] != '\\' ) && ( szName[ 0 ] != '/' ) )
			{
				xFilter += szName[ 0 ];
				szName++;
			}
			if( ( szName[ 0 ] != '\\' ) && ( szName[ 0 ] != '/' ) )
			{
				continue;
			}

			if( xFilter.GetLength() > 0 )
			{
				xFiltersStringArray.Add( xFilter.GetString(), true );
			}
		}
		if( !xFiltersStringArray.IsValid() )
		{
			return( false );
		}
	}

	const int iFilterCount = xFiltersStringArray.GetCount();
	m_xFiltersStringArray.Allocate( s_iDefaultListSize + iFilterCount );
	for( i = 0; i < s_iDefaultListSize; i++ )
	{
		m_xFiltersStringArray.Add( s_pszDefaultList[ i ] );
	}
	for( i = 0; i < iFilterCount; i++ )
	{
		m_xFiltersStringArray.Add( xFiltersStringArray.GetEntry( i ) );
	}

	if( !m_xFiltersStringArray.IsValid() )
	{
		m_xFiltersStringArray.Free();
		return( false );
	}

	m_pxDropDownList->ReplaceStrings( m_xFiltersStringArray.GetCount(), m_xFiltersStringArray.GetArray() );
	return( true );
}
/// End of function 'Tools_UI_Overrides_Filter::Populate'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::SetIndex
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides_Filter::SetIndex( const int iIndex )
{
	if( ( iIndex >= 0 ) && ( iIndex < m_xFiltersStringArray.GetCount() ) )
	{
		m_iIndex = iIndex;
	}
}
/// End of function 'Tools_UI_Overrides_Filter::SetIndex'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_Filter::SetIndex
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides_Filter::SetIndex( const char* const szString )
{
	SetIndex( FindFilter( szString ) );
}
/// End of function 'Tools_UI_Overrides_Filter::SetIndex'. ///



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides_FilterSet::Tools_UI_Overrides_FilterSet
// Author       : Andrew Southgate
// Date         : 18 December 2008
////////////////////////////////////////////////////////////////////////////////
Tools_UI_Overrides_FilterSet::Tools_UI_Overrides_FilterSet( Tools_UI_Overrides*	pxParent, Asura_Overlay* const pxOverlay, Asura_Tabs_Dialog* const pxTabsDialog, Asura_Vector_2 &xOffset, void( *pfnChanged )( void* const pxUserData ), void* const pxChangedUserData, bool( *pfnIsEnabled )( int ) /* = NULL */ )
: m_pxParent(pxParent), 
m_pfnChanged( pfnChanged )
, m_pxChangedUserData( pxChangedUserData )
{
	m_xFilterList1.Init( pxOverlay, pxTabsDialog, xOffset, Changed, (int)this, pfnIsEnabled );
	xOffset.y	+= Asura_Dialog::SmallSpacing();
	m_xFilterList2.Init( pxOverlay, pxTabsDialog, xOffset, Changed, (int)this, pfnIsEnabled );
	xOffset.y	+= Asura_Dialog::SmallSpacing();
	m_xFilterList3.Init( pxOverlay, pxTabsDialog, xOffset, Changed, (int)this, pfnIsEnabled );

	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	PopulateFilters( true );
}
/// End of function 'Tools_UI_Overrides_FilterSet::Tools_UI_Overrides_FilterSet'. ///

void Tools_UI_Overrides_FilterSet::Changed( int iUserData )
{
	Tools_UI_Overrides_FilterSet* pxThis = (Tools_UI_Overrides_FilterSet*)iUserData;
	if( pxThis->PopulateFilters() )
	{
		if( pxThis->m_pfnChanged != NULL )
		{
			pxThis->m_pfnChanged( pxThis->m_pxChangedUserData );
		}
	}
}

bool Tools_UI_Overrides_FilterSet::PopulateFilters( const bool bForce /* = false  */)
{
	const bool bChanged1 = m_xFilterList1.HasChanged();
	const bool bChanged2 = m_xFilterList2.HasChanged();
	const bool bChanged3 = m_xFilterList3.HasChanged();

	if( !bForce )
	{
		if ( !bChanged1 )
		{
			if (!bChanged2)
			{
				m_xFilterList3.ResetChanged();
				return( bChanged3 );
			}
		}
	}

	int iNumEntries = m_pxParent->GetNumEntries();
	const Asura_String *pxNameArray = m_pxParent->GetNameArray();

	if( bForce )
	{
		m_xFilterList1.Populate( iNumEntries, pxNameArray, "", "" );
	}
	if( ( bForce ) || ( bChanged1 ) )
	{
		m_xFilterList2.Populate( iNumEntries, pxNameArray, m_xFilterList1.GetFilter(), "" );
	}
	if( ( bForce ) || ( bChanged1 ) || ( bChanged2 ))
	{
		m_xFilterList3.Populate( iNumEntries, pxNameArray, m_xFilterList1.GetFilter(), m_xFilterList2.GetFilter() );
	}


	m_xFilterList1.ResetChanged();
	m_xFilterList2.ResetChanged();
	m_xFilterList3.ResetChanged();

	return( true );
}

bool Tools_UI_Overrides_FilterSet::MatchesFilters( const char* const szString )
{
	const char* sz = ( szString == NULL ) ? "" : szString;
	const char* szFilter = m_xFilterList1.GetFilter();
	if( ( szFilter != NULL ) && ( szFilter[ 0 ] != 0 ) )
	{
		if( !Asura_StringUtil::StartsWith( sz, szFilter, true ) )
		{	
			return( false );
		}
		sz += strlen( szFilter );
		if( ( sz[ 0 ] != '\\' ) && ( sz[ 0 ] != '/' ) )
		{
			return( false );
		}
		while( ( sz[ 0 ] == '\\' ) || ( sz[ 0 ] == '/' ) )
		{
			sz++;
		}
	}

	szFilter = m_xFilterList2.GetFilter();
	if( ( szFilter != NULL ) && ( szFilter[ 0 ] != 0 ) )
	{
		if( !Asura_StringUtil::StartsWith( sz, szFilter, true ) )
		{	
			return( false );
		}
		sz += strlen( szFilter );
		if( ( sz[ 0 ] != '\\' ) && ( sz[ 0 ] != '/' ) )
		{
			return( false );
		}
		while( ( sz[ 0 ] == '\\' ) || ( sz[ 0 ] == '/' ) )
		{
			sz++;
		}
	}

	szFilter = m_xFilterList3.GetFilter();
	if( ( szFilter != NULL ) && ( szFilter[ 0 ] != 0 ) )
	{
		if( !Asura_StringUtil::StartsWith( sz, szFilter, true ) )
		{	
			return( false );
		}
		sz += strlen( szFilter );
		if( ( sz[ 0 ] != '\\' ) && ( sz[ 0 ] != '/' ) )
		{
			return( false );
		}
	}

	return( true );
}

bool Tools_UI_Overrides_FilterSet::SetDefaults( Tools_UI_Overrides_FilterSet* const pxOverridesFilters )
{
	if( pxOverridesFilters == NULL )
	{
		return( false );
	}

	bool bChanged = false;
	m_xFilterList1.SetIndex( pxOverridesFilters->m_xFilterList1.GetFilter() );
	if( PopulateFilters() )
	{
		bChanged = true;
	}
	m_xFilterList2.SetIndex( pxOverridesFilters->m_xFilterList2.GetFilter() );
	if( PopulateFilters() )
	{
		bChanged = true;
	}
	m_xFilterList3.SetIndex( pxOverridesFilters->m_xFilterList3.GetFilter() );
	if( PopulateFilters() )
	{
		bChanged = true;
	}

	if( bChanged )
	{
		if( m_pfnChanged != NULL )
		{
			m_pfnChanged( m_pxChangedUserData );
		}
	}
	return( true );
}



/// End of file. ///
