//
// Tools_PFX_SourceVolume_Object.cpp
//
// Created by Richard Rice - 13/09/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


#include "Particles/Tools_PFX_SourceVolume_Object.h"

#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Mesh/Asura_Mesh_System.h"


#include "Tools_Overlays.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Render.h"
#include "Objects/ToolS_Object_System.h"

static Asura_Vector_2 g_xOverlayPosition(10,10);



Tools_PFX_SourceVolume_Object::Tools_PFX_SourceVolume_Object(const Asura_PFX_Volume& xVolume)
: Tools_VolumeEdit_Object("PFX Source Volume")
, m_xVolume(xVolume)
, m_bGameEntitiesNeedUpdating(false)
, m_bReadyToDelete(false)
, m_bVolumeTypeChanged(false)
, m_xExtentDialogsOffset(Asura_Maths::ZeroVector2)
{
	//initialise some volume edit settings
	m_bEditingRelativeVolume = true;
	m_pxVolume = &m_xVolume.m_xCuboidDesc.m_xBB;
	m_xRelativeVolume = m_xVolume.m_xCuboidDesc.m_xBB;
	m_pbModifiedFlag = &m_bGameEntitiesNeedUpdating;
	m_bUseOrientation = m_xVolume.m_bUseOrientation;

	if(m_xVolume.m_eType == ASURA_PFX_VOLUME_CUBOID)
	{
		UpdateWorldVolumeFromRelative();
	}

	Tools_Object_System::SelectObject(this);
}

void Tools_PFX_SourceVolume_Object::SetPositionAndOrient(const Asura_Vector_3& xPos,const Asura_Matrix_3x3& xOrient)
{
	Position = xPos;
	Orientation = xOrient;


	if(m_xVolume.m_eType == ASURA_PFX_VOLUME_CUBOID)
	{
		UpdateWorldVolumeFromRelative();
	}
}

void Tools_PFX_SourceVolume_Object::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	m_bGameEntitiesNeedUpdating = true;
	
	if(m_xVolume.m_eType == ASURA_PFX_VOLUME_CUBOID)
	{
		Tools_VolumeEdit_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);
	}

}

void Tools_PFX_SourceVolume_Object::OnSelection()
{
	AddVolumeEditHelp();
	CreateOverlay();
}

void Tools_PFX_SourceVolume_Object::OnDeselection()
{
	RemoveVolumeEditHelp();
	DeleteOverlay();

	//On deselection this object should delete itself
	m_bReadyToDelete = true;
}


void Tools_PFX_SourceVolume_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
 	pxOverlay = new Asura_Overlay(GetTypeDescription(),false,&g_xOverlayPosition);
	
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	Asura_Dialog* pxDialog;

	static const char* s_apszTypeStrings[] =
	{
		"Point/None",
		"Cuboid",
		"Sphere",
		"Spheroid"
	};

	pxDialog = new Asura_DropDown_List_Dialog(xOffset,s_apszTypeStrings,(int*)&m_xVolume.m_eType,4,4,0,&m_bVolumeTypeChanged,"Type");
	pxDialog->SetHelpText("Set the type of volume (if any) in which to generate the particles");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("On Surface",xOffset,&m_xVolume.m_uFlags,ASURA_PFXVOLUME_SPAWNONSURFACE,&m_bGameEntitiesNeedUpdating);
	pxDialog->SetHelpText("If ticked then the particles will be generated on the surface of the volume defined.\nThis does nothing for if the volume is just a point.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Use Orientation",xOffset,&m_xVolume.m_uFlags,ASURA_PFXVOLUME_USEORIENTATION,&m_bGameEntitiesNeedUpdating);
	pxDialog->SetHelpText("If ticked then the volume will use the orientation of the marker object.\nIf not then it will be aligned to the world axes.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Wireframe",xOffset,&Tools_VolumeEdit_Object::s_bWireframe);
	pxDialog->SetHelpText("Forces the volume to be rendered in wireframe.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	m_xExtentDialogsOffset = xOffset;

	#if 0
	float fPos = xOffset.y;
	//add controls for all the volume paramaters
	//for sphere
	g_pxDialog_Radius = pxDialog = new Asura_Numerical_Dialog("Radius",xOffset,&m_xVolume.m_xSphereDesc.m_fRadius,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//for spheroid
	xOffset.y = fPos;
	g_pxDialog_ExtentX = pxDialog = new Asura_Numerical_Dialog("X Extent",xOffset,&m_xVolume.m_xSpheroidDesc.m_fExtentX,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	g_pxDialog_ExtentY = pxDialog = new Asura_Numerical_Dialog("Y Extent",xOffset,&m_xVolume.m_xSpheroidDesc.m_fExtentY,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	g_pxDialog_ExtentZ = pxDialog = new Asura_Numerical_Dialog("Z Extent",xOffset,&m_xVolume.m_xSpheroidDesc.m_fExtentZ,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	//for cuboid
	xOffset.y = fPos;

	const char* pszExtentText = "Volume extents (metres) relative to the volume marker object.";

	g_pxDialog_MinX = pxDialog = new Asura_Numerical_Dialog("1: Min X",xOffset,&m_xOverlayVolume.MinX,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	g_pxDialog_MaxX =pxDialog = new Asura_Numerical_Dialog("2: Max X",xOffset,&m_xOverlayVolume.MaxX,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	g_pxDialog_MinY =pxDialog = new Asura_Numerical_Dialog("3: Min Y",xOffset,&m_xOverlayVolume.MinY,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	g_pxDialog_MaxY =pxDialog = new Asura_Numerical_Dialog("4: Max Y",xOffset,&m_xOverlayVolume.MaxY,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	g_pxDialog_MinZ =pxDialog = new Asura_Numerical_Dialog("5: Min Z",xOffset,&m_xOverlayVolume.MinZ,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	g_pxDialog_MaxZ =pxDialog = new Asura_Numerical_Dialog("6: Max Z",xOffset,&m_xOverlayVolume.MaxZ,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	#endif

	xOffset.y += Asura_Dialog::DefaultSpacing()*5 + Asura_Dialog::LargeSpacing();

		
	pxDialog = new Asura_Function_Dialog("Finish Editing",xOffset,_FinishEditing,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	

	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);

	//sort out control visibilities
	UpdateOverlayControls();
}

void Tools_PFX_SourceVolume_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}

void Tools_PFX_SourceVolume_Object::UpdateOverlayControls()
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_SELECTEDOBJECT);
	if(!pxOverlay) return;


	//delete some old dialogs
	while(m_xExtentDialogs.size())
	{
		pxOverlay->DeleteSingleDialog(m_xExtentDialogs.first_entry());
		m_xExtentDialogs.delete_first_entry();
	}
	
	//create new dialogs for the current volume type
	Asura_Vector_2 xOffset = m_xExtentDialogsOffset;
	Asura_Dialog* pxDialog;

	switch(m_xVolume.m_eType)
	{
	   	case ASURA_PFX_VOLUME_POINT :
			break;
		
		case ASURA_PFX_VOLUME_CUBOID :
		{
			const char* pszExtentText = "Volume extents (metres) relative to the volume marker object.";

			pxDialog = new Asura_Numerical_Dialog("1: Min X",xOffset,&m_xOverlayVolume.MinX,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
			pxDialog->SetHelpText(pszExtentText);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);

			pxDialog = new Asura_Numerical_Dialog("2: Max X",xOffset,&m_xOverlayVolume.MaxX,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
			pxDialog->SetHelpText(pszExtentText);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);

			pxDialog = new Asura_Numerical_Dialog("3: Min Y",xOffset,&m_xOverlayVolume.MinY,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
			pxDialog->SetHelpText(pszExtentText);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
			
			pxDialog = new Asura_Numerical_Dialog("4: Max Y",xOffset,&m_xOverlayVolume.MaxY,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
			pxDialog->SetHelpText(pszExtentText);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
			
			pxDialog = new Asura_Numerical_Dialog("5: Min Z",xOffset,&m_xOverlayVolume.MinZ,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
			pxDialog->SetHelpText(pszExtentText);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
			
			pxDialog = new Asura_Numerical_Dialog("6: Max Z",xOffset,&m_xOverlayVolume.MaxZ,0,1000,0.1f,"%.3f",&m_bOverlayVolumeChanged);
			pxDialog->SetHelpText(pszExtentText);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
			break;
		}

		case ASURA_PFX_VOLUME_SPHERE :
			pxDialog = new Asura_Numerical_Dialog("Radius",xOffset,&m_xVolume.m_xSphereDesc.m_fRadius,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
			break;

		case ASURA_PFX_VOLUME_SPHEROID :
			pxDialog = new Asura_Numerical_Dialog("X Extent",xOffset,&m_xVolume.m_xSpheroidDesc.m_fExtentX,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);

			pxDialog = new Asura_Numerical_Dialog("Y Extent",xOffset,&m_xVolume.m_xSpheroidDesc.m_fExtentY,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
		
			pxDialog = new Asura_Numerical_Dialog("Z Extent",xOffset,&m_xVolume.m_xSpheroidDesc.m_fExtentZ,0.0f,999.0f,0.1f,"%.3f",&m_bGameEntitiesNeedUpdating);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			m_xExtentDialogs.add_entry(pxDialog);
			break;

		default :
			Asura_Assert(0, "Unknown volume type");
			break;
	}
	


	pxOverlay->ResizeOverlay();
}

void Tools_PFX_SourceVolume_Object::UpdateWhileSelected()
{
	//update UseOrientation in parent class
	m_bUseOrientation = m_xVolume.m_bUseOrientation;

	if(m_bVolumeTypeChanged)
	{
		//set some suitable defaults for new volume type
		switch(m_xVolume.m_eType)
		{
			case ASURA_PFX_VOLUME_POINT :
				break;
			
			case ASURA_PFX_VOLUME_CUBOID :
				m_xVolume.m_xCuboidDesc.m_xBB.MinX = -1.0f;
				m_xVolume.m_xCuboidDesc.m_xBB.MaxX = 1.0f;
				m_xVolume.m_xCuboidDesc.m_xBB.MinY = -1.0f;
				m_xVolume.m_xCuboidDesc.m_xBB.MaxY = 1.0f;
				m_xVolume.m_xCuboidDesc.m_xBB.MinZ = -1.0f;
				m_xVolume.m_xCuboidDesc.m_xBB.MaxZ = 1.0f;

				m_xRelativeVolume = m_xVolume.m_xCuboidDesc.m_xBB;
				UpdateWorldVolumeFromRelative();
				break;

			case ASURA_PFX_VOLUME_SPHERE :
				m_xVolume.m_xSphereDesc.m_fRadius = 1.0f;
				break;

			case ASURA_PFX_VOLUME_SPHEROID :
				m_xVolume.m_xSpheroidDesc.m_fExtentX = 1.0f;
				m_xVolume.m_xSpheroidDesc.m_fExtentY = 1.0f;
				m_xVolume.m_xSpheroidDesc.m_fExtentZ = 1.0f;
				break;

			default :
				Asura_Assert(0, "Unknown volume type");
				break;
		}
		//update the visible controls
		UpdateOverlayControls();

		//and change the modified flags
		m_bVolumeTypeChanged = false;
		m_bGameEntitiesNeedUpdating = true;
	}

	if(m_xVolume.m_eType == ASURA_PFX_VOLUME_CUBOID)
	{
		//if editing cuboid , then allow the parent class controls to be used
		Tools_VolumeEdit_Object::UpdateWhileSelected();
	}
	else
	{
		if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
		{
            //Escape is equivalent to pressing the Done button.
            if ( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ESCAPE ) && AllowEndVolumeEditing() && !Tools_Help::IsDisplayingHelp())
			{
				Done();
				return;
			}
		}
	}
}

void Tools_PFX_SourceVolume_Object::Render()
{

	u_int uColour = 0x7f7f7fff;
	Asura_Matrix_3x3 xOrient = m_xVolume.m_bUseOrientation ? Orientation : Asura_Maths::IdentityMatrix3x3;

	switch(m_xVolume.m_eType)
	{
		case ASURA_PFX_VOLUME_POINT :
			Tools_Object::Render();
			break;
		
		case ASURA_PFX_VOLUME_CUBOID :
			Tools_VolumeEdit_Object::Render();
			break;

		case ASURA_PFX_VOLUME_SPHERE :
		{
			Tools_Object::Render();
			Asura_Vector_3 xExtents;
			xExtents.x = m_xVolume.m_xSphereDesc.m_fRadius;
			xExtents.y = m_xVolume.m_xSphereDesc.m_fRadius;
			xExtents.z = m_xVolume.m_xSphereDesc.m_fRadius;
			Tools_Render::RenderSpheroid(Position,xOrient,xExtents,uColour,Tools_VolumeEdit_Object::s_bWireframe);
			break;
		}

		case ASURA_PFX_VOLUME_SPHEROID :
		{
			Tools_Object::Render();
			Asura_Vector_3 xExtents;
			xExtents.x = m_xVolume.m_xSpheroidDesc.m_fExtentX;
			xExtents.y = m_xVolume.m_xSpheroidDesc.m_fExtentY;
			xExtents.z = m_xVolume.m_xSpheroidDesc.m_fExtentZ;
			Tools_Render::RenderSpheroid(Position,xOrient,xExtents,uColour,Tools_VolumeEdit_Object::s_bWireframe);
			break;
		}

		default :
			Asura_Assert(0, "Unknown volume type");
			break;
	}
}

void Tools_PFX_SourceVolume_Object::Done()
{
	//The PFX entity will clear this up upon seeing that it is on longer selected.
	//Not calling the parent implementation of Done() to prevent this from being deleted before then.
	//RAR 16/01/2009
	Tools_Object_System::SelectObject(0);
}



/// End of file. ///
