//
// Tools_RagdollCollision_Object.cpp
//
// Created by KJFL - 22 October 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Ragdoll/Tools_RagdollCollision_Object.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Physics/Asura_Physics_Ragdoll_Description.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"


// Tools
#include "Misc/Tools_Render.h"
#include "Objects/Tools_Object_System.h"
#include "Ragdoll/Tools_Mode_RagdollEditor.h"

Asura_List<Tools_RagdollCollision_Object> Tools_RagdollCollision_Object::s_xList;

Asura_Vector_2 Tools_RagdollCollision_Object::s_xPropertiesOverlayPosition(10,10);


Tools_RagdollCollision_Object::Tools_RagdollCollision_Object( Asura_Physics_Ragdoll_Desc_CollisionObject* pxCollisionObject, u_int uCollisionObjectIndex, const char* szNodeName )
:Tools_Object(TOT_RagdollCollision)
, m_pxCollisionObject( pxCollisionObject )
, m_uCollisionObjectIndex( uCollisionObjectIndex )
{
	m_xName = "Col_";
	m_xName += szNodeName;

	m_fRadius = m_pxCollisionObject->GetCollisionExtents().GetX();
	m_bRadiusModified = false;

	m_fMass = 1.0f / m_pxCollisionObject->GetInvMass();
	m_bMassModified = false;

	s_xList.AddToList( this );
}

Tools_RagdollCollision_Object::~Tools_RagdollCollision_Object()
{
	s_xList.RemoveFromListIfPresent( this );

	Tools_Mode_RagdollEditor::DeleteCollisionObjectCallback( this );
}

const char* Tools_RagdollCollision_Object::GetName( void )
{
	return m_xName.GetString();
}

void Tools_RagdollCollision_Object::UpdatePosition( const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xMat )
{
	Position = xPos;
	Orientation = xMat;
}

void Tools_RagdollCollision_Object::Update()
{
	if (m_bRadiusModified)
	{
		Asura_AVector3 xEntents = m_pxCollisionObject->GetCollisionExtents();

		switch (m_pxCollisionObject->GetCollisionProxyType())
		{
			default:
			case Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_SPHERE:
			{
				// with spheres we change all axes
				xEntents.SetX( m_fRadius );
				xEntents.SetY( m_fRadius );
				xEntents.SetZ( m_fRadius );
				break;
			}
			case Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_CAPSULE:
			{
				// with cylinders we leave the Z axis alone
				xEntents.SetX( m_fRadius );
				xEntents.SetY( m_fRadius );
				break;
			}
		}

		m_pxCollisionObject->Tools_SetCollisionExtents( xEntents );

		m_bRadiusModified = false;
	}

	if (m_bMassModified)
	{
		m_pxCollisionObject->Tools_SetMass( m_fMass );
		m_bMassModified = false;
	}
}
static void RenderCapsule( const Asura_AVector3& xCentre, const Asura_AMatrix3x3& xOrientation, const Asura_AVector3& xExtents, u_int uARGB )
{
	const u_int uNumSlices = 4;
	const u_int uNumSegments = 8;
	const float fRadius = xExtents.GetX();
	const float fHalfCylinderLength = Asura_Maths::Max( xExtents.GetZ() - fRadius, 0.0f );

	const Asura_AVector3	xOldBottomVertex(0.0f, fRadius, -fHalfCylinderLength);
	const Asura_AVector3	xOldTopVertex(0.0f, fRadius, fHalfCylinderLength);

	Asura_AVector3			xTransformedOldBottomVertex;
	Asura_AVector3			xTransformedOldTopVertex;

	//
	// Draw the cylinder part
	//

	xTransformedOldBottomVertex = xOldBottomVertex * xOrientation;
	xTransformedOldTopVertex = (xOldTopVertex)* xOrientation;

	xTransformedOldBottomVertex	+= xCentre;
	xTransformedOldTopVertex	+= xCentre;

	for (u_int uSegment = 1; uSegment <= uNumSegments; uSegment++)
	{
		const Asura_Angle		xAngle	= (Asura_Maths::Deg360 * static_cast<float>(uSegment)) / uNumSegments;
		const float				fX		= fRadius * Asura_Maths::Sine(xAngle);
		const float				fY		= fRadius * Asura_Maths::Cosine(xAngle);

		const Asura_AVector3	xNewBottomVertex(fX, fY, -fHalfCylinderLength);
		const Asura_AVector3	xNewTopVertex(fX, fY, fHalfCylinderLength);

		Asura_AVector3			xTransformedNewBottomVertex;
		Asura_AVector3			xTransformedNewTopVertex;

		xTransformedNewBottomVertex = (xNewBottomVertex)* xOrientation;
		xTransformedNewTopVertex = (xNewTopVertex)* xOrientation;

		xTransformedNewBottomVertex	+= xCentre;
		xTransformedNewTopVertex	+= xCentre;

		Asura_Render::RenderLine(xTransformedOldBottomVertex, xTransformedNewBottomVertex, uARGB);
		Asura_Render::RenderLine(xTransformedNewBottomVertex, xTransformedNewTopVertex, uARGB);
		Asura_Render::RenderLine(xTransformedNewTopVertex, xTransformedOldTopVertex, uARGB);

		// Step onto the next vertex...
		xTransformedOldBottomVertex	= xTransformedNewBottomVertex;
		xTransformedOldTopVertex	= xTransformedNewTopVertex;
	}

	//
	// Draw the end hemisphere parts
	//

	// Add the slices to be rendered...
	// Don't do slice 0 or slice = uNumSlices because their radius is 0
	for (u_int uSlice = 1; uSlice < uNumSlices; uSlice++)
	{
		const Asura_Angle xSliceAngle = (Asura_Maths::Deg90 * static_cast<float>(uSlice)) / uNumSlices;
		const float	fSliceRadius = fRadius * Asura_Maths::Sine(xSliceAngle);
		const float	fZ = fHalfCylinderLength + (fRadius * Asura_Maths::Cosine(xSliceAngle));
		const float	fZ2 = (-fHalfCylinderLength) - (fRadius * Asura_Maths::Cosine(xSliceAngle));

		Asura_AVector3 xLastVertex[2];
		for (u_int uSegment = 0; uSegment <= uNumSegments; uSegment++)
		{
			const Asura_Angle xAngle = (Asura_Maths::Deg360 * static_cast<float>(uSegment)) / uNumSegments;
			const float	fX = (fSliceRadius * Asura_Maths::Sine(xAngle));
			const float	fY = (fSliceRadius * Asura_Maths::Cosine(xAngle));

			for(u_int n = 0; n < 2; n++)
			{
				Asura_AVector3 xThisVertex(fX, fY, n == 0 ? fZ : fZ2);

				if (uSegment > 0)
				{
					Asura_AVector3 xTransformed[2];

					xTransformed[0] = (xLastVertex[n])* xOrientation;
					xTransformed[1] = (xThisVertex)* xOrientation;

					xTransformed[0] += xCentre;
					xTransformed[1] += xCentre;

					Asura_Render::RenderLine(xTransformed[0], xTransformed[1], uARGB);
				}
				xLastVertex[n] = xThisVertex;
			}
		}
	}

	// Add the segments to be rendered...
	for (u_int uSegment = 0; uSegment < uNumSegments; uSegment++)
	{
		Asura_AVector3 xLastVertex[2];
		for (u_int uSlice = 0; uSlice <= uNumSlices; uSlice++)
		{
			const Asura_Angle xSliceAngle = (Asura_Maths::Deg90 * static_cast<float>(uSlice)) / uNumSlices;
			const float	fSliceRadius = fRadius * Asura_Maths::Sine(xSliceAngle);
			const Asura_Angle xAngle = (Asura_Maths::Deg360 * static_cast<float>(uSegment)) / uNumSegments;

			const float	fX = (fSliceRadius * Asura_Maths::Sine(xAngle));
			const float	fY = (fSliceRadius * Asura_Maths::Cosine(xAngle));

			for(u_int n = 0; n < 2; n++)
			{
				float fZ;
				
				if(n == 0)
				{
					fZ = fHalfCylinderLength + (fRadius * Asura_Maths::Cosine(xSliceAngle));
				}
				else
				{
					fZ = (-fHalfCylinderLength) - (fRadius * Asura_Maths::Cosine(xSliceAngle));
				}

				Asura_AVector3 xThisVertex(fX, fY, fZ);

				if (uSlice > 0)
				{
					Asura_AVector3 xTransformed[2];

					xTransformed[0] = (xLastVertex[n])*xOrientation;
					xTransformed[1] = (xThisVertex)*xOrientation;

					xTransformed[0] += xCentre;
					xTransformed[1] += xCentre;

					Asura_Render::RenderLine(xTransformed[0], xTransformed[1], uARGB);
				}
				xLastVertex[n] = xThisVertex;
			}
		}
	}
}

void Tools_RagdollCollision_Object::Render()
{
	const Asura_Vector_3 xSizes( m_pxCollisionObject->GetCollisionExtents() );

	// Render the node as a grey sphere if not selected
	u_int uNodeColour = 0xff3f3f3f;

	// Match what the Tools does for object selection - red for prime selection, blue for rest of a group selection
	if ( RenderAsSelected() )
	{
		uNodeColour = 0xffff0000;
	}
	else if (IsGroupSelected())
	{
		uNodeColour = 0xff0000ff;
	}

	switch (m_pxCollisionObject->GetCollisionProxyType())
	{
		default:
		case Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_SPHERE:
		{
			Tools_Render::RenderSpheroid( Position, Orientation, xSizes, uNodeColour, true );
			break;
		}
		case Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_CAPSULE:
		{
			// take relative orientation into world space
			Asura_AMatrix3x3 xMatrix = m_pxCollisionObject->GetRelativeOrientation();

			// KJFL - 23 October 2008 - must remember that AMatrix3x3 currently has
			// multiply wrong way round!
			xMatrix = xMatrix*Asura_AMatrix3x3( Orientation ) ;
			Asura_Vector_3 xAxis( xMatrix.GetColumn2() );

			Asura_AVector3 xPosition2( Position + xAxis * xSizes.GetZ() );

			//Tools_Render::RenderCylinder( Position, xPosition2, xSizes.GetX(), uNodeColour, true );

			RenderCapsule( xPosition2, xMatrix, m_pxCollisionObject->GetCollisionExtents(), uNodeColour);
		
			break;
		}
	}
}

void Tools_RagdollCollision_Object::GetRagdollObjectOrientationAndPosition( Asura_AMatrix3x3& xMatrix, Asura_AVector3& xPosition ) const
{
	// take relative orientation into world space
	xMatrix = m_pxCollisionObject->GetRelativeOrientation();

	// KJFL - 23 October 2008 - must remember that AMatrix3x3 currently has
	// multiply wrong way round!
	xMatrix = xMatrix*Asura_AMatrix3x3( Orientation );

	xPosition = (m_pxCollisionObject->GetRelativePosition() * xMatrix) + Asura_AVector3(Position);
}

	

void Tools_RagdollCollision_Object::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);
}

bool Tools_RagdollCollision_Object::IsSelectable()
{
	return true;
}

void Tools_RagdollCollision_Object::OnSelection()
{
	CreatePropertiesOverlay();
}

void Tools_RagdollCollision_Object::OnDeselection()
{
	DeletePropertiesOverlay();
}

void Tools_RagdollCollision_Object::OnGroupDeselection()
{
	DeletePropertiesOverlay();
}

bool Tools_RagdollCollision_Object::ObjectIsInLOS(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta, float& fLowest)
{
	float fDist = fLowest;
	bool bInLOS;

	switch (m_pxCollisionObject->GetCollisionProxyType())
	{
		default:
		case Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_SPHERE:
		{
			// Do LOS test against the sphere
			bInLOS = Asura_Maths::RayIntersectsSphere( xAlpha, xBeta, Position, m_pxCollisionObject->GetCollisionExtents().GetX(), &fDist );
			if ( bInLOS )
			{
				fLowest = fDist;
			}												   

			break;
		}
		case Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_CAPSULE:
		{
			// Do LOS test against a cylinder - TODO, do we need trivial reject or some precalculation here?
			
			// take relative orientation into world space
			Asura_AMatrix3x3 xMatrix = m_pxCollisionObject->GetRelativeOrientation();

			// KJFL - 23 October 2008 - must remember that AMatrix3x3 currently has
			// multiply wrong way round!
			xMatrix = xMatrix*Asura_AMatrix3x3( Orientation );
			Asura_Vector_3 xAxis( xMatrix.GetColumn2() );

			Asura_Vector_3 xPosition = Position;
			Asura_Vector_3 xAlongAxis = xAxis * m_pxCollisionObject->GetCollisionExtents().GetZ()*2.0f;
			float fRadius = m_pxCollisionObject->GetCollisionExtents().GetX();

			bInLOS = Asura_Maths::RayIntersectsCylinder( xAlpha, xBeta, xPosition, xAlongAxis, fRadius, &fDist );
			if ( bInLOS )
			{
				fLowest = fDist;
			}

			break;
		}
	}


	return bInLOS;
}


void Tools_RagdollCollision_Object::CreatePropertiesOverlay( void )
{
	Asura_Overlay* pxOverlay = new Asura_Overlay( GetName(), false, &s_xPropertiesOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	
	pxDialog = new Asura_Numerical_Dialog( "Radius (m)", xOffset, &m_fRadius, 0.01f, 1.0f, 0.01f, "%1.2f", &m_bRadiusModified );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Mass (kg)", xOffset, &m_fMass, 0.1f, 100.0f, 0.1f, "%1.1f", &m_bMassModified );
	pxOverlay->AddDialog( pxDialog );


	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
	
}

void Tools_RagdollCollision_Object::DeletePropertiesOverlay( void )
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}


