//
// Tools_RagdollJoint_Object.h
//
// Created by KJFL - 27 October 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _Tools_RagdollJoint_Object_H_
#define _Tools_RagdollJoint_Object_H_

#include "Core/Asura_List.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"

class Asura_Physics_Ragdoll_Desc_Joint;
class Asura_StringDisplay_Dialog;
class Tools_Object_RotateGizmo;

class Tools_RagdollJoint_Object : public Tools_Object
{
	public :

		Tools_RagdollJoint_Object( Asura_Physics_Ragdoll_Desc_Joint* pxJoint, const char* szNodeName );

		~Tools_RagdollJoint_Object();

		const char* GetName();
		const char* GetTypeDescription(){return "RagdollJoint";}

		virtual bool IsSelectable();
		virtual void OnSelection();
		virtual void OnDeselection();
		virtual void OnGroupDeselection();

		virtual void Update();
		virtual void Render();
		virtual void PrintInfo( float& x,float& y );
		virtual bool CanBeDestroyed(){return true;}
		virtual bool CanBeMoved(){return false;}

		void UpdatePosition(const Asura_Vector_3& xPos,const Asura_Matrix_3x3& xMat);
		virtual bool ObjectIsInLOS(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta, float& fLowest);

		static Asura_List<Tools_RagdollJoint_Object>& GetList(void) { return s_xList; }

		Asura_Physics_Ragdoll_Desc_Joint* GetJoint( void ) const
		{
			return m_pxJoint;
		}

		void SetJoint( Asura_Physics_Ragdoll_Desc_Joint* pxJoint )
		{
			m_pxJoint = pxJoint;
		}

		void UpdateDialogLabels( void );

	private :

		void CreatePropertiesOverlay( void );
		void DeletePropertiesOverlay( void );

		static void ChooseCollisionObject( void );
		static void RevertOrientation( void );

		Asura_Physics_Ragdoll_Desc_Joint*				m_pxJoint;
		
		Asura_StringDisplay_Dialog* 					m_pxCol1Dialog;
		Asura_StringDisplay_Dialog* 					m_pxCol2Dialog;

		Asura_String									m_xName;

		Tools_Object_RotateGizmo*						m_apxRotateGizmos[3];

		float											m_fTwistLimit;
		float											m_fSwingLimitX;
		float											m_fSwingLimitY;
		bool											m_bAnglesModified;

		static Asura_List<Tools_RagdollJoint_Object>	s_xList;
		static Asura_Vector_2							s_xPropertiesOverlayPosition;
		static Tools_RagdollJoint_Object*				s_pxCurrentlyEditedObject;
};

class Tools_JointChooseCollisionObject : public Tools_ChooseObject
{
	public :
		Tools_JointChooseCollisionObject ( Tools_RagdollJoint_Object* pxJoint )
		: m_pxJoint( pxJoint )				
		{}

		// override default behaviour as we want two collision objects
		virtual bool SupportMultipleSelection() { return true; }	

		virtual bool CanBeSelected( Tools_Object* pxObject );
		virtual void ObjectSelected( Tools_Object* pxObject );
		virtual void OnFinish() { delete this; }

	private:
		Tools_RagdollJoint_Object*	m_pxJoint;
};

#endif //_Tools_RagdollJoint_Object_H_
