/**
*
* \file Tools_RenderModules.h
*
* Created by Kevin Lea - 04/04/2002 12:03:41
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_RENDERMODULES_H__
#define __TOOLS_RENDERMODULES_H__

#include "Mesh/Asura_Mesh.h" //for TOOLS_NUM_MATERIALRESPONSE_PER_POLY


class Asura_Mesh;
class Asura_Vector_3;


#if !defined ( ASURA_TARGET_SM3 )

enum ToolsRenderModulePass
{
	TOOLS_RENDERMODULEPASS_NONE = -1,
	
	TOOLS_RENDERMODULEPASS_OPAQUE = 0,
	TOOLS_RENDERMODULEPASS_TRANSPARENT,
	TOOLS_RENDERMODULEPASS_ALPHADECAL,
	TOOLS_RENDERMODULEPASS_ADDALPHA,

	TOOLS_NUM_RENDERMODULEPASSES,
};



class Tools_RenderModule_Handler
{
public :
		// (SB - 28/11/06) Hooks to allow adding more rendering passes
		typedef void InitialiseHook(void);
		typedef void ShutdownHook(void);
		typedef void RenderHook(int iPass,const Asura_Mesh& xMesh, const Asura_Vector_3& xPosition, bool bDrawBrightGreen, int iModuleIndex, bool bNeedsMeshUpdating);

		// (SB - 11/12/06) Hook to override render state
#if !defined(ASURA_TARGET_SM3)
		typedef void RenderStateOverride(void);
#endif

		Tools_RenderModule_Handler();
		~Tools_RenderModule_Handler();

		void Initialise(void);
		void Shutdown(void);

		void RenderModule(int iPass,const Asura_Mesh& mesh, const Asura_Vector_3& position, bool drawBrightGreen = false,int moduleIndex = -1);
		void UpdateModuleLighting(int meshIndex);
		void UpdateAllModuleLighting();
		void UpdateModuleMesh(int meshIndex);

		// (SB - 28/11/06) Setup hooks for additional render passes
		static void SetHooks(InitialiseHook* const pfnInitialiseHook, ShutdownHook* const pfnShutdownHook, RenderHook* const pfnRenderHook);
#if !defined(ASURA_TARGET_SM3)
		// (SB - 11/12/06) No external access to Tools_RenderModule so we need to forward the call to it from here
		static void SetRenderStateOverride(RenderStateOverride* const pfnRenderStateOverride = NULL);
#endif
	
	private:

		static InitialiseHook* m_pfnInitialiseHook;
		static ShutdownHook* m_pfnShutdownHook;

		// (SB - 11/12/06) Hook to override render state
		static RenderHook* m_pfnRenderHook;

		int m_iNumberOfRenderModules;
		class Tools_RenderModule* m_pxRenderModules;

};

class Tools_RenderModule_System
{
	public:
		static void Initialise(void);
		static void Shutdown(void);

#if defined(ASURA_USE_STREAMING)
		static void InitialiseCurrentZone(){s_pxCurrentHandler->Initialise();}
#endif

		//gets the number of times RenderModule should be called
		static int GetNumPasses(){return TOOLS_NUM_RENDERMODULEPASSES;}

		static void RenderModule(int iPass,const Asura_Mesh& mesh, const Asura_Vector_3& position, bool drawBrightGreen = false,int moduleIndex = -1)
		{
			GetCurrentHandler().RenderModule(iPass,mesh,position,drawBrightGreen,moduleIndex);
		}

		static void UpdateModuleLighting(int meshIndex) {GetCurrentHandler().UpdateModuleLighting(meshIndex);}
		static void UpdateAllModuleLighting() {GetCurrentHandler().UpdateAllModuleLighting();}
		static void UpdateModuleMesh(int meshIndex) {GetCurrentHandler().UpdateModuleMesh(meshIndex); }

		
		static void SetupShaderConstantsForFog();

#if defined(ASURA_USE_STREAMING)
		// Handler functions...
		static Tools_RenderModule_Handler* CreateHandler(const Asura_Hash_ID uHash);
		static bool DestroyHandler(const Asura_Hash_ID uHash);

		static void CreateAndSetHandler(const Asura_Hash_ID uHash) { s_pxCurrentHandler = CreateHandler(uHash); }
		static void SetCurrentHandler(Tools_RenderModule_Handler* pxHandler);
		static void SetCurrentHandler(const Asura_Hash_ID uHash) { s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
		static Tools_RenderModule_Handler& GetCurrentHandler(void) { return(*s_pxCurrentHandler); }
		static Tools_RenderModule_Handler* GetHandler (const Asura_Hash_ID uHash) { return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
	// Private functions...
		static Tools_RenderModule_Handler& GetCurrentHandler(void) { return(s_xHandler); }
#endif	// defined(ASURA_USE_STREAMING)

	private:

#if defined(ASURA_USE_STREAMING)
		static void InitialiseCallback(Asura_Hash_ID uHash,Tools_RenderModule_Handler* pxHandler);
		static void ShutdownCallback(Asura_Hash_ID,Tools_RenderModule_Handler* pxHandler);
#endif


#if defined(ASURA_USE_STREAMING)
		static	Tools_RenderModule_Handler*							s_pxCurrentHandler;		// Currently active handler.
		static	Asura_TemplateHashTree<Tools_RenderModule_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
		static	Tools_RenderModule_Handler							s_xHandler;				// Single static-instance of a zone.
#endif


};


#else // !defined ( ASURA_TARGET_SM3 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** New Renderer **
**--------------*/

class Tools_RenderModule_System
{
	public:
		static void Initialise(void);
		static void Reset( void );
		static void Shutdown(void);

		//Updates the render data if needed , both locally and in Asura_Direct3D_EnvironmentRenderr
		static void UpdateRenderData();

		static bool PrepareForRendering( void );

		// Render the entire environment, using the visibility system etc.
		// This will actually render the environment on a per-strip, sorted-by-material, basis.
		static void RenderEnvironment( void );

		static const class Tools_RenderModule& GetRenderModule( u_int uModuleIndex );

		static void RenderModule(int iPass,const Asura_Mesh& mesh, const Asura_Vector_3& position, bool drawBrightGreen = false,int moduleIndex = -1);
		static void UpdateModuleLighting(int meshIndex);
		static void UpdateAllModuleLighting();
		static void UpdateModuleMesh(int meshIndex);

		
		static void SetupShaderConstantsForFog();
	private:

};

#include "Rendering/Asura_Direct3D.h"

class Tools_RenderModule_Polygon_Packet
{
	public:
		u_int				m_uStartVertex;
		u_int				m_uEndVertex;
		u_int				m_uStartIndex;
		u_int				m_uNumberOfIndices;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM && !defined ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY
		Asura_Hash_ID		m_auMaterialResponseHashID[TOOLS_NUM_MATERIALRESPONSE_PER_POLY];
#else
		int					m_iMaterialID;
#endif

		Asura_Bounding_Box	m_xBB;
};


class Tools_RenderModule
{
	public:

		Tools_RenderModule();

		~Tools_RenderModule();
	
		void Reset(void);
		
		void Initialise( const Asura_Mesh& xMesh, const Asura_Vector_3& xPosition, int iModuleIndex );
	
		void Render( void );

//		void RenderPacket( u_int uPacket );
	
		void UpdateLighting( const Asura_Mesh& xMesh );

		inline u_int GetNumberOfPackets( void ) const
		{
			return m_uNumberOfPackets;
		}

		inline const Tools_RenderModule_Polygon_Packet* GetPacketList( void ) const
		{
			return m_pxPacketList;
		}

		//Indicates whether the module was hidden when this was generated
		bool IsHidden() const {return m_bHidden;}
		

		inline Direct3D_VertexBuffer*	GetVertexBuffer				(void) const	{ return (m_pxVertexBuffer); }

		inline Direct3D_IndexBuffer*	GetIndexBuffer				(void) const	{ return (m_pxIndexBuffer); }
		inline uint16*					GetIndexBufferData			(void) const	{ return (m_pusIndexBufferData); }
		

		// (AAM - 18.08.2006) Exposed for botanicals on next-gen
		static void SetupShaderConstants();
		static float			s_afShaderConstants[11][4];

		bool 					m_bIsValid;
		bool 					m_bNeedsLightingUpdating;
		bool 					m_bNeedsMeshUpdating;

	private:

		void CalculateTangentSpaceVectors(const class Asura_Mesh* pxMesh, u_int uPoly, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal);


		int									m_iModuleIndex;
		u_int								m_uNumberOfPackets;
		Tools_RenderModule_Polygon_Packet*	m_pxPacketList;

		u_int								m_uNumberOfVertices;

		Direct3D_VertexBuffer*				m_pxVertexBuffer;

		Direct3D_IndexBuffer*				m_pxIndexBuffer;
		uint16*								m_pusIndexBufferData;

		bool								m_bHidden;

	
};

#endif //defined ( ASURA_TARGET_SM3 )



#endif//__TOOLS_RENDERMODULES_H__
