//
// Tools_DSPADPCM_Conversion.cpp
//
// Created by KOR - 08/06/2003 17:09:27
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Sound/Conversion/Tools_DSPADPCM_Conversion.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "Debugging/Asura_Assert.h"

// Tools
#include "Load&Save/Tools_Sound_Export.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#ifndef ASURA_MAIN
#include "FakeTools/ResourcePacker_FakeTools.h"
#endif

// For dll handling
#include<windows.h>
#include <float.h>

typedef unsigned int (*lpFunc1)(unsigned int);
typedef unsigned int (*lpFunc2)(void);
typedef void (*lpFunc3)(short*, unsigned char*, ADPCMINFO*, unsigned int);
typedef void (*lpFunc4)(unsigned char*, short*, ADPCMINFO*, unsigned int);
typedef void (*lpFunc5)(unsigned char*, ADPCMINFO*, unsigned int);

#define READ_WAV_FILE(buffer,size)\
ReadFile(inputFile,(long*)buffer,(size),&bytes_read,0);\
if(bytes_read!=(unsigned)(size))\
	goto EndWavFileRead;

#define WRITE_DSP_FILE(buffer,size)\
WriteFile(outputFile,(long*)buffer,(size),&bytes_read,0);\
if(bytes_read!=(unsigned)(size))\
	goto EndDSPWrite;

typedef struct pwavchunkheader
{
	char chunkName[4];
	int chunkLength;	
} PWAVCHUNKHEADER;

typedef struct pwavriffheader
{
	char type[4];
} PWAVRIFFHEADER;

static bool CleanUp(HINSTANCE hDLL)
{
	if (hDLL)
	{
		FreeLibrary(hDLL);
	}
	// We have usually failed - if not, the returned value is ignored
	return false;
}


// Returns true for success, false for failure
/*--------------------------------------------------------------------------*/
bool ConvertDSPFile(const char* szInputFileName, const char* szOutputFileName, const struct ToolsSoundConversionData* pxConvertData, u_int* puFailureCode /* = 0 */)
{
	// Set to failed to load DLL until we know otherwise
	if(puFailureCode) *puFailureCode = Tools_ConvertDSPFile_FailedToLoadDLL;
	HINSTANCE hDll = LoadLibrary("dsptool.dll");
	
	if (hDll)
	{
		lpFunc1 pfGetBytesForAdpcmBuffer	= (lpFunc1)GetProcAddress(hDll, "getBytesForAdpcmBuffer");
		lpFunc1 pfGetBytesForAdpcmSamples	= (lpFunc1)GetProcAddress(hDll, "getBytesForAdpcmSamples");
		lpFunc1 pfGetNibbleAddress			= (lpFunc1)GetProcAddress(hDll, "getNibbleAddress");
		lpFunc3 pfEncode					= (lpFunc3)GetProcAddress(hDll, "encode");
		lpFunc5 pfGetLoopContext			= (lpFunc5)GetProcAddress(hDll, "getLoopContext");

		if(!(pfGetBytesForAdpcmBuffer && pfGetBytesForAdpcmSamples && pfGetNibbleAddress && pfEncode && pfGetLoopContext))
		{
			return CleanUp(hDll);
		}
		// Loaded DCPADPCM.DLL. Reset error code.
		if(puFailureCode) *puFailureCode = Tools_ConvertDSPFile_Success;

		// Majority of the following stolen from vag create...

		bool okay = false;
		unsigned char* readBuffer = 0;
		unsigned long bytes_read;
		
		//open the the wav file
		HANDLE inputFile = CreateFile(szInputFileName,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_FLAG_RANDOM_ACCESS,0);
		
		if(inputFile == INVALID_HANDLE_VALUE)
		{
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Failed to  open %s",szInputFileName);
			if (puFailureCode) *puFailureCode = Tools_ConvertDSPFile_FailedToOpenWav;
			return false;
		}

		//read the initial headers
		PWAVCHUNKHEADER myChunkHeader;
		READ_WAV_FILE(&myChunkHeader,sizeof(myChunkHeader));

		if((myChunkHeader.chunkName[0]!='R')||(myChunkHeader.chunkName[1]!='I')||
		   (myChunkHeader.chunkName[2]!='F')||(myChunkHeader.chunkName[3]!='F'))
		{
			goto EndWavFileRead;
		}

		PWAVRIFFHEADER myRiffHeader;
		PCMWAVEFORMAT wavFormat;
		READ_WAV_FILE(&myRiffHeader,sizeof(myRiffHeader));

		if(myRiffHeader.type[0]!='W'||myRiffHeader.type[1]!='A'||myRiffHeader.type[2]!='V'||myRiffHeader.type[3]!='E')
		{
			goto EndWavFileRead;
		}

		//find the format chunk
		do
		{
			READ_WAV_FILE(&myChunkHeader,sizeof(myChunkHeader));
			if((myChunkHeader.chunkName[0]=='f')&&(myChunkHeader.chunkName[1]=='m')&&
	   			(myChunkHeader.chunkName[2]=='t')&&(myChunkHeader.chunkName[3]==' '))
			{
				break;
			}

			SetFilePointer(inputFile,myChunkHeader.chunkLength,0,FILE_CURRENT);

		}
		while(true);

		//read the wav format chunk
		if(myChunkHeader.chunkLength==16)
		{
			/* a standard PCM wave format chunk */
			READ_WAV_FILE(&wavFormat,sizeof(wavFormat));
		}
		else if(myChunkHeader.chunkLength>=18)
		{
			/* an extended PCM wave format chunk */
			WAVEFORMATEX temp;
			READ_WAV_FILE(&temp,sizeof(temp));
			
			wavFormat = *(PCMWAVEFORMAT*)&temp;

			//skip the rest of the chunk
			if(myChunkHeader.chunkLength>18)
			{
				SetFilePointer(inputFile,myChunkHeader.chunkLength-18,0,FILE_CURRENT);
			}

		}
		else
		{
			goto EndWavFileRead;
		}

		//file shouldn't be stereo
		if(wavFormat.wf.nChannels>1)
		{
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"%s has more than one channel",szInputFileName);
			goto EndWavFileRead;
		}
		if(wavFormat.wBitsPerSample % 8)
		{
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"%s doesn't have a multiple of 8 bits per sample",szInputFileName);
			goto EndWavFileRead;
		}
		if(wavFormat.wBitsPerSample==8)
		{
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_WARNING,"%s is 8 bit (16 bit is recommended)",szInputFileName);
		}

		//now skip until we reach the data chunk
		do
		{
			READ_WAV_FILE(&myChunkHeader,sizeof(myChunkHeader));
			if((myChunkHeader.chunkName[0]=='d')&&(myChunkHeader.chunkName[1]=='a')&&
	   			(myChunkHeader.chunkName[2]=='t')&&(myChunkHeader.chunkName[3]=='a'))
			{
				break;
			}

			SetFilePointer(inputFile,myChunkHeader.chunkLength,0,FILE_CURRENT);

		}
		while(true);

		readBuffer = new unsigned char[myChunkHeader.chunkLength];

		//read in the data chunk
		READ_WAV_FILE(readBuffer,myChunkHeader.chunkLength);
		
		okay = true;

	EndWavFileRead:

		//finished with the file now
		if(inputFile != INVALID_HANDLE_VALUE) CloseHandle(inputFile);
		if(!okay)
		{
			if(readBuffer) delete [] readBuffer;
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Error reading %s",szInputFileName);
			if (puFailureCode) *puFailureCode = Tools_ConvertDSPFile_FailedToReadWav;
			return false;
		}

		//how many samples read in?
		u_int inputSamples = myChunkHeader.chunkLength/wavFormat.wf.nBlockAlign;

#ifndef DISABLE_R8BRAIN_DLL
		// -- SK - Do resampling here --
		R8BInputFormat xInFormat;

		xInFormat.IEEE = 0;
		xInFormat.pBuffer = readBuffer;
		xInFormat.iRate = wavFormat.wf.nSamplesPerSec;
		xInFormat.iChannels = wavFormat.wf.nChannels;
		xInFormat.iBitDepth = wavFormat.wBitsPerSample;
		xInFormat.iLength = inputSamples;//myChunkHeader.chunkLength/wavFormat.wf.nBlockAlign;
		int iNewSize = 0;
		if( xInFormat.iRate > pxConvertData->m_iMaxFrequency || wavFormat.wBitsPerSample>16)
		{
			unsigned char* pNewData = (unsigned char*)Tools_Sound_Export::ResampleSound_R8Brain( &xInFormat, pxConvertData->m_iMaxFrequency, 16, &iNewSize);
			if( (pNewData != NULL) && (iNewSize > 0))
			{
				inputSamples = iNewSize;

				wavFormat.wf.nSamplesPerSec = pxConvertData->m_iMaxFrequency;
				wavFormat.wBitsPerSample = 16;

				wavFormat.wf.nAvgBytesPerSec = wavFormat.wf.nSamplesPerSec * (wavFormat.wBitsPerSample/8);

				delete []readBuffer;

				readBuffer = pNewData;
			}
		}

#endif // DISABLE_R8BRAIN_DLL

		//determine the number of samples wanted
		u_int outputSamples = inputSamples;
		
		int DSPFrequency = wavFormat.wf.nSamplesPerSec;
		if(wavFormat.wf.nSamplesPerSec > (unsigned)pxConvertData->m_iMaxFrequency)
		{
			outputSamples = (int)(outputSamples * ((float)pxConvertData->m_iMaxFrequency/wavFormat.wf.nSamplesPerSec));
			DSPFrequency = pxConvertData->m_iMaxFrequency;
		}

		//allocate a buffer for the converted wav file
		short* wavBuffer = new short[outputSamples];
		
		//fill in structures for resampling
		Tools_SoundResampleData xInput,xOutput;
		xInput.m_uNumberOfChannels = 1;
		xInput.m_uBitsPerSample = wavFormat.wBitsPerSample;
		xInput.m_uNumberOfSamples = inputSamples;

		xOutput.m_uNumberOfChannels = 1;
		xOutput.m_uBitsPerSample = 16;
		xOutput.m_uNumberOfSamples = outputSamples;

		//get the start for the input buffer (skipping samples if needed)
		unsigned char* pucInputStart=readBuffer;

		Tools_Sound_Export::Resample_Sound(xInput,xOutput,pucInputStart,wavBuffer);

		//finished with the input buffer
		delete [] readBuffer;

		okay = false;

		//open the DSP file for writing
		Tools_FileSystem::EnsureDirectoryExists(szOutputFileName);
		HANDLE outputFile = CreateFile(szOutputFileName,GENERIC_READ+GENERIC_WRITE,0,0,CREATE_ALWAYS,FILE_FLAG_RANDOM_ACCESS,0);
		
		if(outputFile == INVALID_HANDLE_VALUE)
		{
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Failed to open %s for writing",szOutputFileName);
			delete [] readBuffer;
			//delete [] wavBuffer;
			if (puFailureCode) *puFailureCode = Tools_ConvertDSPFile_FailedToCreateDSP;
			return false;
		}

		unsigned char *adpcm = (unsigned char*)Asura_Allocate(pfGetBytesForAdpcmBuffer(outputSamples));
		if (adpcm)
		{
			ADPCMINFO adpcminfo;
			// ok.. lets encode it!
			//pfEncode((short*)readBuffer, adpcm, &adpcminfo, outputSamples);
			pfEncode(wavBuffer, adpcm, &adpcminfo, outputSamples);
			// get ADPCM loop context if sample is looped
			if (pxConvertData->m_bLoop)
			{
				//pfGetLoopContext(adpcm, &adpcminfo, 0);
			}
			// see how many bytes to store the encoded data stream
			unsigned int nBytesToStore = pfGetBytesForAdpcmSamples(outputSamples);

			ADPCMHeader xHeader;
			xHeader.num_samples = inputSamples;
			ASURA_CHUNK_VERIFY(xHeader.num_samples, unsigned int);
			xHeader.num_adpcm_nibbles = pfGetNibbleAddress(outputSamples) - pfGetNibbleAddress(0);
			ASURA_CHUNK_VERIFY(xHeader.num_adpcm_nibbles, unsigned int);
			xHeader.sample_rate = DSPFrequency;
			ASURA_CHUNK_VERIFY(xHeader.sample_rate, unsigned int);
			//xHeader.loop_flag = pxConvertData->m_bLoop;
			xHeader.loop_flag = 0;
			ASURA_CHUNK_VERIFY(xHeader.loop_flag, unsigned short);
			xHeader.format = 0x0000;
			ASURA_CHUNK_VERIFY(xHeader.format, unsigned short);
			xHeader.sa = pfGetNibbleAddress(0);
			ASURA_CHUNK_VERIFY(xHeader.sa, unsigned int);
			xHeader.ea = pfGetNibbleAddress(outputSamples);
			ASURA_CHUNK_VERIFY(xHeader.ea, unsigned int);
			xHeader.ca = 0;
			ASURA_CHUNK_VERIFY(xHeader.ca, unsigned int);

			ASURA_CHUNK_ARRAY_VERIFY(adpcminfo, unsigned short, 23);
			memcpy(&xHeader.coef, &adpcminfo, 23 * sizeof(short));

			WRITE_DSP_FILE(&xHeader, sizeof(ADPCMHeader));

			WRITE_DSP_FILE((void*)adpcm, nBytesToStore);

			// dont need the ADPCM buffer anymore
			Asura_Deallocate(adpcm);
		}
		okay = true;
		
	EndDSPWrite :
		if(outputFile) CloseHandle(outputFile);
		if(wavBuffer) delete [] wavBuffer;
		//delete [] readBuffer;
		if(!okay)
		{
			DeleteFile(szOutputFileName);//kill the partially written file
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Error writing %s",szOutputFileName);
			if (puFailureCode) *puFailureCode = Tools_ConvertDSPFile_FailedToWriteDSP;
			return false;
		}

		CleanUp(hDll);
		return true;
	}
	return false;
}
