//////////////////////////////////////////////////////////////////////////////
//
// Tools_xWMA_Conversion.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Sound/Conversion/Tools_xWMA_Conversion.h"

// Asura Platform-independent
#include "Core/Asura_String.h"
#include "Debugging/Asura_Assert.h"

// Tools
#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Misc
#include <windows.h>

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Tools_ConvertxWMAFile()
//
// Converts a wave file to xWMA. Does nothing if the file already exists and
// doesn't need reconverting
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_ConvertxWMAFile( const char* const szInputFileName, const char* const szOutputFileName,
	const struct ToolsSoundConversionData* const pxConvertData, u_int* const puFailureCode /*= 0*/ )
{
	// Parameter check

	if( !szInputFileName || !szOutputFileName )
	{
		Asura_Assert( false, "NULL parameter passed to Tools_ConvertxWMAFile()." );

		if( puFailureCode )
		{
			*puFailureCode = Tools_ConvertxWMAFile_InvalidParameter;
		}

		return false;
	}

	// Execute the xWMA encoder tool

	Asura_String xCommand( "xWMAEncode.exe \"" );
	xCommand += Asura_String( szInputFileName );
	xCommand += "\" \"";
	xCommand += Asura_String( szOutputFileName );
	xCommand += "\"";

	PROCESS_INFORMATION xProcessInfo;
	STARTUPINFO xStartupInfo = {0};
	xStartupInfo.cb = sizeof( STARTUPINFO );

	if( !CreateProcess( NULL, xCommand.GetString(), NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL,
		&xStartupInfo, &xProcessInfo ) )
	{
		#if defined ASURA_MAIN
			Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR,
				"Failed to encode sound. Check that xWMAEncode.exe is present." );
		#else
			Asura_Assert( false, "Failed to encode sound. Check that xWMAEncode.exe is present." );
		#endif

		if( puFailureCode )
		{
			*puFailureCode = Tools_ConvertxWMAFile_FailedToLoadExe;
		}

		return false;
	}

	WaitForSingleObject( xProcessInfo.hProcess, INFINITE );

	// Check the exit code for failure

	DWORD uExitCode = 0;
	if( !GetExitCodeProcess( xProcessInfo.hProcess, &uExitCode ) )
	{
		Asura_Assert( false, "Unable to query exit code for xWMAEncode process." );
	}

	CloseHandle( xProcessInfo.hThread );
	CloseHandle( xProcessInfo.hProcess );

	if( uExitCode != 0 )
	{
		const char* const szMsg = "xWMAEncode tool reported an error. Exit code %u.";

		#if defined ASURA_MAIN
			Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, szMsg, uExitCode );
		#else
			Asura_Assertf( false, (szMsg, uExitCode) );
		#endif
	}

	if( puFailureCode )
	{
		*puFailureCode = (uExitCode == 0)
					   ? Tools_ConvertxWMAFile_Success
					   : Tools_ConvertxWMAFile_FailedToConvertxWMA;
	}

	return true;
}