// Tools_Audio_Monitor.cpp
// Thomas Young - 30.03.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AUDIO_SYSTEM

#include "Core/Asura_Timers.h"				// Asura
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"
#include "Sound/Asura_XAudio2.h"

#include "Load&Save/Tools_ModeDataFile.h"	// Tools
#include "Sound/Tools_Audio_Monitor.h"
#include "Tools_Overlays.h"

#include <AudioDefs.h>

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uSIGNAL_SUBWINDOW_BUFFER_SIZE	= ( sizeof( float ) * 480 * 8 );

static const u_int uSIGNAL_SUBWINDOW_BACKGROUND		= 0xFF000000;
static const u_int uSIGNAL_SUBWINDOW_ZERO_LINE		= 0x66FFFFFF;
static const u_int uSIGNAL_SUBWINDOW_05_LINE		= 0x33FFFFFF;
static const u_int uSIGNAL_SUBWINDOW_CLIPWARN		= 0x00FF0000;
static const u_int uSIGNAL_SUBWINDOW_LEGEND_BACK	= 0x77FFFFFF;
static const u_int uSIGNAL_SUBWINDOW_LEGEND_BORDER	= 0xA0FFFFFF;
static const float fSIGNAL_SUBWINDOW_LEGEND_WIDTH	= 75.0f;

static const u_int auSIGNAL_SUBWINDOW_OSC[] =		{ 0xDDFFCBCB,
				  									  0xDDCBE4FF,
				  									  0xDDFCFFB5,
				  									  0xDDB5FFC4,
													  0xDDF2B5FF,
													  0xDDB5FBFF,
													  0xDDFFE2B5 };

// S T A T I C   M E M B E R   V A R I A B L E S /////////////////////////////

Tools_Audio_Monitor*	Tools_Audio_Monitor::s_pxThis		= 0;
Asura_Vector_2			Tools_Audio_Monitor::s_xOverlayPos;

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

class Tools_Realtime_Signal_Dialog : public Asura_Subwindow_Dialog
{
	public:

		Tools_Realtime_Signal_Dialog  ( const Asura_Vector_2& xOffset, const float fWidth, const float fHeight );
		virtual ~Tools_Realtime_Signal_Dialog ();

		void	Render( const Asura_Vector_2& xOrigin );

	private:

		const char* GetTextForSpeakerChannel( const u_int uChan );

		char*	m_pcAudioBuffer;
		u_int	m_uChannels;
		u_int	m_uFrames;
		float	m_fClipWarning[2];
};

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

// ___________________________________ Tools_Audio_Monitor ______________________________________

Tools_Audio_Monitor::Tools_Audio_Monitor( Asura_Vector_2* const pxPos ) : Asura_Overlay( "Audio Monitor", false, pxPos )
{
	s_pxThis = this;

	m_uID = TOOLS_OVERLAY_AUDIO_MONITOR;

	Asura_Vector_2 xOffset(1.0f, 2.5f);
	Asura_Dialog* pxDialog;

	//Create the subwindow control for containing all the mixers
	float fSubwindowWidth	= Asura_Maths::Min( 80.0f, ( Asura_Viewport::GetWidth()  - 100 ) / ASURA_TEXT_OUTPUT_FONT_WIDTH  );
	float fSubwindowHeight	= Asura_Maths::Min( 20.0f, ( Asura_Viewport::GetHeight() - 100 ) / ASURA_TEXT_OUTPUT_FONT_HEIGHT );

	pxDialog = new Tools_Realtime_Signal_Dialog( xOffset, fSubwindowWidth, fSubwindowHeight );

	AddDialog( pxDialog );
	xOffset.y += pxDialog->m_xMaxCoord.y;

	SetAlwaysFullBright( true );

	//add control for resizing the subwindow dialog
	pxDialog = new Asura_Resize_Dialog( pxDialog, 0 );
	AddDialog(pxDialog);
}


Tools_Audio_Monitor::~Tools_Audio_Monitor ()
{
	Tools_Audio_Monitor::TransferModeData ( TOOLSMODEDATAFILE_SAVE );

	s_pxThis = 0;
}


void Tools_Audio_Monitor::CreateOverlay	( )
{
	if( s_pxThis )
	{
		return;
	}

	TransferModeData( TOOLSMODEDATAFILE_LOAD );

	s_pxThis = new Tools_Audio_Monitor( &s_xOverlayPos );

	Asura_Assert( s_pxThis, "Failed to allocate DSP overlay." );
	if( !s_pxThis ) 
	{
		return;
	}

	Asura_Overlay_System::SubmitNewOverlay( s_pxThis );
}


void Tools_Audio_Monitor::DestroyOverlay ( )
{
	if( !s_pxThis )
	{
		return;
	}

	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_AUDIO_MONITOR );
}


void Tools_Audio_Monitor::ToggleOverlay	( )
{
	if( IsOverlayOpen() )
	{
		DestroyOverlay();
	}
	else
	{
		CreateOverlay();
	}
}


bool Tools_Audio_Monitor::IsOverlayOpen ( )
{
	return ( s_pxThis != 0 );
}

void Tools_Audio_Monitor::TransferModeData ( const int iMode )
{
	//open the file
	Tools_ModeDataFile xFile( "AudioMonitor.dat", iMode );

	if( false == xFile.GetError() )
	{
		xFile.Transfer( s_xOverlayPos );
	}
}

// ___________________________________ Tools_Realtime_Signal_Dialog ______________________________________

Tools_Realtime_Signal_Dialog::Tools_Realtime_Signal_Dialog  ( const Asura_Vector_2& xOffset, const float fWidth, const float fHeight )
															: Asura_Subwindow_Dialog ( xOffset, fWidth, fHeight, false )
{
	m_pcAudioBuffer = reinterpret_cast< char* >( Asura_Allocate( uSIGNAL_SUBWINDOW_BUFFER_SIZE ) );
	Asura_Mem::Set( m_pcAudioBuffer, 0, uSIGNAL_SUBWINDOW_BUFFER_SIZE );
	m_uChannels = 0;
	m_uFrames = 0;
	m_fClipWarning[0] = 0.0f;
	m_fClipWarning[1] = 0.0f;
}


Tools_Realtime_Signal_Dialog::~Tools_Realtime_Signal_Dialog ()
{
	Asura_Deallocate( m_pcAudioBuffer );
}


void Tools_Realtime_Signal_Dialog::Render( const Asura_Vector_2& xOrigin )
{
	Asura_Subwindow_Dialog::Render( xOrigin );

	Asura_Vector_2 xActualOrigin = xOrigin + m_xOffset;

	xActualOrigin.x *= ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xActualOrigin.y *= ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	const float fWidth = GetWidth() * ASURA_TEXT_OUTPUT_FONT_WIDTH;
	const float fHeight = GetHeight() * ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	// Background
	Asura_Render::Overlay.Colour			= uSIGNAL_SUBWINDOW_BACKGROUND;
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY ( xActualOrigin.x, xActualOrigin.y, fWidth, fHeight );
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	// Zero line, 0.5 lines
	Asura_Render::Render2DLine( xActualOrigin.x, xActualOrigin.y + fHeight * 0.5f, xActualOrigin.x + fWidth, xActualOrigin.y + fHeight * 0.5f, uSIGNAL_SUBWINDOW_ZERO_LINE );
	Asura_Render::Render2DLine( xActualOrigin.x, xActualOrigin.y + fHeight * 0.25f, xActualOrigin.x + fWidth, xActualOrigin.y + fHeight * 0.25f, uSIGNAL_SUBWINDOW_05_LINE );
	Asura_Render::Render2DLine( xActualOrigin.x, xActualOrigin.y + fHeight * 0.75f, xActualOrigin.x + fWidth, xActualOrigin.y + fHeight * 0.75f, uSIGNAL_SUBWINDOW_05_LINE );

	// Grab the audio data from XAudio

	char* pcBuffer = 0;
	u_int uFrames = 0;
	u_int uChannels = 0;

	Asura_XAudio2::GetMonitorBuffer( pcBuffer, uChannels, uFrames );

	// If this data is valid, update our rendering buffer
	if( ( uChannels > 0 ) && ( uFrames > 0 ) )
	{
		if( ( sizeof( float ) * uChannels * uFrames ) > uSIGNAL_SUBWINDOW_BUFFER_SIZE )
		{
			m_uFrames = uSIGNAL_SUBWINDOW_BUFFER_SIZE / uChannels;
		}
		else
		{
			m_uFrames = uFrames;
		}
		m_uChannels = uChannels;
		
		Asura_Mem::Copy( m_pcAudioBuffer, pcBuffer, sizeof( float ) * m_uFrames * m_uChannels );

		Asura_XAudio2::ReadyForMoreMonitorData();
	}

	// Now render
	if( ( m_uFrames > 0 ) && ( m_uChannels > 0 ) )
	{
		const float fHalfHeight = fHeight * 0.5f;
		const float fFramesRecip = 1.0f / static_cast< float >( m_uFrames );
		float* pfFloats = reinterpret_cast< float* >( m_pcAudioBuffer );

		for( u_int uFrame = 0; uFrame < m_uFrames - 1; ++uFrame )
		{
			const float fRenderPosCurrentX = xActualOrigin.x + fFramesRecip * static_cast<float>( uFrame ) * fWidth;
			const float fRenderPosNextX = xActualOrigin.x + fFramesRecip * static_cast<float>( uFrame + 1 ) * fWidth;

			for( u_int uChannel = 0; uChannel < m_uChannels; ++uChannel )
			{
				const u_int uCurrentSample = uFrame * m_uChannels + uChannel;
				const u_int uNextSample = ( uFrame + 1 ) * m_uChannels + uChannel;

				Asura_Render::Render2DLine( fRenderPosCurrentX, xActualOrigin.y + fHalfHeight + pfFloats[ uCurrentSample ] * fHalfHeight, fRenderPosNextX, xActualOrigin.y + fHalfHeight + pfFloats[ uNextSample ] * fHalfHeight, auSIGNAL_SUBWINDOW_OSC[ uChannel ] );

				if( pfFloats[ uCurrentSample ] > 0.98f )
				{
					m_fClipWarning[0] = 1.0f;
				}
				else if( pfFloats[ uCurrentSample ] < -0.98f )
				{
					m_fClipWarning[1] = 1.0f;
				}
			}
		}
	}

	// Clip warnings

	for( int iTopBottom = 0; iTopBottom < 2; ++iTopBottom )
	{
		if( m_fClipWarning[iTopBottom] > 0.0f )
		{
			u_int uOpacity = static_cast<u_int>( m_fClipWarning[iTopBottom] * 255.0f ) << 24;

			Asura_Render::Overlay.Colour			= uSIGNAL_SUBWINDOW_CLIPWARN | uOpacity;
			Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
			if( iTopBottom == 0 )
			{
				Asura_Render::Overlay.SetRectXY ( xActualOrigin.x, xActualOrigin.y, fWidth, fHeight * 0.02f );
			}
			else
			{
				Asura_Render::Overlay.SetRectXY ( xActualOrigin.x, xActualOrigin.y + fHeight * 0.98f, fWidth, fHeight * 0.02f );
			}
			
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();

			m_fClipWarning[iTopBottom] -= Asura_Timers::GetFrameTime();
			if( m_fClipWarning[iTopBottom] < 0.0f )
			{
				m_fClipWarning[iTopBottom] = 0.0f;
			}
		}
	}

	// Legend
	if( IsCursorWithinChildRect() )
	{
		float fPosX = ( xActualOrigin.x + fWidth ) - ( fSIGNAL_SUBWINDOW_LEGEND_WIDTH + 5.0f );
		float fPosY = ( xActualOrigin.y + 10.0f );

		const float fLeft	= fPosX - 5.0f;
		const float fRight	= fPosX + fSIGNAL_SUBWINDOW_LEGEND_WIDTH;
		const float fTop	= fPosY - 5.0f;
		const float fBottom = fTop + static_cast<float>( m_uChannels ) * ASURA_TEXT_OUTPUT_FONT_HEIGHT * 1.5f + 5.0f;

		Asura_Render::Overlay.Colour			= uSIGNAL_SUBWINDOW_LEGEND_BACK;
		Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
		Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
		Asura_Render::Overlay.SetRectXY ( fLeft, fTop, fRight - fLeft, fBottom - fTop );
		Asura_Render::Overlay.Render();
		Asura_Render::FlushDisplay();

		Asura_Render::Render2DLine( fLeft, fTop, fLeft, fBottom, uSIGNAL_SUBWINDOW_LEGEND_BORDER );
		Asura_Render::Render2DLine( fLeft, fTop, fRight, fTop, uSIGNAL_SUBWINDOW_LEGEND_BORDER );
		Asura_Render::Render2DLine( fLeft, fBottom, fRight, fBottom, uSIGNAL_SUBWINDOW_LEGEND_BORDER );
		Asura_Render::Render2DLine( fRight, fTop, fRight, fBottom, uSIGNAL_SUBWINDOW_LEGEND_BORDER );

		for( u_int uChan = 0; uChan < m_uChannels; ++uChan )
		{
			Asura_Render::Overlay.Colour			= auSIGNAL_SUBWINDOW_OSC[ uChan ];
			Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.SetRectXY ( fPosX, fPosY, ASURA_TEXT_OUTPUT_FONT_HEIGHT, ASURA_TEXT_OUTPUT_FONT_HEIGHT );
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();

			Asura_Text_Output::Print ( ( fPosX + ASURA_TEXT_OUTPUT_FONT_HEIGHT * 1.5f ) / ASURA_TEXT_OUTPUT_FONT_WIDTH, fPosY / ASURA_TEXT_OUTPUT_FONT_HEIGHT, 0xFF000000, ASURA_TEXT_FORMAT_SMALLFONT, GetTextForSpeakerChannel( uChan ) );

			fPosY += ASURA_TEXT_OUTPUT_FONT_HEIGHT * 1.5f;
		}
	}
}

const char* Tools_Realtime_Signal_Dialog::GetTextForSpeakerChannel( const u_int uChan )
{
	u_int uMask = Asura_XAudio2::GetDeviceChannelMask();

	u_int uFoundChan = 0;
	while( true )
	{
		if( uMask & SPEAKER_FRONT_LEFT				)	{	if( uFoundChan == uChan )	{	return "Front L";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_FRONT_RIGHT				)	{	if( uFoundChan == uChan )	{	return "Front R";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_FRONT_CENTER			)	{	if( uFoundChan == uChan )	{	return "Front C";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_LOW_FREQUENCY			)	{	if( uFoundChan == uChan )	{	return "LFE";		}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_BACK_LEFT				)	{	if( uFoundChan == uChan )	{	return "Rear L";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_BACK_RIGHT				)	{	if( uFoundChan == uChan )	{	return "Rear R";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_FRONT_LEFT_OF_CENTER	)	{	if( uFoundChan == uChan )	{	return "Centre L";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_FRONT_RIGHT_OF_CENTER	)	{	if( uFoundChan == uChan )	{	return "Centre R";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_BACK_CENTER				)	{	if( uFoundChan == uChan )	{	return "Rear C";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_SIDE_LEFT				)	{	if( uFoundChan == uChan )	{	return "Side L";	}	else	{	++uFoundChan;	}	}
		if( uMask & SPEAKER_SIDE_RIGHT				)	{	if( uFoundChan == uChan )	{	return "Side R";	}	else	{	++uFoundChan;	}	}
	}

	return "";
}

#endif	// ASURA_USE_AUDIO_SYSTEM

//eof
