//
// Tools_DialogueEventTab_Event.cpp
//
// Created by Richard Rice - 30/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_DialogueEventTab_Event.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Sound/Asura_Dialogue.h"

#include "Entity/Sound/Asura_ServerEntity_DialogueController.h"
#include "Entity/Sound/Tools_Entity_DialogueController.h"
#include "Entity/Sound/Tools_Entity_DialogueTrigger.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "Sound/Tools_Dialogue_System.h"
#include "Sound/Tools_PickDialogueEvent_Overlay.h"
#include "Sound/Tools_PickDialogueLine_Overlay.h"


static char* g_aszPlaybackTypes[] =
{
	"All Sequential",
	"One By One",
	"Random",
	"CD Player Random",
};

static char* g_aszPlaybackTypeHelp[] =
{
	"One at a time , triggering next one when previous finishes,",
	"Plays next in list each time the event is played.",
	"Pick at random.",
	"Each time the dialogue is triggered, the line chosen is picked at random from the lines that haven't been played yet. When all lines have been played the process is restarted.",
};



//////////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------------**
** Tools_MessageLogAction_EditDialogue **
**-------------------------------------*/
class Tools_MessageLogAction_EditDialogue : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_EditDialogue(const Asura_Dialogue_Event* pxEvent) {m_uID = pxEvent->GetHashID();}
	void Execute() {Tools_DialogueEvent_Overlay::EditEvent(m_uID);}

private:
	Asura_Hash_ID m_uID;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_DialogueEventProperty_Overlay **
**----------------------------------*/

//Stuff for property copying overlay
enum Tools_DialogueEvent_Properties
{
	Tools_DialogueEventProp_Filename,

	Tools_DialogueEventProp_PlaybackType,
	Tools_DialogueEventProp_ReplayDelayTime,
	Tools_DialogueEventProp_PlayProbability,
	Tools_DialogueEventProp_Priority,
	Tools_DialogueEventProp_GlobalEvent,
	Tools_DialogueEventProp_PlayableList,
	Tools_DialogueEventProp_MessageBlocks,

	Tools_DialogueEventProp_NumProp,
};

static const char* g_aszPropertyNames[Tools_DialogueEventProp_NumProp] =
{
	"Filename",
	"Playback Type",
	"Replay Delay Time",
	"Play Probability",
	"Priority",
	"Global Event",
	"Playable List",
	"Message Blocks",
};


//Overlay for selecting properties to paste

class Tools_DialogueEventProperty_Overlay : public ListSelection_Overlay
{
public :
	Tools_DialogueEventProperty_Overlay()
	:ListSelection_Overlay("Select properties to paste")
	{
		//initialise the list
		SetSortList(false);

		for(int i=0;i<Tools_DialogueEventProp_NumProp;i++)
		{
			AddItem(g_aszPropertyNames[i],i);
			SetItemTicked(i,s_bSelected[i]);
		}
	}
	
	virtual bool DoModal()
	{
		if(!ListSelection_Overlay::DoModal()) return false;

		//copy settings to static array
		for(int i=0;i<Tools_DialogueEventProp_NumProp;i++)
		{
			s_bSelected[i] = GetItemTicked(i);
		}
		return true;
	}

	const bool* GetSelectedArray() const {return s_bSelected;}

protected :
	virtual bool IsTickboxList() const {return true;}
	virtual const char* GetListHeadingText() const {return "Properties :";}
	virtual const char* GetTickAllButtonText() const {return "Select All";}
	virtual const char* GetTickNoneButtonText() const {return "Deselect All";}


	static bool s_bSelected[Tools_DialogueEventProp_NumProp];
};

bool Tools_DialogueEventProperty_Overlay::s_bSelected[Tools_DialogueEventProp_NumProp];

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_DialogueEvent **
**---------------------*/

Tools_DialogueEvent::Tools_DialogueEvent()
{
	m_pxPlayables = 0;
	m_uNumMessages = 0;
	m_ppxMessages = 0;
}

Tools_DialogueEvent::~Tools_DialogueEvent()
{
	delete [] m_pxPlayables;

	for(u_int i=0;i<m_uNumMessages;i++)
	{
		delete m_ppxMessages[i];
	}
	delete [] m_ppxMessages;
}

void Tools_DialogueEvent::CopyFromAsuraEvent(const class Asura_Dialogue_Event* pxEvent)
{
	m_iPlaybackType = pxEvent->GetPlaybackType();
	m_fReplayDelayTime = pxEvent->GetReplayDelayTime();
	m_iPlayProbability = pxEvent->GetPlayProbability();
	m_iPriority = pxEvent->GetPriority();
	m_bGlobalEvent = pxEvent->GetIsGlobal();
	m_uInitialMessage = pxEvent->GetInitialMessageID();
	m_uFinalMessage = pxEvent->GetFinalMessageID();

	m_xFilename = pxEvent->GetFilename();

	m_uNumPlayable = pxEvent->GetNumPlayableID();
	delete [] m_pxPlayables;
	m_pxPlayables = m_uNumPlayable ? new Asura_Dialogue_PlayableIDListItem[m_uNumPlayable] : 0;
	for(u_int i=0;i<m_uNumPlayable;i++)
	{
		m_pxPlayables[i] = pxEvent->GetPlayableID(i);
	}

	//delete old messages
	for(u_int i=0;i<m_uNumMessages;i++)
	{
		delete m_ppxMessages[i];
	}
	delete [] m_ppxMessages;

	//copy new ones from event
	m_uNumMessages = pxEvent->GetMessageStoreSize();
	m_ppxMessages = m_uNumMessages ? new Asura_Dialogue_Message*[m_uNumMessages] : 0;
	for(u_int i=0;i<m_uNumMessages;i++)
	{
		m_ppxMessages[i] = new Asura_Dialogue_Message;
		*m_ppxMessages[i] = *pxEvent->GetMessageFromStore(i);
	}

}

void Tools_DialogueEvent::CopyToAsuraEvent(class Asura_Dialogue_Event* pxEvent) const
{
	pxEvent->SetPlaybackType((ASURA_DIALOGUE_PLAYBACK_TYPE) m_iPlaybackType);
	pxEvent->SetReplayDelayTime(m_fReplayDelayTime);
	pxEvent->SetPlayProbability((u_char) m_iPlayProbability);
	pxEvent->SetPriority((char) m_iPriority);
	pxEvent->SetInitialMessageID(m_uInitialMessage);
	pxEvent->SetFinalMessageID(m_uFinalMessage);

	pxEvent->SetFilename(m_xFilename.GetString());

	if(pxEvent->SetPlayableIDList(m_pxPlayables,m_uNumPlayable))
	{
		//Tools_Mode_Sound::InvalidateSoundSize();
		Tools_PickDialogueLine_Overlay::InvalidateEventList();
		Tools_Dialogue_System::InvalidateCharacterMappings();
	}

	pxEvent->SetMessageStore(m_ppxMessages,m_uNumMessages);

	pxEvent->SetIsGlobal(m_bGlobalEvent && !Tools_Dialogue_System::IsLevelSpecific(pxEvent));
}

void Tools_DialogueEvent::CopyMessagesToAsuraEvent(class Asura_Dialogue_Event* pxEvent) const
{
	pxEvent->SetMessageStore(m_ppxMessages,m_uNumMessages);
}


Tools_DialogueEvent& Tools_DialogueEvent::operator = (const Tools_DialogueEvent& xEventToCopy)
{
	if(this != &xEventToCopy) 
	{
		m_iPlaybackType = xEventToCopy.m_iPlaybackType;
		m_fReplayDelayTime = xEventToCopy.m_fReplayDelayTime;
		m_iPlayProbability = xEventToCopy.m_iPlayProbability;
		m_iPriority = xEventToCopy.m_iPriority;
		m_bGlobalEvent = xEventToCopy.m_bGlobalEvent;
		m_uInitialMessage = xEventToCopy.m_uInitialMessage;
		m_uFinalMessage = xEventToCopy.m_uFinalMessage;

		m_xFilename = xEventToCopy.m_xFilename;

		delete [] m_pxPlayables;
		m_uNumPlayable = xEventToCopy.m_uNumPlayable;
		m_pxPlayables = m_uNumPlayable ? new Asura_Dialogue_PlayableIDListItem[m_uNumPlayable] : 0;
		for(u_int i=0;i<m_uNumPlayable;i++)
		{
			m_pxPlayables[i] = xEventToCopy.m_pxPlayables[i];
		}

		for(u_int i=0;i<m_uNumMessages;i++)
		{
			delete m_ppxMessages[i];
		}
		delete [] m_ppxMessages;

		m_uNumMessages = xEventToCopy.m_uNumMessages;
		m_ppxMessages = m_uNumMessages ? new Asura_Dialogue_Message*[m_uNumMessages] : 0;
		for(u_int i=0;i<m_uNumMessages;i++)
		{
			m_ppxMessages[i] = new Asura_Dialogue_Message;
			*m_ppxMessages[i] = *xEventToCopy.m_ppxMessages[i];
		}


		
	}
	return *this;
}

void Tools_DialogueEvent::Copy(const Asura_Dialogue_Event* pxThisEvent,const Tools_DialogueEvent& xEventToCopy,const bool* pbProperties)
{
	if(this == &xEventToCopy) return;

	if(pbProperties[Tools_DialogueEventProp_PlaybackType])
	{
		m_iPlaybackType = xEventToCopy.m_iPlaybackType;
	}

	if(pbProperties[Tools_DialogueEventProp_ReplayDelayTime])
	{
		m_fReplayDelayTime = xEventToCopy.m_fReplayDelayTime;
	}

	if(pbProperties[Tools_DialogueEventProp_PlayProbability])
	{
		m_iPlayProbability = xEventToCopy.m_iPlayProbability;
	}

	if(pbProperties[Tools_DialogueEventProp_Priority])
	{
		m_iPriority = xEventToCopy.m_iPriority;
	}

	if(pbProperties[Tools_DialogueEventProp_GlobalEvent])
	{
		m_bGlobalEvent = xEventToCopy.m_bGlobalEvent;
	}

	if(pbProperties[Tools_DialogueEventProp_Filename])
	{
		//Make sure it is okay to change the file
		if(Tools_Dialogue_System::CanTransferEvent(pxThisEvent,xEventToCopy.m_xFilename.GetString()))
		{
			m_xFilename = xEventToCopy.m_xFilename;
		}
		else
		{
			//report an error
			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Paste Error",new Tools_MessageLogAction_EditDialogue(pxThisEvent),"Failed to transfer '%s' since source and destination aren't both writeable",pxThisEvent->GetName());
		}
	}

	if(pbProperties[Tools_DialogueEventProp_PlayableList])
	{
		//make sure that pasting over this event won't cause it to include itself
		bool bOkay = true;
		for(u_int i=0;i<xEventToCopy.m_uNumPlayable;i++)
		{
			if(xEventToCopy.m_pxPlayables[i].m_xID.m_eType == ASURA_SOUND_PLAYABLE_EVENT)
			{
				if(!Tools_DialogueEventTab_Event::IsOkayToAddEvent(pxThisEvent->GetHashID(),xEventToCopy.m_pxPlayables[i].m_xID.m_uHashID))
				{
					bOkay = false;
				}
			}
		}
		
		if(bOkay)
		{
			delete [] m_pxPlayables;
			m_uNumPlayable = xEventToCopy.m_uNumPlayable;
			m_pxPlayables = m_uNumPlayable ? new Asura_Dialogue_PlayableIDListItem[m_uNumPlayable] : 0;
			for(u_int i=0;i<m_uNumPlayable;i++)
			{
				m_pxPlayables[i] = xEventToCopy.m_pxPlayables[i];
			}
		}
		else
		{
			//report an error
			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Paste Error",new Tools_MessageLogAction_EditDialogue(pxThisEvent),"Couldn't paste event list over '%s' because it would result in self-inclusion",pxThisEvent->GetName());
		}
	}

	if(pbProperties[Tools_DialogueEventProp_MessageBlocks])
	{
		for(u_int i=0;i<m_uNumMessages;i++)
		{
			delete m_ppxMessages[i];
		}
		delete [] m_ppxMessages;

		m_uNumMessages = xEventToCopy.m_uNumMessages;
		m_ppxMessages = m_uNumMessages ? new Asura_Dialogue_Message*[m_uNumMessages] : 0;
		for(u_int i=0;i<m_uNumMessages;i++)
		{
			m_ppxMessages[i] = new Asura_Dialogue_Message;
			*m_ppxMessages[i] = *xEventToCopy.m_ppxMessages[i];
		}
	}

}


//////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_DialogueEventTab_Event **
**------------------------------*/

bool Tools_DialogueEventTab_Event::s_bPasteBufferValid = false;
Tools_DialogueEvent Tools_DialogueEventTab_Event::s_xPasteBuffer;

Tools_DialogueEventTab_Event::Tools_DialogueEventTab_Event(Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uEventHash)
{
	m_uEventHash = uEventHash;
	m_pxEvent = Asura_Dialogue_Storage::GetEventByHash(uEventHash);
	Asura_Assert(m_pxEvent,"");

	m_xEventData.CopyFromAsuraEvent(m_pxEvent);
//	m_bMinChanged = false;
//	m_bMaxChanged = false;
	m_bFileChanged = false;
//	m_bCategoryChanged = false;
	m_bSelectedPlayableChanged = false;
	m_bChanged = false;

	m_bBeingEdited = false;
	m_bMessageEditing = false;

	m_uPlayableListSize = 0;
	m_pszPlayableList = 0;
	m_pszPlayableHelp = 0;
	m_iSelectedPlayable = 0;
	m_pxPlayableListDialog = 0;

	m_uMessageListSize = 0;
	m_pszMessageList = 0;
	m_iSelectedMessage = 0;
	m_pxMessageListDialog = 0;

	m_pxInitialMessageDialog = 0;
	m_pxFinalMessageDialog = 0;
	m_pxPlayableStartMessageDialog = 0;
	m_pxPlayableEndMessageDialog = 0;
	m_pxEditTemplateDialog = 0;

	m_fPlayableDelay = 0.0f;
	m_uPlayableWeight = 1;

	AddOverlayControls(pxTabsDialog);
}

void Tools_DialogueEventTab_Event::PerFrameUpdate()
{
	if(m_bMessageEditing)
	{
		if(!Tools_Object_System::IsMessageEditing())
		{
			//Finished message editing
			Tools_Dialogue_System::PostMessageEditing();
			Tools_DialogueEvent_Overlay::EditEvent(GetHashID());
			m_bMessageEditing = false;

			//force update of messages in event , since message editing is a bit lax on setting stuff as modified.
			m_xEventData.CopyMessagesToAsuraEvent(m_pxEvent);

			//update message list on overlay
			BuildMessageList();

		}
	}

	bool bUpdateTemplateDialog = false;

	if(m_bSelectedPlayableChanged)
	{
		//get data for new selection
		UpdatePlayableDataFromToolsEvent();

		//update template edit label (and enable/disable)
		bUpdateTemplateDialog = true;

		m_bSelectedPlayableChanged = false;
	}

	if(m_bFileChanged)
	{
		//Make sure it is okay to transfer the event
		if(!Tools_Dialogue_System::CanTransferEvent(m_pxEvent,m_xEventData.m_xFilename.GetString()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Can't transfer event since source and destination aren't both writeable");
		
			m_xEventData.m_xFilename = m_pxEvent->GetFilename();
			m_bFileChanged = false;
		}
		else
		{
			//Set modified before updating the asura data so that both the source and destination files are flagged
			SetModified();
			m_bChanged = true;
		}
		m_bFileChanged = false;
	}

	if(m_bChanged)
	{
		UpdatePlayableDataInToolsEvent();

		//update the asura data
		m_xEventData.CopyToAsuraEvent(m_pxEvent);
		SetModified();

		//update template edit label (and enable/disable)
		bUpdateTemplateDialog = true;

		m_bChanged = false;
	}

	if(bUpdateTemplateDialog)
	{
		if(m_pxEditTemplateDialog)
		{
			Asura_Char szEditTemplate[128];
			Asura_PrintfToBfr(szEditTemplate,"Edit template: %s",128,true,GetSelectedPlayableTemplateName());

			m_pxEditTemplateDialog->UpdateLabel(szEditTemplate);
			m_pxEditTemplateDialog->SetEnabled(GetSelectedPlayableTemplate()!=ASURA_HASH_ID_UNSET);
		}

		bUpdateTemplateDialog = false;
	}
}

Tools_DialogueEventTab_Event::~Tools_DialogueEventTab_Event()
{
	if(m_bMessageEditing) AbortMessageEditing();

	DeletePlayableList();
	DeleteMessageList();
}

void Tools_DialogueEventTab_Event::BuildPlayableList()
{
	DeletePlayableList();

	if(!m_xEventData.m_uNumPlayable)
	{
		//no events added yet , so just put in a dummy entry for an empty list
		m_uPlayableListSize = 1;
		m_pszPlayableList = new char*[1];
		m_pszPlayableList[0] = new char[20];
		strcpy(m_pszPlayableList[0],"*Empty List*");

		m_pszPlayableHelp = new char*[1];
		m_pszPlayableHelp[0] = 0;

	}
	else
	{
		//get a list of all the event/line names
		m_uPlayableListSize = m_xEventData.m_uNumPlayable;
		m_pszPlayableList = new char*[m_uPlayableListSize];
		m_pszPlayableHelp = new char*[m_uPlayableListSize];

		for(u_int i=0;i<m_uPlayableListSize;i++)
		{
			m_pszPlayableHelp[i] = 0;
			
			const char* szPlayableName = 0;
			const char* szPrefix = 0;
			if(m_xEventData.m_pxPlayables[i].m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT)
			{
				const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(m_xEventData.m_pxPlayables[i].m_xID.m_uHashID);
				if(pxEvent) szPlayableName = pxEvent->GetName();
				szPrefix = "E: ";
			}
			else
			{
				Asura_Assert(m_xEventData.m_pxPlayables[i].m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_LINE,"");
				const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(m_xEventData.m_pxPlayables[i].m_xID.m_uHashID);
				if(pxLine) szPlayableName = pxLine->GetName();
				szPrefix = "L: ";

				//Get the line text for the help string
				if(pxLine) 
				{
					const Asura_UnicodeChar* wszText = pxLine->GetText();
					if(wszText)
					{
						int iSize = wcslen(wszText)+1;
						m_pszPlayableHelp[i] = new char[iSize];
						Asura_PrintfToBfr(m_pszPlayableHelp[i],"%S",iSize,true,wszText);
					}
				}

			}

			if(szPlayableName)
			{
				m_pszPlayableList[i] = new char[strlen(szPlayableName)+1+strlen(szPrefix)];
				sprintf(m_pszPlayableList[i],"%s%s",szPrefix,szPlayableName);
			}
			else if(!m_xEventData.m_pxPlayables[i].m_xName.IsBlank())
			{
				//event/line is missing , but we have a name for it
				char szText[100];
				Asura_PrintfToBfr(szText,"%s*Missing* %s",sizeof(szText),true,szPrefix,m_xEventData.m_pxPlayables[i].m_xName.GetString());
				m_pszPlayableList[i] = new char[strlen(szText)+1];
				strcpy(m_pszPlayableList[i],szText);

			}
			else
			{
				//event/line is missing , and all we have is a hash value
				m_pszPlayableList[i] = new char[30];
				Asura_PrintfToBfr(m_pszPlayableList[i],"%s*Missing* 0x%x",30,true,szPrefix,m_xEventData.m_pxPlayables[i].m_xID.m_uHashID);
			}

		}
	}

	m_iSelectedPlayable = Asura_Maths::ClampToRange(m_iSelectedPlayable,0,(int)m_uPlayableListSize);

	//update the dialog if it already exists
	if(m_pxPlayableListDialog)
	{
		m_pxPlayableListDialog->ReplaceStrings(m_uPlayableListSize,m_pszPlayableList);
		m_pxPlayableListDialog->SetListHelpStrings(m_pszPlayableHelp);
	}
}

void Tools_DialogueEventTab_Event::DeletePlayableList()
{
	for(u_int i=0;i<m_uPlayableListSize;i++)
	{
		delete [] m_pszPlayableList[i];
		delete [] m_pszPlayableHelp[i];
	}
	delete [] m_pszPlayableList;
	m_pszPlayableList = 0;
	delete [] m_pszPlayableHelp;
	m_pszPlayableHelp = 0;
	m_uPlayableListSize = 0;
}


void Tools_DialogueEventTab_Event::BuildMessageList()
{
	DeleteMessageList();

	char szText[100];

	if(!m_xEventData.m_uNumMessages)
	{
		//no messages added yet , so just put in a dummy entry for an empty list
		m_uMessageListSize = 1;
		m_pszMessageList = new char*[1];
		m_pszMessageList[0] = new char[20];
		strcpy(m_pszMessageList[0],"*Empty List*");
	}
	else
	{
		//get a list of all the message names
		m_uMessageListSize = m_xEventData.m_uNumMessages;
		m_pszMessageList = new char*[m_uMessageListSize];

		for(u_int i=0;i<m_uMessageListSize;i++)
		{
			const char* szName = m_xEventData.m_ppxMessages[i]->m_xName.GetString();
			int iNumMessages = m_xEventData.m_ppxMessages[i]->m_xMessageBlock.GetNumberOfMessagesInBlock();
			Asura_PrintfToBfr(szText,"%s (%d)",sizeof(szText),true,szName,iNumMessages);

			m_pszMessageList[i] = new char[strlen(szText)+1];
			strcpy(m_pszMessageList[i],szText);
		}
	}

	m_iSelectedMessage = Asura_Maths::ClampToRange(m_iSelectedMessage,0,(int)m_uMessageListSize-1);

	//update the dialog if it already exists
	if(m_pxMessageListDialog)
	{
		m_pxMessageListDialog->ReplaceStrings(m_uMessageListSize,m_pszMessageList);
	}
}

void Tools_DialogueEventTab_Event::DeleteMessageList()
{
	for(u_int i=0;i<m_uMessageListSize;i++)
	{
		delete [] m_pszMessageList[i];
	}
	delete [] m_pszMessageList;
	m_pszMessageList = 0;
	m_uMessageListSize = 0;
}




bool Tools_DialogueEventTab_Event::CanAddEvent()
{
	//return !m_xEventData.m_bEventMapping;
	return true;
}

void Tools_DialogueEventTab_Event::AddEvent()
{
	if(!CanAddEvent()) return;

	//use the event picking overlay to get a new event

	Tools_PickDialogueEvent_Overlay::SetFileFilter(m_pxEvent->GetFilename());


	//Setting m_bBeingEdited makes sure that this can't be deleted as a result of too many tabs being opened
	m_bBeingEdited = true;
	Tools_PickDialogueEvent_Overlay::PickEvent("Pick events to add",0,true);
	m_bBeingEdited = false;

	u_int uNumSelected = Tools_PickDialogueEvent_Overlay::GetNumSelected();
	if(!uNumSelected) return;

	//get the events to be added
	List<Asura_Hash_ID> xEventList;
	u_int uNumSkipped = 0;
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uEventHash = Tools_PickDialogueEvent_Overlay::GetSelection(i);
		//make sure that adding these event won't result in the event including itself
		if(!IsOkayToAddEvent(m_uEventHash,uEventHash))
		{
			uNumSkipped++;
			continue;
		}

		xEventList.add_entry(uEventHash);
	}

	if(uNumSkipped)
	{
  		ShowMessageOverlay("Unable to add %d of the selected events , because it would result in an event including itself.",uNumSkipped);
	}

	if(!xEventList.size()) return;

	//add to the list of playables
	Asura_Dialogue_PlayableIDListItem* pxNewPlayables = new Asura_Dialogue_PlayableIDListItem[m_xEventData.m_uNumPlayable+xEventList.size()];
	for(u_int i=0;i<m_xEventData.m_uNumPlayable;i++)
	{
		pxNewPlayables[i] = m_xEventData.m_pxPlayables[i];
	}
	for(int i=0;i<xEventList.size();i++)
	{
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_xID.m_uHashID = xEventList[i];
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_xID.m_eType = ASURA_DIALOGUE_PLAYABLE_EVENT;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_fDelay = 0.0f;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_ucWeight = 1;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_uStartMessageID = 0;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_uEndMessageID = 0;
	}

	delete [] m_xEventData.m_pxPlayables;
	m_xEventData.m_pxPlayables = pxNewPlayables;
	m_xEventData.m_uNumPlayable += (u_int)xEventList.size();

	//rebuild the overlay list , and select the new event
	BuildPlayableList();
	m_iSelectedPlayable = m_xEventData.m_uNumPlayable-1;
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_DialogueEventTab_Event::IsOkayToAddEvent(Asura_Hash_ID uThisEvent,Asura_Hash_ID uEventToAdd)
{
	//make sure that adding this event won't result in the event including itself
	List<Asura_Hash_ID> xEventQueue;
	xEventQueue.add_entry(uEventToAdd);

	for(LIF<Asura_Hash_ID> xEventLIF(&xEventQueue);!xEventLIF.done();xEventLIF.next())
	{
		if(xEventLIF() == uThisEvent)
		{
			return false;
		}

		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(xEventLIF());
		if(!pxEvent) continue;

		for(u_int i=0;i<pxEvent->GetNumPlayableID();i++)
		{
			const Asura_Dialogue_PlayableID& xID = pxEvent->GetPlayableID(i).m_xID;
			if(xID.m_eType != ASURA_DIALOGUE_PLAYABLE_EVENT) continue;

			if(!xEventQueue.contains(xID.m_uHashID))
			{
				xEventQueue.add_entry(xID.m_uHashID);
			}
		}
	}
	return true;
}

bool Tools_DialogueEventTab_Event::CanAddLine()
{
	//return !m_xEventData.m_bEventMapping;
	return true;
}

void Tools_DialogueEventTab_Event::AddLine()
{
	if(!CanAddLine()) return;

	//show the line picking overlay

	m_bBeingEdited = true;
	Tools_PickDialogueLine_Overlay::PickLine("Pick lines to add",0,true);
	m_bBeingEdited = false;


	u_int uNumSelected = Tools_PickDialogueLine_Overlay::GetNumSelected();
	if(!uNumSelected) return;

	//get the lines to be added
	List<Asura_Hash_ID> xLineList;
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uLineHash = Tools_PickDialogueLine_Overlay::GetSelection(i);

		xLineList.add_entry(uLineHash);
	}


	if(!xLineList.size()) return;

	//add to the list of playables
	Asura_Dialogue_PlayableIDListItem* pxNewPlayables = new Asura_Dialogue_PlayableIDListItem[m_xEventData.m_uNumPlayable+xLineList.size()];
	for(u_int i=0;i<m_xEventData.m_uNumPlayable;i++)
	{
		pxNewPlayables[i] = m_xEventData.m_pxPlayables[i];
	}
	for(int i=0;i<xLineList.size();i++)
	{
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_xID.m_uHashID = xLineList[i];
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_xID.m_eType = ASURA_DIALOGUE_PLAYABLE_LINE;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_fDelay = 0.0f;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_ucWeight = 1;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_uStartMessageID = 0;
		pxNewPlayables[m_xEventData.m_uNumPlayable+i].m_uEndMessageID = 0;
	}

	delete [] m_xEventData.m_pxPlayables;
	m_xEventData.m_pxPlayables = pxNewPlayables;
	m_xEventData.m_uNumPlayable += (u_int)xLineList.size();

	//rebuild the overlay list , and select the new line
	BuildPlayableList();
	m_iSelectedPlayable = m_xEventData.m_uNumPlayable-1;
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
	
}

bool Tools_DialogueEventTab_Event::CanReplaceLine()
{
	return m_xEventData.m_uNumPlayable!=0;
}

void Tools_DialogueEventTab_Event::ReplaceLine()
{
	if(!CanReplaceLine()) return;

	Asura_Dialogue_PlayableIDListItem* pxPlayable = &m_xEventData.m_pxPlayables[m_iSelectedPlayable]; 
	
	//Get the ID of the selected line , if it is a line
	Asura_Hash_ID uInitialLineID = 0;
	if(pxPlayable->m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_LINE)
	{
		uInitialLineID = pxPlayable->m_xID.m_uHashID;
	}


	//show the line picking overlay
	m_bBeingEdited = true;
	Asura_Hash_ID uSelected = Tools_PickDialogueLine_Overlay::PickLine("Pick lines to add",uInitialLineID,false);
	m_bBeingEdited = false;


	if(!uSelected || uSelected == uInitialLineID) return;

	//update the playable ID for the selection
	pxPlayable->m_xID.m_eType =	ASURA_DIALOGUE_PLAYABLE_LINE;
	pxPlayable->m_xID.m_uHashID = uSelected;



	//rebuild the overlay list
	BuildPlayableList();
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
	
}


bool Tools_DialogueEventTab_Event::CanRemovePlayable()
{
	return m_xEventData.m_uNumPlayable!=0 /*&& !m_xEventData.m_bEventMapping*/;
}

void Tools_DialogueEventTab_Event::RemovePlayable()
{
	if(!CanRemovePlayable()) return;

	m_xEventData.m_uNumPlayable--;
	for(u_int i=m_iSelectedPlayable;i<m_xEventData.m_uNumPlayable;i++)
	{
		m_xEventData.m_pxPlayables[i] = m_xEventData.m_pxPlayables[i+1];
	}

	//rebuild the overlay list 
	BuildPlayableList();
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_DialogueEventTab_Event::CanMoveUp()
{
	return (m_iSelectedPlayable > 0);
}

void Tools_DialogueEventTab_Event::MoveUp()
{
	if(!CanMoveUp()) return;

	//swap entries in the hash list
	Asura_Dialogue_PlayableIDListItem xTemp = m_xEventData.m_pxPlayables[m_iSelectedPlayable-1];
	m_xEventData.m_pxPlayables[m_iSelectedPlayable-1] = m_xEventData.m_pxPlayables[m_iSelectedPlayable];
	m_xEventData.m_pxPlayables[m_iSelectedPlayable] = xTemp;

	m_iSelectedPlayable--;

	//rebuild the overlay list 
	BuildPlayableList();
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_DialogueEventTab_Event::CanMoveDown()
{
	return m_iSelectedPlayable+1 < (int)m_xEventData.m_uNumPlayable;
}

void Tools_DialogueEventTab_Event::MoveDown()
{
	if(!CanMoveDown()) return;

	//swap entries in the hash list
	Asura_Dialogue_PlayableIDListItem xTemp = m_xEventData.m_pxPlayables[m_iSelectedPlayable+1];
	m_xEventData.m_pxPlayables[m_iSelectedPlayable+1] = m_xEventData.m_pxPlayables[m_iSelectedPlayable];
	m_xEventData.m_pxPlayables[m_iSelectedPlayable] = xTemp;

	m_iSelectedPlayable++;

	//rebuild the overlay list 
	BuildPlayableList();
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_DialogueEventTab_Event::CanSort()
{
	return m_xEventData.m_uNumPlayable >= 2;
}

static int SortPlayableCallback(const void* p1,const void* p2)
{
	Asura_Dialogue_PlayableIDListItem* pxPlayable1 = (Asura_Dialogue_PlayableIDListItem*)p1;
	Asura_Dialogue_PlayableIDListItem* pxPlayable2 = (Asura_Dialogue_PlayableIDListItem*)p2;

	const char* szName1 = 0;
	if(pxPlayable1->m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT)
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(pxPlayable1->m_xID.m_uHashID);
		if(pxEvent) szName1 = pxEvent->GetName();
	}
	else
	{
		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(pxPlayable1->m_xID.m_uHashID);
		if(pxLine) szName1 = pxLine->GetName();
	}

	const char* szName2 = 0;
	if(pxPlayable2->m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT)
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(pxPlayable2->m_xID.m_uHashID);
		if(pxEvent) szName2 = pxEvent->GetName();
	}
	else
	{
		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(pxPlayable2->m_xID.m_uHashID);
		if(pxLine) szName2 = pxLine->GetName();
	}

	if(szName1 && szName2)
	{
		return stricmp(szName1,szName2);
	}
	else if(szName1)
	{
		return -1;
	}
	else if(szName2)
	{
		return 1;
	}
	else
	{
		return 0;
	}

}

void Tools_DialogueEventTab_Event::Sort()
{
	qsort(m_xEventData.m_pxPlayables,m_xEventData.m_uNumPlayable,sizeof(Asura_Dialogue_PlayableIDListItem),SortPlayableCallback);

	//rebuild the overlay list 
	BuildPlayableList();
	UpdatePlayableDataFromToolsEvent();

	//Force the asura event to be updated
	m_bChanged = true;
}


bool Tools_DialogueEventTab_Event::CanEditPlayable()
{
	if(!m_xEventData.m_uNumPlayable) return false;

	if(m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT)
	{
		return Asura_Dialogue_Storage::GetEventByHash(m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_xID.m_uHashID)!=0;
	}
	else
	{
		return false;//lines aren't editable
	}
}

void Tools_DialogueEventTab_Event::EditPlayable()
{
	if(!CanEditPlayable()) return;

	if(m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT)
	{
		Tools_DialogueEvent_Overlay::EditEvent(m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_xID.m_uHashID);
	}
}

void Tools_DialogueEventTab_Event::ViewLines()
{
	// if there's nothing in the list then exit
	if(!m_xEventData.m_uNumPlayable)
	{
		return;
	}

	// create the overlay if it doesn't exist
	Tools_PickDialogueLine_Overlay::CreateOverlay();
	Tools_PickDialogueLine_Overlay* pxOverlay = static_cast<Tools_PickDialogueLine_Overlay*>(Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_PICKDIALOGUELINE));

	if (pxOverlay)
	{
		// init the line list box from our line hash
		pxOverlay->SetInitialSelection(m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_xID.m_uHashID);
	}
}

void Tools_DialogueEventTab_Event::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog)
{
	//create new tab on the tab dialog
	UpdateTabName();
	pxTabsDialog->AddTab(m_xTabName.GetString());

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = pxTabsDialog->GetTopDialogPosition();

	//filename
	pxDialog = new Tools_DialogueFilename_Dialog(xOffset,&m_xEventData.m_xFilename,&m_bFileChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//playback type
	Asura_DropDown_List_Dialog* pxListDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszPlaybackTypes,&m_xEventData.m_iPlaybackType,4,4,NULL,&m_bChanged,"Playback type");
	pxListDialog->SetListHelpStrings(g_aszPlaybackTypeHelp);
	pxListDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//play probability
	pxDialog = new Asura_Numerical_Dialog("Play Probability",xOffset,&m_xEventData.m_iPlayProbability,0,100,1,"%d",&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetHelpText("The probability that this dialogue event will play at all when triggered.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//priority
	pxDialog = new Asura_Numerical_Dialog("Priority",xOffset,&m_xEventData.m_iPriority,1,200,1,"%d",&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetHelpText("The value used to determine whether this dialogue event will override another dialogue event.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//replay delay
	pxDialog = new Asura_Numerical_Dialog("Replay Delay",xOffset,&m_xEventData.m_fReplayDelayTime,0.0f,9999.0f,1.0f,"%.3f",&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetHelpText("The minimum amount of time that must have passed after it has finished playing before triggering this event again will actually cause the event to play.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//global event
	pxDialog = new Asura_Tickbox_Dialog("Global Event",xOffset,&m_xEventData.m_bGlobalEvent,&m_bChanged);
	pxDialog->SetHelpText("Marks this event for inclusion in the global dialogue file.");
	pxDialog->SetIsEnabledFunction(CanEditGlobal,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//message editing
	pxDialog = new Asura_Dialog("Message Blocks :",xOffset);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	BuildMessageList();
	pxDialog = m_pxMessageListDialog = new Asura_List_Dialog(xOffset,m_pszMessageList,&m_iSelectedMessage,8,m_uMessageListSize,NULL);
	pxDialog->SetHelpText("List of message blocks attached to this event");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	m_pxMessageListDialog->SetExpandWidthToFillOverlay(false);
	m_pxMessageListDialog->SetMinWidth(25.0f);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.5f;

	pxDialog = new Asura_Function_Dialog("Add Message",xOffset,AddMessage,(int)this);
	pxDialog->SetHelpText("Add a new message block");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Delete Message",xOffset,DeleteMessage,(int)this);
	pxDialog->SetHelpText("Delete the selected message block");
	pxDialog->SetIsEnabledFunction(CanDeleteMessage,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 12.0f;

	pxDialog = new Asura_Function_Dialog("Edit Message",xOffset,EditMessage,(int)this);
	pxDialog->SetHelpText("Edits the selected message block.");
	pxDialog->SetIsEnabledFunction(CanEditMessage,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = m_pxInitialMessageDialog = new Asura_Function_Dialog("Initial Msg",xOffset,SetInitialMessage,(int)this);
	pxDialog->SetHelpText("The message block to trigger when starting to play this event.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = m_pxFinalMessageDialog = new Asura_Function_Dialog("Final Msg",xOffset,SetFinalMessage,(int)this);
	pxDialog->SetHelpText("The message block to trigger when the event ends.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	
	
	pxDialog = new Asura_Function_Dialog("Preview Dialogue",xOffset,PlaySound,(int)this);
	pxDialog->SetHelpText("Plays this dialogue event.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Stop Preview",xOffset,StopSound,(int)this);
	pxDialog->SetHelpText("Stops playback of test sound.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);


	//Start a new column to the right
	xOffset = pxTabsDialog->GetTopDialogPosition();
	xOffset.x += 30.0f;
	
	//list of events/samples
	BuildPlayableList();
	
	pxDialog = new Asura_Dialog("Events/Lines :",xOffset);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = m_pxPlayableListDialog = new Asura_List_Dialog(xOffset,m_pszPlayableList,&m_iSelectedPlayable,8,m_uPlayableListSize,NULL,&m_bSelectedPlayableChanged);
	m_pxPlayableListDialog->SetListHelpStrings(m_pszPlayableHelp);
//	pxDialog->SetHelpText("List of events/lines played by this event");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	//data for selected playable
	UpdatePlayableDataFromToolsEvent();

	pxDialog = new Asura_Numerical_Dialog("Delay",xOffset,&m_fPlayableDelay,0.0f,9999.0f,1.0f,"%.3f",&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetHelpText("The amount of time the current will wait before playing after the previous line has finished playing. This is only available if the playback type is set to all.");
	pxDialog->SetIsEnabledFunction(CanEditPlayableDelay,(int)this);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Weight",xOffset,&m_uPlayableWeight,0,100,1,"%d",&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetHelpText("The probability of this line being chosen over any others, which can be made to make the choice of a certain line in the list more or less likely to be chosen. This is only available if the playback type is set to random.");
	pxDialog->SetIsEnabledFunction(CanEditPlayableWeight,(int)this);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	#if 1
	pxDialog = m_pxPlayableStartMessageDialog = new Asura_Function_Dialog("Message",xOffset,SetPlayableStartMessage,(int)this);
 	pxDialog->SetHelpText("The message block to trigger when this playable starts.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetIsEnabledFunction(CanEditPlayableMessage,(int)this);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = m_pxPlayableEndMessageDialog = new Asura_Function_Dialog("Message",xOffset,SetPlayableEndMessage,(int)this);
 	pxDialog->SetHelpText("The message block to trigger when this playable finishes.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetIsEnabledFunction(CanEditPlayableMessage,(int)this);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	#endif



	//function dialogs for manipulating event list
	pxDialog = new Asura_Function_Dialog("Add Line",xOffset,AddLine,(int)this);
	pxDialog->SetHelpText("Add a line to the list");
	pxDialog->SetIsEnabledFunction(CanAddLine,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Replace Line",xOffset,ReplaceLine,(int)this);
	pxDialog->SetHelpText("Replace the selected line with another , preserving weighting etc.");
	pxDialog->SetIsEnabledFunction(CanReplaceLine,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Add Event",xOffset,AddEvent,(int)this);
	pxDialog->SetHelpText("Add an event to the list");
	pxDialog->SetIsEnabledFunction(CanAddEvent,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Remove",xOffset,RemovePlayable,(int)this);
	pxDialog->SetHelpText("Remove the selected event/line from the list.");
	pxDialog->SetIsEnabledFunction(CanRemovePlayable,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("View Line",xOffset,ViewLines,(int)this);
	pxDialog->SetHelpText("View the selected line in the line list.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Put the next 4 over to the right
	xOffset.y -= Asura_Dialog::DefaultSpacing()*3;
	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Edit",xOffset,EditPlayable,(int)this);
	pxDialog->SetHelpText("Edits the selected event/line.");
	pxDialog->SetIsEnabledFunction(CanEditPlayable,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Move Up",xOffset,MoveUp,(int)this);
	pxDialog->SetHelpText("Moves the selected event/line earlier in the list");
	pxDialog->SetIsEnabledFunction(CanMoveUp,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Move Down",xOffset,MoveDown,(int)this);
	pxDialog->SetHelpText("Moves the selected event/line later in the list");
	pxDialog->SetIsEnabledFunction(CanMoveDown,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Sort",xOffset,Sort,(int)this);
	pxDialog->SetHelpText("Sorts the event/line list alphabetically");
	pxDialog->SetIsEnabledFunction(CanSort,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 12.0f;

	Asura_Char szEditTemplate[128];
	Asura_PrintfToBfr(szEditTemplate,"Edit template: %s",128,true,GetSelectedPlayableTemplateName());
	pxDialog = m_pxEditTemplateDialog = new Asura_Function_Dialog(szEditTemplate,xOffset,EditSelectedPlayable,(int)this);
	pxDialog->SetHelpText("Opens a new tab with the template of the currently selected line");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Create Dialogue Trigger",xOffset,CreateTrigger,(int)this);
	pxDialog->SetHelpText("Creates a dialogue trigger that uses this event..");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Dialogue Controller",xOffset,CreateController,(int)this);
	pxDialog->SetHelpText("Creates a dialogue controller that uses this event..");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	UpdateInitialMessageDialog();
	UpdateFinalMessageDialog();
	UpdatePlayableStartMessageDialog();
	UpdatePlayableEndMessageDialog();
}

const char* Tools_DialogueEventTab_Event::GetName() const
{
	return m_pxEvent->GetName();

}

void Tools_DialogueEventTab_Event::PlaySound()
{
	Tools_DialogueEvent_Overlay::PlayDialogue(m_uEventHash);
}

void Tools_DialogueEventTab_Event::StopSound()
{
	Tools_DialogueEvent_Overlay::StopSound();
}


void Tools_DialogueEventTab_Event::UpdateFromAsuraData()
{
	if(m_bMessageEditing)
	{
		AbortMessageEditing();
	}

	m_xEventData.CopyFromAsuraEvent(m_pxEvent);
	BuildPlayableList();
	BuildMessageList();
	UpdatePlayableDataFromToolsEvent();
}

bool Tools_DialogueEventTab_Event::Delete()
{
	if(m_bMessageEditing)
	{
		AbortMessageEditing();
	}
	return Tools_Dialogue_System::DeleteEvent(m_uEventHash,true);
}

void Tools_DialogueEventTab_Event::Copy()
{
	s_xPasteBuffer = m_xEventData;
	s_bPasteBufferValid = true;
	Tools_Timed_Message_System::AddMessage(0xffffff,"%s copied.",m_pxEvent->GetName());
}

void Tools_DialogueEventTab_Event::Paste()
{
	if(s_bPasteBufferValid)
	{
		//make sure that pasting over this event won't cause it to include itself
		for(u_int i=0;i<s_xPasteBuffer.m_uNumPlayable;i++)
		{
			if(s_xPasteBuffer.m_pxPlayables[i].m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT)
			{
				if(!IsOkayToAddEvent(m_uEventHash,s_xPasteBuffer.m_pxPlayables[i].m_xID.m_uHashID))
				{
					ShowMessageOverlay("Unable to paste over this event , because it would result in an event including itself.");
					return;
				}
			}
		}
		
		if(m_bMessageEditing)
		{
			AbortMessageEditing();
		}

		
		m_xEventData = s_xPasteBuffer;
		BuildPlayableList();
		BuildMessageList();
		UpdatePlayableDataFromToolsEvent();
		m_bFileChanged = true;
		m_bChanged = true;
		Tools_Timed_Message_System::AddMessage(0xffffff,"Event pasted.");
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Event paste buffer is empty.");
	}
}


void Tools_DialogueEventTab_Event::PasteMultiple()
{

	//Get a list of events to paste over
	m_bBeingEdited = true;
	Tools_PickDialogueEvent_Overlay::PickEvent("Paste over which events?",0,true);
	m_bBeingEdited = false;


	u_int uNumSelected = Tools_PickDialogueEvent_Overlay::GetNumSelected();
	if(!uNumSelected) return;

	//Determine which properties should be pasted
	Tools_DialogueEventProperty_Overlay xPropertyOverlay;
	if(!xPropertyOverlay.DoModal()) return;

	u_int uMessageLogPos = Tools_MessageLog::GetNextID();
	
	//Do the copying
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uEventHash = Tools_PickDialogueEvent_Overlay::GetSelection(i);
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(uEventHash);
		if(!pxEvent) continue;

		//get a tools copy of the event properties
		Tools_DialogueEvent xToolsDialogueEvent;
		xToolsDialogueEvent.CopyFromAsuraEvent(pxEvent);

		//set modified before the copy in case the file is being changed
		Tools_Dialogue_System::SetModified(pxEvent);

		//Do the copy
		xToolsDialogueEvent.Copy(pxEvent,m_xEventData,xPropertyOverlay.GetSelectedArray());

		//Update the asura event and flag as modified
		xToolsDialogueEvent.CopyToAsuraEvent(pxEvent);
		Tools_Dialogue_System::SetModified(pxEvent);

		//Make sure any tab with this event is updated
		Tools_DialogueEvent_Overlay::UpdateEventFromAsuraData(uEventHash);

	}

	//If there were any errors , open the message log
	if(Tools_MessageLog::AreThereAnyMessages(Tools_MessageLog::DIALOGUE, 0, uMessageLogPos))
	{
		Tools_MessageLog::ShowOverlay(Tools_MessageLog::DIALOGUE,"Paste Error",uMessageLogPos);
	}

}


void Tools_DialogueEventTab_Event::Duplicate()
{
	//get a default name for the event
	//get current sample name without any numbers at the end
	char szBaseName[100];
	Asura_PrintfToBfr(szBaseName,"%s",sizeof(szBaseName),true,m_pxEvent->GetName());
	int iLength = strlen(szBaseName);
	while(iLength>0 && szBaseName[iLength-1]>='0' && szBaseName[iLength-1]<='9') iLength--;
	szBaseName[iLength] = 0;
	

	char szEventName[200];
	for(int iIndex=0;;iIndex++)
	{
		Asura_PrintfToBfr(szEventName,"%s%d",200,false,szBaseName,iIndex);

		if(!Asura_Dialogue_Storage::GetEventByHash(Asura_GetHashID(szEventName))) break;
	}

	
	
	//ask for a name  , offering the default name
	GetString_Overlay xStringOverlay("Name new event",szEventName);
	do
	{
		if(!xStringOverlay.DoModal()) break;
		
		const char* szEventName = xStringOverlay.GetString();
		if(!szEventName || !*szEventName) break;

		//make sure this sample doesn't already exist
		Asura_Hash_ID uEventHash = Asura_GetHashID(szEventName);
		if(Asura_Dialogue_Storage::GetEventByHash(uEventHash))
		{
			ShowMessageOverlay("An event already exists using the same hash ID.");
			continue;
		}
		bool bAlreadyExists;
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::CreateEvent(szEventName, bAlreadyExists);
		Asura_Assert(pxEvent && !bAlreadyExists, "Error creating dialogue event");

		//copy this event's properties to the new one
		m_xEventData.CopyToAsuraEvent(pxEvent);
		Tools_Dialogue_System::SetModified(pxEvent);
		Tools_Dialogue_System::RegisterNewEvent(pxEvent);
		//and edit it
		Tools_DialogueEvent_Overlay::EditEvent(uEventHash);
		break;
		
	}while(true);
}

void Tools_DialogueEventTab_Event::Rename()
{
	const char* szNewName = 0;
	Asura_Hash_ID uOldHash = m_uEventHash;
	Asura_Hash_ID uNewHash = 0;
	bool bOkay = false;
	
	//ask for new name
	GetString_Overlay xStringOverlay("Rename Event",m_pxEvent->GetName());
	while(xStringOverlay.DoModal())
	{
		szNewName = xStringOverlay.GetString();
		if(!szNewName || !*szNewName) return;

		if(!strcmp(szNewName,m_pxEvent->GetName()))
		{
			//name not changed
			return;
		}

		//make sure no existing sample exists with this hash
		uNewHash = Asura_GetHashID(szNewName);

		if(uOldHash != uNewHash && Asura_Dialogue_Storage::GetEventByHash(uNewHash))
		{
			ShowMessageOverlay("An event already exists using the same hash ID.");
			continue;
		}

		bOkay = true;
		break;
	}
	if(!bOkay) return;

	//Update the sample and its position in the storage system
	if(!Asura_Dialogue_Storage::RenameEvent(m_pxEvent->GetName(),szNewName))
	{
		Asura_Assert(0,"Error renaming event");
		return;
	}
	SetModified();

	//update this tab
	m_uEventHash = uNewHash;
	UpdateTabName();

	//Update any events using this event
	Asura_Dialogue_PlayableID xOldID,xNewID;
	xOldID.m_uHashID = uOldHash;
	xOldID.m_eType = ASURA_DIALOGUE_PLAYABLE_EVENT;
	xNewID.m_uHashID = uNewHash;
	xNewID.m_eType = ASURA_DIALOGUE_PLAYABLE_EVENT;
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		if(pxEvent->ReplacePlayableID(xOldID,xNewID))
		{
			Tools_Dialogue_System::SetModified(pxEvent);

			//Update the tab for this event if it was being edited
			Tools_DialogueEvent_Overlay::UpdateEventFromAsuraData(pxEvent->GetHashID());
		}
	}

	//TBD. Update other stuff that may refer to the event

}

void Tools_DialogueEventTab_Event::UpdateTabName()
{
	char szTabName[MAX_TAB_NAME_LENGTH+2];
	Asura_PrintfToBfr(szTabName,"E: %s",sizeof(szTabName),true,m_pxEvent->GetName());
	if(strlen(szTabName) > MAX_TAB_NAME_LENGTH)
	{
		//name is too long for tab , so truncate it
		strcpy(&szTabName[MAX_TAB_NAME_LENGTH-3],"...");
	}

	m_xTabName = szTabName;
}

void Tools_DialogueEventTab_Event::SetModified()
{
	Tools_Dialogue_System::SetModified(m_pxEvent);
}

bool Tools_DialogueEventTab_Event::CanEditPlayableWeight()
{
	return m_xEventData.m_uNumPlayable>0 && m_xEventData.m_iPlaybackType == ASURA_DPT_RANDOM;
}

bool Tools_DialogueEventTab_Event::CanEditPlayableDelay()
{
	return m_xEventData.m_uNumPlayable>0 && m_xEventData.m_iPlaybackType == ASURA_DPT_ALLSEQUENTIAL;
}

void Tools_DialogueEventTab_Event::UpdatePlayableDataFromToolsEvent()
{
	if(m_xEventData.m_uNumPlayable)
	{
		m_fPlayableDelay = m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_fDelay;
		m_uPlayableWeight = m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_ucWeight;

	}
	else
	{
		m_fPlayableDelay = 0.0f;
		m_uPlayableWeight = 1;
	}

	UpdateInitialMessageDialog();
	UpdateFinalMessageDialog();
	UpdatePlayableStartMessageDialog();
	UpdatePlayableEndMessageDialog();

}

void Tools_DialogueEventTab_Event::UpdatePlayableDataInToolsEvent()
{
	if(m_xEventData.m_uNumPlayable)
	{
		m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_fDelay = m_fPlayableDelay;
		m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_ucWeight = (u_char)m_uPlayableWeight;
	}
}

bool Tools_DialogueEventTab_Event::CanEditGlobal()
{
	return !Tools_Dialogue_System::IsLevelSpecific(m_pxEvent);
}


void Tools_DialogueEventTab_Event::AddMessage()
{
	//ask for a name
	GetString_Overlay xOverlay("Name new message block","");
	do
	{
		if(!xOverlay.DoModal()) return;
		const char* szName = xOverlay.GetString();
		if(!szName || !*szName) return;

	
		//make sure the name isn't already used
		Asura_Hash_ID uNameHash = Asura_GetHashID(szName);
		bool bOkay = true;
		for(u_int i=0;i<m_xEventData.m_uNumMessages;i++)
		{
			if(m_xEventData.m_ppxMessages[i]->m_uHashID == uNameHash)
			{
				ShowMessageOverlay("Event already has a message block with that name");
				bOkay = false;
				break;
			}		
		}
		if(!bOkay) continue;

		//create the new block
		Asura_Dialogue_Message* pxNewMessage = new Asura_Dialogue_Message;
		pxNewMessage->m_xName = szName;
		pxNewMessage->m_uHashID = uNameHash;

		//add to the list
		Asura_Dialogue_Message** ppxMessages = new Asura_Dialogue_Message*[m_xEventData.m_uNumMessages+1];
		for(u_int i=0;i<m_xEventData.m_uNumMessages;i++)
		{
			ppxMessages[i] = m_xEventData.m_ppxMessages[i];
		}
		ppxMessages[m_xEventData.m_uNumMessages] = pxNewMessage;

		delete [] m_xEventData.m_ppxMessages;
		m_xEventData.m_ppxMessages = ppxMessages;
		m_xEventData.m_uNumMessages++;

 		m_bChanged = true;

		m_iSelectedMessage = m_xEventData.m_uNumMessages - 1;
		BuildMessageList();

		//edit the new message block
		EditMessage();

		return;

	}while(true);
}

bool Tools_DialogueEventTab_Event::CanDeleteMessage()
{
	return m_xEventData.m_uNumMessages != 0 && !m_bMessageEditing;
}

void Tools_DialogueEventTab_Event::DeleteMessage()
{
	if(!CanDeleteMessage()) return;

	Asura_Dialogue_Message* pxMessageToDelete = m_xEventData.m_ppxMessages[m_iSelectedMessage];
	
	//confirm deletion
	if(!ShowYesNoOverlay("Delete message block '%s'?",pxMessageToDelete->m_xName.GetString())) return;

	//remove from the list ,and shuffle the rest down
	delete pxMessageToDelete;
	m_xEventData.m_uNumMessages--;
	for(u_int i=m_iSelectedMessage;i<m_xEventData.m_uNumMessages;i++)
	{
		m_xEventData.m_ppxMessages[i] = m_xEventData.m_ppxMessages[i+1];
	}

	m_bChanged = true;
	BuildMessageList();

}

bool Tools_DialogueEventTab_Event::CanEditMessage()
{
	return m_xEventData.m_uNumMessages != 0 && !m_bMessageEditing;
//	return false;
}

void Tools_DialogueEventTab_Event::EditMessage()
{
	if(!CanEditMessage()) return;
	m_bMessageEditing = true;
	Tools_Dialogue_System::PreMessageEditing();
	Tools_Object_System::EditMessageBlock("Edit message block",&m_xEventData.m_ppxMessages[m_iSelectedMessage]->m_xMessageBlock,NULL,&m_bChanged);
}

void Tools_DialogueEventTab_Event::AbortMessageEditing()
{
	//force update of messages in event , since message editing is a bit lax on setting stuff as modified.
	m_xEventData.CopyMessagesToAsuraEvent(m_pxEvent);

	m_bMessageEditing = false;
	Tools_Object_System::EndMessageBlockEditing();
	Tools_Dialogue_System::PostMessageEditing();
}


Asura_Hash_ID Tools_DialogueEventTab_Event::SelectMessageBlock(const char* szTitle,Asura_Hash_ID uInitialSelection)
{
	//initialise list selection overlay containing all the message block names
	ListSelection_Overlay xOverlay(szTitle);

	for(u_int i=0;i<m_xEventData.m_uNumMessages;i++)
	{
		const Asura_Dialogue_Message* pxMessage = m_xEventData.m_ppxMessages[i];
		Asura_Hash_ID uID = pxMessage->m_uHashID;

		xOverlay.AddItem(pxMessage->m_xName.GetString(),uID,uID==uInitialSelection);
	}
	xOverlay.AddItem("*None*",uInitialSelection==0);

	if(!xOverlay.DoModal()) return uInitialSelection;

	return (Asura_Hash_ID) xOverlay.GetSelectionData();

}

void Tools_DialogueEventTab_Event::SetInitialMessage()
{
	Asura_Hash_ID uSelection = SelectMessageBlock("Select message block for start of event",m_xEventData.m_uInitialMessage);
	if(uSelection != m_xEventData.m_uInitialMessage)
	{
		m_xEventData.m_uInitialMessage = uSelection;
		m_bChanged = true;
		UpdateInitialMessageDialog();
	}
}

void Tools_DialogueEventTab_Event::SetFinalMessage()
{
	Asura_Hash_ID uSelection = SelectMessageBlock("Select message block for end of event",m_xEventData.m_uFinalMessage);
	if(uSelection != m_xEventData.m_uFinalMessage)
	{
		m_xEventData.m_uFinalMessage = uSelection;
		m_bChanged = true;
		UpdateFinalMessageDialog();
	}
}


bool Tools_DialogueEventTab_Event::CanEditPlayableMessage()
{
	return m_xEventData.m_uNumPlayable>0;
}

void Tools_DialogueEventTab_Event::SetPlayableStartMessage()
{
	if(!CanEditPlayableMessage()) return;

	Asura_Dialogue_PlayableIDListItem* pxPlayable = &m_xEventData.m_pxPlayables[m_iSelectedPlayable];

	Asura_Hash_ID uSelection = SelectMessageBlock("Select message block to trigger at start of playable.",pxPlayable->m_uStartMessageID);
	if(uSelection != pxPlayable->m_uStartMessageID)
	{
		pxPlayable->m_uStartMessageID = uSelection;
		m_bChanged = true;
		UpdatePlayableStartMessageDialog();
	}
}

void Tools_DialogueEventTab_Event::SetPlayableEndMessage()
{
	if(!CanEditPlayableMessage()) return;

	Asura_Dialogue_PlayableIDListItem* pxPlayable = &m_xEventData.m_pxPlayables[m_iSelectedPlayable];

	Asura_Hash_ID uSelection = SelectMessageBlock("Select message block to trigger at end of playable.",pxPlayable->m_uEndMessageID);
	if(uSelection != pxPlayable->m_uEndMessageID)
	{
		pxPlayable->m_uEndMessageID = uSelection;
		m_bChanged = true;
		UpdatePlayableEndMessageDialog();
	}
}

void Tools_DialogueEventTab_Event::UpdateInitialMessageDialog()
{
	UpdateMessageDialog(m_pxInitialMessageDialog,"Initial Msg : ", m_xEventData.m_uInitialMessage);
}

void Tools_DialogueEventTab_Event::UpdateFinalMessageDialog()
{
	UpdateMessageDialog(m_pxFinalMessageDialog,"Final Msg : ", m_xEventData.m_uFinalMessage);
}

void Tools_DialogueEventTab_Event::UpdatePlayableStartMessageDialog()
{
	Asura_Hash_ID uID = m_xEventData.m_uNumPlayable ? m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_uStartMessageID : 0;
	UpdateMessageDialog(m_pxPlayableStartMessageDialog,"Ln Start Msg : ", uID);
}

void Tools_DialogueEventTab_Event::UpdatePlayableEndMessageDialog()
{
	Asura_Hash_ID uID = m_xEventData.m_uNumPlayable ? m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_uEndMessageID : 0;
	UpdateMessageDialog(m_pxPlayableEndMessageDialog,"Ln End Msg : ", uID);
}


void Tools_DialogueEventTab_Event::UpdateMessageDialog(Asura_Dialog* pxDialog,const char* szPrefix,Asura_Hash_ID uMessageID)
{
	if(!pxDialog) return;
	//get the name of the message
	const char* szMessage = 0;
	if(uMessageID)
	{
		szMessage = "*Not Found*";
		for(u_int i=0;i<m_xEventData.m_uNumMessages;i++)
		{
			if(m_xEventData.m_ppxMessages[i]->m_uHashID == uMessageID)
			{
				szMessage = m_xEventData.m_ppxMessages[i]->m_xName.GetString();
				break;
			}
		}
	}
	else
	{
		szMessage = "*None*";
	}

	char szLabel[100];
	Asura_PrintfToBfr(szLabel,"%s%s",sizeof(szLabel),true,szPrefix,szMessage);

	pxDialog->UpdateLabel(szLabel);

}

void Tools_DialogueEventTab_Event::CreateTrigger()
{
	Tools_Entity_DialogueTrigger* pxTrigger = Tools_Entity_DialogueTrigger::Create(false,m_uEventHash);
	if(pxTrigger)
	{
		//assign the new object to the correct entity file
		//could really do with a better way of ensuring this gets done...
		Tools_SecondaryEntityFile_System::AddToCurrentFile(pxTrigger);

		Tools_Object_System::SelectObject(pxTrigger);
	}

}

void Tools_DialogueEventTab_Event::CreateController()
{
	Tools_Entity_DialogueController* pxController = Tools_Entity_DialogueController::Create( false );
	if(pxController)
	{
		//assign the new object to the correct entity file
		//could really do with a better way of ensuring this gets done...
		Tools_SecondaryEntityFile_System::AddToCurrentFile( pxController );

		Asura_ServerEntity_DialogueController* pxControllerEntity = reinterpret_cast<Asura_ServerEntity_DialogueController*>( pxController->Entity );

		bool bHasMessagesToCopy = false;
		// Check if there are any messages on lines which may want to be copied into the entity
		
		for( u_int uPlayable = 0; uPlayable < m_xEventData.m_uNumPlayable; ++ uPlayable )
		{
			if(		( m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_uStartMessageID != ASURA_HASH_ID_UNSET )
				||	( m_xEventData.m_pxPlayables[m_iSelectedPlayable].m_uEndMessageID	!= ASURA_HASH_ID_UNSET ) )
			{
				bHasMessagesToCopy = true;
				break;
			}
		}

		if( bHasMessagesToCopy )
		{
			if( !ShowYesNoOverlay( "There are static messages associated with some of the dialogue lines in this event, do you wish to move them into the created dialogue controller?" ) )
			{
				bHasMessagesToCopy = false;
			}
		}

		pxControllerEntity->SetEventHashID( m_uEventHash, bHasMessagesToCopy );
		pxController->UpdateCharacterMappings();

		Tools_Object_System::SelectObject( pxController );
	}

}

void Tools_DialogueEventTab_Event::EditSelectedPlayable()
{
	const Asura_Hash_ID uTemplateHash = GetSelectedPlayableTemplate();
	if( uTemplateHash != ASURA_HASH_ID_UNSET )
	{
		Tools_DialogueEvent_Overlay::EditLineTemplate( uTemplateHash );
	}
}

Asura_Hash_ID Tools_DialogueEventTab_Event::GetSelectedPlayableTemplate() const
{
	if( m_xEventData.m_uNumPlayable > 0 )
	{
		int iPlayableIdx = m_iSelectedPlayable;

		// If selection is invalid, use first playable
		if( ( m_iSelectedPlayable < 0 ) || ( m_iSelectedPlayable >= (signed)m_xEventData.m_uNumPlayable ) )
		{
			iPlayableIdx = 0;
		}

		if( m_xEventData.m_pxPlayables[iPlayableIdx].m_xID.m_eType == ASURA_DIALOGUE_PLAYABLE_LINE )
		{
			const Asura_Dialogue_Line* const pxLine = Asura_Dialogue_Storage::GetLineByHash( m_xEventData.m_pxPlayables[iPlayableIdx].m_xID.m_uHashID );
			const Asura_Dialogue_Line_Template* const pxTemplate = pxLine ? pxLine->GetTemplate() : NULL;
			if( pxTemplate )
			{
				return pxTemplate->m_uHashID;
			}
		}
	}

	return ASURA_HASH_ID_UNSET;
}

const Asura_Char* Tools_DialogueEventTab_Event::GetSelectedPlayableTemplateName() const
{
	const Asura_Hash_ID uLineTemplateHash = GetSelectedPlayableTemplate();
	if( uLineTemplateHash != ASURA_HASH_ID_UNSET )
	{
		const Asura_Dialogue_Line_Template* const pxTemplate = Asura_Dialogue_Storage::GetLineTemplateByHash( uLineTemplateHash );
		if( pxTemplate )
		{
			return pxTemplate->m_xName.GetString();
		}
	}

	return "";
}

#endif //ASURA_USE_DIALOGUE_SYSTEM