//
// Tools_DialogueEventTab_LineTemplate.h
//
// Created by Richard Rice - 03/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_DIALOGUEEVENTTAB_LINETEMPLATE__
#define __TOOLS_DIALOGUEEVENTTAB_LINETEMPLATE__

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Sound/Asura_Dialogue.h"
#include "Sound/Tools_DialogueEvent_Overlay.h"



//Stores a copy of most things from Asura_Dialogue_Line_Template for editing by the overlay
class Tools_DialogueLineTemplate
{
public :
	Tools_DialogueLineTemplate();
	~Tools_DialogueLineTemplate();

	void CopyFromAsuraTemplate(const class Asura_Dialogue_Line_Template* pxTemplate);
	void CopyToAsuraTemplate(class Asura_Dialogue_Line_Template* pxTemplate) const;


	Tools_DialogueLineTemplate& operator = (const Tools_DialogueLineTemplate& xTemplate);
	void Copy(const Tools_DialogueLineTemplate& xTemplate,const bool* pbProperties);

	float m_fGain;
	float m_fInnerRange;
	float m_fOuterRange;
	float m_fMinGain;
	bool m_bVoiceover;
	bool m_bNoWavForPSP;
	bool m_bNoSubtitle;
	bool m_bStreamed;
	Asura_Hash_ID m_uMixerPreset;

	bool m_bUseSampleRatePS2;
	bool m_bUseSampleRatePSP;
	u_int m_uSampleRateForPS2;
	u_int m_uSampleRateForPSP;
	Asura_String m_xMixingChannelName;
	u_int m_uCompressionQualityPS3;
	u_int m_uCompressionQuality360;
	Asura_Hash_ID m_uDSPChain;

};


//for handling a single event editing tab
class Tools_DialogueEventTab_LineTemplate : public Tools_DialogueEventTab
{
public :
	Tools_DialogueEventTab_LineTemplate(class Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uTemplateHash);
	~Tools_DialogueEventTab_LineTemplate();

	virtual void PerFrameUpdate();
	virtual Asura_Hash_ID GetHashID() {return m_uTemplateHash;}
	virtual bool IsLineTemplate(){return true;}
	virtual const char* GetName() const;
	virtual const char* GetTabName() const {return m_xTabName.GetString();}
	virtual void UpdateFromAsuraData();
	virtual bool Delete();
	virtual void Copy();
	virtual void Paste();
	virtual void PasteMultiple();
	virtual void Duplicate();
	virtual void Rename();
	virtual bool CanBeClosed() const {return !m_bBeingEdited;}

private :
	void SetModified();

	void AddOverlayControls(class Asura_Tabs_Dialog* pxTabsDialog);
	

	void UpdateTabName();

	void UpdatePresetName();
	
	//overlay callback functions
	//defines for routeing static callback functions to non-static versions
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_DialogueEventTab_LineTemplate*)iThis)->__FnName();}\
	void __FnName();

	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_DialogueEventTab_LineTemplate*)iThis)->__FnName();}\
	bool __FnName();


	OVERLAY_BOOL_CALLBACK(IsEnabled_SampleRatePS2)
	OVERLAY_BOOL_CALLBACK(IsEnabled_SampleRatePSP)
	OVERLAY_VOID_CALLBACK(SelectMixerPreset)


	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK

	/////////////

	Asura_Hash_ID m_uTemplateHash;
	class Asura_Dialogue_Line_Template* m_pxTemplate;

	Tools_DialogueLineTemplate m_xTemplateData;

	bool m_bChanged;
	bool m_bMinChanged;
	bool m_bMaxChanged;
	bool m_bBeingEdited;

	Asura_String m_xTabName;
	Asura_String m_xPresetName;

	static bool s_bPasteBufferValid;
	static Tools_DialogueLineTemplate s_xPasteBuffer;
};

#endif //ASURA_USE_DIALOGUE_SYSTEM
#endif //__TOOLS_DIALOGUEEVENTTAB_LINETEMPLATE__
