//////////////////////////////////////////////////////////////////////////////
//
// Tools_Dialogue_System.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

// This file's header.
#include "Sound/Tools_Dialogue_System.h"

// Asura includes
#include "FileAccess/Asura_FileAccess.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Dialogue.h"
#include "Core/Asura_LocalisedText.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_DialogueTrigger.h"
#include "Entity/Sound/Asura_ServerEntity_DialogueController.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"
#include "Sound/Asura_Sound_Mixer.h"

//Tools
#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Sound/Tools_Entity_DialogueTrigger.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_PickDialogueEvent_Overlay.h"
#include "Sound/Tools_PickDialogueLine_Overlay.h"
#include "Sound/Tools_PickDialogueLineTemplate_Overlay.h"
#include "Sound/Tools_DialogueEvent_Overlay.h"
#include "Sound/Tools_PickSound.h"


/////////////////////////////////////////////////////////////////////////////////////////

class Tools_MessageLogAction_DialogueEvent : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_DialogueEvent(Asura_Hash_ID uHashID) {m_uHashID = uHashID;}
	virtual void Execute()
	{
		Tools_DialogueEvent_Overlay::EditEvent(m_uHashID);	
	}

	Asura_Hash_ID m_uHashID;
};

class Tools_MessageLogAction_DialogueLineTemplate : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_DialogueLineTemplate(Asura_Hash_ID uHashID) {m_uHashID = uHashID;}
	virtual void Execute()
	{
		Tools_DialogueEvent_Overlay::EditLineTemplate(m_uHashID);	
	}

	Asura_Hash_ID m_uHashID;
};

/////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_Dialogue : public Asura_Overlay
{
public :
	Tools_Overlay_Dialogue(const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
	:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
	{}

	~Tools_Overlay_Dialogue()
	{
		if(IsClosingByCloseGadget())
		{
			Tools_Dialogue_System::s_bOverlayShouldBeOpen = false;
		}
	}
};

/////////////////////////////////////////////////////////////////////////////////////////

static const char* g_szAudioTemplateFilename = "Dialogue\\AudioTemplates.asr";
static const char* g_szHashToPathFilename = "Dialogue\\HashToPath.asr";



List<Asura_String> Tools_Dialogue_System::s_xModifiedFiles;
bool Tools_Dialogue_System::s_bAudioTemplatesModified = false;

Asura_HashTree Tools_Dialogue_System::s_xNewEventTree;

Asura_NameTree Tools_Dialogue_System::s_xCharacterNameTree;
const char** Tools_Dialogue_System::s_pszCharacterNames;
Asura_Hash_ID* Tools_Dialogue_System::s_puCharacterHashes;


bool Tools_Dialogue_System::s_bEverHadAudioTemplates = false;

bool Tools_Dialogue_System::s_bGlobalSettingsChanged = false;
u_int Tools_Dialogue_System::s_uDefaultSampleRatePS2 = 22050;
u_int Tools_Dialogue_System::s_uDefaultSampleRatePSP = 22050;

List<int> Tools_Dialogue_System::s_xMinimisedOverlays;

Asura_String Tools_Dialogue_System::s_xMostRecentFile;

Asura_TemplateHashTree<Tools_Dialogue_HashToPath> Tools_Dialogue_System::s_xHashToPathTree;


Asura_Vector_2 Tools_Dialogue_System::s_xOverlayPosition(25.0f,2.0f);
bool Tools_Dialogue_System::s_bOverlayShouldBeOpen = false;

static const int g_iSubtitleBufferSize = 1024;
static Asura_UnicodeChar g_awcSubtitleBuffer[g_iSubtitleBufferSize];

Tools_Dialogue_System::FindExtraDialogueHeadingsCallback Tools_Dialogue_System::s_pfnFindExtraDialogueHeadingsCallback = NULL;
Tools_Dialogue_System::PostProcessDialogueLineCallback Tools_Dialogue_System::s_pfnPostProcessDialogueLineCallback = NULL;

void (*Tools_Dialogue_System::s_pfnPostLoadLevelSpecificFileCallback)(void) = NULL;

Tools_Dialogue_System* Tools_Dialogue_System::s_pxThis;

u_int Tools_Dialogue_System::s_uCharacterMappingValidity;

Tools_Dialogue_System::Tools_Dialogue_System()
{
	Asura_Assert(!s_pxThis,"Tools_Dialogue_System already allocated");
	s_pxThis = this;
}

Tools_Dialogue_System::~Tools_Dialogue_System()
{
	Asura_Assert(s_pxThis,"Tools_Dialogue_System not allocated");
	s_pxThis = 0;
}

void Tools_Dialogue_System::Initialise()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Dialogue_System::Initialise" );

	//allocate the tools class for this
	Tools_System::This->Allocate_Tools_Dialogue_System();

	LoadAsuraFile(g_szHashToPathFilename);
	LoadAudioTemplates();
	LoadDialogueLines();
	LoadDialogueFiles();

	TransferModeData(TOOLSMODEDATAFILE_LOAD);
}

void Tools_Dialogue_System::Shutdown()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Dialogue_System::Shutdown" );

	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	s_xCharacterNameTree.Reset();
	delete [] s_pszCharacterNames;
	s_pszCharacterNames = 0;
	delete [] s_puCharacterHashes;
	s_puCharacterHashes = 0;

	s_xHashToPathTree.Reset();

	delete s_pxThis;
}

void Tools_Dialogue_System::TransferModeData(int iMode)
{
	//open the file
	Tools_ModeDataFile xFile("SoundEvent.dat",iMode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	if( false == xFile.GetError())
	{
		xFile.Transfer(Tools_DialogueEvent_Overlay::s_axOverlayPosition[0]);
		xFile.Transfer(Tools_DialogueEvent_Overlay::s_axOverlayPosition[1]);
		xFile.Transfer(Tools_DialogueEvent_Overlay::s_bCombineOverlays);
		xFile.Transfer(Tools_PickDialogueEvent_Overlay::s_xOverlayPosition);
		xFile.Transfer(Tools_PickDialogueLine_Overlay::s_xOverlayPosition);
		xFile.Transfer(Tools_PickDialogueLineTemplate_Overlay::s_xOverlayPosition);
		xFile.Transfer(s_xOverlayPosition);
		xFile.Transfer(s_bOverlayShouldBeOpen);
	}
}

void Tools_Dialogue_System::Update()
{
	if(s_bGlobalSettingsChanged)
	{
		s_bGlobalSettingsChanged = false;
		SetAudioTemplatesModified();
	}
}


void Tools_Dialogue_System::SetModified(Asura_Dialogue_Event* pxEvent)
{
	Asura_Assert(pxEvent,"");
	SetModified(pxEvent->GetFilename());
}


void Tools_Dialogue_System::SetModified(const char* szFilename)
{
	Asura_Assert(szFilename && *szFilename,"Invalid filename");

	//file already in list?
	for(LIF<Asura_String> xModifiedLIF(&s_xModifiedFiles);!xModifiedLIF.done();xModifiedLIF.next())
	{
		if(!xModifiedLIF().Compare(szFilename)) return;
	}

	//File wasn't already flagged as modified ,so check whether its going to be okay to write to it
	Asura_String xFullFilename;
	GetFullFilename(szFilename,xFullFilename);

	Tools_SaveAsura::WarnIfReadOnly(xFullFilename.GetString());	

	s_xModifiedFiles.add_entry(szFilename);
}

bool Tools_Dialogue_System::CanSaveFile(Asura_Dialogue_Event* pxEvent)
{
	Asura_Assert(pxEvent,"");
	return CanSaveFile(pxEvent->GetFilename());
}


bool Tools_Dialogue_System::CanSaveFile(const char* szFilename)
{
	Asura_Assert(szFilename && *szFilename,"Invalid filename");

	Asura_String xFullFilename;
	GetFullFilename(szFilename,xFullFilename);

	return Tools_SaveAsura::CanSaveFile(xFullFilename.GetString(), false);	
}


void Tools_Dialogue_System::SetAudioTemplatesModified()
{
	if(s_bAudioTemplatesModified) return;

	Tools_SaveAsura::WarnIfReadOnly(g_szAudioTemplateFilename);	
	s_bAudioTemplatesModified = true;
}

void Tools_Dialogue_System::GetFullFilename(const char* szFilename,Asura_String& xFullFilename)
{
	char szTemp[MAX_PATH];
	
	if(!stricmp(szFilename,"Level"))
	{
		if(Tools_SaveAsura::IsEditingMission())
		{
			Asura_PrintfToBfr(szTemp,"%s.dlg",sizeof(szTemp),true,Tools_SaveAsura::GetMissionFilenameWithoutExtension());
		}
		else
		{
			Asura_PrintfToBfr(szTemp,"%s.dlg",sizeof(szTemp),true,Tools_SaveAsura::GetNameWithoutExtension());
		}
	}
	else
	{
		Asura_PrintfToBfr(szTemp,"Dialogue\\%s.dlg",sizeof(szTemp),true,szFilename);
	}
	xFullFilename = szTemp;

}

bool Tools_Dialogue_System::IsModified()
{
	return (s_xModifiedFiles.size() > 0) || s_bAudioTemplatesModified;
}

void Tools_Dialogue_System::SaveAll()
{
	//Dialogue files don't get saved in this way during level exports.RAR 02/11/2006
	if(Tools_Level_Export::IsExport()) return;
	

	int iJunkIndex;
	Asura_NameTree xFileTree;

	//attempt to save all the modified files
	//(this ensures that any files that have lost all their events will get saved)
	for(LIF<Asura_String> xModifiedLIF(&s_xModifiedFiles);!xModifiedLIF.done();)
	{
		//keep track of the files done
		xFileTree.FindNodeIndexByName(xModifiedLIF().GetString(),true,iJunkIndex);

		if(SaveFile(xModifiedLIF().GetString()))
		{
			xModifiedLIF.delete_current();
		}
		else
		{
			xModifiedLIF.next();
		}
	}

	//scan through all events for other files to save
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		const char* szFilename = pxEvent->GetFilename();

		if(!xFileTree.FindNodeIndexByName(szFilename,true,iJunkIndex))
		{
			SaveFile(szFilename);
		}
	}

	SaveAudioTemplates();

}

void Tools_Dialogue_System::SaveModified()
{
	//Dialogue files don't get saved in this way during level exports.RAR 02/11/2006
	if(Tools_Level_Export::IsExport()) return;

	for(LIF<Asura_String> xModifiedLIF(&s_xModifiedFiles);!xModifiedLIF.done();)
	{
		if(SaveFile(xModifiedLIF().GetString()))
		{
			xModifiedLIF.delete_current();
		}
		else
		{
			xModifiedLIF.next();
		}
	}

	if(s_bAudioTemplatesModified) SaveAudioTemplates();
}

bool Tools_Dialogue_System::SaveFile(const char* szFilename)
{
	//Dialogue files don't get saved in this way during level exports.RAR 02/11/2006
	if(Tools_Level_Export::IsExport()) return false;

	Asura_String xFullFilename;
	GetFullFilename(szFilename,xFullFilename);
	if(!Tools_SaveAsura::CanSaveFile(xFullFilename.GetString(),true))return false;


	Asura_SaveBlock xSaveBlock;

	//Attempt to save the events in an order that will result in a balanced tree on reload. RAR 17/08/2006
	if(Asura_Dialogue_Storage::GetNumberOfEvents())
	{
		for(Asura_TemplateHashTree_SaveIterator xIt(Asura_Dialogue_Storage::GetNumberOfEvents());!xIt.Done();xIt.Next())
		{
			const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(xIt.GetCurrent());
			if(!stricmp(pxEvent->GetFilename(),szFilename))
			{
				Asura_Chunk_Dialogue_Event::Create(pxEvent,&xSaveBlock);
			}
		}
	}
 
	//write out the save block

	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(xFullFilename.GetString());
	bool bSuccess = xSaveBlock.SaveFile(xFullFilename.GetString(),true);
    if(bSuccess)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.",xFullFilename.GetString());
		Tools_LoadedFileList::UpdateModifiedTime(xFullFilename.GetString());
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s",xFullFilename.GetString());
	}

	return bSuccess;	
}

bool Tools_Dialogue_System::SaveAudioTemplates()
{
	//This file doesn't get saved in this way during level exports.RAR 02/11/2006
	if(Tools_Level_Export::IsExport()) return false;

	if(!Tools_SaveAsura::CanSaveFile(g_szAudioTemplateFilename,true))return false;

	if(!s_bEverHadAudioTemplates)
	{
		//Never had any templates , so avoid saving an empty file. RAR 06/11/2006
		s_bAudioTemplatesModified = false;
		return true;
	}


	Asura_SaveBlock xSaveBlock;

	//Attempt to save the line templates in an order that will result in a balanced tree on reload. RAR 02/11/2006
	if(Asura_Dialogue_Storage::GetNumberOfLineTemplates())
	{
		for(Asura_TemplateHashTree_SaveIterator xIt(Asura_Dialogue_Storage::GetNumberOfLineTemplates());!xIt.Done();xIt.Next())
		{
			const Asura_Dialogue_Line_Template* pxTemplate = Asura_Dialogue_Storage::GetLineTemplateByIndex(xIt.GetCurrent());
			Asura_Chunk_Dialogue_Line_Template::Create(pxTemplate,&xSaveBlock);
		}
	}

	//add other global dialogue settings
	Asura_Chunk_Dialogue_ToolsGlobalSettings::Create(&xSaveBlock);
 
	//write out the save block

	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(g_szAudioTemplateFilename);
	bool bSuccess = xSaveBlock.SaveFile(g_szAudioTemplateFilename,true);
    if(bSuccess)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.",g_szAudioTemplateFilename);
		Tools_LoadedFileList::UpdateModifiedTime(g_szAudioTemplateFilename);
		s_bAudioTemplatesModified = false;
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s",g_szAudioTemplateFilename);
	}

	return bSuccess;	
}

void Tools_Dialogue_System::ToggleOverlay()
{
	if(OverlayIsOn())
	{
		s_bOverlayShouldBeOpen = false;
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DIALOGUE);
	}
	else
	{
		s_bOverlayShouldBeOpen = true;
		CreateOverlay();
	}
}

bool Tools_Dialogue_System::OverlayIsOn()
{
	return Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_DIALOGUE);
}

void Tools_Dialogue_System::CreateOverlay()
{
	if(OverlayIsOn()) return;

	Asura_Overlay* pxOverlay = new Tools_Overlay_Dialogue("Dialogue",false,&s_xOverlayPosition);
	Asura_Vector_2 xOffset(1.0f,3.0f);
	AddOverlayControls(pxOverlay,xOffset);
	pxOverlay->m_uID = TOOLS_OVERLAY_DIALOGUE;
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay);

	//If this is the first time , then do a verify
	static bool s_bFirstTime = true;
	if(s_bFirstTime)
	{
		s_bFirstTime = false;
		Verify(true);
	}
}

void Tools_Dialogue_System::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	//Edit Dialogue Event
	pxDialog = new Asura_Function_Dialog("View Events",xOffset,EditEvent);
	pxDialog->SetHelpText("Creates an overlay that lists all dialogue events.\nAlso used for editing,creating and previewing dialogue events.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("View Lines",xOffset,EditLine);
	pxDialog->SetHelpText("Creates an overlay that lists all dialogue lines.\nAlso used for previewing dialogue lines.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("View Templates",xOffset,EditAudioTemplate);
	pxDialog->SetHelpText("Creates an overlay that lists all audio templates.\nAlso used for editing andcreating templates.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reload Lines",xOffset,ReloadDialogueLines);
	pxDialog->SetHelpText("Reloads the dialogue lines from the unicode text files.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Update HashToPath.asr",xOffset,UpdateHashToPathTree);
	pxDialog->SetHelpText("Searches for all speech wav files and stores the result in HashToPath.asr.\nThis is required for tools to find the correct wav files in cases where  the dialogue line IDs don't contain any path information.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("List Lines In Exports",xOffset,ListDialogueLinesInExports);
	pxDialog->SetHelpText("Searches all the english exports in a specified directory and subdirectories and writes out a list of all the dialogue lines that are used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Combine Overlays",xOffset,&Tools_DialogueEvent_Overlay::s_bCombineOverlays);
	pxDialog->SetHelpText("If ticked then the event and audio template overlays will be combined into one.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("PS2 Sample Rate",xOffset,&s_uDefaultSampleRatePS2, 1,99999,1,"%d",&s_bGlobalSettingsChanged);
	pxDialog->SetHelpText("The default maximum sample rate for dialogue files exported to PS2");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("PSP Sample Rate",xOffset,&s_uDefaultSampleRatePSP, 1,99999,1,"%d",&s_bGlobalSettingsChanged);
	pxDialog->SetHelpText("The default maximum sample rate for dialogue files exported to PSP");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Verify",xOffset,Verify);
	pxDialog->SetHelpText("Checks for various dialogue system errors.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

}


void Tools_Dialogue_System::CloseOverlays()
{
	Tools_PickDialogueEvent_Overlay::CloseOverlay();
	Tools_PickDialogueLine_Overlay::CloseOverlay();
	Tools_PickDialogueLineTemplate_Overlay::CloseOverlay();
	Tools_DialogueEvent_Overlay::CloseOverlay();
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DIALOGUE);
}


void Tools_Dialogue_System::EditEvent()
{
	//pick event
	Tools_PickDialogueEvent_Overlay::CreateOverlay();
}

void Tools_Dialogue_System::EditLine()
{
	Tools_PickDialogueLine_Overlay::CreateOverlay();
}

void Tools_Dialogue_System::EditAudioTemplate()
{
	Tools_PickDialogueLineTemplate_Overlay::CreateOverlay();
}

void Tools_Dialogue_System::SetFilesToCheckForExternalModification()
{
	int iJunkIndex;
	Asura_NameTree xFileTree;

	//scan through all events for files being used
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		const char* szFilename = pxEvent->GetFilename();

		if(!xFileTree.FindNodeIndexByName(szFilename,true,iJunkIndex))
		{
			Asura_String xFullFilename;
			GetFullFilename(szFilename,xFullFilename);

			Tools_LoadedFileList::TrackExternalModification(xFullFilename.GetString());
		}
	}

	Tools_LoadedFileList::TrackExternalModification(g_szAudioTemplateFilename);

}

void Tools_Dialogue_System::RegisterNewEvent(const class Asura_Dialogue_Event* pxEvent)
{
	int iJunk;
	s_xNewEventTree.FindNodeIndexByHash(pxEvent->GetHashID(),true,iJunk);
}

bool Tools_Dialogue_System::CanTransferEvent(const class Asura_Dialogue_Event* pxEvent,const char* szDestFile)
{
	//If this is a new event , then allow it
	int iJunk;
	if(s_xNewEventTree.FindNodeIndexByHash(pxEvent->GetHashID(),false,iJunk)) return true;

	const char* szSourceFile = pxEvent->GetFilename();

	//Ignore if the source and dest are the same
	if(!stricmp(szSourceFile,szDestFile)) return true;

	//make sure the source and destination files can be written to
	Asura_String xFullFilename;
	GetFullFilename(szSourceFile,xFullFilename);
	if(!Tools_SaveAsura::CanSaveFile(xFullFilename.GetString(),false))return false;

	GetFullFilename(szDestFile,xFullFilename);
	if(!Tools_SaveAsura::CanSaveFile(xFullFilename.GetString(),false))return false;

	return true;
}

Asura_Hash_ID Tools_Dialogue_System::CreateEvent(const char* szEventName,const List<Asura_Hash_ID>* pxLineList)
{
	
	Asura_Hash_ID uEventHash = 0;

	char szDefaultName[200] = "";
	if(pxLineList && pxLineList->size() && !szEventName)
	{
		//If an event is being made	from some lines ,then default to naming after one of those lines
		Asura_Hash_ID uLineHash = pxLineList->first_entry();
		Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(uLineHash);
		if(pxLine)
		{
			Asura_PrintfToBfr(szDefaultName,"%s",sizeof(szDefaultName),true,pxLine->GetName());
		}
		
	}

	GetString_Overlay xStringOverlay("Name new event",szDefaultName);

	if(szEventName && *szEventName)
	{
		//confirm that the event doesn't already exist
		uEventHash = Asura_GetHashID(szEventName);
		if(Asura_Dialogue_Storage::GetEventByHash(uEventHash))
		{
			ShowMessageOverlay("An event already exists using the same hash ID.");
			return 0;
		}
	}
	else
	{

		//no name has been given , so ask for one
		do
		{
			if(!xStringOverlay.DoModal()) return 0;
			
			szEventName = xStringOverlay.GetString();
			if(!szEventName || !*szEventName) return 0;

			//make sure this event doesn't already exist
			uEventHash = Asura_GetHashID(szEventName);
			if(Asura_Dialogue_Storage::GetEventByHash(uEventHash))
			{
				ShowMessageOverlay("An event already exists using the same hash ID.");
				continue;
			}

			break;
			
		}while(true);
	}

	//Determine the file to use. Go for the most recently selected one  ,or ask if none has been selected yet.
	Asura_String xFile = GetMostRecentFile();
	if(xFile.IsBlank())
	{
		ListSelection_Overlay xFileOverlay("Select file for new event");
	
		Asura_NameTree xNameTree;
		int iJunkIndex;

		//add level specific file if a level is being edited
		if(Tools_SaveAsura::GetFileType() == AsuraFile_Environment)
		{
			xNameTree.FindNodeIndexByName(GetLevelSpecificFile(),true,iJunkIndex);
			xFileOverlay.AddItem(GetLevelSpecificFile());
		}

		//add the names of files used by existing events
		for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
		{
			const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
			const char* szFilename = pxEvent->GetFilename();

			if(szFilename && *szFilename)
			{
				if(!xNameTree.FindNodeIndexByName(szFilename,true,iJunkIndex))
				{
					xFileOverlay.AddItem(szFilename);
				}
			}
		}

		xFileOverlay.AddItem("*New*");
		if(!xFileOverlay.DoModal()) return 0;

		xFile = xFileOverlay.GetSelectionString();

		if(!strcmp(xFile.GetString(),"*New*"))
		{
			//ask for a name for a new file
			GetString_Overlay xStringOverlay("New Filename (without path/extension):","");
			if(!xStringOverlay.DoModal()) return 0;
			
			xFile = xStringOverlay.GetString();
			if(xFile.IsBlank()) return 0;

			if(!stricmp(xFile.GetString(),Tools_Dialogue_System::GetLevelSpecificFile()))
			{
				//don't allow 'Level' to be picked.
				ShowMessageOverlay("%s is reserved for the level specific file",Tools_Dialogue_System::GetLevelSpecificFile());
				return 0;
			}
			
		}
	}
	
	bool bAlreadyExists;
	Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::CreateEvent(szEventName, bAlreadyExists);
	Asura_Assert(pxEvent && !bAlreadyExists,"Error creating dialogue event");
	//Set the initial file
	pxEvent->SetFilename(xFile.GetString());
	SetMostRecentFile(xFile.GetString());

	if(pxLineList && pxLineList->size())
	{
		//add the listed samples to the new event
		Asura_Dialogue_PlayableIDListItem* pxPlayables = new Asura_Dialogue_PlayableIDListItem[pxLineList->size()];
		int iIndex = 0;
		for(CLIF<Asura_Hash_ID> xLineLIF(pxLineList);!xLineLIF.done();xLineLIF.next())
		{
			pxPlayables[iIndex].SetDefaults();
			pxPlayables[iIndex].m_xID.m_eType = ASURA_DIALOGUE_PLAYABLE_LINE;
			pxPlayables[iIndex].m_xID.m_uHashID = xLineLIF();
			iIndex++;
		}

		pxEvent->SetPlayableIDList(pxPlayables,pxLineList->size());
		delete [] pxPlayables;
	}


	SetModified(pxEvent);
	RegisterNewEvent(pxEvent);

	Tools_PickDialogueEvent_Overlay::InvalidateEventList();
	Tools_PickDialogueLine_Overlay::InvalidateEventList();

	//edit the new event
	Tools_DialogueEvent_Overlay::EditEvent(uEventHash);
	return uEventHash;
}

Asura_Hash_ID Tools_Dialogue_System::CreateLineTemplate(const char* szName /*= 0*/)
{
	
	Asura_Hash_ID uTemplateHash = 0;
	GetString_Overlay xStringOverlay("Name new template","");

	if(szName && *szName)
	{
		//confirm that the event doesn't already exist
		uTemplateHash = Asura_GetHashID(szName);
		if(Asura_Dialogue_Storage::GetLineTemplateByHash(uTemplateHash))
		{
			ShowMessageOverlay("A template already exists using the same hash ID.");
			return 0;
		}
	}
	else
	{
		//no name has been given , so ask for one
		do
		{
			if(!xStringOverlay.DoModal()) return 0;
			
			szName = xStringOverlay.GetString();
			if(!szName || !*szName) return 0;

			//make sure this template doesn't already exist
			uTemplateHash = Asura_GetHashID(szName);
			if(Asura_Dialogue_Storage::GetLineTemplateByHash(uTemplateHash))
			{
				ShowMessageOverlay("A template already exists using the same hash ID.");
				continue;
			}

			break;
			
		}while(true);
	}

	bool bAlreadyExists;
	Asura_Dialogue_Storage::CreateLineTemplate(szName, bAlreadyExists);

	SetAudioTemplatesModified();

	Tools_PickDialogueLineTemplate_Overlay::InvalidateLineTemplateList();

	//edit the new template
	Tools_DialogueEvent_Overlay::EditLineTemplate(uTemplateHash);
	
	//see if any dialogue lines were waiting for this to be created
	UpdateReferencesToTemplates();

	return uTemplateHash;
}



bool Tools_Dialogue_System::DeleteEvent(Asura_Hash_ID uHash,bool bConfirm)
{
	Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(uHash);
	if(!pxEvent) return false;

	Asura_Dialogue_PlayableID xID;
	xID.m_uHashID = uHash;
	xID.m_eType = ASURA_DIALOGUE_PLAYABLE_EVENT;

	if(bConfirm)
	{
		//See how many events use this
		int iEventCount = 0;
		for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);

			u_int uNumPlayable = pxEvent->GetNumPlayableID();
			for(u_int j=0;j<uNumPlayable;j++)
			{
				if(pxEvent->GetPlayableID(j).m_xID == xID)
				{
					iEventCount++;
					break;
				}
			}
		}

		//Confirm deletion
		char szEventCount[100];
		Asura_PrintfToBfr(szEventCount,"It is used by %d event(s).",sizeof(szEventCount),true,iEventCount);

		if(!ShowYesNoOverlay("Delete event '%s'?\n%s",pxEvent->GetName(),iEventCount ? szEventCount : ""))
		{
			return false;
		}
	}

	//Delete the tab for this event (if any)
	if(!Tools_DialogueEvent_Overlay::CloseEventTab(uHash))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to delete %s",pxEvent->GetName());
		return false;
	}


	//Go through the events again removing any references to this event
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		if(pxEvent->RemovePlayableID(xID))
		{
			Tools_Dialogue_System::SetModified(pxEvent);

			//Update the tab for this event if it was being edited
			Tools_DialogueEvent_Overlay::UpdateEventFromAsuraData(pxEvent->GetHashID());
		}
	}

	//Remove any references to the event from entities.
	Asura_ServerEntity_DialogueTrigger::HandleEventDeletion(pxEvent);
	Asura_ServerEntity_DialogueController::HandleEventDeletion( pxEvent );

	//Remove references to this from other stuff (TBD)
	Tools_PickDialogueEvent_Overlay::InvalidateEventList();
	Tools_PickDialogueLine_Overlay::InvalidateEventList();

	//And from any currently playing sounds
	Asura_Dialogue_System::StopAll();

	Tools_Dialogue_System::SetModified(pxEvent);
	
	Asura_Dialogue_Storage::DeleteEvent(pxEvent);
	return true;

}

bool Tools_Dialogue_System::DeleteLineTemplate(Asura_Hash_ID uHash,bool bConfirm)
{
	Asura_Dialogue_Line_Template* pxTemplate = Asura_Dialogue_Storage::GetLineTemplateByHash(uHash);
	if(!pxTemplate) return false;

	if(bConfirm)
	{
		//See how many lines use this
		int iLineCount = 0;
		for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfLines();i++)
		{
			Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByIndex(i);
			if(pxLine->GetTemplate() == pxTemplate)
			{
				iLineCount++;
				break;
			}
		}

		//Confirm deletion
		if(iLineCount)
		{
			//prevent deletion of templates that are in use
			ShowMessageOverlay("Deletion of template %s aborted.\nIt is used by %d lines.",pxTemplate->m_xName.GetString(),iLineCount);
			return false;
		}
		else
		{
			if(!ShowYesNoOverlay("Delete template '%s'?",pxTemplate->m_xName.GetString()))
			{
				return false;
			}
		}
	}

	//Delete the tab for this template (if any)
	if(!Tools_DialogueEvent_Overlay::CloseLineTemplateTab(uHash))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to delete %s",pxTemplate->m_xName.GetString());
		return false;
	}



	//Remove references to this from other stuff
	Tools_PickDialogueLineTemplate_Overlay::InvalidateLineTemplateList();


	Tools_Dialogue_System::SetAudioTemplatesModified();
	
	Asura_Dialogue_Storage::DeleteLineTemplate(pxTemplate);
	return true;

}



void Tools_Dialogue_System::LoadAudioTemplates()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Dialogue_System::LoadAudioTemplates" );
	LoadAsuraFile(g_szAudioTemplateFilename);
}


bool Tools_Dialogue_System::DoesEventContainCharacter(const class Asura_Dialogue_Event* pxEvent,Asura_Hash_ID uCharacterID)
{
	Asura_Assert(pxEvent,"");

	u_int uNumPlayable = pxEvent->GetNumPlayables();

	for(u_int i=0;i<uNumPlayable;i++)
	{
		const Asura_Dialogue_Playable* pxPlayable = pxEvent->GetPlayable(i);
		if(pxPlayable->IsEvent())
		{
			//search through child event
			const Asura_Dialogue_Event* pxSubEvent = (const Asura_Dialogue_Event*) pxPlayable;
			if(DoesEventContainCharacter(pxSubEvent,uCharacterID)) return true;
		}
		else
		{
			const Asura_Dialogue_Line* pxLine = (const Asura_Dialogue_Line*) pxPlayable;
			if(pxLine->GetCharacter() == uCharacterID) return true;
		}
	}

	return false;
}

//enumeration of headings for dialogue line file
enum
{
	DIALOGUELINE_ID,
	DIALOGUELINE_CHARACTER,
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	DIALOGUELINE_FACETYPE,
	DIALOGUELINE_BASEFACEPOSE,
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	DIALOGUELINE_AUDIOTEMPLATE,
	DIALOGUELINE_ANIMATION,
	DIALOGUELINE_FIRSTLANGUAGE,

	DIALOGUELINE_NUMHEADINGS
};

//text to search for in headings
static const Asura_UnicodeChar* g_awszDialogueLineHeadings[DIALOGUELINE_NUMHEADINGS] =
{
	L"Name/Unique ID",
	L"Character",
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	L"Face-Type",
	L"Base Face-Pose",
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	L"Audio Template",
	L"Animation",
	L"English",
	
};



void Tools_Dialogue_System::LoadDialogueLines()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Dialogue_System::LoadDialogueLines" );

	//find local files
	List<Asura_String> xFileList;

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER(
			"Tools_Dialogue_System::LoadDialogueLines - find local files" );

		Tools_FileSystem::FindFiles("Dialogue\\*.txt", xFileList);
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER(
			"Tools_Dialogue_System::LoadDialogueLines - find network files" );

		#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
		//find network files
		char szNetSearch[MAX_PATH+20];
		Asura_FileAccess::PlatformGetNetworkDir("Dialogue",szNetSearch,MAX_PATH);
		strcat(szNetSearch,"\\*.txt");

		Tools_FileSystem::FindFiles(szNetSearch, xFileList);
		#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	}

	Asura_NameTree xNameTree;

	//add characters.txt to tree so it doesn't get picked up
	int iJunkIndex;
	xNameTree.FindNodeIndexByName("Characters.txt",true,iJunkIndex);

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER(
			"Tools_Dialogue_System::LoadDialogueLines - load files" );

		//load the files in the list
		for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
		{
			//make sure this file hasn't already been dealt with
			const char* szName = Tools_FileSystem::StripPath(xFileLIF().GetString());

			if(xNameTree.FindNodeIndexByName(szName,true,iJunkIndex)) continue;

			LoadDialogueLines(xFileLIF().GetString());
		}
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER(
			"Tools_Dialogue_System::LoadDialogueLines - make character name list" );

		//copy character names into a sorted array
		int iNumCharacters = s_xCharacterNameTree.GetNumberOfNodes();
		if(iNumCharacters)
		{
			s_pszCharacterNames = new const char*[iNumCharacters];
			s_xCharacterNameTree.MakeSortedNameList(s_pszCharacterNames);

			s_puCharacterHashes = new Asura_Hash_ID[iNumCharacters];
			for(int i=0;i<iNumCharacters;i++)
			{
				s_puCharacterHashes[i] = Asura_GetHashID(s_pszCharacterNames[i]);
			}
		}
	}
}

void Tools_Dialogue_System::LoadDialogueLines(const char* szFilename)
{
	Asura_Assert(s_pxThis,"Tools_Dialogue_System not allocated");

	Tools_UnicodeTabbedStringTable xTable(szFilename);
	if(xTable.IsError())
	{
		Asura_Assertf(0,("Error loading %s for dialogue lines",szFilename));
		return;
	}

	//find all the required columns
	int aiColumns[DIALOGUELINE_NUMHEADINGS];

	for(int i=0;i<DIALOGUELINE_NUMHEADINGS;i++)
	{
		aiColumns[i] = -1;
		for(int j=0;j<xTable.GetWidth();j++)
		{
			const Asura_UnicodeChar* wszEntry = xTable.GetEntry(0,j);
			if( wszEntry && !wcsicmp(wszEntry,g_awszDialogueLineHeadings[i]) )
			{
				//found heading
				aiColumns[i] = j;
				break;
			}
		}
		if(aiColumns[i] == -1)
		{
			switch(i)
			{
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
				case DIALOGUELINE_FACETYPE:
				case DIALOGUELINE_BASEFACEPOSE:
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
				case DIALOGUELINE_ANIMATION:
				{
					// These headings are optional...
					break;
				}

				default:
				{
					//failed to find a required heading
					Asura_Assertf(0,("Failed to find heading '%S' in dialogue file '%s'",g_awszDialogueLineHeadings[i],szFilename));
					return;
				}
			}		// End of switch.
		}
	}
	char szBuffer[200];
	const Asura_UnicodeChar* wszEntry;
	
	//Check the remaining columns , and determine which language each corresponds to
	int* piColToLang = new int[xTable.GetWidth()];
	u_int uNumLanguages = 0;
	for(int i=0;i<xTable.GetWidth();i++)
	{
		piColToLang[i] = -1;
		if(i >= aiColumns[DIALOGUELINE_FIRSTLANGUAGE])
		{
			//convert column heading from unicode
			wszEntry = xTable.GetEntry(0,i);
			if(!wszEntry || !*wszEntry) continue;

			Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);

			//compare against each language name
			for(int j=0;j<ASURA_LT_L_NUM_LANGUAGES;j++)
			{
				if(!stricmp(Asura_LocalisedText::GetNameForLang((Asura_LocalisedText_Language)j),szBuffer))
				{
					piColToLang[i] = j;
					uNumLanguages++;
					break;
				}
			}
		}
	}
	//Should have managed to find at least english by this point
	Asura_Assert(uNumLanguages > 0,"Should have found at least one language");

	if( s_pfnFindExtraDialogueHeadingsCallback )
	{
		s_pfnFindExtraDialogueHeadingsCallback(xTable, szFilename);
	}
	
	//Allocate array for passing to the SetText() function
	Asura_Dialogue_Text* pxLocalisedText = new Asura_Dialogue_Text[uNumLanguages];


	char szFullWavFileName[MAX_PATH];
	
	//process the rest of the lines
	for(int i=1;i<xTable.GetHeight();i++)
	{
		//get the ID
		wszEntry = xTable.GetEntry(i,aiColumns[DIALOGUELINE_ID]);
		if(!wszEntry || !*wszEntry) continue;

		//ID needs to be converted from unicode
		Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);


		//convert any /'s in the buffer to \'s 
		for(char* pcPos = szBuffer;*pcPos;pcPos++)
		{
			if(*pcPos == '/') *pcPos = '\\';
		}

		Asura_String xLineName,xWavName;
		const char* szLineName = szBuffer;
		const char* szWavName = szBuffer;

		if(s_pxThis->Project_GetLineAndWavName(szBuffer,xLineName,xWavName))
		{
			szLineName = xLineName.GetString();
			szWavName = xWavName.GetString();

		}

		
		//make sure a matching line doesn't already exist
		Asura_Hash_ID uLineHash = Asura_GetHashID(szLineName);
		const Asura_Dialogue_Line* pxExistingLine = Asura_Dialogue_Storage::GetLineByHash(uLineHash);
		if(pxExistingLine)
		{
			Asura_Assertf( 0, ("ID of dialogue line clashes with a previous line.\nDialogue File : %s.\nNew ID: %s.\nExisting ID: %s comes from file: %s\n",
						   szFilename, szLineName, pxExistingLine->GetName(), pxExistingLine->GetSourceDialogueFilename() ) );
			continue;
		}
	
		bool bAlreadyExists;
		Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::CreateLine(szLineName, bAlreadyExists);
		Asura_Assert(pxLine && !bAlreadyExists, "Error createing dialogue line");

		// save which dialogue file this line of dialogue came from
		pxLine->SetSourceDialogueFilename( szFilename );

		//get the full (relative) wavname
		Asura_Hash_ID uWavHash = Asura_GetHashID(szWavName);

		Tools_Dialogue_HashToPath* pxHashToPath = s_xHashToPathTree.GetDataByHash(uWavHash);
		if(pxHashToPath)
		{
			//use filename from hash to path tree
			Asura_PrintfToBfr(szFullWavFileName,"%s",sizeof(szFullWavFileName),true,pxHashToPath->m_xFilename.GetString());
		}
		else
		{
			//guess filename
			Asura_PrintfToBfr(szFullWavFileName,"Sounds\\Speech\\%s\\%s.wav",sizeof(szFullWavFileName),true,Asura_LocalisedText::GetTwoCharCodeForLang(ASURA_LT_L_ENGLISH),szBuffer);
		}

		pxLine->SetSoundFilename(szFullWavFileName);

		//character
		wszEntry = xTable.GetEntry(i,aiColumns[DIALOGUELINE_CHARACTER]);
		if(wszEntry && *wszEntry)
		{
			Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);
			pxLine->SetCharacter(szBuffer);		

			//add to list of all characters
			int iJunkIndex;
			s_xCharacterNameTree.FindNodeIndexByName(szBuffer,true,iJunkIndex);

		}

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		// Face-type
		if(aiColumns[DIALOGUELINE_FACETYPE] != -1)
		{
			wszEntry = xTable.GetEntry(i,aiColumns[DIALOGUELINE_FACETYPE]);
			if(wszEntry && *wszEntry)
			{
				Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);
				pxLine->SetFaceType(szBuffer);		
			}
		}

		// Face-type
		if(aiColumns[DIALOGUELINE_BASEFACEPOSE] != -1)
		{
			wszEntry = xTable.GetEntry(i,aiColumns[DIALOGUELINE_BASEFACEPOSE]);
			if(wszEntry && *wszEntry)
			{
				Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);
				pxLine->SetBaseFacePose(szBuffer);		
			}
		}
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

		//audio template
		wszEntry = xTable.GetEntry(i,aiColumns[DIALOGUELINE_AUDIOTEMPLATE]);
		if(wszEntry && *wszEntry)
		{
			Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);
			pxLine->SetTemplate(szBuffer);
		}
		else
		{
			//There is already code that sets the 'default' template name in the constructor , but
			//SetTemplate() needs to be called so that it will actually search for the template.
			//RAR 27/04/2009
			pxLine->SetTemplate("default");
		}

		//animation
		if(aiColumns[DIALOGUELINE_ANIMATION] != -1)
		{
			wszEntry = xTable.GetEntry(i,aiColumns[DIALOGUELINE_ANIMATION]);
			if(wszEntry && *wszEntry)
			{
				Asura_PrintfToBfr(szBuffer,"%S",sizeof(szBuffer),true,wszEntry);
				pxLine->SetAnimation(s_pxThis->ParseAnimationString(szBuffer));
			}
		}


		//get the text for each language
		u_int uLang = 0;
		for(int j=aiColumns[DIALOGUELINE_FIRSTLANGUAGE] ;j<xTable.GetWidth(); j++)
		{
			if(piColToLang[j] != -1)
			{
				pxLocalisedText[uLang].m_iLanguage = piColToLang[j];
				//Casting away constness to avoid having to make another copy of the text (in addition to the one
				//that SetText will make anyway)
				pxLocalisedText[uLang].m_wszText = const_cast<Asura_UnicodeChar*>(xTable.GetEntry(i,j));
				uLang++;
			}
		}
		Asura_Assert(uLang == uNumLanguages,"");

		pxLine->SetText(pxLocalisedText,uNumLanguages);


		if( s_pfnPostProcessDialogueLineCallback )
		{
			s_pfnPostProcessDialogueLineCallback(xTable, i, pxLine);
		}
	}

	delete [] piColToLang;
	delete [] pxLocalisedText;

}

void Tools_Dialogue_System::LoadDialogueFiles()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Dialogue_System::LoadDialogueFiles" );

	Tools_FileSystem::LoadAsuraFiles("Dialogue\\*.dlg");

	//Now need to go through all the events and sort out the pointers to the playables
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		pxEvent->UpdatePlayableListFromIDList();
	}
}

void Tools_Dialogue_System::ReloadDialogueLines()
{
	//stop all playing dialogue , since their pointers to the lines will be invalid
	Asura_Dialogue_System::StopAll();

	//clear out old lines.
	Asura_Dialogue_Storage::ResetLines();

	//and the list of characters used by those lines
	s_xCharacterNameTree.Reset();
	delete [] s_pszCharacterNames;
	s_pszCharacterNames = 0;
	delete [] s_puCharacterHashes;
	s_puCharacterHashes = 0;

	LoadDialogueLines();
	//update all events to look at the new lines
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		pxEvent->UpdatePlayableListFromIDList();
	}

	InvalidateCharacterMappings();

	Tools_Timed_Message_System::AddMessage(0xffffff,"Reloaded dialogue lines.");

}

void Tools_Dialogue_System::LoadLevelSpecificFile()
{
	if(Tools_SaveAsura::GetFileType() != AsuraFile_Environment) return;

	Asura_String xFullFilename;
	GetFullFilename(GetLevelSpecificFile(),xFullFilename);

	LoadAsuraFile(xFullFilename.GetString());

	//Fix up the playable pointers. No convenient way of determining the new ones just yet ,so we do them all again
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		pxEvent->UpdatePlayableListFromIDList();
	}

	if( s_pfnPostLoadLevelSpecificFileCallback )
	{
		s_pfnPostLoadLevelSpecificFileCallback();
	}
}


void Tools_Dialogue_System::UpdateReferencesToTemplates()
{
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfLines();i++)
	{
		Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByIndex(i);
		pxLine->FindTemplate();		
	}
}

const char* Tools_Dialogue_System::GetCharacterNameByHash(Asura_Hash_ID uHash)
{
	for(int i=0;i<GetNumberOfCharacters();i++)
	{
		if(s_puCharacterHashes[i] == uHash)
		{
			return s_pszCharacterNames[i];
		}
	}
	return 0;
}

void Tools_Dialogue_System::Verify(bool bAutomatic)
{
	if(bAutomatic && !Tools_MessageLog::ShouldShowForDialogue())
	{
		return;
	}

	//Clear out existing messages
	Tools_MessageLog::DeleteMessages(Tools_MessageLog::DIALOGUE,0);
	//Get current position to see if any errors were logged
	u_int uInitialPos = Tools_MessageLog::GetNextID();

	//check each event
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);
		Asura_Hash_ID uEventHash = pxEvent->GetHashID();

		bool bLevelSpecific = Tools_Dialogue_System::IsLevelSpecific(pxEvent);
		
		//Make sure the playables exist
		int iMissing = 0;
		for(u_int j=0;j<pxEvent->GetNumPlayableID();j++)
		{
			Asura_Dialogue_PlayableID xID = pxEvent->GetPlayableID(j).m_xID;

			if(xID.m_eType == ASURA_DIALOGUE_PLAYABLE_LINE)
			{
				if(!Asura_Dialogue_Storage::GetLineByHash(xID.m_uHashID)) iMissing++;
			}
			else
			{
				const Asura_Dialogue_Event* pxChildEvent = Asura_Dialogue_Storage::GetEventByHash(xID.m_uHashID);
				if(pxChildEvent)
				{
					//make sure global events aren't using level specific events
					if(Tools_Dialogue_System::IsLevelSpecific(pxChildEvent) && !bLevelSpecific)
					{
						Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Using level specific",new Tools_MessageLogAction_DialogueEvent(uEventHash),
													"'%s' is not level specific but uses a level specific event ('%s')",
													pxEvent->GetName(),pxChildEvent->GetName());
					}
				}
				else
				{
					iMissing++;
				}
			}
		}
		if(iMissing)
		{
			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Missing playable",new Tools_MessageLogAction_DialogueEvent(uEventHash),
										"'%s' uses %d events/lines that can't be found",
		   								pxEvent->GetName(),iMissing);
		}

		//check message blocks used
		int iMissingBlocks = 0;
		
		Asura_Hash_ID uMessageID = pxEvent->GetInitialMessageID();
		if(uMessageID && !pxEvent->GetMessageFromID(uMessageID))
		{
			iMissingBlocks++;
		}

		uMessageID = pxEvent->GetFinalMessageID();
		if(uMessageID && !pxEvent->GetMessageFromID(uMessageID))
		{
			iMissingBlocks++;
		}

		for(u_int j=0;j<pxEvent->GetNumPlayableID();j++)
		{
			uMessageID = pxEvent->GetPlayableID(j).m_uStartMessageID;
			if(uMessageID && !pxEvent->GetMessageFromID(uMessageID))
			{
				iMissingBlocks++;
			}
			uMessageID = pxEvent->GetPlayableID(j).m_uEndMessageID;
			if(uMessageID && !pxEvent->GetMessageFromID(uMessageID))
			{
				iMissingBlocks++;
			}

		}

		if(iMissingBlocks)
		{
			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Missing message",new Tools_MessageLogAction_DialogueEvent(uEventHash),
										"'%s' uses %d message blocks that don't exist.",
		   								pxEvent->GetName(),iMissingBlocks);
		}
			
		//check contents of message blocks
		if(!bLevelSpecific)
		{
			for(u_int j=0;j<pxEvent->GetMessageStoreSize();j++)
			{
				const Asura_Dialogue_Message* pxBlock = pxEvent->GetMessageFromStore(j);
				for(u_int k=0;k<pxBlock->m_xMessageBlock.GetNumberOfMessagesInBlock();k++)
				{
					const Asura_StaticMessage* pxMessage = pxBlock->m_xMessageBlock.GetMessagePointer(k);

					//non level specific events shouldn't refer to tools created guids
					if(pxMessage->m_uSendTo >= ASURA_GUID_TOOLSCREATEDENTITY_START && pxMessage->m_uSendTo <= ASURA_GUID_TOOLSCREATEDENTITY_END)
					{
						Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Invalid message",new Tools_MessageLogAction_DialogueEvent(uEventHash),
													"Message %d of block '%s' in '%s' targets a level specific guid.",
					   								k,pxBlock->m_xName.GetString(),pxEvent->GetName());
					}
				}
			}
		}

	}

	//check each line
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfLines();i++)
	{
		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByIndex(i);
		//Asura_Hash_ID uLineHash = pxLine->GetHashID();

		if(!pxLine->GetTemplate())
		{
			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Missing template",
										"Can't find template '%s' for line '%s'",
		   								pxLine->GetTemplateName(),pxLine->GetName());
		}
	}

	//check the line templates
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfLineTemplates();i++)
	{
		const Asura_Dialogue_Line_Template* pxLineTemplate = Asura_Dialogue_Storage::GetLineTemplateByIndex(i);
		const char* szMixer = pxLineTemplate->m_xMixingChannelName.GetString();
		Asura_Hash_ID uTemplateHash = pxLineTemplate->m_uHashID;

		if(!Asura_Sound_Mixer_System::FindChannel(Asura_GetHashID(szMixer)))
		{
			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Missing mixer",new Tools_MessageLogAction_DialogueLineTemplate(uTemplateHash),
										"Can't find mixer '%s' for line template '%s'",
		   								szMixer,pxLineTemplate->m_xName.GetString());
		}
	}

	//check dialogue triggers
	for(Asura_Hashed_List_It<Asura_ServerEntity_DialogueTrigger> xIt(Asura_ServerEntity_DialogueTrigger::GetList());!xIt.Done();xIt.Next())
	{
		Asura_ServerEntity_DialogueTrigger* pxEntity = xIt.GetCurrent();
		if(pxEntity->IsDestroyed()) continue;
		
		//check for triggers using missing events
		Asura_Hash_ID uEventID = pxEntity->GetEventHashID();
		if(uEventID && !Asura_Dialogue_Storage::GetEventByHash(uEventID))
		{
			Asura_Assert(pxEntity->ToolsData,"");

			Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Missing event",new Tools_MessageLogAction_GotoEntity(pxEntity->GetGuid()),
										"Trigger '%s' uses missing event 0x%x",
		   								pxEntity->ToolsData->Name.GetString(),uEventID);
		}
	}

	//search for static messages that refer to non-existant events/lines
	for(Tools_StaticMessageIterator xMessageIt(true);!xMessageIt.Done();xMessageIt.Next())
	{
		Asura_StaticMessage* pxMessage = xMessageIt.GetCurrent();
		//is it a logic trigger intput affecting message?
		if(pxMessage->m_usMsgID == ASURA_MSG_DIALOGUE_PLAYLINE ||
			pxMessage->m_usMsgID == ASURA_MSG_DIALOGUE_PLAYLINEONENTITY)
		{
			const Asura_Message_Dialogue_PlayLine* pxData = (const Asura_Message_Dialogue_PlayLine*) pxMessage->m_pData;
			const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::FindLine(pxData->m_uLineID);
			
			if(!pxLine)
			{
				Tools_StaticMessageID xMessageID;
				xMessageIt.GetCurrentMessageID(xMessageID);

				Asura_String xSenderName;
				xMessageID.GetSenderInfo(NULL, &xSenderName, NULL);


				Tools_MessageLog::AddMessage(Tools_MessageLog::DIALOGUE,"Missing line",new Tools_MessageLogAction_StaticMessage(xMessageID),
											"Play dialogue line message (sent by %s) uses missing line 0x%x",
											xSenderName.IsBlank() ? "*Unknown*" : xSenderName.GetString(),
											pxData->m_uLineID);
			}
			
		}
	}



	if(Tools_MessageLog::AreThereAnyMessages(Tools_MessageLog::DIALOGUE, 0, uInitialPos))
	{
		//Found some errors ,so show message log
		Tools_MessageLog::ShowOverlay(Tools_MessageLog::DIALOGUE,0,uInitialPos);
	}
	else if(!bAutomatic)
	{
		ShowMessageOverlay("No dialogue system errors found.");
	}
}


bool Tools_Dialogue_System::IsLevelSpecific(const class Asura_Dialogue_Event* pxEvent)
{
	return !stricmp(pxEvent->GetFilename(),GetLevelSpecificFile());
}

void Tools_Dialogue_System::PreMessageEditing()
{
	while(s_xMinimisedOverlays.size()) s_xMinimisedOverlays.delete_first_entry();

	//minimise all overlays associated with dialogue editing
	int aiOverlayIDs [] =
	{
		TOOLS_OVERLAY_DIALOGUE,
		TOOLS_OVERLAY_DIALOGUEEVENT1,
		TOOLS_OVERLAY_DIALOGUEEVENT2,
		TOOLS_OVERLAY_PICKDIALOGUEEVENT,
		TOOLS_OVERLAY_PICKDIALOGUELINE,
		TOOLS_OVERLAY_PICKDIALOGUELINETEMPLATE,
	};
	int iNumIDs = sizeof(aiOverlayIDs) / sizeof(int);

	for(int i=0;i<iNumIDs;i++)
	{
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(aiOverlayIDs[i]);
		if(pxOverlay && !pxOverlay->IsMinimised())
		{
			pxOverlay->SetMinimised(true,true);
			s_xMinimisedOverlays.add_entry(aiOverlayIDs[i]);
		}
	}

}

void Tools_Dialogue_System::PostMessageEditing()
{
	//restore the previously minimised overlays
	while(s_xMinimisedOverlays.size())
	{
		int iID = s_xMinimisedOverlays.first_entry();
		s_xMinimisedOverlays.delete_first_entry();

		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(iID);
		if(pxOverlay)
		{
			pxOverlay->SetMinimised(false,true);
		}
	}
}

void Tools_Dialogue_System::RenderSubtitle(float& fX,float& fY)
{
	if(Asura_Dialogue_System::IsSubtitle())
	{
		Asura_Text_Output::Print(fX,fY++,0xffffffff,0,"Dialogue : %S",g_awcSubtitleBuffer);
	}
}

void Tools_Dialogue_System::SetSubtitle(const Asura_UnicodeChar* wszText)
{
	wcsncpy(g_awcSubtitleBuffer,wszText,g_iSubtitleBufferSize);
	g_awcSubtitleBuffer[g_iSubtitleBufferSize-1] = 0;
}

void Tools_Dialogue_System::AddMessageOverlayControls_PlayLine(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	if(pxMessage->m_usMsgID != ASURA_MSG_DIALOGUE_PLAYLINE && pxMessage->m_usMsgID != ASURA_MSG_DIALOGUE_PLAYLINEONENTITY)
	{
		Asura_Assert(0,"Unexpected message type");
		return;
	}
	Asura_Assert(pxMessage->m_pData,"");

	Asura_Message_Dialogue_PlayLine* pxData = (Asura_Message_Dialogue_PlayLine*) pxMessage->m_pData;
	Asura_Dialog* pxDialog;

	
	//determine the text for the line selection control
	const char* szLine;
	if(pxData->m_uLineID)
	{
		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::FindLine(pxData->m_uLineID);
		if(pxLine)
		{
			szLine = pxLine->GetName();
		}
		else
		{
			szLine = "*Unknown*";
		}
	}
	else
	{
		szLine = "*Not Set*";
	}

	char szText[100];
	Asura_PrintfToBfr(szText,"Line : %s",sizeof(szText),true,szLine);

	
	pxDialog = new Asura_Function_Dialog(szText,xOffset,EditMessage_SelectLine,(int)pxMessage);
	pxDialog->SetHelpText("Select the dialogue line to play");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


}

void Tools_Dialogue_System::EditMessage_SelectLine(int iData)
{
	Asura_StaticMessage* pxMessage = (Asura_StaticMessage*) iData;
	Asura_Message_Dialogue_PlayLine* pxData = (Asura_Message_Dialogue_PlayLine*) pxMessage->m_pData;

	//show the dialogue line selection overlay
	pxData->m_uLineID = Tools_PickDialogueLine_Overlay::PickLine("Select line",pxData->m_uLineID);

	Tools_Message_Editing::UpdateMessageOverlay();
}

void Tools_Dialogue_System::AddFilesToExportListFromMessage(const struct Asura_StaticMessage* pxMessage)
{
	if(pxMessage->m_usMsgID != ASURA_MSG_DIALOGUE_PLAYLINE && pxMessage->m_usMsgID != ASURA_MSG_DIALOGUE_PLAYLINEONENTITY)
	{
		Asura_Assert(0,"Unexpected message type");
		return;
	}
	Asura_Assert(pxMessage->m_pData,"");
	
	//add the dialogue line used
	const Asura_Message_Dialogue_PlayLine* pxData = (const Asura_Message_Dialogue_PlayLine*) pxMessage->m_pData;
	if(pxData->m_uLineID)
	{
		Tools_Level_Export::AddDialogueLineToExportList(pxData->m_uLineID);
	}
}

int Tools_Dialogue_System::GetSampleRate(int iPlatform, const Asura_Dialogue_Line* pxLine)
{
	int iSampleRate = 0;
	//Only have settings for PS2/PSP. Any other platform just gets 0 , which will result in some default setting
	if(iPlatform == Tools_Level_Export::Export_PSP || iPlatform == Tools_Level_Export::Export_PS2)
	{
		bool bPS2 = (iPlatform == Tools_Level_Export::Export_PS2);
		
		//get the setting from the template if there is one
		if(pxLine->GetTemplate())
		{
			iSampleRate = bPS2 ? pxLine->GetTemplate()->m_usSampleRateForPS2 : pxLine->GetTemplate()->m_usSampleRateForPSP;
		}
		//otherwise go for the global setting
		if(!iSampleRate)
		{
			iSampleRate = bPS2 ? s_uDefaultSampleRatePS2 : s_uDefaultSampleRatePSP;
		}
	}
	return iSampleRate;
}


void Tools_Dialogue_System::UpdateHashToPathTree()
{
	//clear current tree.
	s_xHashToPathTree.Reset();

	//get a iist of all speech wav files
	char szSearch[MAX_PATH];
	List<Asura_String> xFileList;

	//check network
	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	const char* szNetPath = Asura_FileAccess::PlatformGetNetworkPath();
	
	if(szNetPath && *szNetPath)
	{
		Asura_PrintfToBfr(szSearch,"%s\\Sounds\\Speech\\%s\\*.wav",sizeof(szSearch),true,szNetPath,Asura_LocalisedText::GetTwoCharCodeForLang(ASURA_LT_L_ENGLISH));

		Tools_FileSystem::FindFiles(szSearch, xFileList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR);
	}
	#endif

	//and locally
 	Asura_PrintfToBfr(szSearch,"Sounds\\Speech\\%s\\*.wav",sizeof(szSearch),true,Asura_LocalisedText::GetTwoCharCodeForLang(ASURA_LT_L_ENGLISH));
 	Tools_FileSystem::FindFiles(szSearch, xFileList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR);


	//add all these to our hash tree
	char szBuffer[MAX_PATH];
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{
		//get the filename without path/extension to use for the hash ID
		strcpy(szBuffer,Tools_FileSystem::StripPath(xFileLIF().GetString()));

		char* pcDot = strrchr(szBuffer,'.');
		if(pcDot) *pcDot = 0;

		Asura_Hash_ID uHash = Asura_GetHashID(szBuffer);

		Tools_Dialogue_HashToPath* pxEntry = s_xHashToPathTree.AddNode(uHash);
		if(pxEntry)
		{
			pxEntry->m_uHash = uHash;
			pxEntry->m_xFilename = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xFileLIF().GetString());
		}
	}

	//Reload the dialogue lines , since some missing wav files may have been found now
	ReloadDialogueLines();

	//save the changes
	Asura_SaveBlock xSaveBlock;

	Asura_Chunk_Dialogue_ToolsHashToPath::Create(&xSaveBlock);
 
	//write out the save block

	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(g_szHashToPathFilename);
	
	bool bSuccess = xSaveBlock.SaveFile(g_szHashToPathFilename,false);
    if(bSuccess)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.",g_szHashToPathFilename);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s",g_szHashToPathFilename);
	}

}

const char* Tools_Dialogue_System::GetFullFilenameFromHash(Asura_Hash_ID uHash)
{
	const Tools_Dialogue_HashToPath* pxHashToPath = s_xHashToPathTree.GetDataByHash(uHash);
	return pxHashToPath ? pxHashToPath->m_xFilename.GetString() : 0;
}


void Tools_Dialogue_System::ListDialogueLinesInExports()
{
	Asura_Assert(s_pxThis,"");

	//get the directory to scan
	Tools_FileSystem_SelectFolderInfo xSelectFolder;
	xSelectFolder.InitialSelection = "";
	xSelectFolder.DisplayText = "Scan the exports in which directory?";

	if(!Tools_FileSystem::SelectFolder(xSelectFolder)) return;

	//get the file to save the output to
	Tools_FileSystem_DialogInfo xInfo;
	xInfo.DefaultPath = ".";
	xInfo.DefaultFilename = "UsedDialogueLines.txt";
	xInfo.FileFilter = "Text Files\0*.txt\0";
	xInfo.WindowTitle = "Output used dialogue lines to file...";

	const char* szFilename = Tools_FileSystem::GetSaveFilename(xInfo);
	if(!*szFilename) return;

	//attempt to open the output file
	FILE* pFile = fopen(szFilename,"wb");
	if(!pFile)
	{
		ShowMessageOverlay("Failed to write to %s",szFilename);
		return;
	}

	//get a list of all the .en files.
	char szSearch[MAX_PATH];
	Asura_PrintfToBfr(szSearch,"%s\\*.en",sizeof(szSearch),true,xSelectFolder.SelectedFolder);
	List<Asura_String> xFileList;
	Tools_FileSystem::FindFiles(szSearch,xFileList,TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR);

	if(!xFileList.size())
	{
		fclose(pFile);
		ShowMessageOverlay("Failed to find any .en files");
		return;
	}

	Asura_StringTree xLineNameTree;

	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{
		const char* const szExportedFile = xFileLIF().GetString();
		Asura_StringTree xLineNamePerFileTree;

		Tools_AsuraFileChunkLister xChunkLister;
		if(!xChunkLister.LoadAsuraFile(szExportedFile,false))
		{
			//TBD , complain , probably
			continue;
		}

		//scan for dialogue line chunks
		List<const Asura_Chunk_Header*> xChunkList;
		xChunkLister.ListChunksByID(ASURA_CHUNK_DIALOGUE_LINE,xChunkList);

		for(LIF<const Asura_Chunk_Header*> xChunkLIF(&xChunkList);!xChunkLIF.done(); xChunkLIF.next())
		{
			//the name should be just beyond the header
			const char* szLineID = (const char*)(xChunkLIF()+1);

			//add to the global list of lines
			int iJunk;
			xLineNameTree.FindNodeIndexByName(szLineID, true, iJunk);

			//add to the list of lines per file
			xLineNamePerFileTree.FindNodeIndexByName( szLineID, true, iJunk );
		}
		OutputDialogueExportDetails( xLineNamePerFileTree, szExportedFile, pFile );
	}

	const int iNumLines = xLineNameTree.GetNumberOfNodes();
	if ( !iNumLines )
	{
		fclose(pFile);
		ShowMessageOverlay("Failed to find any dialogue lines in the exported files");
		return;
	}

	OutputDialogueExportDetails( xLineNameTree, "All Files", pFile );

	ShowMessageOverlay("Found %d lines used by %d files",iNumLines,xFileList.size());

	fclose(pFile);
}

void Tools_Dialogue_System::OutputDialogueExportDetails( Asura_StringTree& xLines, const char* const szExportFileName, FILE*& pFile )
{
	const int iNumLines = xLines.GetNumberOfNodes();
	if (!iNumLines )
	{
		return;
	}

	//sort the list of IDs
	const char** pszLineList = new const char*[iNumLines];
	xLines.MakeSortedNameList(pszLineList);

	int iBufferSize = 4096;	
	Asura_UnicodeChar* pwcBuffer = new Asura_UnicodeChar[4096];

	//write unicode identifier.
	const Asura_UnicodeChar wcUnicodeHeader = 0xFEFF;
	fwrite(&wcUnicodeHeader,sizeof(Asura_UnicodeChar),1,pFile);

	Asura_PrintfToBfr( pwcBuffer, L"############################\r\n", iBufferSize,true, szExportFileName );
	fwrite( pwcBuffer, wcslen(pwcBuffer), sizeof(Asura_UnicodeChar), pFile );
	Asura_PrintfToBfr( pwcBuffer, L"File: '%S'\r\n", iBufferSize,true, szExportFileName );
	fwrite( pwcBuffer, wcslen(pwcBuffer), sizeof(Asura_UnicodeChar), pFile );
	Asura_PrintfToBfr( pwcBuffer, L"############################\r\n", iBufferSize,true, szExportFileName );
	fwrite( pwcBuffer, wcslen(pwcBuffer), sizeof(Asura_UnicodeChar), pFile );

	// dump everything to the text file
	for(int i=0;i<iNumLines;i++)
	{
		//write the ID
		Asura_String xNameToWrite = pszLineList[i];
		s_pxThis->Project_ModifyLineNameForLogFile(xNameToWrite);

		Asura_PrintfToBfr(pwcBuffer,L"\"%S\"\t",iBufferSize,true,xNameToWrite.GetString());
		fwrite(pwcBuffer,wcslen(pwcBuffer),sizeof(Asura_UnicodeChar),pFile);

		//attempt to get the text for the line
		Asura_Hash_ID uLineHash = Asura_GetHashID(pszLineList[i]);
		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(uLineHash);

		if(pxLine)
		{

			const Asura_UnicodeChar* pwcSource = pxLine->GetText();
			Asura_UnicodeChar* pwcDest = pwcBuffer;

			*pwcDest++ = L'\"';

			//need to double up any "'s
			if(pwcSource)
			{
				while(*pwcSource)
				{
					//make sure there is plenty of space left
					if(((pwcDest-pwcBuffer)) > (iBufferSize-10)) break;

						
					if(*pwcSource == L'\"')
					{
						*pwcDest++ = L'\"';
						*pwcDest++ = L'\"';
						*pwcSource++;
					}
					else
					{
						*pwcDest++ = *pwcSource++; 
					}
				}
			}
			*pwcDest++ = L'\"';
			*pwcDest++ = 0;
			fwrite(pwcBuffer,wcslen(pwcBuffer),sizeof(Asura_UnicodeChar),pFile);

			// JAT 28/09/07 output the source filename as well
			Asura_PrintfToBfr( pwcBuffer, L" Dialogue File:%S", iBufferSize, true, pxLine->GetSourceDialogueFilename() );
			fwrite(pwcBuffer,wcslen(pwcBuffer),sizeof(Asura_UnicodeChar),pFile);

			pwcDest = pwcBuffer;
			*pwcDest++ = L'\r';
			*pwcDest++ = L'\n';
			*pwcDest++ = 0;
			fwrite(pwcBuffer,wcslen(pwcBuffer),sizeof(Asura_UnicodeChar),pFile);

		}
		else
		{
			Asura_PrintfToBfr(pwcBuffer,L"\"**Not Found**\"\r\n",iBufferSize,true,pszLineList[i]);
			fwrite(pwcBuffer,wcslen(pwcBuffer),sizeof(Asura_UnicodeChar),pFile);
		}
	}

	delete [] pwcBuffer;
	delete [] pszLineList;
}


#endif // #ifdef ASURA_USE_DIALOGUE_SYSTEM
