//
// Tools_Mode_MetaMusic.h
//
// Created by Richard Rice - 20/01/04
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_METAMUSIC_H__
#define __TOOLS_MODE_METAMUSIC_H__

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Tools_Mode.h"

class Asura_MetaMusic_Handler;
class Asura_StringTree;

class Tools_Mode_MetaMusic : public Tools_Mode
{
	public:

		Tools_Mode_MetaMusic(){s_pxThis = this;}
		~Tools_Mode_MetaMusic(){s_pxThis = 0;}

		void InitialiseMode(void);
		void ShutdownMode(void);
		void Update(void);
		void RenderWorld(void);
		void RenderUI(void);

		void QuickSave(void);
 		bool IsModified();

		#ifdef ASURA_USE_METAMUSICZONES
		static const char* GetZoneNameFromID(u_int uID);
		static u_int SelectZone(const char* pszMessage,u_int uInitialID);
		#endif

		#ifdef ASURA_USE_STREAMING
		//Gets the meta music handler that contains the global tracks
		static Asura_MetaMusic_Handler* GetGlobalHandler();
		#endif


	protected :
		#ifdef ASURA_USE_METAMUSICZONES
		//If using meta music zones a project derived class can allocate
		//a derived  editor for it
		virtual class Tools_MetaMusicZone_Editor* AllocateZoneEditor(class Asura_Zone_Controller* pxController);
		#endif


	private:
		void TransferModeData(int mode);

		

		static void _CreateMusic(){s_pxThis->CreateMusic();}
		void CreateMusic();
		int CreateEmptyMusicTrack();

		static void _DeleteMusic(){s_pxThis->DeleteMusic();}
		void DeleteMusic();

		static void _Rename(){s_pxThis->Rename();}
		void Rename();

		static void _AddFile(){s_pxThis->AddFile();}
		void AddFile();

		static void _RemoveFile(){s_pxThis->RemoveFile();}
		void RemoveFile();

		static void _AlterState(){s_pxThis->AlterState();}
		void AlterState();

		static void _ExportTrack(){s_pxThis->ExportTrack();}
		void ExportTrack();

		static void _ImportTrack(){s_pxThis->ImportTrack();}
		void ImportTrack();

		static void _CreateMetaMusicTrigger(){s_pxThis->CreateMetaMusicTrigger();}
		void CreateMetaMusicTrigger();

	
	public :
		#ifdef ASURA_USE_METAMUSICZONES
		static void _StartZoneEditing(){s_pxThis->StartZoneEditing();}
		void StartZoneEditing();
		
		static void _EndZoneEditing(){s_pxThis->EndZoneEditing();}
		void EndZoneEditing();
		#endif

	private :

		static void _Play(){s_pxThis->Play();}
		void Play();
		static void Stop();
		static void ToAmbient();
		static void ToTension();
		static void ToAction();

		bool IsEnabled_Fades(int iState,int iType);
		static bool IsEnabled_FadeIn(int iState);
		static bool IsEnabled_FadeOut(int iState);
		static bool IsEnabled_Crossfade(int iState);


		void HandleKeyboard(void);
		
		void UpdateMusicNameList();
		void DeleteMusicNameList();

		void UpdateMusicIndexFromListIndex();
		void UpdateListIndexFromMusicIndex();

		void UpdateFilenameList();
		void DeleteFilenameList();
		
		void UpdateMusicPropertiesOverlay();
		
		void CreateModeOverlay(void);

		Asura_Vector_2 m_xModeOverlayPosition;
		Asura_Vector_2 m_xMusicPropOverlayPosition;


		Asura_Handle m_iMenuHandle;

		int m_iMetaMusicNameArraySize;
		const char** m_ppszMetaMusicNames;
		Asura_StringTree* m_pxMetaMusicNameTree;

		int m_iMusicIndex;
		int m_iMusicListIndex;//index into sorted list
		bool m_bMusicListIndexChanged;
		class Asura_MetaMusic* m_pxCurrentTrack;
		bool m_bModified;
		int m_iIntroLoopIndex;

		int m_iFilenameArraySize;
		char** m_ppszFilenames;
		int m_iFilenameListSel;
		int m_iFilenameListOffset;

		u_int m_uSelectedTab;

		float m_fFileVolume;
		bool m_bFileVolumeModified;

		#ifdef ASURA_USE_METAMUSICZONES
		class Tools_MetaMusicZone_Editor* m_pxZoneEditor;
		#endif

		#ifdef ASURA_USE_STREAMING
		bool m_bEditGlobalTracks;
		bool m_bEditGlobalTracksChanged;
		#endif

		//handler containing the tracks that are being edited		
		Asura_MetaMusic_Handler* m_pxHandler;
		

		static Tools_Mode_MetaMusic* s_pxThis;
};

#endif // #ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

#endif //__TOOLS_MODE_METAMUSIC_H__

//eof
