//
// Tools_Mode_Sound.h
//
// Created by Kevin Lea - 26/06/2001 16:12:54
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_SOUND_H__
#define __TOOLS_MODE_SOUND_H__

#include "Tools_Mode.h"
#include "Core/Asura_Maths.h"
#include "Sound/Asura_Phonon.h"
#include "Core/Asura_String.h"
#include "Misc/list_tem.hpp"

#define MAX_NO_OF_SOUND_FILES 100

class Tools_Phonon_Object;
class Asura_StringValuePair_DropDownList_Dialog;

class Tools_Mode_Sound : public Tools_Mode
{
	public:

		Tools_Mode_Sound()
		{
			Initialise();
		}
		~Tools_Mode_Sound()
		{
			Shutdown();
		}
		void Initialise(void);
	
		void Update(void);
		void RenderWorld_PostToneMap(void);
		void RenderUI(void);
		void Shutdown(void);

		void QuickSave(void);

 		virtual bool IsModified();

		virtual void PreMessageBlockEditing();
		virtual void PostMessageBlockEditing();

		virtual void BuildSoundTypeList( Asura_StringValuePair_DropDownList_Dialog* pxDialog ) {};

        // (MH - 15.11.2002) extra bits to get the phonon objects displayed in object mode
        static void CreatePhononToolsObjects();
        static void DestroyPhononToolsObjects();
        static bool IsInitialised() { return s_bIsInitialised; }
        static void SetShowRadius(bool bEnable) { ShowOuterRadius = bEnable; }

		static bool SelectStreamingSound(Asura_String& xFilename);
		static int GetChannelsInStreamingSound(const char* pszFilename);

		static void InvalidateSoundSize(){s_bSoundSizeValid = false;}
	private:
		//loads/saves data for this mode
		static void TransferModeData(int mode);
	
		static void CreateSoundModeOverlay(void);
		static void CreateVolumeScaleOverlay(void);
		//static void CreateSelectSoundOverlay(char* wildcard, void(*function)(void));

		static void UpdateSoundModeOverlay();

		static void HandleKeyboard();

// wolf static void SelectWolfSound(void);
		//static void SetWolfSound(void);


		//static void PreviewSound(void);		

		static void CreatePhonon(void);
		static Asura_Phonon* CreateDefaultPhonon();

		
		static void UpdateSoundSize();
		static bool GetSoundSizeForFile(const char* filename,int ps2Freq,int& outputPcSize,int& outputPs2Size);
        


		static void ChangeSubMode(int iSubMode);
		static bool IsReverbEditing(){return s_iSubMode == SUBMODE_REVERB;}

		static void LeavePhononMode();
		static void EnterPhononMode();
		static void LeaveReverbMode();
		static void EnterReverbMode();
		static void LeaveBackgroundMode();
		static void EnterBackgroundMode();

		#ifndef ASURA_USE_AUDIO_SYSTEM
		static const char* GetReverbTypeName(int iReverbType);
		static bool SelectReverbType(const char* pszTitle,int* piReverbType);

		static void SelectCurrentReverbType();
		static void SelectDefaultReverbType();

		static void PreviousReverbType();
		static void NextReverbType();

		static void ApplyReverb();
		static void RemoveReverb();
		#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

		static void FindPhononsByEvent();

		static void SelectDefaultStreamingSound();
		static void ClearDefaultStreamingSound();

		//Copy settings for the default streaming sound from/to the asura system
		static void CopyDefaultSBSFromAsura();
		static void CopyDefaultSBSToAsura();

		static void WriteStreamingSoundReport();


		static void VerifySoundEvent();
		static void FindReferencesToWav();

		static void HideEntities(bool bHide);


		//static void StopTestSound(void);

		static Asura_Vector_2 ModeOverlayPosition;
		static Asura_Vector_2 PhononPropertiesOverlayPosition;
		static u_int s_uPhononPropertiesOverlayMinimised;
		
	
		static List<Tools_Phonon_Object*> PhononList;
		
		static bool HideIcons;
		static bool ShowOuterRadius;

		static u_int s_PCSoundSize;
		static u_int s_PS2SoundSize;
		static u_int s_PS3SoundSize;
		static u_int s_Xbox360SoundSize;

		static bool AllFilesCounted;//false if there was some error checking file sizes for some of the files
		static bool s_bSoundSizeValid;

		//tree containing additional details of loaded sounds
		static Asura_TemplateHashTree<struct ToolsSoundData>* s_pxSoundDataTree;

		static char** FileList;
		static char** ShortNameList;
		static int NumberOfSoundFiles;
		static int ListOffset;
		static int ListSelection;


		static Asura_Handle TestSound;
        static bool s_bIsInitialised;


//		static bool s_bReverbEditing;//true if currently in reverb editing mode
//		static bool s_bRequestReverbEditing;//true if the reverb editing checkbox has been checked
		static int s_iCurrentReverbType;
		static bool s_bHighlightCurrentReverb;

		static bool s_bChanged;

		static float s_fSBSDefaultVolume;
		static Asura_Hash_ID s_uSBSDefaultMixer;
		static bool s_bSBSPerLightSetFlagChanged;

		static bool s_bLightSetChanged;

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
		static class Tools_ReverbVolume_Editor* s_pxReverbVolumeEditor;
#endif


		enum SubMode
		{
			SUBMODE_PHONON,
			SUBMODE_REVERB,
			SUBMODE_BACKGROUNDSOUND,
			SUBMODE_NUMSUBMODES

		};
		static int s_iSubMode;//taken from enum above
		static int s_iRequestedSubMode;//current selection in list box


		enum DialogID
		{
			DialogID_DefaultReverb,
			DialogID_CurrentReverb,
			DialogID_Filename,
		};

		friend class Tools_Phonon_Object;

};


//for storing some handy details about loaded wav files
struct ToolsSoundData
{
	ToolsSoundData() { m_bValid=false,m_bInitialised = false; }

#ifdef ASURA_USE_AUDIO_SYSTEM
	void Initialise(Asura_Hash_ID uSampleHash,const char* szFilename);
	Asura_Hash_ID m_eSoundID;
#else
	void Initialise(Asura_Sound_ID eSoundID,const char* szFilename);
	Asura_Sound_ID m_eSoundID;
#endif

	Asura_String m_xFilename;	// relative path + filename of WAV
	int m_iFrequency;			// samples per second
	int m_iSamples;				// total samples in the whole WAV
	int m_iDataChunkSize;		// byte size of PCM data in whole WAV
	bool m_bValid;				// true if the above values have been filled in
	bool m_bInitialised;		// true if an attempt to find the WAV file has been made
};

#endif//__TOOLS_MODE_SOUND_H__
