//
// Tools_PickDialogueEvent_Overlay.h
//
// Created by Richard Rice - 30/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PICKDIALOGUEEVENT_OVERLAY__
#define __TOOLS_PICKDIALOGUEEVENT_OVERLAY__
#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Core/Asura_Handle.h"
#include "Interface/Asura_Overlay.h"
#include "Misc/list_tem.hpp"


class Tools_PickDialogueEvent_Overlay : public Asura_Overlay
{
public :
	~Tools_PickDialogueEvent_Overlay();

	static void CreateOverlay();
	static void CloseOverlay();

	static bool OverlayExists();

	//pick an event using a modal overlay
	static Asura_Hash_ID PickEvent(const char* szTitle,Asura_Hash_ID uInitialSelection = 0, bool bMultipleSelect = false);

	//gets result of last modal event picking
	static bool WasCancelled(){return s_bCancelled;}

	//Gets selection if multiple selection is used
	static int GetNumSelected();
	static Asura_Hash_ID GetSelection(int iIndex);

	//forces list to be recalcualted in response to events being created/deleted etc
	static void InvalidateEventList(){s_bEventListInvalidated = true;}


	static void SetFileFilter(const char* szFileFilter);

private :
	Tools_PickDialogueEvent_Overlay();


	void SetDefaultTitle();
	void InitialiseOverlay();

	void BuildFilterLists();
	void DeleteFilterLists();

	void BuildEventList();
	void DeleteEventList();
	Asura_Hash_ID GetEventHash(int iIndex) const;

	void PerFrameUpdate();

	bool IsModal() const;
	void EndModal();
	void EnableMultipleSelection(bool bMultipleSelect);

	void SetInitialSelection(Asura_Hash_ID uSelected);


	//overlay callbacks
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_PickDialogueEvent_Overlay*)iThis)->__FnName();}\
	void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_PickDialogueEvent_Overlay*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(OnOkay)
	OVERLAY_VOID_CALLBACK(OnCancel)
	OVERLAY_VOID_CALLBACK(OnNewEvent)
	OVERLAY_VOID_CALLBACK(OnEditEvent)
	OVERLAY_VOID_CALLBACK(OnFindEventByID)
	OVERLAY_VOID_CALLBACK(OnViewLines)

	OVERLAY_VOID_CALLBACK(DeleteEvents)

	OVERLAY_VOID_CALLBACK(PlaySound)
	OVERLAY_VOID_CALLBACK(StopSound)

	OVERLAY_VOID_CALLBACK(OnDoubleClickList)

	OVERLAY_BOOL_CALLBACK(IsEnabled_OnOkay)
	OVERLAY_BOOL_CALLBACK(IsEnabled_OnCancel)

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK



	int m_iSelectedFileFilter;
	bool m_bFilterChanged;
	static bool s_bEventListInvalidated;

	//for filtering by character
	static Asura_Hash_ID s_uCharacterFilter;

	Asura_Hash_ID m_uInitialSelection;
	
	Asura_Hash_ID m_uMostRecentSelectedEvent;//hash of most recent event selected by user
	static Asura_Hash_ID s_uSelectedEvent;//hash of currently selected event
	int m_iEventIndex;
	bool m_bEventChanged;

	bool m_bMultipleSelection;
	static List<Asura_Hash_ID> s_xSelectionList;

	static bool s_bCancelled;//result of most recent use as a modal overlay

	//list for file filter
	u_int m_uFileListSize;
	const char** m_apszFileList;
	class Asura_StringTree* m_pxFileStringTree;

	//text search filter
	int m_iSearchOption;
	Asura_String m_xSearchText;
	

	//list of event names
	u_int m_uEventListSize;
	const char** m_apszEventList;
	class Asura_StringTree* m_pxEventStringTree;

	class Asura_List_Dialog* m_pxEventListDialog;

	Asura_Handle m_iDialogueHandle;
	static bool s_bAutoPreview;


	//most recently used filters
	static Asura_String s_xFileFilter;
	static Asura_String s_xLastFindEventID;

	static Tools_PickDialogueEvent_Overlay* s_pxThis;

public :
	static Asura_Vector_2 s_xOverlayPosition;

};


#endif //ASURA_USE_DIALOGUE_SYSTEM
#endif //__TOOLS_PICKDIALOGUEEVENT_OVERLAY__
