//
// Tools_PickDialogueLineTemplate_Overlay.cpp
//
// Created by Richard Rice - 30/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_PickDialogueLineTemplate_Overlay.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

//asura
#include "Core/Asura_NameTree.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"
#include "Sound/Asura_Dialogue.h"

//tools
#include "Tools_Overlays.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Sound/Tools_Dialogue_System.h"
#include "Sound/Tools_DialogueEvent_Overlay.h"


Asura_Vector_2 Tools_PickDialogueLineTemplate_Overlay::s_xOverlayPosition(25.0f,2.0f);
Tools_PickDialogueLineTemplate_Overlay* Tools_PickDialogueLineTemplate_Overlay::s_pxThis = 0;

Asura_Hash_ID Tools_PickDialogueLineTemplate_Overlay::s_uSelectedLineTemplate = 0;
List<Asura_Hash_ID> Tools_PickDialogueLineTemplate_Overlay::s_xSelectionList;
bool Tools_PickDialogueLineTemplate_Overlay::s_bCancelled = false;
bool Tools_PickDialogueLineTemplate_Overlay::s_bLineTemplateListInvalidated;

static const char* g_szFilterNone = "*All*";
static const char* g_szLineTemplateNone = "*None*";
static const char* g_szLineTemplateUnknown = "*Unknown*";

//search types
enum
{
	DIALOGUE_SEARCH_BEGINS,
	DIALOGUE_SEARCH_CONTAINS,
};

static const char* g_aszSearchOptions[] =
{
	"Begins With",
	"Contains",
};


Tools_PickDialogueLineTemplate_Overlay::Tools_PickDialogueLineTemplate_Overlay()
:Asura_Overlay(" ",false,&s_xOverlayPosition)
{
	Asura_Assert(!s_pxThis,"");
	s_pxThis = this;
	SetDefaultTitle();

	m_uLineTemplateListSize = 0;
	m_apszLineTemplateList = 0;
	m_pxLineTemplateStringTree = new Asura_StringTree;
	m_pxLineTemplateListDialog = 0;

	m_uInitialSelection = 0;
	m_uMostRecentSelectedLineTemplate = 0;
	s_uSelectedLineTemplate = 0;
	m_iLineTemplateIndex = 0;

	m_iSearchOption = 0;

	m_bMultipleSelection = true;

	m_bFilterChanged = false;
}

Tools_PickDialogueLineTemplate_Overlay::~Tools_PickDialogueLineTemplate_Overlay()
{
	Asura_Assert(s_pxThis == this,"");

	DeleteLineTemplateList();

	delete m_pxLineTemplateStringTree;

	s_pxThis = 0;
}


void Tools_PickDialogueLineTemplate_Overlay::SetDefaultTitle()
{
	SetTitle("Audio Templates");	
}

void Tools_PickDialogueLineTemplate_Overlay::CreateOverlay()
{
	//overlay already open?
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PICKDIALOGUELINETEMPLATE))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_PICKDIALOGUELINETEMPLATE);
		return;
	}

	//create the overlay then
	Tools_PickDialogueLineTemplate_Overlay* pxOverlay = new Tools_PickDialogueLineTemplate_Overlay();
	
	pxOverlay->InitialiseOverlay();

	pxOverlay->m_uID = TOOLS_OVERLAY_PICKDIALOGUELINETEMPLATE;
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay);
}

void Tools_PickDialogueLineTemplate_Overlay::InitialiseOverlay()
{
	m_pxLineTemplateListDialog = 0;


	//build temporary list , which will be corrected on the first frame
	m_apszLineTemplateList = new const char*[1];
	m_apszLineTemplateList[0] = g_szLineTemplateNone;
	m_uLineTemplateListSize = 1;
	m_bFilterChanged = true;


	//determine the maximum number of lines to use for the various lists
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const int iMaxLines = Asura_Maths::FloatToInt_Chop((fMaxPrintY - 24.0f)/1.2f);


	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset(1.0f,3.0f);


	Asura_DropDown_List_Dialog* pxDropListDialog;
	
	//Search/Search Type
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszSearchOptions,&m_iSearchOption,iMaxLines,2,NULL,&m_bFilterChanged,"");
	pxDropListDialog->SetHelpText("Determines how the search text should be used for filtering the list.");
	AddDialog(pxDropListDialog);


	xOffset.x = pxDropListDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_String_Dialog("",xOffset,&m_xSearchText,30,&m_bFilterChanged);
	((Asura_String_Dialog*)pxDialog)->SetUpdateImmediatelyWhileTyping(true);
	pxDialog->SetHelpText("Filters the template list so that it only shows those beginning with/containing this string");;
	xOffset.x = 1.0f;
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	



	//list of templates
	m_bLineTemplateChanged = false;
	pxDialog = m_pxLineTemplateListDialog = new Asura_List_Dialog(xOffset,m_apszLineTemplateList,&m_iLineTemplateIndex,iMaxLines,m_uLineTemplateListSize,NULL,&m_bLineTemplateChanged);
	m_pxLineTemplateListDialog->EnableMultipleSelection(m_bMultipleSelection);
	m_pxLineTemplateListDialog->SetDoubleClickFunction(OnDoubleClickList,(int)this);
	AddDialog(pxDialog);
	xOffset.y += iMaxLines*1.2f + 1.0f;


	pxDialog = new Asura_Function_Dialog("New Template",xOffset,OnNewLineTemplate,(int)this);
	pxDialog->SetHelpText("Create and select a new template.");
	AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Edit Template",xOffset,OnEditLineTemplate,(int)this);
	pxDialog->SetHelpText("Selects the current template and opens the overlay for editing it.");
	AddDialog(pxDialog);
	xOffset.x -= 12.0f;

//	xOffset.x += 24.0f;
//	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteLineTemplates,(int)this);
//	pxDialog->SetHelpText("Delete the selected event(s).");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.x -= 24.0f;

	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OnOkay,(int)this);
	pxDialog->SetIsEnabledFunction(IsEnabled_OnOkay,(int)this);
	AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,OnCancel,(int)this);
	pxDialog->SetIsEnabledFunction(IsEnabled_OnCancel,(int)this);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 12.0f;

}

void Tools_PickDialogueLineTemplate_Overlay::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PICKDIALOGUELINETEMPLATE);	
}

bool Tools_PickDialogueLineTemplate_Overlay::OverlayExists()
{
	//return Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PICKDIALOGUELINETEMPLATE);
	return s_pxThis != 0;
}

Asura_Hash_ID Tools_PickDialogueLineTemplate_Overlay::PickLineTemplate(const char* szTitle,Asura_Hash_ID uInitialSelection /*= 0*/, bool bMultipleSelect /*= false*/)
{
	//make sure the overlay is open
	bool bOverlayInitiallyOpen = OverlayExists(); 

	if(!bOverlayInitiallyOpen) CreateOverlay();
	
	Asura_Assert(s_pxThis,"Failed to get pick template overlay");
	if(!s_pxThis) return 0;

	//set the title
	if(szTitle) s_pxThis->SetTitle(szTitle);

	//update selection
	s_pxThis->EnableMultipleSelection(bMultipleSelect);
	s_pxThis->m_uInitialSelection = uInitialSelection;
	s_pxThis->SetInitialSelection(uInitialSelection);

	//run as modal overlay
	s_bCancelled = true;
	Tools_Modal_Overlay::DoModal(s_pxThis);

	//close overlay if it was opened just for this
	if(bOverlayInitiallyOpen && OverlayExists())
	{
		s_pxThis->SetDefaultTitle();
		s_pxThis->EnableMultipleSelection(true);
	}
	else
	{
		CloseOverlay();
	}

	return s_bCancelled ? uInitialSelection : s_uSelectedLineTemplate;
}



void Tools_PickDialogueLineTemplate_Overlay::BuildLineTemplateList()
{
	DeleteLineTemplateList();
	s_bLineTemplateListInvalidated = false;

	char szWildcardSearchString[100];
	if(!m_xSearchText.IsBlank() && m_iSearchOption == DIALOGUE_SEARCH_CONTAINS)
	{
		//prepare search string for wildcard search
		Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,m_xSearchText.GetString());
	}

	//get a list of all templates that meet the current filter requirements
	int iJunkIndex;
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfLineTemplates();i++)
	{
		const Asura_Dialogue_Line_Template* pxLineTemplate = Asura_Dialogue_Storage::GetLineTemplateByIndex(i);


		if(!m_xSearchText.IsBlank())
		{
			if(m_iSearchOption == DIALOGUE_SEARCH_BEGINS)
			{
				if(strnicmp(m_xSearchText.GetString(),pxLineTemplate->m_xName.GetString(),m_xSearchText.GetLength()))
				{
					continue;
				}
			}
			else
			{
				if(!Asura_StringUtil::CompareAgainstWildcardString(pxLineTemplate->m_xName.GetString(),szWildcardSearchString,true))
				{
					continue;
				}
			}
		}

		m_pxLineTemplateStringTree->FindNodeIndexByName(pxLineTemplate->m_xName.GetString(),true,iJunkIndex);

	}

	//need an entry for 'unknown' ? 
	bool bNeedUnknown = false;
	if(m_uInitialSelection && !Asura_Dialogue_Storage::GetLineTemplateByHash(m_uInitialSelection))
	{
		bNeedUnknown = true;
	}

	u_int uSpecialEntries = bNeedUnknown ? 2 : 1;

	//get array of template names , and fill in the special entries that aren't in the tree
	m_uLineTemplateListSize = m_pxLineTemplateStringTree->GetNumberOfNodes()+uSpecialEntries;
	m_apszLineTemplateList = new const char*[m_uLineTemplateListSize];
	m_apszLineTemplateList[0] = g_szLineTemplateNone;
	if(bNeedUnknown) m_apszLineTemplateList[1] = g_szLineTemplateUnknown;
	m_pxLineTemplateStringTree->MakeSortedNameList(m_apszLineTemplateList+uSpecialEntries);

	//Update the current selected index to be as close as possible to the previous selection
	if(m_uMostRecentSelectedLineTemplate == 0)
	{
		m_iLineTemplateIndex = 0;
	}
	else if(bNeedUnknown && m_uMostRecentSelectedLineTemplate == m_uInitialSelection)
	{
		m_iLineTemplateIndex = 1;
	}
	else
	{
		const Asura_Dialogue_Line_Template* pxLineTemplate = Asura_Dialogue_Storage::GetLineTemplateByHash(m_uMostRecentSelectedLineTemplate);
		m_iLineTemplateIndex = 0;
		if(pxLineTemplate)
		{
			for(u_int i=uSpecialEntries;i<m_uLineTemplateListSize;i++)
			{
				if(stricmp(pxLineTemplate->m_xName.GetString(),m_apszLineTemplateList[i])>=0)
				{
					m_iLineTemplateIndex = i;
				}
				else
				{
					break;
				}
			}
		}
	}

	//get the hash ID of the event selected
	s_uSelectedLineTemplate = GetLineTemplateHash(m_iLineTemplateIndex);

	//multiple selection list will be junk after list change
	while(s_xSelectionList.size()) s_xSelectionList.delete_first_entry();

	//update the list dialog if it has been created yet
	if(m_pxLineTemplateListDialog)
	{
		m_pxLineTemplateListDialog->ReplaceStrings(m_uLineTemplateListSize,m_apszLineTemplateList);
	}
}

void Tools_PickDialogueLineTemplate_Overlay::DeleteLineTemplateList()
{
	delete [] m_apszLineTemplateList;
	m_apszLineTemplateList = 0;
	m_uLineTemplateListSize = 0;
	m_pxLineTemplateStringTree->Reset();
}

Asura_Hash_ID Tools_PickDialogueLineTemplate_Overlay::GetLineTemplateHash(int iIndex) const
{
	if(!stricmp(m_apszLineTemplateList[iIndex],g_szLineTemplateNone))
	{
		return 0;
	}
	else if(!stricmp(m_apszLineTemplateList[iIndex],g_szLineTemplateUnknown))
	{
		return m_uInitialSelection;
	}
	else
	{
		return Asura_GetHashID(m_apszLineTemplateList[iIndex]);
	}
}


void Tools_PickDialogueLineTemplate_Overlay::PerFrameUpdate()
{
	Asura_Overlay::PerFrameUpdate();

	if(m_bFilterChanged || s_bLineTemplateListInvalidated)
	{
		//rebuild the event list
		BuildLineTemplateList();
		m_bFilterChanged = false;
	}


	if(m_bLineTemplateChanged)
	{
		//get the hash of the new event
		m_uMostRecentSelectedLineTemplate = s_uSelectedLineTemplate = GetLineTemplateHash(m_iLineTemplateIndex);
		m_bLineTemplateChanged = false;

		//If doing multiple selection , get hashes of all in list
		if(m_bMultipleSelection)
		{
			while(s_xSelectionList.size()) s_xSelectionList.delete_first_entry();

			int iNumSelected = m_pxLineTemplateListDialog->GetNumSelected();
			for(int i=0;i<iNumSelected;i++)
			{
				int iIndex = m_pxLineTemplateListDialog->GetSelection(i);
				Asura_Hash_ID uHash = GetLineTemplateHash(iIndex);
				if(uHash) s_xSelectionList.add_entry(uHash);
			}
		}
	}

	if(IsModal() && !Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE) && IsEnabled_OnCancel())
		{
			OnCancel();
		}
	}
}

bool Tools_PickDialogueLineTemplate_Overlay::IsModal() const
{
	return m_bAlwaysOnTop;
}

void Tools_PickDialogueLineTemplate_Overlay::EndModal()
{
	m_bAlwaysOnTop = false;
}

void Tools_PickDialogueLineTemplate_Overlay::EnableMultipleSelection(bool bMultipleSelect)
{
	m_bMultipleSelection = bMultipleSelect;
	if(m_pxLineTemplateListDialog)
	{
		m_pxLineTemplateListDialog->EnableMultipleSelection(m_bMultipleSelection);
	}
	if(!bMultipleSelect)
	{
		while(s_xSelectionList.size())
		{
			s_xSelectionList.delete_first_entry();
		}
	}

}

int Tools_PickDialogueLineTemplate_Overlay::GetNumSelected() 
{
	return s_bCancelled ? 0 : s_xSelectionList.size();
}

Asura_Hash_ID Tools_PickDialogueLineTemplate_Overlay::GetSelection(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex < s_xSelectionList.size(),"Invalid selection index");
	return s_xSelectionList[iIndex];
}

void Tools_PickDialogueLineTemplate_Overlay::OnOkay()
{
	s_bCancelled = false;
	EndModal();
}

void Tools_PickDialogueLineTemplate_Overlay::OnCancel()
{
	s_bCancelled = true;
	EndModal();
}

void Tools_PickDialogueLineTemplate_Overlay::OnNewLineTemplate()
{
	Asura_Hash_ID uNewHash = Tools_Dialogue_System::CreateLineTemplate();

	if(uNewHash)
	{
		//update the selection
		m_uMostRecentSelectedLineTemplate = uNewHash;
		BuildLineTemplateList();
		s_xSelectionList.add_entry(uNewHash);

		s_bCancelled = false;
		EndModal();
	}
}

void Tools_PickDialogueLineTemplate_Overlay::OnEditLineTemplate()
{
	if(m_bMultipleSelection)
	{
		for(LIF<Asura_Hash_ID> xSelectionLIF(&s_xSelectionList);!xSelectionLIF.done();xSelectionLIF.next())
		{
			Tools_DialogueEvent_Overlay::EditLineTemplate(xSelectionLIF());
		}
	}
	else
	{
		if(Asura_Dialogue_Storage::GetLineTemplateByHash(s_uSelectedLineTemplate)) Tools_DialogueEvent_Overlay::EditLineTemplate(s_uSelectedLineTemplate);
	}
	s_bCancelled = false;
	EndModal();
}

void Tools_PickDialogueLineTemplate_Overlay::OnDoubleClickList()
{
	if(IsModal())
	{
		OnOkay();
	}
	else
	{
		OnEditLineTemplate();
	}
}

bool Tools_PickDialogueLineTemplate_Overlay::IsEnabled_OnOkay()
{
	return IsModal();
}

bool Tools_PickDialogueLineTemplate_Overlay::IsEnabled_OnCancel()
{
	return IsModal();
}

void Tools_PickDialogueLineTemplate_Overlay::SetInitialSelection(Asura_Hash_ID uSelected)
{
	m_uInitialSelection = uSelected;
	m_uMostRecentSelectedLineTemplate = uSelected;
	
	//make sure the selection will be visible with the current filters.
	const Asura_Dialogue_Line_Template* pxLineTemplate = Asura_Dialogue_Storage::GetLineTemplateByHash(uSelected);
	if(pxLineTemplate)
	{
		
		if(!m_xSearchText.IsBlank())
		{
			if(m_iSearchOption == DIALOGUE_SEARCH_BEGINS)
			{
				if(strnicmp(m_xSearchText.GetString(),pxLineTemplate->m_xName.GetString(),m_xSearchText.GetLength()))
				{
					m_xSearchText.MakeNull();
					m_bFilterChanged = true;
				}
			}
			else
			{
				char szWildcardSearchString[100];
				Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,m_xSearchText.GetString());
				if(!Asura_StringUtil::CompareAgainstWildcardString(pxLineTemplate->m_xName.GetString(),szWildcardSearchString,true))
				{
					m_xSearchText.MakeNull();
					m_bFilterChanged = true;
				}
			}
		}
	}


	BuildLineTemplateList();
} 



#endif //ASURA_USE_DIALOGUE_SYSTEM

