//
// Tools_PickDialogueEvent_Overlay.h
//
// Created by Richard Rice - 30/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PICKDIALOGUELINETEMPLATE_OVERLAY__
#define __TOOLS_PICKDIALOGUELINETEMPLATE_OVERLAY__
#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Interface/Asura_Overlay.h"
#include "Misc/list_tem.hpp"


class Tools_PickDialogueLineTemplate_Overlay : public Asura_Overlay
{
public :
	~Tools_PickDialogueLineTemplate_Overlay();

	static void CreateOverlay();
	static void CloseOverlay();

	static bool OverlayExists();

	//pick an event using a modal overlay
	static Asura_Hash_ID PickLineTemplate(const char* szTitle,Asura_Hash_ID uInitialSelection = 0, bool bMultipleSelect = false);

	//gets result of last modal event picking
	static bool WasCancelled(){return s_bCancelled;}

	//Gets selection if multiple selection is used
	static int GetNumSelected();
	static Asura_Hash_ID GetSelection(int iIndex);

	//forces list to be recalculated in response to templates being created/deleted etc
	static void InvalidateLineTemplateList(){s_bLineTemplateListInvalidated = true;}


private :
	Tools_PickDialogueLineTemplate_Overlay();


	void SetDefaultTitle();
	void InitialiseOverlay();

	void BuildLineTemplateList();
	void DeleteLineTemplateList();
	Asura_Hash_ID GetLineTemplateHash(int iIndex) const;

	void PerFrameUpdate();

	bool IsModal() const;
	void EndModal();
	void EnableMultipleSelection(bool bMultipleSelect);

	void SetInitialSelection(Asura_Hash_ID uSelected);


	//overlay callbacks
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_PickDialogueLineTemplate_Overlay*)iThis)->__FnName();}\
	void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_PickDialogueLineTemplate_Overlay*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(OnOkay)
	OVERLAY_VOID_CALLBACK(OnCancel)
	OVERLAY_VOID_CALLBACK(OnNewLineTemplate)
	OVERLAY_VOID_CALLBACK(OnEditLineTemplate)

	OVERLAY_VOID_CALLBACK(OnDoubleClickList)

	OVERLAY_BOOL_CALLBACK(IsEnabled_OnOkay)
	OVERLAY_BOOL_CALLBACK(IsEnabled_OnCancel)

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK



	bool m_bFilterChanged;
	static bool s_bLineTemplateListInvalidated;

	Asura_Hash_ID m_uInitialSelection;
	
	Asura_Hash_ID m_uMostRecentSelectedLineTemplate;//hash of most recent template selected by user
	static Asura_Hash_ID s_uSelectedLineTemplate;//hash of currently selected template
	int m_iLineTemplateIndex;
	bool m_bLineTemplateChanged;

	bool m_bMultipleSelection;
	static List<Asura_Hash_ID> s_xSelectionList;

	static bool s_bCancelled;//result of most recent use as a modal overlay

	//text search filter
	int m_iSearchOption;
	Asura_String m_xSearchText;
	

	//list of template names
	u_int m_uLineTemplateListSize;
	const char** m_apszLineTemplateList;
	class Asura_StringTree* m_pxLineTemplateStringTree;

	class Asura_List_Dialog* m_pxLineTemplateListDialog;

	static Tools_PickDialogueLineTemplate_Overlay* s_pxThis;

public :
	static Asura_Vector_2 s_xOverlayPosition;

};


#endif //ASURA_USE_DIALOGUE_SYSTEM
#endif //__TOOLS_PICKDIALOGUELINETEMPLATe_OVERLAY__

