//
// Tools_PickDialogueLine_Overlay.cpp
//
// Created by Richard Rice - 01/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_PickDialogueLine_Overlay.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Core/Asura_NameTree.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FacialAnimation/Asura_FacialAnimation_System.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Audio_Voice.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Mixer.h"

//tools
#include "Tools_Overlays.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "FacialAnimation/Tools_FacialAnimation_LipsyncConversion.h"
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Misc/Tools_Modal_Overlay.h"
#include "Sound/Tools_DialogueEvent_Dialogs.h"
#include "Sound/Tools_DialogueEvent_Overlay.h"
#include "Sound/Tools_Dialogue_System.h"
#include "Sound/Tools_PickDialogueLineTemplate_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"

#ifdef ASURA_USE_ARTICULATION_SYSTEM
#include "LipSync/Tools_Lipsync.h"
#endif



Asura_Hash_ID Tools_PickDialogueLine_Overlay::s_uCharacterFilter = 0;

Asura_Vector_2 Tools_PickDialogueLine_Overlay::s_xOverlayPosition(30.0f,2.0f);
Tools_PickDialogueLine_Overlay* Tools_PickDialogueLine_Overlay::s_pxThis;

int Tools_PickDialogueLine_Overlay::s_iSearchOption;
Asura_String Tools_PickDialogueLine_Overlay::s_xSearchText;

Asura_Hash_ID Tools_PickDialogueLine_Overlay::s_uSelectedLine = 0;
List<Asura_Hash_ID> Tools_PickDialogueLine_Overlay::s_xSelectionList;
bool Tools_PickDialogueLine_Overlay::s_bCancelled = false;
bool Tools_PickDialogueLine_Overlay::s_bEventListInvalidated;
bool Tools_PickDialogueLine_Overlay::s_bLineListInvalidated;

bool Tools_PickDialogueLine_Overlay::m_bRegenAllLines = false;
int Tools_PickDialogueLine_Overlay::m_iNumLines = 0;
int Tools_PickDialogueLine_Overlay::m_iCurrentLine = 0;

bool Tools_PickDialogueLine_Overlay::s_bAutoPreview = true;
#ifdef ASURA_USE_ARTICULATION_SYSTEM
bool Tools_PickDialogueLine_Overlay::s_bHasArticulation = false;
#endif

bool Tools_PickDialogueLine_Overlay::s_bSingleEventOnly = false;
Asura_Hash_ID Tools_PickDialogueLine_Overlay::s_uEventHash = ASURA_HASH_ID_UNSET;

static const char* g_szFilterNone = "*All*";
static const char* g_szLineNone = "*None*";
static const char* g_szLineUnknown = "*Unknown*";

//search types
enum
{
	DIALOGUE_SEARCH_BEGINS,
	DIALOGUE_SEARCH_CONTAINS,
};

static const char* g_aszSearchOptions[] =
{
	"Begins With",
	"Contains",
};


Tools_PickDialogueLine_Overlay::Tools_PickDialogueLine_Overlay()
:Asura_Overlay(" ",false,&s_xOverlayPosition)
{
	Asura_Assert(!s_pxThis,"");
	s_pxThis = this;
	SetDefaultTitle();

	m_uLineListSize = 0;
	m_apszLineList = NULL;
	m_pxLineStringTree = new Asura_StringTree;

	m_bAnyEvents = false;
	m_uEventListSize = 0;
	m_apszEventList = NULL;
	m_pxEventStringTree = new Asura_StringTree;
	m_iEventIndex = 0;	

	m_pxIDDialog = NULL;
	m_pxCharacterDialog = NULL;
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	m_pxFaceTypeDialog	= NULL;
	m_pxBaseFacePoseDialog	= NULL;
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	m_pxTemplateDialog = NULL;
	m_pxTextDialog = NULL;

	m_iSoundHandle = ASURA_INVALID_HANDLE;

	m_bMultipleSelection = true;

	m_bFilterChanged = false;

	m_bRegenAllLines = false;
	m_iCurrentLine = 0;
	m_iNumLines = 0;
}

Tools_PickDialogueLine_Overlay::~Tools_PickDialogueLine_Overlay()
{
	Asura_Assert(s_pxThis == this,"");

	StopSound();

	DeleteLineList();
	DeleteEventList();

	delete m_pxLineStringTree;
	delete m_pxEventStringTree;

	s_pxThis = 0;
}

void Tools_PickDialogueLine_Overlay::SetDefaultTitle()
{
	SetTitle("Dialogue Line List");	
}

void Tools_PickDialogueLine_Overlay::CreateOverlay()
{
	//overlay already open?
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PICKDIALOGUELINE))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_PICKDIALOGUELINE);
		return;
	}

	//create the overlay then
	Tools_PickDialogueLine_Overlay* pxOverlay = new Tools_PickDialogueLine_Overlay();
	
	pxOverlay->InitialiseOverlay();

	pxOverlay->m_uID = TOOLS_OVERLAY_PICKDIALOGUELINE;
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay);

	s_pxThis->UpdatePropertiesText();
}



void Tools_PickDialogueLine_Overlay::InitialiseOverlay()
{
	m_pxEventListDialog = 0;
	m_pxLineListDialog = 0;
	m_pxIDDialog = 0;
	m_pxCharacterDialog = 0;
	m_pxTemplateDialog = 0;
	m_pxTextDialog = 0;


	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	// Ensure that we're not permanently thinking this is cancelled...
	s_bCancelled	= false;

	//build temporary list , which will be corrected on the first frame
	m_apszLineList = new const char*[1];
	m_apszLineList[0] = g_szLineNone;
	m_uLineListSize = 1;
	m_bFilterChanged = true;

	//determine the maximum number of lines to use for the various lists
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const int iMaxLines = Asura_Maths::FloatToInt_Chop((fMaxPrintY - 41.0f)/1.2f);


	//add drop down lists for filters
	Asura_DropDown_List_Dialog* pxDropListDialog;
	
	//character filter
	pxDropListDialog = new Tools_DialogueCharacter_Dialog(xOffset,iMaxLines,&s_uCharacterFilter,&m_bFilterChanged,"Character");
	pxDropListDialog->SetHelpText("Filter list to those lines spoken by a given character.");
	AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Search/Search Type
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszSearchOptions,&s_iSearchOption,iMaxLines,2,NULL,&m_bFilterChanged,"");
	pxDropListDialog->SetHelpText("Determines how the search text should be used for filtering the list.");
	AddDialog(pxDropListDialog);


	xOffset.x = pxDropListDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_String_Dialog("",xOffset,&s_xSearchText,30,&m_bFilterChanged);
	((Asura_String_Dialog*)pxDialog)->SetUpdateImmediatelyWhileTyping(true);
	pxDialog->SetHelpText("Filters the line list so that it only shows those beginning with/containing this string");;
	xOffset.x = 1.0f;
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//list of lines
	m_bLineChanged = false;
	pxDialog = m_pxLineListDialog = new Asura_List_Dialog(xOffset,m_apszLineList,&m_iLineIndex,iMaxLines,m_uLineListSize,NULL,&m_bLineChanged);
	m_pxLineListDialog->EnableMultipleSelection(m_bMultipleSelection);
	m_pxLineListDialog->SetDoubleClickFunction(OnDoubleClickList,0);
	m_pxLineListDialog->SetTooltipCallback(ShowTooltipForLine);
	AddDialog(pxDialog);
	xOffset.y += iMaxLines*1.2f + 1.0f;

	//properties of selected line
	pxDialog = m_pxIDDialog = new Asura_Dialog("ID : ",xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = m_pxCharacterDialog = new Asura_Dialog("Character : ",xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	pxDialog = m_pxFaceTypeDialog = new Asura_Dialog("Face-type : ",xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = m_pxBaseFacePoseDialog = new Asura_Dialog("Base face-pose : ",xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	pxDialog = m_pxTemplateDialog = new Asura_Dialog("Template : ",xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 2;	// writing 2 lines of text here (template and Mixer inside template).

	pxDialog = m_pxTextDialog = new Asura_Dialog("Text : ",xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if( !s_bSingleEventOnly )
	{
		pxDialog = new Asura_Dialog("Events :",xOffset);
		AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();



		BuildEventList();
		pxDialog = m_pxEventListDialog = new Asura_List_Dialog(xOffset,m_apszEventList,&m_iEventIndex,6,m_uEventListSize,NULL);
		m_pxEventListDialog->EnableMultipleSelection(true);
		m_pxEventListDialog->SetDoubleClickFunction(OnDoubleClickEventList,0);
		m_pxEventListDialog->SetHelpText("List of events using the selected line.");
		AddDialog(pxDialog);
		xOffset.y += 6*1.2f + 1.0f;


		pxDialog = new Asura_Function_Dialog("Edit Event",xOffset,OnEditEvent,(int)this);
		pxDialog->SetIsEnabledFunction(CanEditEvent,(int)this);
		pxDialog->SetHelpText("Edit the selected events from the list below.");
		AddDialog(pxDialog);
	}
	
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Preview Sound",xOffset,PlaySound,(int)this);
	pxDialog->SetHelpText("Tests the current dialogue line.");
	AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Stop Preview",xOffset,StopSound,(int)this);
	pxDialog->SetHelpText("Stops the test sound.");
	AddDialog(pxDialog);
	xOffset.x -= 12.0f;

	xOffset.x += 24.0f;
	pxDialog = new Asura_Tickbox_Dialog("Auto Preview",xOffset,&s_bAutoPreview);
	pxDialog->SetHelpText("Automatically previews each sound as it is selected.");
	AddDialog(pxDialog);
	xOffset.x -= 24.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

#ifdef ASURA_USE_ARTICULATION_SYSTEM

	pxDialog = new Asura_Function_Dialog("Articulate Textless",xOffset,GenerateArticulationTextless,(int)this);
	pxDialog->SetHelpText("Stops the test sound.");
	AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Articulate",xOffset,GenerateArticulationText,(int)this);
	pxDialog->SetHelpText("Stops the test sound.");
	AddDialog(pxDialog);
	xOffset.x -= 12.0f;

	xOffset.x += 24.0f;
	pxDialog = m_pxCheckboxArticulation= new Asura_Tickbox_Dialog("Has Articulation",xOffset,&s_bHasArticulation);
	pxDialog->SetHelpText("Automatically previews each sound as it is selected.");
	AddDialog(pxDialog);
	xOffset.x -= 24.0f;

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Regen ALL articulation for already built data (inc translation)",xOffset,RegenerateArticulation,(int)this);
	pxDialog->SetHelpText("Regenerate all of the articulation files.\n This can take a long time.\n This determines which lines to regenerate based on what articulation already exists for English..");
	AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
#endif

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	pxDialog	= new Asura_Function_Dialog("Auto-generate Facial-Animation", xOffset, AutoGenerateFacialAnimation, reinterpret_cast<int>(this));
	pxDialog->SetIsEnabledFunction(CanAutoGenerateFacialAnimation, reinterpret_cast<int>(this));
	pxDialog->SetHelpText("Generates a facial-animation file from the .WAV file associated with this line of dialogue.");
	AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	if( !s_bSingleEventOnly )
	{
		pxDialog = new Asura_Function_Dialog("Create Event From Lines",xOffset,CreateEventFromLines,(int) this);
		pxDialog->SetHelpText("Create a new event using the selected lines");
		AddDialog(pxDialog);

		xOffset.x += 24.0f;
		pxDialog = new Asura_Function_Dialog("Edit Template",xOffset,EditTemplate,(int) this);
		pxDialog->SetIsEnabledFunction(CanEditTemplate, reinterpret_cast<int>(this));
		pxDialog->SetHelpText("Edit the Template for the selected line");
		AddDialog(pxDialog);
		xOffset.x -= 24.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();
	}

	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OnOkay,(int)this);
	pxDialog->SetIsEnabledFunction(IsEnabled_OnOkay,(int)this);
	AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,OnCancel,(int) this);
	pxDialog->SetIsEnabledFunction(IsEnabled_OnCancel,(int)this);
	AddDialog(pxDialog);
	xOffset.x -= 12.0f;


	xOffset.y += Asura_Dialog::DefaultSpacing();

}

void Tools_PickDialogueLine_Overlay::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PICKDIALOGUELINE);	
}

bool Tools_PickDialogueLine_Overlay::OverlayExists()
{
	return s_pxThis != 0;
}

Asura_Hash_ID Tools_PickDialogueLine_Overlay::PickLine(const char* szTitle,Asura_Hash_ID uInitialSelection /*= 0*/, bool bMultipleSelect /*= false*/, const Asura_Hash_ID uEventHash /*=ASURA_HASH_ID_UNSET*/)
{
	//set if we are selecting lines from one event (before we create the overlay since it chages which controls are shown)
	if( uEventHash != ASURA_HASH_ID_UNSET )
	{
		s_uEventHash = uEventHash;
		s_bSingleEventOnly = true;
	}

	//make sure the overlay is open
	bool bOverlayInitiallyOpen = OverlayExists(); 

	if( bOverlayInitiallyOpen )
	{
		CloseOverlay();
	}
	CreateOverlay();
	
	Asura_Assert(s_pxThis,"Failed to get pick dialogue line overlay");
	if(!s_pxThis) return 0;

	//set the title
	if(szTitle) s_pxThis->SetTitle(szTitle);

	//update selection
	s_pxThis->EnableMultipleSelection(bMultipleSelect);
	s_pxThis->m_uInitialSelection = uInitialSelection;
	s_pxThis->SetInitialSelection(uInitialSelection);

	//run as modal overlay
	s_bCancelled = true;
	Tools_Modal_Overlay::DoModal(s_pxThis);

	// Only this function can set these single event variables on, so set them off
	s_bSingleEventOnly = false;
	s_uEventHash = ASURA_HASH_ID_UNSET;

	//close overlay if it was opened just for this
	// TY 20.11.08 - We just showed this dialog modally, how can it be open?
	if(bOverlayInitiallyOpen && OverlayExists())
	{
		s_pxThis->SetDefaultTitle();
		s_pxThis->EnableMultipleSelection(true);
	}
	else
	{
		CloseOverlay();
	}

	return s_bCancelled ? uInitialSelection : s_uSelectedLine;
}




void Tools_PickDialogueLine_Overlay::BuildLineList()
{
	DeleteLineList();
	s_bLineListInvalidated = false;
	
	char szWildcardSearchString[100];
	if(!s_xSearchText.IsBlank() && s_iSearchOption == DIALOGUE_SEARCH_CONTAINS)
	{
		//prepare search string for wildcard search
		Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,s_xSearchText.GetString());
	}

	//get a list of all samples that meet the current filter requirements
	int iJunkIndex;
	
	if( s_bSingleEventOnly )
	{
		// For single events just loop over the lines it contains
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash( s_uEventHash );
		Asura_Assert( pxEvent, "Failed to to find dialogue event" );
		if( pxEvent )
		{
			const u_int uNumItems = pxEvent->GetNumPlayables();
			for( u_int uItem = 0; uItem < uNumItems; ++uItem )
			{
				const Asura_Dialogue_Playable* pxPlayable = pxEvent->GetPlayable( uItem );
				if( pxPlayable && pxPlayable->IsLine() )
				{
					const Asura_Dialogue_Line* pxLine = reinterpret_cast<const Asura_Dialogue_Line*>( pxPlayable );

					if(s_uCharacterFilter && s_uCharacterFilter != pxLine->GetCharacter())
					{
						continue;
					}

					if(!s_xSearchText.IsBlank())
					{
						if(!DoesLineSatisfySearchFilter(pxLine,szWildcardSearchString))
						{
							continue;
						}
					}

					m_pxLineStringTree->FindNodeIndexByName(pxLine->GetName(),true,iJunkIndex);
				}
			}
		}
	}
	else
	{
		//get a list of all files and categories used by the samples
		for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfLines();i++)
		{
			const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByIndex(i);

			if(s_uCharacterFilter && s_uCharacterFilter != pxLine->GetCharacter())
			{
				continue;
			}

			if(!s_xSearchText.IsBlank())
			{
				if(!DoesLineSatisfySearchFilter(pxLine,szWildcardSearchString))
				{
					continue;
				}
			}

			m_pxLineStringTree->FindNodeIndexByName(pxLine->GetName(),true,iJunkIndex);
		}
	}

	//need an entry for 'unknown' ? 
	bool bNeedUnknown = false;
	if(m_uInitialSelection && !Asura_Dialogue_Storage::GetLineByHash(m_uInitialSelection))
	{
		bNeedUnknown = true;
	}

	u_int uSpecialEntries = bNeedUnknown ? 2 : 1;

	//get array of sample names , and fill in the special samples that arne't in the tree
	m_uLineListSize = m_pxLineStringTree->GetNumberOfNodes()+uSpecialEntries;
	m_apszLineList = new const char*[m_uLineListSize];
	m_apszLineList[0] = g_szLineNone;
	if(bNeedUnknown) m_apszLineList[1] = g_szLineUnknown;
	m_pxLineStringTree->MakeSortedNameList(m_apszLineList+uSpecialEntries);

	//Update the current selected index to be as close as possible to the previous selection
	if(m_uMostRecentSelectedLine == 0)
	{
		m_iLineIndex = 0;
	}
	else if(bNeedUnknown && m_uMostRecentSelectedLine == m_uInitialSelection)
	{
		m_iLineIndex = 1;
	}
	else
	{
		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(m_uMostRecentSelectedLine);
		m_iLineIndex = 0;
		if(pxLine)
		{
			for(u_int i=uSpecialEntries;i<m_uLineListSize;i++)
			{
				if(stricmp(pxLine->GetName(),m_apszLineList[i])>=0)
				{
					m_iLineIndex = i;
				}
				else
				{
					break;
				}
			}
		}
	}

	//get the hash ID of the line selected
	s_uSelectedLine = GetLineHash(m_iLineIndex);

	//multiple selection list will be junk after list change
	while(s_xSelectionList.size()) s_xSelectionList.delete_first_entry();

	//update the list dialog if it has been created yet
	if(m_pxLineListDialog)
	{
		m_pxLineListDialog->ReplaceStrings(m_uLineListSize,m_apszLineList);

		if(m_uLineListSize > 0)
		{
			//select the item in the list
			m_pxLineListDialog->SetSelection(m_iLineIndex);

			// force the list to be rebuilt on the first update
			m_bLineChanged = true;
		}
	}
}

void Tools_PickDialogueLine_Overlay::DeleteLineList()
{
	delete [] m_apszLineList;
	m_apszLineList = 0;
	m_uLineListSize = 0;
	m_pxLineStringTree->Reset();
}


Asura_Hash_ID Tools_PickDialogueLine_Overlay::GetLineHash(int iIndex) const
{
	if(!stricmp(m_apszLineList[iIndex],g_szLineNone))
	{
		return 0;
	}
	else if(!stricmp(m_apszLineList[iIndex],g_szLineUnknown))
	{
		return m_uInitialSelection;
	}
	else
	{
		return Asura_GetHashID(m_apszLineList[iIndex]);
	}
}

void Tools_PickDialogueLine_Overlay::BuildEventList()
{
	DeleteEventList();
	s_bEventListInvalidated = false;

	
	char szWildcardSearchString[100];
	if(!s_xSearchText.IsBlank() && s_iSearchOption == DIALOGUE_SEARCH_CONTAINS)
	{
		//prepare search string for wildcard search
		Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,s_xSearchText.GetString());
	}

	const Asura_Dialogue_Line* pxSelectedLine = Asura_Dialogue_Storage::GetLineByHash(s_uSelectedLine);


	//get a list of all events that use this line
	int iJunkIndex;
	//get a list of all files and categories used by the samples
	for(u_int i=0;i<Asura_Dialogue_Storage::GetNumberOfEvents();i++)
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(i);

		u_int uNumPlayable = pxEvent->GetNumPlayables();

		bool bUsed = false;
		for(u_int i=0;i<uNumPlayable;i++)
		{
			const Asura_Dialogue_Playable* pxPlayable = pxEvent->GetPlayable(i);
			if(pxPlayable->IsLine())
			{
				const Asura_Dialogue_Line* pxLine = (const Asura_Dialogue_Line*) pxPlayable;
				if(pxLine == pxSelectedLine)
				{
					bUsed = true;
					break;
				}
			}
		}
	
		if(bUsed)
		{
			m_pxEventStringTree->FindNodeIndexByName(pxEvent->GetName(),true,iJunkIndex);
		}
	}

	m_bAnyEvents = m_pxEventStringTree->GetNumberOfNodes() != 0;
	if(!m_bAnyEvents)
	{
		//empty list
		m_pxEventStringTree->FindNodeIndexByName(g_szLineNone,true,iJunkIndex);

	}


	//get array of event names
	m_uEventListSize = m_pxEventStringTree->GetNumberOfNodes();
	m_apszEventList = new const char*[m_uEventListSize];
	m_pxEventStringTree->MakeSortedNameList(m_apszEventList);

	//clamp selection to be in range
	if(m_iEventIndex >= (int)m_uEventListSize) m_iEventIndex = 0;


	//junk the multiple selection of events
	while(m_xEventSelectionList.size()) m_xEventSelectionList.delete_first_entry();


	//update the list dialog if it has been created yet
	if(m_pxEventListDialog)
	{
		m_pxEventListDialog->ReplaceStrings(m_uEventListSize,m_apszEventList);
		if(m_bAnyEvents)
		{
			//select the first item in the list
			m_pxEventListDialog->SetSelection(0);
		}
	}
}

void Tools_PickDialogueLine_Overlay::DeleteEventList()
{
	delete [] m_apszEventList;
	m_apszEventList = 0;
	m_uEventListSize = 0;
	m_pxEventStringTree->Reset();
	m_bAnyEvents = false;
}

Asura_Hash_ID Tools_PickDialogueLine_Overlay::GetEventHash(int iIndex) const
{
	if(!stricmp(m_apszEventList[iIndex],g_szLineNone))
	{
		return 0;
	}
	else
	{
		return Asura_GetHashID(m_apszEventList[iIndex]);
	}
}



void Tools_PickDialogueLine_Overlay::OnOkay()
{
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// ADDED: EP.2.12.2008  In case that this window is open as "View Facial-Animation" also handle facial animation stuff.

	Asura_FacialAnimation_AnimationHandler*	pxAnimationHandler	= Asura_FacialAnimation_System::FindAnimationHandler(ASURA_GUID_TOOLSUSER);
	if(pxAnimationHandler != NULL)
	{
		pxAnimationHandler->StopAllFaceAnimations();
	}

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// END EP


	s_bCancelled = false;
	EndModal();
}

void Tools_PickDialogueLine_Overlay::OnCancel()
{

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// ADDED: EP.2.12.2008  In case that this window is open as "View Facial-Animation" also handle facial animation stuff.

	Asura_FacialAnimation_AnimationHandler*	pxAnimationHandler	= Asura_FacialAnimation_System::FindAnimationHandler(ASURA_GUID_TOOLSUSER);
	if(pxAnimationHandler != NULL)
	{
		pxAnimationHandler->StopAllFaceAnimations();
	}

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// END EP...

	s_bCancelled = true;
	EndModal();
}



void Tools_PickDialogueLine_Overlay::PerFrameUpdate()
{
#ifdef ASURA_USE_ARTICULATION_SYSTEM
	if(m_bRegenAllLines)
	{

		Tools_Timed_Message_System::AddMessage(0x00ff00,"Regenerating line %d of %d All languages", m_iCurrentLine, m_iNumLines);

		const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByIndex(m_iCurrentLine);
		Tools_LipSync::CreateArticulationData( pxLine, pxLine->GetHashID(), false, true );

		m_iCurrentLine++;
		if(m_iCurrentLine >= m_iNumLines)
		{
			m_bRegenAllLines = false;
			m_iNumLines = 0;
			m_iCurrentLine = 0;
		}

		return;		// Nothing else while this is happening
	}
#endif // ASURA_USE_ARTICULATION_SYSTEM

	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// ADDED: EP.2.12.2008  In case that this window is open as "View Facial-Animation" also handle facial animation stuff. BUT THERE IS tricky part!!! because of unknown reason, the line Asura_FacialAnimation_System::Update is not called in it's proper place at all. So I need to substitute it and call it here.
		Asura_FacialAnimation_System::Update();
	#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	Asura_Overlay::PerFrameUpdate();

	if(m_bFilterChanged || s_bLineListInvalidated)
	{

		//rebuild the line list
		BuildLineList();
		m_bFilterChanged = false;
	}

	if(m_bLineChanged)
	{
		//get the hash of the new sample
		m_uMostRecentSelectedLine = s_uSelectedLine = GetLineHash(m_iLineIndex);
		m_bLineChanged = false;

		//If doing multiple selection , get hashes of all in list
		if(m_bMultipleSelection)
		{
			while(s_xSelectionList.size()) s_xSelectionList.delete_first_entry();

			int iNumSelected = m_pxLineListDialog->GetNumSelected();
			for(int i=0;i<iNumSelected;i++)
			{
				int iIndex = m_pxLineListDialog->GetSelection(i);
				Asura_Hash_ID uHash = GetLineHash(iIndex);
				if(uHash) s_xSelectionList.add_entry(uHash);
			}
		}

		UpdatePropertiesText();
		s_bEventListInvalidated = true;

#ifdef ASURA_USE_ARTICULATION_SYSTEM
		s_bHasArticulation = false;
		const Asura_Dialogue_Line* const pxSelectedLine = Asura_Dialogue_Storage::GetLineByHash( s_uSelectedLine );
		if( pxSelectedLine && pxSelectedLine->HasArticulation() )
		{
			s_bHasArticulation = true;
			m_pxCheckboxArticulation->UpdateLabel("Has Articulation");
		}
#endif

		if(s_bAutoPreview)
		{
			PlaySound();
		}
	}

	if(s_bEventListInvalidated)
	{
		BuildEventList();
	}

#ifdef ASURA_USE_ARTICULATION_SYSTEM
	Asura_Dialogue_Line* const pxSelectedLine = Asura_Dialogue_Storage::GetLineByHash( s_uSelectedLine );
	if( pxSelectedLine )
	{
		if ( pxSelectedLine->HasArticulation() != s_bHasArticulation )
		{
			Asura_LipSync_Articulation* pxArt = Asura_LipSync_Manager::GetArticulationByHash(pxSelectedLine->GetHashID());
			if ( !pxArt )
			{
				Tools_LipSync::CreateArticulationData( pxSelectedLine, pxSelectedLine->GetHashID(), false );
				pxArt = Asura_LipSync_Manager::GetArticulationByHash( pxSelectedLine->GetHashID() );
			}
			pxSelectedLine->SetArticulation( pxArt );
		}
	}
#endif

	if(IsModal() && !Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE) && IsEnabled_OnCancel())
		{
			OnCancel();
		}
	}


}


bool Tools_PickDialogueLine_Overlay::IsModal() const
{
	return m_bAlwaysOnTop;
}

void Tools_PickDialogueLine_Overlay::EndModal()
{
	m_bAlwaysOnTop = false;
}

void Tools_PickDialogueLine_Overlay::EnableMultipleSelection(bool bMultipleSelect)
{
	m_bMultipleSelection = bMultipleSelect;
	if(m_pxLineListDialog)
	{
		m_pxLineListDialog->EnableMultipleSelection(m_bMultipleSelection);
	}
	if(!bMultipleSelect)
	{
		while(s_xSelectionList.size())
		{
			s_xSelectionList.delete_first_entry();
		}
	}

}

int Tools_PickDialogueLine_Overlay::GetNumSelected()
{
	return s_bCancelled ? 0 : s_xSelectionList.size();
}

Asura_Hash_ID Tools_PickDialogueLine_Overlay::GetSelection(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex < s_xSelectionList.size(),"Invalid selection index");
	return s_xSelectionList[iIndex];
}

void Tools_PickDialogueLine_Overlay::UpdatePropertiesText()
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_PICKDIALOGUELINE);
	if(!pxOverlay) return;

	const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(s_uSelectedLine);

	char szBuffer[80];

	// only display summary info if we have a single item selected
	if(pxLine && (s_xSelectionList.size() <= 1))
	{
		Asura_PrintfToBfr(szBuffer,"ID : %s",sizeof(szBuffer),true,pxLine->GetName());
		pxOverlay->UpdateDialogLabel(m_pxIDDialog,szBuffer);
		
		const char* szCharacter = pxLine->GetCharacterName();
		Asura_PrintfToBfr(szBuffer,"Character : %s",sizeof(szBuffer),true,szCharacter ? szCharacter : "*None*");
		pxOverlay->UpdateDialogLabel(m_pxCharacterDialog,szBuffer);

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		const char* szFaceType = pxLine->GetFaceTypeName();
		Asura_PrintfToBfr(szBuffer,"Face-type : %s",sizeof(szBuffer),true,szFaceType ? szFaceType : "*None*");
		pxOverlay->UpdateDialogLabel(m_pxFaceTypeDialog,szBuffer);

		const char* szBaseFacePose = pxLine->GetBaseFacePoseName();
		Asura_PrintfToBfr(szBuffer,"Base face-pose : %s",sizeof(szBuffer),true,szBaseFacePose ? szBaseFacePose : "*None*");
		pxOverlay->UpdateDialogLabel(m_pxBaseFacePoseDialog,szBuffer);
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

		const char* szTemplate = pxLine->GetTemplateName();
		Asura_String xMixerName;

#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
		if ( pxLine->GetTemplate() )
		{
			xMixerName = "-----> Mixer : " + pxLine->GetTemplate()->m_xMixingChannelName;
		}
		else
		{
			xMixerName = "-----> Mixer : [Template Not Found!]";
		}
#endif
		// Output template and mixer
		Asura_PrintfToBfr(szBuffer,"Template : %s\n%s",sizeof(szBuffer),true,szTemplate ? szTemplate : "*None*", xMixerName.GetString());
		pxOverlay->UpdateDialogLabel(m_pxTemplateDialog,szBuffer);

		const Asura_UnicodeChar* wszText = pxLine->GetText();
		Asura_PrintfToBfr(szBuffer,"Text : %S",sizeof(szBuffer)-3,true,wszText ? wszText : L"*None*");
		//add ellipsis if too long
		if(strlen(szBuffer) == sizeof(szBuffer)-4)
		{
			strcat(szBuffer,"...");
		}
		//replace newline characters by spaces
		for(char* pcNewLine = strchr(szBuffer,'\n');pcNewLine;pcNewLine = strchr(pcNewLine,'\n'))
		{
			*pcNewLine = ' ';
		}
		pxOverlay->UpdateDialogLabel(m_pxTextDialog,szBuffer);

		//add full text as help text
		//(or as full as will fit in this larget buffer anyway)
		char szHelpText[1024];
		Asura_PrintfToBfr(szHelpText,"%S",sizeof(szHelpText),true,wszText);
		m_pxTextDialog->SetHelpText(szHelpText);
	}
	else
	{
		pxOverlay->UpdateDialogLabel(m_pxIDDialog,"ID : None");
		pxOverlay->UpdateDialogLabel(m_pxCharacterDialog,"Character : N/A");
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		pxOverlay->UpdateDialogLabel(m_pxFaceTypeDialog,"Face-type : N/A");
		pxOverlay->UpdateDialogLabel(m_pxBaseFacePoseDialog,"Base face-pose : N/A");
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		pxOverlay->UpdateDialogLabel(m_pxTemplateDialog,"Template : N/A");
		pxOverlay->UpdateDialogLabel(m_pxTextDialog,"Text : N/A");
		m_pxTextDialog->SetHelpText("");
	}
}

bool Tools_PickDialogueLine_Overlay::IsEnabled_OnOkay()
{
	return IsModal();
}

bool Tools_PickDialogueLine_Overlay::IsEnabled_OnCancel()
{
	return IsModal();
}


void Tools_PickDialogueLine_Overlay::SetInitialSelection(Asura_Hash_ID uSelected)
{
	m_uInitialSelection = uSelected;
	m_uMostRecentSelectedLine = uSelected;

	//make sure the selection will be visible with the current filters.
	const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(uSelected);
	if(pxLine)
	{
		if(s_uCharacterFilter)
		{
			if(s_uCharacterFilter != pxLine->GetCharacter())
			{
				s_uCharacterFilter = 0;
				m_bFilterChanged = true;
			}
		}
		
		if(!s_xSearchText.IsBlank())
		{
			char szWildcardSearchString[100];
			Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,s_xSearchText.GetString());
			if(!DoesLineSatisfySearchFilter(pxLine,szWildcardSearchString))
			{
				s_xSearchText.MakeNull();
				m_bFilterChanged = true;
			}
		}
	}


	BuildLineList();

	UpdatePropertiesText();
	BuildEventList();
} 

bool Tools_PickDialogueLine_Overlay::CanEditTemplate()
{
	return (GetNumSelected() == 1) ? true : false;
}


void Tools_PickDialogueLine_Overlay::OnEditEvent()
{
	//open the selected events for editing
	int iNumSelected = m_pxEventListDialog->GetNumSelected();
	for(int i=0;i<iNumSelected;i++)
	{
		int iIndex = m_pxEventListDialog->GetSelection(i);
		Asura_Hash_ID uHash = GetEventHash(iIndex);
		if(uHash)
		{
			Tools_DialogueEvent_Overlay::EditEvent(uHash);
		}
	}
}

bool Tools_PickDialogueLine_Overlay::CanEditEvent()
{
	return m_bAnyEvents && m_pxEventListDialog->GetNumSelected() != 0;
}

#ifdef ASURA_USE_ARTICULATION_SYSTEM
void Tools_PickDialogueLine_Overlay::GenerateArticulationTextless()
{
	GenerateArticulation(true);
}

void Tools_PickDialogueLine_Overlay::GenerateArticulationText()
{
	GenerateArticulation(false);
}

void Tools_PickDialogueLine_Overlay::RegenerateArticulation()
{
	m_iNumLines = Asura_Dialogue_Storage::GetNumberOfLines();
	m_iCurrentLine = 0;
	m_bRegenAllLines = true;
}

void Tools_PickDialogueLine_Overlay::GenerateArticulation(bool bTextless)
{
	const int iNumSelected = m_pxLineListDialog->GetNumSelected();
	for( int i = 0;i < iNumSelected; i++ )
	{
		const int iIndex = m_pxLineListDialog->GetSelection(i);
		const Asura_Hash_ID uHash = GetLineHash(iIndex);

		Asura_Dialogue_Line* const pxSelectedLine = Asura_Dialogue_Storage::GetLineByHash( uHash );
		if ( pxSelectedLine )
		{
			if ( bTextless )
			{
				Tools_LipSync::CreateArticulationData( pxSelectedLine, pxSelectedLine->GetHashID(), true );
			}
			else
			{
				Tools_LipSync::CreateArticulationData( pxSelectedLine, pxSelectedLine->GetHashID(), false );
			}
			Asura_LipSync_Articulation* const pxArt = Asura_LipSync_Manager::GetArticulationByHash( pxSelectedLine->GetHashID() );
			pxSelectedLine->SetArticulation( pxArt );
		}
	}
}
#endif

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

bool Tools_PickDialogueLine_Overlay::CanAutoGenerateFacialAnimation(void) const
{
	const int	iNumSelected	= GetNumSelected();

	for(int i = 0; i < iNumSelected; i++)
	{
		const Asura_Hash_ID			uHash			= GetSelection(i);
		const Asura_Dialogue_Line*	pxSelectedLine	= Asura_Dialogue_Storage::GetLineByHash(uHash);

		if((pxSelectedLine != NULL) && pxSelectedLine->HasFaceType())
		{
			return(true);
		}
	}
	return(false);
}

void Tools_PickDialogueLine_Overlay::AutoGenerateFacialAnimation(void) const
{
	Tools_FacialAnimation_LipsyncConversion*	pxLipsyncConversion	= Tools_FacialAnimation_System::GetLipsyncConversion();

	if(pxLipsyncConversion == NULL)
	{
		Asura_Assert(false, "Lip-sync conversion method has not being set - cannot auto-generate facial-animation.");
	}
	else
	{
		if(pxLipsyncConversion->BeginConversion())
		{
			const int	iNumSelected	= GetNumSelected();

			for(int i = 0; i < iNumSelected; i++)
			{
				const Asura_Hash_ID			uHash			= GetSelection(i);
				const Asura_Dialogue_Line*	pxSelectedLine	= Asura_Dialogue_Storage::GetLineByHash(uHash);

				// Go through each of the selected lines of dialogue in-turn...
				if(pxSelectedLine != NULL)
				{
					const char*								szFaceTypeName		= pxSelectedLine->GetFaceTypeName();
					const char*								szBaseFacePoseName	= pxSelectedLine->GetBaseFacePoseName();
					const char*								szWAVFilename		= pxSelectedLine->GetSoundFilename();
					Tools_FacialAnimation_FaceAnimation*	pxToolsAnimation	= pxLipsyncConversion->ConvertTextless(szFaceTypeName, szBaseFacePoseName, szWAVFilename);

					// Check whether our conversion was successful...
					if(pxToolsAnimation == NULL)
					{
						Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to create facial-animation from \"%s\"", szWAVFilename);
					}
					else
					{
						const Asura_String							xAnimationName(Tools_FacialAnimation_System::ConvertSoundFilenameToFaceAnimationName(szWAVFilename));

						const Asura_Hash_ID							uAnimationHash		= xAnimationName.GetHash();
						const Asura_FacialAnimation_FaceAnimation*	pxAsuraAnimation	= Asura_FacialAnimation_System::FindFaceAnimation(uAnimationHash);

						if(pxAsuraAnimation == NULL)
						{
							Asura_Assert(false, "Failed to find Asura-animation following auto-generation of face-animation.");
						}
						else
						{
							const Asura_String	xAnimationFilename(Tools_FacialAnimation_System::CreateAnimationFilename(xAnimationName, true));
							const char*			szAnimationFilename	= xAnimationFilename.GetString();

							// Time to save our auto-generated animation...
							if(Tools_FacialAnimation_System::WriteAnimationFile(szAnimationFilename, uAnimationHash, *pxToolsAnimation, *pxAsuraAnimation))
							{
								Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_SUCCESS, "Successfully saved facial-animation \"%s\"", szAnimationFilename);
							}
							else
							{
								Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "Failed to save facial-animation \"%s\"", szAnimationFilename);
							}
						}
					}
				}
			}

			pxLipsyncConversion->EndConversion();
		}
	}
}

bool Tools_PickDialogueLine_Overlay::CanAutoGenerateFacialAnimation(int iUserData)
{
	const Tools_PickDialogueLine_Overlay*	pxThis	= reinterpret_cast<Tools_PickDialogueLine_Overlay*>(iUserData);

	return(pxThis->CanAutoGenerateFacialAnimation());
}

void Tools_PickDialogueLine_Overlay::AutoGenerateFacialAnimation(int iUserData)
{
	const Tools_PickDialogueLine_Overlay*	pxThis	= reinterpret_cast<Tools_PickDialogueLine_Overlay*>(iUserData);

	pxThis->AutoGenerateFacialAnimation();
}

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

void Tools_PickDialogueLine_Overlay::PlaySound()
{
	StopSound();
	
	const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(s_uSelectedLine);
	if(!pxLine) return;

	#if defined ASURA_USE_AUDIO_SYSTEM

		Asura_Hash_ID uWave = pxLine->GetSoundHash();
		if(uWave == ASURA_HASH_ID_UNSET) return;

		// Possibly need to load the sound from disk here. Similar line in
		// Asura_Dialogue_Controller::CreateSound()
		Asura_Audio_System::LoadWave(pxLine->GetSoundFilename(),false);

		m_iSoundHandle = Asura_Audio_System::CreateVoice(uWave,false,false);
		if(m_iSoundHandle == ASURA_INVALID_HANDLE) return;

		Asura_Audio_Voice* const pxVoice = Asura_Audio_System::GetVoice(m_iSoundHandle);

		if(pxLine->GetTemplate())
		{
			pxVoice->SetVolume(pxLine->GetTemplate()->m_fGain);

			if(pxLine->GetTemplate()->GetMixerChannel())
			{
				pxVoice->SetMixerChannel(pxLine->GetTemplate()->GetMixerChannel()->GetHashID());
			}
		}

		// Set voice position to camera to avoid 'no position set' assert (AN 03/04/09)
		pxVoice->SetPos( *Asura_Camera::GetPosition() );
		pxVoice->SetHasPosition( true );
		pxVoice->Start();

	#else

		//get the sound ID to play
		Asura_Sound_ID eSoundID = (Asura_Sound_ID)pxLine->GetSoundID();
		if(eSoundID == ASURA_SID_NO_SOUND) return;

		//play as a 2d sound
		m_iSoundHandle = Asura_Sound_System::CreateSound(eSoundID);
		if(m_iSoundHandle == ASURA_INVALID_HANDLE) return;

		if(pxLine->GetTemplate())
		{
			Asura_Sound_System::SetSoundVolume(m_iSoundHandle,pxLine->GetTemplate()->m_fGain);
		}

		Asura_Sound_System::PlaySound(m_iSoundHandle,false);

	#endif



	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// ADDED: EP.2.12.2008  In case that this window is open as "View Facial-Animation" also handle facial animation stuff. In this case, attempt to play face animation along with sound

		Asura_FacialAnimation_AnimationHandler*	pxAnimationHandler	= Asura_FacialAnimation_System::FindAnimationHandler(ASURA_GUID_TOOLSUSER);
		if(pxAnimationHandler != NULL)
		{
			pxAnimationHandler->PlayFaceAnimation(*pxLine, m_iSoundHandle);
		}

	#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)	// END EP

}

void Tools_PickDialogueLine_Overlay::StopSound()
{
	#if defined ASURA_USE_AUDIO_SYSTEM
		Asura_Audio_System::DestroyVoice(m_iSoundHandle);
	#else
		Asura_Sound_System::DestroySound(m_iSoundHandle);
	#endif
}

bool Tools_PickDialogueLine_Overlay::DoesLineSatisfySearchFilter(const Asura_Dialogue_Line* pxLine,const char* szWildcardSearchString)
{
	//check against the name first
	if(s_iSearchOption == DIALOGUE_SEARCH_BEGINS)
	{
		if(!strnicmp(s_xSearchText.GetString(),pxLine->GetName(),s_xSearchText.GetLength()))
		{
			return true;
		}
	}
	else
	{
		if(Asura_StringUtil::CompareAgainstWildcardString(pxLine->GetName(),szWildcardSearchString,true))
		{
			return true;
		}
	}

	//convert text from unicode , and check that
	char szText[256];
	Asura_PrintfToBfr(szText,"%S",sizeof(szText),true,pxLine->GetText());

	if(s_iSearchOption == DIALOGUE_SEARCH_BEGINS)
	{
		if(!strnicmp(s_xSearchText.GetString(),szText,s_xSearchText.GetLength()))
		{
			return true;
		}
	}
	else
	{
		if(Asura_StringUtil::CompareAgainstWildcardString(szText,szWildcardSearchString,true))
		{
			return true;
		}
	}

	return false;

}

void Tools_PickDialogueLine_Overlay::CreateEventFromLines()
{
	Tools_Dialogue_System::CreateEvent(0,&s_xSelectionList);		
}

void Tools_PickDialogueLine_Overlay::EditTemplate()
{
	// only is there's one selected
	if (s_xSelectionList.size() != 1)
	{
		return;
	}

	const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(s_uSelectedLine);

	if (pxLine && pxLine->GetTemplate() )
	{
		Tools_DialogueEvent_Overlay::EditLineTemplate(pxLine->GetTemplate()->m_uHashID);
	}
	else if ( pxLine )
	{
		Asura_Assertf( false, ( "Template %s does not exist!", pxLine->GetTemplateName() ));
	}
}

void Tools_PickDialogueLine_Overlay::ShowTooltipForLine(int iIndex, const char* szLineName)
{
	if(!szLineName || !*szLineName) return;
	if(!strcmp(szLineName,g_szLineNone) || !strcmp(szLineName,g_szLineUnknown)) return;

	//find the dialogue line from the name
	Asura_Hash_ID uLineHash = Asura_GetHashID(szLineName);
	const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::GetLineByHash(uLineHash);
	if(!pxLine) return;

	//get the text and convert from unicode
	const Asura_UnicodeChar* wszText = pxLine->GetText();
	if(!wszText || !*wszText) return;

	char szHelpText[1024];
	Asura_PrintfToBfr(szHelpText,"%S",sizeof(szHelpText),true,wszText);

	Asura_Overlay_System::SetHelpText(szHelpText);
}


#endif //ASURA_USE_DIALOGUE_SYSTEM
