//
// Tools_PickSoundEvent_Dialog.cpp
//
// Created by Richard Rice - 04/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Sound/Tools_PickSoundEvent_Dialog.h"

#include "Interface/Asura_Overlay.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_Storage.h"

#include "Sound/Tools_PickSoundEvent_Overlay.h"
#include "Sound/Tools_SoundEvent_Overlay.h"


Tools_PickSoundEvent_Dialog::Tools_PickSoundEvent_Dialog(const char* szPrefix,const Asura_Vector_2& xOffset,Asura_Hash_ID* puEventHash,bool* pbModified /*= 0*/)
:Asura_Function_Dialog(szPrefix,xOffset,OnClicked,0)
{
	m_xPrefix = szPrefix;
	m_puEventHash = puEventHash;
	m_uEventBackup = *puEventHash;
	m_pbModified = pbModified;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdateEventLabel();
}

void Tools_PickSoundEvent_Dialog::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the event hash has changed externally
	if(m_uEventBackup != *m_puEventHash)
	{
		m_uEventBackup = *m_puEventHash;
		UpdateEventLabel();
	}
}

void Tools_PickSoundEvent_Dialog::UpdateEventLabel()
{
	char szText[100];
	if(*m_puEventHash == 0)
	{
		Asura_PrintfToBfr(szText,"%s*None*",sizeof(szText),true,m_xPrefix.GetString());
	}
	else
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(*m_puEventHash);
		if(pxEvent)
		{
			Asura_PrintfToBfr(szText,"%s%s",sizeof(szText),true,m_xPrefix.GetString(),pxEvent->GetName());
		}
		else
		{
			Asura_PrintfToBfr(szText,"%s*Unknown* (0x%x)",sizeof(szText),true,m_xPrefix.GetString(),*m_puEventHash);
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}




void Tools_PickSoundEvent_Dialog::OnClicked()
{
	//show the event picking overlay
	Tools_PickSoundEvent_Overlay xOverlay("Select sound event");
	Asura_Hash_ID uHash = xOverlay.DoModal(*m_puEventHash);

	if(uHash != *m_puEventHash)
	{
		*m_puEventHash = uHash;
		m_uEventBackup = uHash;
		if(m_pbModified) *m_pbModified = true;
		UpdateEventLabel();
	}

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_EditSoundEvent_Dialog **
**-----------------------------*/

Tools_EditSoundEvent_Dialog::Tools_EditSoundEvent_Dialog(const char* szName,const Asura_Vector_2& xOffset,Asura_Hash_ID* puEventHash)
:Asura_Function_Dialog(szName,xOffset,OnClicked,0)
{
	m_puEventHash = puEventHash;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	//set some default help text
	SetHelpText("Edit the selected sound event.");
}

void Tools_EditSoundEvent_Dialog::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//disable control if no valid event has been selected
	SetEnabled(Asura_Sound_Event_Storage::GetEventByHash(*m_puEventHash)!=0);
}



void Tools_EditSoundEvent_Dialog::OnClicked()
{
	Tools_SoundEvent_Overlay::EditEvent(*m_puEventHash);
}

//eof
