//
// Tools_PickSoundEvent_Overlay.h
//
// Created by Richard Rice - 04/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PICKSOUNDEVENT_OVERLAY__
#define __TOOLS_PICKSOUNDEVENT_OVERLAY__

#include "Core/Asura_Hash.h"
#include "Core/Asura_Handle.h"
#include "Misc/Tools_Modal_Overlay.h"

class Tools_PickSoundEvent_Overlay : public Tools_Modal_Overlay
{
public :
	Tools_PickSoundEvent_Overlay(const char* szTitle,bool bMultipleSelect = false);
	~Tools_PickSoundEvent_Overlay();

	//Sets default filters to use when opening overlay
	//(If one are set then the most recently used will be kept)
	static void SetFileFilter(const char* szFileFilter){s_xFileFilter = szFileFilter;}
	static void SetCategoryFilter(const char* szCategoryFilter){s_xCategoryFilter = szCategoryFilter;};
	static void SetSubCategoryFilter(const char* szSubCategoryFilter){s_xSubCategoryFilter = szSubCategoryFilter;};

	//The initial selection is ignored if using multiple selection
	Asura_Hash_ID DoModal(Asura_Hash_ID uInitialSelection);

	bool WasCancelled() const {return m_eResult == OVERLAY_CANCEL;}

	//Gets selection if multiple selection is used
	int GetNumSelected() const;
	Asura_Hash_ID GetSelection(int iIndex) const;


private :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	virtual void Update();

	void BuildFilterLists();
	void DeleteFilterLists();
	void BuildSubCategoryList();
	void DeleteSubCategoryList();

	float CalculateDropDownListWidth();

	void BuildEventList();
	void DeleteEventList();
	Asura_Hash_ID GetEventHash(int iIndex) const;

	static void OnOkay();
	static void OnCancel();
	static void OnNewEvent();
	static void OnNewCodeEvent();
	static void OnEditEvent();
	static void OnFindEventByID();

	static void OnDoubleClickList(int iJunk){OnOkay();}

	static void TestSound();
	static void StopSound();

	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_PickSoundEvent_Overlay*)iThis)->__FnName();}\
	void __FnName();
	
	OVERLAY_VOID_CALLBACK(DeleteEvents)

	#undef OVERLAY_VOID_CALLBACK

	enum
	{
		OVERLAY_CANCEL,
		OVERLAY_OKAY,
		OVERLAY_NEWEVENT,
		OVERLAY_NEWCODEEVENT,
		OVERLAY_EDITEVENT,
	} m_eResult;



	Asura_Hash_ID m_uInitialSelection;

	Asura_Hash_ID m_uMostRecentSelectedEvent;//hash of most recent event selected by user
	Asura_Hash_ID m_uSelectedEvent;//hash of currently selected event
	int m_iEventIndex;
	bool m_bEventChanged;

	bool m_bMultipleSelection;
	List<Asura_Hash_ID> m_xSelectionList;


	int m_iSelectedFileFilter;
	int m_iSelectedCategoryFilter;
	int m_iSelectedSubCategoryFilter;
	int m_iSelectedMixerFilter;
	bool m_bFilterChanged;
	bool m_bCategoryChanged;

	//list for file filter
	u_int m_uFileListSize;
	const char** m_apszFileList;
	class Asura_StringTree* m_pxFileStringTree;

	//list for category filter
	u_int m_uCategoryListSize;
	const char** m_apszCategoryList;
	class Asura_StringTree* m_pxCategoryStringTree;

	//list for subcategory filter
	u_int m_uSubCategoryListSize;
	const char** m_apszSubCategoryList;
	class Asura_StringTree* m_pxSubCategoryStringTree;

	//list for mixer filter
	u_int m_uMixerListSize;
	const char** m_apszMixerList;
	class Asura_StringTree* m_pxMixerStringTree;

	//list of event names
	u_int m_uEventListSize;
	const char** m_apszEventList;
	class Asura_StringTree* m_pxEventStringTree;

	class Asura_Dialog* m_pxInputDialog;
	float m_fInputTimer;
	char m_szInput[20];

	//text search filter
	int m_iSearchOption;
	Asura_String m_xSearchText;


	class Asura_List_Dialog* m_pxEventListDialog;
	class Asura_DropDown_List_Dialog* m_pxSubCategoryListDialog;

	Asura_Handle m_iSoundHandle;
	static bool s_bAutoPreview;


	//most recently used filters
	static Asura_String s_xFileFilter;
	static Asura_String s_xCategoryFilter;
	static Asura_String s_xSubCategoryFilter;
	static Asura_String s_xMixerFilter;
	static Asura_String s_xLastFindEventID;
	static bool s_bShowCodeEvent;
	static bool s_bShowNonCodeEvent;


	static Tools_PickSoundEvent_Overlay* s_pxThis;
};

#endif //__TOOLS_PICKSOUNDEVENT_OVERLAY__

//eof
