//
// Tools_PickSoundSample_Overlay.cpp
//
// Created by Richard Rice - 04/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_PickSoundSample_Overlay.h"

#include "Core/Asura_NameTree.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound.h"

#if defined ASURA_USE_AUDIO_SYSTEM
#include "Sound/Asura_Audio_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#endif

//tools
#include "Misc/Tools_Dialog.h"
#include "Sound/Tools_SoundEvent_Overlay.h"
#include "Sound/Tools_SoundEvent_System.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_SoundEvent_System.h"

// Compression quality strings from Tools_Mode_Sound.cpp
extern const char* g_aszCompressionQualityStrings[];

Asura_String Tools_PickSoundSample_Overlay::s_xFileFilter;
Asura_String Tools_PickSoundSample_Overlay::s_xCategoryFilter;
Asura_String Tools_PickSoundSample_Overlay::s_xSubCategoryFilter;
bool Tools_PickSoundSample_Overlay::s_bAutoPreview = true;
Tools_PickSoundSample_Overlay* Tools_PickSoundSample_Overlay::s_pxThis;

static const char* g_szFilterNone = "*All*";
static const char* g_szSampleNone = "*None*";
static const char* g_szSampleUnknown = "*Unknown*";

//search types
enum
{
	DIALOGUE_SEARCH_BEGINS,
	DIALOGUE_SEARCH_CONTAINS,
};

static const char* g_aszSearchOptions[] =
{
	"Begins With",
	"Contains",
};


Tools_PickSoundSample_Overlay::Tools_PickSoundSample_Overlay(const char* szTitle,bool bMultipleSelection)
:Tools_Modal_Overlay(szTitle)
{
	m_uFileListSize = 0;
	m_apszFileList = 0;
	m_pxFileStringTree = new Asura_StringTree;

	m_uCategoryListSize = 0;
	m_apszCategoryList = 0;
	m_pxCategoryStringTree = new Asura_StringTree;

	m_uSubCategoryListSize = 0;
	m_apszSubCategoryList = 0;
	m_pxSubCategoryStringTree = new Asura_StringTree;

	m_uSampleListSize = 0;
	m_apszSampleList = 0;
	m_pxSampleStringTree = new Asura_StringTree;

	m_pxSoundSamplePreview = new Tools_SoundSamplePreview;

	m_bMultipleSelection = bMultipleSelection;

	m_iSearchOption = 0;

	int iSampleArrayIdx = 0;
	int iCompressionArrayIdx = 0;
	for( int iPlatformIdx = 0; iPlatformIdx < TOOLS_PLATFORM_NUM_PLATFORMS; iPlatformIdx++ )
	{
		if( Tools_SoundEvent_System::DoesPlatformUseSampleRate( iPlatformIdx ) && ( iSampleArrayIdx < uNUM_SAMPLERATE_PLATFORMS ) )
		{
			m_aiSamplePlatforms[iSampleArrayIdx] = iPlatformIdx;
			m_aszSamplePlatformNames[iSampleArrayIdx++] = Tools_Platform::GetPlatformName(iPlatformIdx);
		}
		else
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality( iPlatformIdx ) && ( iCompressionArrayIdx < uNUM_COMPRESSION_PLATFORMS ) )
		{
			m_aiCompressionPlatforms[iCompressionArrayIdx] = iPlatformIdx;
			m_aszCompressionPlatformNames[iCompressionArrayIdx++] = Tools_Platform::GetPlatformName(iPlatformIdx);
		}
	}
}

Tools_PickSoundSample_Overlay::~Tools_PickSoundSample_Overlay()
{
	DeleteFilterLists();
	DeleteSampleList();

	delete m_pxFileStringTree;
	delete m_pxCategoryStringTree;
	delete m_pxSubCategoryStringTree;
	delete m_pxSampleStringTree;
	delete m_pxSoundSamplePreview;
}


Asura_Hash_ID Tools_PickSoundSample_Overlay::DoModal(Asura_Hash_ID uInitialSelection)
{
	m_uInitialSelection = uInitialSelection;
	m_uMostRecentSelectedSample = m_uSelectedSample = m_uInitialSelection;
	m_fInputTimer = 0.0f;

	// Reset filters & their flag;
	m_szInput[0] = 0;
	m_iSampleRateFilter = 0;
	m_bSampleFilterEnabled = false;
	m_iCompressionQualityFilter = 0;
	m_bCompressionFilterEnabled = false;

	m_iSamplePlatformIndex = 0;
	m_iCompressionPlatformIndex = 0;

	m_pxSampleListDialog = 0;
	m_pxSubCategoryListDialog = 0;
	m_eResult = OVERLAY_CANCEL;

	s_pxThis = this;
	ShowOverlay();
	StopSound();
	s_pxThis = 0;

	if(m_bMultipleSelection)
	{
		//return value isn't that useful for multuple selection , but default to the first in the list
		m_uSelectedSample = m_xSelectionList.size() ? m_xSelectionList.first_entry() : 0;
	}

	switch(m_eResult)
	{
		case OVERLAY_CANCEL :
			while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();
			return m_uInitialSelection;

		case OVERLAY_OKAY :
			return m_uSelectedSample;

		case OVERLAY_NEWSAMPLE :
		{
			while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();
			Tools_SoundEvent_System::CreateSample(s_xFileFilter.GetString(),s_xCategoryFilter.GetString(),s_xSubCategoryFilter.GetString(),&m_xSelectionList);
			return m_xSelectionList.size() ? m_xSelectionList.first_entry() : m_uInitialSelection;
 		}

		case OVERLAY_EDITSAMPLE :
			if(m_bMultipleSelection)
			{
				for(LIF<Asura_Hash_ID> xSelectionLIF(&m_xSelectionList);!xSelectionLIF.done();xSelectionLIF.next())
				{
					Tools_SoundEvent_Overlay::EditSample(xSelectionLIF());
				}
			}
			else
			{
				if(Asura_Sound_Event_Storage::GetSampleByHash(m_uSelectedSample)) Tools_SoundEvent_Overlay::EditSample(m_uSelectedSample);
			}
			return m_uSelectedSample;
	}

	return m_uInitialSelection;
}

void Tools_PickSoundSample_Overlay::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	BuildFilterLists();
	//make sure the default selection will be visible by the current filters.
	const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByHash(m_uInitialSelection);
	if(pxSample)
	{
		if(!s_xFileFilter.IsBlank())
		{
			const char* szFilename = pxSample->GetFilename();

			if(!szFilename || s_xFileFilter.Compare(szFilename))
			{
				s_xFileFilter.MakeNull();
				m_iSelectedFileFilter = 0;
			}
		}
		if(!s_xCategoryFilter.IsBlank())
		{
			const char* szCategory = pxSample->GetSoundType();
			if(!szCategory || s_xCategoryFilter.Compare(szCategory))
			{
				s_xCategoryFilter.MakeNull();
				m_iSelectedCategoryFilter = 0;
			}
		}
		if(!s_xSubCategoryFilter.IsBlank())
		{
			const char* szSubCategory = pxSample->GetSoundSubType();
			if(!szSubCategory || s_xSubCategoryFilter.Compare(szSubCategory))
			{
				s_xSubCategoryFilter.MakeNull();
				m_iSelectedSubCategoryFilter = 0;
			}
		}

	}
	BuildSampleList();

	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	m_bFilterChanged = false;
	m_bCategoryChanged = false;

	//determine the maximum number of lines to use for the various lists
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const int iMaxLines = Asura_Maths::FloatToInt_Chop((fMaxPrintY - 27.0f)/1.2f);

	float fDropDownListWidth = CalculateDropDownListWidth();

	//add drop down lists for filters
	Asura_DropDown_List_Dialog* pxDropListDialog;
	
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset,m_apszFileList,&m_iSelectedFileFilter,iMaxLines,m_uFileListSize,NULL,&m_bFilterChanged,"File");
	pxDropListDialog->SetHelpText("Filter sample list by the file that they belong to.\nAlso used as tne initial file for new samples.");
	pxDropListDialog->SetMinimumWidth(fDropDownListWidth);
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset,m_apszCategoryList,&m_iSelectedCategoryFilter,iMaxLines,m_uCategoryListSize,NULL,&m_bCategoryChanged,"Category");
	pxDropListDialog->SetHelpText("Filter sample list by the category that they belong to.\nAlso used as tne initial category for new samples.");
	pxDropListDialog->SetMinimumWidth(fDropDownListWidth);
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDropListDialog = m_pxSubCategoryListDialog = new Asura_DropDown_List_Dialog(xOffset,m_apszSubCategoryList,&m_iSelectedSubCategoryFilter,iMaxLines,m_uSubCategoryListSize,NULL,&m_bFilterChanged,"Subcategory");
	pxDropListDialog->SetHelpText("Filter sample list by the subcategory that they belong to.\nAlso used as tne initial subcategory for new samples.");
	pxDropListDialog->SetMinimumWidth(fDropDownListWidth);
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Filters: Sample rate
	{
		const Asura_Char* const szHelpText = "Filters sample list based on sample rate.";
		pxDialog = new Asura_Tickbox_Dialog("Sample rate",xOffset,&m_bSampleFilterEnabled,&m_bFilterChanged);
		pxDialog->SetHelpText(szHelpText);
		pxOverlay->AddDialog(pxDialog);
		const float fXOffset = xOffset.x;
		xOffset.x = pxDialog->GetMaxCoords().x + Asura_Dialog::SmallSpacing();

		pxDialog = new Asura_DropDown_List_Dialog(xOffset,m_aszSamplePlatformNames,&m_iSamplePlatformIndex,uNUM_SAMPLERATE_PLATFORMS,uNUM_SAMPLERATE_PLATFORMS,NULL,&m_bFilterChanged);
		pxDialog->SetHelpText(szHelpText);
		pxDialog->SetIsEnabledFunction(IsFilterEnabled,0);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x = pxDialog->GetMaxCoords().x;

		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_iSampleRateFilter,0,99999,1,"%d",&m_bFilterChanged);
		pxDialog->SetHelpText(szHelpText);
		pxDialog->SetIsEnabledFunction(IsFilterEnabled,0);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x = pxDialog->m_xMaxCoord.x + Asura_Dialog::SmallSpacing();

		const int iPRESET_NUM = 3;
		const int aiPesetValues[iPRESET_NUM] = {11025, 22050, 44100};
		for(int iIdx = 0; iIdx<iPRESET_NUM; iIdx++)
		{
			Asura_Char szFnName[32];
			Asura_PrintfToBfr(szFnName,"%uk",32,true,aiPesetValues[iIdx]/1002);
			pxDialog = new Asura_Function_Dialog(szFnName,xOffset,SetFilterPreset,aiPesetValues[iIdx]);
			pxDialog->SetHelpText(szHelpText);
			pxDialog->SetIsEnabledFunction(IsFilterEnabled,0);

			pxOverlay->AddDialog(pxDialog);
			xOffset.x = pxDialog->m_xMaxCoord.x + 0.3f;
		}

		xOffset.x = fXOffset;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	// Compression quality
	{
		const Asura_Char* const szHelpText = "Filters sample list based on compression quality.";
		pxDialog = new Asura_Tickbox_Dialog("Compression quality",xOffset,&m_bCompressionFilterEnabled,&m_bFilterChanged);
		pxDialog->SetHelpText(szHelpText);
		pxOverlay->AddDialog(pxDialog);
		const float fXOffset = xOffset.x;
		xOffset.x = pxDialog->GetMaxCoords().x + Asura_Dialog::SmallSpacing();

		pxDialog = new Asura_DropDown_List_Dialog(xOffset,m_aszCompressionPlatformNames,&m_iCompressionPlatformIndex,uNUM_COMPRESSION_PLATFORMS,uNUM_COMPRESSION_PLATFORMS,NULL,&m_bFilterChanged);
		pxDialog->SetHelpText(szHelpText);
		pxDialog->SetIsEnabledFunction(IsFilterEnabled,1);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x = pxDialog->GetMaxCoords().x + Asura_Dialog::SmallSpacing();

		for(int iQuality = 0; iQuality<TSS_QUALITY_NUM_ENTRIES; iQuality++)
		{
			int iIconIndex = (g_aszCompressionQualityStrings[iQuality][0] - 'A') + TOOLS_ICON_A;
			pxDialog = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), iIconIndex);
			pxDialog->SetHelpText(szHelpText);
			pxDialog->SetIsEnabledFunction(IsFilterEnabled,1);
			((Asura_Icon_Dialog*)pxDialog)->SetCheckboxFlag(&m_iCompressionQualityFilter,1<<iQuality);

			Asura_Callback_MemberFunctionWithUserData<void,Tools_PickSoundSample_Overlay,int> xCallback(this,&Tools_PickSoundSample_Overlay::UpdateCompressionFlags, iQuality);
			((Asura_Icon_Dialog*)pxDialog)->SetClickedFunction(xCallback);

			pxOverlay->AddDialog(pxDialog);
			xOffset.x = pxDialog->m_xMaxCoord.x + 0.1f;
		}

		xOffset.x = fXOffset;
		xOffset.y += Asura_Dialog::LargeSpacing();
	}

	//Search/Search Type
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszSearchOptions,&m_iSearchOption,iMaxLines,2,NULL,&m_bFilterChanged,"");
	pxDropListDialog->SetHelpText("Determines how the search text should be used for filtering the list.");
	pxOverlay->AddDialog(pxDropListDialog);


	xOffset.x = pxDropListDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_String_Dialog("",xOffset,&m_xSearchText,30,&m_bFilterChanged);
	((Asura_String_Dialog*)pxDialog)->SetUpdateImmediatelyWhileTyping(true);
	pxDialog->SetHelpText("Filters the sample list so that it only shows those beginning with/containing this string");
	xOffset.x = 1.0f;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	

	//control for searching by first few letters in name
	m_pxInputDialog = new Asura_Dialog( "", xOffset );
	pxOverlay->AddDialog( m_pxInputDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	//list of events
	m_bSampleChanged = false;
	pxDialog = m_pxSampleListDialog = new Asura_List_Dialog(xOffset,m_apszSampleList,&m_iSampleIndex,iMaxLines,m_uSampleListSize,NULL,&m_bSampleChanged);
	m_pxSampleListDialog->EnableMultipleSelection(m_bMultipleSelection);
	m_pxSampleListDialog->SetDoubleClickFunction(OnDoubleClickList,0);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += iMaxLines*1.2f + 1.0f;

	pxDialog = new Asura_Dynamic_Dialog(&m_xSampleInfoText,xOffset);
	pxDialog->SetHelpText("Displays information about the currently selected sample");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	UpdateSampleInfoText();


	pxDialog = new Asura_Function_Dialog("Play Sample",xOffset,TestSound);
	pxDialog->SetHelpText("Tests the current sound sample.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Stop Sample",xOffset,StopSound);
	pxDialog->SetHelpText("Stops the test sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;

	xOffset.x += 24.0f;
	pxDialog = new Asura_Tickbox_Dialog("Auto Preview",xOffset,&s_bAutoPreview);
	pxDialog->SetHelpText("Automatically previews each sound as it is selected.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 24.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("New Sample",xOffset,OnNewSample);
	pxDialog->SetHelpText("Create and select a new sample.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Edit Sample",xOffset,OnEditSample);
	pxDialog->SetHelpText("Selects the current sample and opens the overlay for editing it.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;


	xOffset.x += 24.0f;
	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteSamples,(int)this);
	pxDialog->SetHelpText("Delete the selected sample(s).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 24.0f;

	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OnOkay);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,OnCancel);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;



	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_PickSoundSample_Overlay::BuildFilterLists()
{
	DeleteFilterLists();

	int iJunkIndex;
	//get a list of all files and categories used by the events and samples
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfSamples();i++)
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByIndex(i);

		const char* szCategory = pxSample->GetSoundType();
		if(szCategory && *szCategory)
		{
			m_pxCategoryStringTree->FindNodeIndexByName(szCategory,true,iJunkIndex);
		}


		const char* szFilename = pxSample->GetFilename();
		if(szFilename && *szFilename)
		{
			m_pxFileStringTree->FindNodeIndexByName(szFilename,true,iJunkIndex);
		}
	}
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfEvents();i++)
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByIndex(i);

		/*
		const char* szCategory = pxEvent->GetEventType();
		if(szCategory && *szCategory)
		{
			m_pxCategoryStringTree->FindNodeIndexByName(szCategory,true,iJunkIndex);
		}
		*/

		const char* szFilename = pxEvent->GetFilename();
		if(szFilename && *szFilename)
		{
			m_pxFileStringTree->FindNodeIndexByName(szFilename,true,iJunkIndex);
		}
	}


	//get lists from the name tree for the list dialogs
	m_uFileListSize = m_pxFileStringTree->GetNumberOfNodes()+1;
	m_apszFileList = new const char*[m_uFileListSize];
	m_apszFileList[0] = g_szFilterNone;
	m_pxFileStringTree->MakeSortedNameList(m_apszFileList+1);


	m_uCategoryListSize = m_pxCategoryStringTree->GetNumberOfNodes()+1;
	m_apszCategoryList = new const char*[m_uCategoryListSize];
	m_apszCategoryList[0] = g_szFilterNone;
	m_pxCategoryStringTree->MakeSortedNameList(m_apszCategoryList+1);



	//determine the index of the initial selections
	m_iSelectedFileFilter = 0;
	if(!s_xFileFilter.IsBlank())
	{
		for(u_int i=1;i<m_uFileListSize;i++)
		{
			if(!s_xFileFilter.Compare(m_apszFileList[i]))
			{
				m_iSelectedFileFilter = i;
				break;
			}
		}
		if(!m_iSelectedFileFilter) s_xFileFilter.MakeNull();
	}

	m_iSelectedCategoryFilter = 0;
	if(!s_xCategoryFilter.IsBlank())
	{
		for(u_int i=1;i<m_uCategoryListSize;i++)
		{
			if(!s_xCategoryFilter.Compare(m_apszCategoryList[i]))
			{
				m_iSelectedCategoryFilter = i;
				break;
			}
		}
		if(!m_iSelectedCategoryFilter) s_xCategoryFilter.MakeNull();
	}

	BuildSubCategoryList();

}

void Tools_PickSoundSample_Overlay::DeleteFilterLists()
{
	delete [] m_apszFileList;
	m_apszFileList = 0;
	m_uFileListSize = 0;
	m_pxFileStringTree->Reset();

	delete [] m_apszCategoryList;
	m_apszCategoryList = 0;
	m_uCategoryListSize = 0;
	m_pxCategoryStringTree->Reset();

	DeleteSubCategoryList();

}

void Tools_PickSoundSample_Overlay::BuildSubCategoryList()
{
	DeleteSubCategoryList();

	int iJunkIndex;

	//get a list of all subcategories used by the samples with the current category filter
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfSamples();i++)
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByIndex(i);

		//sample uses current category?
		if(!s_xCategoryFilter.IsBlank())
		{
			const char* szCategory = pxSample->GetSoundType();
			if(!szCategory || s_xCategoryFilter.Compare(szCategory))
			{
				continue;
			}
		}
		
		const char* szSubCategory = pxSample->GetSoundSubType();
		if(szSubCategory && *szSubCategory)
		{
			m_pxSubCategoryStringTree->FindNodeIndexByName(szSubCategory,true,iJunkIndex);
		}
	}


	m_uSubCategoryListSize = m_pxSubCategoryStringTree->GetNumberOfNodes()+1;
	m_apszSubCategoryList = new const char*[m_uSubCategoryListSize];
	m_apszSubCategoryList[0] = g_szFilterNone;
	m_pxSubCategoryStringTree->MakeSortedNameList(m_apszSubCategoryList+1);


	//determine the index of the initial selection
	int iSelectedSubCategoryFilter = 0;
	if(!s_xSubCategoryFilter.IsBlank())
	{
		for(u_int i=1;i<m_uSubCategoryListSize;i++)
		{
			if(!s_xSubCategoryFilter.Compare(m_apszSubCategoryList[i]))
			{
				iSelectedSubCategoryFilter = i;
				break;
			}
		}
		if(!iSelectedSubCategoryFilter) s_xSubCategoryFilter.MakeNull();
	}

	//update the list dialog if it has been created yet
	if(m_pxSubCategoryListDialog)
	{
		m_pxSubCategoryListDialog->ReplaceStrings(m_uSubCategoryListSize,m_apszSubCategoryList);
	}
	m_iSelectedSubCategoryFilter = iSelectedSubCategoryFilter;

}

void Tools_PickSoundSample_Overlay::DeleteSubCategoryList()
{
	delete [] m_apszSubCategoryList;
	m_apszSubCategoryList = 0;
	m_uSubCategoryListSize = 0;
	m_pxSubCategoryStringTree->Reset();
}

void Tools_PickSoundSample_Overlay::BuildSampleList()
{
	DeleteSampleList();
	
	char szWildcardSearchString[100];
	if(!m_xSearchText.IsBlank() && m_iSearchOption == DIALOGUE_SEARCH_CONTAINS)
	{
		//prepare search string for wildcard search
		Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,m_xSearchText.GetString());
	}
	
	//get a list of all samples that meet the current filter requirements
	int iJunkIndex;
	//get a list of all files and categories used by the samples
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfSamples();i++)
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByIndex(i);

		if(!s_xFileFilter.IsBlank())
		{
			const char* szFilename = pxSample->GetFilename();
			if(!szFilename || s_xFileFilter.Compare(szFilename))
			{
				continue;
			}
		}

		if(m_bSampleFilterEnabled)
		{
			if( pxSample->GetSampleRate( m_aiSamplePlatforms[m_iSamplePlatformIndex] ) != m_iSampleRateFilter )
			{
				continue;
			}
		}

		if(m_bCompressionFilterEnabled)
		{
			if( pxSample->GetCompressionQuality( m_aiCompressionPlatforms[m_iCompressionPlatformIndex] ) != GetCompressionFromFilter( m_iCompressionQualityFilter ) )
			{
				continue;
			}
		}

		if(!s_xCategoryFilter.IsBlank())
		{
			const char* szCategory = pxSample->GetSoundType();
			if(!szCategory || s_xCategoryFilter.Compare(szCategory))
			{
				continue;
			}
		}
		if(!s_xSubCategoryFilter.IsBlank())
		{
			const char* szSubCategory = pxSample->GetSoundSubType();
			if(!szSubCategory || s_xSubCategoryFilter.Compare(szSubCategory))
			{
				continue;
			}
		}

		if(!m_xSearchText.IsBlank())
		{
			if(m_iSearchOption == DIALOGUE_SEARCH_BEGINS)
			{
				if(strnicmp(m_xSearchText.GetString(),pxSample->GetName(),m_xSearchText.GetLength()))
				{
					continue;
				}
			}
			else
			{
				if(!Asura_StringUtil::CompareAgainstWildcardString(pxSample->GetName(),szWildcardSearchString,true))
				{
					continue;
				}
			}
		}

		m_pxSampleStringTree->FindNodeIndexByName(pxSample->GetName(),true,iJunkIndex);
	}

	//need an entry for 'unknown' ? 
	bool bNeedUnknown = false;
	if(m_uInitialSelection && !Asura_Sound_Event_Storage::GetSampleByHash(m_uInitialSelection))
	{
		bNeedUnknown = true;
	}

	u_int uSpecialEntries = bNeedUnknown ? 2 : 1;

	//get array of sample names , and fill in the special samples that arne't in the tree
	m_uSampleListSize = m_pxSampleStringTree->GetNumberOfNodes()+uSpecialEntries;
	m_apszSampleList = new const char*[m_uSampleListSize];
	m_apszSampleList[0] = g_szSampleNone;
	if(bNeedUnknown) m_apszSampleList[1] = g_szSampleUnknown;
	m_pxSampleStringTree->MakeSortedNameList(m_apszSampleList+uSpecialEntries);

	//Update the current selected index to be as close as possible to the previous selection
	if(m_uMostRecentSelectedSample == 0)
	{
		m_iSampleIndex = 0;
	}
	else if(bNeedUnknown && m_uMostRecentSelectedSample == m_uInitialSelection)
	{
		m_iSampleIndex = 1;
	}
	else
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByHash(m_uMostRecentSelectedSample);
		m_iSampleIndex = 0;
		if(pxSample)
		{
			for(u_int i=uSpecialEntries;i<m_uSampleListSize;i++)
			{
				if(stricmp(pxSample->GetName(),m_apszSampleList[i])>=0)
				{
					m_iSampleIndex = i;
				}
				else
				{
					break;
				}
			}
		}
	}

	//get the hash ID of the sample selected
	m_uSelectedSample = GetSampleHash(m_iSampleIndex);

	//multiple selection list will be junk after list change
	while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();

	//update the list dialog if it has been created yet
	if(m_pxSampleListDialog)
	{
		m_pxSampleListDialog->ReplaceStrings(m_uSampleListSize,m_apszSampleList);
	}
}

Asura_Hash_ID Tools_PickSoundSample_Overlay::GetSampleHash(int iIndex) const
{
	if(!stricmp(m_apszSampleList[iIndex],g_szSampleNone))
	{
		return 0;
	}
	else if(!stricmp(m_apszSampleList[iIndex],g_szSampleUnknown))
	{
		return m_uInitialSelection;
	}
	else
	{
		return Asura_GetHashID(m_apszSampleList[iIndex]);
	}
}

void Tools_PickSoundSample_Overlay::DeleteSampleList()
{
	delete [] m_apszSampleList;
	m_apszSampleList = 0;
	m_uSampleListSize = 0;
	m_pxSampleStringTree->Reset();
}


void Tools_PickSoundSample_Overlay::OnOkay()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis->m_eResult = OVERLAY_OKAY;
	s_pxThis->GetOverlay()->m_bShouldClose = true;			

}

void Tools_PickSoundSample_Overlay::OnCancel()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis->m_eResult = OVERLAY_CANCEL;
	s_pxThis->GetOverlay()->m_bShouldClose = true;			
}

void Tools_PickSoundSample_Overlay::OnNewSample()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis->m_eResult = OVERLAY_NEWSAMPLE;
	s_pxThis->GetOverlay()->m_bShouldClose = true;			
}

void Tools_PickSoundSample_Overlay::OnEditSample()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis->m_eResult = OVERLAY_EDITSAMPLE;
	s_pxThis->GetOverlay()->m_bShouldClose = true;			
}



void Tools_PickSoundSample_Overlay::Update()
{
	m_pxSoundSamplePreview->Update();

	if(m_bCategoryChanged)
	{
		if(m_iSelectedCategoryFilter)
		{
			s_xCategoryFilter = m_apszCategoryList[m_iSelectedCategoryFilter];
		}
		else
		{
			s_xCategoryFilter.MakeNull();
		}

		BuildSubCategoryList();
		m_bCategoryChanged = false;
		m_bFilterChanged = true;
	}

	if(m_bFilterChanged)
	{
		if(m_iSelectedFileFilter)
		{
			s_xFileFilter = m_apszFileList[m_iSelectedFileFilter];
		}
		else
		{
			s_xFileFilter.MakeNull();
		}
		

		if(m_iSelectedSubCategoryFilter)
		{
			s_xSubCategoryFilter = m_apszSubCategoryList[m_iSelectedSubCategoryFilter];
		}
		else
		{
			s_xSubCategoryFilter.MakeNull();
		}
 
		//rebuild the sample list
		BuildSampleList();
		m_bFilterChanged = false;
	}

	if(m_fInputTimer>0)
	{
		//update timer to see if next key pressed should be considered to have been done quickly enough
		//to be part of the same string
		m_fInputTimer -= Asura_Timers::GetFrameTime();
		if(m_fInputTimer<=0)
		{ 
			m_fInputTimer = 0;
			m_szInput[0] = 0;
			m_pxInputDialog->UpdateLabel(m_szInput);
		}
	}
	
	if(!Asura_Overlay_System::GetFocusGadget())
	{
		//see if any letter key has been pressed
		char cKey = Asura_Gadget::GetAsciiKeyPressed();

		if(cKey)
		{
			int iLength = strlen(m_szInput);
			if(iLength<sizeof(m_szInput)-1)
			{
				m_szInput[iLength++] = cKey;
				m_szInput[iLength++] = 0;
				m_fInputTimer = 0.5f;
				m_pxInputDialog->UpdateLabel(m_szInput);

			}

			//advance list to first entry greater than the current input string
			for(u_int i=0;i<m_uSampleListSize;i++)
			{
				if( stricmp(m_apszSampleList[i], m_szInput) >= 0 )
				{
					//move selection to this
					m_iSampleIndex = i;
					m_bSampleChanged = true;
					break;
				}
			}
		}
	}

	if(m_bSampleChanged)
	{
		//get the hash of the new sample
		m_uMostRecentSelectedSample = m_uSelectedSample = GetSampleHash(m_iSampleIndex);
		m_bSampleChanged = false;

		//If doing multiple selection , get hashes of all in list
		if(m_bMultipleSelection)
		{
			while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();

			int iNumSelected = m_pxSampleListDialog->GetNumSelected();
			for(int i=0;i<iNumSelected;i++)
			{
				int iIndex = m_pxSampleListDialog->GetSelection(i);
				Asura_Hash_ID uHash = GetSampleHash(iIndex);
				if(uHash) m_xSelectionList.add_entry(uHash);
			}
		}

		UpdateSampleInfoText();

		if(s_bAutoPreview)
		{
			TestSound();
		}
	}

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER))
	{
		GetOverlay()->m_bShouldClose = true;			
	}
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
	{
		GetOverlay()->m_bShouldClose = true;			
	}

}

void Tools_PickSoundSample_Overlay::TestSound()
{
	if(!s_pxThis) return;

	s_pxThis->m_pxSoundSamplePreview->Play(s_pxThis->m_uSelectedSample);
}

void Tools_PickSoundSample_Overlay::StopSound()
{
	if(!s_pxThis) return;
	s_pxThis->m_pxSoundSamplePreview->Stop();
}


int Tools_PickSoundSample_Overlay::GetNumSelected() const
{
	Asura_Assert(m_bMultipleSelection,"Multiple selection not enabled");
	return m_xSelectionList.size();
}

Asura_Hash_ID Tools_PickSoundSample_Overlay::GetSelection(int iIndex) const
{
	Asura_Assert(m_bMultipleSelection,"Multiple selection not enabled");
	Asura_Assert(iIndex>=0 && iIndex < m_xSelectionList.size(),"Invalid selection index");
	return m_xSelectionList[iIndex];
}

void Tools_PickSoundSample_Overlay::DeleteSamples()
{
	if(!ShowYesNoOverlay("Delete the selected samples?")) return;

	if(m_bMultipleSelection)
	{
		while(m_xSelectionList.size())
		{
			Tools_SoundEvent_System::DeleteSample(m_xSelectionList.first_entry(),false);
			m_xSelectionList.delete_first_entry();
		}
	}
	else
	{
		Tools_SoundEvent_System::DeleteSample(m_uSelectedSample,false);
	}
	//flag the filter as changed so that the deleted entries will be removed.
	m_bFilterChanged = true;
}



float Tools_PickSoundSample_Overlay::CalculateDropDownListWidth()
{
	//get the max width of all the names that will appear in the drop down lists , so that they can be made the same size
	float fWidth = 0.0f;

	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfSamples();i++)
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByIndex(i);

		
		const char* szCategory = pxSample->GetSoundType();
		if(szCategory && *szCategory)
		{
			fWidth = Asura_Maths::Max(Asura_DropDown_List_Dialog::GetWidthForString(szCategory), fWidth);
		}

		const char* szSubCategory = pxSample->GetSoundSubType();
		if(szSubCategory && *szSubCategory)
		{
			fWidth = Asura_Maths::Max(Asura_DropDown_List_Dialog::GetWidthForString(szSubCategory), fWidth);
		}

		const char* szFilename = pxSample->GetFilename();
		if(szFilename && *szFilename)
		{
			fWidth = Asura_Maths::Max(Asura_DropDown_List_Dialog::GetWidthForString(szFilename), fWidth);
		}
	}
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfEvents();i++)
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByIndex(i);

		
		const char* szCategory = pxEvent->GetEventType();
		if(szCategory && *szCategory)
		{
			fWidth = Asura_Maths::Max(Asura_DropDown_List_Dialog::GetWidthForString(szCategory), fWidth);
		}

		const char* szSubCategory = pxEvent->GetEventSubType();
		if(szSubCategory && *szSubCategory)
		{
			fWidth = Asura_Maths::Max(Asura_DropDown_List_Dialog::GetWidthForString(szSubCategory), fWidth);
		}

		const char* szFilename = pxEvent->GetFilename();
		if(szFilename && *szFilename)
		{
			fWidth = Asura_Maths::Max(Asura_DropDown_List_Dialog::GetWidthForString(szFilename), fWidth);
		}
	}
	
	return fWidth;
}

void Tools_PickSoundSample_Overlay::UpdateSampleInfoText()
{
#if defined ASURA_USE_AUDIO_SYSTEM
	const Asura_Sound_Sample* const pxSample = Asura_Sound_Event_Storage::GetSampleByHash(m_uSelectedSample);
	m_xSampleInfoText = "Wav file name: ";

	if( pxSample )
	{
		const char* szFilename = Asura_Audio_System::GetWaveFilename( pxSample->GetWaveHash() );
		szFilename = Tools_FileSystem::StripPath( szFilename );
		m_xSampleInfoText += szFilename;
	}
#endif // ASURA_USE_AUDIO_SYSTEM
}

bool Tools_PickSoundSample_Overlay::IsFilterEnabled( int iFilterIdx )
{
	if( !s_pxThis )
	{
		return false;
	}

	switch( iFilterIdx )
	{
		case 0: return s_pxThis->m_bSampleFilterEnabled;
		case 1: // ...
		default: return s_pxThis->m_bCompressionFilterEnabled;
	}
}

void Tools_PickSoundSample_Overlay::SetFilterPreset( int iPresetValue )
{
	if( !s_pxThis )
	{
		return;
	}

	s_pxThis->m_iSampleRateFilter = iPresetValue;
	s_pxThis->m_bFilterChanged = true;
}

int Tools_PickSoundSample_Overlay::GetCompressionFromFilter( int iFilter )
{
	int iRet = -1;
	while( iFilter )
	{
		iFilter >>= 1;
		iRet++;
	}

	return iRet;
}

void Tools_PickSoundSample_Overlay::UpdateCompressionFlags(int iQualityIdx)
{
	m_iCompressionQualityFilter = 1<<iQualityIdx;
	m_bFilterChanged = true;
}

//eof
