//////////////////////////////////////////////////////////////////////////////
//
// Tools_ReverbPresets.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __TOOLS_REVERBPRESETS_H__
#define __TOOLS_REVERBPRESETS_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Debugging/Asura_Assert.h"

/******************************************************************************/
/* Structures                                                                 */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Tools_ReverbPreset
//
// Similar to Asura_Audio_Reverb, but just the data values, for storing presets
//
////////////////////////////////////////////////////////////////////////////////

struct Tools_ReverbPreset
{
	int   m_iRoom;
	int   m_iRoomHF;
	float m_fRoomRolloffFactor;
	float m_fDecayTime;
	float m_fDecayHFRatio;
	int   m_iReflections;
	float m_fReflectionsDelay;
	int   m_iReverb;
	float m_fReverbDelay;
	float m_fDiffusion;
	float m_fDensity;
	float m_fHFReference;
};

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

// Keeping these in a separate header file to avoid bloating up others. Ensure the order here
// matches the array below, and the order from the old system

const char* const aszTOOLS_REVERB_PRESET_NAMES[] =
{
	"None",
	"Generic",
	"Padded cell",
	"Room",
	"Bathroom",
	"Livingroom",
	"Stoneroom",
	"Auditorium",
	"Concert hall",
	"Cave",
	"Arena",
	"Hangar",
	"Carpetted hallway",
	"Hallway",
	"Stone corridor",
	"Alley",
	"Forest",
	"City",
	"Mountains",
	"Quarry",
	"Plain",
	"Parking lot",
	"Sewer pipe",
	"Underwater",
	"Drugged",
	"Dizzy",
	"Psychotic",
	"Castle small room",
	"Castle short passage",
	"Castle medium room",
	"Castle long passage",
	"Castle large room",
	"Castle hall",
	"Castle cupboard",
	"Castle courtyard",
	"Castle alcove",
	"Factory alcove",
	"Factory short passage",
	"Factory medium room",
	"Factory long passage",
	"Factory large room",
	"Factory hall",
	"Factory cupboard",
	"Factory courtyard",
	"Factory smallroom",
	"Ice palace alcove",
	"Ice palace short passage",
	"Ice palace medium room",
	"Ice palace long passage",
	"Ice palace large room",
	"Ice palace hall",
	"Ice palace cupboard",
	"Ice palace courtyard",
	"Ice palace smallroom",
	"Space station alcove",
	"Space station medium room",
	"Space station short passage",
	"Space station long passage",
	"Space station large room",
	"Space station hall",
	"Space station cupboard",
	"Space station small room",
	"Wooden alcove",
	"Wooden short passage",
	"Wooden medium room",
	"Wooden long passage",
	"Wooden large room",
	"Wooden hall",
	"Wooden cupboard",
	"Wooden smallroom",
	"Wooden courtyard",
	"Sport empty stadium",
	"Sport squash court",
	"Sport small swimming pool",
	"Sport large swimming pool",
	"Sport gymnasium",
	"Sport full stadium",
	"Sport stadium tannoy",
	"Prefab workshop",
	"Prefab schoolroom",
	"Prefab practise room",
	"Prefab outhouse",
	"Prefab caravan",
	"Dome tomb",
	"Dome Saint Paul's",
	"Pipe small",
	"Pipe long thin",
	"Pipe large",
	"Pipe resonant",
	"Outdoors backyard",
	"Outdoors rolling plains",
	"Outdoors deep canyon",
	"Outdoors creek",
	"Outdoors valley",
	"Mood Heaven",
	"Mood Hell",
	"Mood memory",
	"Driving commentator",
	"Driving pit garage",
	"Driving in-car racer",
	"Driving in-car sports",
	"Driving in-car luxury",
	"Driving full grandstand",
	"Driving empty grandstand",
	"Driving tunnel",
	"City streets",
	"City subway",
	"City museum",
	"City library",
	"City underpass",
	"City abandoned",
	"Dustyroom",
	"Chapel",
	"Small water room",
};

const u_int uTOOLS_NUM_REVERB_PRESET_NAMES =
	sizeof( aszTOOLS_REVERB_PRESET_NAMES ) / sizeof( aszTOOLS_REVERB_PRESET_NAMES[0] );

const Tools_ReverbPreset axTOOLS_REVERB_PRESETS[] =
{
	{-10000,    0,0.0f, 1.00f,0.50f,-10000,0.020f,-10000,0.040f,100.0f,100.0f,5000.0f}, // Default
	{ -1000, -100,0.0f, 1.49f,0.83f, -2602,0.007f,   200,0.011f,100.0f,100.0f,5000.0f}, // Generic
	{ -1000,-6000,0.0f, 0.17f,0.10f, -1204,0.001f,   207,0.002f,100.0f,100.0f,5000.0f}, // Padded cell
	{ -1000, -454,0.0f, 0.40f,0.83f, -1646,0.002f,    53,0.003f,100.0f,100.0f,5000.0f}, // Room
	{ -1000,-1200,0.0f, 1.49f,0.54f,  -370,0.007f,  1030,0.011f,100.0f, 60.0f,5000.0f}, // Bathroom
	{ -1000,-6000,0.0f, 0.50f,0.10f, -1376,0.003f, -1104,0.004f,100.0f,100.0f,5000.0f}, // Living room
	{ -1000, -300,0.0f, 2.31f,0.64f,  -711,0.012f,    83,0.017f,100.0f,100.0f,5000.0f}, // Stone room
	{ -1000, -476,0.0f, 4.32f,0.59f,  -789,0.020f,  -289,0.030f,100.0f,100.0f,5000.0f}, // Auditorium
	{ -1000, -500,0.0f, 3.92f,0.70f, -1230,0.020f,    -2,0.029f,100.0f,100.0f,5000.0f}, // Concert hall
	{ -1000,    0,0.0f, 2.91f,1.30f,  -602,0.015f,  -302,0.022f,100.0f,100.0f,5000.0f}, // Cave
	{ -1000, -698,0.0f, 7.24f,0.33f, -1166,0.020f,    16,0.030f,100.0f,100.0f,5000.0f}, // Arena
	{ -1000,-1000,0.0f,10.05f,0.23f,  -602,0.020f,   198,0.030f,100.0f,100.0f,5000.0f}, // Hangar
	{ -1000,-4000,0.0f, 0.30f,0.10f, -1831,0.002f, -1630,0.030f,100.0f,100.0f,5000.0f}, // Carpetted hallway
	{ -1000, -300,0.0f, 1.49f,0.59f, -1219,0.007f,   441,0.011f,100.0f,100.0f,5000.0f}, // Hallway
	{ -1000, -237,0.0f, 2.70f,0.79f, -1214,0.013f,   395,0.020f,100.0f,100.0f,5000.0f}, // Stone corridor
	{ -1000, -270,0.0f, 1.49f,0.86f, -1204,0.007f,    -4,0.011f,100.0f,100.0f,5000.0f}, // Alley
	{ -1000,-3300,0.0f, 1.49f,0.54f, -2560,0.162f,  -613,0.088f, 79.0f,100.0f,5000.0f}, // Forest
	{ -1000, -800,0.0f, 1.49f,0.67f, -2273,0.007f, -2217,0.011f, 50.0f,100.0f,5000.0f}, // City
	{ -1000,-2500,0.0f, 1.49f,0.21f, -2780,0.300f, -2014,0.100f, 27.0f,100.0f,5000.0f}, // Mountains
	{ -1000,-1000,0.0f, 1.49f,0.83f,-10000,0.061f,   500,0.025f,100.0f,100.0f,5000.0f}, // Quarry
	{ -1000,-2000,0.0f, 1.49f,0.50f, -2466,0.179f, -2514,0.100f, 21.0f,100.0f,5000.0f}, // Plain
	{ -1000,    0,0.0f, 1.65f,1.50f, -1363,0.008f, -1153,0.012f,100.0f,100.0f,5000.0f}, // Parking lot
	{ -1000,-1000,0.0f, 2.81f,0.14f,   429,0.014f,   648,0.021f, 80.0f, 60.0f,5000.0f}, // Sewer pipe
	{ -1000,-4000,0.0f, 1.49f,0.10f,  -449,0.007f,  1700,0.011f,100.0f,100.0f,5000.0f}, // Underwater

	{ -1000,    0, 0.00f,  8.39f, 1.39f,  -115, 0.002f,  985, 0.030f, 50.0f, 100.0f, 5000.0f}, // Drugged
	{ -1000, -400, 0.00f, 17.23f, 0.56f, -1713, 0.020f, -613, 0.030f, 60.0f, 100.0f, 5000.0f}, // Dizzy
	{ -1000, -151, 0.00f,  7.56f, 0.91f,  -626, 0.020f,  774, 0.030f, 50.0f, 100.0f, 5000.0f}, // Psychotic

	{ -1000,  -800, 0.00f, 1.22f, 0.83f,  -100, 0.022f,  600, 0.011f, 89.0f, 100.0f, 5168.6f}, // Castle small room
	{ -1000, -1000, 0.00f, 2.32f, 0.83f,  -100, 0.007f,  200, 0.023f, 89.0f, 100.0f, 5168.6f}, // Castle short passage
	{ -1000, -1100, 0.00f, 2.04f, 0.83f,  -400, 0.022f,  400, 0.011f, 93.0f, 100.0f, 5168.6f}, // Castle medium room
	{ -1000,  -800, 0.00f, 3.42f, 0.83f,  -100, 0.007f,  300, 0.023f, 89.0f, 100.0f, 5168.6f}, // Castle long passage
	{ -1000, -1100, 0.00f, 2.53f, 0.83f,  -700, 0.034f,  200, 0.016f, 82.0f, 100.0f, 5168.6f}, // Castle large room
	{ -1000, -1100, 0.00f, 3.14f, 0.79f, -1500, 0.056f,  100, 0.024f, 81.0f, 100.0f, 5168.6f}, // Castle hall
	{ -1000, -1100, 0.00f, 0.67f, 0.87f,   300, 0.010f, 1100, 0.007f, 89.0f, 100.0f, 5168.6f}, // Castle cupboard
	{ -1000,  -700, 0.00f, 2.13f, 0.61f, -1300, 0.160f, -300, 0.036f, 42.0f, 100.0f, 5000.0f}, // Castle courtyard
	{ -1000,  -600, 0.00f, 1.64f, 0.87f,     0, 0.007f,  300, 0.034f, 89.0f, 100.0f, 5168.6f}, // Castle alcove

	{ -1200,  -200, 0.00f, 3.14f, 0.65f,   300, 0.010f,    0, 0.038f, 59.0f, 100.0f, 3762.6f}, // Factory alcove
	{ -1200,  -200, 0.00f, 2.53f, 0.65f,     0, 0.010f,  200, 0.038f, 64.0f, 100.0f, 3762.6f}, // Factory short passage
	{ -1200,  -200, 0.00f, 2.76f, 0.65f, -1100, 0.022f,  300, 0.023f, 82.0f, 100.0f, 3762.6f}, // Factory medium room
	{ -1200,  -200, 0.00f, 4.06f, 0.65f,     0, 0.020f,  200, 0.037f, 64.0f, 100.0f, 3762.6f}, // Factory long passage
	{ -1200,  -300, 0.00f, 4.24f, 0.51f, -1500, 0.039f,  100, 0.023f, 75.0f, 100.0f, 3762.6f}, // Factory large room
	{ -1000,  -300, 0.00f, 7.43f, 0.51f, -2400, 0.073f, -100, 0.027f, 75.0f, 100.0f, 3762.6f}, // Factory hall
	{ -1200,  -200, 0.00f, 0.49f, 0.65f,   200, 0.010f,  600, 0.032f, 63.0f, 100.0f, 3762.6f}, // Factory cupboard
	{ -1000, -1000, 0.00f, 2.32f, 0.29f, -1300, 0.140f, -800, 0.039f, 57.0f, 100.0f, 3762.6f}, // Factory courtyard
	{ -1000,  -200, 0.00f, 1.72f, 0.65f,  -300, 0.010f,  500, 0.024f, 82.0f, 100.0f, 3762.6f}, // Factory small room

	{ -1000,  -500, 0.00f, 2.76f, 1.46f,   100, 0.010f,  -100, 0.030f, 84.0f, 100.0f, 12428.5f}, // Ice palace alcove
	{ -1000,  -500, 0.00f, 1.79f, 1.46f,  -600, 0.010f,   100, 0.019f, 75.0f, 100.0f, 12428.5f}, // Ice palace short passage
	{ -1000,  -500, 0.00f, 2.22f, 1.53f,  -800, 0.039f,   100, 0.027f, 87.0f, 100.0f, 12428.5f}, // Ice palace medium room
	{ -1000,  -500, 0.00f, 3.01f, 1.46f,  -200, 0.012f,   200, 0.025f, 77.0f, 100.0f, 12428.5f}, // Ice palace long passage
	{ -1000,  -500, 0.00f, 3.14f, 1.53f, -1200, 0.039f,     0, 0.027f, 81.0f, 100.0f, 12428.5f}, // Ice palace large room
	{ -1000,  -700, 0.00f, 5.49f, 1.53f, -1900, 0.054f,  -400, 0.052f, 76.0f, 100.0f, 12428.5f}, // Ice palace hall
	{ -1000,  -600, 0.00f, 0.76f, 1.53f,   100, 0.012f,   600, 0.016f, 83.0f, 100.0f, 12428.5f}, // Ice palace cupboard
	{ -1000, -1100, 0.00f, 2.04f, 1.20f, -1000, 0.173f, -1000, 0.043f, 59.0f, 100.0f, 12428.5f}, // Ice palace courtyard
	{ -1000,  -500, 0.00f, 1.51f, 1.53f,  -100, 0.010f,   300, 0.011f, 84.0f, 100.0f, 12428.5f}, // Ice palace small room

	{ -1000, -300, 0.00f, 1.16f, 0.81f,   300, 0.007f,    0, 0.018f, 78.0f, 100.0f, 3316.1f}, // Space station alcove
	{ -1000, -400, 0.00f, 3.01f, 0.50f,  -800, 0.034f,  100, 0.035f, 75.0f, 100.0f, 3316.1f}, // Space station medium room
	{ -1000, -400, 0.00f, 3.57f, 0.50f,     0, 0.012f,  100, 0.016f, 87.0f, 100.0f, 3316.1f}, // Space station short passage
	{ -1000, -400, 0.00f, 4.62f, 0.62f,     0, 0.012f,  200, 0.031f, 82.0f, 100.0f, 3316.1f}, // Space station long passage
	{ -1000, -400, 0.00f, 3.89f, 0.38f, -1000, 0.056f, -100, 0.035f, 81.0f, 100.0f, 3316.1f}, // Space station large room
	{ -1000, -400, 0.00f, 7.11f, 0.38f, -1500, 0.100f, -400, 0.047f, 87.0f, 100.0f, 3316.1f}, // Space station hall
	{ -1000, -300, 0.00f, 0.79f, 0.81f,   300, 0.007f,  500, 0.018f, 56.0f, 100.0f, 3316.1f}, // Space station cupboard
	{ -1000, -300, 0.00f, 1.72f, 0.82f,  -200, 0.007f,  300, 0.013f, 70.0f, 100.0f, 3316.1f}, // Space station small room

	{ -1000, -1800, 0.00f, 1.22f, 0.62f,  100, 0.012f,  -300, 0.024f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon alcove
	{ -1000, -1800, 0.00f, 1.75f, 0.50f, -100, 0.012f,  -400, 0.024f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon short passage
	{ -1000, -2000, 0.00f, 1.47f, 0.42f, -100, 0.049f,  -100, 0.029f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon medium room
	{ -1000, -2000, 0.00f, 1.99f, 0.40f,    0, 0.020f,  -700, 0.036f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon long passage
	{ -1000, -2100, 0.00f, 2.65f, 0.33f, -100, 0.066f,  -200, 0.049f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon large room
	{ -1000, -2200, 0.00f, 3.45f, 0.30f, -100, 0.088f,  -200, 0.063f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon hall
	{ -1000, -1700, 0.00f, 0.56f, 0.46f,  100, 0.012f,   100, 0.028f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon cupboard
	{ -1000, -1900, 0.00f, 0.79f, 0.32f,    0, 0.032f,  -100, 0.029f, 100.0f, 100.0f, 4705.0f}, // Wooden galleon small room
	{ -1000, -2200, 0.00f, 1.79f, 0.35f, -500, 0.123f, -2000, 0.032f,  65.0f, 100.0f, 4705.0f}, // Wooden galleon courtyard

	{ -1000,  -700, 0.00f, 6.26f, 0.51f, -2400, 0.183f,  -800, 0.038f, 100.0f, 100.0f, 5000.0f}, // Sports empty stadium
	{ -1000, -1000, 0.00f, 2.22f, 0.91f,  -700, 0.007f,  -200, 0.011f,  75.0f, 100.0f, 7176.9f}, // Sports squash court
	{ -1000,  -200, 0.00f, 2.76f, 1.25f,  -400, 0.020f,  -200, 0.030f,  70.0f, 100.0f, 5000.0f}, // Sports small swimming pool
	{ -1000,  -200, 0.00f, 5.49f, 1.31f,  -700, 0.039f,  -600, 0.049f,  82.0f, 100.0f, 5000.0f}, // Sports large swimming pool
	{ -1000,  -700, 0.00f, 3.14f, 1.06f,  -800, 0.029f,  -500, 0.045f,  81.0f, 100.0f, 7176.9f}, // Sports gymnasium
	{ -1000, -2300, 0.00f, 5.25f, 0.17f, -2000, 0.188f, -1100, 0.038f, 100.0f, 100.0f, 5000.0f}, // Sports full stadium
	{ -1000,  -500, 0.00f, 2.53f, 0.88f, -1100, 0.230f,  -600, 0.063f,  78.0f, 100.0f, 5000.0f}, // Sports stadium tannoy

	{ -1000, -1700, 0.00f, 0.76f, 1.00f,    0, 0.012f,  100, 0.012f, 100.0f, 100.0f, 5000.0f}, // Prefab workshop
	{ -1000,  -400, 0.00f, 0.98f, 0.45f,  300, 0.017f,  300, 0.015f,  69.0f, 100.0f, 7176.9f}, // Prefab schoolroom
	{ -1000,  -800, 0.00f, 1.12f, 0.56f,  200, 0.010f,  300, 0.011f,  87.0f, 100.0f, 7176.9f}, // Prefab practise room
	{ -1000, -1900, 0.00f, 1.38f, 0.38f, -100, 0.024f, -400, 0.044f,  82.0f, 100.0f, 2854.4f}, // Prefab outhouse
	{ -1000, -2100, 0.00f, 0.43f, 1.50f,    0, 0.012f,  600, 0.012f, 100.0f, 100.0f, 5000.0f}, // Prefab caravan

	{ -1000, -900, 0.00f,  4.18f, 0.21f,  -825, 0.030f,  450, 0.022f,  79.0f, 100.0f, 2854.4f}, // Dome tomb
	{ -1000, -900, 0.00f,  5.04f, 0.10f,  -600, 0.032f,  800, 0.015f, 100.0f, 100.0f, 2854.4f}, // Pipe small
	{ -1000, -900, 0.00f, 10.48f, 0.19f, -1500, 0.090f,  200, 0.042f,  87.0f, 100.0f, 2854.4f}, // Dome St. Paul's
	{ -1000, -700, 0.00f,  9.21f, 0.18f,  -300, 0.010f, -300, 0.022f,  91.0f, 100.0f, 2854.4f}, // Pipe long thin
	{ -1000, -900, 0.00f,  8.45f, 0.10f,  -800, 0.046f,  400, 0.032f, 100.0f, 100.0f, 2854.4f}, // Pipe large
	{ -1000, -700, 0.00f,  6.81f, 0.18f,  -300, 0.010f,    0, 0.022f,  91.0f, 100.0f, 2854.4f}, // Pipe resonant

	{ -1000, -1200, 0.00f, 1.12f, 0.34f,  -700, 0.069f,  -300, 0.023f, 45.0f, 100.0f, 4399.1f}, // Outdoors backyard
	{ -1000, -3900, 0.00f, 2.13f, 0.21f, -1500, 0.300f,  -700, 0.019f,  0.0f, 100.0f, 4399.1f}, // Outdoors rolling plains
	{ -1000, -1500, 0.00f, 3.89f, 0.21f, -1000, 0.223f,  -900, 0.019f, 74.0f, 100.0f, 4399.1f}, // Outdoors deep canyon
	{ -1000, -1500, 0.00f, 2.13f, 0.21f,  -800, 0.115f, -1400, 0.031f, 35.0f, 100.0f, 4399.1f}, // Outdoors creek
	{ -1000, -3100, 0.00f, 2.88f, 0.26f, -1700, 0.263f,  -800, 0.100f, 28.0f, 100.0f, 2854.4f}, // Outdoors valley

	{ -1000, -200, 0.00f, 5.04f, 1.12f,  -1230, 0.020f, 200, 0.029f, 94.0f, 100.0f, 5000.0f}, // Mood Heaven
	{ -1000, -900, 0.00f, 3.57f, 0.49f, -10000, 0.020f, 300, 0.030f, 57.0f, 100.0f, 5000.0f}, // Mood Hell
	{ -1000, -400, 0.00f, 4.06f, 0.82f,  -2800, 0.000f, 100, 0.000f, 85.0f, 100.0f, 5000.0f}, // Mood memory

	{     0,  -500, 0.00f, 2.42f, 0.88f, -1400, 0.093f, -1200, 0.017f,   0.0f, 100.0f,  5000.0f}, // Driving commentator
	{ -1000,  -300, 0.00f, 1.72f, 0.93f,  -500, 0.000f,   200, 0.016f,  59.0f, 100.0f,  5000.0f}, // Driving pit garage
	{ -1000,     0, 0.00f, 0.17f, 2.00f,   500, 0.007f,  -300, 0.015f,  80.0f, 100.0f, 10268.2f}, // Driving in-car racer
	{ -1000,  -400, 0.00f, 0.17f, 0.75f,     0, 0.010f,  -500, 0.000f,  80.0f, 100.0f, 10268.2f}, // Driving in-car sports
	{ -1000, -2000, 0.00f, 0.13f, 0.41f,  -200, 0.010f,   400, 0.010f, 100.0f, 100.0f, 10268.2f}, // Driving in-car luxury
	{ -1000, -1100, 0.00f, 3.01f, 1.37f,  -900, 0.090f, -1500, 0.049f, 100.0f, 100.0f, 10420.2f}, // Driving full grandstand
	{ -1000,     0, 0.00f, 4.62f, 1.75f, -1363, 0.090f, -1200, 0.049f, 100.0f, 100.0f, 10420.2f}, // Driving empty grandstand
	{ -1000,  -800, 0.00f, 3.42f, 0.94f,  -300, 0.051f,  -300, 0.047f,  81.0f, 100.0f,  5000.0f}, // Driving tunnel

	{ -1000,  -300, 0.00f, 1.79f, 1.12f, -1100, 0.046f, -1400, 0.028f, 78.0f, 100.0f, 5000.0f}, // City streets
	{ -1000,  -300, 0.00f, 3.01f, 1.23f,  -300, 0.046f,   200, 0.028f, 74.0f, 100.0f, 5000.0f}, // City subway
	{ -1000, -1500, 0.00f, 3.28f, 1.40f, -1200, 0.039f,  -100, 0.034f, 82.0f, 100.0f, 2854.4f}, // City museum
	{ -1000, -1100, 0.00f, 2.76f, 0.89f,  -900, 0.029f,  -100, 0.020f, 82.0f, 100.0f, 2854.4f}, // City library
	{ -1000,  -700, 0.00f, 3.57f, 1.12f,  -800, 0.059f,  -100, 0.037f, 82.0f, 100.0f, 5000.0f}, // City underpass
	{ -1000,  -200, 0.00f, 3.28f, 1.17f,  -700, 0.044f, -1100, 0.024f, 69.0f, 100.0f, 5000.0f}, // City abandoned

	{ -1000, -200, 0.00f, 1.79f, 0.38f, -600, 0.002f,  200, 0.006f, 56.0f, 100.0f, 13046.0f}, // Dusty room
	{ -1000, -500, 0.00f, 4.62f, 0.64f, -700, 0.032f, -200, 0.049f, 84.0f, 100.0f,  5000.0f}, // Chapel
	{ -1000, -698, 0.00f, 1.51f, 1.25f, -100, 0.020f,  300, 0.030f, 70.0f, 100.0f,  5000.0f}, // Small water room

	// For some reason these weren't in the old system's list
	//{ -1000, -600,0.0f, 1.10f,0.83f,  -400,0.005f,   500,0.010f,100.0f,100.0f,5000.0f}, // Small room
	//{ -1000, -600,0.0f, 1.30f,0.83f, -1000,0.010f,  -200,0.020f,100.0f,100.0f,5000.0f}, // Medium room
	//{ -1000, -600,0.0f, 1.50f,0.83f, -1600,0.020f, -1000,0.040f,100.0f,100.0f,5000.0f}, // Large room
	//{ -1000, -600,0.0f, 1.80f,0.70f, -1300,0.015f,  -800,0.030f,100.0f,100.0f,5000.0f}, // Medium hall
	//{ -1000, -600,0.0f, 1.80f,0.70f, -2000,0.030f, -1400,0.060f,100.0f,100.0f,5000.0f}, // Large hall
	//{ -1000, -200,0.0f, 1.30f,0.90f,     0,0.002f,     0,0.010f,100.0f, 75.0f,5000.0f}, // Plate
};

const u_int uTOOLS_NUM_REVERB_PRESETS =
	sizeof( axTOOLS_REVERB_PRESETS ) / sizeof( axTOOLS_REVERB_PRESETS[0] );

ASURA_STATIC_ASSERT( uTOOLS_NUM_REVERB_PRESET_NAMES == uTOOLS_NUM_REVERB_PRESETS );

#endif // !defined __TOOLS_REVERBPRESETS_H__
