//
// Tools_SoundEventTab_Sample.cpp
//
// Created by Richard Rice - 07/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Sound/Tools_SoundEventTab_Sample.h"

#include "Core/Asura_String.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Mixer.h"

//tools
#include "Misc/Tools_Dialog.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Sound/Tools_Mode_Sound.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_PickSoundSample_Overlay.h"
#include "Sound/Tools_SoundConversionPreview.h"
#include "Sound/Tools_Sound_System.h"
#include "Sound/Tools_SoundEvent_System.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_MessageLogAction_EditSample **
**----------------------------------*/
class Tools_MessageLogAction_EditSample : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_EditSample(const Asura_Sound_Sample* pxSample) {m_uID = pxSample->GetHashID();}
	void Execute() {Tools_SoundEvent_Overlay::EditSample(m_uID);}

private:
	Asura_Hash_ID m_uID;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_SoundSampleProperty_Overlay **
**----------------------------------*/

//Stuff for property copying overlay
enum Tools_SoundSample_Properties
{
	Tools_SoundSampleProp_Filename,
	Tools_SoundSampleProp_Category,
	Tools_SoundSampleProp_SubCategory,
	Tools_SoundSampleProp_Wav,

	Tools_SoundSampleProp_Volume,
	Tools_SoundSampleProp_Pitch,
	Tools_SoundSampleProp_Duration,
	Tools_SoundSampleProp_Looping,
	
	Tools_SoundSampleProp_SampleRate_PS2,
	Tools_SoundSampleProp_SampleRate_PSP,
	Tools_SoundSampleProp_SampleRate_Wii,

	Tools_SoundSampleProp_CompressionQuality_PS3,
	Tools_SoundSampleProp_CompressionQuality_XBox360,

	Tools_SoundSampleProp_NumProp,
};

static const char* g_aszPropertyNames[Tools_SoundSampleProp_NumProp] =
{
	"Filename",
	"Category",
	"Subcategory",
	"Wav File",

	"Volume",
	"Pitch",
	"Duration",
	"Looping",
	
	"Sample Rate - PS2",
	"Sample Rate - PSP",
	"Sample Rate - Wii",

	"Compression Quality - PS3",
	"Compression Quality - XBox360",
};

static const char* g_aszCompressionQualityStrings[] =	// This should match the TOOLS_SOUND_SAMPLE_QUALITY enum
{
	"Perfect",
	"High",
	"Medium",
	"Low",
	"Very Low"
};

//Overlay for selecting properties to paste

class Tools_SoundSampleProperty_Overlay : public ListSelection_Overlay
{
public :
	Tools_SoundSampleProperty_Overlay()
	:ListSelection_Overlay("Select properties to paste")
	{
		//initialise the list
		SetSortList(false);

		for(int i=0;i<Tools_SoundSampleProp_NumProp;i++)
		{
			AddItem(g_aszPropertyNames[i],i);
			SetItemTicked(i,s_bSelected[i]);
		}
	}
	
	virtual bool DoModal()
	{
		if(!ListSelection_Overlay::DoModal()) return false;

		//copy settings to static array
		for(int i=0;i<Tools_SoundSampleProp_NumProp;i++)
		{
			s_bSelected[i] = GetItemTicked(i);
		}
		return true;
	}

	const bool* GetSelectedArray() const {return s_bSelected;}

protected :
	virtual bool IsTickboxList() const {return true;}
	virtual const char* GetListHeadingText() const {return "Properties :";}
	virtual const char* GetTickAllButtonText() const {return "Select All";}
	virtual const char* GetTickNoneButtonText() const {return "Deselect All";}


	static bool s_bSelected[Tools_SoundSampleProp_NumProp];
};

bool Tools_SoundSampleProperty_Overlay::s_bSelected[Tools_SoundSampleProp_NumProp];

///////////////////////////////////////////////////////////////////////////////////
/*-------------------**
** Tools_SoundSample **
**-------------------*/
void Tools_SoundSample::CopyFromAsuraSample(const class Asura_Sound_Sample* pxSample)
{
#ifdef ASURA_USE_AUDIO_SYSTEM
	m_uWave = pxSample->m_uWave;
#else
	m_eSoundID = pxSample->m_eSoundID;
#endif

	m_fVolume = pxSample->m_fVolume;
	m_fPitch = pxSample->m_fPitch;
	m_fDuration = pxSample->m_fDuration;
	m_bLooping = pxSample->m_bLooping;
	
	m_xCategory = pxSample->GetSoundType();
	m_xSubCategory = pxSample->GetSoundSubType();
	m_xFilename = pxSample->GetFilename();

	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if( Tools_SoundEvent_System::DoesPlatformUseSampleRate(i) )
		{
			m_aiSampleRate[i] = pxSample->GetSampleRate(i);
			//If the sample rate is zero , then it isn't being used
			m_abUseSampleRate[i] = m_aiSampleRate[i] != 0;
			if(!m_abUseSampleRate[i])
			{
				m_aiSampleRate[i] = Tools_SoundEvent_System::GetPlatformSampleRate(i);
			}
		}
		
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality(i) )
		{
			m_aiCompressionQuality[i] = pxSample->GetCompressionQuality(i);
			//If the compression quality is -1 then the sample is using the platform default
			m_abUseCompressionQuality[i] = m_aiCompressionQuality[i] != TSS_QUALITY_USE_DEFAULT;
			if(!m_abUseCompressionQuality[i])
			{
				m_aiCompressionQuality[i] = Tools_SoundEvent_System::GetDefaultPlatformCompressionQuality(i);
			}
		}
	}
}

void Tools_SoundSample::CopyToAsuraSample(class Asura_Sound_Sample* pxSample) const
{
#ifdef ASURA_USE_AUDIO_SYSTEM

	if(pxSample->m_uWave != m_uWave)
	{
		Tools_Mode_Sound::InvalidateSoundSize();
		pxSample->m_uWave = m_uWave;
	}

	if(pxSample->m_bLooping != m_bLooping)
	{
		Asura_Audio_System::SetWaveLooping(m_uWave, m_bLooping);
	}

#else

	if(pxSample->m_eSoundID != m_eSoundID)
	{
		Tools_Mode_Sound::InvalidateSoundSize();
		pxSample->m_eSoundID = m_eSoundID;
	}

#endif

	pxSample->m_fVolume = m_fVolume;
	pxSample->m_fPitch = m_fPitch;
	pxSample->m_fDuration = m_fDuration;
	pxSample->m_bLooping = m_bLooping;
	
	pxSample->SetSoundType(m_xCategory.GetString());
	pxSample->SetSoundSubType(m_xSubCategory.GetString());
	pxSample->SetFilename(m_xFilename.GetString());

	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		//If the sample rate isn't being used , set it as zero
		Tools_Mode_Sound::InvalidateSoundSize();
		if( Tools_SoundEvent_System::DoesPlatformUseSampleRate(i) )
		{
			int iSampleRate = m_abUseSampleRate[i] ? m_aiSampleRate[i] : 0;
			if(iSampleRate != pxSample->GetSampleRate(i))
			{
				pxSample->SetSampleRate(i,iSampleRate);
			}
		}

		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality(i) )
		{
			if(m_abUseCompressionQuality[i])
			{
				pxSample->SetCompressionQuality( i, m_aiCompressionQuality[i] );
			}
			else
			{
				pxSample->SetCompressionQuality( i, TSS_QUALITY_USE_DEFAULT );
			}
		}
	}
}

Tools_SoundSample& Tools_SoundSample::operator = (const Tools_SoundSample& xSample)
{
	if(this != &xSample) 
	{
#ifdef ASURA_USE_AUDIO_SYSTEM
		m_uWave = xSample.m_uWave;
#else
		m_eSoundID = xSample.m_eSoundID;
#endif

		m_fVolume = xSample.m_fVolume;
		m_fPitch = xSample.m_fPitch;
		m_fDuration = xSample.m_fDuration;
		m_bLooping = xSample.m_bLooping;
		
		m_xCategory = xSample.m_xCategory;
		m_xSubCategory = xSample.m_xSubCategory;
		m_xFilename = xSample.m_xFilename;

		for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
		{
			if( Tools_SoundEvent_System::DoesPlatformUseSampleRate(i) )
			{
				m_aiSampleRate[i] = xSample.m_aiSampleRate[i];
				m_abUseSampleRate[i] = xSample.m_abUseSampleRate[i];
			}
			
			if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality(i) )
			{
				m_abUseCompressionQuality[i] = xSample.m_abUseCompressionQuality[i];
				m_aiCompressionQuality[i] = xSample.m_aiCompressionQuality[i];
			}
		}

	}
	return *this;
}

void Tools_SoundSample::Copy(const Asura_Sound_Sample* pxThisSample,const Tools_SoundSample& xSampleToCopy,const bool* pbProperties)
{
	if(this == &xSampleToCopy) return;


	if(pbProperties[Tools_SoundSampleProp_Wav])
	{
#ifdef ASURA_USE_AUDIO_SYSTEM
		m_uWave = xSampleToCopy.m_uWave;
#else
		m_eSoundID = xSampleToCopy.m_eSoundID;
#endif
	}
	 
	if(pbProperties[Tools_SoundSampleProp_Volume])
	{
		m_fVolume = xSampleToCopy.m_fVolume;
	}
	 
	if(pbProperties[Tools_SoundSampleProp_Pitch])
	{
		m_fPitch = xSampleToCopy.m_fPitch;
	}

	if(pbProperties[Tools_SoundSampleProp_Duration])
	{
		m_fDuration = xSampleToCopy.m_fDuration;
	}
	 
	if(pbProperties[Tools_SoundSampleProp_Looping])
	{
		m_bLooping = xSampleToCopy.m_bLooping;
	}
	
	 
	if(pbProperties[Tools_SoundSampleProp_Category])
	{
 		m_xCategory = xSampleToCopy.m_xCategory;
	}

	if(pbProperties[Tools_SoundSampleProp_SubCategory])
	{
 		m_xSubCategory = xSampleToCopy.m_xSubCategory;
	}

	 
	if(pbProperties[Tools_SoundSampleProp_Filename])
	{
		//Make sure it is okay to change the file
		if(Tools_SoundEvent_System::CanTransferSample(pxThisSample,xSampleToCopy.m_xFilename.GetString()))
		{
			m_xFilename = xSampleToCopy.m_xFilename;
		}
		else
		{
			//report an error
			Tools_MessageLog::AddMessage(Tools_MessageLog::SOUND_EVENT,"Paste Error",new Tools_MessageLogAction_EditSample(pxThisSample),"Failed to transfer '%s' since source and destination aren't both writeable",pxThisSample->GetName());
		}
		m_xFilename = xSampleToCopy.m_xFilename;
	}

	if(pbProperties[Tools_SoundSampleProp_SampleRate_PS2])
	{
		m_aiSampleRate[TOOLS_PLATFORM_PS2] = xSampleToCopy.m_aiSampleRate[TOOLS_PLATFORM_PS2];
		m_abUseSampleRate[TOOLS_PLATFORM_PS2] = xSampleToCopy.m_abUseSampleRate[TOOLS_PLATFORM_PS2];
	}

	if(pbProperties[Tools_SoundSampleProp_SampleRate_PSP])
	{
		m_aiSampleRate[TOOLS_PLATFORM_PSP] = xSampleToCopy.m_aiSampleRate[TOOLS_PLATFORM_PSP];
		m_abUseSampleRate[TOOLS_PLATFORM_PSP] = xSampleToCopy.m_abUseSampleRate[TOOLS_PLATFORM_PSP];
	}

	if(pbProperties[Tools_SoundSampleProp_SampleRate_Wii])
	{
		m_aiSampleRate[TOOLS_PLATFORM_WII] = xSampleToCopy.m_aiSampleRate[TOOLS_PLATFORM_WII];
		m_abUseSampleRate[TOOLS_PLATFORM_WII] = xSampleToCopy.m_abUseSampleRate[TOOLS_PLATFORM_WII];
	}

	if(pbProperties[Tools_SoundSampleProp_CompressionQuality_PS3])
	{
		m_aiCompressionQuality[TOOLS_PLATFORM_PS3] = xSampleToCopy.m_aiCompressionQuality[TOOLS_PLATFORM_PS3];
		m_abUseCompressionQuality[TOOLS_PLATFORM_PS3] = xSampleToCopy.m_abUseCompressionQuality[TOOLS_PLATFORM_PS3];
	}
	
	if(pbProperties[Tools_SoundSampleProp_CompressionQuality_XBox360])
	{
		m_aiCompressionQuality[TOOLS_PLATFORM_XBOX360] = xSampleToCopy.m_aiCompressionQuality[TOOLS_PLATFORM_XBOX360];
		m_abUseCompressionQuality[TOOLS_PLATFORM_XBOX360] = xSampleToCopy.m_abUseCompressionQuality[TOOLS_PLATFORM_XBOX360];
	}

}



///////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_SoundEventTab_Sample **
**----------------------------*/

bool Tools_SoundEventTab_Sample::s_bPasteBufferValid = false;
Tools_SoundSample Tools_SoundEventTab_Sample::s_xPasteBuffer;

Tools_SoundEventTab_Sample::Tools_SoundEventTab_Sample(Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uSampleHash)
{
	m_uSampleHash = uSampleHash;
	m_pxSample = Asura_Sound_Event_Storage::GetSampleByHash(uSampleHash);
	Asura_Assert(m_pxSample,"");

	m_xSampleData.CopyFromAsuraSample(m_pxSample);
	m_bFileChanged = false;
	m_bChanged = false;

	m_pxWavDialog = 0;
	m_pxWavDataStringDialog = 0;

	AddOverlayControls(pxTabsDialog);
}


void Tools_SoundEventTab_Sample::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog)
{

	//create new tab on the tab dialog
	UpdateTabName();
	pxTabsDialog->AddTab(m_xTabName.GetString());

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = pxTabsDialog->GetTopDialogPosition();

	//filename
	pxDialog = new Tools_SoundFilename_Dialog(xOffset,&m_xSampleData.m_xFilename,&m_bFileChanged);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//category
	pxDialog = new Tools_SoundCategory_Dialog(xOffset,&m_xSampleData.m_xCategory,&m_bChanged);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//sub category
	pxDialog = new Tools_SoundSubCategory_Dialog(xOffset,&m_xSampleData.m_xCategory,&m_xSampleData.m_xSubCategory,&m_bChanged);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::LargeSpacing();


	//wav file control
	pxDialog = m_pxWavDialog = new Asura_Function_Dialog("Wav",xOffset,SelectWav,(int)this);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::LargeSpacing();
	UpdateWavDialog();

	// Store y offset for wave data display
	const float fWavDataY = xOffset.y;

	pxDialog = new Asura_Numerical_Dialog("Volume",xOffset,&m_xSampleData.m_fVolume, 0.01f,1.33f,0.01f,"%0.2f ",&m_bChanged);
	pxDialog->SetHelpText("Volume multiplier.\nNote that 1.33 is effectively maximum , since all volumes are scaled by 0.75 and capped at 1.0 at the end.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Pitch",xOffset,&m_xSampleData.m_fPitch, 0.01f,2.0f,0.01f,"%0.2f ",&m_bChanged);
	pxDialog->SetHelpText("Pitch multiplier.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Duration",xOffset,&m_xSampleData.m_fDuration, 0.0f,999.0f,0.01f,"%0.1f ",&m_bChanged);
	pxDialog->SetHelpText("If non-zero this determines the (maximum) duration for the sample.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Tickbox_Dialog("Looping",xOffset,&m_xSampleData.m_bLooping,&m_bChanged);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Store y offset again
	const float fAfterWavDataY = xOffset.y;

	// Wav data string
	xOffset.x += 22.0f;
	xOffset.y = fWavDataY;

	m_pxWavDataStringDialog = new Asura_StringDisplay_Dialog( "", xOffset );
	pxTabsDialog->AddDialog( m_pxWavDataStringDialog );
	m_pxWavDataStringDialog->SetRelativePosition( pxTabsDialog, Asura_Tabs_Dialog::s_uTopLeftPositionID );
	m_pxWavDataStringDialog->SetHelpText( "Displays additional information about source wav file" );
	UpdateWavDataStringDialog();

	xOffset.x -= 22.0f;
	xOffset.y = fAfterWavDataY;

	//Sample rates
	pxDialog = new Asura_Dialog("Sample Rates :",xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if(Tools_SoundEvent_System::DoesPlatformUseSampleRate(i))
		{
			//tickbox for using sample rate
			pxDialog = new Asura_Tickbox_Dialog("",xOffset,&m_xSampleData.m_abUseSampleRate[i],&m_bChanged);
			pxTabsDialog->AddDialog(pxDialog);
			pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);

			//sample rate value
			xOffset.x += 3.0f;
			pxDialog = new Asura_Numerical_Dialog(Tools_Platform::GetPlatformName(i),xOffset,&m_xSampleData.m_aiSampleRate[i], 1,99999,1,"%d",&m_bChanged);
			pxDialog->SetIsEnabledFunction(IsEnabled_SampleRate,(int)&m_xSampleData.m_abUseSampleRate[i]);
			pxDialog->SetHelpText("Maximum sample rate to use when exporting this sound.");
			pxTabsDialog->AddDialog(pxDialog);
			pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			xOffset.x -= 3.0f;

		}
	}
	xOffset.y += Asura_Dialog::SmallSpacing();

	// Compression quality
	pxDialog = new Asura_Dialog("Compression quality :",xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();



	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality(i) )
		{
			float fX = xOffset.x;
			
			//tickbox for using compression quality 
			pxDialog = new Asura_Tickbox_Dialog("",xOffset,&m_xSampleData.m_abUseCompressionQuality[i],&m_bChanged);
			pxTabsDialog->AddDialog(pxDialog);
			pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
			
			
			xOffset.x += 4.0f;
			pxDialog = new Asura_DropDown_List_Dialog	( xOffset
														, g_aszCompressionQualityStrings
														, &m_xSampleData.m_aiCompressionQuality[i]
														, TSS_QUALITY_NUM_ENTRIES
														, TSS_QUALITY_NUM_ENTRIES
														, 0
														, &m_bChanged
														, Tools_Platform::GetPlatformName(i) );
				
			pxDialog->SetHelpText("Quality setting when compressing this sound during export");
			pxTabsDialog->AddDialog(pxDialog);
			pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
			pxDialog->SetIsEnabledFunction(IsEnabled_CompressionQuality,(int)&m_xSampleData.m_abUseCompressionQuality[i]);


			xOffset.x = fX;
			xOffset.x += 18.0f;
			switch(i)
			{
				case TOOLS_PLATFORM_PS3 :
					pxDialog = new Asura_Function_Dialog("Preview", xOffset, PreviewPS3, (int) this);
					break;

				case TOOLS_PLATFORM_XBOX360 :
					pxDialog = new Asura_Function_Dialog("Preview", xOffset, PreviewXBox360, (int) this);
					break;
				default :
					pxDialog = 0;
					break;
			}
			if(pxDialog)
			{
				pxDialog->SetHelpText("Preview the sound using the current quality setting");
				pxTabsDialog->AddDialog(pxDialog);
				pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
			}
			xOffset.x = fX;
			xOffset.x += 28.0f;
			//Add some icons for previewing each of the quality levels without changing the current setting for the sample
			for(int iQuality = 0; iQuality<TSS_QUALITY_NUM_ENTRIES; iQuality++)
			{
				int iIconIndex = (g_aszCompressionQualityStrings[iQuality][0] - 'A') + TOOLS_ICON_A;
				pxDialog = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), iIconIndex);

				//pact the quality and platform settings into an int to be used by the callback
				int iPlatformAndQuality = (i<<16)|iQuality;
				Asura_Callback_MemberFunctionWithUserData<void,Tools_SoundEventTab_Sample,int> xCallback(this,&Tools_SoundEventTab_Sample::PreviewCompression_PlatformAndQuality, iPlatformAndQuality);
				((Asura_Icon_Dialog*)pxDialog)->SetClickedFunction(xCallback);

				char acHelpText[100];
				Asura_PrintfToBfr(acHelpText, "%s : Preview the sound at this quality setting", sizeof(acHelpText), true, g_aszCompressionQualityStrings[iQuality]);
				pxDialog->SetHelpText(acHelpText);

				pxTabsDialog->AddDialog(pxDialog);
				pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
				xOffset.x = pxDialog->m_xMaxCoord.x;

			}

			xOffset.x = fX;
			xOffset.y += Asura_Dialog::DefaultSpacing();

		}
	}
	xOffset.x += 18.0f;
	pxDialog = new Asura_Function_Dialog("Play Original", xOffset, PreviewOriginal, (int) this);
	pxDialog->SetHelpText("Preview the original sound , without being modified by volume/pitch settings");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	
	xOffset.x -= 18.0f;

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Create Event From Sample",xOffset,CreateEventFromSample,(int)this);
	pxDialog->SetHelpText("Creates a new event that uses this sample.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Find Ref. To Wav",xOffset,FindReferencesToWav,(int)this);
	pxDialog->SetHelpText("Searches for samples and events that make use of this wav file.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::LargeSpacing();

	
	pxDialog = new Asura_Function_Dialog("Play Sample",xOffset,PlaySound,(int)this);
	pxDialog->SetHelpText("Plays this sound sample.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Stop Sample",xOffset,StopSound,(int)this);
	pxDialog->SetHelpText("Stops playback of test sound.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reload Wav",xOffset,ReloadWav,(int)this);
	pxDialog->SetHelpText("Reloads the wav associated with this sample.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);


}

void Tools_SoundEventTab_Sample::PerFrameUpdate()
{
	if(m_bFileChanged)
	{
		//Make sure it is okay to transfer the event
		if(!Tools_SoundEvent_System::CanTransferSample(m_pxSample,m_xSampleData.m_xFilename.GetString()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Can't transfer sample since source and destination aren't both writeable");

			m_xSampleData.m_xFilename = m_pxSample->GetFilename();
			m_bFileChanged = false;
		}
		else
		{
			//Set modified before updating the asura data so that both the source and destination files are flagged
			SetModified();
			m_bChanged = true;
		}
		m_bFileChanged = false;
	}
	
	if(m_bChanged)
	{
		//update the asura data
		m_xSampleData.CopyToAsuraSample(m_pxSample);
		SetModified();

		m_bChanged = false;
	}

	//update the sample rate and compression qualities for any that are using the default
	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if( Tools_SoundEvent_System::DoesPlatformUseSampleRate(i) )
		{
			if(!m_xSampleData.m_abUseSampleRate[i])
			{
				m_xSampleData.m_aiSampleRate[i] = Tools_SoundEvent_System::GetPlatformSampleRate(i);
			}
		}
		
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality(i) )
		{
			if(!m_xSampleData.m_abUseCompressionQuality[i])
			{
				m_xSampleData.m_aiCompressionQuality[i] = Tools_SoundEvent_System::GetDefaultPlatformCompressionQuality(i);
			}
		}
	}
}


void Tools_SoundEventTab_Sample::SelectWav()
{
	#if defined ASURA_USE_AUDIO_SYSTEM

		Asura_Hash_ID uWave = Tools_PickSound::PickSoundForSoundEventSample(m_xSampleData.m_uWave);
		if(uWave == m_xSampleData.m_uWave) return;

		m_xSampleData.m_uWave = uWave;

	#else

		Asura_Sound_ID eSoundID = Tools_PickSound::PickSoundForSoundEventSample(m_xSampleData.m_eSoundID);
		if(eSoundID == m_xSampleData.m_eSoundID) return;

		m_xSampleData.m_eSoundID = eSoundID;

	#endif

	UpdateWavDialog();
	UpdateWavDataStringDialog();
	m_bChanged = true;
}

void Tools_SoundEventTab_Sample::UpdateWavDialog()
{
	Asura_Assert(m_pxWavDialog,"");

	char szText[200];
	Asura_String xFullFilename,xFilename;

	#if defined ASURA_USE_AUDIO_SYSTEM

		if(m_xSampleData.m_uWave == ASURA_HASH_ID_UNSET)
		{
			Asura_PrintfToBfr(szText,"Wav : *None*",sizeof(szText));
		}
		else
		{
			xFullFilename = Asura_Audio_System::GetWaveFilename(m_xSampleData.m_uWave);
			if(!xFullFilename.IsBlank()) xFullFilename.SplitFilename(0,&xFilename,0);
			Asura_PrintfToBfr(szText,"Wav : %s",sizeof(szText),true,xFilename.IsBlank() ? "*Not Found*" : xFilename.GetString());
		}

	#else

		if(m_xSampleData.m_eSoundID == ASURA_SID_NO_SOUND)
		{
			strcpy(szText,"Wav : *None*");
		}
		else
		{
			xFullFilename = Asura_Sound_System::GetFileNameFromSoundID(m_xSampleData.m_eSoundID);
			if(!xFullFilename.IsBlank()) xFullFilename.SplitFilename(0,&xFilename,0);
			Asura_PrintfToBfr(szText,"Wav : %s",sizeof(szText),true,xFilename.IsBlank() ? "*Not Found*" : xFilename.GetString());
		}

	#endif

	m_pxWavDialog->UpdateLabel(szText);
	m_pxWavDialog->SetHelpText(xFullFilename.GetString());
}

void Tools_SoundEventTab_Sample::UpdateWavDataStringDialog()
{
	if( !m_pxWavDataStringDialog )
	{
		Asura_Assert( 0, "Wav data string dialog missing" );
		return;
	}

	WAVEFORMATEX xWaveData;
	MMCKINFO xWaveCkInfo;
	bool bSoundDataAvailable = false;

#ifdef ASURA_USE_AUDIO_SYSTEM
	bSoundDataAvailable = Tools_Sound_System::GetSoundWaveData( m_xSampleData.m_uWave, xWaveData, xWaveCkInfo );
#else
	bSoundDataAvailable = Tools_Sound_System::GetSoundWaveData( m_xSampleData.m_eSoundID, xWaveData, xWaveCkInfo );
#endif

	const int iWAV_DATA_STRING_LENGTH = 256;
	Asura_Char szWavDataString[iWAV_DATA_STRING_LENGTH];

	if( bSoundDataAvailable )
	{
		Asura_PrintfToBfr( szWavDataString, "Number of channels: %u\nSample rate: %uhz\nBit depth: %u\nLength: %.2fs",
											iWAV_DATA_STRING_LENGTH, true, xWaveData.nChannels, xWaveData.nSamplesPerSec, xWaveData.wBitsPerSample,
											xWaveCkInfo.cksize / static_cast<float>( xWaveData.nAvgBytesPerSec ) );

	}
	else
	{
		Asura_PrintfToBfr( szWavDataString, "No additional wav data available", iWAV_DATA_STRING_LENGTH, true );
	}

	m_pxWavDataStringDialog->UpdateLabel( szWavDataString );
}

void Tools_SoundEventTab_Sample::SetModified()
{
	Tools_SoundEvent_System::SetModified(m_pxSample);
}

void Tools_SoundEventTab_Sample::PlaySound()
{
	Tools_SoundEvent_Overlay::PlaySoundSample(m_uSampleHash);
}

void Tools_SoundEventTab_Sample::StopSound()
{
	Tools_SoundEvent_Overlay::StopSound();
}

void Tools_SoundEventTab_Sample::CreateEventFromSample()
{
	//get a name for the new event , defaulting to the sample name if possible
	char szEventName[200];
	Asura_Hash_ID uEventNameHash = ASURA_HASH_ID_UNSET;

	for(int iIndex=0;;iIndex++)
	{
		if(iIndex)
			Asura_PrintfToBfr(szEventName,"%s%d",200,false,m_pxSample->GetName(),iIndex);
		else
			Asura_PrintfToBfr(szEventName,"%s",200,false,m_pxSample->GetName());

		uEventNameHash = Asura_GetHashID(szEventName);
		if(!Asura_Sound_Event_Storage::GetEventByHash(uEventNameHash)) break;
	}

	//Create the event
	bool bAlreadyExists;
	Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::CreateEvent(szEventName, bAlreadyExists);
	Asura_Assert(pxEvent && !bAlreadyExists, "Error creating sound event.");
	//use the same file/category as the sample
	pxEvent->SetEventType(m_pxSample->GetSoundType());
	pxEvent->SetEventSubType(m_pxSample->GetSoundSubType());
	pxEvent->SetFilename(m_pxSample->GetFilename());

	//If there is a mixer channel that mixes the category , then use that
	if(!stricmp(m_pxSample->GetSoundType(),"default"))
	{
		Asura_Hash_ID uMixerHash = Asura_GetHashID(m_pxSample->GetSoundType());
		if(Asura_Sound_Mixer_System::FindChannel(uMixerHash))
		{
			pxEvent->SetMixerChannelName(m_pxSample->GetSoundType());
		}
	}

	//add this sample to the event
	Asura_Sound_PlayableID xID;
	xID.m_eType = ASURA_SOUND_PLAYABLE_SAMPLE;
	xID.m_uHashID = m_pxSample->GetHashID();
	pxEvent->SetPlayableIDList(&xID,1);

	//Finish up and edit the event
	Tools_SoundEvent_System::SetModified(pxEvent);
	Tools_SoundEvent_System::RegisterNewEvent(pxEvent);

	Tools_SoundEvent_Overlay::EditEvent(uEventNameHash);
	
}

const char* Tools_SoundEventTab_Sample::GetName()  const
{
	return m_pxSample->GetName();
}

void Tools_SoundEventTab_Sample::UpdateFromAsuraData()
{
	m_xSampleData.CopyFromAsuraSample(m_pxSample);
	UpdateWavDialog();
	UpdateWavDataStringDialog();
}

bool Tools_SoundEventTab_Sample::HandleReload()
{
	m_pxSample = Asura_Sound_Event_Storage::GetSampleByHash(m_uSampleHash);
	if(m_pxSample)
	{
		UpdateFromAsuraData();
		return true;
	}
	return false;
}



bool Tools_SoundEventTab_Sample::Delete()
{
	return Tools_SoundEvent_System::DeleteSample(m_uSampleHash,true);
}


void Tools_SoundEventTab_Sample::Copy()
{
	s_xPasteBuffer = m_xSampleData;
	s_bPasteBufferValid = true;
	Tools_Timed_Message_System::AddMessage(0xffffff,"%s copied.",m_pxSample->GetName());
}

void Tools_SoundEventTab_Sample::Paste()
{
	if(s_bPasteBufferValid)
	{
		m_xSampleData = s_xPasteBuffer;
		m_bFileChanged = true;
		m_bChanged = true;
		Tools_Timed_Message_System::AddMessage(0xffffff,"Sample pasted.");
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Sample paste buffer is empty.");
	}
}

void Tools_SoundEventTab_Sample::PasteMultiple()
{

	//Get a list of events to paste over
	Tools_PickSoundSample_Overlay xOverlay("Paste over which samples?",true);
	xOverlay.DoModal(0);

	u_int uNumSelected = xOverlay.GetNumSelected();
	if(!uNumSelected) return;

	//Determine which properties should be pasted
	Tools_SoundSampleProperty_Overlay xPropertyOverlay;
	if(!xPropertyOverlay.DoModal()) return;

	u_int uMessageLogPos = Tools_MessageLog::GetNextID();
	
	//Do the copying
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uSampleHash = xOverlay.GetSelection(i);
		Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByHash(uSampleHash);
		if(!pxSample) continue;

		//get a tools copy of the sample properties
		Tools_SoundSample xToolsSoundSample;
		xToolsSoundSample.CopyFromAsuraSample(pxSample);

		//set modified before the copy in case the file is being changed
		Tools_SoundEvent_System::SetModified(pxSample);

		//Do the copy
		xToolsSoundSample.Copy(pxSample,m_xSampleData,xPropertyOverlay.GetSelectedArray());

		//Update the asura sample and flag as modified
		xToolsSoundSample.CopyToAsuraSample(pxSample);
		Tools_SoundEvent_System::SetModified(pxSample);

		//Make sure any tab with this sample is updated
		Tools_SoundEvent_Overlay::UpdateSampleFromAsuraData(uSampleHash);
	}

	//If there were any errors , open the message log
	if(Tools_MessageLog::AreThereAnyMessages(Tools_MessageLog::SOUND_EVENT, 0, uMessageLogPos))
	{
		Tools_MessageLog::ShowOverlay(Tools_MessageLog::SOUND_EVENT,"Paste Error",uMessageLogPos);
	}

}


void Tools_SoundEventTab_Sample::Duplicate()
{
	//get a default name for the sample
	//get current sample name without any numbers at the end
	char szBaseName[100];
	Asura_PrintfToBfr(szBaseName,"%s",sizeof(szBaseName),true,m_pxSample->GetName());
	int iLength = strlen(szBaseName);
	while(iLength>0 && szBaseName[iLength-1]>='0' && szBaseName[iLength-1]<='9') iLength--;
	szBaseName[iLength] = 0;
	

	char szSampleName[200];
	for(int iIndex=0;;iIndex++)
	{
		Asura_PrintfToBfr(szSampleName,"%s%d",200,false,szBaseName,iIndex);

		if(!Asura_Sound_Event_Storage::GetSampleByHash(Asura_GetHashID(szSampleName))) break;
	}

	
	
	//ask for a name  , offering the default name
	GetString_Overlay xStringOverlay("Name new sample",szSampleName);
	do
	{
		if(!xStringOverlay.DoModal()) break;
		
		const char* szSampleName = xStringOverlay.GetString();
		if(!szSampleName || !*szSampleName) break;

		//make sure this sample doesn't already exist
		Asura_Hash_ID uSampleHash = Asura_GetHashID(szSampleName);
		if(Asura_Sound_Event_Storage::GetSampleByHash(uSampleHash))
		{
			ShowMessageOverlay("A sample already exists using the same hash ID.");
			continue;
		}

		bool bAlreadyExists;
		Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::CreateSample(szSampleName, bAlreadyExists);
		Asura_Assert(pxSample && !bAlreadyExists, "Error creating new sample");

		//copy this sample's properties to the new one
		m_xSampleData.CopyToAsuraSample(pxSample);
		Tools_SoundEvent_System::SetModified(pxSample);
		Tools_SoundEvent_System::RegisterNewSample(pxSample);
		//and edit it
		Tools_SoundEvent_Overlay::EditSample(uSampleHash);
		break;
		
	}while(true);
}


void Tools_SoundEventTab_Sample::Rename()
{
	const char* szNewName = 0;
	Asura_Hash_ID uOldHash = m_uSampleHash;
	Asura_Hash_ID uNewHash = 0;
	bool bOkay = false;
	
	//ask for new name
	GetString_Overlay xStringOverlay("Rename Sample",m_pxSample->GetName());
	while(xStringOverlay.DoModal())
	{
		szNewName = xStringOverlay.GetString();
		if(!szNewName || !*szNewName) return;

		if(!strcmp(szNewName,m_pxSample->GetName()))
		{
			//name not changed
			return;
		}

		//make sure no existing sample exists with this hash
		uNewHash = Asura_GetHashID(szNewName);

		if(uOldHash != uNewHash && Asura_Sound_Event_Storage::GetSampleByHash(uNewHash))
		{
			ShowMessageOverlay("A sample already exists using the same hash ID.");
			continue;
		}

		bOkay = true;
		break;
	}
	if(!bOkay) return;

	//Update the sample and its position in the storage system
	if(!Asura_Sound_Event_Storage::RenameSample(m_pxSample->GetName(),szNewName))
	{
		Asura_Assert(0,"Error renaming sample");
		return;
	}
	SetModified();

	//update this tab
	m_uSampleHash = uNewHash;
	UpdateTabName();

	//Update any events using this sample
	Asura_Sound_PlayableID xOldID,xNewID;
	xOldID.m_uHashID = uOldHash;
	xOldID.m_eType = ASURA_SOUND_PLAYABLE_SAMPLE;
	xNewID.m_uHashID = uNewHash;
	xNewID.m_eType = ASURA_SOUND_PLAYABLE_SAMPLE;
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfEvents();i++)
	{
		Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByIndex(i);
		if(pxEvent->ReplacePlayableID(xOldID,xNewID))
		{
			Tools_SoundEvent_System::SetModified(pxEvent);

			//Update the tab for this event if it was being edited
			Tools_SoundEvent_Overlay::UpdateEventFromAsuraData(pxEvent->GetHashID());
		}
	}
}

void Tools_SoundEventTab_Sample::UpdateTabName()
{
	char szTabName[MAX_TAB_NAME_LENGTH+2];
	Asura_PrintfToBfr(szTabName,"S: %s",sizeof(szTabName),true,m_pxSample->GetName());
	if(strlen(szTabName) > MAX_TAB_NAME_LENGTH)
	{
		//name is too long for tab , so truncate it
		strcpy(&szTabName[MAX_TAB_NAME_LENGTH-3],"...");
	}
	m_xTabName = szTabName;
}

void Tools_SoundEventTab_Sample::ReloadWav()
{
	Tools_SoundEventTab_Sample::StopSound();
	Tools_SoundEvent_System::ReloadWavsForSample(m_uSampleHash, true);
}

void Tools_SoundEventTab_Sample::FindReferencesToWav()
{
	#if defined ASURA_USE_AUDIO_SYSTEM
		if(m_xSampleData.m_uWave != ASURA_HASH_ID_UNSET)
		{
			Tools_SoundEvent_System::FindReferencesToWav(m_xSampleData.m_uWave);
		}
	#else
		if(m_xSampleData.m_eSoundID != ASURA_SID_NO_SOUND)
		{
			Tools_SoundEvent_System::FindReferencesToWav(m_xSampleData.m_eSoundID);
		}
	#endif
}


void Tools_SoundEventTab_Sample::PreviewOriginal()
{
	#if defined ASURA_USE_AUDIO_SYSTEM
	if(m_xSampleData.m_uWave)
	{
		Asura_String xFilename = Asura_Audio_System::GetWaveFilename(m_xSampleData.m_uWave);
		if(xFilename.IsBlank()) return;

		Tools_SoundConversionPreview::PreviewSound(xFilename.GetString());
	}
	#else
	if(m_xSampleData.m_eSoundID != ASURA_SID_NO_SOUND)
	{
		Asura_String xFilename = Asura_Sound_System::GetFileNameFromSoundID(m_xSampleData.m_eSoundID);
		if(xFilename.IsBlank()) return;

		Tools_SoundConversionPreview::PreviewSound(xFilename.GetString());
	}
	#endif 
}


void Tools_SoundEventTab_Sample::PreviewPS3()
{
	PreviewCompression_Platform(TOOLS_PLATFORM_PS3);
}

void Tools_SoundEventTab_Sample::PreviewXBox360()
{
	PreviewCompression_Platform(TOOLS_PLATFORM_XBOX360);
}

void Tools_SoundEventTab_Sample::PreviewCompression_Platform(int iPlatform)
{
	int iQuality;
	if(m_xSampleData.m_abUseCompressionQuality[iPlatform])
	{
		iQuality = m_xSampleData.m_aiCompressionQuality[iPlatform];
	}
	else
	{
		iQuality = Tools_SoundEvent_System::GetDefaultPlatformCompressionQuality(iPlatform);		
	}
	PreviewCompression(iPlatform, iQuality);
}

void Tools_SoundEventTab_Sample::PreviewCompression_PlatformAndQuality(int iPlatformAndQuality)
{
	int iPlatform = (iPlatformAndQuality>>16);
	int iQuality = iPlatformAndQuality & 0xffff;

	PreviewCompression(iPlatform, iQuality);
}

void Tools_SoundEventTab_Sample::PreviewCompression(int iPlatform, int iQuality)
{
	#if defined ASURA_USE_AUDIO_SYSTEM
	if(m_xSampleData.m_uWave)
	{
		Asura_String xFilename = Asura_Audio_System::GetWaveFilename(m_xSampleData.m_uWave);
		if(xFilename.IsBlank()) return;

		Tools_SoundConversionPreview::PreviewSound(xFilename.GetString(), iPlatform, iQuality );
	}
	#else
	if(m_xSampleData.m_eSoundID != ASURA_SID_NO_SOUND)
	{
		Asura_String xFilename = Asura_Sound_System::GetFileNameFromSoundID(m_xSampleData.m_eSoundID);
		if(xFilename.IsBlank()) return;

		Tools_SoundConversionPreview::PreviewSound(xFilename.GetString(), iPlatform, iQuality );
	}
	#endif 
	
}

//eof
