// Tools_SoundMixer_Overlay.h
// Created by Richard Rice - 06/07/2006
// Revised by Thomas Young - 08/07/2009
// Copyright (C) 2006 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SOUNDMIXER_OVERLAY__
#define __TOOLS_SOUNDMIXER_OVERLAY__

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura_List.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Mixer.h"

#include "Misc/list_tem.hpp"

// D E F I N E S /////////////////////////////////////////////////////////////

#define TOOLS_MIXERS_MAX_DIALOG_POSITIONS	(32)

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_NameTree;
class Tools_SoundMixerPresetList_Dialog;
class Tools_MixerTree_Dialog;

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

// _______________________________ Tools_TempMixerChannel __________________________________

class Tools_TempMixerChannel
{
public:

	void CopyFromMixerChannel	( const Asura_Hash_ID xChannelHash );
	void CopyToMixerChannel		( const Asura_Hash_ID xChannelHash );
	void SetDefaults			( );

	Asura_Hash_ID m_xChannelPresetHash;	// Current mixer preset associated with this channel
	bool m_bIncludeInPreset;
	
	Asura_Hash_ID m_xChannelHash;		// Hash of the channel name
	Asura_Hash_ID m_xOutputChannelHash;	// Hash of the channel we output to

	bool m_bModified;
	
	Asura_Sound_Mixer_Channel_Settings m_xSettings;

};

// ____________________________ Tools_Mixer_DialogPos __________________________________

class Tools_Mixer_DialogPos
{
	// This class is for remembering the positions of mixer channels

	public:

		Asura_Hash_ID	m_xMixerChannel;
		Asura_Vector_2	m_xPosition;
};

// _______________________________ Tools_SoundMixer_Overlay __________________________________

class Tools_SoundMixer_Overlay : public Asura_Overlay
{

	public :

		// _______________ Functions _________________

		static void ShowMixersOverlay();
		static void CloseMixersOverlay();
		static bool IsOpen();

		static void TransferModeData(int iMode);
		
	private :
	
		// _______________ Functions _________________

		Tools_SoundMixer_Overlay							( );
		~Tools_SoundMixer_Overlay							( );

		void					ShowOverlay					( );
		void					PerFrameUpdate				( );
		
		static	void			_CreateMixerChannel 		( )	{ s_pxThis->CreateMixerChannel(); }
				void			CreateMixerChannel 			( );

		static	void			_SaveMixerChannel			( )	{ s_pxThis->SaveMixerChannel(); }
				void			SaveMixerChannel			( );

		static	void			_DeleteMixerChannel 		( )	{ s_pxThis->DeleteMixerChannel(); }
				void			DeleteMixerChannel			( );

		static	void			_RenameMixerChannel 		( )	{ s_pxThis->RenameMixerChannel(); }
				void			RenameMixerChannel			( );

		static	void			_NewPreset					( )	{ s_pxThis->NewPreset(); }
				void			NewPreset					( );
	
		static	void			_DeletePreset				( )	{ s_pxThis->DeletePreset(); }
				void			DeletePreset				( );
		
		void					LoadPreset					( );

		static bool 			Callback_RequestSelection	( Asura_Hash_ID xChannelHash );
		static void 			Callback_PositionChanged	( Asura_Hash_ID xChannelHash, Asura_Vector_2 xPosition );

		static bool				IsMixerChannelSelected		( )			{ return ( s_xSelectedMixerChannel.m_xChannelHash != ASURA_HASH_ID_UNSET ); }
		static void				SetMixerChannelModified		( )			{ s_xSelectedMixerChannel.m_bModified = true; }
		static void				SaveMixerChannelIfModified	( );
		static void				ClearSelectedMixerChannel	( );

		static bool				IsDefaultPresetSelected		( )			{ return ( GetSelectedPreset() == Asura_GetHashID( "default" ) ); }
		static bool				IsDefaultPresetNotSelected	( )			{ return !IsDefaultPresetSelected(); }

		static Asura_Hash_ID	GetSelectedPreset			( );

		// _______________ Variables _________________

		static Asura_Vector_2						s_xOverlayPosition;
		static Asura_Vector_2						s_xSubwindowSize;

		static u_int								s_uPresetPriority;
		static float								s_fPresetFadeInTime;
		static float								s_fPresetFadeOutTime;
		static float								s_fPresetLifeTime;
		static Asura_Hash_ID						s_uPresetDSP;

		static Tools_Mixer_DialogPos				s_axDialogPositions[ TOOLS_MIXERS_MAX_DIALOG_POSITIONS ];

		static Tools_SoundMixer_Overlay*			s_pxThis;
		static Tools_SoundMixerPresetList_Dialog*	s_pxMixerPresetListDialog;
		static Asura_Dialog*						s_pxMixerChannelNameDialog;
		static bool									s_bShouldUpdateMixerChannelNameDialog;
		static Tools_MixerTree_Dialog*				s_pxMixerTreeDialog;

		static Tools_TempMixerChannel				s_xSelectedMixerChannel;

		static bool									s_bMixerPresetModified;
};


// _______________________________ Tools_MixerChannel_Dialog __________________________________

class Tools_MixerChannel_Dialog : public Asura_Dialog
{
	//This dialog is for rendering a mixer channel

	public:

		// _______________ Functions _________________

		Tools_MixerChannel_Dialog					( const Asura_Hash_ID xChannelHash, const Asura_Vector_2 xOffset );
		virtual ~Tools_MixerChannel_Dialog			( );

		void			Render						( const Asura_Vector_2& xOrigin );
		void			Update						( const Asura_Vector_2& xOrigin );

		Asura_Hash_ID	GetChannelHash				( )															{	return m_xChannelHash;	}

		Asura_Vector_2	GetInputNodePos				( ) const;
		Asura_Vector_2	GetOutputNodePos			( ) const;

		void			SetAsMaster					( )															{ m_bIsMaster = true; }
		bool			IsMaster					( ) const													{ return m_bIsMaster; }
		void			SetIncludedInPreset			( const bool bIncluded )									{ m_bIncludedInPreset = bIncluded; }
		
		void			MoveTo						( const Asura_Vector_2 xPosition );
		Asura_Vector_2	GetOffset					( ) const													{ return m_xOffset; }
	

		static void SetCallback_RequestSelection	( bool (* pfnCallback )( Asura_Hash_ID ) )					{ s_pfnRequestSelection = pfnCallback; }
		static void SetCallback_PositionChanged		( void (* pfnCallback )( Asura_Hash_ID, Asura_Vector_2 ) )	{ s_pfnPositionChanged	= pfnCallback; }

		static void SetMixerChannelDataPointer		( Tools_TempMixerChannel* pxData )							{ s_pxSelectedChannelData = pxData; }
		
		static Tools_MixerChannel_Dialog*				FindMixerDialog		( const Asura_Hash_ID xMixerHash );
		static Asura_List<Tools_MixerChannel_Dialog>*	GetDialogList		( )									{ return &s_xMixerDialogList; }

		static void SetDialogsAcceptMouseInput		( const bool bAccept )										{ s_bDialogsAcceptMouseInput = bAccept; }

	private :

		// _______________ Functions _________________

		bool IsSelected	() { return s_pxSelectedChannelData ? ( s_pxSelectedChannelData->m_xChannelHash == m_xChannelHash ) : false; }

		// _______________ Variables _________________

		Asura_Hash_ID					m_xChannelHash;
		Asura_Vector_2					m_xOrigin;

		bool							m_bIsMaster;

		bool							m_bDragging;
		Asura_Vector_2					m_xDragOrigin;
		
		bool							m_bSelectingOutput;

		bool							m_bHighlightName;
		bool							m_bHighlightBody;
		bool							m_bHighlightInNode;
		bool							m_bHighlightOutNode;

		bool							m_bIncludedInPreset;
		
		static bool (* s_pfnRequestSelection )	( Asura_Hash_ID );					// Can this channel be selected? Passes the hash of this channel
		static void (* s_pfnPositionChanged )	( Asura_Hash_ID, Asura_Vector_2 );	// This dialog has been moved. Passes the hash of this channel + new position

		// _______________ Static variables _________________

		static bool				s_bAnyChannelBeingDragged;			// Are any of the dialogs of this type being dragged?
		static bool				s_bAnyChannelSelectingOutput;		// Are any of the dialogs picking an ouput channel?
		static Asura_Hash_ID	s_xInputNodeSelected;				// If the above is true, dialogs will set this if the cursor is over their input node

		static bool				s_bDialogsAcceptMouseInput;

		static Tools_TempMixerChannel* s_pxSelectedChannelData;

		static Asura_List<Tools_MixerChannel_Dialog> s_xMixerDialogList;
};

// _______________________________ Tools_MixerTree_Dialog __________________________________

class Tools_MixerTree_Dialog : public Asura_Subwindow_Dialog
{
	// This class holds a load of Tools_MixerChannel_Dialog's

	public:
		
		Tools_MixerTree_Dialog  ( const Asura_Vector_2& xOffset, const float fWidth, const float fHeight );
		virtual ~Tools_MixerTree_Dialog ();

		void	SetMixerPreset			( const Asura_Hash_ID xPreset );
		void	SetDialogPositions		( const Tools_Mixer_DialogPos* pxArray, const int iArraySize );
		void	GetDialogPositions		( Tools_Mixer_DialogPos* pxArray, const int iArraySize );

		void	Update( const Asura_Vector_2& xOrigin );
		void	Render( const Asura_Vector_2& xOrigin );

		void	DeleteDialog ( const Asura_Hash_ID xChannelHash );

	protected:

		virtual bool IsPointInDialog (const Asura_Vector_2& xOrigin,const Asura_Vector_2& xPos) const;

	private:

		int		GetMixerChannelDepth ( const Asura_Hash_ID xChannelHash );

		Asura_Hash_ID	m_xMixerPreset;
};

// _______________________________ Tools_SoundMixerPresetList_Dialog __________________________________

class Tools_SoundMixerPresetList_Dialog : public Asura_List_Dialog
{
	// This class displays a list of mixer presets

	public:

		Tools_SoundMixerPresetList_Dialog	( const Asura_Vector_2& xOffset );
		~Tools_SoundMixerPresetList_Dialog	( );

		bool SelectionChanged				( )															{ return m_bSelectionChanged; }
		void ClearSelectionChangedFlag		( )															{ m_bSelectionChanged = false; }

		void UpdateMixerPresetList			( const Asura_Hash_ID xToSelect = ASURA_HASH_ID_UNSET );

		Asura_Hash_ID GetSelectedPresetHash ( )															{ return Asura_GetHashID( m_pszMixerPresetList[ m_iSelectedIndex ] ); }

	private:

		void DeleteMixerPresetList			( );

		u_int			m_uMixerPresetListSize;
		const char**	m_pszMixerPresetList;
		Asura_NameTree*	m_pxMixerPresetNameTree;

		int				m_iSelectedIndex;
		bool			m_bSelectionChanged;
};

// _______________________________ Tools_SoundMixer_Dialog __________________________________

class Tools_SoundMixer_Dialog : public Asura_DropDown_List_Dialog
{
	//drop down list for selecting/creating a sound mixer
	
	public :
		
		Tools_SoundMixer_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxMixer,bool* pbModified,const char* szLabel = "Mixer");
		~Tools_SoundMixer_Dialog();

		static void InvalidateLists(){s_iListValidID++;}
		static int GetListValidID(){return s_iListValidID;}

	private:
		
		void PerFrameUpdate();

		static void OnSelectionChanged(int iThis){((Tools_SoundMixer_Dialog*)iThis)->OnSelectionChanged();}
		void OnSelectionChanged();

		void UpdateMixerList();
		void DeleteMixerList();

		const char* CreateMixerChannel();

		Asura_String* m_pxMixer;
		bool* m_pbModified;


		Asura_String m_xMixerBackup;
		bool m_bSelectionChanged;


		int m_iSelectedIndex;
		bool m_bInitialised;

		u_int m_uMixerListSize;
		const char** m_pszMixerList;
		class Asura_NameTree* m_pxMixerNameTree;

		//for determining validity of current list
		int m_iListValidID;
		static int s_iListValidID;

};

// _______________________________ Tools_SoundMixer_PresetMessage __________________________________

class Tools_SoundMixerHash_Dialog : public Asura_Hash_DropDownList_Dialog
{
	//Drop down list for selecting mixers by hash ID
	public :
	
		Tools_SoundMixerHash_Dialog(Asura_Vector_2& xOffset,int iNumberOfLines,Asura_Hash_ID* puMixerID,bool* pbChangedFlag = 0 ,const char* szName = "Mixer")
		:Asura_Hash_DropDownList_Dialog(xOffset)
		{
			Initialise(xOffset,iNumberOfLines,puMixerID,pbChangedFlag,szName);
		}

	protected :

		virtual u_int GetNumberOfHashValues() const 
		{
			return Asura_Sound_Mixer_System::GetNumberOfChannels();
		}
		virtual const char* GetNameByIndex(int iIndex) const
		{
			const Asura_Sound_Mixer_Channel* pxChannel = Asura_Sound_Mixer_System::GetChannelByIndex(iIndex);
			return pxChannel ? pxChannel->GetName() : "*Error*";
		}

		virtual const char* GetZeroHashText() const {return "*Default*";}

		virtual int GetListValidID() const {return Tools_SoundMixer_Dialog::GetListValidID();}

		static int s_iListValidID;
};

// _______________________________ Tools_SoundMixer_PresetMessage __________________________________

class Tools_SoundMixer_PresetMessage
{
	public:
		
		static void AddOverlayControls(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
};

// _______________________________ Tools_SoundMixerPreset_DropDownList_Dialog __________________________________

class Tools_SoundMixerPreset_DropDownList_Dialog : public Asura_Hash_DropDownList_Dialog
{
	//Used by Tools_SoundMixer_PresetMessage

	public:
	
		Tools_SoundMixerPreset_DropDownList_Dialog(const char* szName, Asura_Vector_2& xOffset,int iNumberOfLines,Asura_Hash_ID* puSelected,bool* pbChangedFlag = 0)
		:Asura_Hash_DropDownList_Dialog(xOffset)
		{
			Initialise(xOffset,iNumberOfLines,puSelected,pbChangedFlag,szName);
		}
	
	protected:
	
		virtual u_int GetNumberOfHashValues() const
		{
			return Asura_Sound_Mixer_System::GetNumberOfPresets();
		}
		
		virtual const char* GetNameByIndex(int iIndex) const
		{
			const Asura_Sound_Mixer_Preset* pxPreset = Asura_Sound_Mixer_System::GetPresetByIndex(iIndex);
			return pxPreset ? pxPreset->GetName() : "*Error*";
		}
};

#endif //__TOOLS_SOUNDMIXER_OVERLAY__

//eof
