//
// Tools_StreamingBackgroundSound_Object.h
//
// Created by Richard Rice - 09/01/04
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_STREAMINGBACKGROUNDSOUND_OBJECT_H__
#define __TOOLS_STREAMINGBACKGROUNDSOUND_OBJECT_H__

#include "Objects/Tools_Object.h"
#include "Objects/Tools_VolumeEdit_Object.h"


class Tools_StreamingBackgroundSound_Object : public Tools_Object
{
public :
	Tools_StreamingBackgroundSound_Object(int iIndex);
	~Tools_StreamingBackgroundSound_Object();

	virtual void UpdatePositionInAsuraData(bool /*bUpdateLinkedObjects*/);
	virtual void Render();
	virtual void Update();
	virtual void UpdateWhileSelected();
	virtual void Destroy();
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual bool IsHidden();
	virtual bool IsSelectable();

	virtual const char* GetName(){return m_szName;}
	virtual const char* GetTypeDescription(){return "Background Sound";}

	virtual Asura_Guid GetGuid() const;
    virtual bool AcquireGuid();
	virtual bool HandlesThisMessage(int iMessageID);


	//copy/paste support
	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const {return true;}
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

	//create a new streaming sound
	static void Create();

	//create all the tools objects from the asura sound system
	static void CreateAllObjects();
	//delete all the tools objects
	static void DeleteAllObjects();

	//Makes sure the streaming sound system and the controller entities agree with each other
	static void SyncWithControllerEntities();

	class Asura_StreamingBackgroundSound* GetSound();

	static Tools_StreamingBackgroundSound_Object* GetSoundObjectFromID(u_int uID);

	static bool DoToolsObjectsExist(){return s_xObjectList.size() > 0;}

	//Returns true if the sounds file can be saved
	bool CanSaveFile() const;

private :
	void CreateOverlay();
	void DeleteOverlay();
	void UpdateOverlay();

	static void _EditOuterVolume(int iThis){((Tools_StreamingBackgroundSound_Object*)iThis)->EditOuterVolume();}
	void EditOuterVolume();

	static void _EditInnerVolume(int iThis){((Tools_StreamingBackgroundSound_Object*)iThis)->EditInnerVolume();}
	void EditInnerVolume();

	static void _EditClipVolume(int iThis){((Tools_StreamingBackgroundSound_Object*)iThis)->EditClipVolume();}
	void EditClipVolume();


	static void _SelectFile(int iThis){((Tools_StreamingBackgroundSound_Object*)iThis)->SelectFile();}
	void SelectFile();

	static void _ClearFile(int iThis){((Tools_StreamingBackgroundSound_Object*)iThis)->ClearFile();}
	void ClearFile();


	bool IsEnabled_Range();
	bool IsEnabled_Height();
	bool IsEnabled_Box();
	bool IsSphere() const;
	bool IsCylinder() const;
	bool IsCuboid() const;
	


	void UpdateOverlapList();

	//Gets bounding box of box or sphere depending on type
	Asura_Bounding_Box GetInnerExtents() const;
	Asura_Bounding_Box GetOuterExtents() const;
	


	int m_iIndex;//index in asura system

	int m_iType;
	float m_fVolume;
	Asura_Hash_ID m_uMixer;
	Asura_Bounding_Box m_xInnerBox;
	Asura_Bounding_Box m_xOuterBox;
	Asura_Quat m_xOrient;
	bool m_bUseOrient;
	u_int m_uLightSets;
	u_int m_uSoundFlags;
	float m_fInnerRadius;
	float m_fOuterRadius;
	float m_fInnerHeight;
	float m_fOuterHeight;
	int m_iPriority;


	bool m_bInnerBoxChanged;
	bool m_bOuterBoxChanged;
	bool m_bChanged;

	int m_iInnerVolumeID;
	int m_iOuterVolumeID;
	class Tools_Default_VolumeGroupEdit_Controller* m_pxClipVolumeController;


	char m_szName[24];


	enum DialogID
	{
		DialogID_Filename,
	};

	bool m_bOverlapListValid;
	List<Asura_Bounding_Box> m_xOverlapList;
	u_int m_uOverlapListLightSets;//light set that the overlap list was calculated under

	Asura_Vector_3 m_xPreviousPosition;


	//statics
	static Asura_Vector_2 s_xOverlayPosition;
	static bool s_bShowInnerBox;
	static bool s_bShowOuterBox;
	static bool s_bShowSelectedOnly;
	static bool s_bWireframe;
	static bool s_bShowOverlap;

	static List<Tools_StreamingBackgroundSound_Object*> s_xObjectList;

	friend class Tools_StreamingVolumeEdit_Object;

};


//Variant of volume edit object that has extra controls for snapping the sides to the edges
//of other streaming sound objects
class Tools_StreamingVolumeEdit_Object : public Tools_VolumeEdit_Object
{
public :
	Tools_StreamingVolumeEdit_Object(const char* pszName,Asura_Bounding_Box* pxBox,bool* pbModifiedFlag,bool bOuter);

	void UpdateWhileSelected();
	virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);

protected :
	virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);

	virtual Asura_Vector_3 GetRelativePivot() const;
	virtual Asura_Vector_3 GetWorldPivot() const;

private :
	bool m_bOuter;

	//snaps the current face to the nearest volume face in the specified direction
	void SnapToVolume(bool bOut);
	void SetToOtherVolume();

	static void SnapIn(int iThis){((Tools_StreamingVolumeEdit_Object*)iThis)->SnapToVolume(false);}
	static void SnapOut(int iThis){((Tools_StreamingVolumeEdit_Object*)iThis)->SnapToVolume(true);}

	static void SetToOtherVolume(int iThis){((Tools_StreamingVolumeEdit_Object*)iThis)->SetToOtherVolume();}
	
};


#endif //__TOOLS_STREAMINGBACKGROUNDSOUND_OBJECT_H__
