//
// Tools_Mode_Streaming.cpp
//
// Created by Richard Rice - 03/03/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#ifdef ASURA_USE_STREAMING
#include "Streaming/Tools_Mode_Streaming.h"

#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Module.h"
#include "Streaming/Asura_Streaming_System.h"
#include "Streaming/Asura_Streaming_Zone.h"

#include "Tools_Overlays.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Streaming/Tools_StreamingConnection_Object.h"
#include "Streaming/Tools_Streaming_System.h"
#include "Streaming/Tools_StreamingModuleNames.h"


Asura_Vector_2 Tools_StreamingConnection_Object::s_xOverlayPosition(30,10);

Asura_Streaming_Zone* Tools_Mode_Streaming::s_pxZone;

int Tools_Mode_Streaming::s_iConnectionIndex = 0;
int Tools_Mode_Streaming::s_iConnectionStringListSize = 0;
char** Tools_Mode_Streaming::s_pszConnectionStrings = 0;

static Asura_Hash_ID g_uSelectedHash = 0;



void Tools_Mode_Streaming::InitialiseMode()
{
	m_xModeOverlayPosition = Asura_Vector_2(3.0f,10.0f);

	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	//Find or create the zone data for this level
	s_pxZone = Tools_Streaming_System::GetStreamingZoneForThisLevel();

	//create connection objects 
	for(u_int i=0;i<s_pxZone->GetNumConnection();i++)
	{
		Asura_Streaming_Connection* pxConnection = s_pxZone->GetConnection(i);
		new Tools_StreamingConnection_Object(pxConnection->GetToZoneName(),s_pxZone);
	}
	CreateModeOverlay();

	//initialise object system
	Tools_Object_System::EnableAllControls(true);
}

void Tools_Mode_Streaming::ShutdownMode()
{
	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	//clearup object system
	Tools_Object_System::DeleteAllModeSpecific();
	Tools_Object_System::EnableAllControls(false);


	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);

	DeleteConnectionStringList();
}

void Tools_Mode_Streaming::QuickSave()
{
	Tools_SaveAsura::SaveStreamingConnections();
}

bool Tools_Mode_Streaming::IsModified()
{
	return Tools_SaveAsura::IsStreamingConnectionsModified();
}

void Tools_Mode_Streaming::Update()
{
	//determine if selection has changed
	if(s_iConnectionIndex<(int)s_pxZone->GetNumConnection())
	{
		const Asura_Streaming_Connection* pxConnection = s_pxZone->GetConnection(s_iConnectionIndex);
		if(g_uSelectedHash != pxConnection->GetToZoneHashID())
		{
			//seelct the  tools object corresponding to the connection
			g_uSelectedHash = pxConnection->GetToZoneHashID();
			Tools_StreamingConnection_Object* pxObject = Tools_StreamingConnection_Object::FindByName(pxConnection->GetToZoneName());
			if(pxObject)
			{
				Tools_Object_System::SelectObject(pxObject);
			}

		}
	}

}

void Tools_Mode_Streaming::RenderUI()
{
}

void Tools_Mode_Streaming::TransferModeData(int iMode)
{
	//open the file
	Tools_ModeDataFile xFile("StreamingMode.dat",iMode);

	//transfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(m_xModeOverlayPosition);
}

void Tools_Mode_Streaming::CreateModeOverlay()
{
	//delete old overlay if it exists
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Streaming Mode",false,&m_xModeOverlayPosition);

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Function_Dialog("Add",xOffset,AddConnection);
	pxDialog->SetHelpText("Add a new connected zone");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteConnection);
	pxDialog->SetHelpText("Delete the connection to the zone selected from the list below");
	pxDialog->SetIsEnabledFunction(CanDeleteConnection);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dialog("Connections :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	//setup list box
	BuildConnectionStringList();

	pxDialog = new Asura_List_Dialog(xOffset,s_pszConnectionStrings,&s_iConnectionIndex,8,s_iConnectionStringListSize,0);
	pxDialog->m_iControlID = Asura_GetHashID("ConnectionList");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;


	pxOverlay->m_uID = TOOLS_OVERLAY_MODE;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Mode_Streaming::BuildConnectionStringList()
{
	DeleteConnectionStringList();

	if(s_pxZone->GetNumConnection())
	{
		s_iConnectionStringListSize = s_pxZone->GetNumConnection();
		s_pszConnectionStrings = new char*[s_iConnectionStringListSize];

		for(int i=0;i<s_iConnectionStringListSize;i++)
		{
			const Asura_Streaming_Connection* pxConnection = s_pxZone->GetConnection(i);
			const char* szName = pxConnection->GetToZoneName();

			s_pszConnectionStrings[i] = new char[strlen(szName)+1];
			strcpy(s_pszConnectionStrings[i],szName);
			
		}
	}
	else
	{
		s_iConnectionStringListSize = 1;
		s_pszConnectionStrings = new char*[1];
		s_pszConnectionStrings[0] = new char[10];
		strcpy(s_pszConnectionStrings[0],"*None*");
	}

	s_iConnectionIndex = Asura_Maths::ClampToRange(s_iConnectionIndex,0,s_iConnectionStringListSize-1);
}

void Tools_Mode_Streaming::DeleteConnectionStringList()
{
	for(int i=0;i<s_iConnectionStringListSize;i++)
	{
		delete [] s_pszConnectionStrings[i];
	}
	delete [] s_pszConnectionStrings;

	s_iConnectionStringListSize = 0;
	s_pszConnectionStrings = 0;
}

void Tools_Mode_Streaming::UpdateConnectionList()
{
	//get hold of the list control
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_MODE);
	if(!pxOverlay) return;

	Asura_List_Dialog* pxDialog = (Asura_List_Dialog*) pxOverlay->GetDialog(Asura_GetHashID("ConnectionList"));
	if(!pxDialog) return;

	//and update
	BuildConnectionStringList();
	pxDialog->ReplaceStrings(s_iConnectionStringListSize,s_pszConnectionStrings);
}



void Tools_Mode_Streaming::AddConnection()
{
	//Build a list of environment names to determine available connections
	char szSearch[MAX_PATH+20];
	Asura_FileAccess::PlatformGetNetworkDir("Environments",szSearch,MAX_PATH);
	strcat(szSearch, "\\");
	Asura_String xToRemove(szSearch);
	strcat(szSearch,"*.asr");

	List<Asura_String> xFilenames;
	Tools_FileSystem::FindFiles(szSearch,xFilenames,TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR);

	//add all the names to a selection overlay
	ListSelection_Overlay xOverlay("Add a connection to which zone?");

	for(LIF<Asura_String> xFileLIF(&xFilenames);!xFileLIF.done();xFileLIF.next())
	{
		//want name without path and extension
		Asura_String xName(xFileLIF());
		xName.Replace(xToRemove.GetString(), "");

		//make sure the name doesn't belong to this zone , or an already connected zone
		Asura_Hash_ID uHash = Asura_GetHashID(xName.GetString());
		if(uHash == s_pxZone->GetHashID()) continue;

		if(s_pxZone->HasConnection(uHash)) continue;
		
		xOverlay.AddItem(xName.GetString(),(int)xFileLIF().GetString());	
	}

	if(!xOverlay.DoModal()) return;

	//Add the selected zone as a new connection
	Asura_String xDisplayedFilename(xOverlay.GetSelectionString());
	Asura_String xName;
	const char* szFilename = (const char*) xOverlay.GetSelectionData();

	xDisplayedFilename.SplitFilename(NULL, &xName, NULL);
	s_pxZone->AddConnection(xName.GetString(),Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(szFilename));

	//module system needs to know about the new connection
	Asura_Module_System::FindConnectedHandlers();

	UpdateConnectionList();
	Tools_SaveAsura::SetStreamingConnectionsModified();

	//select the last entry in the list , since this should be the new connection
	s_iConnectionIndex = s_pxZone->GetNumConnection()-1;

	//create tools object for this connection
	new Tools_StreamingConnection_Object(xName.GetString(),s_pxZone);

	//make sure the new zone is loaded
	Tools_Streaming_System::LoadConnectedZones();

}

bool Tools_Mode_Streaming::CanDeleteConnection()
{
	return s_iConnectionIndex>=0 && (s_iConnectionIndex < (int)s_pxZone->GetNumConnection());
}


void Tools_Mode_Streaming::DeleteConnection()
{
	if(s_iConnectionIndex>=(int)s_pxZone->GetNumConnection()) return;

	const Asura_Streaming_Connection* pxConnection = s_pxZone->GetConnection(s_iConnectionIndex);

	if(!ShowYesNoOverlay("Delete the connection to %s?",pxConnection->GetToZoneName())) return;

	Asura_Hash_ID uHash = Asura_GetHashID(pxConnection->GetToZoneName());

	//unload the connected zone
	Tools_Streaming_System::UnloadZone(uHash);


	//delete tools object for connection
	Tools_StreamingConnection_Object* pxObject = Tools_StreamingConnection_Object::FindByName(pxConnection->GetToZoneName());
	if(pxObject)
	{
		Tools_Object_System::DeleteObject(pxObject);
	}

	//and connection in asura code
	s_pxZone->DeleteConnection(s_iConnectionIndex);

	//module system needs to know about the removed connection
	Asura_Module_System::FindConnectedHandlers();
	//Redo module link indices
	Tools_StreamingModuleNames::ResyncAll();

	UpdateConnectionList();
	Tools_SaveAsura::SetStreamingConnectionsModified();
}


#endif //ASURA_USE_STREAMING
