//
// Tools_Streaming_System.h
//
// Created by Richard Rice - 06/04/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_STREAMING_SYSTEM_H__
#define __TOOLS_STREAMING_SYSTEM_H__
#ifdef ASURA_USE_STREAMING

#include "Core/Asura_Hash.h"
#include "Debugging/Asura_Assert.h"
#include "Streaming/Asura_Streaming_System.h"

class Asura_Streaming_Zone;

class Tools_Streaming_System
{
	friend class Asura_Handler_System;
public :
	static void Initialise();

	//Gets/Allocates streaming zone for this level
	static Asura_Streaming_Zone* GetStreamingZoneForThisLevel();

	static Asura_Hash_ID GetHashForThisLevel()
	{
		Asura_Assert(s_uHashForThisLevel,"Not setup yet.");
		return s_uHashForThisLevel;
	}

	//returns true if the streaming system is set to the zone for the edited level
	static bool IsSetToMainZone()
	{
		Asura_Streaming_Zone*	pxZone	= Asura_Streaming_System::GetCurrentZone();

		Asura_Assert(s_uHashForThisLevel,"Not setup yet.");
		return s_uHashForThisLevel == pxZone->GetHashID();
	}

	static void LoadConnectedZones();
	static void UnloadZone(Asura_Hash_ID uHash);

private :
	//functions used by the streaming subsystem handlers
	static void CreateZoneData(Asura_Hash_ID uHash);
	static void DestroyZoneData(Asura_Hash_ID uHash);
	static void SetZoneData(Asura_Hash_ID uHash);
	
	//Ensures tools systems get initialised for newly loaded zones
	static void InitialiseZoneDataForCurrentZone();

	//sorts out filenames for old connections
	static void UpdateConnectionFilename(int iIndex);


	static Asura_Hash_ID s_uHashForThisLevel;


};

/*
This class iterates over the main zone and all of the connected zones.
If bSetZone is set to true then the current zone will be updated in Asura_Streaming_System
as it goes.
The zone will be reset to the initial one when the iterator is destroyed , or it reaches the
end of the iteration  ,or when End() is called.
*/

class Tools_Streaming_Iterator
{
public :
	Tools_Streaming_Iterator(bool bSetZone = true);
	~Tools_Streaming_Iterator(){End();}

	void Restart();
	//Ends the iteration , resetting the handler if needed
	void End();
	//Moves to next zone
	void Next();
	//Returns true when iteration has finished
	bool Done() const;

	Asura_Hash_ID GetHash() const;

	//Returns true if currently set to the main zone
	bool IsMainZone() const {return m_iIndex == 0;}

private :
	bool m_bSetZone;
	bool m_bInitialZoneValid;
	const Asura_Streaming_Zone* m_pxMainZone;
	const Asura_Streaming_Zone* m_pxInitialZone;
	const Asura_Streaming_Zone* m_pxCurrentZone;
	int m_iIndex;

};


#else //ASURA_USE_STREAMING

//Dummy version of the zone iterator so that code using it will work when streaming is turned off

class Tools_Streaming_Iterator
{
public :
	Tools_Streaming_Iterator(bool bSetZone = true){Restart();}

	void Restart(){m_bDone = false;}
	//Ends the iteration , resetting the handler if needed
	void End(){m_bDone = true;}
	//Moves to next zone
	void Next(){m_bDone = true;}
	//Returns true when iteration has finished
	bool Done() const {return m_bDone;}

	Asura_Hash_ID GetHash() const{return 0;}
	bool IsMainZone() const {return true;}
private :
	bool m_bDone;

};

#endif //ASURA_USE_STREAMING
#endif //__TOOLS_STREAMING_SYSTEM_H__