//
// Tools_Mode_TextureTiler.cpp
//
// Created by Nick Brett (NDB) as a direct port from 
// the stand alone Texture_Tiler application written 
// by Neil Massey
//
// 07/09/2007 
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "TextureTiler/Tools_Mode_TextureTiler.h"

#include "Core/Asura_UserInput.h"
#include "Core/Asura_Windows.h"
#include "ChunkSystem/Asura_Chunk_TiledTexture.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Platform.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Text_Output.h"

// Tools
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_DynamicArray.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Texture_Management.h"

// <-- class Tools_Mode_TextureTiler -->
Tools_Mode_TextureTiler* Tools_Mode_TextureTiler::s_pxPtrToSelf( NULL );

Asura_String Tools_Mode_TextureTiler::s_xDefaultFileToLoad("");

bool Tools_Mode_TextureTiler::s_abOptionFlag[TOOLS_TT_NUM_OPTIONS_FLAGS];
bool Tools_Mode_TextureTiler::s_abTextureExportFlag[Tools_TT_FileType::TOOLS_TT_FILE_NAME_EXTENSION_NUM];
bool Tools_Mode_TextureTiler::s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_FILE_NAME_EXTENSION_NUM];		  
bool Tools_Mode_TextureTiler::s_abExportPlatformFlags[TOOLS_PLATFORM_NUM_PLATFORMS];

int	  Tools_Mode_TextureTiler::s_iTextureFlags(0);
u_int Tools_Mode_TextureTiler::s_uPreviewDisplayedTextureIndex(0);

Tools_TT_OverlayStatus Tools_Mode_TextureTiler::s_xOptionsOverlayStatus;
Tools_TT_OverlayStatus Tools_Mode_TextureTiler::s_xTextureOverlayStatus;
Tools_TT_OverlayStatus Tools_Mode_TextureTiler::s_xPreviewOverlayStatus;
Tools_TT_OverlayStatus Tools_Mode_TextureTiler::s_xExportOverlayStatus;

static const char* Tools_TT_HelpStrings [] =
{
	"Use the Left Mouse Button (LMB) to select textures and the Right Mouse Button (RMB)",
	"to remove textures. Pressing the RMB when the mouse cursor is over a selected      ",
	"texture will remove that texture from the TTL. Pressing the RMB when the cursor is ",
	"over any other texture will select that texture. The selected texture is shown in  ",
	"the preview overlay, along with its size in pixels and the position of its top-left",
	"and bottom right corners ((u1,v1) and (u2,v2) respectivly) in the TTL as fractions ",
	"of the TTL's width and height.                                                     ",
	"                                                                                   ",
	"Keybord Shortcuts:                                                                 ",
	"Delete,Del or Backspace:               Remove selected texture                     ",
	"Ctl + s:                               Save work as a *.ttl file                   ",
	"Ctl + shift + s:                       Save work as a *.ttl file with a specified  ",
	"                                       file name                                   ",
	"                                                                                   ",
	"Ctl + t:                               Save work as a texture file. The type of    ",
	"                                       texure (bmp,tga etc) is set by the extension",
	"                                       on the file name provided.                  ",
	"Ctl + e:                               Export work as an asr file for a specific   ",
	"                                       platform.                                   ",
	"Ctl + shift + e:                       Export work as an asr file with a specified ",
	"                                       file name for a specific platform           ",
	"                                                                                   ",
	"F1                                     Set Alpha Mode to Opaque                    ",
	"F2                                     Set Alpha Mode to transparent black         ",
	"F3                                     Set Alpha Mode to transparent magenta       ",
	"Ctl + h:                               Show this help text                         ",
	NULL,
};

static const char* Tools_TT_TextureFlagNames [] =
{
	"8BitAlpha",                                //Tells Tim2 exporter that 8 bit alpha should be used
	"NoMipMaps",                               
	"WIIOnly_NoMipMaps",
	"PS2Allow512",                     
	"PS2DontHalveSize",
	"DontPremultiplyAlphaInPaletteGeneration",  //For textures where alpha channel isn't being used for normal translucency. Don't premultiply rgb by alpha when working out distances between colours
	"ReduceExtraForConsoles", 
	"PS2DontHalveExtraForMultiplayer",			//Prevents the additional reduction in texture size for PS2 multiplayer
	"16ColourPalette",							//(RAR 12/01/05) Causes the texture to be exported with 4 bit palette (PS2 only most likely)
	"HalfForXBox",
	"HalfForPSP",								//(RAR 15/05/2005 force halving texture size for PSP export.)
	"HighQualityCompression",
	
	NULL
};

static const int Tools_TT_TextureFlag [] =
{                            
	AsuraTextureFlag_8BitAlpha,
	AsuraTextureFlag_NoMipMaps,
	AsuraTextureFlag_WIIOnly_NoMipMaps,
	AsuraTextureFlag_PS2Allow512,
	AsuraTextureFlag_PS2DontHalveSize,
	AsuraTextureFlag_DontPremultiplyAlphaInPaletteGeneration,
	AsuraTextureFlag_ReduceExtraForConsoles,
	AsuraTextureFlag_PS2DontHalveExtraForMultiplayer,
	AsuraTextureFlag_16ColourPalette,
	AsuraTextureFlag_HalfForXBox,
	AsuraTextureFlag_HalfForPSP,
	AsuraTextureFlag_HighQualityCompression,
};

static const u_int uTextureFlagIndexMax(12);

Tools_Mode_TextureTiler::Tools_Mode_TextureTiler( void )
{}

Tools_Mode_TextureTiler::~Tools_Mode_TextureTiler( void )
{}

void Tools_Mode_TextureTiler::InitialiseMode()
{
	InitialiseTTMode();
}

void Tools_Mode_TextureTiler::ShutdownMode()
{
	ShutdownTTMode();
}

void Tools_Mode_TextureTiler::InitialiseTTMode()
{
	// there should only ever be one instance of this class at any one time
	Asura_Assert( s_pxPtrToSelf == NULL,"Pointer To Self should be NULL at this point" );
	
	// set static pointer to self to refer to this instance of TextureTiler. There can be only one
	s_pxPtrToSelf = this;

	m_uOverlayColour  = Tools_TT_Colour::Black;
	m_uUIBorderColour = Tools_TT_Colour::White;

	m_xCurrentTTL.SetBorderColour( m_uUIBorderColour );
	m_xMenuHandle = ASURA_INVALID_HANDLE;
	m_uTTLSelectedTextureIndex = 0;

	// initialise texture export flags
	s_iTextureFlags = AsuraTextureFlag_NoMipMaps | AsuraTextureFlag_PS2Allow512 | AsuraTextureFlag_HighQualityCompression | AsuraTextureFlag_8BitAlpha;
	 
	InitialiseMenubarForMode();

	// if default file name has been defined then attempt to load TTL from it
	if( !s_xDefaultFileToLoad.IsBlank() )
	{
		// check file extension and call appropriate load routine
		char szDefaultFileName[256];
		strncpy( szDefaultFileName,s_xDefaultFileToLoad.GetString(),256);
		char* pcDot( strrchr( szDefaultFileName,'.') );
	
		if( pcDot && ( stricmp(pcDot,".ttl")==0 ) )
		{
			m_xTTL.ReadFile( s_xDefaultFileToLoad );
		}
		else if( ( pcDot && ( stricmp(pcDot,".asr")==0 ) ) || ( pcDot && ( stricmp(pcDot,".asrbe")==0 )) )
		{
			m_xTTL.ReadChunk( s_xDefaultFileToLoad );
		}
		else
		{
			Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::Red,"Could not determine file type for %s, file not loaded",s_xDefaultFileToLoad.GetString() );
			NewTTL();
		}
	}
	else
	{
		NewTTL();
	}

	Asura_Texture_Management::LoadTextures();

	UpdateTTL();
	ResizeUI();

	// TTL was either loaded from disk or a default so should be marked as unmodified
	m_xTTL.SetModified(false);

	// set default options
	s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_ALL] = false;
	s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_TGA] = true;
	s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_JPG] = true;
	s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_PNG] = true;
	s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_BMP] = true;
	s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_DDS] = true;

	s_abOptionFlag[TOOLS_TT_AUTOTILE]	            			= false;
	s_abOptionFlag[TOOLS_TT_COMPRESS_CHUNKS]          			= false;
	s_abOptionFlag[TOOLS_TT_FIX_TTL_DIMENSIONS_TO_POWER_OF_TWO] = false;
	s_abOptionFlag[TOOLS_TT_HAS_PADDING]              			= false;
	s_abOptionFlag[TOOLS_TT_PREFIX_IMAGE_DATA_NAME]   			= false;
	s_abOptionFlag[TOOLS_TT_RELOAD_CHUNK_AFTER_EXPORT]			= true;
	s_abOptionFlag[TOOLS_TT_SHOW_REMOVE_CONFIRMATION] 			= true;
	s_abOptionFlag[TOOLS_TT_CREATE_TEXTURE_ON_EXPORT]           = true;

	ResetExportFlags();
	
	// configure overalys
	const float fViewportWidthInOverlayFormat(m_iLastKnownViewportWidth/ASURA_TEXT_OUTPUT_FONT_WIDTH);
	const float fViewportHeightInOverlayFormat(m_iLastKnownViewportHeight/ASURA_TEXT_OUTPUT_FONT_HEIGHT);

	s_xTextureOverlayStatus.SetPosition( fViewportWidthInOverlayFormat-20.0f,0.0f );
	s_xPreviewOverlayStatus.SetPosition( fViewportWidthInOverlayFormat-20.0f,fViewportHeightInOverlayFormat-27.5f );
	s_xOptionsOverlayStatus.SetPosition( fViewportWidthInOverlayFormat*0.5f - 10.0f, fViewportHeightInOverlayFormat*0.5f - 15.0f );
	s_xExportOverlayStatus.SetPosition(  fViewportWidthInOverlayFormat*0.5f - 10.0f, fViewportHeightInOverlayFormat*0.5f - 15.0f );

	s_xTextureOverlayStatus.SetCreated(false);
	s_xPreviewOverlayStatus.SetCreated(false);
	s_xOptionsOverlayStatus.SetCreated(false);
	s_xExportOverlayStatus.SetCreated(false);

	m_fPreviewOverlayWidth = 256.0f;

	m_eAlphaMode = TOOLS_TT_ALPHA_TRANSPARENT_BLACK;
	m_ePreviewDisplayedAlphaMode = m_eAlphaMode;
	
	// set user interface to mirror state of current TTL
	SynchronizeUIWithCurrentTTL();

	// initialise overlays
	ToggleTextureOverlay();
	TogglePreviewOverlay();

	Tools_System::SetHideToolsStatusText(true);

	Tools_Help::AddHelpTopic( "Texture Tiler Help",Tools_TT_HelpStrings );
}

void Tools_Mode_TextureTiler::ShutdownTTMode()
{
	// wait untill confirmation box is closed
	while( m_xConfirmationOverlay.IsActive() );

	Tools_Help::RemoveHelpTopic( "Texture Tiler Help" );

	Tools_System::SetHideToolsStatusText(false);

	ShutdownMenubarForMode();

	// unload all textures loaded by this mode
	for( u_int u(0);u<m_xLoadedTextureCollection.GetSize();++u )
	{
		Asura_Texture_Management::DestroyTexture(m_xLoadedTextureCollection[u]);
	}

	s_pxPtrToSelf = NULL;
}

void Tools_Mode_TextureTiler::SetDefaultFileToLoad( const Asura_String& xDefaultFile )
{
	s_xDefaultFileToLoad = xDefaultFile;
}

void Tools_Mode_TextureTiler::ResizeUI()
{
	m_iLastKnownViewportWidth  = Asura_Viewport::GetWidth();
	m_iLastKnownViewportHeight = Asura_Viewport::GetHeight();
	const float fPadding( static_cast<float>(m_iLastKnownViewportWidth + m_iLastKnownViewportHeight) * 0.005f );

	// find width and height of inner rectangle on which TTL will be displayed
	const float fInnerWidth( m_iLastKnownViewportWidth - 4.0f*fPadding );
	const float fInnerHeight( m_iLastKnownViewportHeight - 4.0f*fPadding );

	float fDisplayX;
	float fDisplayY;

	// if TTL is bigger than viewport then scale it to fit, else draw it actual size
	if( (m_xTTL.GetWidth()  > fInnerWidth) ||
		(m_xTTL.GetHeight() > fInnerHeight) )
	{
		float fScaleFactor;

		if( m_xTTL.GetWidth() > m_xTTL.GetHeight() )
		{
			fScaleFactor = fInnerWidth/static_cast<float>(m_xTTL.GetWidth());
		}
		else
		{
			fScaleFactor = fInnerHeight/static_cast<float>(m_xTTL.GetHeight());
		}
	
		fDisplayX = fScaleFactor*static_cast<float>(m_xTTL.GetWidth());
		fDisplayY = fScaleFactor*static_cast<float>(m_xTTL.GetHeight());
	}
	else
	{
		fDisplayX = static_cast<float>(m_xTTL.GetWidth());
		fDisplayY = static_cast<float>(m_xTTL.GetHeight());
	}

	m_xCurrentTTL.SetOuterRectangle( fPadding,fPadding,fDisplayX + 2.0f*fPadding ,fDisplayY + 2.0f*fPadding );
	m_xCurrentTTL.SetPadding( fPadding );
}

void Tools_Mode_TextureTiler::InitialiseMenubarForMode()
{	
	//destroy the old menubar
	ShutdownMenubarForMode();
	
	//create a new menubar
	m_xMenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* xTitle;

	xTitle = Asura_Menubar_System::AddTitle(m_xMenuHandle,"TextureTiler");
	xTitle->AddItem( "Texture",ToggleTextureOverlay,TextureOverlayCreated );
	xTitle->AddItem( "Preview",TogglePreviewOverlay,PreviewOverlayCreated );
	xTitle->AddItem( "Options",ToggleOptionsOverlay,OptionsOverlayCreated );
}

void Tools_Mode_TextureTiler::ShutdownMenubarForMode()
{
	// shutdown overlays
	CloseOverlay( TOOLS_OVERLAY_TT_OPTIONS );
	CloseOverlay( TOOLS_OVERLAY_TT_EXPORT );
	CloseOverlay( TOOLS_OVERLAY_TT_TEXTURE );
	CloseOverlay( TOOLS_OVERLAY_TT_CONFIRMATION );
	CloseOverlay( TOOLS_OVERLAY_TT_PREVIEW );

	if(m_xMenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(m_xMenuHandle);
		m_xMenuHandle = ASURA_INVALID_HANDLE;
	}
}

bool Tools_Mode_TextureTiler::IsSelectedTextureValid() const 
{ 
	return (m_uTTLSelectedTextureIndex < m_xTTL.Size()) && (m_xTTL[m_uTTLSelectedTextureIndex].GetTextureID() != ASURA_TEXTURE_NONE);
}

void Tools_Mode_TextureTiler::RemoveSelectedTextureFromTTL() 
{
	if( !s_pxPtrToSelf->IsSelectedTextureValid() )
	{
		return;
	}

	s_pxPtrToSelf->m_xTTL.Erase( s_pxPtrToSelf->m_xTTL[s_pxPtrToSelf->m_uTTLSelectedTextureIndex].GetTextureID());
	s_pxPtrToSelf->UpdateTTL();
	s_pxPtrToSelf->UpdatePreviewOverlay(true);
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();
}

void Tools_Mode_TextureTiler::RemoveSelectedTextureFromTTLWithConfirmation() 
{
	if( !s_pxPtrToSelf->IsSelectedTextureValid() )
	{
		return;
	}

	if( s_abOptionFlag[TOOLS_TT_SHOW_REMOVE_CONFIRMATION] )
	{
		Asura_String xText;
		xText += "Really Remove \"";
		xText += s_pxPtrToSelf->m_xTTL[s_pxPtrToSelf->m_uTTLSelectedTextureIndex].GetName();
		xText += "\" From TTL?";

		s_pxPtrToSelf->m_xConfirmationOverlay.Create( xText,&RemoveSelectedTextureFromTTL );
	}
	else
	{
		RemoveSelectedTextureFromTTL();
	}
}

void Tools_Mode_TextureTiler::NewTTL()
{
	// configure default TTL
	s_pxPtrToSelf->m_xTTL = Tools_TT_TiledTextureList();
	
	s_pxPtrToSelf->m_xTTL.SetWidth(256);
	s_pxPtrToSelf->m_xTTL.SetHeight(256);

	s_pxPtrToSelf->m_xTTLFileName.MakeBlank();
	s_pxPtrToSelf->m_xTTLChunkName.MakeBlank();
	
	// set user interface to mirror state of current TTL
	s_pxPtrToSelf->UpdatePreviewOverlay(true);
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();
	s_pxPtrToSelf->m_xTTL.SetModified(false);
}

void Tools_Mode_TextureTiler::NewTTLWithConfirmation()
{
	if( s_pxPtrToSelf->m_xTTL.IsModified() )
	{
		s_pxPtrToSelf->m_xConfirmationOverlay.Create( "Save Current TTL Before Creating A New One?",&SaveTTL,&NewTTL );	
	}
	else
	{
		NewTTL();
	}
}

void Tools_Mode_TextureTiler::SaveTTL()
{
	// call SaveAs() method if file name not defined
	if( (s_pxPtrToSelf->m_xTTLFileName.IsNull()) || (s_pxPtrToSelf->m_xTTLFileName.IsBlank()) )
	{
		SaveTTLAs();
	}
	else 
	{
		Asura_String xFileName( s_pxPtrToSelf->m_xTTLFileName );
		Tools_TT_ReplaceFileExtension( xFileName,".ttl" );       // ensure file has .ttl extension

		if( s_pxPtrToSelf->m_xTTL.WriteFile(xFileName)  )
		{
			SetDefaultFileToLoad( s_pxPtrToSelf->m_xTTLFileName ); // make this latest successfull save the default file
			Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"%s saved to disk",xFileName.GetString() );
		}
		else
		{
			Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::Red,"%s NOT saved to disk",xFileName.GetString() );
		}
	}
}

void Tools_Mode_TextureTiler::SaveTTLAs()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = ".ttl";

	Asura_String xFileName( (s_pxPtrToSelf->m_xTTLFileName.IsBlank())? s_xDefaultFileToLoad:s_pxPtrToSelf->m_xTTLFileName );
	Tools_TT_ReplaceFileExtension( xFileName,"" );

	info.DefaultFilename  = xFileName.GetString();

	info.DefaultPath = "C:\\";
	info.FileFilter = "TTL Files\0*.ttl;\0";
	info.WindowTitle = "Save TTL As";
	info.AllowMultipleSelect = false;

	xFileName = Tools_FileSystem::GetSaveFilename(info);

	if( xFileName.IsNull() || xFileName.IsBlank() )
	{
		return;
	}

	s_pxPtrToSelf->m_xTTLFileName = xFileName;

	SaveTTL();
}

void Tools_Mode_TextureTiler::SaveTextureAs()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	Asura_String xDefaultFileName( (s_pxPtrToSelf->m_xTTLFileName.IsBlank())? s_xDefaultFileToLoad:s_pxPtrToSelf->m_xTTLFileName );
	Tools_TT_CreateTextureFileName( xDefaultFileName );       
	
	info.DefaultExtension = "";
	info.DefaultFilename = xDefaultFileName.GetString();
	info.DefaultPath = "Graphics\\";
	info.WindowTitle = "Save Texture As (file type determined by extension)";
	info.AllowMultipleSelect = false;
	info.FileFilter = "*.* \0 *.*; \0";

	Asura_String xPath( Tools_FileSystem::GetSaveFilename(info) );

	if( xPath.IsBlank() )
	{
		return;
	}

	// determine file extension and call appropriate save texture method 
	s_pxPtrToSelf->m_xTTL.WriteTexture( xPath );
}

void Tools_Mode_TextureTiler::LoadTTLFromDiskWithConfirmation()
{
	if( s_pxPtrToSelf->m_xTTL.IsModified() )
	{
		s_pxPtrToSelf->m_xConfirmationOverlay.Create( "Save TTL Before Load?",&SaveTTL,&LoadTTLFromDisk );	
	}
	else
	{
		LoadTTLFromDisk();
	}
}

void Tools_Mode_TextureTiler::LoadTTLFromDisk()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	Asura_String xDefaultFileName( (s_pxPtrToSelf->m_xTTLFileName.IsBlank())? s_xDefaultFileToLoad:s_pxPtrToSelf->m_xTTLFileName );
	Tools_TT_ReplaceFileExtension( xDefaultFileName,".ttl" );       

	info.DefaultExtension = ".ttl";
	info.DefaultFilename  = xDefaultFileName.GetString();
	info.DefaultPath = "";
	info.FileFilter = "TTL Files\0*.ttl;\0";
	info.WindowTitle = "Load TTL...";
	info.AllowMultipleSelect = false;

	Asura_String xFileName( Tools_FileSystem::GetLoadFilename(info) );

	// if filename is blank assume file picker was canceled/closed and do nothing - NDB
	if( xFileName.IsNull() || xFileName.IsBlank() )
	{
		return;
	}

	// assign TTL member data  
	s_pxPtrToSelf->m_xTTL.ReadFile( xFileName );
	
	s_pxPtrToSelf->UpdatePreviewOverlay(true);
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();
	s_pxPtrToSelf->m_xTTL.SetModified(false);

	s_pxPtrToSelf->m_xTTLFileName = xFileName;
	SetDefaultFileToLoad( xFileName );
}

const Tools_DynamicArray<int> Tools_Mode_TextureTiler::LoadTextureFromDisk_Implimentation()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = "";
	info.DefaultFilename  = "";
	info.DefaultPath = "Graphics\\";
	info.WindowTitle = "Add Texture To TTL";
	info.AllowMultipleSelect = true;
	
	// build texture file name extension filter list
	Asura_String xFileFilter; 

	if( s_abFileExtensionFilterFlag[Tools_TT_FileType::TOOLS_TT_ALL] )
	{
		xFileFilter += "*";
		xFileFilter += Tools_TT_FileType::GetName(Tools_TT_FileType::TOOLS_TT_ALL);
		xFileFilter += ";";
	}
	else
	{
		for( u_int u(1);u<Tools_TT_FileType::TOOLS_TT_FILE_NAME_EXTENSION_NUM;++u )
		{
			if( s_abFileExtensionFilterFlag[u] )
			{
				xFileFilter += "*";
				xFileFilter += Tools_TT_FileType::GetName(u);
				xFileFilter += ";";
			}
		}
	}
	
	// copy file name extensions into filter specifcation string, being careful to preserve null 
	// characters before and after  - NDB
	char szFileFilter[128];
	strcpy( szFileFilter,"Texture Files");
	strcpy( szFileFilter+strlen(szFileFilter)+1,xFileFilter.GetString() );
	
	info.FileFilter = szFileFilter;

	// retrive first file name from file picker
	Asura_String xFileName( Tools_FileSystem::GetLoadFilename(info) );

	Tools_DynamicArray<int> xTextureIDCollection;
	int iTextureID;

	// if filename is blank assume file picker was canceled/closed and do nothing - NDB
	if( xFileName.IsBlank() )
	{
		return xTextureIDCollection;
	}

	// load textures
	for(;!xFileName.IsNull();xFileName = Tools_FileSystem::GetNextLoadFilename())
	{
		Asura_String xRelativeFileName;

		{
			_strlwr(xFileName.GetString());
			char *start = strstr(xFileName.GetString(),"graphics");

			if (!start)
			{
				Asura_Assert(0,"File not in 'graphics' folder");
				xRelativeFileName = xFileName;
			}
			else
			{
				start+=strlen("graphics");
				xRelativeFileName = start;
			}
		}

		//make sure this isn't one of the generated DDS files
		Asura_String xPath;
		xFileName.SplitFilename(&xPath,0,0);
		const char* pcSlashPos = strrchr(xPath.GetString(),'\\');
		const char* pszLastDir = pcSlashPos ? pcSlashPos+1 : xPath.GetString();
		
		if(!stricmp(pszLastDir,"dds"))
		{
			Asura_Assert(0,"Textures shouldn't be added from the generated DDS directories");
			continue;
		}

		iTextureID = Asura_Texture_Management::AddTexture( xRelativeFileName );
		xTextureIDCollection += iTextureID;

		// keep record of all textures loaded by this so they can be unloaded when on TT mode shutdown
		m_xLoadedTextureCollection += iTextureID;
	}

	Asura_Texture_Management::LoadTextures();

	return xTextureIDCollection;
}

void Tools_Mode_TextureTiler::LoadTextureFromDisk()
{
	s_pxPtrToSelf->LoadTextureFromDisk_Implimentation();
}

void Tools_Mode_TextureTiler::AddTextureFromDisk()
{
	Tools_DynamicArray<int> xTextureIDCollection( s_pxPtrToSelf->LoadTextureFromDisk_Implimentation() );

	// add textures to TTL
	for( u_int u(0);u<xTextureIDCollection.GetSize();++u )
	{
		s_pxPtrToSelf->m_xTTL.Insert( xTextureIDCollection[u] );
	}

	// update TTL
	s_pxPtrToSelf->UpdateTTL();
	s_pxPtrToSelf->UpdatePreviewOverlay(true);
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();
}

void Tools_Mode_TextureTiler::ToggleOptionsOverlay()
{
	if( s_xOptionsOverlayStatus.IsCreated() )
	{
		CloseOverlay( TOOLS_OVERLAY_TT_OPTIONS );
		s_xOptionsOverlayStatus.SetCreated(false);
	}
	else
	{
		Asura_Overlay* pxOverlay = new Asura_Overlay( "Options",false,s_xOptionsOverlayStatus.GetPtrToPosition(),s_xOptionsOverlayStatus.GetPtrToMinimised() );
		
		Asura_Dialog* pxDialog;
		Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

		// file filter options
		pxDialog = new Asura_Dialog( "Texture File Filters:",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// add check box for each know file type
		for( u_int uFileExtension(0);uFileExtension<Tools_TT_FileType::TOOLS_TT_FILE_NAME_EXTENSION_NUM;++uFileExtension )
		{
			pxDialog = new Asura_Tickbox_Dialog( Tools_TT_FileType::GetName(uFileExtension),xOffset,&s_abFileExtensionFilterFlag[uFileExtension] );
			pxDialog->SetHelpText("");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
		}

		xOffset.y += Asura_Dialog::DefaultSpacing();

		// export options
		pxDialog = new Asura_Dialog( "Export Options:",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog( "Prefix Tile Name",xOffset,&s_abOptionFlag[TOOLS_TT_PREFIX_IMAGE_DATA_NAME] );
		pxDialog->SetHelpText( "If ticked, the image texture name in any exported chunks will be prefixed with \"tile_\"" );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog( "Reload After Export",xOffset,&s_abOptionFlag[TOOLS_TT_RELOAD_CHUNK_AFTER_EXPORT] );
		pxDialog->SetHelpText( "If ticked, a texture tiler chunk file will be loaded automatically loaded after export, making it availible to the rest of tools." );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		// user interface options
		pxDialog = new Asura_Dialog( "User Interface:",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog( "Check Mipmap Compatibility",xOffset,&s_abOptionFlag[TOOLS_TT_FIX_TTL_DIMENSIONS_TO_POWER_OF_TWO] );
		pxDialog->SetHelpText( "If ticked, the width and height of the TTL will be fixed to the nearest power of 2" );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog( "Show Remove Confirmation",xOffset,&s_abOptionFlag[TOOLS_TT_SHOW_REMOVE_CONFIRMATION] );
		pxDialog->SetHelpText( "If ticked, a confirmation box will be shown before removeing a texture from the TTL" );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxOverlay->m_uID = TOOLS_OVERLAY_TT_OPTIONS;

		Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);

		s_xOptionsOverlayStatus.SetCreated(true);
	}
}

void Tools_Mode_TextureTiler::CreateExportOverlay()
{
	Asura_Overlay* pxOverlay = new Asura_Overlay( "Export:",false,s_xExportOverlayStatus.GetPtrToPosition(),s_xExportOverlayStatus.GetPtrToMinimised() );
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_String xMsg;

	Asura_Tabs_Dialog* pxTabsDialog( new Asura_Tabs_Dialog(xOffset) );

	pxOverlay->AddDialog( pxTabsDialog );

	// add options to export for each platform
	pxTabsDialog->AddTab( "Platform" );
	xOffset.y += Asura_Dialog::LargeSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_StringDisplay_Dialog( "Export For Platform:", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	const Asura_Vector_2 xTabTopLeftCorner(xOffset);

	for( int iPlatform = 0; iPlatform<TOOLS_PLATFORM_NUM_PLATFORMS; ++iPlatform )
	{		
		pxDialog = new Asura_Tickbox_Dialog( Tools_Platform::GetPlatformName( iPlatform ),xOffset,s_abExportPlatformFlags+iPlatform );

		xMsg =  "Export tiled texture as a .asr file for the ";
		xMsg += Tools_Platform::GetPlatformName( iPlatform );
		xMsg += " platform";

		pxDialog->SetHelpText( xMsg.GetString() );
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	
		// disable tickbox for unsupported platform (EXPORT_TYPE and Tools_Platform_Type enums are directly compatible)
		if( !Tools_Level_Export::IsPlatformSupported( static_cast<Tools_Level_Export::EXPORT_TYPE>(iPlatform) ) )
		{		
			pxDialog->SetEnabled( false );
		}
	}

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Export",xOffset,&Export );
	pxDialog->SetHelpText( "Export tiled texture for all selected platforms" );
	pxTabsDialog->AddDialog(pxDialog);
		
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// add option to close overlay without taking any action
	pxDialog = new Asura_Function_Dialog( "Cancel",xOffset,&CloseOverlay,TOOLS_OVERLAY_TT_EXPORT );
	pxDialog->SetHelpText( "Cancel export operation and close this overlay" );
	pxTabsDialog->AddDialog(pxDialog);

	// texture tab
	pxTabsDialog->AddTab( "Texture" );

	xOffset = xTabTopLeftCorner;

	for( u_int u(0);u<uTextureFlagIndexMax;++u )
	{
		pxDialog = new Asura_FlagTickbox_Dialog( Tools_TT_TextureFlagNames[u],xOffset,&s_iTextureFlags,Tools_TT_TextureFlag[u] );
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	// options tab
	pxTabsDialog->AddTab( "Options" );

	xOffset = xTabTopLeftCorner;

	// dds compression
	pxDialog = new Asura_Dialog( "dds compression:",xOffset );
	pxTabsDialog->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Tools_TT_AssignStringCollection( s_pxPtrToSelf->m_xExportOverlayDDSFormat,Tools_TT_DDSCompressionFormat::s_apcName,
									 Tools_TT_DDSCompressionFormat::TOOLS_TT_EXPORT_DDS_COMPRESSION_FORMAT_NUM );

	pxDialog = s_pxPtrToSelf->m_xExportOverlayDDSFormat.CreateAsuraDialog( xOffset,&ChooseExportOptions );
	pxDialog->SetHelpText("Select DDS compression format");
	pxTabsDialog->AddDialog( pxDialog );
	
	s_pxPtrToSelf->m_xExportOverlayDDSFormat.SetSelectionIndex( s_pxPtrToSelf->m_xTTL.GetDDSFormat() );

	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	// tpl alpha bits
	pxDialog = new Asura_Dialog( "TPL alpha bits:",xOffset );
	pxTabsDialog->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// disable if TPL using platforms are unsupported (EXPORT_TYPE and Tools_Platform_Type enums are directly compatible)
	if( !Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_Wii) )
	{		
		pxDialog->SetEnabled( false );
	}

	Tools_TT_AssignStringCollection( s_pxPtrToSelf->m_xExportOverlayTPLAlphaBits,Tools_TT_TPLAlphaBits::s_apcName,Tools_TT_TPLAlphaBits::TOOLS_TT_EXPORT_TPL_ALPHA_BITS_NUM );
	pxDialog = s_pxPtrToSelf->m_xExportOverlayTPLAlphaBits.CreateAsuraDialog( xOffset,&ChooseExportOptions );
	pxDialog->SetHelpText("Select TPL alpha bits (Only used by Wii)");
	pxTabsDialog->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	// disable if TPL using platforms are unsupported (EXPORT_TYPE and Tools_Platform_Type enums are directly compatible)
	if( !Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_Wii) )
	{		
		pxDialog->SetEnabled( false );
	}

	// common options
	pxDialog = new Asura_Dialog( "common:",xOffset );
	pxTabsDialog->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Compress Chunks",xOffset,&s_abOptionFlag[TOOLS_TT_COMPRESS_CHUNKS] );
	pxDialog->SetHelpText("If ticked exported chunk files are compressed");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("AutoTile",xOffset,&s_abOptionFlag[TOOLS_TT_AUTOTILE] );
	pxDialog->SetHelpText("When there are too many textures to fit into the current tiled texture additional tiled texture chunks are generated to contain the textures left out of the first. The additional chunks are appended to the same asr file as the first and given a similar name to the first with the adition of a numerical postfix. All textures that are successfully tiled and exported are removed from the current TTL." );
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Create Texture",xOffset,&s_abOptionFlag[TOOLS_TT_CREATE_TEXTURE_ON_EXPORT] );
	pxDialog->SetHelpText("If ticked then a tga texture is created with the same file name as the tiled texture asr file but with \"_texture\" appended." );
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	// select platform tab
	pxTabsDialog->SelectTab(0);

	pxOverlay->m_uID = TOOLS_OVERLAY_TT_EXPORT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_TextureTiler::CreateExportOverlayAs()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = ".asr";
	info.DefaultFilename  = ((s_pxPtrToSelf->m_xTTLChunkName.IsBlank())? s_xDefaultFileToLoad:s_pxPtrToSelf->m_xTTLChunkName).GetString();

	info.DefaultPath = "";
	info.FileFilter = "Asura Chunk Files\0*.asr;\0";
	info.WindowTitle = "Export TTL As";
	info.AllowMultipleSelect = false;

	Asura_String xFileName( Tools_FileSystem::GetSaveFilename(info) );

	if( xFileName.IsNull() || xFileName.IsBlank() )
	{
		return;
	}

	// change TTL file name
	s_pxPtrToSelf->m_xTTLChunkName = xFileName;

	CreateExportOverlay();
}

void Tools_Mode_TextureTiler::ToggleTextureOverlay()
{
	if( s_xTextureOverlayStatus.IsCreated() )
	{
		CloseOverlay( TOOLS_OVERLAY_TT_TEXTURE );
		s_xTextureOverlayStatus.SetCreated( false );
	}
	else
	{
		
		Asura_Overlay* pxOverlay = new Asura_Overlay("Texture",false,s_xTextureOverlayStatus.GetPtrToPosition(),s_xTextureOverlayStatus.GetPtrToMinimised() );
		
		Asura_Dialog*  pxDialog;
		Asura_Vector_2 xDiskOpTopLeft(1.0f,3.0f);
		Asura_Vector_2 xPropertiesTopLeft;
		Asura_Vector_2 xSubtextureTopLeft;
		Asura_Vector_2 xOffset;
		float          fColumnSeparation;

		// disk op left hand column
		xOffset = xDiskOpTopLeft;

		pxDialog = new Asura_Dialog( "Load/Save:",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xDiskOpTopLeft = xOffset;

		pxDialog = new Asura_Function_Dialog( "Save TTL",xOffset,&SaveTTL );
		pxDialog->SetHelpText("Save a text file describing the current TTL to disk");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Save TTL As",xOffset,&SaveTTLAs );
		pxDialog->SetHelpText("Save a text file describing the current TTL to disk with a specified file name");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Load TTL",xOffset,&LoadTTLFromDiskWithConfirmation );
		pxDialog->SetHelpText("Load a text file describing a TTL from disk (replaces current TTL)");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("New TTL",xOffset,&NewTTLWithConfirmation );
		pxDialog->SetHelpText("Create a new blank TTL (replaces current TTL)");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Save Texture As",xOffset,&SaveTextureAs );
		pxDialog->SetHelpText("Save a texture containing an image of the current TTL to disk");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		// properties left hand column
		pxDialog = new Asura_Dialog( "Properties:",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
		
		xPropertiesTopLeft = xOffset;

		s_pxPtrToSelf->m_xTextureOverlayTilingAlgorithm.SetCapacity(3);
		s_pxPtrToSelf->m_xTextureOverlayTilingAlgorithm.Append( "classic" );               // TOOLS_TT_ALGORITHM_CLASSIC
		s_pxPtrToSelf->m_xTextureOverlayTilingAlgorithm.Append( "first availible space" ); // TOOLS_TT_ALGORITHM_FIRSTAVAILABLESPACE
		s_pxPtrToSelf->m_xTextureOverlayTilingAlgorithm.Append( "rectangle tree" );		   // TOOLS_TT_ALGORITHM_RECTTREE
		pxDialog = s_pxPtrToSelf->m_xTextureOverlayTilingAlgorithm.CreateAsuraDialog( xOffset,&ChooseTilingAlgorithm );
		pxDialog->SetHelpText("Select texture tiling method");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = s_pxPtrToSelf->m_xTextureOverlayTTLWidth.CreateAsuraDialog( "",xOffset,0,4096,1,"%i" );
		pxDialog->SetHelpText("Set width of current tiled texture in pixels");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = s_pxPtrToSelf->m_xTextureOverlayTTLHeight.CreateAsuraDialog( "",xOffset,0,4096,1,"%i" );
		pxDialog->SetHelpText( "Set height of current tiled texture in pixels" );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		s_pxPtrToSelf->m_xTextureOverlayBPP.SetCapacity(2);
		s_pxPtrToSelf->m_xTextureOverlayBPP.Append( "32" );               
		s_pxPtrToSelf->m_xTextureOverlayBPP.Append( "8" );
		pxDialog = s_pxPtrToSelf->m_xTextureOverlayBPP.CreateAsuraDialog( xOffset,&ChooseBPP );
		pxDialog->SetHelpText( "Set bits per pixel of current tiled texture" );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("",xOffset,&s_abOptionFlag[TOOLS_TT_HAS_PADDING] );
		pxDialog->SetHelpText("If ticked, a 1 pixel wide boarder will be added to each subtexture before it is tiled");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		xSubtextureTopLeft = xOffset;

		// disk op right hand column
		pxOverlay->CalculateOverlaySize();
		fColumnSeparation = pxOverlay->m_fWidth;

		xOffset = xDiskOpTopLeft;
		xOffset.x += fColumnSeparation;

		pxDialog = new Asura_Function_Dialog( "Export",xOffset,&CreateExportOverlay );
		pxDialog->SetHelpText("Export a chunk describing the current tiled texture to disk");
		pxDialog->SetIsEnabledFunction( &IsChunkNameValid );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "ExportAs",xOffset,&CreateExportOverlayAs );
		pxDialog->SetHelpText("Export a chunk describing the current tiled texture to disk");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Save",xOffset,&SaveChunk );
		pxDialog->SetHelpText("Save the current tiled texture to disk");
		pxDialog->SetIsEnabledFunction( &IsChunkNameValid );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "SaveAs",xOffset,&SaveChunkAs );
		pxDialog->SetHelpText("Save the current tiled texture to disk");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Load Chunk",xOffset,&LoadChunkFromDiskWithConfirmation );
		pxDialog->SetHelpText("Load a chunk containing a TTL from disk (replaces current TTL)");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		xOffset.y+=Asura_Dialog::DefaultSpacing();

		// properties right hand column
		xOffset    = xPropertiesTopLeft;
		xOffset.x += fColumnSeparation;

		pxDialog = new Asura_Dialog( "Select Tiling Method",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Dialog( "Set Width",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Dialog( "Set Height",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Dialog( "Set BPP",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Dialog( "Add Padding",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// full width 
		xOffset = xSubtextureTopLeft;

		pxDialog = new Asura_Dialog( "Operations:",xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		xSubtextureTopLeft = xOffset;

		pxDialog = new Asura_Function_Dialog("Add Texture From Disk",xOffset,&AddTextureFromDisk );
		pxDialog->SetHelpText("Pick a texture from disk and add it to the current TTL");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Dialog("Textures in TTL:",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::SmallSpacing();
		
		Tools_TT_AssignStringCollection( s_pxPtrToSelf->m_xTextureOverlayTTLSubtextureList,s_pxPtrToSelf->m_xTTL );
		pxDialog = s_pxPtrToSelf->m_xTextureOverlayTTLSubtextureList.CreateAsuraDialog( xOffset,&RemFromTextureOverlay );
		pxOverlay->AddDialog( pxDialog );
		
		pxOverlay->CalculateOverlaySize();
		xOffset.y = pxOverlay->m_fHeight;
		
		// copy to clipbord button
		pxDialog = new Asura_Function_Dialog( "Copy Tile Hash String To Clipboard",xOffset,&CopySelectedTileHashStringToClipboard );
		pxDialog->SetHelpText("copies the string used to generate the hash id of the selected tile to the windows clipboard");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxOverlay->m_uID = TOOLS_OVERLAY_TT_TEXTURE;

		Asura_Overlay_System::SubmitNewOverlay( pxOverlay,false );

		s_xTextureOverlayStatus.SetCreated( true );
	}
}

void Tools_Mode_TextureTiler::RemFromTextureOverlay()
{
	if( s_pxPtrToSelf->IsSelectedTextureValid() )
	{
		s_pxPtrToSelf->m_uTTLSelectedTextureIndex = s_pxPtrToSelf->m_xTextureOverlayTTLSubtextureList.GetSelectionIndex();		
		RemoveSelectedTextureFromTTLWithConfirmation();
	}
}

void Tools_Mode_TextureTiler::ChooseBPP()
{
	const Tools_TT_TiledTextureList::TOOLS_TT_BYTESPERPIXEL eBPP( static_cast<Tools_TT_TiledTextureList::TOOLS_TT_BYTESPERPIXEL>(s_pxPtrToSelf->m_xTextureOverlayBPP.GetSelectionIndex() ) );
	u_int uBPP( s_pxPtrToSelf->m_xTTL.GetBytesPP() );
	
	if( eBPP == Tools_TT_TiledTextureList::TOOLS_TT_BYTESPERPIXEL_1 )
	{
		uBPP = 1;
	}
	else if( eBPP == Tools_TT_TiledTextureList::TOOLS_TT_BYTESPERPIXEL_4 )
	{
		uBPP = 4;
	}

	s_pxPtrToSelf->m_xTTL.SetBytesPP( uBPP );

	// retile TTL using newly selected Algorithm
	s_pxPtrToSelf->UpdateTTL();
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();
}


void Tools_Mode_TextureTiler::ChooseTilingAlgorithm()
{
	const TOOLS_TT_ALGORITHM_TYPE eAlgorithm( static_cast<TOOLS_TT_ALGORITHM_TYPE>( s_pxPtrToSelf->m_xTextureOverlayTilingAlgorithm.GetSelectionIndex() ) );

	if( eAlgorithm < TOOLS_TT_NUM_ALGORITHM )
	{
		s_pxPtrToSelf->m_xTTL.SetTilingAlgorithm( eAlgorithm );

		// retile TTL using newly selected Algorithm
		s_pxPtrToSelf->UpdateTTL();
		s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();
	}
}

void Tools_Mode_TextureTiler::ChooseExportOptions()
{
	s_pxPtrToSelf->m_xTTL.SetDDSFormat( static_cast<Tools_TT_DDSCompressionFormat::TOOLS_TT_EXPORT_DDS_COMPRESSION_FORMAT>( s_pxPtrToSelf->m_xExportOverlayDDSFormat.GetSelectionIndex() ) );
	s_pxPtrToSelf->m_xTTL.SetPSPPixelFormat( Tools_TT_PSPPixelFormat::GetValue(s_pxPtrToSelf->m_xExportOverlayPSPPixelFormat.GetSelectionIndex()) );
	s_pxPtrToSelf->m_xTTL.SetTPLAlphaBits( static_cast<Tools_TT_TPLAlphaBits::TOOLS_TT_EXPORT_TPL_ALPHA_BITS>( s_pxPtrToSelf->m_xExportOverlayTPLAlphaBits.GetSelectionIndex() ) );
}

void Tools_Mode_TextureTiler::Update()
{
	// if viewport or ui compoents have been resized then resise UI	
	if( (m_iLastKnownViewportWidth  != Asura_Viewport::GetWidth()) ||
		(m_iLastKnownViewportHeight != Asura_Viewport::GetHeight()) )
	{
		ResizeUI();
	}

	UpdatePreviewOverlay();
	UpdateBPP();

	// read and act on user input
	ProcessUserInput();

	Tools_SaveAsura::SetTextureTilerModified( IsModified() );
}

void Tools_Mode_TextureTiler::RenderWorld_PostToneMap()
{
	const float fViewportWidth( static_cast<float>(Asura_Viewport::GetWidth()) );
	const float fViewportHeight( static_cast<float>(Asura_Viewport::GetHeight()) );

	// first draw a big black poly over everything that has taken place so far...
	Asura_Render::Overlay.SetRectXY( 0,0,fViewportWidth,fViewportHeight );
	Asura_Render::Overlay.SetRectUV( 0,0,1,1 );
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex     = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour           = m_uOverlayColour;
	Asura_Render::Overlay.Render();

	Asura_Render::FlushDisplay();
	
	RenderCurrentTTL();
}

void Tools_Mode_TextureTiler::RenderUI()
{}

bool Tools_Mode_TextureTiler::IsModified()
{
	return m_xTTL.IsModified();
}

void Tools_Mode_TextureTiler::QuickSave()
{}

void Tools_Mode_TextureTiler::PreSave()
{
	SaveTTL();
	Tools_SaveAsura::SetTextureTilerModified( false );
}

void Tools_Mode_TextureTiler::PostSave()
{}

void Tools_Mode_TextureTiler::ProcessUserInput()
{
	// ignore user input if menu is active or remove confirmation box is open
	if( Asura_Menubar_System::IsUsingMenus() || m_xConfirmationOverlay.IsActive() )
	{
		return;
	}

	// process keybord input
	bool bSelectedTextureAddedToTTL(false);
	bool bSelectedTextureRemovedFromTTL(false); 

	if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER) )
	{
		bSelectedTextureAddedToTTL = true;
	}

	if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_BACKSPACE) ||
		Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_DEL) ||
		Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPADDEL) )
	{
		bSelectedTextureRemovedFromTTL = true;
	}	

	if( Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) )
	{
		if( Asura_UserInput::DebouncedKeyPressed('s') ) // 'Ctrl + s' saves TTL
		{
			SaveTTL();
		}

		if( Asura_UserInput::DebouncedKeyPressed('e') ) // 'Ctrl + e' opens export dialog
		{
			CreateExportOverlay();
		}

		if( Asura_UserInput::DebouncedKeyPressed('t') ) // 'Ctrl + t' saves texture as 
		{
			SaveTextureAs();
		}


		if( Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) )
		{
			if( Asura_UserInput::DebouncedKeyPressed('s') ) // 'Ctrl + shift + s' saves TTL As
			{
				SaveTTLAs();
			}
		
			if( Asura_UserInput::DebouncedKeyPressed('e') ) // 'Ctrl + shift + e' export TTL As
			{
				CreateExportOverlayAs();
			}
		
		}
	}
	
	//F-Keys change alpha mode
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F1))
	{
		m_eAlphaMode = TOOLS_TT_ALPHA_OPAQUE;
	}
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F2))
	{
		m_eAlphaMode = TOOLS_TT_ALPHA_TRANSPARENT_BLACK;
	}
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F3))
	{
		m_eAlphaMode = TOOLS_TT_ALPHA_TRANSPARENT_MAGENTA;
	}

	// if mouse over an overlay look for changes in overlays
	if( Asura_Overlay_System::CursorIsOverAnOverlay() )
	{
		if( m_xTextureOverlayTTLSubtextureList.GetSelectionIndex() < static_cast<int>(m_xTTL.Size()) )
		{
			m_uTTLSelectedTextureIndex = m_xTextureOverlayTTLSubtextureList.GetSelectionIndex();
		}
		else
		{
			m_uTTLSelectedTextureIndex = m_xTTL.Size();
		}

		// if TLL properties have been changed in texture overlay update TTL 
		if( (m_xTextureOverlayTTLWidth.GetValue()	!= m_xTTL.GetWidth())   || 
			(m_xTextureOverlayTTLHeight.GetValue()	!= m_xTTL.GetHeight())  ||
			(s_abOptionFlag[TOOLS_TT_HAS_PADDING]	!= m_xTTL.HasPadding()) )
		{
			if( s_abOptionFlag[TOOLS_TT_FIX_TTL_DIMENSIONS_TO_POWER_OF_TWO] )
			{
				// width
				if( m_xTextureOverlayTTLWidth.GetValue() > m_xTTL.GetWidth() )
				{
					m_xTextureOverlayTTLWidth.SetValue( Tools_TT_FindNextLargerPowerOfTwo(m_xTextureOverlayTTLWidth.GetValue()) ); 
				}
				else
				{
					m_xTextureOverlayTTLWidth.SetValue( Tools_TT_FindNextSmallerPowerOfTwo(m_xTextureOverlayTTLWidth.GetValue()) ); 
				}

				// height
				if( m_xTextureOverlayTTLHeight.GetValue() > m_xTTL.GetHeight() )
				{
					m_xTextureOverlayTTLHeight.SetValue( Tools_TT_FindNextLargerPowerOfTwo(m_xTextureOverlayTTLHeight.GetValue()) );
				}
				else
				{
					m_xTextureOverlayTTLHeight.SetValue( Tools_TT_FindNextSmallerPowerOfTwo(m_xTextureOverlayTTLHeight.GetValue()) );
				}
			}
			
			m_xTTL.SetWidth( m_xTextureOverlayTTLWidth.GetValue() );
			m_xTTL.SetHeight( m_xTextureOverlayTTLHeight.GetValue() );
		
			m_xTTL.SetPadding( s_abOptionFlag[TOOLS_TT_HAS_PADDING] );

			UpdateTTL();
			SynchronizeUIWithCurrentTTL();
		}

		m_xTTL.SetDataNamePrefix( s_abOptionFlag[TOOLS_TT_PREFIX_IMAGE_DATA_NAME] );

	}
	// if mouse is not over an overlay process mouse input on TTL
	else
	{
		// process mouse input
		int iCursorX;
		int iCursorY;
		Asura_Cursor::GetScreenSpaceCoords( iCursorX,iCursorY );
		
		// select texture under mouse pointer
		if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) )
		{
			const u_int uTTLTextureIndex( FindIndexOfTTLSubtextureUnderCursor( iCursorX,iCursorY ) );
			
			if( uTTLTextureIndex < m_xTTL.Size() )
			{
				m_uTTLSelectedTextureIndex = uTTLTextureIndex;
			}
			else
			{
				m_uTTLSelectedTextureIndex = m_xTTL.Size();
			}
		}
		else if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2) )
		{
			const u_int uTTLTextureIndex( FindIndexOfTTLSubtextureUnderCursor( iCursorX,iCursorY ) );

			if( !(uTTLTextureIndex < m_xTTL.Size()) )
			{
				m_uTTLSelectedTextureIndex = m_xTTL.Size();
			}
			else if( uTTLTextureIndex == m_uTTLSelectedTextureIndex )
			{
				bSelectedTextureRemovedFromTTL = true;
			}
			else
			{
				m_uTTLSelectedTextureIndex = uTTLTextureIndex;
			}
		}
	}

	// remove selected texture from TLL if required
	if(bSelectedTextureRemovedFromTTL && IsSelectedTextureValid() )
	{
		RemoveSelectedTextureFromTTLWithConfirmation();
	}

	// update textures overlay to reflect change of selected texture
	if( IsSelectedTextureValid() )
	{
		m_xTextureOverlayTTLSubtextureList.SetSelectionIndex( m_uTTLSelectedTextureIndex );
	}
	
}

void Tools_Mode_TextureTiler::SetChunkNameFromFilePicker()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = ".asr";
	info.DefaultFilename = "";

	info.DefaultPath = "Graphics\\";
	info.FileFilter = "Asura Chunk Files\0*.asr;\0";
	info.WindowTitle = "Export As";
	info.AllowMultipleSelect = false;

	Asura_String xFileName( Tools_FileSystem::GetSaveFilename(info) );

	if( xFileName.IsBlank() )
	{
		return;
	}

	// change TTL file name
	s_pxPtrToSelf->m_xTTLChunkName = xFileName;
}

const u_int Tools_Mode_TextureTiler::FindIndexOfTTLSubtextureUnderCursor( const int iCursorX, const int iCursorY ) const
{
	// convert screen co-ords to texture u,v 
	const Asura_Vector_2 xCursorUV( (iCursorX - 2.0f*m_xCurrentTTL.GetPadding()) / m_xCurrentTTL.GetInnerWidth(), (iCursorY - 2.0f*m_xCurrentTTL.GetPadding()) / m_xCurrentTTL.GetInnerHeight() ); 
	
	if( (xCursorUV.x > 1.0f) || (xCursorUV.y > 1.0f) )
	{
		return m_xTTL.Size();
	}

	// test against each texture in TTL
	u_int u(0);

	for( u = 0;u<m_xTTL.Size();++u )
	{
		if( Tools_TT_PointInRectangle( xCursorUV.x,xCursorUV.y,m_xTTL[u].GetUVRectangle() ) )
		{
			break;
		}
	}

	return u;
}

void Tools_Mode_TextureTiler::RenderCurrentTTL() const
{
	const ASURA_TRANSLUCENCY_MODE eTranslucency( (m_eAlphaMode == TOOLS_TT_ALPHA_OPAQUE)? ASURA_TRANSLUCENCY_OFF:ASURA_TRANSLUCENCY_NORMAL );
	const int iBackgroundColour( (m_eAlphaMode == TOOLS_TT_ALPHA_TRANSPARENT_MAGENTA)? Tools_TT_Colour::Magenta : Tools_TT_Colour::Black );

	Tools_TT_RenderUIBorder( m_xCurrentTTL );
	Tools_TT_RenderTTL( m_xCurrentTTL.GetInnerRectangle(),m_xTTL,iBackgroundColour,eTranslucency );

	if( IsSelectedTextureValid() ) 
	{
		Tools_TT_RenderTTLSubTextureHighlight( m_xCurrentTTL.GetInnerRectangle(),m_xTTL[m_uTTLSelectedTextureIndex],m_xCurrentTTL.GetBorderColour() );//.GetTextureFromID(m_pxTextureInfoCollection[m_uSelectedTextureIndex].iTextureID),m_xCurrentTTL.GetBorderColour() );
	}
}

void Tools_Mode_TextureTiler::UpdatePreviewOverlay( const bool bForceUpdate )
{
	// do nothing if preview does no exist, is minimised or is already displying selected texture
	if( (!bForceUpdate) &&
		((!s_xPreviewOverlayStatus.IsCreated() && !s_xPreviewOverlayStatus.IsMinimised()) || ((s_uPreviewDisplayedTextureIndex == m_uTTLSelectedTextureIndex) ) && (m_ePreviewDisplayedAlphaMode == m_eAlphaMode)) )		
	{
		return;
	}

	// close stale overlay
	CloseOverlay( TOOLS_OVERLAY_TT_PREVIEW );
	
	// create new overlay
	Asura_Overlay* pxOverlay = new Asura_Overlay("Preview",false,s_xPreviewOverlayStatus.GetPtrToPosition(),s_xPreviewOverlayStatus.GetPtrToMinimised() );
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	s_uPreviewDisplayedTextureIndex = m_uTTLSelectedTextureIndex;

	if( !IsSelectedTextureValid() )
	{
		pxDialog = new Asura_Dialog( "No valid texture selected",xOffset );
		pxDialog->SetHelpText("Select a valid texture to see a preview");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	else
	{
		const Tools_TT_Tile xTile( m_xTTL[m_uTTLSelectedTextureIndex] );
		const int iTextureID( xTile.GetTextureID() );
		
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		
		// display texture
		float fImageWidth;
		float fImageHeight;

		if( xTile.GetWidth() > xTile.GetHeight() )
		{
			fImageWidth  = m_fPreviewOverlayWidth;
			fImageHeight = (m_fPreviewOverlayWidth/static_cast<float>(xTile.GetWidth())) * xTile.GetHeight();
	    }
		else
		{
			fImageWidth  = (m_fPreviewOverlayWidth/static_cast<float>(xTile.GetHeight())) * xTile.GetWidth();
			fImageHeight = m_fPreviewOverlayWidth;
		}


		const bool bTranslucency( (m_eAlphaMode == TOOLS_TT_ALPHA_OPAQUE)? false:true );
		const int  iBackgroundColour( (m_eAlphaMode == TOOLS_TT_ALPHA_TRANSPARENT_MAGENTA)? Tools_TT_Colour::Magenta : Tools_TT_Colour::Black );

		m_ePreviewDisplayedAlphaMode = m_eAlphaMode;

		Asura_Image_Dialog* pxImageDialog( new Asura_Image_Dialog( xOffset,iTextureID,fImageWidth,fImageHeight ) );
		pxImageDialog->SetBackground( iBackgroundColour,2 );
		pxImageDialog->SetTextureUV( xTile.GetImageUV() );
		pxImageDialog->SetTranslucent( bTranslucency );
		pxOverlay->AddDialog(pxImageDialog);
		pxOverlay->CalculateOverlaySize();
		xOffset.y = pxOverlay->m_fHeight;

		// display name
		pxDialog = new Asura_Dialog( xTile.GetName().GetString(),xOffset );
		pxDialog->SetHelpText("Texture file name");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// display width, height and u,v
		Asura_String xMsg;
		Asura_String xHelpText("Texture Width x Height in pixels");
		Asura_String xTmp;

		xTmp.MakeBlank();
		Asura_StringUtil::ToString( xTile.GetWidth(),xTmp );
		xMsg += xTmp;
		
		xMsg += "x";
		
		xTmp.MakeBlank();
		Asura_StringUtil::ToString( xTile.GetHeight(),xTmp );
		xMsg += xTmp;

		xMsg += " uv (";

		xTmp.MakeBlank();
		Asura_StringUtil::ToString( xTile.GetUVRectangle().MinX,xTmp );
		xMsg += xTmp;
		
		xMsg += ",";
		
		xTmp.MakeBlank();
		Asura_StringUtil::ToString( xTile.GetUVRectangle().MinY,xTmp );
		xMsg += xTmp;

		xMsg += ") to (";

		Asura_StringUtil::ToString( xTile.GetUVRectangle().MaxX,xTmp );
		xMsg += xTmp;
		
		xMsg += ",";
		
		xTmp.MakeBlank();
		Asura_StringUtil::ToString( xTile.GetUVRectangle().MaxY,xTmp );
		xMsg += xTmp;

		xMsg += ")";

		xHelpText += " and uv coordinates in current TTL";
		
		pxDialog = new Asura_Dialog(xMsg.GetString(),xOffset );
		pxDialog->SetHelpText(xHelpText.GetString());
		pxOverlay->AddDialog(pxDialog);
	}

	pxOverlay->m_uID = TOOLS_OVERLAY_TT_PREVIEW;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);	
}

void Tools_Mode_TextureTiler::TogglePreviewOverlay()
{
	if( s_xPreviewOverlayStatus.IsCreated() )
	{
		CloseOverlay( TOOLS_OVERLAY_TT_PREVIEW );
		s_xPreviewOverlayStatus.SetCreated( false );
	}
	else
	{
		s_xPreviewOverlayStatus.SetCreated( true );
		s_pxPtrToSelf->UpdatePreviewOverlay(true);
	}
}


bool Tools_Mode_TextureTiler::TextureOverlayCreated()
{
	return s_xTextureOverlayStatus.IsCreated();
}

bool Tools_Mode_TextureTiler::PreviewOverlayCreated()
{
	return s_xPreviewOverlayStatus.IsCreated();
}

bool Tools_Mode_TextureTiler::OptionsOverlayCreated()
{
	return s_xOptionsOverlayStatus.IsCreated();
}

void Tools_Mode_TextureTiler::Export()
{
	if( s_pxPtrToSelf->m_xTTLChunkName.IsNull() || s_pxPtrToSelf->m_xTTLChunkName.IsBlank() )
	{
		ExportAs();
	}
	else
	{
		s_pxPtrToSelf->Export_Implimentation();
	}
}

void Tools_Mode_TextureTiler::ExportAs()
{
	s_pxPtrToSelf->SetChunkNameFromFilePicker();
	s_pxPtrToSelf->Export_Implimentation();
}

void Tools_Mode_TextureTiler::Export_Implimentation()
{
	// udate texture export flags
	m_xTTL.SetTextureFlags( s_iTextureFlags );

	Export_ImplimentationForPlatforms();
	Export_ImplimentationForTools();

	// close export overlay after use
	CloseOverlay(TOOLS_OVERLAY_TT_EXPORT);

	// update ttl and user interface
	s_pxPtrToSelf->UpdateTTL();
	s_pxPtrToSelf->UpdatePreviewOverlay(true);
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();

	s_pxPtrToSelf->m_xTTL.SetModified(false);
}

void Tools_Mode_TextureTiler::Export_Implimentation( const int iPlatformType,const Asura_String& xExportPath )
{
	bool bSuccess(false);

	// create export path if nessary
	Tools_FileSystem::EnsureDirectoryExists( xExportPath.GetString() );

	// call appropriate chunk writing method
	if( s_abOptionFlag[TOOLS_TT_AUTOTILE] )
	{
		bSuccess = s_pxPtrToSelf->WriteMultipleChunks( xExportPath,static_cast<Tools_Platform_Type>(iPlatformType) );
	}
	else
	{
		bSuccess = s_pxPtrToSelf->WriteChunk( xExportPath,static_cast<Tools_Platform_Type>(iPlatformType) );
	}

	// report any failiure to user
	if( !bSuccess )
	{
		Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::Red,"ERROR %s export of file %s FAILED",
				Tools_Platform::GetPlatformName( iPlatformType ),s_pxPtrToSelf->m_xTTLChunkName.GetString() );
	}
	else
	{
		Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"Successfully exported '%s'",xExportPath.GetString() );
	}
}


void Tools_Mode_TextureTiler::Export_ImplimentationForPlatforms()
{
	for( int iPlatform(0); iPlatform<TOOLS_PLATFORM_NUM_PLATFORMS; ++iPlatform )
	{		
		if( s_abExportPlatformFlags[iPlatform] )
		{
			Export_Implimentation( iPlatform,CreateExportPathForPlatform(iPlatform) );
		}
	}
}

void Tools_Mode_TextureTiler::Export_ImplimentationForTools()
{
	if( s_abOptionFlag[TOOLS_TT_RELOAD_CHUNK_AFTER_EXPORT] )
	{
		// export tools version - should be a better way to get tiles into tile management - NDB 100308
		Export_Implimentation( TOOLS_PLATFORM_PC,s_pxPtrToSelf->m_xTTLChunkName );
		ReloadChunkAfterExport();
	}

	// write texture file generated from tiled texture - currently used by GUI menu system
	if( s_abOptionFlag[TOOLS_TT_CREATE_TEXTURE_ON_EXPORT] )
	{
		const Asura_String xTextureFileName( Tools_TT_CreateTextureFileName( s_pxPtrToSelf->m_xTTLChunkName ) );

		s_pxPtrToSelf->m_xTTL.WriteTexture( xTextureFileName );

		// (re)load texture file
		const char* szStripedPath( strstr(xTextureFileName.GetString(),"\\graphics" ) );

		if( szStripedPath )
		{
			szStripedPath += strlen( "\\graphics" );

			const int iTextureIndex( Asura_Texture_Management::GetTextureIndex(szStripedPath) );

			if( iTextureIndex == ASURA_TEXTURE_NONE )
			{
				Asura_Texture_Management::AddTexture( szStripedPath );
				Asura_Texture_Management::LoadTextures();
			}
			else
			{
				Asura_Texture_Management::ReloadTexture( iTextureIndex );
			}
		}
	}

	
	// set default file name to last file saved - this means it will get loaded if user leaves then re-enters TT mode
	SetDefaultFileToLoad( s_pxPtrToSelf->m_xTTLChunkName );
}

const Asura_String Tools_Mode_TextureTiler::CreateExportPathForPlatform( const int iPlatformType )
{
	Asura_String xPath( s_pxPtrToSelf->m_xTTLChunkName );

	Tools_TT_ReplaceFileExtension( xPath,".asr" );

	return Tools_TT_CreateExportPathForPlatform( xPath,iPlatformType );
}

void Tools_Mode_TextureTiler::ReloadChunkAfterExport()
{
	// destroy any existing tiles with the same hash as the new tiles
	s_pxPtrToSelf->DeleteTilesFromManager();
	
	// destroy texture from previous chunk in texture management
	if( !( s_pxPtrToSelf->m_xTTLChunkName.IsNull() || s_pxPtrToSelf->m_xTTLChunkName.IsBlank() ) )
	{
		Asura_String xTextureName;
		Tools_TT_CreateResourceChunkName( xTextureName,s_pxPtrToSelf->m_xTTLChunkName,TOOLS_PLATFORM_PC,s_pxPtrToSelf->m_xTTL.HasDataNamePrefix() );
		Asura_Texture_Management::DestroyTexture( Asura_Texture_Management::GetIndexFromHashID(xTextureName.GetHash_IgnoreExtension()) );
	}

	// load new chunk file
	Asura_ChunkLoading_System::LoadAsuraFile( s_pxPtrToSelf->m_xTTLChunkName.GetString() );
	Asura_TiledTextureManagement::SetTextureHandlesFromHash();
}

const bool Tools_Mode_TextureTiler::WriteChunk( const Asura_String& xOutputFileName,const Tools_Platform_Type eTargetPlatform )
{
	return m_xTTL.WriteChunk( xOutputFileName,eTargetPlatform,s_abOptionFlag[TOOLS_TT_COMPRESS_CHUNKS] );
}

const bool Tools_Mode_TextureTiler::WriteMultipleChunks( const Asura_String& xOutputFileName,const Tools_Platform_Type eTargetPlatform )
{
	return m_xTTL.WriteMultipleChunks( xOutputFileName,eTargetPlatform,s_abOptionFlag[TOOLS_TT_COMPRESS_CHUNKS] );
}

void Tools_Mode_TextureTiler::CloseOverlay( int iOverlayID )
{
	if( Asura_Overlay_System::DoesOverlayExist( iOverlayID ) )
	{
		Asura_Overlay_System::CloseOverlays( iOverlayID );
	}
}

void Tools_Mode_TextureTiler::SaveChunk()
{
	if( !s_pxPtrToSelf->m_xTTLChunkName.IsBlank() )
	{
		s_pxPtrToSelf->Export_ImplimentationForTools();

		// update ttl and user interface
		s_pxPtrToSelf->UpdateTTL();
		s_pxPtrToSelf->UpdatePreviewOverlay(true);
		s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();

		s_pxPtrToSelf->m_xTTL.SetModified(false);	
	}
}

void Tools_Mode_TextureTiler::SaveChunkAs()
{
	s_pxPtrToSelf->SetChunkNameFromFilePicker();
	SaveChunk();
}

void Tools_Mode_TextureTiler::LoadChunkFromDisk()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	Asura_String xDefaultFileName( (s_pxPtrToSelf->m_xTTLChunkName.IsBlank())? s_xDefaultFileToLoad:s_pxPtrToSelf->m_xTTLChunkName );
	Tools_TT_ReplaceFileExtension( xDefaultFileName,".asr" );
	info.DefaultFilename  = xDefaultFileName.GetString();

	info.DefaultExtension = ".asr";
	info.DefaultPath = "Graphics\\";
	info.WindowTitle = "Load Chunk File";
	info.AllowMultipleSelect = false;	
	info.FileFilter = "Asura Chunk Files\0*.asr;\0";

	Asura_String xFileName( Tools_FileSystem::GetLoadFilename(info) );

	// if filename is blank assume file picker was canceled/closed and do nothing - NDB
	if( xFileName.IsBlank() )
	{
		return;
	}
	
	s_pxPtrToSelf->m_xTTLChunkName = xFileName;

	s_pxPtrToSelf->LoadChunkFromDisk_Implimentation(xFileName);
}

void Tools_Mode_TextureTiler::LoadChunkFromDisk_Implimentation( const Asura_String& xFileName, const bool bExtractTextures /* = false */ )
{
	// check that this is indeed a TTL chunk
	if( Tools_AsuraFileContents::GetAsuraFileType( xFileName.GetString() ) != AsuraFile_TextureTiler )
	{
		Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::Red,"\"%s\" has not been recognised as a texture tiler file",xFileName.GetString() );	
		Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::Red,"(This means that it either contains no TT chunks or some thing other than TT chunks)" );

		return;
	}

	// assign TTL member data  
	TOOLS_TT_ERROR_CODE eError( s_pxPtrToSelf->m_xTTL.ReadChunk( xFileName,bExtractTextures ) );	
	
	if( eError == TOOLS_TT_FILE_NOT_FOUND )
	{
		s_pxPtrToSelf->m_xConfirmationOverlay.Create( "Some texture files are missing. Do you want to extract them from the chunk?",&LoadChunkAndExtractTextures );
	}

	// record newly loaded chunk IDs
	for( u_int u(0);u<s_pxPtrToSelf->m_xTTL.Size();++u )
	{
		s_pxPtrToSelf->m_xLoadedTextureCollection += s_pxPtrToSelf->m_xTTL[u].GetTextureID();
	}

	Asura_Texture_Management::LoadTextures();

	// update UI
	s_pxPtrToSelf->UpdateTTL();
	s_pxPtrToSelf->UpdatePreviewOverlay(true);
	s_pxPtrToSelf->SynchronizeUIWithCurrentTTL();

	s_pxPtrToSelf->m_xTTL.SetModified(false);

	SetDefaultFileToLoad( xFileName );
}

void Tools_Mode_TextureTiler::DeleteTilesFromManager()
{
	const char* pszHashString(NULL);

	for( u_int u(0);u<m_xTTL.Size();++u )
	{
		pszHashString = Tools_FileSystem::StripPath( m_xTTL[u].GetName().GetString() );

		if(pszHashString != NULL)
		{
			Asura_TiledTextureManagement::DeleteTile( Asura_GetHashID(pszHashString) );
		}
	}
}

void Tools_Mode_TextureTiler::LoadChunkFromDiskWithConfirmation()
{
	if( s_pxPtrToSelf->m_xTTL.IsModified() )
	{
		s_pxPtrToSelf->m_xConfirmationOverlay.Create( "Save TTL Before Loading From Chunk?",&SaveTTL,&LoadChunkFromDisk );	
	}
	else
	{
		LoadChunkFromDisk();
	}
}

void Tools_Mode_TextureTiler::LoadChunkAndExtractTextures()
{
	s_pxPtrToSelf->LoadChunkFromDisk_Implimentation( s_xDefaultFileToLoad,true );
}

void Tools_Mode_TextureTiler::UpdateTTL()
{
	// retile TTL
	const u_int uTiledTextureCount( m_xTTL.Regenerate() );

	// report any untiled textures to user (unless the autotile option is selected)
	if( uTiledTextureCount < m_xTTL.Size() && !s_abOptionFlag[TOOLS_TT_AUTOTILE] )
	{
		if( (m_xTTL.Size() - uTiledTextureCount) < 2 )
		{
			Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"WARNING The following texture was NOT tiled:" );
		}
		else
		{
			Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"WARNING The following %u textures were NOT tiled:",m_xTTL.Size() - uTiledTextureCount );
		}

		for( u_int u(0);u<m_xTTL.Size();++u )
		{
			if( !( m_xTTL[u].IsTiled()) )
			{
				Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"%s NOT tiled",m_xTTL[u].GetName().GetString() );
			}
		}

		Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"Try increasing the size of the TTL or changing tiling algorithm" );
	}
}

void Tools_Mode_TextureTiler::UpdateBPP()
{	
	const u_int uBPP( s_pxPtrToSelf->m_xTTL.GetBytesPP() );
	
	if( uBPP == 1 )
	{
		m_xTextureOverlayBPP.SetSelectionIndex( static_cast<int>(Tools_TT_TiledTextureList::TOOLS_TT_BYTESPERPIXEL_1) );
	}
	else if( uBPP == 4 )
	{
		m_xTextureOverlayBPP.SetSelectionIndex( static_cast<int>(Tools_TT_TiledTextureList::TOOLS_TT_BYTESPERPIXEL_4) );
	}
}

void Tools_Mode_TextureTiler::SynchronizeUIWithCurrentTTL()
{
	// update properties
	m_xTextureOverlayTTLWidth.SetValue( m_xTTL.GetWidth() );
	m_xTextureOverlayTTLHeight.SetValue( m_xTTL.GetHeight() );
	s_abOptionFlag[TOOLS_TT_HAS_PADDING] = m_xTTL.HasPadding();
	
	// update tiling algorithm list
	m_xTextureOverlayTilingAlgorithm.SetSelectionIndex( m_xTTL.GetTilingAlgorithm() );
	
	// update tile list
	Tools_TT_AssignStringCollection( m_xTextureOverlayTTLSubtextureList,m_xTTL );
	m_xTextureOverlayTTLSubtextureList.UpdateDialog();

	// update preview overlay
	UpdatePreviewOverlay();

	// update export options
	m_xExportOverlayDDSFormat.SetSelectionIndex( static_cast<int>(m_xTTL.GetDDSFormat()) );      
	m_xExportOverlayPSPPixelFormat.SetSelectionIndex( (m_xTTL.GetPSPPixelFormat() == 4)?4:8 ); 
	m_xExportOverlayTPLAlphaBits.SetSelectionIndex( static_cast<int>(m_xTTL.GetTPLAlphaBits()) );   

	// resize TTL display
	ResizeUI();
}

const u_int Tools_Mode_TextureTiler::GetTextureIndexFromID( const int iTextureID ) const
{
	u_int u(0);

	for( ;u<m_xTTL.Size();++u )
	{
		if( m_xTTL[u].GetTextureID() == iTextureID )
		{
			break;
		}
	}

	return u;
}

void Tools_Mode_TextureTiler::CopySelectedTileHashStringToClipboard()
{
	if( !s_pxPtrToSelf->IsSelectedTextureValid() )
	{
		return;
	}

	// strip preceeding path
	const char* pszHashString( Tools_FileSystem::StripPath( s_pxPtrToSelf->m_xTTL[s_pxPtrToSelf->m_uTTLSelectedTextureIndex].GetName().GetString() ) );

	if( pszHashString != NULL ) 
	{
		Tools_TT_CopyToClipboard( pszHashString );
	}
}


void Tools_Mode_TextureTiler::ResetExportFlags()
{
	for( int iPlatform(0); iPlatform<TOOLS_PLATFORM_NUM_PLATFORMS; ++iPlatform )
	{		
		s_abExportPlatformFlags[iPlatform] = false;
	}
}


void Tools_TT_CopyToClipboard( const Asura_String& xString )
{
	if( OpenClipboard(NULL) )
	{
		const u_int uLength( xString.GetLength() + 2 );

		HGLOBAL hMem = GlobalAlloc( GMEM_MOVEABLE, uLength+1 );
		
		if( hMem )
		{
			char* pcDest = reinterpret_cast<char*>( GlobalLock(hMem) );
			*pcDest = 0;

			//copy the string into the clipboard memory
			strcpy(pcDest,xString.GetString());
			pcDest += xString.GetLength();

			//add newline
			strcpy(pcDest,"\r\n");
			pcDest +=2;
	
			EmptyClipboard();
			SetClipboardData( CF_TEXT, hMem );

			Tools_Timed_Message_System::AddMessage( Tools_TT_Colour::White,"%s added to windows clipboard",xString.GetString() );
		}

		CloseClipboard();
	}
}

