//
// Tools_PickTexture.cpp
//
// Created by Nick Brett - 12/10/2007 11:04:08
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "TextureTiler/Tools_PickTiledTexture.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Windows.h"
#include "Interface/Asura_Cursor.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"


// Tools
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "TextureTiler/Tools_TextureTiler_Export.h" 
#include "TextureTiler/Tools_TextureTiler_Render.h" 


// <-- class Tools_PickTiledTexture -->
Tools_PickTiledTexture* Tools_PickTiledTexture::s_pxPtrToSelf(NULL);

Tools_PickTiledTexture::Tools_PickTiledTexture()
{
	Asura_Assert( s_pxPtrToSelf == NULL,"Tools_PickTiledTexture already alocated" );

	if( s_pxPtrToSelf == NULL )
	{
		s_pxPtrToSelf = this;
	}
}

Tools_PickTiledTexture::~Tools_PickTiledTexture()
{
	Asura_Assert( s_pxPtrToSelf == this,"Tools_PickTiledTexture static 'this' pointer corrupted" );

	if( s_pxPtrToSelf == this )
	{
		s_pxPtrToSelf = NULL;
	}
}

void Tools_PickTiledTexture::Initialise()
{
	m_puThumbnailList = NULL;
	m_iThumbnailListSize = 0;
	m_iSelectedThumbnailRow = 0;
	m_iSelectedThumbnailCol = 0;
	
	m_fOptimalThumbnailSizePixels = 128;
	m_fViewportWidth  = 0.0f;
	m_fViewportHeight = 0.0f;
	m_bMouseFocus     = false;

	m_iThumbnailFirstRowDisplayed = 0;
	m_iThumbnailColCount = 0;

	m_eAlphaMode = PICKTILE_TRANSPARENT_BLACK;
}

void Tools_PickTiledTexture::Shutdown()
{
	ClearThumbnailList();
}

const int Tools_PickTiledTexture::PickTiledTexture( const char* szMessage,const int iCurrentSelection )
{
	return s_pxPtrToSelf->PickTiledTexture_Implimentation( szMessage,iCurrentSelection );
}

const int Tools_PickTiledTexture::PickTiledTextureFromDisk()
{
	s_pxPtrToSelf->PickTiledTextureFromDisk_Implimentation();

	return 0;
}

const int Tools_PickTiledTexture::PickTiledTexture_Implimentation( const char* szMessage,const int iCurrentSelection )
{
	BuildThumbnailList();

	// if there are no tiled textures in the manager, ask the user to pick one from disk
	if( m_iThumbnailListSize < 1 )
	{
		PickTiledTextureFromDisk_Implimentation();
	}

	ResizeViewport();

	m_xMessage.MakeBlank();

	if(szMessage)
	{
		m_xMessage = szMessage;
	}

	if( m_xMessage.IsNull() || m_xMessage.IsBlank() )
	{
		m_xMessage = "Selected Tiled Texture";
	}
	
	// set mouse focus to false so that the currently selected tile remains selected until the mouse is moved
	ProcessUserInput();
	m_bMouseFocus = false;

	// set selected thumbnail to the index passed in as an argument
	SetSelectedThumbnailIndex( iCurrentSelection );
	m_iThumbnailFirstRowDisplayed = 0;

	m_bExitRequested = false;
	m_bCancelled = false;

	// main loop
	while(!m_bExitRequested)
	{
		Asura_Windows::CheckForWindowsMessages();
		Asura_UserInput::Update();
		Asura_Cursor::OncePerFrameUpdate();

		ResizeViewport();
		
		ProcessUserInput();
		Render();
	}

	const int iSelectedTileIndex(m_puThumbnailList[GetSelectedThumbnailIndex()]);

	ClearThumbnailList();

	return m_bCancelled ? iCurrentSelection : iSelectedTileIndex;
}

void Tools_PickTiledTexture::PickTiledTextureFromDisk_Implimentation()
{
	// initialise file picker
	Tools_FileSystem_DialogInfo info;

	info.DefaultFilename  = "";
	info.DefaultExtension = ".asr";
	info.DefaultPath = "";
	info.WindowTitle = "Load Chunk File";
	info.AllowMultipleSelect = false;	
	info.FileFilter = "Asura Chunk Files\0*.asr;\0";

	Asura_String xFileName( Tools_FileSystem::GetLoadFilename(info) );

	// if filename is blank assume file picker was canceled/closed and do nothing - NDB
	if( xFileName.IsBlank() )
	{
		return;
	}

	Asura_ChunkLoading_System::LoadAsuraFile( xFileName.GetString() );

	// Set texture indexs in v1 tiles (this happens on load for v2 tiles) - NDB
	Asura_TiledTextureManagement::SetTextureHandlesFromHash();
}

void Tools_PickTiledTexture::BuildThumbnailList()
{
	ClearThumbnailList();

	m_iThumbnailListSize = Asura_TiledTextureManagement::Size();
	m_puThumbnailList    = new u_int[m_iThumbnailListSize];

	Asura_TiledTextureManagement::MakeTileIDList( m_puThumbnailList,m_iThumbnailListSize );
}

void Tools_PickTiledTexture::ClearThumbnailList()
{
	delete [] m_puThumbnailList;
	m_puThumbnailList = NULL;
	m_iThumbnailListSize = 0;
}

void Tools_PickTiledTexture::ProcessUserInput()
{
	int iSelectedThumbnailRow(m_iSelectedThumbnailRow);
	int iSelectedThumbnailCol(m_iSelectedThumbnailCol);
	int iThumbnailFirstRowDisplayed(m_iThumbnailFirstRowDisplayed);

	// process mouse input
	const bool bLMBClick( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) );
	m_bMouseFocus |= bLMBClick;
	
	int iCursorX(0);
	int iCursorY(0);
	Asura_Cursor::GetScreenSpaceCoords( iCursorX,iCursorY );
	
	// if mouse has moved then set input focus to mouse and update cursor coords
	if( (iCursorX != m_iCursorX) || (iCursorY != m_iCursorY) )
	{
		m_iCursorX = iCursorX;
		m_iCursorY = iCursorY;
	
		m_bMouseFocus = true;
	}

	if( m_bMouseFocus )
	{
		const float fCursorCoordsY( m_iCursorY - m_fThumbnailRegionStartY );

		if( fCursorCoordsY > 0.0f )
		{
			iSelectedThumbnailRow = static_cast<int>(fCursorCoordsY / m_fThumbnailSizePixels) + m_iThumbnailFirstRowDisplayed;
			iSelectedThumbnailCol = static_cast<int>(iCursorX / m_fThumbnailSizePixels);

			// exit on mouse click over selected texture
			if( (iSelectedThumbnailRow == m_iSelectedThumbnailRow) && (iSelectedThumbnailCol == m_iSelectedThumbnailCol) && (bLMBClick) )
			{	
				m_bExitRequested = true;
			}

			// scroll on mouse wheel
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEWHEELDOWN) )
			{
				iThumbnailFirstRowDisplayed += 1;
			}
			else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEWHEELUP) )
			{
				iThumbnailFirstRowDisplayed -= 1;
			}
		}
	}

	// <-- cursor keys select tile -->

	// move selction up one row
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_UP) )
	{
		iSelectedThumbnailRow -= 1;
		m_bMouseFocus = false;
	}
	// move selection down one row
	else if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_DOWN) )
	{
		iSelectedThumbnailRow += 1;
		m_bMouseFocus = false;
	}
	// move selection left one or wrap around to previous row
	else if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_LEFT) )
	{
		iSelectedThumbnailCol -= 1;
		m_bMouseFocus = false;
	}
	// move selection right one or wrap around to next row
	else if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_RIGHT) )
	{
		iSelectedThumbnailCol += 1;
		m_bMouseFocus = false;
	}

	// process keybord input
	if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER) || Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE) )
	{
		m_bExitRequested = true;

		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			m_bCancelled = true;
		}
	}

	if( Asura_UserInput::DebouncedKeyPressed('l') )
	{
		PickTiledTextureFromDisk_Implimentation();
		BuildThumbnailList();		
	}

	//F-Keys change alpha mode
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F1))
	{
		m_eAlphaMode = PICKTILE_OPAQUE;
	}
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F2))
	{
		m_eAlphaMode = PICKTILE_TRANSPARENT_BLACK;
	}
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F3))
	{
		m_eAlphaMode = PICKTILE_TRANSPARENT_MAGENTA;
	}

	const int iNumberOfRowsDisplayed( Asura_Maths::FloatToInt_Near((m_fViewportHeight-m_fThumbnailRegionStartY)/m_fThumbnailSizePixels) );

	// ensure selected tile is valid
	if( (iSelectedThumbnailRow>=0) && (iSelectedThumbnailRow < m_iThumbnailRowCount) )
	{
		m_iSelectedThumbnailRow = iSelectedThumbnailRow;
	}

	if( (iSelectedThumbnailCol>=0) && (iSelectedThumbnailCol < m_iThumbnailColCount) )
	{
		m_iSelectedThumbnailCol = iSelectedThumbnailCol;
	}

	// if the last row is not full && selected tile is on the last row && selected tile is beyond the last occupied column in that row
	if( (m_iThumbnailListSize%m_iThumbnailColCount) && (iSelectedThumbnailRow >= (m_iThumbnailRowCount-1) ) && (m_iSelectedThumbnailCol >= (m_iThumbnailListSize%m_iThumbnailColCount) )  )
	{
		m_iSelectedThumbnailCol = m_iThumbnailListSize%m_iThumbnailColCount - 1;
	}

	// make sure new selected tile appears on screen
	if( m_iSelectedThumbnailRow < iThumbnailFirstRowDisplayed )
	{
		iThumbnailFirstRowDisplayed = m_iSelectedThumbnailRow;
	}
	else if( m_iSelectedThumbnailRow >= (iThumbnailFirstRowDisplayed+iNumberOfRowsDisplayed) )
	{
		iThumbnailFirstRowDisplayed = m_iSelectedThumbnailRow - iNumberOfRowsDisplayed + 1;
	}

	// ensure first row displayed is valid
	if( (iThumbnailFirstRowDisplayed>=0) && (iThumbnailFirstRowDisplayed<(m_iThumbnailRowCount - iNumberOfRowsDisplayed + 1) ) )
	{
		m_iThumbnailFirstRowDisplayed = iThumbnailFirstRowDisplayed;
	}
	
	// TODO - check if tools has exit request

	// unit testing 
	Asura_Assert( GetSelectedThumbnailIndex()<m_iThumbnailListSize,"selected tile out of range" );
	Asura_Assert( m_iSelectedThumbnailRow<m_iThumbnailRowCount,"selected tile out of range" );
	Asura_Assert( m_iSelectedThumbnailCol<m_iThumbnailColCount,"selected tile out of range" );
}

void Tools_PickTiledTexture::Render()
{
	Asura_Render::BeginFrame();

	// first draw a big black poly over everything that has taken place so far...
	Asura_Render::Overlay.SetRectXY( 0,0,m_fViewportWidth,m_fViewportHeight );
	Asura_Render::Overlay.SetRectUV( 0,0,1,1 );
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex     = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour           = Tools_TT_Colour::Black;
	Asura_Render::Overlay.Render();
	
	Asura_Render::FlushDisplay();	

	const float fThumbnailPaddPixels( m_fThumbnailSizePixels * 0.1f );
	const ASURA_TRANSLUCENCY_MODE eTranslucency( (m_eAlphaMode == PICKTILE_OPAQUE)? ASURA_TRANSLUCENCY_OFF:ASURA_TRANSLUCENCY_NORMAL );
	const int iBackgroundColour( (m_eAlphaMode == PICKTILE_TRANSPARENT_MAGENTA)? Tools_TT_Colour::Magenta : Tools_TT_Colour::Black );

	// render thumbnails
	const Asura_TiledTextureInfo* pxTileInfo;
	int iTextureID(0);
	Asura_Bounding_Rectangle xScreenSpace( fThumbnailPaddPixels,m_fThumbnailRegionStartY + fThumbnailPaddPixels,
										   m_fThumbnailSizePixels-fThumbnailPaddPixels,
										   m_fThumbnailRegionStartY+m_fThumbnailSizePixels-fThumbnailPaddPixels );
	Asura_Bounding_Rectangle xUV;

	// draw thumbnails
	for( int i(m_iThumbnailFirstRowDisplayed*m_iThumbnailColCount);(i<m_iThumbnailListSize) && (xScreenSpace.MaxY<m_fViewportHeight);++i )
	{
		// retrive tile
		pxTileInfo = Asura_TiledTextureManagement::GetTileInfoByID(m_puThumbnailList[i]);
	 
	 	if( pxTileInfo )
		{
			iTextureID = Asura_Texture_Management::GetIndexFromHashID( pxTileInfo->m_uTextureHashID );
			xUV.Set( pxTileInfo->m_fu1,pxTileInfo->m_fv1,pxTileInfo->m_fu2,pxTileInfo->m_fv2 );

			// render tile thumbnail
			Tools_TT_RenderTexture( xScreenSpace,xUV,iTextureID,iBackgroundColour,eTranslucency );
		}

		// update screen space coords for next thumbnail
		xScreenSpace.MinX += m_fThumbnailSizePixels;
		xScreenSpace.MaxX += m_fThumbnailSizePixels;

		if( xScreenSpace.MaxX > m_fViewportWidth )
		{
			xScreenSpace.MinX = fThumbnailPaddPixels;
			xScreenSpace.MaxX = m_fThumbnailSizePixels - fThumbnailPaddPixels;

			xScreenSpace.MinY += m_fThumbnailSizePixels;
			xScreenSpace.MaxY += m_fThumbnailSizePixels;
		}
	}

	// highlight the selected tile thumbnail
	xScreenSpace.MinX = m_iSelectedThumbnailCol * m_fThumbnailSizePixels;
	xScreenSpace.MaxX = xScreenSpace.MinX + m_fThumbnailSizePixels;
	xScreenSpace.MinY = m_fThumbnailRegionStartY + ((m_iSelectedThumbnailRow-m_iThumbnailFirstRowDisplayed) * m_fThumbnailSizePixels);
	xScreenSpace.MaxY = xScreenSpace.MinY + m_fThumbnailSizePixels;
		
	Tools_TT_RenderBorder( xScreenSpace,Tools_TT_Colour::White );

	// draw larger previews in top left and right corners
	float fPreviewSizePixels(m_fThumbnailRegionStartY - ASURA_TEXT_OUTPUT_FONT_HEIGHT);

	// draw the selected tile preview (top left)
	xScreenSpace.MinX = fThumbnailPaddPixels;
	xScreenSpace.MaxX = fPreviewSizePixels - fThumbnailPaddPixels;
	xScreenSpace.MinY = xScreenSpace.MinX;
	xScreenSpace.MaxY = xScreenSpace.MaxX;

	pxTileInfo = Asura_TiledTextureManagement::GetTileInfoByID(m_puThumbnailList[GetSelectedThumbnailIndex()]);

	float print_x(0.0f);
	float print_y(0.0f);

	if( pxTileInfo )
	{
		iTextureID = Asura_Texture_Management::GetIndexFromHashID( pxTileInfo->m_uTextureHashID );
		xUV.Set( pxTileInfo->m_fu1,pxTileInfo->m_fv1,pxTileInfo->m_fu2,pxTileInfo->m_fv2 );

		Tools_TT_RenderTexture( xScreenSpace,xUV,iTextureID,iBackgroundColour,eTranslucency );

		// add tile details text underneath tile preview 
		Asura_String xPreviewDetailsString;
		BuildTileDetailsString( pxTileInfo,xPreviewDetailsString );

		print_x = xScreenSpace.MinX;
		print_y = fPreviewSizePixels;

		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		Asura_Text_Output::Print( print_x,print_y,Tools_TT_Colour::White,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,xPreviewDetailsString.GetString() );

		// draw the tiled texture of which the selected tile is a member (top right)
		xScreenSpace.MinX = m_fViewportWidth - fPreviewSizePixels + fThumbnailPaddPixels;
		xScreenSpace.MaxX = m_fViewportWidth - fThumbnailPaddPixels;
		xScreenSpace.MinY = fThumbnailPaddPixels;
		xScreenSpace.MaxY = fPreviewSizePixels - fThumbnailPaddPixels;

		Tools_TT_RenderTexture( xScreenSpace,iTextureID,iBackgroundColour,eTranslucency );

		// highlight slected tile in tiled texture
		const float fPreviewContentsSizePixels( fPreviewSizePixels - fThumbnailPaddPixels - fThumbnailPaddPixels ); // size of texture within preview region

		xUV.MinX = xScreenSpace.MinX + (fPreviewContentsSizePixels * pxTileInfo->m_fu1);
		xUV.MaxX = xScreenSpace.MinX + (fPreviewContentsSizePixels * pxTileInfo->m_fu2);
		xUV.MinY = xScreenSpace.MinY + (fPreviewContentsSizePixels * pxTileInfo->m_fv1);
		xUV.MaxY = xScreenSpace.MinY + (fPreviewContentsSizePixels * pxTileInfo->m_fv2);

		Tools_TT_RenderBorder( xUV,Tools_TT_Colour::White );

		// add texture details text underneath texture preview
		BuildTextureDetailsString( pxTileInfo,xPreviewDetailsString );

		print_x = xScreenSpace.MaxX;
		print_y = fPreviewSizePixels;

		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		Asura_Text_Output::Print( print_x,print_y,Tools_TT_Colour::White,ASURA_TEXT_FORMAT_RIGHTJUSTIFIED,xPreviewDetailsString.GetString() );
	}

	// draw some instructions
	print_x = 0.5f*m_fViewportWidth;
	print_y = 0.0f;

	Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
	print_y = 2.0f;
	Asura_Text_Output::Print(print_x,print_y++,Tools_TT_Colour::Green,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Left click or Enter to choose a texture");
	Asura_Text_Output::Print(print_x,print_y++,Tools_TT_Colour::Green,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"mouse or cursor keys to navigate");
	Asura_Text_Output::Print(print_x,print_y++,Tools_TT_Colour::Green,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"scroll with mouse wheel");
	Asura_Text_Output::Print(print_x,print_y++,Tools_TT_Colour::Green,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"L : Load a tiled texture");
	Asura_Text_Output::Print(print_x,print_y++,Tools_TT_Colour::Green,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Escape : Cancel");

	//show the message passed to the function
	Asura_Text_Output::Print( print_x,0.0f,Tools_TT_Colour::White,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,m_xMessage.GetString() );

	// finsh render
	Asura_Render::FlushDisplay();

	Tools_Timed_Message_System::Render();

	Asura_Cursor::SetType(ASURA_CURSOR_CROSSHAIR);
	Asura_Cursor::Render();

	Asura_Render::EndFrame();
}
	
void Tools_PickTiledTexture::ResizeViewport()
{
	if( (m_fViewportWidth  != static_cast<float>(Asura_Viewport::GetWidth())) ||
		(m_fViewportHeight != static_cast<float>(Asura_Viewport::GetHeight())) )
	{
		m_fViewportWidth  = static_cast<float>(Asura_Viewport::GetWidth());
		m_fViewportHeight = static_cast<float>(Asura_Viewport::GetHeight());

		m_iThumbnailColCount   = Asura_Maths::FloatToInt_Near(m_fViewportWidth / m_fOptimalThumbnailSizePixels);
		m_fThumbnailSizePixels = m_fViewportWidth / static_cast<float>(m_iThumbnailColCount);

		Asura_Assert( m_fThumbnailSizePixels>=0.0f,"Thumbnail size should never become negative" );

		if( m_fThumbnailSizePixels > (m_fViewportHeight * 0.75f) )
		{
			// make sure at least one row of thumbnails is visible
			m_fThumbnailSizePixels   = (m_fThumbnailSizePixels<m_fViewportHeight)? m_fThumbnailSizePixels:m_fViewportHeight;
			m_fThumbnailRegionStartY = 0.0f;
		}
		else
		{
			// set thumbnail region to start about 1/4 of the way down the screen but fit an integer number of thumbnails
			m_fThumbnailRegionStartY = m_fViewportHeight - static_cast<float>( Asura_Maths::FloatToInt_Chop((m_fViewportHeight*0.75f)/m_fThumbnailSizePixels) )*m_fThumbnailSizePixels;
		}
	}	

	// find number of rows
	m_iThumbnailRowCount = m_iThumbnailListSize/m_iThumbnailColCount;

	if(m_iThumbnailListSize%m_iThumbnailColCount)
	{
		++m_iThumbnailRowCount;
	}
}

const Asura_String& Tools_PickTiledTexture::BuildTileDetailsString( const Asura_TiledTextureInfo* const pxTileInfo, Asura_String& xPreviewDetailsText )
{
	if( pxTileInfo )
	{
		Asura_Char acTmpCharArray[32];
		Asura_String xTmpString;
		xPreviewDetailsText.MakeBlank();

		xPreviewDetailsText += "Tile hash: ";

		Asura_PrintfToBfr( acTmpCharArray,"0x%X",32,true,pxTileInfo->m_uTileHashID );
		xPreviewDetailsText += acTmpCharArray;

		xPreviewDetailsText += " ";

		xTmpString.MakeBlank();
		Asura_StringUtil::ToString( static_cast<int>(pxTileInfo->m_TiledToWidth * (pxTileInfo->m_fu2 - pxTileInfo->m_fu1)),xTmpString );
		xPreviewDetailsText += xTmpString;

		xPreviewDetailsText += "x";

		xTmpString.MakeBlank();
		Asura_StringUtil::ToString( static_cast<int>(pxTileInfo->m_TiledToHeight * (pxTileInfo->m_fv2 - pxTileInfo->m_fv1)),xTmpString );
		xPreviewDetailsText += xTmpString;

		//xPreviewDetailsText += " px u,v: (";

		//xTmpString.MakeBlank();
		//Asura_StringUtil::ToString( pxTileInfo->m_fu1,xTmpString );
		//xTmpString.Truncate(4);
		//xPreviewDetailsText += xTmpString;

		//xPreviewDetailsText += ",";

		//xTmpString.MakeBlank();
		//Asura_StringUtil::ToString( pxTileInfo->m_fv1,xTmpString );
		//xTmpString.Truncate(4);
		//xPreviewDetailsText += xTmpString;

		//xPreviewDetailsText += ") to (";

		//xTmpString.MakeBlank();
		//Asura_StringUtil::ToString( pxTileInfo->m_fu2,xTmpString );
		//xTmpString.Truncate(4);
		//xPreviewDetailsText += xTmpString;

		//xPreviewDetailsText += ",";

		//xTmpString.MakeBlank();
		//Asura_StringUtil::ToString( pxTileInfo->m_fv2,xTmpString );
		//xTmpString.Truncate(4);
		//xPreviewDetailsText += xTmpString;

		//xPreviewDetailsText += ")";
	}

	return xPreviewDetailsText;
}

const Asura_String& Tools_PickTiledTexture::BuildTextureDetailsString( const Asura_TiledTextureInfo* const pxTileInfo, Asura_String& xPreviewDetailsText )
{
	if( pxTileInfo == NULL )
	{
		return xPreviewDetailsText;
	}

	Asura_String xTmpString;
	xPreviewDetailsText.MakeBlank();

	xPreviewDetailsText += "Texture: ";
	xPreviewDetailsText += pxTileInfo->m_xNameOfTextureUsed;

	xPreviewDetailsText += " ";

	xTmpString.MakeBlank();
	Asura_StringUtil::ToString( static_cast<int>(pxTileInfo->m_TiledToWidth),xTmpString );
	xPreviewDetailsText += xTmpString;

	xPreviewDetailsText += "x";

	xTmpString.MakeBlank();
	Asura_StringUtil::ToString( static_cast<int>(pxTileInfo->m_TiledToHeight),xTmpString );
	xPreviewDetailsText += xTmpString;

	xPreviewDetailsText += " px";

	return xPreviewDetailsText;
}

