//
// Tools_PickTexture.h
//
// Created by Nick Brett - 12/10/2007 11:04:08
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PICKTILEDTEXTURE_H__
#define __TOOLS_PICKTILEDTEXTURE_H__

#include "Core/Asura.h"
#include "Core/Asura_TiledTextureManagement.h"

class Tools_PickTiledTexture
{
	enum TOOLS_PICKTILE_ALPHA_MODE
	{
		PICKTILE_OPAQUE,
		PICKTILE_TRANSPARENT_BLACK,
		PICKTILE_TRANSPARENT_MAGENTA,
	};

public:
	Tools_PickTiledTexture();
	~Tools_PickTiledTexture();

	void Initialise();
	void Shutdown();

	// allows user to select a tile from all the tiles loaded in Asura_TiledTextureManagement
	static const int PickTiledTexture( const char* szMessage,const int iCurrentSelection );	
	static const int PickTiledTextureFromDisk();

	static Tools_PickTiledTexture* GetThis()
	{
		return s_pxPtrToSelf;
	}

private:
	// <-- private methods --> 
	const int PickTiledTexture_Implimentation( const char* szMessage,const int iCurrentSelection );
	void PickTiledTextureFromDisk_Implimentation();

	// replaces the contents of the thumbnail list with pointers to all the tiles held in the Asura_TiledTextureManager
	void BuildThumbnailList();

	// removes all entries from the thumbnail list
	void ClearThumbnailList();

	// handles all keybord and mouse input and changes the state of *this appropriatly
	void ProcessUserInput();

	// calculates the index of the selected thumbnail in the thumbnail list using the selcted thumbnail column and row
	inline const int GetSelectedThumbnailIndex() const;

	// sets the selcted thumbnail column and row to corespond with the index supplied as arguement (only makes sense 
	// after m_iThumbnailColCount has been calculated by a call to ResizeViewport() )
	inline void SetSelectedThumbnailIndex( const int iIndex );

	// returns true iff the argument is withnin the bounds of the thumbnail list index
	inline const bool IsIndexValid( const int iIndex );

	// updates the display
	void Render();

	// re-calculates all data member which are dependent on the dimensions of the Tools window if those dimsions have changed since the last call 
	void ResizeViewport();

	// assembles the string of text to be displayed underneath the tile preview in the top left corner of the screen
	const Asura_String& BuildTileDetailsString( const Asura_TiledTextureInfo* const pxTileInfo, Asura_String& xPreviewDetailsText );
	
	// assembles the string of text to be displayed underneath the texture preview in the top right corner of the screen
	const Asura_String& BuildTextureDetailsString( const Asura_TiledTextureInfo* const pxTileInfo, Asura_String& xPreviewDetailsText );
	
	// <-- private data members -->
	Asura_String   m_xMessage;  // title message, displayed at the top of the screen and optionally passed in on a call to PickTiledTexture*()
	
	// last known position of the mouse cursor (updated by a call to ProcessUserInput())
	int m_iCursorX;
	int m_iCursorY;
	
	u_int* m_puThumbnailList;
	int    m_iThumbnailListSize;
	int    m_iThumbnailColCount;          // number of thumbnails in each row
	int    m_iThumbnailRowCount;          // number of rows
	int    m_iThumbnailFirstRowDisplayed; // index of the first row of thumbnails currently shown on screen - used in scrolling
	int	   m_iSelectedThumbnailRow;       
	int	   m_iSelectedThumbnailCol;
	bool   m_bExitRequested;              
	bool   m_bCancelled;
	bool   m_bMouseFocus;                 // if true, the position of the mouse pointer determines which tile is selected
	TOOLS_PICKTILE_ALPHA_MODE m_eAlphaMode;

	float m_fOptimalThumbnailSizePixels; // Thumbnails will be as close as possible to this size whilst remaining an integer factor of screen width

	float m_fThumbnailSizePixels;        // height|width of each thumbnail (inc padding)
	float m_fThumbnailRegionStartY;      // point below which the thumbnails are drawn

	// last know viewport dimensions (updated by a call to ResizeViewport())
	float m_fViewportWidth;              
	float m_fViewportHeight;

	// <-- private static data members --> 
	static Tools_PickTiledTexture* s_pxPtrToSelf;
};

inline const int Tools_PickTiledTexture::GetSelectedThumbnailIndex() const
{
	return m_iSelectedThumbnailRow*m_iThumbnailColCount + m_iSelectedThumbnailCol;
}

inline void Tools_PickTiledTexture::SetSelectedThumbnailIndex( const int iIndex )
{
	if( IsIndexValid(iIndex) )
	{
		m_iSelectedThumbnailCol = iIndex % m_iThumbnailColCount;
		m_iSelectedThumbnailRow = iIndex / m_iThumbnailColCount;
	}
	else
	{
		m_iSelectedThumbnailCol = 0;
		m_iSelectedThumbnailRow = 0;
	}
}

inline const bool Tools_PickTiledTexture::IsIndexValid( const int iIndex ) 
{
	return (iIndex < m_iThumbnailListSize) && (iIndex > -1);
}

#endif // __TOOLS_PICKTILEDTEXTURE_H__