//
// Tools_Overlay_ViewTextures.h
//
// Created by Richard Rice - 12/09/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_OVERLAY_VIEWTEXTURES__
#define __TOOLS_OVERLAY_VIEWTEXTURES__


#include "Interface/Asura_Overlay.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_TextureFlags.h"
#include "Objects/Tools_ObjectSystem_Param.h"
#include "Misc/list_tem.hpp"

class Asura_Image_Dialog;
class Asura_Subwindow_Dialog;
template <class T> class Tools_OVT_SharedValue;


enum Tools_OVT_TextureUsage
{
	TOOLS_OVT_USAGE_ANY = 0, //Used to disable filtering by usage
	TOOLS_OVT_USAGE_ENVIRONMENT,//For textures used on environment polygons
	TOOLS_OVT_USAGE_LIGHTSET,//environment polygons in the current light set
	TOOLS_OVT_USAGE_CUBEMAP,
	TOOLS_OVT_USAGE_GUIMENU,
	TOOLS_OVT_USAGE_SKYBOX,

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	//material response flags (should match ordering in Asura_MaterialResponse)
	TOOLS_OVT_USAGE_ALBEDO,
	TOOLS_OVT_USAGE_NORMAL_MAP,
	TOOLS_OVT_USAGE_SPECULAR_MAP,
	TOOLS_OVT_USAGE_TRANSPARENCY_MAP,
	TOOLS_OVT_USAGE_SELFILLUMINATION_MAP,
	TOOLS_OVT_USAGE_ENVIRONMENTMAP_MAP,
	TOOLS_OVT_USAGE_REFLECTIVITY_MASK,
	TOOLS_OVT_USAGE_PARALLAX_MAP,
	TOOLS_OVT_USAGE_DETAIL_MAP,
	TOOLS_OVT_USAGE_DETAIL_NORMAL_MAP,

	TOOLS_OVT_USAGE_FIRSTMATERIALRESPONSE = TOOLS_OVT_USAGE_ALBEDO,
	TOOLS_OVT_USAGE_LASTMATERIALRESPONSE = TOOLS_OVT_USAGE_DETAIL_NORMAL_MAP,
	//end of material response
	#endif //ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	TOOLS_OVT_NUMUSAGE,

};


#define TOOLS_OVT_NUMFLAGTEXTDIALOG 10 //number of spaces for showing the names of the flags of the texture under the cursor


class Tools_OVT_Texture
{
public :
	Tools_OVT_Texture()
	{
		m_iTextureID = ASURA_TEXTURE_NONE;
		m_uNumPolygons = 0;
		m_pxImageDialog = 0;
		m_uUsageFlags = 0;
		m_iWidth = 0;
		m_iHeight = 0;
		m_iSize = 0;
		m_uMostRecentMaterial = 0;
		m_bVisible = false;
		m_bHighlight = false;
		m_bSelected = false;
		m_iVisibleIndex = -1;
	}

	int m_iTextureID;
	u_int m_uNumPolygons;
	Asura_Image_Dialog* m_pxImageDialog;

	Asura_String m_xTextureName;
	Asura_String m_xTexturePath;
	u_int m_uUsageFlags;
	int m_iWidth;
	int m_iHeight;
	int m_iSize;
	Asura_Hash_ID m_uMostRecentMaterial;
	bool m_bVisible;
	bool m_bHighlight;
	bool m_bSelected;
	int m_iVisibleIndex;//index in sorted visible list
	
};

enum
{
	TOOLS_OVT_RESULT_OKAY,
	TOOLS_OVT_RESULT_CANCEL,
	TOOLS_OVT_RESULT_LOADTEXTURE,
};




class Tools_Overlay_ViewTextures : public Asura_Overlay
{
public :
	~Tools_Overlay_ViewTextures();

	static void CreateOverlay();
	static void CloseOverlay();

	static bool OverlayExists();

	//pick a material using a modal overlay
	static int PickTexture(const char* szTitle,int iInitialSelection = ASURA_TEXTURE_NONE, int iInitialTextureFlags = 0, Asura_Hash_ID* puResult = 0);
	//Can be used to set the initial display according to what the texture picking is being done for
	//Filter reverts back to the user's last selection after PickTexture() finishes
	static void SetTextureUsageFilter(Tools_OVT_TextureUsage eUsageFilter);

	static bool WasCancelled();
	
	//////////////////////////////
	static bool GetParameterFromSelection(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam,bool* pbMixed);
	static bool SetParameterForSelection(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);
private :
	Tools_Overlay_ViewTextures();

	virtual void PerFrameUpdate();

	void InitialiseOverlay();
	void UpdateThumbnails();
	void UpdateTextureInfo();
	static void ListGUITexturesCallback(int iTextureIndex,void* pList);

	void UpdateDisplayMode();
	void UpdateDisplayMode(Asura_Image_Dialog* pxDialog);

	void AddTexture(int iTextureID, u_int uUsageFlags, Asura_Hash_ID uMaterial, int iPolyCount);
	static int SortTextureCallback(const void* pElem1,const void* pElem2);

	void SetSelectedTexture(int iTextureID);

	void SetCurrentTexture(Tools_OVT_Texture* pxTexture);
	void SetSelected(Tools_OVT_Texture* pxTexture, bool bSelect);
	void SelectMultiple(Tools_OVT_Texture* pxStart, Tools_OVT_Texture* pxEnd, bool bAddToSelection);
	void ClearSelection();

	void SetDefaultTitle() {SetTitle("Textures");}

	void EndModal() {m_bAlwaysOnTop = false;}

	//overlay callbacks
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_Overlay_ViewTextures*)iThis)->__FnName();}\
	void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_Overlay_ViewTextures*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(OnOkay)
	OVERLAY_VOID_CALLBACK(OnCancel)

	OVERLAY_VOID_CALLBACK(OnClose)
	OVERLAY_VOID_CALLBACK(LoadTexture)
	OVERLAY_VOID_CALLBACK(ReloadTexture)



	OVERLAY_BOOL_CALLBACK(IsNotModal)
	OVERLAY_BOOL_CALLBACK(IsModal)
	OVERLAY_BOOL_CALLBACK(IsTextureSelected)

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK


	static void ShowKeys();

	//for getting/setting the flags on the selected textures
	bool GetParameterFromTexture(Tools_OVT_Texture* pxTexture,Asura_Hash_ID uParamID,class Tools_Parameter* pxParam);
	bool SetParameterForTexture(Tools_OVT_Texture* pxTexture,Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);


	//////

	Asura_Image_Dialog* m_pxLargeImageDialog;
	Asura_Subwindow_Dialog* m_pxSubwindowDialog;
	Asura_Dialog* m_pxTextureNameDialog;
	Asura_Dialog* m_pxTexturePathDialog;
	Asura_Dialog* m_pxTextureDimensionDialog;
	Asura_Dialog* m_pxPolygonsDialog;

	Asura_Dialog* m_apxFlagTextDialog[TOOLS_OVT_NUMFLAGTEXTDIALOG];

	bool m_bUpdateThumbnails;
	bool m_bUpdateDisplayMode;
	bool m_bUsageFilterChanged;

	float m_fSubwindowWidth;

	List<Tools_OVT_Texture*> m_xSelection;

	Tools_OVT_Texture* m_pxCurrentTexture;
	Tools_OVT_Texture* m_pxLastClickedTexture;
	
	int m_iNumVisibleTextures;
	Tools_OVT_Texture** m_ppxVisibleTextures;

	Tools_OVT_SharedValue<bool>* m_pxFlagSharedValues;


	Asura_TemplateHashTree<Tools_OVT_Texture> m_xTextureTree;

	static int s_iSortMode;
	static int s_iUsageFilter;
	static int s_iLastUsageFilter;//most recently set value by the user
	static int s_iTextureFlagFilter;
	static int s_iTextureFlagMode;

	static int s_iDisplayMode;

	static int s_iResult;
	static int s_iSelectedTexture;

	static Tools_Overlay_ViewTextures* s_pxThis;
};



//Derived class from the shared paramater class , for dialog values
//that are shared across the texture selection

template <class T> class Tools_OVT_SharedValue : public Asura_Dialog_SharedValue<T>
{
public :
	Tools_OVT_SharedValue()
	{
		m_uParamID = 0;
	}
	
	
	Tools_OVT_SharedValue(Asura_Hash_ID uParamID)
	{
		m_uParamID = uParamID;
	}	

	void SetParamID(Asura_Hash_ID uParamID)
	{
		m_uParamID = uParamID;
	}

	
	virtual void GetValue(T* pxValue,bool* pbMixed) const
	{
		if(pxValue || pbMixed)
		{
			Tools_ParameterDerived<T> xParam;
			Tools_Overlay_ViewTextures::GetParameterFromSelection(m_uParamID,&xParam,pbMixed);

			if(pxValue) *pxValue = xParam.GetValue();
		}
	}

	virtual void SetValue(const T& xValue)
	{
		Tools_ParameterDerived<T> xParam(xValue);
		Tools_Overlay_ViewTextures::SetParameterForSelection(m_uParamID,&xParam);
	}


private :
	Asura_Hash_ID m_uParamID;
};



#endif //__TOOLS_OVERLAY_VIEWTEXTURES__
