//
// Tools_PickTexture.cpp
//
// Created by Kevin Lea - 02/07/2001 15:11:50
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Texturing/Tools_PickTexture.h"

// Asura Platform-independent
#include "Animation/Asura_TextureAnimation.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_NameTree.h"
#include "Core/Asura_TextureFlags.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Windows.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_CPUProfile_Renderer.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Key_Combination.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Env_System.h"
#include "Mesh/Asura_Env.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/Asura_Skybox.h"

// Asura PC
#include "Rendering/Asura_Direct3D_Effects_System.h"

// Tools
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Dialog.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Skybox/Tools_Mode_Skybox.h"
#include "Texturing/Tools_Texture_Management.h"
#include "Texturing/Tools_Overlay_ViewTextures.h"

// globals
static Asura_String LastFileLoaded;

#define NO_OF_TEXTURE_FLAGS (22) 
static char* s_apszTextureFlagNames[NO_OF_TEXTURE_FLAGS] =
{
	"8 Bit Alpha",
	"16 Colour Palette",
	"PS2 Moire Fix",
	"No Mip Maps",
	"Wii Only No Mip Maps",
	"PS2 Distant Texture",
	//	"Reflective",
	"Full Motion Video",
	//    "Flowing Water",
	//    "Waterfall",
	//    "Subdivide Polygon",
	//    "PS2 Additive Alpha",
	"PS2 Allow 512",
	"PS2 Allow Mipmaps on 512",
	"PS2 Don't Halve Size",
	"PS2 Don't Halve Extra For Multiplayer",
	//	"Rain Splashes"
	"Reduce extra level for consoles",
	"PS2 Sharpen",
	"PS2 Sharpen More",
	"XBox Half Size",
	"PSP Half Size",
	"Accept Carpet",
	"AD Receiver",
	"High-Quality Compression (Uses More Memory)",
	"Sharpen Mip Maps",
	"Clamp-U",
	"Clamp-V"
};

static int s_aiTextureFlagValues[NO_OF_TEXTURE_FLAGS] =
{
	AsuraTextureFlag_8BitAlpha,
	AsuraTextureFlag_16ColourPalette,
	AsuraTextureFlag_PS2MoireFix,
	AsuraTextureFlag_NoMipMaps,
	AsuraTextureFlag_WIIOnly_NoMipMaps,
	AsuraTextureFlag_PS2DistantTexture,
	//	AsuraTextureFlag_Reflective,
	AsuraTextureFlag_FMV,
	//    AsuraTextureFlag_FlowingWater,
	//    AsuraTextureFlag_Waterfall,
	//    AsuraTextureFlag_SubdividePolygon,
	//    AsuraTextureFlag_PS2AdditiveAlpha,
	AsuraTextureFlag_PS2Allow512,
	AsuraTextureFlag_PS2AllowMipMapsOn512,
	AsuraTextureFlag_PS2DontHalveSize,
	AsuraTextureFlag_PS2DontHalveExtraForMultiplayer,
	//	AsuraTextureFlag_RainSplashes,
	AsuraTextureFlag_ReduceExtraForConsoles,
	AsuraTextureFlag_Sharpen,
	AsuraTextureFlag_SharpenMore,
	AsuraTextureFlag_HalfForXBox,
	AsuraTextureFlag_HalfForPSP,
	AsuraTextureFlag_AcceptCarpet,
	AsuraTextureFlag_AdReceiver,
	AsuraTextureFlag_HighQualityCompression,
	AsuraTextureFlag_SharpenMipMaps,
	AsuraTextureFlag_ClampU,
	AsuraTextureFlag_ClampV
};

#define NO_OF_MATERIAL_FLAGS (16) // DWW was 15
static char* s_apszMaterialFlagNames[NO_OF_MATERIAL_FLAGS] =
{
	"Self Illuminate",
	"Transparent",
	"Use Detail Map",
	"Reflective",
	"FMV Texture",
	"Flowing Water",
	"Waterfall",
	"Subdivided Polygon",
	"Additive Alpha",
	"3d Decal",
	"Rain Splashes",
	"Specular - soft highlights",
	"Specular - sharp highlights",
	"Alpha Decal (Not PS2)",
	"Parallax Mapping",
	"Ad receiver"
};


static int s_aiMaterialFlagValues[NO_OF_MATERIAL_FLAGS] =
{
	AsuraMaterialFlag_SelfIlluminate,
	AsuraMaterialFlag_Transparent,
	AsuraMaterialFlag_UseDetailMap,
	AsuraMaterialFlag_Reflective,
	AsuraMaterialFlag_FMV,
	AsuraMaterialFlag_FlowingWater,
	AsuraMaterialFlag_Waterfall,
	AsuraMaterialFlag_SubdividePolygon,
	AsuraMaterialFlag_PS2AdditiveAlpha,
	AsuraMaterialFlag_3dDecal,
	AsuraMaterialFlag_RainSplashes,
	AsuraMaterialFlag_Specular_Soft,
	AsuraMaterialFlag_Specular_Sharp,
	AsuraMaterialFlag_AlphaDecal,
	AsuraMaterialFlag_ParallaxMapping,
	AsuraMaterialFlag_AdReceiver // DWW
};

// static members
Tools_PickTexture* Tools_PickTexture::s_pxThis = 0;
bool (*Tools_PickTexture::s_pfnTextureFilter)(int,void*) = 0;
void* Tools_PickTexture::s_pTextureFilterUserData = 0;


Tools_PickTexture::Tools_PickTexture()
{
	Asura_Assert(!s_pxThis,"Tools_PickTexture already allocated");
	s_pxThis = this;

	m_pszTextureFlagNames = s_apszTextureFlagNames;
	m_piTextureFlagValues = s_aiTextureFlagValues;
	m_uNumberOfTextureFlags = NO_OF_TEXTURE_FLAGS;
	m_bShowCurrentLightSetOnly = false;
}

Tools_PickTexture::~Tools_PickTexture()
{
	while(m_xMaterialList.size()) m_xMaterialList.delete_first_entry();
	s_pxThis = 0;
}

void Tools_PickTexture::Initialise()
{
	m_iCurrentTextureFlagIndex = 0;
	m_iCurrentMaterialFlagIndex = 0;
	m_bShowingTextureFlags = true;

	RebuildMaterialList();
}

void Tools_PickTexture::RebuildMaterialListMain()
{
	while(m_xMaterialList.size()) m_xMaterialList.delete_first_entry();

	//get a list of all materials that are currently used by the modules
	int iNumberOfModules = Asura_Module_System::GetNumberOfModules();

	for(int i=0;i<iNumberOfModules;i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		Asura_Assert(pxMesh,"");

		for(int j=0;j<pxMesh->NumberOfPolygons;j++)
		{
			if(!m_xMaterialList.contains(pxMesh->TextureDataPtr[j].m_iMaterialID))
			{
				AddToMaterialListMain(pxMesh->TextureDataPtr[j].m_iMaterialID);
			}
		}
	}
}

int Tools_PickTexture::PickTextureFromDisk(int iInitialTextureFlags /*= 0*/, Asura_Hash_ID* puResult /*= NULL*/)
{
	return s_pxThis->PickTextureFromDisk_Main(iInitialTextureFlags, puResult);
}

int Tools_PickTexture::PickTextureFromDisk_Main(int iInitialTextureFlags, Asura_Hash_ID* puResult)
{
	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = "";
	info.DefaultFilename = LastFileLoaded.GetString();
	if (!info.DefaultFilename) info.DefaultFilename = "";

	info.DefaultPath = "Graphics\\";
	info.FileFilter = "Texture Files\0*.bmp;*.jpg;*.dds;*.tga;*.png\0";
	info.WindowTitle = "Load Texture...";
	info.AllowMultipleSelect = true;

	Asura_String name = Tools_FileSystem::GetLoadFilename(info);

	int newTexture = ASURA_TEXTURE_NONE;
	if(puResult)
	{
		*puResult = ASURA_HASH_ID_UNSET;
	} 

	for(;!name.IsNull();name = Tools_FileSystem::GetNextLoadFilename())
	{
		LastFileLoaded = name;
		Asura_String relative_name;

		{
			_strlwr(name.GetString());
			char *start = strstr(name.GetString(),"graphics");

			if (!start)
			{
				Asura_Assert(0,"File not in 'graphics' folder");
				relative_name = name;
			}
			else
			{
				start+=strlen("graphics");
				relative_name = start;
			}
		}

		//make sure this isn't one of the generated DDS files
		Asura_String xPath;
		name.SplitFilename(&xPath,0,0);
		const char* pcSlashPos = strrchr(xPath.GetString(),'\\');
		const char* pszLastDir = pcSlashPos ? pcSlashPos+1 : xPath.GetString();
		if(!stricmp(pszLastDir,"dds"))
		{
			Asura_Assert(0,"Textures shouldn't be added from the generated DDS directories");
			continue;
		}

		bool bNewTexture;
		newTexture = Asura_Texture_Management::AddTexture(relative_name,&bNewTexture);

		if(puResult)
		{
			*puResult = relative_name.GetHash_IgnoreExtension();
		}

		//apply any default texture flags. RAR 08/05/2006
		if(iInitialTextureFlags)
		{
			int iTextureFlags = Asura_Texture_Management::GetTextureFlags(newTexture);
			Asura_Texture_Management::SetTextureFlags(newTexture,iTextureFlags | iInitialTextureFlags);
		}
		
		if(!bNewTexture)
		{
			//texture was already loaded , check to see if the extension of the existing texture is the same
			const Asura_String* pxExistingName = Asura_Texture_Management::GetTextureName(newTexture);
			
			if(stricmp(pxExistingName->GetString(),relative_name.GetString()))
			{
				//see if the texture should be replaced
				char szMessage[500];
				sprintf(szMessage,"%s is already loaded as %s.\nWould you like to replace the old texture?.",relative_name.GetString(),pxExistingName->GetString());

				YesNo_Overlay xOverlay(szMessage);
				if(xOverlay.DoModal())
				{
					if(Asura_Texture_Management::RenameTexture(newTexture,relative_name.GetString()))
					{
						//force the texture to be reloaded
						Asura_Texture_Management::ReloadTexture(newTexture);
					}
				}
				
			}
			
		}


	}
	Asura_Texture_Management::LoadTextures();

	return newTexture;
}

int Tools_PickTexture::PickMaterial(const char* Message /*= 0*/,int iCurrentSelection /*=ASURA_TEXTURE_NONE*/,bool bPreferSinglePass /*= false*/)
{
	return s_pxThis->PickMaterial_Main(Message,iCurrentSelection,bPreferSinglePass);
}


int Tools_PickTexture::PickMaterial_Main(const char* Message,int iCurrentSelection,bool bPreferSinglePass)
{
	int iTextureID = Asura_Material_System::GetTextureID(iCurrentSelection);
	iTextureID = PickTexture(Message,iTextureID,PICKTEXTURE_DEFAULT);

	if(iTextureID == ASURA_TEXTURE_NONE) return ASURA_MATERIAL_NONE;
	
	//get a list of materials using this texture
	List<int> xMaterialList;

	
	for(LIF<int> xMaterialLIF(&m_xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
			
		if(Asura_Material_System::GetTextureID(xMaterialLIF()) == iTextureID)
		{
			xMaterialList.add_entry(xMaterialLIF());
		}
	}


	if(!xMaterialList.size())
	{
		//create a material for this texture
		int iMaterialID = Asura_Material_System::GetMaterialIDForTexture(iTextureID,true);
		AddToMaterialList(iMaterialID);
		Tools_SaveAsura::SetMainFileModified();

		return iMaterialID;
	}

	if(xMaterialList.size()==1)
	{
		//only one material , just select it then
		return xMaterialList.first_entry();
	}

	//Multiple materials for the texture. Bring up a selection overlay
	ListSelection_Overlay xOverlay("Select a material");
	char szName[100];
	
	int iMaterialID = ASURA_MATERIAL_NONE;

	if(bPreferSinglePass)
	{
		//compare the materials in the list , and remove any dual pass materials
		//if they are identical to single pass ones , apart from the secondary texture
		for(LIF<int> xMatLIF1(&xMaterialList);!xMatLIF1.done();xMatLIF1.next())
		{
			Asura_Material xMat1 = *Asura_Material_System::GetMaterial(xMatLIF1());
			if(	xMat1.m_iTextureID[1] == ASURA_TEXTURE_NONE && 
				xMat1.m_iNormalMapID[1] == ASURA_TEXTURE_NONE &&
				xMat1.m_iSpecularMapID[1] == ASURA_TEXTURE_NONE)
			{
				for(LIF<int> xMatLIF2(&xMaterialList);!xMatLIF2.done();)
				{
					if(xMatLIF1() != xMatLIF2())
					{
						Asura_Material xMat2 = *Asura_Material_System::GetMaterial(xMatLIF2());
						if(	xMat2.m_iTextureID[1] != ASURA_TEXTURE_NONE || 
							xMat2.m_iNormalMapID[1] != ASURA_TEXTURE_NONE ||
							xMat2.m_iSpecularMapID[1] != ASURA_TEXTURE_NONE)
						{
							//second material has secondary textures.. see if its identical apart from that
							xMat2.m_iTextureID[1] = ASURA_TEXTURE_NONE;
							xMat2.m_iNormalMapID[1] = ASURA_TEXTURE_NONE;
							xMat2.m_iSpecularMapID[1] = ASURA_TEXTURE_NONE;

							if(Asura_Material::MaterialsAreEquivalent(&xMat1,&xMat2,false))
							{
								xMatLIF2.delete_current();
								continue;
							}
						}
					}
					xMatLIF2.next();
				}
			}
		}
	}
	
		
	for(LIF<int> xMaterialLIF(&xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
		Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(xMaterialLIF());
		Asura_Assert(pxMaterial,"");

	   	if(pxMaterial->m_xMaterialName.IsBlank())
		{
			Asura_PrintfToBfr(szName,"%03d: <Unnamed>",100,true,xMaterialLIF());
		}
		else
		{
			Asura_PrintfToBfr(szName,"%03d: %s",100,true,xMaterialLIF(),pxMaterial->m_xMaterialName.GetString());
		}

		xOverlay.AddItem(szName,xMaterialLIF());
		iMaterialID = xMaterialLIF();

	}
	
	if(xOverlay.GetNumItems()==1) return iMaterialID;//only one material found , so just return it

	if(!xOverlay.DoModal()) return ASURA_MATERIAL_NONE;

	return xOverlay.GetSelectionData();


}

struct TextureNameAndID
{
	Asura_String Name;
	int	TextureID;
};

int TextureSortCallback(const void* t1,const void* t2)
{
	const TextureNameAndID* pxTex1 = (const TextureNameAndID*) t1;
	const TextureNameAndID* pxTex2 = (const TextureNameAndID*) t2;

	const char* pxName1 = pxTex1->Name.GetString();
	const char* pxName2 = pxTex2->Name.GetString();

	//want to compare names without path
	pxName1 = Tools_FileSystem::StripPath(pxName1);
	pxName2 = Tools_FileSystem::StripPath(pxName2);

	return stricmp(pxName1,pxName2);

}

void Tools_PickTexture::BuildThumbnailArray(u_int uOptionFlags,int& iOutputNumTextures,struct TextureNameAndID*& pxOutputThumbnailArray)
{
	iOutputNumTextures = 0;
	pxOutputThumbnailArray = 0;

	//create an array to get the texture names
	const Asura_String** NameList = new const Asura_String*[ASURA_MAX_NO_OF_TEXTURES];
	//initialise array
	int i;
	for(i=0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
	{
		NameList[i] = NULL;	
	}

	//and get those names
	Asura_Texture_Management::MakeNameList(NameList);

	//determine which textures to show
	bool* pbShowTexture = new bool[ASURA_MAX_NO_OF_TEXTURES];
	for(i = 0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
	{
		pbShowTexture[i] = (uOptionFlags & PICKTEXTURE_ALLTEXTURES) ? true : false;

		// START JAT 15/03/07
		// Don't show textures which are extension of .dds but the file doesn't actually exist
		// This happens when textures are loaded from resource files which have manually had DDS files added to them e.g. fonts
		const Asura_String* const pxTexName = NameList[i];
		if ( pxTexName != NULL )
		{
			const char* const pszExtension = strrchr( pxTexName->GetString(), '.' );
			if ( pszExtension )
			{
				if ( stricmp( pszExtension, ".dds" ) == 0 )
				{
					Asura_String xFileName( "graphics" );
					xFileName += *pxTexName;
					// a .dds texture - so see if the file exists
					if ( Tools_FileSystem::FileExists( xFileName.GetString() ) == false )
					{
						pbShowTexture[i] = false;
					}
				}
			}
		}
		// END JAT 15/03/07
	}

	if(uOptionFlags & (PICKTEXTURE_PRIMARY|PICKTEXTURE_SECONDARY|PICKTEXTURE_BUMP))
	{
		List<int> xTempMaterialList;
		const List<int>* pxMaterialList;

		if(m_bShowCurrentLightSetOnly)
		{
			pxMaterialList = &xTempMaterialList;
			//get a new material list based on those used by the modules in the current light set.
			//RAR 28/05/2008
			Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();
			int iNumberOfModules = Asura_Module_System::GetNumberOfModules();

			for(int i=0;i<iNumberOfModules;i++)
			{

				if(!Tools_LightSets::IsMemberOfCurrentLightSets(pxEnv->GetLightSets(i)))
				{
					continue;
				}
				Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
				Asura_Assert(pxMesh,"");

				for(int j=0;j<pxMesh->NumberOfPolygons;j++)
				{
					int iMaterialID = pxMesh->TextureDataPtr[j].m_iMaterialID;
					if(iMaterialID != ASURA_MATERIAL_NONE && !xTempMaterialList.contains(iMaterialID))
					{
						xTempMaterialList.add_entry(iMaterialID);
					}
				}
			}

		}
		else
		{
			pxMaterialList = &m_xMaterialList;
		}

	//check all materials in environment material list
		for(CLIF<int> xMaterialLIF(pxMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
		{
		
			for(int iChannel=0;iChannel < ASURA_MATERIAL_MAXTEXTURES;iChannel++)
			{
				if(iChannel == 0 && (uOptionFlags & PICKTEXTURE_PRIMARY))
				{
					int iTextureID = Asura_Material_System::GetTextureID(xMaterialLIF(),iChannel);
					if(iTextureID != ASURA_TEXTURE_NONE) pbShowTexture[iTextureID] = true;
				}
				if(iChannel != 0 && (uOptionFlags & PICKTEXTURE_SECONDARY))
				{
					int iTextureID = Asura_Material_System::GetTextureID(xMaterialLIF(),iChannel);
					if(iTextureID != ASURA_TEXTURE_NONE) pbShowTexture[iTextureID] = true;
				}

				if((uOptionFlags & PICKTEXTURE_BUMP))
				{
					int iTextureID = Asura_Material_System::GetNormalMapID(xMaterialLIF(),iChannel);
					if(iTextureID != ASURA_TEXTURE_NONE) pbShowTexture[iTextureID] = true;
				}
			}
		}
	}

	if(uOptionFlags & PICKTEXTURE_SKYBOX)
	{
		//add skybox textures
		for(int i=0;i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES;i++)
		{
			Asura_String xTextureName = Asura_Skybox::GetTexture(i);
			if(xTextureName.IsBlank()) continue;

			int iTextureID = Asura_Texture_Management::GetTextureIndex(xTextureName);
			if(iTextureID != ASURA_TEXTURE_NONE) pbShowTexture[iTextureID] = true;
		}
		//and reflectance texture
		{
			Asura_String xTextureName = Asura_Render::GetReflectanceMapName();
			if(!xTextureName.IsBlank())
			{
				int iTextureID = Asura_Texture_Management::GetTextureIndex(xTextureName);
				if(iTextureID != ASURA_TEXTURE_NONE) pbShowTexture[iTextureID] = true;
			}
		}
	}

	if((uOptionFlags & PICKTEXTURE_USECALLBACKFILTER) && s_pfnTextureFilter)
	{
		for(int i=0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
		{
			if(NameList[i])
			{
				pbShowTexture[i] |= s_pfnTextureFilter(i,s_pTextureFilterUserData);
	}
		}
	}
	
	if( uOptionFlags & PICKTEXTURE_CUBEMAP )
	{
		for( u_int u = 0; u < ASURA_MAX_NO_OF_TEXTURES; ++u )
		{
			pbShowTexture[u] |= Asura_Texture_Management::IsTextureCubemap(u);
		}
	}

	//count the number of textures we are going to be showing
	for(i = 0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
	{
		if(NameList[i])
		{
			if(pbShowTexture[i])
			{
				//add to the count of env textures
				iOutputNumTextures++;
			}
			else
			{
				//make the name null , so we don't have to do the check again
				//in our next search through
				NameList[i] = NULL;
			}
		}
	}

	delete [] pbShowTexture;

	if(!iOutputNumTextures)
	{
		delete [] NameList;
		return;
	}

	//allocate the array to store the texture info
	pxOutputThumbnailArray = new TextureNameAndID[iOutputNumTextures];
	TextureNameAndID* current = pxOutputThumbnailArray;

	//go through the name list again copying into our array
	for(i = 0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
	{
		if(NameList[i])
		{
			current->Name = *NameList[i];
			current->TextureID = i;
			current++;
		}
	}

	//sort the names
	qsort(pxOutputThumbnailArray,iOutputNumTextures,sizeof(TextureNameAndID),TextureSortCallback);

	delete [] NameList;
}

int Tools_PickTexture::PickTexture(const char* Message /*= 0*/,
								   int iCurrentSelection /*=ASURA_TEXTURE_NONE*/,
								   u_int uOptionFlags /*=PICKTEXTURE_DEFAULT*/,
								   int iInitialTextureFlags /*=0*/,
								   Asura_Hash_ID* puResult /*=NULL*/)
{
	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	//Temporary (in theory) fix to use the new texture picking overlay.
	u_int uModifiedFlags = uOptionFlags &~ PICKTEXTURE_MATERIALEDIT;
	if(uModifiedFlags == PICKTEXTURE_SKYBOX)
	{
		Tools_Overlay_ViewTextures::SetTextureUsageFilter(TOOLS_OVT_USAGE_SKYBOX);
	}
	else if(uModifiedFlags == PICKTEXTURE_CUBEMAP)
	{
		Tools_Overlay_ViewTextures::SetTextureUsageFilter(TOOLS_OVT_USAGE_CUBEMAP);
	}
	else if(uModifiedFlags == PICKTEXTURE_USECALLBACKFILTER)
	{
		Tools_Overlay_ViewTextures::SetTextureUsageFilter(TOOLS_OVT_USAGE_GUIMENU);
	}
	return Tools_Overlay_ViewTextures::PickTexture(Message, iCurrentSelection, iInitialTextureFlags, puResult);

	#else
	return s_pxThis->PickTexture_Main(Message,iCurrentSelection,uOptionFlags,iInitialTextureFlags,puResult);
	#endif

}

int Tools_PickTexture::PickTexture_Main(const char* Message ,int iCurrentSelection,u_int uOptionFlags,int iInitialTextureFlags,Asura_Hash_ID* puResult)
{

	//pause profiling to avoid assert
	ASURA_CPUPROFILE_SCOPED_PAUSE

	if(!Message) Message = "Choose a texture";
	
	int SelectedTextureID = ASURA_TEXTURE_NONE;
	if(puResult) *puResult = ASURA_HASH_ID_UNSET;

	PickTextureAlphaMode eAlphaMode= PICKTEXTURE_OPAQUE;

	bool loopAgain = false;
	do
	{
		loopAgain = false;

		int NumberOfThumbnails = 0;
		TextureNameAndID* ThumbnailArray = 0;
		BuildThumbnailArray(uOptionFlags,NumberOfThumbnails,ThumbnailArray);

		if(!NumberOfThumbnails)
		{
			if(m_bShowCurrentLightSetOnly)
			{
				//try again without limiting to the current light set
				m_bShowCurrentLightSetOnly = false;
				BuildThumbnailArray(uOptionFlags,NumberOfThumbnails,ThumbnailArray);
			}
			if(!NumberOfThumbnails)
			{
			//no suitable textures loaded yet ,so load from disk
				return PickTextureFromDisk(iInitialTextureFlags, puResult);
			}
		}

		const int iReservedHeightAboveMainImage = 20;//point where large image starts
		const int iReservedHeightBelowMainImage = 28;//space reserved for text below main image
		const int iTotalReservedHeight = iReservedHeightAboveMainImage + iReservedHeightBelowMainImage;

		//get the width and height allocated for thumbails
		int width_for_tn = Asura_Viewport::GetWidth();
		int height_for_tn = ((Asura_Viewport::GetHeight()-iTotalReservedHeight)*2)/3;

		//determine the largest size we can make the thumbnails
		int ThumbnailSize;
		for(ThumbnailSize = 1;ThumbnailSize<height_for_tn/3;ThumbnailSize++)
		{
			if((width_for_tn/ThumbnailSize)*(height_for_tn/ThumbnailSize)<NumberOfThumbnails) break;
		}
		ThumbnailSize--;
		int ThumbnailsPerRow = width_for_tn / ThumbnailSize;
		int ThumbnailStartY = (Asura_Viewport::GetHeight()-iTotalReservedHeight)/3 + iTotalReservedHeight;


		//count the number of polygons each texture is used by
		int* piTextureUsage = new int[ASURA_MAX_NO_OF_TEXTURES];
		memset(piTextureUsage,0,sizeof(int)*ASURA_MAX_NO_OF_TEXTURES);

		for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
		{
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
			if(!pxMesh->TextureDataPtr) continue;

			for(int j=0;j<pxMesh->NumberOfPolygons;j++)
			{
				Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[j];
				if(pxTex->m_iMaterialID  == ASURA_MATERIAL_NONE) continue;

				Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(pxTex->m_iMaterialID);

				for(int iChan=0;iChan<ASURA_MATERIAL_MAXTEXTURES;iChan++)
				{
					if(pxMaterial->m_iTextureID[iChan] != ASURA_TEXTURE_NONE)
					{
						piTextureUsage[pxMaterial->m_iTextureID[iChan]]++;
					}

					if(	pxMaterial->m_iNormalMapID[iChan] != ASURA_TEXTURE_NONE &&
						pxMaterial->m_iNormalMapID[iChan] != pxMaterial->m_iTextureID[iChan])
					{
						piTextureUsage[pxMaterial->m_iNormalMapID[iChan]]++;
					}

					if(	pxMaterial->m_iSpecularMapID[iChan] != ASURA_TEXTURE_NONE &&
						pxMaterial->m_iSpecularMapID[iChan] != pxMaterial->m_iTextureID[iChan])
					{
						piTextureUsage[pxMaterial->m_iSpecularMapID[iChan]]++;
					}
				}
			}
		}


		//make sure that we aren't in wireframe mode
		Asura_Render::CurrentStates.RequestWireFrameMode(false);
	
		//now for our main loop
		do
		{
			Asura_Windows::CheckForWindowsMessages();
			Asura_UserInput::Update();
			Asura_Cursor::OncePerFrameUpdate();

			//get the current mouse position
			int mx,my;
			Asura_Cursor::GetScreenSpaceCoords(mx,my);


			//work out which image the mouse is over
			int MouseOverIndex = -1;
			if(mx<ThumbnailSize * ThumbnailsPerRow && my>=ThumbnailStartY)
			{
				MouseOverIndex = mx/ThumbnailSize + ((my-ThumbnailStartY)/ThumbnailSize)*ThumbnailsPerRow;
				if(MouseOverIndex>=NumberOfThumbnails)
				{
					MouseOverIndex = -1;	
				}
			}

			//check input...
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) && MouseOverIndex>=0)
			{
				//select this one
				SelectedTextureID = ThumbnailArray[MouseOverIndex].TextureID;
				if(puResult)
				{
					*puResult = Asura_Texture_Management::GetHashIDFromIndex(SelectedTextureID);
				}
				break;
			}
			if(Asura_UserInput::DebouncedKeyPressed('l'))
			{
				//load a new texture
				SelectedTextureID = PickTextureFromDisk(iInitialTextureFlags, puResult);
				break;
			}
			if(Asura_UserInput::DebouncedKeyPressed('d'))
			{
				//replace textures with those from another directory
				ReplaceTextures();
				//need to break out and generate the list of names again
				loopAgain = true;
				break;
			}
			if(Asura_UserInput::DebouncedKeyPressed('i') && Tools_LightSets::IsUsingLightSets())
			{
				m_bShowCurrentLightSetOnly = !m_bShowCurrentLightSetOnly;
				//need to break out and generate the list of names again
				loopAgain = true;
				break;
			}
			if(Asura_UserInput::DebouncedKeyPressed('m') && MouseOverIndex>=0 && (uOptionFlags & PICKTEXTURE_MATERIALEDIT))
			{
				//edit materials
				ShowEditMaterialOverlay(ThumbnailArray[MouseOverIndex].TextureID);
			}


			if(Asura_UserInput::DebouncedKeyPressed('r') && MouseOverIndex>=0)
			{
				//destroy this texture , so that it wil be reloaded
				Asura_Texture_Management::ReloadTexture(ThumbnailArray[MouseOverIndex].TextureID);
				//Asura_Texture_Management::LoadTextures();
				Tools_Timed_Message_System::AddMessage(0xffffff,"Reloading %s",ThumbnailArray[MouseOverIndex].Name.GetString());
			}

			//F-Keys change alpha mode
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F1))
			{
				eAlphaMode = PICKTEXTURE_OPAQUE;
			}
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F2))
			{
				eAlphaMode = PICKTEXTURE_TRANSPARENT_BLACK;
			}
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F3))
			{
				eAlphaMode = PICKTEXTURE_TRANSPARENT_MAGENTA;
			}

			//do the rendering
			Asura_Render::BeginFrame();
			float width = (float)Asura_Viewport::GetWidth();
			float height = (float)Asura_Viewport::GetHeight();
			//first draw a big black poly over everything that has taken place so far...
			Asura_Render::Overlay.SetRectXY(0,0,width,height);
			Asura_Render::Overlay.SetRectUV(0,0,1,1);
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
			Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.Colour=0xff000000;
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();


			//draw each of the thumbnails
			float border = max(ThumbnailSize/20.0f,1.0f);
			for(int i=0;i<NumberOfThumbnails;i++)
			{
				//get the position
				float x = (float)(i%ThumbnailsPerRow)*ThumbnailSize;
				float y = (float)(i/ThumbnailsPerRow)*ThumbnailSize;
				y += (float)ThumbnailStartY;

				if(ThumbnailArray[i].TextureID == iCurrentSelection)
				{
					//draw a red background to show the texture that was previously selected
					Asura_Render::Overlay.SetRectXY(x,y,(float)ThumbnailSize,(float)ThumbnailSize);
					Asura_Render::Overlay.SetRectUV(0,0,1,1);
					Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
					Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
					Asura_Render::Overlay.Colour=0xffff0000;
					Asura_Render::Overlay.Render();
					Asura_Render::FlushDisplay();

				}
				
				RenderTexture(ThumbnailArray[i].TextureID,eAlphaMode,x+border,y+border,ThumbnailSize-border*2.0f,ThumbnailSize-border*2.0f);
						
			}

   			float print_x;
   			float print_y;

			if(MouseOverIndex!=-1)
			{
				Asura_Assert(MouseOverIndex<NumberOfThumbnails,"Index out of range");

				//determine the size to draw the larger image
				float size = (height-iTotalReservedHeight)/3;
				border = size/20.0f;

				RenderTexture(ThumbnailArray[MouseOverIndex].TextureID,eAlphaMode,border,border+20,size-border*2.0f,size-border*2.0f);

				//show the name of the image
				print_x = 0;
				print_y = size+20;
				Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
				//print_y -= 0.5f;
				
		
				int iTextureWidth;
				int iTextureHeight;
				if(Tools_Texture_Management::GetTextureSize(ThumbnailArray[MouseOverIndex].TextureID,iTextureWidth,iTextureHeight))
				{
					Asura_Text_Output::Print(print_x,print_y,0xffffffff,0,"%dx%d",iTextureWidth,iTextureHeight);
				}

				Asura_String xPath,xName,xExt;
				ThumbnailArray[MouseOverIndex].Name.SplitFilename(&xPath,&xName,&xExt);
				
				Asura_Text_Output::Print(print_x+11,print_y,0xffffffff,0,"%s.%s",xName.GetString(),xExt.GetString());
				if(xPath.GetString())
				{
					Asura_Text_Output::Print(print_x+11,print_y+1,0xffffffff,0,"%s",xPath.GetString());
				}

				print_x = Asura_Viewport::GetWidth()-150.0f;
				print_y = size+20;
				Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);

				int iPolygons = piTextureUsage[ThumbnailArray[MouseOverIndex].TextureID];
				Asura_Text_Output::Print(print_x,print_y,0xffffffff,0,"Polys: %d",iPolygons);

				
			}
			Asura_Render::FlushDisplay();

			//show some instruction
			print_x = width/2;
			print_y = 0;
			Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
			print_y = 2.0f;
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Left click to choose a texture");
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"R : Reload texture mouse is over");
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"D : Change directory");
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"L : Load a new texture");
			if(uOptionFlags & PICKTEXTURE_MATERIALEDIT)
			{
				Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"M : Edit Materials");
			}
			if(Tools_LightSets::IsUsingLightSets())
			{
				Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"I : Filter by light set (%s)",m_bShowCurrentLightSetOnly ? "On" : "Off");
			}
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"F1-F3 : Various display modes");
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Escape : Cancel");

			//show the message passed to the function
			Asura_Text_Output::Print(print_x,0.0f,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,Message);
			
				
			Tools_Timed_Message_System::Render();

			Asura_Cursor::SetType(ASURA_CURSOR_CROSSHAIR);
			Asura_Cursor::Render();
			Asura_Render::EndFrame();
	
		}
		while(!Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE));

		delete [] ThumbnailArray;

		delete [] piTextureUsage;

	}
	while(loopAgain);

	//apply any default texture flags. RAR 08/05/2006
	if(iInitialTextureFlags && (SelectedTextureID != ASURA_TEXTURE_NONE))
	{
		int iTextureFlags = Asura_Texture_Management::GetTextureFlags(SelectedTextureID);
		Asura_Texture_Management::SetTextureFlags(SelectedTextureID,iTextureFlags | iInitialTextureFlags);
	}

	return SelectedTextureID;

}


void Tools_PickTexture::EditTextureFlags(u_int uOptionFlags, bool* pbModifiedFlag)
{
	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	//Temporary (in theory) fix to use the new texture picking overlay.
	u_int uModifiedFlags = uOptionFlags &~ PICKTEXTURE_MATERIALEDIT;
	if(uModifiedFlags == PICKTEXTURE_SKYBOX)
	{
		Tools_Overlay_ViewTextures::SetTextureUsageFilter(TOOLS_OVT_USAGE_SKYBOX);
	}
	else if(uModifiedFlags == PICKTEXTURE_CUBEMAP)
	{
		Tools_Overlay_ViewTextures::SetTextureUsageFilter(TOOLS_OVT_USAGE_CUBEMAP);
	}
	else if(uModifiedFlags == PICKTEXTURE_USECALLBACKFILTER)
	{
		Tools_Overlay_ViewTextures::SetTextureUsageFilter(TOOLS_OVT_USAGE_GUIMENU);
	}
	Tools_Overlay_ViewTextures::CreateOverlay();
	#else

	#ifdef ASURA_TARGET_SM3
	// DMR 17/05/07
	// Textures only referenced by the material system weren't showing up as flaggable... this is a temporary workaround				
	s_pxThis->EditTextureFlags_Main(PICKTEXTURE_ALLTEXTURES, pbModifiedFlag);
	#else
	s_pxThis->EditTextureFlags_Main(uOptionFlags, pbModifiedFlag);
	#endif				
	#endif
}

void Tools_PickTexture::EditTextureFlags_Main(u_int uOptionFlags, bool* pbModifiedFlag)
{
	//pause profiling to avoid assert
	ASURA_CPUPROFILE_SCOPED_PAUSE

	PickTextureAlphaMode eAlphaMode= PICKTEXTURE_OPAQUE;

	bool bLoopAgain;
	do
{
		bLoopAgain = false;

	int NumberOfThumbnails = 0;
	TextureNameAndID* ThumbnailArray = 0;
	BuildThumbnailArray(uOptionFlags,NumberOfThumbnails,ThumbnailArray);


	if(!NumberOfThumbnails)
	{
			if(m_bShowCurrentLightSetOnly)
			{
				//try again without limiting to the current light set
				m_bShowCurrentLightSetOnly = false;
				BuildThumbnailArray(uOptionFlags,NumberOfThumbnails,ThumbnailArray);
			}
			if(!NumberOfThumbnails)
			{
		//no textures , so forget it
		Tools_Timed_Message_System::AddMessage(0xff0000,"No textures have been loaded yet");
		return;
	}
		}


	const int iReservedHeightAboveMainImage = 20;//point where large image starts
	const int iReservedHeightBelowMainImage = 28;//space reserved for text below main image
	const int iTotalReservedHeight = iReservedHeightAboveMainImage + iReservedHeightBelowMainImage;

	//get the width and height allocated for thumbails
	int width_for_tn = Asura_Viewport::GetWidth();
	int height_for_tn = ((Asura_Viewport::GetHeight()-iTotalReservedHeight)*2)/3;

	//determine the largest size we can make the thumbnails
	int ThumbnailSize;
	for(ThumbnailSize = 1;ThumbnailSize<height_for_tn/3;ThumbnailSize++)
	{
		if((width_for_tn/ThumbnailSize)*(height_for_tn/ThumbnailSize)<NumberOfThumbnails) break;
	}
	ThumbnailSize--;
	int ThumbnailsPerRow = width_for_tn / ThumbnailSize;
	int ThumbnailStartY = (Asura_Viewport::GetHeight()-iTotalReservedHeight)/3 + iTotalReservedHeight;


	char** ppszFlagNames = m_bShowingTextureFlags ? m_pszTextureFlagNames : s_apszMaterialFlagNames;
	int* piFlagValues = m_bShowingTextureFlags ? m_piTextureFlagValues : s_aiMaterialFlagValues;
	int iNumFlags = m_bShowingTextureFlags ? m_uNumberOfTextureFlags : NO_OF_MATERIAL_FLAGS; 
	int* piFlagIndex = m_bShowingTextureFlags ?  &(m_iCurrentTextureFlagIndex) : &m_iCurrentMaterialFlagIndex;

	int iCurrentFlagValue = piFlagValues[*piFlagIndex];

	
	//make sure that we aren't in wireframe mode
	Asura_Render::CurrentStates.RequestWireFrameMode(false);
	
	//now for our main loop
	do
	{
		Asura_Windows::CheckForWindowsMessages();
		Asura_UserInput::Update();
		Asura_Cursor::OncePerFrameUpdate();

		//get the current mouse position
		int mx,my;
		Asura_Cursor::GetScreenSpaceCoords(mx,my);


		//work out which image the mouse is over
		int MouseOverIndex = -1;
		if(mx<ThumbnailSize * ThumbnailsPerRow && my>=ThumbnailStartY)
		{
			MouseOverIndex = mx/ThumbnailSize + ((my-ThumbnailStartY)/ThumbnailSize)*ThumbnailsPerRow;
			if(MouseOverIndex>=NumberOfThumbnails)
			{
				MouseOverIndex = -1;	
			}
		}

		//check input...
		if(Asura_UserInput::DebouncedKeyPressed('f') && MouseOverIndex>=0)
		{
			//add or remove the current flag
				int iTextureID = ThumbnailArray[MouseOverIndex].TextureID;
			if(m_bShowingTextureFlags)
			{
					SetTextureFlag(iTextureID,iCurrentFlagValue,!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT));
				}
				else
				{
					SetMaterialFlag(iTextureID, iCurrentFlagValue,!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT));
				}

				if(pbModifiedFlag)
				{
					*pbModifiedFlag = true;					
				}
				else if(Tools_SkyboxOverlay::IsSkyboxTexture(iTextureID) || Tools_SkyboxOverlay::IsReflectanceTexture(iTextureID))
				{
					Tools_SaveAsura::SetSkyboxModified();
			}
			else
			{
					Tools_SaveAsura::SetMainFileModified();
			}
		}

		if(Asura_UserInput::DebouncedKeyPressed('m') && MouseOverIndex>=0 && (uOptionFlags & PICKTEXTURE_MATERIALEDIT))
		{
			//edit materials
			ShowEditMaterialOverlay(ThumbnailArray[MouseOverIndex].TextureID);
		}


		if(Asura_UserInput::DebouncedKeyPressed('='))
		{
			(*piFlagIndex)++;
			if(*piFlagIndex>=iNumFlags) *piFlagIndex = 0;
			iCurrentFlagValue = piFlagValues[*piFlagIndex];

		}
		if(Asura_UserInput::DebouncedKeyPressed('-'))
		{
			(*piFlagIndex)--;
			if(*piFlagIndex<0) *piFlagIndex = iNumFlags-1;
			iCurrentFlagValue = piFlagValues[*piFlagIndex];
		}
			if(Asura_UserInput::DebouncedKeyPressed('i') && Tools_LightSets::IsUsingLightSets())
			{
				m_bShowCurrentLightSetOnly = !m_bShowCurrentLightSetOnly;
				//need to break out and generate the list of names again
				bLoopAgain = true;
				break;
			}


		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_TAB) && (uOptionFlags & PICKTEXTURE_MATERIALEDIT))
		{
			//Toggle between showing material and texture flags
			m_bShowingTextureFlags = !m_bShowingTextureFlags;
			
			ppszFlagNames = m_bShowingTextureFlags ? m_pszTextureFlagNames : s_apszMaterialFlagNames;
			piFlagValues = m_bShowingTextureFlags ? m_piTextureFlagValues : s_aiMaterialFlagValues;
			iNumFlags = m_bShowingTextureFlags ? m_uNumberOfTextureFlags : NO_OF_MATERIAL_FLAGS; 
			piFlagIndex = m_bShowingTextureFlags ?  &(m_iCurrentTextureFlagIndex) : &m_iCurrentMaterialFlagIndex;
			iCurrentFlagValue = piFlagValues[*piFlagIndex];
		}

		//F-Keys change alpha mode
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F1))
		{
			eAlphaMode = PICKTEXTURE_OPAQUE;
		}
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F2))
		{
			eAlphaMode = PICKTEXTURE_TRANSPARENT_BLACK;
		}
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_F3))
		{
			eAlphaMode = PICKTEXTURE_TRANSPARENT_MAGENTA;
		}



		//do the rendering
		Asura_Render::BeginFrame();
		float width = (float)Asura_Viewport::GetWidth();
		float height = (float)Asura_Viewport::GetHeight();
		//first draw a big black poly over everything that has taken place so far...
		Asura_Render::Overlay.SetRectXY(0,0,width,height);
		Asura_Render::Overlay.SetRectUV(0,0,1,1);
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
		Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
		Asura_Render::Overlay.Colour=0xff000000;
		Asura_Render::Overlay.Render();
		Asura_Render::FlushDisplay();


		//draw each of the thumbnails
		float border = max(ThumbnailSize/20.0f,1.0f);
		for(int i=0;i<NumberOfThumbnails;i++)
		{
			//get the position
			float x = (float)(i%ThumbnailsPerRow)*ThumbnailSize;
			float y = (float)(i/ThumbnailsPerRow)*ThumbnailSize;
			y += (float)ThumbnailStartY;

			bool bOutline = false;
			u_int uOutlineColour = 0;
			
			//determine if the flag is used by the current texture , and if so draw a red outline
			if(m_bShowingTextureFlags)
			{
				if(Asura_Texture_Management::GetTextureFlags(ThumbnailArray[i].TextureID) & iCurrentFlagValue)
				{
					bOutline = true;
					uOutlineColour = 0xffff0000;
				}
			}
			else
			{
				bool bFlagUsed,bFlagUsedByAll;
				GetMaterialFlagUseage(ThumbnailArray[i].TextureID,iCurrentFlagValue,&bFlagUsed,&bFlagUsedByAll);
				if(bFlagUsed)
				{
					bOutline = true;
					uOutlineColour = bFlagUsedByAll ? 0xffff0000 : 0xff7f0000;
				}
			}
			
			if(bOutline)
			{
				Asura_Render::Overlay.SetRectXY(x,y,(float)ThumbnailSize,(float)ThumbnailSize);
				Asura_Render::Overlay.SetRectUV(0,0,1,1);
				Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
				Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
				Asura_Render::Overlay.Colour=uOutlineColour;
				Asura_Render::Overlay.Render();
				Asura_Render::FlushDisplay();

			}
			
			RenderTexture(ThumbnailArray[i].TextureID,eAlphaMode,x+border,y+border,ThumbnailSize-border*2.0f,ThumbnailSize-border*2.0f);
			
					
		}

   		float print_x;
   		float print_y;

		if(MouseOverIndex!=-1)
		{
			Asura_Assert(MouseOverIndex<NumberOfThumbnails,"Index out of range");

			//determine the size to draw the larger image
			float size = (height-iTotalReservedHeight)/3;
			border = size/20.0f;

			RenderTexture(ThumbnailArray[MouseOverIndex].TextureID,eAlphaMode,border,border+20,size-border*2.0f,size-border*2.0f);

			//show the name of the image
			print_x = 0;
			print_y = size+20;
			Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
			//print_y -= 0.5f;
			
	
			int iTextureWidth;
			int iTextureHeight;
			if(Tools_Texture_Management::GetTextureSize(ThumbnailArray[MouseOverIndex].TextureID,iTextureWidth,iTextureHeight))
			{
				Asura_Text_Output::Print(print_x,print_y,0xffffffff,0,"%dx%d",iTextureWidth,iTextureHeight);
			}

			Asura_String xPath,xName,xExt;
			ThumbnailArray[MouseOverIndex].Name.SplitFilename(&xPath,&xName,&xExt);
			
			Asura_Text_Output::Print(print_x+11,print_y,0xffffffff,0,"%s.%s",xName.GetString(),xExt.GetString());
			if(xPath.GetString())
			{
				Asura_Text_Output::Print(print_x+11,print_y+1,0xffffffff,0,"%s",xPath.GetString());
			}
			
			
		}
		Asura_Render::FlushDisplay();


		//show some instruction
		print_x = width-300;
		print_y = 0;
		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		print_y += 2.0f;
		Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"F : Apply flag");
		Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"Shift+F : Remove Flag");
		Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"+/- : Select flag");
		if(uOptionFlags & PICKTEXTURE_MATERIALEDIT)
		{
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"Tab : Toggle Material/Texture Flags");
			Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"M : Edit Materials");
		}
			if(Tools_LightSets::IsUsingLightSets())
			{
				Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"I : Filter by light set (%s)",m_bShowCurrentLightSetOnly ? "On" : "Off");
			}

		Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"F1-F3 : Various display modes");
		Asura_Text_Output::Print(print_x,print_y++,0xff00ff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"Escape : Finish flag editing");

		print_x = width/2;
		print_y = 0;
		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		//show the message passed to the function
		Asura_Text_Output::Print(print_x,0.0f,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,m_bShowingTextureFlags ? "Texture Flags" : "Material Flags");

		//show the status text
		print_x = (height-40)/3;
		print_y = 0;
		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		print_y = 2.0f;

		Asura_Text_Output::Print(print_x,print_y++,0xffff0000,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"Current Flag : %s",ppszFlagNames[*piFlagIndex]);
		if(MouseOverIndex!=-1)
		{
			if(m_bShowingTextureFlags)
			{
				Asura_Text_Output::Print(print_x,print_y++,0xffffff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"Flags On Texture :");
				print_x++;
				int flags = Asura_Texture_Management::GetTextureFlags(ThumbnailArray[MouseOverIndex].TextureID);
				if(flags)
				{
					for(u_int u=0;u<m_uNumberOfTextureFlags;u++)
					{
						if(flags & m_piTextureFlagValues[u])
						{
							Asura_Text_Output::Print(print_x,print_y++,0xffffff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"%s",m_pszTextureFlagNames[u]);
						}	
					}
				}
				else
				{
					Asura_Text_Output::Print(print_x,print_y++,0xffffff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"(None)");
				}
			}
			else
			{
				Asura_Text_Output::Print(print_x,print_y++,0xffffff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"Flags Used By Material(s) :");
				print_x++;
				
				int iFlagCount = 0;
				for(int i=0;i<NO_OF_MATERIAL_FLAGS;i++)
				{
					bool bFlagUsed,bFlagUsedByAll;
					GetMaterialFlagUseage(ThumbnailArray[MouseOverIndex].TextureID,s_aiMaterialFlagValues[i],&bFlagUsed,&bFlagUsedByAll);
					if(bFlagUsed)
					{
						iFlagCount++;
						Asura_Text_Output::Print(print_x,print_y++,bFlagUsedByAll ? 0xffffff00 : 0xff7f7f00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"%s",s_apszMaterialFlagNames[i]);
							
					}
					
				}
				if(!iFlagCount)
				{
					Asura_Text_Output::Print(print_x,print_y++,0xffffff00,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"(None)");
				}
			}
		}
		

		
			
		Tools_Timed_Message_System::Render();

		Asura_Cursor::SetType(ASURA_CURSOR_CROSSHAIR);
		Asura_Cursor::Render();
		Asura_Render::EndFrame();
	
	}
	while(!Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE));

	delete [] ThumbnailArray;

	}while(bLoopAgain);
	
}

void Tools_PickTexture::SetTextureFlag(int textureIndex,int flag,bool add)
{
	int iOldFlags = Asura_Texture_Management::GetTextureFlags(textureIndex);
	int iNewFlags = iOldFlags;
	if(add)
	{
		iNewFlags |= flag;
	}
	else
	{
		iNewFlags &=~ flag;
	}
	//some iNewFlags may need other things to be done when set
	//if(add && (flag & AsuraTextureFlag_SelfIlluminate))
	//{
		//add 8 bit alpha as well
	//	iNewFlags |= AsuraTextureFlag_8BitAlpha;
	//}

	//deal with sily flag combinations
	if( add && flag == AsuraTextureFlag_PS2DontHalveSize)
	{
		iNewFlags &=~ AsuraTextureFlag_ReduceExtraForConsoles;
	}

	if( add && flag == AsuraTextureFlag_ReduceExtraForConsoles)
	{
		iNewFlags &=~ AsuraTextureFlag_PS2DontHalveSize;
	}


	Asura_Texture_Management::SetTextureFlags(textureIndex,iNewFlags);

	//force texture to be reloaded if the compression settings change
	const int iChangedFlags = iOldFlags ^ iNewFlags;
	const int iFlagsToCheck = AsuraTextureFlag_HighQualityCompression | AsuraTextureFlag_NoMipMaps | AsuraTextureFlag_SharpenMipMaps;
	if( iChangedFlags & iFlagsToCheck )
	{
		Asura_Texture_Management::ReloadTexture(textureIndex);
	}

	}

void Tools_PickTexture::SetMaterialFlag(int iTextureID,int iFlag,bool bAdd)
{
	//search for all materials using the texture
	for(LIF<int> xMaterialLIF(&m_xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
		if(Asura_Material_System::GetTextureID(xMaterialLIF()) == iTextureID)
		{
			//add remove/flag from this material
			Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(xMaterialLIF());
			if(bAdd)
			{
				pxMaterial->m_uMaterialFlags |= iFlag;
			}
			else
			{
				pxMaterial->m_uMaterialFlags &=~ iFlag;
			}
			
		}
	}
}

void Tools_PickTexture::GetMaterialFlagUseage(int iTextureID,int iFlag,bool* pbFlagUsed,bool* pbFlagUsedByAll)
{
	if(pbFlagUsed)* pbFlagUsed = false;	
	if(pbFlagUsedByAll)* pbFlagUsedByAll = true;

	//search for all materials using the texture
	for(LIF<int> xMaterialLIF(&m_xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
		if(Asura_Material_System::GetTextureID(xMaterialLIF()) == iTextureID)
		{
			//check for the flag
			Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(xMaterialLIF());
			if(pxMaterial->m_uMaterialFlags & iFlag)
			{
				//flag used by at least one material
				if(pbFlagUsed)* pbFlagUsed = true;	
			}
			else
			{
				//flag not used by at least one material
				if(pbFlagUsedByAll)* pbFlagUsedByAll = false;
			}
			
		}
	}
}


#if 0
void Tools_PickTexture::SelectTextureFlag()
{
	ListSelection_Overlay overlay("Select Texture Flag");

	for(int i=0;i<m_uNumberOfTextureFlags;i++)
	{
		overlay.AddItem(TextureFlagNames[i],i);
	}

	//show the overlay
	if(!overlay.DoModal()) return;

	m_iCurrentTextureFlagIndex = overlay.GetSelectionData();
}
#endif


void Tools_PickTexture::ReplaceTextures()
{
	//first need to ask for the directory to get textures from
	Tools_FileSystem_SelectFolderInfo info;
	char startDirectory[MAX_PATH];
	GetCurrentDirectory(MAX_PATH,startDirectory);
	strcat(startDirectory,"\\graphics");

	info.StartDirectory = startDirectory;
	info.DisplayText = "Pick the directory to use for texture replacements";

	if(!Tools_FileSystem::SelectFolder(info)) return;

	Asura_Assert(strlen(info.SelectedFolder)>=strlen(startDirectory),"");

	//next get the list of textures
	//create an array to get the texture names
	const Asura_String** NameList = new const Asura_String*[ASURA_MAX_NO_OF_TEXTURES];
	//initialise array
	for(int i=0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
	{
		NameList[i] = NULL;	
	}

	//and get those names
	Asura_Texture_Management::MakeNameList(NameList);



	//check each texture to see if it exists in the new directory
	for(int i=0;i<ASURA_MAX_NO_OF_TEXTURES;i++)
	{
		if(!NameList[i]) continue;
		//only interested in environment textures
		if(!TextureIsUsedByMaterials(i)) continue;

		//get the name/extension without the path
		Asura_String filename;
		Asura_String extension;
		NameList[i]->SplitFilename(NULL,&filename,&extension);

		//build the filename in the new directory
		char fullFilename[MAX_PATH];
		sprintf(fullFilename,"%s\\%s.%s",info.SelectedFolder,filename.GetString(),extension.GetString());
	

		//see if the file exists
		if(GetFileAttributes(fullFilename)==0xffffffff)continue;

		//get the filename relative to the graphics directory
		strlwr(fullFilename);
		Asura_String relativeName = &fullFilename[strlen(startDirectory)];

		if(Asura_Texture_Management::RenameTexture(i,relativeName.GetString()))
		{
			//force the texture to be reloaded
			Asura_Texture_Management::ReloadTexture(i);
		}
	}

	//update the stored names for skybox textures
	for(int i=0;i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES;i++)
	{
		Asura_String xFullFilename = Asura_Skybox::GetTexture(i);
		if(xFullFilename.IsBlank()) continue;

		//get the name/extension without the path
		Asura_String xFilename;
		Asura_String xExtension;
		xFullFilename.SplitFilename(NULL,&xFilename,&xExtension);

		//build the filename in the new directory
		char fullFilename[MAX_PATH];
		sprintf(fullFilename,"%s\\%s.%s",info.SelectedFolder,xFilename.GetString(),xExtension.GetString());

		//see if the file exists
		if(GetFileAttributes(fullFilename)==0xffffffff)continue;

		//get the filename relative to the graphics directory
		strlwr(fullFilename);
		Asura_String xRelativeName = &fullFilename[strlen(startDirectory)];

		//set the new skybox texture
		Asura_Skybox::SetTexture(xRelativeName.GetString(),i);
		Tools_SaveAsura::SetSkyboxModified();

		
	}

	delete [] NameList;
}

int Tools_PickTexture::GetNextMaterial(int iMaterialID)
{
	return s_pxThis->GetNextMaterial_Main(iMaterialID);
}

int Tools_PickTexture::GetNextMaterial_Main(int iMaterialID)
{
	if(!m_xMaterialList.size()) return ASURA_MATERIAL_NONE;

	for(LIF<int> xMaterialLIF(&m_xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
		if(xMaterialLIF() == iMaterialID)
		{
			xMaterialLIF.next();
			if(xMaterialLIF.done()) xMaterialLIF.restart();

			return xMaterialLIF();
		}
	}
	//failed to find material in list , just choose the next material arbitrarily
	return m_xMaterialList.first_entry();
}

int Tools_PickTexture::GetPreviousMaterial(int iMaterialID)
{
	return s_pxThis->GetPreviousMaterial_Main(iMaterialID);
}

int Tools_PickTexture::GetPreviousMaterial_Main(int iMaterialID)
{
	if(!m_xMaterialList.size()) return ASURA_MATERIAL_NONE;

	for(LIB<int> xMaterialLIB(&m_xMaterialList);!xMaterialLIB.done();xMaterialLIB.next())
	{
		if(xMaterialLIB() == iMaterialID)
		{
			xMaterialLIB.next();
			if(xMaterialLIB.done()) xMaterialLIB.restart();

			return xMaterialLIB();
		}
	}
	//failed to find material in list , just choose the previous material arbitrarily
	return m_xMaterialList.last_entry();
}

void Tools_PickTexture::AddToMaterialListMain(int iMaterialID)
{
	int iTextureID = Asura_Material_System::GetTextureID(iMaterialID);
	if(iTextureID == ASURA_TEXTURE_NONE) return;

	//add material to list so that the texture IDs are in ascending order
	//sort in order of material ID for each texture
	for(LIF<int> xMaterialLIF(&m_xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
		if(xMaterialLIF() == iMaterialID) return;

		int iTextureID2 = Asura_Material_System::GetTextureID(xMaterialLIF());
		if((iTextureID2 > iTextureID) || ((iTextureID2 == iTextureID) && (xMaterialLIF() > iMaterialID)))
		{
			m_xMaterialList.add_entry_before(iMaterialID,xMaterialLIF());
			return;
		}
	}
	m_xMaterialList.add_entry(iMaterialID);
}

bool Tools_PickTexture::TextureIsInMaterialList(int iTextureID)
{
	for(LIF<int> xMaterialLIF(&m_xMaterialList);!xMaterialLIF.done();xMaterialLIF.next())
	{
		for(int iChannel=0;iChannel < ASURA_MATERIAL_MAXTEXTURES;iChannel++)
		{
			if(Asura_Material_System::GetTextureID(xMaterialLIF(),iChannel) == iTextureID)
			{
				return true;
			}
			if(Asura_Material_System::GetNormalMapID(xMaterialLIF(),iChannel) == iTextureID)
			{
				return true;
			}
		}
	}
	return false;
}

void Tools_PickTexture::ShowEditMaterialOverlay(int iTextureID)
{
	Tools_Material_Overlay xOverlay(iTextureID);
	xOverlay.DoModal();
}

bool Tools_PickTexture::CreateOrFindExistingMaterial(const Asura_Material* pxDesiredMat,bool bCompareNames,int& iOutputMatID,bool* pbIsNew)
{

	//search for an existing material that will do
	for(CLIF<int> xMatLIF(GetMaterialList()) ; !xMatLIF.done();xMatLIF.next())
	{
		const Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(xMatLIF());
		Asura_Assert(pxMaterial,"");

		if(Asura_Material::MaterialsAreEquivalent(pxDesiredMat,pxMaterial,bCompareNames))
		{
			//this material works
			iOutputMatID = xMatLIF();
			if(pbIsNew) *pbIsNew = false;
			return true;
		}
	}

	//okay... need to create a new material then
	int iMaterialID = Asura_Material_System::CreateMaterial();
	if(iMaterialID == ASURA_MATERIAL_NONE) return false;

	//initialise it
	Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(iMaterialID);
	*pxMaterial = *pxDesiredMat;

	AddToMaterialList(iMaterialID);

	iOutputMatID = iMaterialID;
	if(pbIsNew) *pbIsNew = true;

	return true;
}

void Tools_PickTexture::EditMaterial(int iMaterialID)
{
	Asura_Assert(s_pxThis,"");

	if(!iMaterialID)
	{
		ShowMessageOverlay("Can't edit 'no material'");
		return;
	}
	//set the material that is being edited
	Tools_Material_Overlay::SetInitiallySelectedMaterial(iMaterialID);

	//and show the overlay
	s_pxThis->ShowEditMaterialOverlay(Asura_Material_System::GetTextureID(iMaterialID,0));

}

bool Tools_PickTexture::TextureIsUsedByMaterials(int iTextureID)
{
	for(LIF<int> xMatLIF(&m_xMaterialList);!xMatLIF.done();xMatLIF.next())
	{
		int iMatID = xMatLIF();
		for(int iChan=0;iChan<2;iChan++)
		{
			if(Asura_Material_System::GetTextureID(iMatID,iChan) == iTextureID)
			{
				return true;
			}
			if(Asura_Material_System::GetNormalMapID(iMatID,iChan) == iTextureID)
			{
				return true;
			}
		}
	}
	return false;
}


void Tools_PickTexture::RenderTexture(int iTextureID,PickTextureAlphaMode eAlphaMode,float fX,float fY,float fWidth,float fHeight)
{
	//draw background
	u_int uColour = 0xff000000;
	if(eAlphaMode == PICKTEXTURE_TRANSPARENT_MAGENTA)
	{
		uColour = 0xffff00ff;

	}
	Asura_Render::Overlay.SetRectXY(fX,fY,fWidth,fHeight);
	Asura_Render::Overlay.SetRectUV(0,0,1,1);
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour=uColour;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	//draw the texture.
	//set the correct render modes
	switch(eAlphaMode)
	{
		default :
		case PICKTEXTURE_OPAQUE :
		{
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
		}
		break;
		case PICKTEXTURE_TRANSPARENT_BLACK :
		case PICKTEXTURE_TRANSPARENT_MAGENTA :
		{
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
		}
		break;
	}

	Asura_Render::Overlay.TextureIndex = iTextureID;
	Asura_Render::Overlay.Colour=0xffffffff;

	#if defined ASURA_TARGET_SM3

		if( Asura_Texture_Management::IsTextureCubemap( iTextureID ) )
		{
			Asura_Direct3D_Effect* const pxEffect = Asura_Direct3D_Effects_System::GetEffect( "OverlayQuad_Cubemap" );
			if( pxEffect )
			{
				// Since override effects might use vertex shaders, we have to pass in
				// untransformed vertices. That means taking them from screen-space to
				// [-1,1] clip space
				{
					const float fViewportWidth = static_cast< float >( Asura_Viewport::GetWidth() );
					const float fViewportHeight = static_cast< float >( Asura_Viewport::GetHeight() );

					fX = (fX / fViewportWidth) * 2.0f - 1.0f;
					fY = 1.0f - (fY / fViewportHeight) * 2.0f;
					fWidth = (fWidth / fViewportWidth) * 2.0f;
					fHeight = -(fHeight / fViewportHeight) * 2.0f;
				}

	Asura_Render::Overlay.SetRectXY(fX,fY,fWidth,fHeight);
				Asura_Render::Overlay.SetRectUV( 1.0f, 1.0f, -2.0f, -2.0f );
				Asura_Direct3D::AddQuadToBatch( &Asura_Render::Overlay, pxEffect );
			}
		}
		else

	#endif // defined ASURA_TARGET_SM3

	{
		Asura_Render::Overlay.SetRectXY(fX,fY,fWidth,fHeight);
	Asura_Render::Overlay.SetRectUV(0,0,1,1);
   	Asura_Render::Overlay.Render();
	}

   	Asura_Render::FlushDisplay();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------**
** Tools_TextureAnim_DropDownList_Dialog **
**---------------------------------------*/

Tools_TextureAnim_DropDownList_Dialog::Tools_TextureAnim_DropDownList_Dialog( Asura_Vector_2& xOffset, int iNumberOfLines, Asura_Hash_ID* puSelectedAnim, bool* pbChangedFlag /*= 0*/ ,const char* szName /*= NULL*/ )
:Asura_Hash_DropDownList_Dialog(xOffset)
{
	Initialise(xOffset,iNumberOfLines,puSelectedAnim,pbChangedFlag,szName);
}
	
u_int Tools_TextureAnim_DropDownList_Dialog::GetNumberOfHashValues() const
{
	return Asura_TextureAnimation_System::GetNumberOfAnimations();
}
	
const char* Tools_TextureAnim_DropDownList_Dialog::GetNameByIndex(int iIndex) const
{
	const Asura_TextureAnimation* pxAnim = Asura_TextureAnimation_System::GetAnimationByIndex(iIndex);
	return pxAnim ? pxAnim->GetName() : "*Error*";
}


/////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------**
** Tools_Material_Overlay **
**------------------------*/

Tools_Material_Overlay* Tools_Material_Overlay::s_pxThis = 0;
int Tools_Material_Overlay::s_iInitiallySelectedMaterial = ASURA_MATERIAL_NONE;

Tools_Material_Overlay::Tools_Material_Overlay(int iTextureID)
:Tools_Modal_Overlay("Edit Materials")
{
	m_iTextureID = iTextureID;
	m_iNumberOfMaterials = 0;
	m_ppszMaterialNames = new char*[ASURA_MAX_NO_OF_MATERIALS];
	m_iCurrentSelection = 0;
}

Tools_Material_Overlay::~Tools_Material_Overlay()
{
	ClearMaterialList();
	delete [] m_ppszMaterialNames;
}


void Tools_Material_Overlay::DoModal()
{
	UpdateMaterialList();
	if(!m_iNumberOfMaterials) return;

	if(s_iInitiallySelectedMaterial != ASURA_MATERIAL_NONE)
	{
		//search for the initial selection
		bool bFound = false;
		for(int i=0;i<m_iNumberOfMaterials;i++)
		{
			if(m_xMaterialList[i] == s_iInitiallySelectedMaterial)
			{
				m_iCurrentSelection = i;
				bFound = true;
			}
		}
		s_iInitiallySelectedMaterial = ASURA_MATERIAL_NONE;

		if(!bFound)
		{
			Asura_Assert(0,"Failed to find material that was supposed to be edited");
			return;
		}
	}
	
	s_pxThis = this;
	ShowOverlay();
	s_pxThis = 0;
}

void Tools_Material_Overlay::Update()
{
	//has a different material been selected?
	if(m_iCurrentSelection != m_iListOffset+m_iListSelection)
	{
		SelectMaterial(m_iListOffset+m_iListSelection);

	}

	//if the material properties have been altered , update it in the main material list
	if(m_bMaterialChanged)
	{
		UpdateMaterial();
	}

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE) || Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER))
	{
		Asura_Overlay* pxOverlay = GetOverlay();
		if(pxOverlay) pxOverlay->m_bShouldClose = true;
	}
}

void Tools_Material_Overlay::UpdateMaterial()
{
	//slightly ugly, but ensures texture anim material flag gets set appropriately
	m_xMaterial.SetTextureAnim(m_xMaterial.m_uAnimHash);

	int iMaterialID = m_xMaterialList[m_iCurrentSelection];
	Asura_Material_System::SetMaterial(iMaterialID,m_xMaterial);

	Tools_SaveAsura::SetMainFileModified();
	m_bMaterialChanged = false;
}

void Tools_Material_Overlay::UpdateMaterialFromMaterialSystem()
{
	int iMaterialID = m_xMaterialList[m_iCurrentSelection];
	m_xMaterial = *Asura_Material_System::GetMaterial(iMaterialID);
}

void Tools_Material_Overlay::SelectMaterial(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<m_iNumberOfMaterials,"");

	//set the offset and selection values so that this item is visible
	if(iIndex<m_iListOffset)
	{
		m_iListOffset = iIndex;
	}
	else if(iIndex<m_iListOffset + m_iNumberOfLines)
	{
		m_iListSelection = iIndex - m_iListOffset;		
	}
	else
	{
		m_iListOffset = iIndex+1-m_iNumberOfLines;
		m_iListSelection = m_iNumberOfLines-1;
	}
	m_iCurrentSelection = m_iListOffset + m_iListSelection;

	//update the copy of the material
	int iMaterialID = m_xMaterialList[m_iCurrentSelection];
	m_xMaterial = *Asura_Material_System::GetMaterial(iMaterialID);

	OnSelectionChange();
	UpdateOverlayControls();
}

bool Tools_Material_Overlay::CurrentMaterialIsSpecular()
{
	return (s_pxThis->m_xMaterial.m_uMaterialFlags & (AsuraMaterialFlag_Specular_Soft | AsuraMaterialFlag_Specular_Sharp)) != 0;
}

void Tools_Material_Overlay::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog* pxDialog;	
	m_iListOffset = 0;
	m_iListSelection = 0;
	m_bMaterialChanged = false;
	m_iNumberOfLines = 6;

	Asura_Assert(m_iCurrentSelection<m_xMaterialList.size(),"");
 	int iMaterialID = m_xMaterialList[m_iCurrentSelection];
 	m_xMaterial = *Asura_Material_System::GetMaterial(iMaterialID);

	//set list selection/offset to match current selection
	m_iListSelection = m_iCurrentSelection;
	if(m_iListSelection >= m_iNumberOfLines)
	{
		m_iListOffset = m_iListSelection+1-m_iNumberOfLines;
		m_iListSelection = m_iNumberOfLines - 1;
	}


	Asura_Dialog* pxListDialog = new Asura_DynamicList_Dialog(xOffset,m_ppszMaterialNames,&m_iListOffset,&m_iListSelection,m_iNumberOfLines,&m_iNumberOfMaterials,NULL);
	pxListDialog->SetHelpText("Selected material.");
	pxOverlay->AddDialog(pxListDialog);
	
	//put the next few controls to the right of the list box
	xOffset.x += 40.0f;
	pxDialog = new Asura_Function_Dialog("Create Material",xOffset,_CreateMaterial,(int)this);
	pxDialog->SetHelpText("Create a new material for this texture");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;

	pxDialog = new Asura_Function_Dialog("Rename Material",xOffset,_RenameMaterial,(int)this);
	pxDialog->SetHelpText("Rename the selected material");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;
	
	//and now move under the list box
	xOffset.x = 1.0f;
	xOffset.y=pxListDialog->m_xMaxCoord.y;
	xOffset.y+=1.0f;

	float fCheckboxStartY = xOffset.y;
	
	//(insert exciting properties here)
	pxDialog= new Asura_FlagTickbox_Dialog("Self Illuminate",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_SelfIlluminate,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Transparent",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_Transparent,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Use Detail Map",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_UseDetailMap,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Reflective",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_Reflective,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("FMV Texture",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_FMV,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Flowing Water",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_FlowingWater,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Waterfall",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_Waterfall,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Subdivided Polygon",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_SubdividePolygon,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Additive Alpha",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_PS2AdditiveAlpha,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("3d Decal",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_3dDecal,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Rain Splashes",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_RainSplashes,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Specular - soft highlights",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_Specular_Soft,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = pxDialog->m_xMaxCoord.x + 3.f;

	Tools_ARGBColourPicker_Dialog* pxColourPicker = NULL;
	pxDialog = pxColourPicker = new Tools_ARGBColourPicker_Dialog("Specular material colour", xOffset, Asura_Vector_2(2.f, 2.f), &m_xMaterial.m_uSpecularColour, &m_bMaterialChanged);
	pxColourPicker->SetModal(true);
	pxDialog->SetIsEnabledFunction(CurrentMaterialIsSpecular);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 1.f;
	xOffset.y+=1.5f;

	pxDialog= new Asura_FlagTickbox_Dialog("Specular - sharp highlights",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_Specular_Sharp,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;
    
	pxDialog= new Asura_FlagTickbox_Dialog("Alpha Decal (Not PS2)",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_AlphaDecal,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

#if defined( ASURA_TARGET_SM3 )
	pxDialog= new Asura_FlagTickbox_Dialog("Parallax Mapping",xOffset,&m_xMaterial.m_uMaterialFlags,AsuraMaterialFlag_ParallaxMapping,&m_bMaterialChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;
#endif

	float fCheckboxEndY = xOffset.y;

	xOffset.y = fCheckboxStartY;

	//add controls for showing the textures used
	xOffset.x = 23.0f;
	pxDialog = new Asura_Dialog("Textures :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;


	for(int i=0;i<ASURA_MATERIAL_MAXTEXTURES;i++)
	{
		xOffset.x = 21.0f;
		if(i)
		{
			Asura_Icon_Dialog* pxIcon = new Asura_Icon_Dialog(xOffset,Tools_Dialog::GetIconTexture(),TOOLS_ICON_DELETE);
			pxIcon->SetClickedFunction(_ClearTexture,i);
			pxIcon->SetHelpText("Clear the texture for this slot");
			pxOverlay->AddDialog(pxIcon);
		}
		//xOffset.y += 0.2f;


		xOffset.x = 23.0f;
		pxDialog = m_apxTextureDialog[i] = new Asura_Function_Dialog("",xOffset,_EditTexture,i);
		pxDialog->SetHelpText("Set the texture to use in this slot for this material");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}
	xOffset.y += 1.0f;

	//and for normal or parallax maps
	pxDialog = new Asura_Dialog("Normal/Parallax Maps :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;


	for(int i=0;i<ASURA_MATERIAL_MAXTEXTURES;i++)
	{
		xOffset.x = 21.0f;
		Asura_Icon_Dialog* pxIcon = new Asura_Icon_Dialog(xOffset,Tools_Dialog::GetIconTexture(),TOOLS_ICON_DELETE);
		pxIcon->SetClickedFunction(_ClearTexture,i+ASURA_MATERIAL_MAXTEXTURES);
		pxIcon->SetHelpText("Clear the normal map for this slot");
		pxOverlay->AddDialog(pxIcon);
		
		//xOffset.y += 0.2f;


		xOffset.x = 23.0f;
		pxDialog = m_apxTextureDialog[i+ASURA_MATERIAL_MAXTEXTURES] = new Asura_Function_Dialog("",xOffset,_EditTexture,i+ASURA_MATERIAL_MAXTEXTURES);
		pxDialog->SetHelpText("Set the normal map to use in this slot for this material");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += 1.5f;
	}

	xOffset.y += 1.0f;

	//add anim selection list
	pxDialog = new Tools_TextureAnim_DropDownList_Dialog(xOffset,10,&m_xMaterial.m_uAnimHash,&m_bMaterialChanged,"Anim");
	pxDialog->SetHelpText("The texture animation for this material");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f;

	xOffset.x = 1.0f;
	xOffset.y = fCheckboxEndY;



	Project_AddOverlayControls(pxOverlay, xOffset);

	pxDialog = new Asura_FunctionWithKeybinding_Dialog("Ok",xOffset,0,ASURA_KEYID_ENTER,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);

	UpdateOverlayControls();


}

void Tools_Material_Overlay::UpdateOverlayControls()
{
	char szText[200];
	
	//set the texture names for each texture in the current material
	for(int i=0;i<ASURA_MATERIAL_MAXTEXTURES*2;i++)
	{
		int iTextureID;
		if(i<ASURA_MATERIAL_MAXTEXTURES)
		{
			iTextureID = m_xMaterial.m_iTextureID[i];
		}
		else if(i<2*ASURA_MATERIAL_MAXTEXTURES)
		{
			iTextureID = m_xMaterial.m_iNormalMapID[i-ASURA_MATERIAL_MAXTEXTURES];
		}

		if(iTextureID == ASURA_TEXTURE_NONE)
		{
			Asura_PrintfToBfr(szText,"%d: *None*",200,true,i%ASURA_MATERIAL_MAXTEXTURES);
		}
		else
		{
			const Asura_String* pxName = Asura_Texture_Management::GetTextureName(iTextureID);
			if(pxName)
			{
				Asura_PrintfToBfr(szText,"%d: %s",200,true,i%ASURA_MATERIAL_MAXTEXTURES,Tools_FileSystem::StripPath(pxName->GetString()));
			}
			else
			{
				Asura_PrintfToBfr(szText,"%d: *Unknown*",200,true,i%ASURA_MATERIAL_MAXTEXTURES);
			}
		}

		m_apxTextureDialog[i]->UpdateLabel(szText);
	}
	Project_UpdateOverlayControls();
}

void Tools_Material_Overlay::UpdateMaterialList()
{
	//ditch old list of materials
	ClearMaterialList();
	
	//build a list of all materials using the texture

	const List<int>* pxAllMaterials = Tools_PickTexture::GetMaterialList();
	for(CLIF<int> xMaterialLIF(pxAllMaterials);!xMaterialLIF.done();xMaterialLIF.next())
	{
		if(Asura_Material_System::GetTextureID(xMaterialLIF()) == m_iTextureID)
		{
			m_iNumberOfMaterials++;
			m_xMaterialList.add_entry(xMaterialLIF());
		}
	}

	if(!m_iNumberOfMaterials)
	{
		//create a material for the texture
		int iMaterialID = Asura_Material_System::GetMaterialIDForTexture(m_iTextureID,true);
		Tools_PickTexture::GetThis()->AddToMaterialList(iMaterialID);
		Tools_SaveAsura::SetMainFileModified();

		m_iNumberOfMaterials++;
		m_xMaterialList.add_entry(iMaterialID);
		
	}

	//build an array of names for the list dialog control
	char szTempName[100];
	for(int i=0;i<m_iNumberOfMaterials;i++)
	{
		Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(m_xMaterialList[i]);
		Asura_Assert(pxMaterial,"");

	   	if(pxMaterial->m_xMaterialName.IsBlank())
		{
			Asura_PrintfToBfr(szTempName,"%03d: <Unnamed>",100,true,m_xMaterialList[i]);
		}
		else
		{
			Asura_PrintfToBfr(szTempName,"%03d: %s",100,true,m_xMaterialList[i],pxMaterial->m_xMaterialName.GetString());
		}

		m_ppszMaterialNames[i] = new char[strlen(szTempName)+1];
		strcpy(m_ppszMaterialNames[i],szTempName);

	}
}

void Tools_Material_Overlay::ClearMaterialList()
{
	for(int i=0;i<m_iNumberOfMaterials;i++)
	{
		delete [] m_ppszMaterialNames[i];
	}
	m_iNumberOfMaterials = 0;

	while(m_xMaterialList.size()) m_xMaterialList.delete_first_entry();
}

void Tools_Material_Overlay::CreateMaterial()
{
	//create the new material
	int iMaterialID = Asura_Material_System::CreateMaterial();
	if(iMaterialID == ASURA_MATERIAL_NONE) return;

	Tools_SaveAsura::SetMainFileModified();

	//copy values from selected one (clearing the name)
	Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(iMaterialID);
	*pxMaterial = m_xMaterial;
	pxMaterial->m_xMaterialName.MakeNull();

	//need to add to main list
	Tools_PickTexture::GetThis()->AddToMaterialList(iMaterialID);

	//update the list of materials , and find material in the list
	UpdateMaterialList();
	int iNewSelection = -1;
	for(int i=0;i<m_iNumberOfMaterials;i++)
	{
		if(m_xMaterialList[i] == iMaterialID)
		{
			iNewSelection = i;
			break;
		}
	}
	Asura_Assert(iNewSelection!=-1,"");

	SelectMaterial(iNewSelection);

	//and name it
	RenameMaterial();
}

void Tools_Material_Overlay::RenameMaterial()
{
	//ask for a new name
	GetString_Overlay xOverlay("Enter name for material",m_xMaterial.m_xMaterialName.GetString());

	if(xOverlay.DoModal())
	{
		//update the copy of the material , and set the modified flag
		m_xMaterial.m_xMaterialName = xOverlay.GetString();
		UpdateMaterial();
		UpdateMaterialList();
	}
}

void Tools_Material_Overlay::EditTexture(int iIndex)
{
	char szMessage[200];
	int* piTextureID = 0;

	bool bBumpMap = false;

	int iInitialTextureFlags = 0;

	if(iIndex >= ASURA_MATERIAL_MAXTEXTURES)
	{
		iIndex -= ASURA_MATERIAL_MAXTEXTURES;
		bBumpMap = true;
		Asura_PrintfToBfr(szMessage,"Select normal map %d for the material",200,true,iIndex);
		piTextureID = &m_xMaterial.m_iNormalMapID[iIndex];
		iInitialTextureFlags = AsuraTextureFlag_NormalMap;

	}
	else
	{
		Asura_PrintfToBfr(szMessage,"Select texture %d for the material",200,true,iIndex);
		piTextureID = &m_xMaterial.m_iTextureID[iIndex];
		iInitialTextureFlags = AsuraTextureFlag_DiffuseMap;
	}

	u_int uOptionFlags = 0;
	if(bBumpMap)
		uOptionFlags = PICKTEXTURE_BUMP;
	else if(iIndex>0)
		uOptionFlags = PICKTEXTURE_SECONDARY;
	else
		uOptionFlags = PICKTEXTURE_PRIMARY;


	int iTextureID = Tools_PickTexture::PickTexture(szMessage,*piTextureID,uOptionFlags,iInitialTextureFlags);
	if(iTextureID != ASURA_TEXTURE_NONE)
	{
		*piTextureID = iTextureID;
	
		UpdateOverlayControls();	
		m_bMaterialChanged = true;
	}
}

void Tools_Material_Overlay::ClearTexture(int iIndex)
{
	Asura_Assert(iIndex,"Shouldn't be clearing the first texture");
	if(iIndex < ASURA_MATERIAL_MAXTEXTURES)
	{
		m_xMaterial.m_iTextureID[iIndex] = ASURA_TEXTURE_NONE;
	}
	else if(iIndex < 2*ASURA_MATERIAL_MAXTEXTURES)
	{
 		iIndex -= ASURA_MATERIAL_MAXTEXTURES;
		m_xMaterial.m_iNormalMapID[iIndex] = ASURA_TEXTURE_NONE;
	}

	UpdateOverlayControls();	
	m_bMaterialChanged = true;
}

