//
// Tools_PickTexture.h
//
// Created by Kevin Lea - 02/07/2001 15:18:08
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PICKTEXTURE_H__
#define __TOOLS_PICKTEXTURE_H__

#include "Core/Asura_TextureManagement.h"//for ASURA_TEXTURE_NONE definition
#include "Core/Asura_Material_System.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Colour.h"

#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/list_tem.hpp"

//flags for PickTexture
#define	PICKTEXTURE_MATERIALEDIT		0x00000001 //Allow material editing y pressing M over a texture
#define PICKTEXTURE_ALLTEXTURES			0x00000002 //Force all loaded textures to be shown
#define PICKTEXTURE_PRIMARY				0x00000004 //Show textures used as the primary texture on environment polys
#define PICKTEXTURE_SECONDARY			0x00000008 //Show textures used as the secondary texture on environment polys
#define PICKTEXTURE_BUMP				0x00000010 //Show textures used as bump maps on environment polys
#define PICKTEXTURE_SKYBOX				0x00000020 //Show textures used on skybox + reflectance texture
#define PICKTEXTURE_DEPRECATED			0x00000040 //DEPRECATED - was "Show textures used as specular maps on environment polys"
#define PICKTEXTURE_USECALLBACKFILTER	0x00000080 //Use callback function for determining which textures to show
#define PICKTEXTURE_CUBEMAP				0x00000100 //Show cubemaps

//default options
#define PICKTEXTURE_DEFAULT			(PICKTEXTURE_MATERIALEDIT|PICKTEXTURE_PRIMARY)
#define PICKTEXTURE_DEFAULTFLAGEDIT (PICKTEXTURE_MATERIALEDIT|PICKTEXTURE_PRIMARY|PICKTEXTURE_SECONDARY|PICKTEXTURE_BUMP)


enum PickTextureAlphaMode
{
	PICKTEXTURE_OPAQUE,
	PICKTEXTURE_TRANSPARENT_BLACK,
	PICKTEXTURE_TRANSPARENT_MAGENTA,
};

class Tools_PickTexture
{
	public:
		Tools_PickTexture();
		virtual ~Tools_PickTexture();

		void Initialise();
		
		static int PickTextureFromDisk(int iInitialTextureFlags = 0, Asura_Hash_ID* puResult = NULL);
		//pick texture allows the user to pick an environment texture from those already loaded
		//also allows a new texture to be loaded with the appropriate keypress
		//uOptionFlags taken from the pick texture flags above
		//iInitialTextureFlags are texture flags that will be added by default to the selected texture
		//puResult is the hash of the picked name, set immediately. Needed for processing MR texture formats during load
		static int PickTexture(const char* Message = 0,
							   int iCurrentSelection = ASURA_TEXTURE_NONE,
							   u_int uOptionFlags = PICKTEXTURE_DEFAULT,
							   int iInitialTextureFlags = 0,
							   Asura_Hash_ID* puResult = NULL);

		static int PickMaterial(const char* Message = 0,int iCurrentSelection = ASURA_MATERIAL_NONE, bool bPreferSinglePass = false); 

		//Brings up the texture flag editing screen
		//(not exactly texture picking , but the code is much the same)
		//Uses same option flags as for pick texture
		//If pbModifiedFlag is passed in, it will be set instead of flagging the main asura files as modified
		static void EditTextureFlags(u_int uOptionFlags, bool* pbModifiedFlag = 0);


		//get next/previous material for this file
		static int GetNextMaterial(int iMaterialID);
		static int GetPreviousMaterial(int iMaterialID);

		//get the list of materials for this file
		static const List<int>* GetMaterialList() {return &s_pxThis->m_xMaterialList;}

		static Tools_PickTexture* GetThis(){return s_pxThis;}

		//add material to list used by this level
		static void AddToMaterialList(int iMaterialID){s_pxThis->AddToMaterialListMain(iMaterialID);}

		//rebuilds the list of materials used by this level
		static void RebuildMaterialList(){s_pxThis->RebuildMaterialListMain();}

		//Finds equivalent material in list , or creates a new one if needed
		static bool CreateOrFindExistingMaterial(const Asura_Material* pxMat,bool bCompareNames,int& iOutputMatID,bool* pbIsNew = NULL);

		//Brings up the material overlay for the specified texture... only
		//expected on environment textures atm
		static void EditMaterial(int iMaterialID);

		//Sets callback that will be used for determining which textures to show.
		//Only used if PICKTEXTURE_USECALLBACKFILTER is set
		static void SetTextureFilterCallback(bool (*pfnTextureFilter)(int iTexture,void* pUserData), void* pUserData = 0) { s_pfnTextureFilter = pfnTextureFilter; s_pTextureFilterUserData = pUserData;}

	protected :
		//Edits material flags using Tools_Material_Overlay , or derived class
		virtual void ShowEditMaterialOverlay(int iTextureID);

		char** m_pszTextureFlagNames;
		int*   m_piTextureFlagValues;
		u_int  m_uNumberOfTextureFlags;

	private :
		int PickTextureFromDisk_Main(int iInitialTextureFlags, Asura_Hash_ID* puResult);
		int PickTexture_Main(const char* Message,int iCurrentSelection,u_int uOptionFlags,int iInitialTextureFlags,Asura_Hash_ID* puResult); 
		int PickMaterial_Main(const char* Message,int iCurrentSelection,bool bPreferSinglePass); 
		void EditTextureFlags_Main(u_int uOptionFlags, bool* pbModifiedFlag);

		void BuildThumbnailArray(u_int uOptionFlags,int& iOuputNumTextures,struct TextureNameAndID*& pxOutputThumbnailArray);


		//get next/previous material for this file
		int GetNextMaterial_Main(int iMaterialID);
		int GetPreviousMaterial_Main(int iMaterialID);
				
		void SelectTextureFlag();
		void SetTextureFlag(int textureIndex,int flag,bool add);
		//sets material flag on all materials with the texture
		void SetMaterialFlag(int iTextureID,int iFlag,bool bAdd);

		//checks all materials useing a texture to see if any have the given material flag
		void GetMaterialFlagUseage(int iTextureID,int iFlag,bool* pbFlagUsed,bool* pbFlagUsedByAll);
		
		void ReplaceTextures();

		void AddToMaterialListMain(int iMaterialID);
		bool TextureIsInMaterialList(int iTextureID);

		void RebuildMaterialListMain();

		bool TextureIsUsedByMaterials(int iTextureID);

		void RenderTexture(int iTextureID,PickTextureAlphaMode eAlphaMode,float fX,float fY,float fWidth,float fHeight);

//		static void EditMaterialsForTexture(int iTextureID);

		List<int> m_xMaterialList;//list of materials for the current file
		int m_iCurrentTextureFlagIndex;
		int m_iCurrentMaterialFlagIndex;
		bool m_bShowingTextureFlags;//as opposed to material flags
		bool m_bShowCurrentLightSetOnly;

		static bool (*s_pfnTextureFilter)(int,void*);
		static void* s_pTextureFilterUserData;

		friend class Tools_Material_Overlay;

		static Tools_PickTexture* s_pxThis;

};

/* Example usage:

   	int newTexture = Tools_PickTexture::PickTextureFromDisk();

	if (newTexture!=ASURA_TEXTURE_NONE)
	{
		CurrentlySelectedTextureIndex = newTexture;
	}

*/

class Tools_Material_Overlay : public Tools_Modal_Overlay
{
public :
	Tools_Material_Overlay(int iTextureID);
	~Tools_Material_Overlay();

	void DoModal();
	virtual void Update();

	static void _UpdateMaterial(int iThis){((Tools_Material_Overlay*)iThis)->UpdateMaterial();}

    int GetCurrentSelectedMaterialID() const { return m_xMaterialList[ m_iCurrentSelection ]; }

	//get list of materials being shown
	const List<int>* GetMaterialList() const {return & m_xMaterialList;}

	//update the local copy of the current material from the material system
	void UpdateMaterialFromMaterialSystem();

	//sets the material that should be seelected on the next showing of the overlay
	//(to get around the slightly awkward way that the overlay gets created
	//in Tools_PickTexture)
	static void SetInitiallySelectedMaterial(int iMaterialID) {s_iInitiallySelectedMaterial = iMaterialID;}

protected :
	virtual void OnSelectionChange(){};//called when a new material is selected

private :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	void Project_AddOverlayControls(class Asura_Overlay *pxNewOverlay, class Asura_Vector_2& xOffset);
	void Project_UpdateOverlayControls();

	void UpdateMaterialList();
	void ClearMaterialList();

	void CreateMaterial();
	void RenameMaterial();

	void UpdateMaterial();
	void SelectMaterial(int iIndex);

	void EditTexture(int iIndex);
	void ClearTexture(int iIndex);

	void UpdateOverlayControls();

	static void _CreateMaterial(int iThis){((Tools_Material_Overlay*)iThis)->CreateMaterial();}
	static void _RenameMaterial(int iThis){((Tools_Material_Overlay*)iThis)->RenameMaterial();}

	static void _EditTexture(int iIndex){s_pxThis->EditTexture(iIndex);}
	static void _ClearTexture(int iIndex){s_pxThis->ClearTexture(iIndex);}

	static bool CurrentMaterialIsSpecular();

	int m_iTextureID;

	int m_iListOffset;
	int m_iListSelection;
	int m_iNumberOfLines;

	int m_iNumberOfMaterials;
	List<int> m_xMaterialList;
	char** m_ppszMaterialNames;

	int m_iCurrentSelection;//index into list above
	Asura_Material m_xMaterial;
	bool m_bMaterialChanged;

	class Asura_Function_Dialog* m_apxTextureDialog[ASURA_MATERIAL_MAXTEXTURES*2];

	static Tools_Material_Overlay* s_pxThis;

	static int s_iInitiallySelectedMaterial;

};

//anim selection drop down list
class Tools_TextureAnim_DropDownList_Dialog : public Asura_Hash_DropDownList_Dialog
{
public :
	Tools_TextureAnim_DropDownList_Dialog( Asura_Vector_2& xOffset, int iNumberOfLines, Asura_Hash_ID* puSelectedAnim, bool* pbChangedFlag = 0, const char* szName = NULL );
	
protected :
	virtual u_int GetNumberOfHashValues() const;
	
	virtual const char* GetNameByIndex( int iIndex ) const;
};

#endif//__TOOLS_PICKTEXTURE_H__
