//
// Tools_TextureAnimation.h
//
// Created by Richard Rice - 17/05/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_TEXTUREANIMATION_H__
#define __TOOLS_TEXTUREANIMATION_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_Hash.h"

class Tools_TextureAnimation
{
public :

	static void Initialise();
	static void Shutdown();

	static void Update();

	static void ToggleOverlay();
	static bool OverlayIsOn();
	static void CreateOverlay();

	static void CloseOverlays();

//overlay stuff that gets saved to mode ini file
	static bool s_bOverlayOpen;
	static Asura_Vector_2 s_xOverlayPosition;
	static Asura_Vector_2 s_xEditAnimOverlayPosition;

	//adds overlay controls for the message editing overlay
	static void AddOverlayControls_TextureAnimationMessage(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

private :
	//deals with the list of names for the overlay
	static void BuildAnimNames();
	static void DeleteAnimNames();

	//overlay functions
	static void CreateAnim();
	static void DeleteAnim();
	static void DuplicateAnim();
	static bool IsAnimSelected();

	static void ApplyPrimaryAnim();
	static void ApplySecondaryAnim();
	static void RemovePrimaryAnim();
	static void RemoveSecondaryAnim();
	static void ApplyAnim(Asura_Hash_ID uAnimHash, int iMaterialSlot);
	
	static void FindAnim();

	//opens overlay for editing selected anim
	static void EditAnim();
	static void EditAnim(Asura_Hash_ID uAnimHash);
	//and closes
	static void CloseEditAnimOverlay();

	static void RenameAnim();

	static void CreateSeq();
	static void DeleteSeq();
	static bool IsEnabled_DeleteSeq();

	static void MoveSeqEarlier();
	static bool IsEnabled_MoveSeqEarlier();

	static void MoveSeqLater();
	static bool IsEnabled_MoveSeqLater();


	static void CreateFrame();
	static void DeleteFrame();
	static bool IsEnabled_DeleteFrame();

	static void AddMaterialFlag();
	static void RemoveMaterialFlag();
	static void DeleteMaterialFlag();
	static u_int SelectMaterialFlag(const char* szTitle);

	static void BuildMaterialFlagList();
	static void DeleteMaterialFlagList();
	static void UpdateMaterialFlagListControl();

	static const char* GetMaterialFlagName(u_int uValue);

	static struct Asura_TextureAnimationFrame* GetEditedFrame();

	//set the file that contains the animations as modified
	static void SetModified();

	//gets the index of the anim from the list shown on the overlay.
	//(not the same as the index in the animation system).
	static int GetListIndexFromName(const char* pszName);

	//updates the copy of the variables used by the overlays from the Asura_TextureAnimation
	static void UpdateOverlayVariablesFromAnim();
	//and the reverse
	static void UpdateAnimFromOverlayVariables();

	//is enabled functions for message editing
	static bool IsEnabled_Sequence();
	static bool IsEnabled_Speed();
	


	static bool IsEnabled_FrameTime(){return s_iSelectedFrame!=0;}


	static int s_iSelectedAnim;
	static int s_iNumAnimNames;
	static const char** s_ppszAnimNames;
	static bool s_bShowEnvAnimsOnly;
	static bool s_bShowEnvAnimsOnlyChanged;

	static Asura_Hash_ID s_uEditedAnim;

	static int s_iSelectedSequence;
	static int s_iSelectedFrame;

	static class Asura_String s_xNumSequenceText;
	static class Asura_String s_xNumFrameText;

	static bool s_bNeedUpdateFromAnim;
	static bool s_bChanged;

	static float s_fSeqDuration;
	static bool	s_bSeqLoop;
	static bool	s_bSeqLoopBackAndForth;
	static bool s_bSeqInterpolate;
	static bool s_bSeqRandom;
	static bool s_bDurationChanged;

	static float s_fFrameU;
	static float s_fFrameV;
	static float s_fFrameTime;
	static bool s_bFrameTimeInSecs;

	static u_int s_uFrameAddedMaterialFlags;
	static u_int s_uFrameRemovedMaterialFlags;

	static int s_iSelectedMaterialFlag;
	static int s_iMaterialFlagListSize;
	static char** s_pszMaterialFlagListStrings;

	static bool s_bUseUVScale;
	static bool s_bUseUVScaleChanged;
	static float s_fUVScale;


	//For showing the currently edited frame
	static bool s_bShowEditedFrame;

	static struct Asura_Message_TextureAnimation* s_pxEditedMessage;

};

#endif //__TOOLS_TEXTUREANIMATION_H__
