#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_TEXTURING_FIND_H__
#define __TOOLS_TEXTURING_FIND_H__

#include "Misc/Tools_Polygon_Selection.h"
#include "Misc/list_tem.hpp"

//Contains functions for handling the find overlay in texturing mode

//Options for the main FindPolygons function
#define TOOLS_FINDPOLYGONS_QUIET				0x00000001 //Don't display the number of polygons found on screen
#define TOOLS_FINDPOLYGONS_DONTCHANGESELECTION	0x00000002 //Ignore the select found polygons flag

class Tools_Texturing_Find
{
	public :
		static void Initialise(void);
		static void Shutdown(void);

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		static void FindBlankMaterial();
#else
		static void FindUntexturedPolygons();
#endif

		static void FindTexturedPolygons();
		static void FindMaterial();
		static void FindIllegalPsx2Coordinates();
		static void FindPolygonsWithStretchedTexture();
		static void FindPolygonFlag();
		static void FindPolygonsBySize();
		static void Find8BitTransparent();
		static void FindTextureRepeat();
		static void FindTextureAnim(Asura_Hash_ID uAnimID);
		static void FindTangentErrors();

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		static void FindMaterial(Asura_Hash_ID uMaterial);
#endif


		//Finds polygons with both 8 bit alpha and transparency , adding them to the find list.
		//Returns a count of offending polygons that would be exported
		static int Find8BitTransparent(bool bQuiet);

		static void FindPolygonsWithStretchedTexture(float fTexelPerMetreThreshhold);

		static void FindPolygonsBySize(float fMinSize,float fMaxSize,bool bUseMax);

		//takes the camera to the next/previous polygon that has been found
		static void NextFoundPolygon();
		static void PreviousFoundPolygon();
		static void NextFoundModule();
		static void PreviousFoundModule();

		//overlay creation
		static void ToggleFindOverlay(void);
		static bool FindOverlayIsOn(void);
		static void CreateFindOverlay(void);

		static float GetTexelPerMetre(const class Asura_Mesh* pxMesh,int iPoly);

		//main polygon find function
		static int FindPolygons(class Tools_PolygonFind* pxFind, u_int uFlags = 0);
	private :

		static void ShowFindList();



		static List<PolygonID> FindPolygonList;//list of polygons that have been found
		static LIL<PolygonID> FindPolygonLIL;//iterator for going to each found polygon
		static bool SelectOnFind;
		static bool s_bClearCurrentSelection;
		static bool s_bIncludeObjectPolygons;
		static float s_fTexelPerMetreThreshold;

		static float s_fFindBySize_Min;
		static float s_fFindBySize_Max;
		static bool s_bFindBySize_UseMax;

		static bool s_bTangentError_CheckAllLoadedMeshes;


		//for the overlay
		static Asura_Vector_2 FindOverlayPosition;
		static bool	FindOverlayOpen;
		static u_int s_uFindOverlayMinimised;

		//loads/saves data for this mode
		static void TransferModeData(int mode);
		
		//takes the camera to the polygon
		static void GotoPolygon(const PolygonID& polygon);
};


//class containing virtual functions for the main FindPolygons thingy
class Tools_PolygonFind
{
public :
	//return true if this polygon should be included in the list of found polygons
	virtual bool IncludePolygon(Asura_Mesh* pxMesh,int iPolyIndex){return false;}
	virtual bool IncludePolygon(int iModule,int iPolyIndex){return false;}
	
	//Display message at end of find process
	virtual void DisplayMessage(){};
};

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

class Tools_PolygonFind_Material : public Tools_PolygonFind
{
public:

	Tools_PolygonFind_Material( const Asura_Hash_ID uMaterialResponseHashID, int iChannelsToCheck )
		: m_uMaterialResponseHashID( uMaterialResponseHashID )
		, m_iChannelsToCheck( iChannelsToCheck)
	{
	}

private:

	virtual bool IncludePolygon( Asura_Mesh* pxMesh, int iPolygonIndex )
	{
		const Asura_Mesh_TextureInfo* const pxTex = &pxMesh->TextureDataPtr[ iPolygonIndex ];
		for(int iMat=0; iMat<TOOLS_NUM_MATERIALRESPONSE_PER_POLY; iMat++)
		{
			if(m_iChannelsToCheck & (1<<iMat))
			{
				if(pxTex->m_auMaterialResponseHashID[iMat] == m_uMaterialResponseHashID) return true;
			}
		}
		return false;
	}

	const Asura_Hash_ID m_uMaterialResponseHashID;
	const int m_iChannelsToCheck;

};

#else // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

class Tools_PolygonFind_Material : public Tools_PolygonFind
{
public :
	Tools_PolygonFind_Material(int iMaterialID)
	{
		m_iMaterialID = iMaterialID;
	}
	
private :
	virtual bool IncludePolygon(Asura_Mesh* pxMesh,int iPolyIndex)
	{
		//Check polygon for desired material
		Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[iPolyIndex];

		return pxTex->m_iMaterialID == m_iMaterialID;
	}

	int m_iMaterialID;
};

#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#endif //__TOOLS_TEXTURING_FIND_H__
