//////////////////////////////////////////////////////////////////////////////
//
// Tools_Texturing_MaterialResponse.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __TOOLS_TEXTURING_MATERIALRESPONSE_H__
#define __TOOLS_TEXTURING_MATERIALRESPONSE_H__

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_Hash.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Hash_DropDownList_Dialog;
class Asura_Mesh;
class Asura_String;
class Asura_TreeDialog_Item;
class Asura_TreeDialog_Tree;
class Asura_Vector_2;
class Tools_MaterialResponse;
class Tools_MaterialResponse_Undo;
template <class T> class List;

//maximum number of edit material overlays that should be allowed at a time.
//this needs to correspond to the number of overlay IDs available in Tools_Overlay.h
static const int iTOOLS_MAX_EDIT_MATERIAL_RESPONSE_OVERLAY = 4;


////////////////////////////////////////////////////////////////////////////////
//
// Tools_Texturing_MaterialResponse
//
// Handles in-Tools editing and preview of material responses
//
////////////////////////////////////////////////////////////////////////////////


class Tools_Texturing_MaterialResponse
{
	friend class Tools_MaterialResponse_Overlay;
	friend class Tools_MaterialProperties_Overlay;

public:
	Tools_Texturing_MaterialResponse();
	virtual ~Tools_Texturing_MaterialResponse();

	static void Initialise();
	static void Shutdown();

	static void EditMaterial( const Asura_Hash_ID uMaterialHash );
//	static void ClearUndo();
	static bool IsModified();
	static void SaveChangedMaterials();
	static void SetMeshModified( const Asura_Mesh* pxMesh, const bool bModified);
	static void ToggleOverlay();
	static void Update();
	static void UpdatePropertiesOverlay();

	static void ShowBadTextureUsageReport();

	typedef void (*MaterialProjectUICallback)(Tools_MaterialResponse* const pxMaterialResponse, class Asura_Tabs_Dialog *pxDialog, Asura_Vector_2 xOffset);

	static void SetProjectUICallback(MaterialProjectUICallback pfnProjectUICallback) { s_pfnProjectUICallback = pfnProjectUICallback; }
	static MaterialProjectUICallback GetProjectUICallback() { return s_pfnProjectUICallback; }

	static bool OverlayIsOn() { return s_bOverlayOpen; }

	//Sets the selected material (if the overlay is open)
	static void SetSelectedMaterial( Asura_Hash_ID uMaterialHash);

#if !defined(ASURA_MATERIAL_RESPONSE_SURFACES_USE_HASH_IDS)
	static u_int GetNumSurfaceTypes() {Asura_Assert(s_pxThis,"");return s_pxThis->Project_GetNumSurfaceTypes();}
	static const char* GetSurfaceTypeName(u_int uIndex) {Asura_Assert(s_pxThis,"");return s_pxThis->Project_GetSurfaceTypeName(uIndex);}
#endif	// !defined(ASURA_MATERIAL_RESPONSE_SURFACES_USE_HASH_IDS)

	static void ConvertOldMaterials();

	//Updates old-style materials to use equivalent textures to the material responses
	static void UpdateOldMaterialsFromMaterialResponseAndGetUsedMaterials(const List<Asura_Mesh*>& xMeshList,Asura_HashTree& xOutputMaterialResponseTree);


	static void OutputToMaterialLogFile(const char* szFormat,...);

	//Gets the single instance of this class for using the project virtual functions.
	static Tools_Texturing_MaterialResponse* GetThis(){return s_pxThis;}

	//Apply selected material only works if the material response overlay is open
	static void ApplySelectedMaterialAsPrimary() {ApplySelectedMaterial(0);}
	static void ApplySelectedMaterialAsSecondary() {ApplySelectedMaterial(1);}
	static void ApplySelectedMaterial(int iMatSlot);

	//Apply edited material (if any) or selected material (if material response overlay is open)
	static void ApplyEditedOrSelectedMaterial();

	//Creates a tree containing just those materials used by the 'modules'
	static void CreateMaterialNameTree_FilteredByEnv(Asura_TreeDialog_Tree& xTree);

	//Fills in a hash tree containing materials used on the environment
	static void FindMaterialsUsedByEnv(Asura_HashTree& xOutputMaterialTree);

	//Generates a new material response for the appropriate half of the old material
	static Tools_MaterialResponse* CreateMaterialResponseFromOldMaterial(int iOldMaterialID, int iMatSlot, bool bAutomatic);

	//Various functions for logging changes for undo
	static void LogApplyMaterial( const u_int uMeshIndex, const u_int uPolygonIndex );
	static void LogImportMaterial( const u_int uMeshIndex, const u_int uPolygonIndex );
	static void LogRemoveMaterial( const u_int uMeshIndex, const u_int uPolygonIndex );

	//Makes	a copy of a material (prompting user for a name) and returns the ID of the copy
	//If bOpenOverlay is true then the properties overlay for the new material will be opened
	static Asura_Hash_ID DuplicateMaterial( Asura_Hash_ID uMaterialHash, bool bOpenOverlay );
	static bool ShowNewFileDialogue( Asura_String& xFilename );

	static void RegenerateOverlay();

protected:

	// Static functions

	static void CreateOverlay();
	static void DestroyOverlay();

	static void ApplyPrimaryMaterial(int iMaterialHash) {ApplyMaterial(iMaterialHash,0);}
	static void ApplySecondaryMaterial(int iMaterialHash) {ApplyMaterial(iMaterialHash,1);}

	static void ApplyMaterial( int iMaterialHash,int iMatSlot );
	static void CreateMaterialResponse();
	static void DuplicateSelectedMaterial();
	static void DuplicateMaterial( int iMaterialHash) {DuplicateMaterial(iMaterialHash, true);}
	static void EditSelectedMaterial();
	static Tools_MaterialResponse* GenerateSecondaryMaterialFromOldMaterial(const Tools_MaterialResponse* pxOldMaterial); 

	static void MoveMaterial( int iMaterialHash );

	static void AddSlaves( int iMaterialHash );
	static void RemoveSlave( int iMaterialHash );
	static void EnslaveToOther( int iMaterialHash );
	static void BreakFromParent( int iMaterialHash );

	static void CopyPropertiesFromOther( int iMaterialHash );
	static void CopyPropertiesToOther( int iMaterialHash );

	static void ApplySpecularSharpness( int iMaterialHash );

	static void RemoveMaterialsFromMesh();
	static bool SaveMaterial( Tools_MaterialResponse& xMaterial );
	static void TransferModeData( const int iMode );

	static void ShowStatistics();
	static void AddMeshStatsToTree( Asura_TreeDialog_Tree& xInfoTree, const Asura_Mesh& xMesh, Asura_TreeDialog_Item* pxRootNode = NULL );

	static void FindMissingMaterials();


#if defined(ASURA_MATERIAL_RESPONSE_SURFACES_USE_HASH_IDS)
	static Asura_Hash_DropDownList_Dialog* Project_CreateSurfaceTypeDialog(Asura_Vector_2& xOffset, const int iNumberOfLines, Asura_Hash_ID* puSelected, bool* pbChangedFlag = NULL, const char* szName = NULL);
	static bool Project_ValidateSurfaceTypeHash(Asura_Hash_ID& uHash);
#else
	//For getting list of project defined surface types
	virtual u_int Project_GetNumSurfaceTypes() const {return 0;}
	virtual const char* Project_GetSurfaceTypeName(u_int uIndex) const {return 0;}
#endif	// defined(ASURA_MATERIAL_RESPONSE_SURFACES_USE_HASH_IDS)


	// Static data

	static Asura_Vector_2 s_xOverlayPosition;
	static Asura_Vector_2 s_xUserSetTreeSize;
	static bool s_bOverlayOpen;
	static u_int s_uOverlayMinimised;

	static float s_fSpecularSharpness;


	static bool s_bMeshModified;

	//store of details of material conversion
	static int s_iMaterialConversion_NumChanged;
	static int s_iMaterialConversion_NumCreated;
	static bool s_bMaterialConversion_ShowMessage;

	static Tools_Texturing_MaterialResponse* s_pxThis;

	static MaterialProjectUICallback s_pfnProjectUICallback;
};

#endif // defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#endif // !defined __TOOLS_TEXTURING_MATERIALRESPONSE_H__
