//
// Tools_UVEdit.cpp
//
// Created by Richard Rice - 24/04/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Texturing/Tools_UVEdit.h"

#include "Core/Asura_Material_System.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

#include "Tools_Overlays.h"
#include "Tools_Undo.h"
#include "Interface/Tools_ColourPicker.h"
#include "Interface/Tools_WindowsCursor.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Dialog.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Polygon_Selection.h"
#include "Misc/Tools_ProgressIndicator.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Mode_Texturing.h"
#include "Texturing/Tools_Texture_Management.h"

#include "Texturing/Tools_UVEdit_Internal.h"


extern unsigned int Asura_OverlayBrightness;

static const float g_fUVEditDragBorder = 3.0f;
static const float g_fDragGizmoOffset = 1.0f;
static const float g_fDragGizmoHalfWidth = 3.0f;
static const float g_fCentreHalfWidth = 4.0f;
static const float g_fNodeSelectionDistance = 5.0f;//how close to node the cursor should be to be considered to be clicking on it

static const Asura_Hash_ID g_uDialog_U = 0x00000075; //Asura_GetHashID("U");
static const Asura_Hash_ID g_uDialog_V = 0x00000076; //Asura_GetHashID("V");


static const char* g_aszUVEditHelpStrings [] =
{
	"Mousewheel :    Zoom in/out",
	"Z :             Zoom extents of the UVs for the selected polygons",
	"Space :         Lock/Unlock current node selection",
	"Hold MMB :      Pan view in main UV edit control",
	"",
	"While editing UVs with LMB :",	
	"Hold Shift :    Restrict movement to one axis (where appropriate)",
	"Hold Control :  Uniform scale (when stretching in freeform mode)",
	"Hold Control :  Snap rotation to multiples of a specified angle",
	"",
	"While selecting with LMB :",
	"Hold Alt     :  Deselects nodes",
	"Hold Control :  Toggle selection (single selection)",
	"Hold Control :  Adds to current selection (drag selection)",
	"",
	"Shift+Z :       Undo selection (while cursor is on overlay)",
	"Shift+Y :       Redo selection (while cursor is on overlay)",
	"Shift+< :       Undo selection (while cursor is on overlay)",
	"Shift+> :       Redo selection (while cursor is on overlay)",
	0
};


///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Tools_UVEdit **
**--------------*/

//Defines for making the callback functions for Tools_UVEdit
#define CALLBACK_VOID(__Function) Asura_Callback_MemberFunction<void,Tools_UVEdit>(this, &Tools_UVEdit::__Function) 
#define CALLBACK_BOOL(__Function) Asura_Callback_MemberFunction<bool,Tools_UVEdit>(this, &Tools_UVEdit::__Function) 
#define CALLBACK_VOID_BOOL(__Function) Asura_Callback_MemberFunction_1Arg<void,bool,Tools_UVEdit>(this, &Tools_UVEdit::__Function) 

Asura_Vector_2 Tools_UVEdit::s_xPosition(10.0f,10.0f);
u_int Tools_UVEdit::s_uMinimised = 0;

Asura_Vector_2 Tools_UVEdit::s_xScaleOverlayPosition(20.0f,10.0f);
Asura_Vector_2 Tools_UVEdit::s_xSettingsOverlayPosition(30.0f,10.0f);

Tools_UVEdit::Tools_UVEdit()
:m_xEditAreaSize(256.0f, 256.0f)
,m_fScale(256.0f)
,m_bOverrideTextureSize(false)
,m_xTextureSize(256.0f,256.0f)
,m_fTextureBrightness(0.5f)
,m_xTopLeftUV(0.0f,0.0f)
,m_fGridSize(64.0f)
,m_bShowGrid(true)
,m_bButtonXRestrict(false)
,m_bButtonYRestrict(false)
,m_bLockSelection(false)
,m_bUVControlEnabled(false)
,m_bUVControlChanged_U(false)
,m_bUVControlChanged_V(false)
,m_bShowUVAsPixels(false)
,m_xSelectedUV(0.0f,0.0f)
,m_eSelectMode(TOOLS_UVEDITSELECTMODE_VERTEX)
,m_bDragXRestrict(false)
,m_bDragYRestrict(false)
,m_bShiftXRestrict(false)
,m_bShiftYRestrict(false)
,m_bUpdateOverlayControls(false)
,m_iRelaxNode_NumIteration(100)
,m_fRelaxNode_Amount(0.1f)
,m_bRelaxNode_LockBoundary(false)
,m_fRotationSnap(Asura_Maths::DegToRad(45.0f))
,m_bScaleOverlay_Active(false)
,m_fScaleOverlay_U(1.0f)
,m_fScaleOverlay_V(1.0f)
,m_bScaleOverlay_Uniform(true)
,m_bScaleOverlay_UChanged(false)
,m_bScaleOverlay_VChanged(false)
,m_bRenderSelectedVerts(true)
,m_bRenderCursorVert(true)
,m_pxNodeUnderCursor(0)
{
	m_iOverlayID = (int) this;
	m_iEditedChannels = 1;
	m_bShowAllChannels = false;
	m_bNeedToRecreateNodes = false;
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	m_auMaterialResponseHashID[0] = 0;
	m_auMaterialResponseHashID[1] = 0;
	#endif
	m_iMaterialID = ASURA_MATERIAL_NONE;
	m_iTextureID = ASURA_TEXTURE_NONE;
	m_bUseOverrideTexture = false;
	m_iOverrideTextureID = ASURA_TEXTURE_NONE;

	m_eMouseMode = TOOLS_UVEDITMOUSEMODE_FREEFORM;

	m_eDragMode = TOOLS_UVEDITDRAGMODE_NONE;
	m_xMouseDragStartRelative = Asura_Maths::ZeroVector2;
	m_xMouseDragStartScreen = Asura_Maths::ZeroVector2;
	m_bIsMouseDrag = false;

	m_xSelectionRectangleUV.Set(0.0f,0.0f,0.0f,0.0f);
	m_xSelectionCentreUV = Asura_Maths::ZeroVector2;

	m_xPreDragSelectionRectangleUV.Set(0.0f,0.0f,0.0f,0.0f);
	m_xPreDragSelectionCentreUV = Asura_Maths::ZeroVector2;
	m_xPreDragTopLeftUV = Asura_Maths::ZeroVector2;
	m_xPreDragEditAreaSize = Asura_Maths::ZeroVector2;
	m_xPreDragOverlayPosition = Asura_Maths::ZeroVector2;
	m_xPreDragNodeUV = Asura_Maths::ZeroVector2;
	m_xPreScaleSelectionCentreUV = Asura_Maths::ZeroVector2;

	SetDefaultColours();

	m_pxNodeManager = new Tools_UVEdit_NodeManager_Environment(this);
	m_uPolygonSelectionFrameStamp = 0;


	m_eSnapMode = TOOLS_UVEDITSNAP_NONE;

	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	Tools_Help::AddHelpTopic("UV Edit Overlay",g_aszUVEditHelpStrings);

}

Tools_UVEdit::~Tools_UVEdit()
{
	Tools_Help::RemoveHelpTopic("UV Edit Overlay");

	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	CloseOverlay();
	delete m_pxNodeManager;
}

void Tools_UVEdit::TransferModeData(int iMode)
{
	//open the file
	Tools_ModeDataFile xFile("UVEdit.dat",iMode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(m_xEditAreaSize);
	xFile.Transfer(m_fTextureBrightness);
	xFile.Transfer(m_fGridSize);
	xFile.Transfer(m_bShowGrid);
	xFile.Transfer(m_eMouseMode);
	xFile.Transfer(m_eSnapMode);
	xFile.Transfer(m_eSelectMode);
	xFile.Transfer(s_xPosition);
	xFile.Transfer(m_iRelaxNode_NumIteration);
	xFile.Transfer(m_fRelaxNode_Amount);
	xFile.Transfer(m_bRelaxNode_LockBoundary);
	xFile.Transfer(m_fRotationSnap);
	xFile.Transfer(m_uSelectedNodeColour);
	xFile.Transfer(m_uUnselectedNodeColour);
	xFile.Transfer(m_uEquivalentNodeColour);
	xFile.Transfer(m_uPrimaryEdgeColour);
	xFile.Transfer(m_uSecondaryEdgeColour);
	xFile.Transfer(m_uSelectionRectangleColour);
	xFile.Transfer(m_uGridColour);
	xFile.Transfer(m_uGridAxisColour);
	xFile.Transfer(s_xScaleOverlayPosition);
	xFile.Transfer(s_xSettingsOverlayPosition);
	xFile.Transfer(m_xOverrideTextureName);
	xFile.Transfer(m_bRenderSelectedVerts);
	xFile.Transfer(m_bRenderCursorVert);
	xFile.Transfer(m_uSelectedVertexColour);
	xFile.Transfer(m_uCursorVertexColour);
}

void Tools_UVEdit::ShowOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(m_iOverlayID))
	{
		Asura_Overlay_System::BringOverlayToTop(m_iOverlayID);
		return;
	}

	m_pxNodeManager->UpdateNodes();
	m_pxNodeUnderCursor = 0;
	m_uPolygonSelectionFrameStamp = Tools_Polygon_Selection::GetSelectionFrameStamp();


	Asura_Overlay* pxOverlay = new Tools_UVEdit_Overlay("UV Edit", false,&s_xPosition, &s_uMinimised);
	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	//test (probably) controls for resizing the edit area
//	pxDialog = new Asura_Numerical_Dialog("W", xOffset, &m_xEditAreaSize.x, 64.0f, 1024.0f, 1.0f, "%.0f");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.x = pxDialog->MaxCoord.x + 1.0f;

//	pxDialog = new Asura_Numerical_Dialog("H", xOffset, &m_xEditAreaSize.y, 64.0f, 1024.0f, 1.0f, "%.0f");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.x = 1.0f;
//	xOffset.y += Asura_Dialog::DefaultSpacing();

	//buttons for setting the mouse mode
	Asura_Icon_Dialog* pxIcon;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_MOVE);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_MouseMode_Move), CALLBACK_VOID_BOOL(SetChecked_MouseMode_Move));
	pxIcon->SetHelpText("Move");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_ROTATE);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_MouseMode_Rotate), CALLBACK_VOID_BOOL(SetChecked_MouseMode_Rotate));
	pxIcon->SetHelpText("Rotate");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_SCALE);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_MouseMode_Scale), CALLBACK_VOID_BOOL(SetChecked_MouseMode_Scale));
	pxIcon->SetHelpText("Scale");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;


	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_FREEFORM);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_MouseMode_Freeform), CALLBACK_VOID_BOOL(SetChecked_MouseMode_Freeform));
	pxIcon->SetHelpText("Freeform");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;


	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'X'-'A');
	pxIcon->SetCheckboxBool(&m_bButtonXRestrict);
	pxIcon->SetHelpText("Restrict movement to the X axis.\nAlternatively , holding down shift while dragging will restrict movement to one axis");
	pxIcon->SetRadioButtonGroup(1, false);
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'Y'-'A');
	pxIcon->SetCheckboxBool(&m_bButtonYRestrict);
	pxIcon->SetHelpText("Restrict movement to the Y axis.\nAlternatively , holding down shift while dragging will restrict movement to one axis");
	pxIcon->SetRadioButtonGroup(1, false);
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_MIRROR_HORI);
	pxIcon->SetClickedFunction(CALLBACK_VOID(MirrorHorizontal));
	pxIcon->SetHelpText("Mirror horizontal");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_MIRROR_VERT);
	pxIcon->SetClickedFunction(CALLBACK_VOID(MirrorVertical));
	pxIcon->SetHelpText("Mirror vertical");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'R'-'A');
	pxIcon->SetClickedFunction(CALLBACK_VOID(RelaxNodes));
	pxIcon->SetHelpText("Relax Nodes.\nMoves the selected nodes in an attempt to reduce any skewed or stretched texturing.\nThe number of iteration and amount of movement can be changed in the 'settings' tab.");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'W'-'A');
	pxIcon->SetClickedFunction(CALLBACK_VOID(WeldNodes));
	pxIcon->SetHelpText("Weld Nodes.\nAny selected nodes that correspond to vertices with the same world position will have their UVs set to be the same.");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;


	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_ZOOMEXTENT);
	pxIcon->SetClickedFunction(CALLBACK_VOID(ZoomExtents));
	pxIcon->SetHelpText("Zoom extents\nShortcut - Z");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;


	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'G'-'A');
	pxIcon->SetCheckboxBool(&m_bShowGrid);
	pxIcon->SetHelpText("Show grid");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_EDITPRIMARY);
	pxIcon->SetCheckboxFlag(&m_iEditedChannels, 0x1);
	pxIcon->SetClickedFlag(&m_bNeedToRecreateNodes);
	pxIcon->SetHelpText("Edit primary UVs.");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_EDITSECONDARY);
	pxIcon->SetCheckboxFlag(&m_iEditedChannels, 0x2);
	pxIcon->SetClickedFlag(&m_bNeedToRecreateNodes);
	pxIcon->SetHelpText("Edit secondary UVs.");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;


	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_SHOWALLCHANNEL);
	pxIcon->SetCheckboxBool(&m_bShowAllChannels);
	pxIcon->SetClickedFlag(&m_bNeedToRecreateNodes);
	pxIcon->SetHelpText("Show all UV channels, with the non-edited ones being greyed out.");
	pxOverlay->AddDialog(pxIcon);
	xOffset.x = pxIcon->m_xMaxCoord.x;



	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();


	//the main uv editing thing
	Tools_UVEdit_Dialog* pxUVDialog = new Tools_UVEdit_Dialog(xOffset, this);
	pxOverlay->AddDialog(pxUVDialog);
	xOffset.y = pxUVDialog->m_xMaxCoord.y + 0.5f;



	//grid snap options
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'G'-'A');
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_SnapMode_Grid), CALLBACK_VOID_BOOL(SetChecked_SnapMode_Grid));
	pxIcon->SetHelpText("Snap to grid - Only used if 'Move' mode is selected.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'P'-'A');
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_SnapMode_Pixel), CALLBACK_VOID_BOOL(SetChecked_SnapMode_Pixel));
	pxIcon->SetHelpText("Snap to pixel - Only used if 'Move' mode is selected.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'V'-'A');
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_SnapMode_Vertex), CALLBACK_VOID_BOOL(SetChecked_SnapMode_Vertex));
	pxIcon->SetHelpText("Snap to vertex/node - Only used if 'Move' mode is selected.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	xOffset.x += 1.0f;

	//selection mode
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_VERTEX);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_SelectMode_Vertex), CALLBACK_VOID_BOOL(SetChecked_SelectMode_Vertex));
	pxIcon->SetHelpText("Vertex selection.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_EDGE);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_SelectMode_Edge), CALLBACK_VOID_BOOL(SetChecked_SelectMode_Edge));
	pxIcon->SetHelpText("Edge selection.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;

	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_POLYGON);
	pxIcon->SetCheckboxFunction(CALLBACK_BOOL(IsChecked_SelectMode_Polygon), CALLBACK_VOID_BOOL(SetChecked_SelectMode_Polygon));
	pxIcon->SetHelpText("Polygon selection.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;
	xOffset.x += 1.0f;

	//lock selection
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_PADLOCK);
	pxIcon->SetCheckboxBool(&m_bLockSelection);
	pxIcon->SetHelpText("Lock selection.\nShortcut - Space (while cursor is over main UV editing control).");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;
	xOffset.x += 1.0f;

	xOffset.x += 1.0f;
	pxDialog = pxIcon = new Asura_Icon_Dialog(xOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'S'-'A');
	pxIcon->SetClickedFunction(CALLBACK_VOID(OpenScaleOverlay));
	pxIcon->SetHelpText("Opens the UV scale overlay which allows the UVs to be scaled by a specified amount.");
	pxOverlay->AddDialog(pxIcon);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxIcon->m_xMaxCoord.x;




	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	//setup some tabs below
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,0,0);
	pxOverlay->AddDialog(pxTabsDialog);
	pxTabsDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	
	//UV tab
	pxTabsDialog->AddTab("Set UV");
		
	//Numeric controls for showing/editing the UV of the selected node

	pxDialog = new Asura_Tickbox_Dialog("Show As Pixels", xOffset, &m_bShowUVAsPixels, &m_bUpdateOverlayControls);
	pxDialog->SetHelpText("Determines whether the UVs will be shown as pixel coordinates");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ScaledNumerical_Dialog("U", xOffset, 1.0f, &m_xSelectedUV.x, -999.0f, 999.0f, 0.1f, "%.4f", &m_bUVControlChanged_U);
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_UVControl));
	pxDialog->m_iControlID = g_uDialog_U;
	pxDialog->SetHelpText("Sets the UV for the selected verts.");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ScaledNumerical_Dialog("V", xOffset, 1.0f, &m_xSelectedUV.y, -999.0f, 999.0f, 0.1f, "%.4f", &m_bUVControlChanged_V);
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_UVControl));
	pxDialog->m_iControlID = g_uDialog_V;

	pxDialog->SetHelpText("Sets the UV for the selected verts.");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//end of uv tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//texture tab
	pxTabsDialog->AddTab("Texture");
	xOffset.y = fTabsStartY;


	pxDialog = new Asura_Tickbox_Dialog("Texture Size", xOffset, &m_bOverrideTextureSize);
	pxDialog->SetHelpText("Tick to override the texture size so that snap to pixel will snap to different values.");
	((Asura_Tickbox_Dialog*)pxDialog)->SetClickedFunction(CALLBACK_VOID(UpdateTextureSize));
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("W", xOffset, &m_xTextureSize.x, 1.0f, 9999.0f, 1.0f, "%.0f", &m_bUpdateOverlayControls);
	pxDialog->SetHelpText("The width of the current texture , or the override value for the width.");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_TextureSize));
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxDialog->GetMaxCoords().x + 1.0f;

	pxDialog = new Asura_Numerical_Dialog("H", xOffset, &m_xTextureSize.y, 1.0f, 9999.0f, 1.0f, "%.0f", &m_bUpdateOverlayControls);
	pxDialog->SetHelpText("The height of the current texture , or the override value for the width.");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_TextureSize));
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	pxDialog = new Asura_Numerical_Dialog("Bright", xOffset, &m_fTextureBrightness, 0.0f, 1.0f, 0.1f, "%.2f");
	pxDialog->SetHelpText("How bright to draw the texture.");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("", xOffset, &m_bUseOverrideTexture);
	pxDialog->SetHelpText("Enables the override texture");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.x = pxDialog->GetMaxCoords().x + 0.1f;

	pxDialog = new Asura_Function_Dialog("Set Override", xOffset, CALLBACK_VOID(SetOverrideTexture));
	pxDialog->SetHelpText("Forces the specified texture to be displayed in the UV editing window , instead of whatever texture is being used by the polygons");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;


	//end of texture tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	#if 0
	//settings tab
	pxTabsDialog->AddTab("Settings");
	xOffset.y = fTabsStartY;
	
	pxDialog = new Asura_Numerical_Dialog("Grid Size", xOffset, &m_fGridSize, 1.0f, 9999.0f, 1.0f, "%.0f");
	pxDialog->SetHelpText("The spacing of the grid lines in pixels.");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Numerical_Dialog("Relax Iterations", xOffset, &m_iRelaxNode_NumIteration, 0, 999, 1, "%d");
	pxDialog->SetHelpText("The number of iterations to run each time 'Relax Nodes' is used");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Relax Amount", xOffset, &m_fRelaxNode_Amount, 0.01f, 1.0f, 0.1f, "%.2f");
	pxDialog->SetHelpText("The amount each node should attempt to move towards its desired destination in each iteration of 'Relax Nodes'");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Relax - Lock Boundary", xOffset, &m_bRelaxNode_LockBoundary);
	pxDialog->SetHelpText("Prevents the boundary of the selection from moving when using 'Relax Nodes'");
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxUVDialog,true,false);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	//end of settings tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;
	#endif

	//end of tabs
	xOffset.y = fTabsEndY + Asura_Dialog::DefaultSpacing();
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);

    pxDialog = new Asura_Function_Dialog( "Edit Settings", xOffset, CALLBACK_VOID(OpenSettingsOverlay) );
	pxDialog->SetHelpText("Opens the UV Editor Settings overlay , which allows various preferences to be set");
	pxDialog->SetRelativePosition(pxTabsDialog, true, false);
    pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_Function_Dialog( "Show Keys", xOffset, ShowKeys );
	pxDialog->SetHelpText("Shows all the shortcut keys");
	pxDialog->SetRelativePosition(pxTabsDialog, true, false);
    pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();



	pxDialog = new Asura_Resize_Dialog(pxUVDialog, NULL);
	pxOverlay->AddDialog(pxDialog);


	pxOverlay->m_uID = m_iOverlayID;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	UpdateMaterial();
	UpdateOverlayControls();

}

void Tools_UVEdit::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(m_iOverlayID);	
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_UVEDIT_SETTINGS);
	CloseScaleOverlay();
}

void Tools_UVEdit::ToggleOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(m_iOverlayID))
	{
		CloseOverlay();
	}
	else
	{
		ShowOverlay();
	}
}

bool Tools_UVEdit::IsOverlayOpen() const
{
	return Asura_Overlay_System::DoesOverlayExist(m_iOverlayID);
}

void Tools_UVEdit::Update()
{
	if(m_bNeedToRecreateNodes || m_uPolygonSelectionFrameStamp != Tools_Polygon_Selection::GetSelectionFrameStamp())
	{
		m_bNeedToRecreateNodes = false;
		m_pxNodeManager->UpdateNodes();
		m_uPolygonSelectionFrameStamp = Tools_Polygon_Selection::GetSelectionFrameStamp();
		m_pxNodeUnderCursor = 0;

		UpdateMaterial();
	}

	UpdateTextureSize();

	if(m_bUpdateOverlayControls)
	{
		UpdateOverlayControls();
	}

	UpdateUVControl();
	UpdateScaleOverlay();

	m_pxNodeManager->Update();
}


void Tools_UVEdit::UpdateOverlayControls()
{
	m_bUpdateOverlayControls = false;

	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(m_iOverlayID);

	//update the UV controls according to whether they should be being displayed as pixel coordinates

	Asura_ScaledNumerical_Dialog* pxDialog = (Asura_ScaledNumerical_Dialog*) pxOverlay->GetDialog(g_uDialog_U);
	if(pxDialog)
	{
		if(m_bShowUVAsPixels)
		{
			pxDialog->SetPrintFormat("%.0f");
			pxDialog->SetScale(m_xTextureSize.x,-999999.0f,999999.0f);
			pxDialog->SetDelta(1.0f);
		}
		else
		{
			pxDialog->SetScale(1.0f, -999.0f, 999.0f);
			pxDialog->SetPrintFormat("%.4f");
			pxDialog->SetDelta(0.1f);
		}
	}

	pxDialog = (Asura_ScaledNumerical_Dialog*) pxOverlay->GetDialog(g_uDialog_V);
	if(pxDialog)
	{
		if(m_bShowUVAsPixels)
		{
			pxDialog->SetPrintFormat("%.0f");
			pxDialog->SetScale(m_xTextureSize.y,-999999.0f,999999.0f);
			pxDialog->SetDelta(1.0f);
		}
		else
		{
			pxDialog->SetScale(1.0f, -999.0f, 999.0f);
			pxDialog->SetPrintFormat("%.4f");
			pxDialog->SetDelta(0.1f);
		}
	}
}

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
void Tools_UVEdit::SetMaterialResponse(int iChannel, Asura_Hash_ID uMaterialResponseHashID)
{
	m_iMaterialID = ASURA_MATERIAL_NONE;
	m_iTextureID = ASURA_TEXTURE_NONE;

	m_auMaterialResponseHashID[iChannel] = uMaterialResponseHashID;
}
#endif 

void Tools_UVEdit::SetMaterialID(int iMaterialID)
{
	ClearMaterialAndTexture();
	m_iMaterialID = iMaterialID;
}

void Tools_UVEdit::SetTextureID(int iTextureID)
{
	ClearMaterialAndTexture();
	m_iTextureID = iTextureID;
}

void Tools_UVEdit::UpdateTextureSize()
{
	if(m_bOverrideTextureSize) return;

	int iTextureID = GetTextureToDisplay();
	if(iTextureID != ASURA_TEXTURE_NONE)
	{
		int iW,iH;
		if(Tools_Texture_Management::GetTextureSize(iTextureID, iW, iH))
		{
			float fW = (float) iW;
			float fH = (float) iH;

			if(m_xTextureSize.x != fW || m_xTextureSize.y != fH)
			{
				m_xTextureSize.x = (float) iW;
				m_xTextureSize.y = (float) iH;

				//If the texture size has changed then the UV control may need updating
				m_bUpdateOverlayControls = true;			
			}
		}
	}
}

void Tools_UVEdit::ClearMaterialAndTexture()
{
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	m_auMaterialResponseHashID[0] = ASURA_HASH_ID_UNSET;
	m_auMaterialResponseHashID[1] = ASURA_HASH_ID_UNSET;
	#endif
	m_iMaterialID = ASURA_MATERIAL_NONE;
	m_iTextureID = ASURA_TEXTURE_NONE;

}

int Tools_UVEdit::GetTextureToDisplay() const
{
	if(m_bUseOverrideTexture)
	{
		if(m_iOverrideTextureID == ASURA_TEXTURE_NONE && !m_xOverrideTextureName.IsBlank())
		{
			m_iOverrideTextureID = Asura_Texture_Management::AddTexture(m_xOverrideTextureName.GetString());
			Asura_Texture_Management::LoadTextures();
		}

		if(m_iOverrideTextureID != ASURA_TEXTURE_NONE)
		{
			return m_iOverrideTextureID;
		}
	}

	//show the primary texture , unless only edited the secondary UVs
	int iTextureChannel = (m_iEditedChannels == 0x2) ? 1 : 0;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( m_auMaterialResponseHashID[iTextureChannel] );
	// Make sure we don't get the default material
	if( m_auMaterialResponseHashID[iTextureChannel] != ASURA_HASH_ID_UNSET && pxMaterial )
	{
		return pxMaterial->GetTexture(Asura_MaterialResponse::ALBEDO_TEXTURE);
	}
#endif
	if(m_iMaterialID != ASURA_MATERIAL_NONE)
	{
		return Asura_Material_System::GetTextureID(m_iMaterialID,iTextureChannel);
	}

	return m_iTextureID;
}

Asura_Vector_2 Tools_UVEdit::ConvertUVToPosition(const Asura_Vector_2& xUV, bool bSnapToPixel) const
{
	float fX = (xUV.x - m_xTopLeftUV.x) * GetScaleX();
	float fY = (xUV.y - m_xTopLeftUV.y) * GetScaleY();

	if(bSnapToPixel)
	{
		fX = (float) Asura_Maths::FloatToInt_Near(fX);
		fY = (float) Asura_Maths::FloatToInt_Near(fY);
	}

	return Asura_Vector_2(fX, fY);
}

Asura_Vector_2 Tools_UVEdit::ConvertPositionToUV(const Asura_Vector_2& xPos) const
{
	return m_xTopLeftUV + Asura_Vector_2(xPos.x/GetScaleX(), xPos.y/GetScaleY());
}

Asura_Vector_2 Tools_UVEdit::ConvertUVDeltaToPositionDelta(const Asura_Vector_2& xUV) const
{
	return Asura_Vector_2(xUV.x * GetScaleX(), xUV.y * GetScaleY());
}


void Tools_UVEdit::ZoomIn(const Asura_Vector_2& xZoomFocus)
{

	//get the UV coordinate of the current focus point
	Asura_Vector_2 xUV = ConvertPositionToUV(xZoomFocus);

	//increase the scale
	m_fScale *= 1.4f;

	//set the origin so that is puts the UV at the focus point the same
	m_xTopLeftUV.x = xUV.x - xZoomFocus.x/GetScaleX();
	m_xTopLeftUV.y = xUV.y - xZoomFocus.y/GetScaleY();
}

void Tools_UVEdit::ZoomOut(const Asura_Vector_2& xZoomFocus)
{

	//get the UV coordinate of the current focus point
	Asura_Vector_2 xUV = ConvertPositionToUV(xZoomFocus);

	//decrease the scale
	m_fScale /= 1.4f;

	//set the origin so that is puts the UV at the focus point the same
	m_xTopLeftUV.x = xUV.x - xZoomFocus.x/GetScaleX();
	m_xTopLeftUV.y = xUV.y - xZoomFocus.y/GetScaleY();
}

void Tools_UVEdit::UpdateMaterial()
{
	if(!IsOverlayOpen()) return;

	//Select a texture//material from the selected polygons , giving preference to the current one
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_Hash_ID uMaterial = 0;
	int iTextureChannel = (m_iEditedChannels == 0x2) ? 1 : 0;
	#else
	int iMaterialID = ASURA_MATERIAL_NONE;
	#endif

		

	//For now , just find the first textured polygon
	PolygonID xPolyID;
	Tools_Polygon_Selection::StartIteration();
	while(Tools_Polygon_Selection::GetNextSelected(&xPolyID))
	{
		const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(xPolyID.ModuleIndex);
		Asura_Assert(pxMesh->TextureDataPtr,"");

		const Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[xPolyID.PolyIndex];
		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		if(pxTex->m_auMaterialResponseHashID[iTextureChannel])
		{
			uMaterial = pxTex->m_auMaterialResponseHashID[iTextureChannel];
			if(uMaterial == m_auMaterialResponseHashID[iTextureChannel]) return;
		}
		#else
		if(pxTex->m_iMaterialID != ASURA_MATERIAL_NONE)
		{
			iMaterialID = pxTex->m_iMaterialID;
			if(iMaterialID == m_iMaterialID) return;
		}
		#endif
	}

  	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	SetMaterialResponse(iTextureChannel,uMaterial);					
	#else
	SetMaterialID(iMaterialID);
	#endif

}

void Tools_UVEdit::HandleMouse(const Asura_Vector_2& xRelCursorPos, int iButton)
{
	

 	if (iButton == 0 || Asura_UserInput::DebouncedKeyPressed(iButton))
	{
		//button just clicked , so check where , to determine how the click/drag should be used
		m_eDragMode = TOOLS_UVEDITDRAGMODE_NONE;

		if(iButton == ASURA_KEYID_MOUSEBUTTON1 || iButton == 0)
		{
			if(iButton) CloseScaleOverlay();
			
			
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !m_bLockSelection)
			{
				//Alt is pressed , so in deselection mode regardless of other settings
				m_eDragMode = TOOLS_UVEDITDRAGMODE_DESELECTION;
				if(iButton)
				{
					StorePreDragState(xRelCursorPos);
					m_bIsMouseDrag = false;
				}
			}
			else if(m_eMouseMode == TOOLS_UVEDITMOUSEMODE_FREEFORM)
			{
				HandleLeftMouseClick_Freeform(xRelCursorPos,iButton!=0);
			}
			else
			{
				HandleLeftMouseClick_NonFreeform(xRelCursorPos,iButton!=0);
			}
		}
		else if(iButton == ASURA_KEYID_MOUSEBUTTON3)
		{
			//middle mouse button pans the view
			m_eDragMode = TOOLS_UVEDITDRAGMODE_PAN;
			StorePreDragState(xRelCursorPos);
		}
	}
 	else if (Asura_UserInput::KeyPressed(iButton))
	{
		UpdateAxisRestrict(xRelCursorPos);

		//mouse button held down
		switch(m_eDragMode)
		{
			case TOOLS_UVEDITDRAGMODE_SELECTION :
			case TOOLS_UVEDITDRAGMODE_DESELECTION :
				UpdateDrag_Selection(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_ROTATE :
				UpdateDrag_Rotate(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_FREEFORMROTATE :
				UpdateDrag_FreeformRotate(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_MOVE :
			case TOOLS_UVEDITDRAGMODE_MOVEIFDRAG :
				UpdateDrag_Move(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_SCALE :
				UpdateDrag_Scale(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_STRETCH_TL :
			case TOOLS_UVEDITDRAGMODE_STRETCH_TR :
			case TOOLS_UVEDITDRAGMODE_STRETCH_BL :
			case TOOLS_UVEDITDRAGMODE_STRETCH_BR :
				UpdateDrag_Stretch(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_MOVECENTRE :
				UpdateDrag_MoveCentre(xRelCursorPos);
				break;

			case TOOLS_UVEDITDRAGMODE_PAN :
				UpdateDrag_Pan(xRelCursorPos);
				break;
		}
	}
	else
	{
		//mouse button released
		if(m_eDragMode == TOOLS_UVEDITDRAGMODE_SELECTION || m_eDragMode == TOOLS_UVEDITDRAGMODE_DESELECTION)
		{
			bool bSelect = (m_eDragMode == TOOLS_UVEDITDRAGMODE_SELECTION);
			

			if(bSelect && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
			{
				m_pxNodeManager->ClearSelection();
			}
		
			//do the selection
			if(m_bIsMouseDrag)
			{

				//select everything within the rectangle
				Asura_Bounding_Rectangle xClampedRect = GetClampedDragRectangle(xRelCursorPos);
				Asura_Bounding_Rectangle xUVRect;
				xUVRect.Reset();
				xUVRect.Set(ConvertPositionToUV(Asura_Vector_2(xClampedRect.MinX, xClampedRect.MinY)));
				xUVRect.Set(ConvertPositionToUV(Asura_Vector_2(xClampedRect.MaxX, xClampedRect.MaxY)));

				
				if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_VERTEX)
				{
					for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
					{
						Tools_UVEdit_Node* pxNode = xNodeLIF();
						if(!pxNode->IsSelectable()) continue;

						if(xUVRect.Contains(pxNode->GetUV()))
						{
							if(bSelect)
								m_pxNodeManager->Select(pxNode, false);
							else
								m_pxNodeManager->Deselect(pxNode, false);
						}
					}
				}
				else if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_EDGE)
				{
					for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
					{
						Tools_UVEdit_Node* pxNode = xNodeLIF();
						if(!pxNode->IsSelectable()) continue;
							
						if(RectangleIntersectsEdge(xUVRect, pxNode))
						{
							if(bSelect)
								m_pxNodeManager->Select(pxNode, m_eSelectMode,true);
							else
								m_pxNodeManager->Deselect(pxNode, m_eSelectMode,true);
						}
					}
				}
				else if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_POLYGON)
				{
					for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
					{
						Tools_UVEdit_Node* pxNode = xNodeLIF();
						if(!pxNode->IsSelectable()) continue;
						if(pxNode->GetIndexInPoly() != 0) continue;
							
						if(RectangleIntersectsPolygon(xUVRect, pxNode))
						{
							if(bSelect)
								m_pxNodeManager->Select(pxNode, m_eSelectMode,true);
							else
								m_pxNodeManager->Deselect(pxNode, m_eSelectMode,true);

						}
					}
				}

			
			}
			else
			{
				
				if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_VERTEX)
				{

					float fDist;
					Tools_UVEdit_Node* pxNearNode = GetNearestNode(xRelCursorPos, fDist);
					if(pxNearNode && fDist < 10.0f)
					{
						if(bSelect)
							m_pxNodeManager->Select(pxNearNode, true);
						else
							m_pxNodeManager->Deselect(pxNearNode, true);
					}
				}
				else if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_EDGE)
				{
					float fDist;
					Tools_UVEdit_Node* pxNearNode = GetNearestEdge(xRelCursorPos, fDist);
					if(pxNearNode && fDist < 10.0f)
					{
						if(bSelect)
							m_pxNodeManager->Select(pxNearNode, m_eSelectMode, true);
						else
							m_pxNodeManager->Deselect(pxNearNode, m_eSelectMode, true);
					}
				}
				else if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_POLYGON)
				{
					Tools_UVEdit_Node* pxNearNode = GetPolygon(xRelCursorPos);
					if(pxNearNode)
					{
						if(bSelect)
							m_pxNodeManager->Select(pxNearNode, m_eSelectMode, true);
						else
							m_pxNodeManager->Deselect(pxNearNode, m_eSelectMode, true);
					}
				}
			}
			UpdateSelectionRectangle(true);

			UpdateEnabledStateForUVControl();
		}

		m_eDragMode = TOOLS_UVEDITDRAGMODE_NONE;
	}

	if(!iButton)
	{
		m_eDragMode = TOOLS_UVEDITDRAGMODE_NONE;
	}

	{
		float fDist;
		Tools_UVEdit_Node* pxNode = GetNearestNode(xRelCursorPos, fDist);
		if(pxNode && fDist < g_fNodeSelectionDistance)
		{
			m_pxNodeUnderCursor = pxNode;
		}
	}


}

void Tools_UVEdit::HandleLeftMouseClick_Freeform(const Asura_Vector_2& xRelCursorPos, bool bClicked)
{
	if(IsAnySelection())
	{
		Asura_Bounding_Rectangle xSelRect = GetSelectionRectangleToRender();

		//cursor near the centre?
		if(ShouldShowSelectionCentre())
		{
			Asura_Vector_2 xDiff = xRelCursorPos - GetSelectionCentre();
			if(Asura_Maths::Abs(xDiff.x) <= g_fCentreHalfWidth && Asura_Maths::Abs(xDiff.y) <= g_fCentreHalfWidth)
			{
				m_eDragMode = TOOLS_UVEDITDRAGMODE_MOVECENTRE;
				Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_MOVELARGE);
			}
		}
		
		//cursor near one of the corners?
		for(int i=0; i<4; i++)
		{
			Asura_Vector_2 xCorner;
			xCorner.x = (i & 1) ?  (xSelRect.MaxX+g_fDragGizmoOffset) : ((xSelRect.MinX-g_fDragGizmoOffset));
			xCorner.y = (i & 2) ?  (xSelRect.MaxY+g_fDragGizmoOffset) : ((xSelRect.MinY-g_fDragGizmoOffset));

			Asura_Vector_2 xDiff = xRelCursorPos - xCorner;
			if(Asura_Maths::Abs(xDiff.x) <= g_fDragGizmoHalfWidth && Asura_Maths::Abs(xDiff.y) <= g_fDragGizmoHalfWidth)
			{
				m_eDragMode = (Tools_UVEditDragMode)(TOOLS_UVEDITDRAGMODE_STRETCH_TL + i);
				Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_SCALE);
				break;
			}
		}

		if(m_eDragMode == TOOLS_UVEDITDRAGMODE_NONE)
		{
			//near the centre of one of the edges?
			for(int i=0;i<4;i++)
			{
				Asura_Vector_2 xCentre = xSelRect.CalculateCentre();

				switch(i)
				{
					case 0 : xCentre.x = xSelRect.MinX - g_fDragGizmoOffset; break;
					case 1 : xCentre.x = xSelRect.MaxX + g_fDragGizmoOffset; break;
					case 2 : xCentre.y = xSelRect.MinY - g_fDragGizmoOffset; break;
					case 3 : xCentre.y = xSelRect.MaxY + g_fDragGizmoOffset; break;
				}

				Asura_Vector_2 xDiff = xRelCursorPos - xCentre;
				if(Asura_Maths::Abs(xDiff.x) <= g_fDragGizmoHalfWidth && Asura_Maths::Abs(xDiff.y) <= g_fDragGizmoHalfWidth)
				{
					m_eDragMode = TOOLS_UVEDITDRAGMODE_FREEFORMROTATE;
					Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_ROTATE);
					break;
				}
			}
		}

		if(m_eDragMode == TOOLS_UVEDITDRAGMODE_NONE)
		{
			//within the selection rectangle?
			if(xSelRect.Contains(xRelCursorPos))
			{
				m_eDragMode = TOOLS_UVEDITDRAGMODE_MOVE;
				Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_MOVE);
			}
		}
	}

	if(m_eDragMode == TOOLS_UVEDITDRAGMODE_NONE && !m_bLockSelection)
	{
		//not clicked near anything else,so go for selection
		m_eDragMode = TOOLS_UVEDITDRAGMODE_SELECTION;
	}

	if(bClicked && m_eDragMode != TOOLS_UVEDITDRAGMODE_NONE)
	{
		StorePreDragState(xRelCursorPos);

		if(m_eDragMode != TOOLS_UVEDITDRAGMODE_SELECTION && 
			m_eDragMode != TOOLS_UVEDITDRAGMODE_MOVECENTRE)
		{
			m_pxNodeManager->UpdateInitialUVs();
			m_pxNodeManager->PrepareUndo();
		}
	}
}

void Tools_UVEdit::HandleLeftMouseClick_NonFreeform(const Asura_Vector_2& xRelCursorPos, bool bClicked)
{
	
	
	//First see if the click was near any of the nodes
	Tools_UVEdit_Node* pxNearNode = 0;
	if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_VERTEX)
	{

		float fDist;
		Tools_UVEdit_Node* pxNode = GetNearestNode(xRelCursorPos, fDist);
		if(pxNode && fDist < g_fNodeSelectionDistance)
		{
			pxNearNode = pxNode;
		}
	}
	else if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_EDGE)
	{
		float fDist;
		Tools_UVEdit_Node* pxNode = GetNearestEdge(xRelCursorPos, fDist);
		if(pxNode && fDist < g_fNodeSelectionDistance)
		{
			pxNearNode = pxNode;
		}
	}
	else if(m_eSelectMode == TOOLS_UVEDITSELECTMODE_POLYGON)
	{
		pxNearNode = GetPolygon(xRelCursorPos);
	}


	if(pxNearNode)
	{
		bool bSelected = pxNearNode->IsSelected(m_eSelectMode);
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			//control pressed,  so just toggle selection of the node
			if(!m_bLockSelection)
			{
				if(bClicked)
				{
					if(bSelected)
					{
						m_pxNodeManager->Deselect(pxNearNode, m_eSelectMode,true);
					}
					else
					{
						m_pxNodeManager->Select(pxNearNode, m_eSelectMode,true);
					}
					UpdateEnabledStateForUVControl();
				}
				Tools_WindowsCursor::SetCursor(TOOLS_UVEDITCURSOR_SINGLESELECT);
			}
		}
		else
		{
			if(!bSelected && !m_bLockSelection)
			{
				//node wasn't previously selected , so replace the selection
				if(bClicked)
				{
					m_pxNodeManager->ClearSelection();
					m_pxNodeManager->Select(pxNearNode, m_eSelectMode, true);
					if(m_eMouseMode == TOOLS_UVEDITMOUSEMODE_MOVE)
					{
						m_eDragMode = TOOLS_UVEDITDRAGMODE_MOVEIFDRAG;
					}
					UpdateEnabledStateForUVControl();
				}
				Tools_WindowsCursor::SetCursor(TOOLS_UVEDITCURSOR_SINGLESELECT);
			}
			else
			{
				//Start drag based on the current mouse mode
				switch(m_eMouseMode)
				{
					case TOOLS_UVEDITMOUSEMODE_MOVE:
						m_eDragMode = TOOLS_UVEDITDRAGMODE_MOVE;
						Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_MOVE);
						break;

					case TOOLS_UVEDITMOUSEMODE_ROTATE:
						m_eDragMode = TOOLS_UVEDITDRAGMODE_ROTATE;
						Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_ROTATE);
						break;

					case TOOLS_UVEDITMOUSEMODE_SCALE:
						m_eDragMode = TOOLS_UVEDITDRAGMODE_SCALE;
						Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_SCALE);
						break;
				}
				if(bClicked)
				{
					m_pxNodeManager->UpdateInitialUVs();
					m_pxNodeManager->PrepareUndo();
				}
 			}
			
			if(bClicked)
			{
				UpdateSelectionRectangle(true);
				StorePreDragState(xRelCursorPos);
				m_xPreDragNodeUV = pxNearNode->GetUV();
			}


		}
	}
	else
	{
		//Not clicked near a node, so do drag selection
		if(bClicked && !m_bLockSelection)
		{
			m_eDragMode = TOOLS_UVEDITDRAGMODE_SELECTION;
			StorePreDragState(xRelCursorPos);
			m_bIsMouseDrag = false;
		}
	}
}

void Tools_UVEdit::StorePreDragState(const Asura_Vector_2& xRelCursorPos)
{
	int iX,iY;
	Asura_Cursor::GetScreenSpaceCoords(iX, iY);
	m_xMouseDragStartScreen.x = (float)iX;
	m_xMouseDragStartScreen.y = (float)iY;


	m_xMouseDragStartRelative = xRelCursorPos;
	m_xPreDragSelectionRectangleUV = m_xSelectionRectangleUV;
	m_xPreDragSelectionCentreUV = m_xSelectionCentreUV;
	m_xPreDragTopLeftUV = m_xTopLeftUV;
	m_xPreDragEditAreaSize = m_xEditAreaSize;
	m_xPreDragOverlayPosition = s_xPosition;
	m_xPreDragNodeUV = Asura_Maths::ZeroVector2;
	m_bIsMouseDrag = false;
	m_bDragXRestrict = false;
	m_bDragYRestrict = false;
	m_bShiftXRestrict = false;
	m_bShiftYRestrict = false;
}

void Tools_UVEdit::UpdateAxisRestrict(const Asura_Vector_2& xRelCursorPos)
{
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		m_bDragXRestrict = false;
		m_bDragYRestrict = false;

		//see if the axis restrict direction has been determined yet
		if(m_bShiftXRestrict)
		{
			m_bDragXRestrict = true;
		}
		else if(m_bShiftYRestrict)
		{
			m_bDragYRestrict = true;
		}
		else
		{
			Asura_Vector_2 xDiff = xRelCursorPos - m_xMouseDragStartRelative;
			if(Asura_Maths::Abs(xDiff.x) > Asura_Maths::Abs(xDiff.y))
			{
				m_bDragXRestrict = true;
				if(Asura_Maths::Abs(xDiff.x) > 8.0f )
				{
					m_bShiftXRestrict = true;
				}
			}
			else
			{
				m_bDragYRestrict = true;
				if(Asura_Maths::Abs(xDiff.y) > 8.0f )
				{
					m_bShiftYRestrict = true;
				}
			}
		}
	}
	else
	{
		//use the restrict values taken from the button/icons
		m_bDragXRestrict = m_bButtonXRestrict;
		m_bDragYRestrict = m_bButtonYRestrict;
	}
}

void Tools_UVEdit::UpdateDrag_Selection(const Asura_Vector_2& xRelCursorPos)
{
	//mouse moved enough to be considered a drag selection?
	if(!m_bIsMouseDrag)
	{
		Asura_Vector_2 xDiff = xRelCursorPos - m_xMouseDragStartRelative;
		if(Asura_Maths::Abs(xDiff.x) > 5.0f || Asura_Maths::Abs(xDiff.y) > 5.0f)
		{
			m_bIsMouseDrag = true;
		}
	}
}

void Tools_UVEdit::UpdateDrag_Rotate(const Asura_Vector_2& xRelCursorPos)
{
	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_ROTATE);
	
	//rotate about the centre based on the vertical movement of the mouse
	float fYDiff = xRelCursorPos.y - m_xMouseDragStartRelative.y;
	float fProportionalMove = fYDiff/Asura_Viewport::GetHeight();

	//convert 1/3 screen height to 180 degree rotation
	Asura_Angle fAngle = -fProportionalMove*3.0f*Asura_Maths::Pi;

	//If shift is held then snap the angle to the nearest multiple of m_fRotationSnap
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && m_fRotationSnap > 0.0f)
	{
		fAngle = Asura_Maths::FloatToInt_Near(fAngle / m_fRotationSnap) * m_fRotationSnap;
	}
	

	{
		//rotate all the selected nodes around the centre by this amount
		for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
		{
			Tools_UVEdit_Node* pxNode = xNodeLIF();

			Asura_Vector_2 xUV = pxNode->GetInitialUV();

			//get position relative to the centre
			Asura_Vector_2 xOffset = xUV - m_xSelectionCentreUV;
			
			//rotate the nodes
			xUV.x = xOffset.x*Asura_Maths::Cosine(fAngle) - xOffset.y*Asura_Maths::Sine(fAngle);
			xUV.y = xOffset.x*Asura_Maths::Sine(fAngle) + xOffset.y*Asura_Maths::Cosine(fAngle);
		
			//and add the centre back on
			xUV += m_xSelectionCentreUV;

			pxNode->SetUV(xUV);
		}
	}

	m_pxNodeManager->ApplyUV();

	UpdateSelectionRectangle(false);

}

void Tools_UVEdit::UpdateDrag_FreeformRotate(const Asura_Vector_2& xRelCursorPos)
{
	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_ROTATE);

	Asura_Vector_2 xMouseDragStartUV = ConvertPositionToUV(m_xMouseDragStartRelative);
	Asura_Vector_2 xCursorPosUV = ConvertPositionToUV(xRelCursorPos);


	Asura_Vector_2 xInitialOffset = xMouseDragStartUV - m_xSelectionCentreUV;
	Asura_Vector_2 xCurrentOffset = xCursorPosUV - m_xSelectionCentreUV;
	float fInitialDist = xInitialOffset.Magnitude();
	float fCurrentDist = xCurrentOffset.Magnitude();

	if(fInitialDist>0.001f && fCurrentDist >0.001f)
	{
		//nodes are rotated through an angle equal to the angle that the
		//mouse travels around the centre.the following calculations work
		//out the angle.

		//normalise the offsets
		xInitialOffset.Normalise();
		xCurrentOffset.Normalise();

		Asura_Vector_2 xCurrentOffsetPerp;
		xCurrentOffsetPerp.x = xCurrentOffset.y;
		xCurrentOffsetPerp.y = -xCurrentOffset.x;
		
		float fSinAngle = Asura_Maths::ClampToRange(xInitialOffset*xCurrentOffsetPerp,-1.0f,1.0f);
		Asura_Angle fAngle = Asura_Maths::ArcSine(fSinAngle);
		if(xInitialOffset*xCurrentOffset<0.0f) fAngle = Asura_Maths::Pi-fAngle;

		//If shift is held then snap the angle to the nearest multiple of m_fRotationSnap
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && m_fRotationSnap > 0.0f)
		{
			fAngle = Asura_Maths::FloatToInt_Near(fAngle / m_fRotationSnap) * m_fRotationSnap;
		}

		//rotate all the selected nodes around the centre by this amount
		for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
		{
			Tools_UVEdit_Node* pxNode = xNodeLIF();

			Asura_Vector_2 xUV = pxNode->GetInitialUV();

			//get position relative to the centre
			Asura_Vector_2 xOffset = xUV - m_xSelectionCentreUV;
			
			//rotate the nodes
			xUV.x = xOffset.x*Asura_Maths::Cosine(fAngle) - xOffset.y*Asura_Maths::Sine(fAngle);
			xUV.y = xOffset.x*Asura_Maths::Sine(fAngle) + xOffset.y*Asura_Maths::Cosine(fAngle);
		
			//and add the centre back on
			xUV += m_xSelectionCentreUV;

			pxNode->SetUV(xUV);
		}
	}

	m_pxNodeManager->ApplyUV();

	UpdateSelectionRectangle(false);

}

void Tools_UVEdit::UpdateDrag_Move(const Asura_Vector_2& xRelCursorPos)
{
	if(m_eDragMode == TOOLS_UVEDITDRAGMODE_MOVEIFDRAG)
	{
		//See if the mouse has moved enough to start moving the node
		Asura_Vector_2 xDiff = xRelCursorPos - m_xMouseDragStartRelative;
		if(Asura_Maths::Abs(xDiff.x) > 5.0f || Asura_Maths::Abs(xDiff.y) > 5.0f)
		{
			m_eDragMode = TOOLS_UVEDITDRAGMODE_MOVE;
			m_pxNodeManager->UpdateInitialUVs();
			m_pxNodeManager->PrepareUndo();

		}
		else
		{
			Tools_WindowsCursor::SetCursor(TOOLS_UVEDITCURSOR_SINGLESELECT);
			return;
		}
	}

	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_MOVE);


	Asura_Vector_2 xMouseDragStartUV = ConvertPositionToUV(m_xMouseDragStartRelative);
	Asura_Vector_2 xCursorPosUV = ConvertPositionToUV(xRelCursorPos);

	Asura_Vector_2 xUVDiff = xCursorPosUV - xMouseDragStartUV;

	//If not using freeform movement , check to see if any snapping should be done
	if(m_eMouseMode == TOOLS_UVEDITMOUSEMODE_MOVE && m_eSelectMode == TOOLS_UVEDITSELECTMODE_VERTEX)
	{
		if(m_eSnapMode == TOOLS_UVEDITSNAP_PIXEL)
		{
			//snap cursor pos to the nearest pixel boundary
			Asura_Vector_2 xSnappedUV;
			xSnappedUV.x = Asura_Maths::FloatToInt_Near(xCursorPosUV.x * m_xTextureSize.x) / m_xTextureSize.x;
			xSnappedUV.y = Asura_Maths::FloatToInt_Near(xCursorPosUV.y * m_xTextureSize.y) / m_xTextureSize.y;

			xUVDiff = xSnappedUV - m_xPreDragNodeUV;
		}
		else if(m_eSnapMode == TOOLS_UVEDITSNAP_GRID)
		{
			if(m_fGridSize > 0.0f)
			{

				const float fGridSizeU = GetGridSizeU();
				const float fGridSizeV = GetGridSizeV();
				
				//get the nearest point on the grid
				Asura_Vector_2 xGridUV;
				xGridUV.x = Asura_Maths::FloatToInt_Near(xCursorPosUV.x / fGridSizeU) * fGridSizeU;
				xGridUV.y = Asura_Maths::FloatToInt_Near(xCursorPosUV.y / fGridSizeV) * fGridSizeV;

				Asura_Vector_2 xDiff = xGridUV - xCursorPosUV;

				//tolerance levels set so that while zoomed in it will be based on proximity in UV coordinates, and
				//while zoomed out based on proximity on screen
				float fToleranceU = Asura_Maths::Max(fGridSizeU*0.2f, 2.0f/GetScaleX());
				float fToleranceV = Asura_Maths::Max(fGridSizeV*0.2f, 2.0f/GetScaleY());

				//snap each extent independently as required
				if(Asura_Maths::Abs(xDiff.x) < fToleranceU)
				{
					xUVDiff.x = xGridUV.x - m_xPreDragNodeUV.x;
				}
				if(Asura_Maths::Abs(xDiff.y) < fToleranceV)
				{
					xUVDiff.y = xGridUV.y - m_xPreDragNodeUV.y;
				}

			}
		}
		else if(m_eSnapMode == TOOLS_UVEDITSNAP_VERTEX)
		{
			//first check to see if the cursor is near enough to another unselected node (in pixels)
			float fNearDistSquared = 10.0f*10.0f;
			Tools_UVEdit_Node* pxNearNode = 0;

			for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
			{
				Tools_UVEdit_Node* pxNode = xNodeLIF();
				//interested in un-selected nodes that can be selected
				if(pxNode->IsSelected()) continue;
				if(!pxNode->IsSelectable()) continue;

				float fDistSquared = ConvertUVDeltaToPositionDelta(xCursorPosUV - pxNode->GetUV()).MagnitudeSquared();

				if(fDistSquared < fNearDistSquared)
				{
					fNearDistSquared = fDistSquared;
					pxNearNode = pxNode;
				}
			}

			if(pxNearNode)
			{
				//snap to this nearest node
				xUVDiff = pxNearNode->GetUV() - m_xPreDragNodeUV;
			}
		}
	}

	if(m_bDragXRestrict) xUVDiff.y = 0.0f;
	if(m_bDragYRestrict) xUVDiff.x = 0.0f;


	//add the uv offset to each of the selected nodes
	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();

		pxNode->SetUV(pxNode->GetInitialUV() + xUVDiff);
	}


	m_pxNodeManager->ApplyUV();

	m_xSelectionCentreUV = m_xPreDragSelectionCentreUV + xUVDiff;
	UpdateSelectionRectangle(false);
}

void Tools_UVEdit::UpdateDrag_MoveCentre(const Asura_Vector_2& xRelCursorPos)
{
	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_MOVELARGE);

	Asura_Vector_2 xMouseDragStartUV = ConvertPositionToUV(m_xMouseDragStartRelative);
	Asura_Vector_2 xCursorPosUV = ConvertPositionToUV(xRelCursorPos);

	Asura_Vector_2 xUVDiff = xCursorPosUV - xMouseDragStartUV;
	if(m_bDragXRestrict) xUVDiff.y = 0.0f;
	if(m_bDragYRestrict) xUVDiff.x = 0.0f;

	//add the uv offset to the centre
	m_xSelectionCentreUV = m_xPreDragSelectionCentreUV + xUVDiff;
}


void Tools_UVEdit::UpdateDrag_Scale(const Asura_Vector_2& xRelCursorPos)
{
	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_SCALE);

	//scale about the centre based on the vertical movement of the mouse
	float fYDiff = xRelCursorPos.y - m_xMouseDragStartRelative.y;
	float fProportionalMove = fYDiff/Asura_Viewport::GetHeight();

	//convert 1/6 screen height to a doubling of the size
	float fScale = Asura_Maths::Power(2.0f, -fProportionalMove*6.0f);

	//scale all the selected nodes about the centre

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();

		Asura_Vector_2 xUV = pxNode->GetInitialUV();

		xUV -= m_xPreDragSelectionCentreUV;
		if(!m_bDragYRestrict) xUV.x *= fScale;
		if(!m_bDragXRestrict) xUV.y *= fScale;
		xUV += m_xPreDragSelectionCentreUV;


		pxNode->SetUV(xUV);

	}

	m_pxNodeManager->ApplyUV();
	UpdateSelectionRectangle(true);

}

void Tools_UVEdit::UpdateDrag_Stretch(const Asura_Vector_2& xRelCursorPos)
{
	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_SCALE);

	Asura_Vector_2 xMouseDragStartUV = ConvertPositionToUV(m_xMouseDragStartRelative);
	Asura_Vector_2 xCursorPosUV = ConvertPositionToUV(xRelCursorPos);

	Asura_Vector_2 xUVDiff = xCursorPosUV - xMouseDragStartUV;
	if(m_bDragXRestrict) xUVDiff.y = 0.0f;
	if(m_bDragYRestrict) xUVDiff.x = 0.0f;

	int iStretchType = m_eDragMode - TOOLS_UVEDITDRAGMODE_STRETCH_TL;
	
	//determine the modified selection rectangle
	Asura_Bounding_Rectangle xNewRectUV = m_xPreDragSelectionRectangleUV;

	if(iStretchType & 1)
	{
		xNewRectUV.MaxX += xUVDiff.x;
		if(xNewRectUV.MaxX < xNewRectUV.MinX) xNewRectUV.MaxX = xNewRectUV.MinX;
	}
	else
	{
		xNewRectUV.MinX += xUVDiff.x;
		if(xNewRectUV.MaxX < xNewRectUV.MinX) xNewRectUV.MinX = xNewRectUV.MaxX;
	}

	if(iStretchType & 2)
	{
		xNewRectUV.MaxY += xUVDiff.y;
		if(xNewRectUV.MaxY < xNewRectUV.MinY) xNewRectUV.MaxY = xNewRectUV.MinY;
	}
	else
	{
		xNewRectUV.MinY += xUVDiff.y;
		if(xNewRectUV.MaxY < xNewRectUV.MinY) xNewRectUV.MinY = xNewRectUV.MaxY;
	}

	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && !m_bDragXRestrict && !m_bDragYRestrict)
	{
		//If control is held down , go for a uniform scale
		float fOldWidth = m_xPreDragSelectionRectangleUV.GetWidth();
		float fOldHeight = m_xPreDragSelectionRectangleUV.GetHeight();
		float fNewWidth = xNewRectUV.GetWidth();
		float fNewHeight = xNewRectUV.GetHeight();

		//go with the highest out of the U and V scales , discounting any inifinite scale
		float fXScale = fOldWidth ? fNewWidth/fOldWidth : 0.0f;
		float fYScale = fOldHeight ? fNewHeight/fOldHeight : 0.0f;

		float fScale = Asura_Maths::Max(fXScale,fYScale);

		//recalculate the new rectangle using this scale
		if(iStretchType & 1)
		{
			xNewRectUV.MaxX = xNewRectUV.MinX + fOldWidth*fScale;
		}
		else
		{
			xNewRectUV.MinX = xNewRectUV.MaxX - fOldWidth*fScale;
		}

		if(iStretchType & 2)
		{
			xNewRectUV.MaxY = xNewRectUV.MinY + fOldHeight*fScale;
		}
		else
		{
			xNewRectUV.MinY = xNewRectUV.MaxY - fOldHeight*fScale;
		}

	}

	float fInitialWidth = m_xPreDragSelectionRectangleUV.GetWidth();
	float fInitialHeight = m_xPreDragSelectionRectangleUV.GetHeight();
	float fNewWidth = xNewRectUV.GetWidth();
	float fNewHeight = xNewRectUV.GetHeight();

	//stretch all the selected nodes to match the new rectangle

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();

		Asura_Vector_2 xUV = pxNode->GetInitialUV();

		if(fInitialWidth >= 0.0001f)
		{
			if(iStretchType & 1)
			{
				float fUOffset = xUV.x - m_xPreDragSelectionRectangleUV.MinX;
				float fNewUOffset = (fUOffset/fInitialWidth) * fNewWidth;
				xUV.x = m_xPreDragSelectionRectangleUV.MinX + fNewUOffset;
			}
			else
			{
				float fUOffset = xUV.x - m_xPreDragSelectionRectangleUV.MaxX;
				float fNewUOffset = (fUOffset/fInitialWidth) * fNewWidth;
				xUV.x = m_xPreDragSelectionRectangleUV.MaxX + fNewUOffset;
			}
		}

		if(fInitialHeight >= 0.0001f)
		{
			if(iStretchType & 2)
			{
				float fVOffset = xUV.y - m_xPreDragSelectionRectangleUV.MinY;
				float fNewVOffset = (fVOffset/fInitialHeight) * fNewHeight;
				xUV.y = m_xPreDragSelectionRectangleUV.MinY + fNewVOffset;
			}
			else
			{
				float fVOffset = xUV.y - m_xPreDragSelectionRectangleUV.MaxY;
				float fNewVOffset = (fVOffset/fInitialHeight) * fNewHeight;
				xUV.y = m_xPreDragSelectionRectangleUV.MaxY + fNewVOffset;
			}
		}

		pxNode->SetUV(xUV);

	}

	m_pxNodeManager->ApplyUV();
	UpdateSelectionRectangle(true);

}

void Tools_UVEdit::UpdateDrag_Pan(const Asura_Vector_2& xRelCursorPos)
{
	Tools_WindowsCursor::SetCursor(TOOLS_CURSOR_PAN);

	Asura_Vector_2 xMouseDragDelta = xRelCursorPos - m_xMouseDragStartRelative;
	Asura_Vector_2 xMouseDragDeltaUV( xMouseDragDelta.x/GetScaleX(), xMouseDragDelta.y/GetScaleY());

	m_xTopLeftUV = m_xPreDragTopLeftUV - xMouseDragDeltaUV;
//	m_xTopLeftUV = SnapToNearestTexel(m_xTopLeftUV);
}


void Tools_UVEdit::UpdateDrag_ResizeEditArea(u_int uResizeFlags, const Asura_Vector_2& xDelta)
{
	const float fMinSize = 64.0f;
		
	
	if(uResizeFlags & ASURA_OVERLAY_RESIZE_HORIZONTAL)
	{
		m_xEditAreaSize.x = (float)Asura_Maths::FloatToInt_Near(m_xPreDragEditAreaSize.x + xDelta.x);
		if(m_xEditAreaSize.x < fMinSize)
		{
			m_xEditAreaSize.x = fMinSize;
		}
		

		//Shift the origin so that the UV position at the centre stays the same
		float fDeltaUsed = m_xEditAreaSize.x - m_xPreDragEditAreaSize.x;
		m_xTopLeftUV.x = m_xPreDragTopLeftUV.x-fDeltaUsed/(GetScaleX()*2.0f);

	}

	if(uResizeFlags & ASURA_OVERLAY_RESIZE_VERTICAL)
	{
		m_xEditAreaSize.y = (float)Asura_Maths::FloatToInt_Near(m_xPreDragEditAreaSize.y + xDelta.y);
		if(m_xEditAreaSize.y < fMinSize)
		{
			m_xEditAreaSize.y = fMinSize;
		}

		//Shift the origin so that the UV position at the centre stays the same
		float fDeltaUsed = m_xEditAreaSize.y - m_xPreDragEditAreaSize.y;
		m_xTopLeftUV.y = m_xPreDragTopLeftUV.y-fDeltaUsed/(GetScaleY()*2.0f);
	}
}



const List<Tools_UVEdit_Node*>* Tools_UVEdit::GetNodeList()
{
	return m_pxNodeManager->GetNodeList();
}

const List<Tools_UVEdit_Node*>* Tools_UVEdit::GetSelectedNodeList()
{
	return m_pxNodeManager->GetSelectedNodeList();
}

bool Tools_UVEdit::IsAnySelection() const
{
	return m_pxNodeManager->IsAnySelection();
}

void Tools_UVEdit::UpdateEquivalentToSelectionFlags() const
{
	m_pxNodeManager->UpdateEquivalentToSelectionFlags();
}

void Tools_UVEdit::UpdateSelectionRectangle(bool bUpdateCentre)
{
	if(!IsAnySelection())
	{
		m_xSelectionRectangleUV.Set(0.0f,0.0f,0.0f,0.0f);
		m_xSelectionCentreUV = Asura_Maths::ZeroVector2;
		return;
	}

	m_xSelectionRectangleUV.Reset();
	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		m_xSelectionRectangleUV.Set(xNodeLIF()->GetUV());		
	}

	if(bUpdateCentre)
	{
		m_xSelectionCentreUV = m_xSelectionRectangleUV.CalculateCentre();
	}
}

Asura_Bounding_Rectangle Tools_UVEdit::GetSelectionRectangleToRender() const
{
	const float fMinSize = 20.0f;
	const float fPadAmount = 10.0f;
	const bool bShouldPad = !(m_eDragMode>=TOOLS_UVEDITDRAGMODE_STRETCH_TL && m_eDragMode<=TOOLS_UVEDITDRAGMODE_STRETCH_BR);

	Asura_Bounding_Rectangle xRect;
	if(!IsAnySelection())
	{
		xRect.Set(0.0f,0.0f,0.0f,0.0f);
	}
	else
	{
		//get the cornersof the rectangle in pixel coords
		Asura_Vector_2 xTLUV(m_xSelectionRectangleUV.MinX,m_xSelectionRectangleUV.MinY);
		Asura_Vector_2 xTL = ConvertUVToPosition(xTLUV, true);

		Asura_Vector_2 xBRUV(m_xSelectionRectangleUV.MaxX,m_xSelectionRectangleUV.MaxY);
		Asura_Vector_2 xBR = ConvertUVToPosition(xBRUV, true);

		if(bShouldPad)
		{
			//make sure the rectangle is large enough
			if(xBR.x-xTL.x < fMinSize)
			{
				xBR.x += fPadAmount;
				xTL.x -= fPadAmount;
			}
			if(xBR.y-xTL.y < fMinSize)
			{
				xBR.y += fPadAmount;
				xTL.y -= fPadAmount;
			}
		}

		xRect.Reset();
		xRect.Set(xTL);
		xRect.Set(xBR);
	}

	return xRect;
}

Asura_Bounding_Rectangle Tools_UVEdit::GetClampedDragRectangle(const Asura_Vector_2& xRelCursorPos) const
{
	Asura_Bounding_Rectangle xRect;
	xRect.Reset();
	xRect.Set(m_xMouseDragStartRelative);
	xRect.Set(xRelCursorPos);

	//clamp to the edit area
	xRect.MinX = Asura_Maths::ClampToRange(xRect.MinX, 0.0f, m_xEditAreaSize.x);
	xRect.MaxX = Asura_Maths::ClampToRange(xRect.MaxX, 0.0f, m_xEditAreaSize.x);
	xRect.MinY = Asura_Maths::ClampToRange(xRect.MinY, 0.0f, m_xEditAreaSize.y);
	xRect.MaxY = Asura_Maths::ClampToRange(xRect.MaxY, 0.0f, m_xEditAreaSize.y);

	return xRect;

}

void Tools_UVEdit::UpdateNodesFromSource()
{
	if(!IsOverlayOpen()) return;

	m_pxNodeManager->UpdateNodesFromSource();
	UpdateSelectionRectangle(true);
}

bool Tools_UVEdit::ShouldShowSelectionRectangle() const
{
	if(m_eMouseMode != TOOLS_UVEDITMOUSEMODE_FREEFORM) return false;
	if(!IsAnySelection()) return false;
	return true;
}

bool Tools_UVEdit::ShouldShowSelectionCentre() const
{
	if(!IsAnySelection()) return false;
	if(m_xSelectionRectangleUV.GetWidth() < 0.001f && m_xSelectionRectangleUV.GetHeight() < 0.0001) return false;

	return true;
}

Asura_Vector_2 Tools_UVEdit::GetSelectionCentre() const
{
	return ConvertUVToPosition(m_xSelectionCentreUV,true);
}

Asura_Vector_2 Tools_UVEdit::SnapToNearestTexel(const Asura_Vector_2& xUV) const
{
	Asura_Vector_2 xResult;

	xResult.x = Asura_Maths::FloatToInt_Near(xUV.x*m_xTextureSize.x) / m_xTextureSize.x;
	xResult.y = Asura_Maths::FloatToInt_Near(xUV.y*m_xTextureSize.y) / m_xTextureSize.y;

	return xResult;
}


void Tools_UVEdit::MirrorHorizontal()
{
	if(!IsAnySelection()) return;

	m_pxNodeManager->PrepareUndo();

	//Mirror the UVs about the centre of the selection
	Asura_Vector_2 xCentre = m_xSelectionRectangleUV.CalculateCentre();

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();

		Asura_Vector_2 xUV = pxNode->GetUV();
		xUV.x = 2*xCentre.x - xUV.x;

		pxNode->SetUV(xUV);
	}
	m_pxNodeManager->ApplyUV();
}

void Tools_UVEdit::MirrorVertical()
{
	if(!IsAnySelection()) return;

	m_pxNodeManager->PrepareUndo();

	//Mirror the UVs about the centre of the selection
	Asura_Vector_2 xCentre = m_xSelectionRectangleUV.CalculateCentre();

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();

		Asura_Vector_2 xUV = pxNode->GetUV();
		xUV.y = 2*xCentre.y - xUV.y;

		pxNode->SetUV(xUV);
	}
	m_pxNodeManager->ApplyUV();
}

void Tools_UVEdit::ZoomExtents()
{
	//get the extents of the nodes
	Asura_Bounding_Rectangle xRect;
	xRect.Reset();

	const List<Tools_UVEdit_Node*>* pxNodeList = GetNodeList();
	if(!pxNodeList->size()) return;

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(pxNodeList); !xNodeLIF.done(); xNodeLIF.next())
	{
		xRect.Set(xNodeLIF()->GetUV());
	}

	//pad out the extents to leave a little room around the edge
	float fWidth = xRect.GetWidth();
	float fPaddedWidth = fWidth * 1.1f;
	if(fPaddedWidth < 0.1f)
	{
		float fPad = (0.1f-fPaddedWidth)/2.0f;
		xRect.MinX -= fPad;
		xRect.MaxX += fPad;
		fPaddedWidth = 0.1f;
	}

	float fHeight = xRect.GetHeight();
	float fPaddedHeight = fHeight * 1.1f;
	if(fPaddedHeight < 0.1f)
	{
		float fPad = (0.1f-fPaddedHeight)/2.0f;
		xRect.MinY -= fPad;
		xRect.MaxY += fPad;
		fPaddedHeight = 0.1f;
	}

	m_fScale = Asura_Maths::Min(m_xEditAreaSize.x/fPaddedWidth, m_xEditAreaSize.y/fPaddedHeight * (m_xTextureSize.x/m_xTextureSize.y));

	//centre on the nodes
	Asura_Vector_2 xEditAreaSizeUV(m_xEditAreaSize.x/GetScaleX(), m_xEditAreaSize.y/GetScaleY());

	m_xTopLeftUV.x = xRect.MinX - (xEditAreaSizeUV.x-fWidth)/2.0f;
	m_xTopLeftUV.y = xRect.MinY - (xEditAreaSizeUV.y-fHeight)/2.0f;
}

void Tools_UVEdit::UpdateEnabledStateForUVControl()
{
	m_bUVControlEnabled = m_pxNodeManager->IsAnySelection();
	m_bUVControlChanged_U = false;
	m_bUVControlChanged_V = false;
	
	if(m_bUVControlEnabled)
	{
		//update from the selection
		const List<Tools_UVEdit_Node*>* pxSelectedNodeList = GetSelectedNodeList();
		if(pxSelectedNodeList->size())
		{
			m_xSelectedUV = pxSelectedNodeList->first_entry()->GetUV();
		}
	}	
}

bool Tools_UVEdit::IsEnabled_UVControl()
{
	return m_bUVControlEnabled;
}

void Tools_UVEdit::UpdateUVControl()
{
	if(!m_bUVControlEnabled) return;

	const List<Tools_UVEdit_Node*>* pxSelectedNodeList = GetSelectedNodeList();
	if(m_bUVControlChanged_U || m_bUVControlChanged_V)
	{
		//apply the values from the numerical control
		m_pxNodeManager->PrepareUndo();

		for(CLIF<Tools_UVEdit_Node*> xNodeLIF(pxSelectedNodeList); !xNodeLIF.done(); xNodeLIF.next())
		{
			Tools_UVEdit_Node* pxNode = xNodeLIF();
			//set the altered component
			Asura_Vector_2 xUV = pxNode->GetUV();
			if(m_bUVControlChanged_U) xUV.x = m_xSelectedUV.x;
			if(m_bUVControlChanged_V) xUV.y = m_xSelectedUV.y;
			pxNode->SetUV(xUV);
		}
		m_pxNodeManager->ApplyUV();
		UpdateSelectionRectangle(true);

		m_bUVControlChanged_U = false;
		m_bUVControlChanged_V = false;


	}
	else
	{
		//update the numeric controls from the nodes	
		if(pxSelectedNodeList->size())
		{
			m_xSelectedUV = pxSelectedNodeList->first_entry()->GetUV();
		}
	}
}


Tools_UVEdit_Node* Tools_UVEdit::GetNearestNode(const Asura_Vector_2& xPos,float& fDist)
{
	Tools_UVEdit_Node* pxNearNode = 0;
	float fNearDistSquared = Asura_Maths::LargeFloat;

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();
		if(!pxNode->IsSelectable()) continue;

 		Asura_Vector_2 xNodePos = ConvertUVToPosition(pxNode->GetUV(),false);
		float fDistSquared = (xPos - xNodePos).MagnitudeSquared();

		if(fDistSquared < fNearDistSquared)
		{
			pxNearNode = pxNode;
			fNearDistSquared = fDistSquared;
		}
	}

	if(pxNearNode)
	{
		fDist = Asura_Maths::Sqrt(fNearDistSquared);
	}
	return pxNearNode;
}

Tools_UVEdit_Node* Tools_UVEdit::GetNearestEdge(const Asura_Vector_2& xPos,float& fDist)
{
	Tools_UVEdit_Node* pxNearNode = 0;
	float fNearDistSquared = Asura_Maths::LargeFloat;

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();
		if(!pxNode->IsSelectable()) continue;

		const Tools_UVEdit_Node* pxNextNode = pxNode->GetNextNode();
		if(!pxNextNode || !pxNextNode->IsSelectable()) continue;

 		Asura_Vector_2 xNode1Pos = ConvertUVToPosition(pxNode->GetUV(),false);
 		Asura_Vector_2 xNode2Pos = ConvertUVToPosition(pxNextNode->GetUV(),false);

		
	    // Find the nearest point on the segment. If the edge length is zero,
	    // just take the start of the segment to be the nearest point. Otherwise
	    // work out the nearest point on an infinite line, and then clamp that
	    // to be on the segment (so it gets clamped to the start or end points).
	    const Asura_Vector_2 xLine              = xNode2Pos - xNode1Pos;
	    const float          fLineLengthSquared = xLine.MagnitudeSquared();

	    float fLambda = 0.0f;
	    if ( fLineLengthSquared > 0.0f )
	    {
	        fLambda = Asura_Maths::ClampToRange( ( ( xPos - xNode1Pos ) * xLine ) / fLineLengthSquared, 0.0f, 1.0f );
	    }
	    Asura_Vector_2 xNearestPoint = xNode1Pos + xLine * fLambda;

		float fDistSquared = (xPos - xNearestPoint).MagnitudeSquared();
		
		if(fDistSquared < fNearDistSquared)
		{
			pxNearNode = pxNode;
			fNearDistSquared = fDistSquared;
		}
	}

	if(pxNearNode)
	{
		fDist = Asura_Maths::Sqrt(fNearDistSquared);
	}
	return pxNearNode;
}

Tools_UVEdit_Node* Tools_UVEdit::GetPolygon(const Asura_Vector_2& xPos)
{
	Asura_Vector_2 xUVPos = ConvertPositionToUV(xPos);

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();
		if(!pxNode->IsSelectable()) continue;
		//only consider the first node in each polygon to avoid checking each polygon multiple times
		if(pxNode->GetIndexInPoly() != 0) continue;

		if(PolygonContainsPoint(xUVPos, pxNode)) return pxNode;
	}
	return 0;
}

bool Tools_UVEdit::PolygonContainsPoint(const Asura_Vector_2& xUVPos, const Tools_UVEdit_Node* pxFirstNode) const
{

	//check to see if the point is within the polygon by seeing how many edges a (vertical) line going from it crosses.
	int iEdgesCrossed = 0;
	int iVertIndex = 0;		
	for(const Tools_UVEdit_Node* pxNode1 = pxFirstNode; pxNode1 != pxFirstNode || iVertIndex==0;iVertIndex++, pxNode1 = pxNode1->GetNextNode())
	{
		const Tools_UVEdit_Node* pxNode2 = pxNode1->GetNextNode();
		Asura_Assert(pxNode2,"");

 		const Asura_Vector_2& xNode1Pos = pxNode1->GetUV();
 		const Asura_Vector_2& xNode2Pos = pxNode2->GetUV();

		if(xNode1Pos.x==xUVPos.x) continue;
		if(xNode2Pos.x==xUVPos.x)
		{
			if(xNode2Pos.y<xUVPos.y) continue;
			//first point isn't directly beneath the test position, but the second point is
			//search along the edges till we get the next point to the left/right
			while(pxNode2->GetUV().x == xUVPos.x) pxNode2 = pxNode2->GetNextNode();
			//determine whether the edge turns back the same way or not
			if(xNode1Pos.x<xUVPos.x && pxNode2->GetUV().x<xUVPos.x) continue;
			if(xNode1Pos.x>xUVPos.x && pxNode2->GetUV().x>xUVPos.x) continue;
			iEdgesCrossed++;
			continue;			
		}
		else
		{
			//ignore if both nodes are beneath, to the left ,or to the right of the test position
			if(xNode1Pos.y<=xUVPos.y && xNode2Pos.y<=xUVPos.y) continue;
			if(xNode1Pos.x< xUVPos.x && xNode2Pos.x< xUVPos.x) continue;
			if(xNode1Pos.x> xUVPos.x && xNode2Pos.x> xUVPos.x) continue;
			//if both points are above, then the line crosses the edge between them
			if(xNode1Pos.y>=xUVPos.y && xNode2Pos.y>=xUVPos.y)
			{
				iEdgesCrossed++;
				continue;	
			}
			else
			{
				float fXDif=xNode2Pos.x - xNode1Pos.x;
				float fYDif=xNode2Pos.y - xNode1Pos.y;
				if(fXDif>0)
				{ 
					if( (xUVPos.x-xNode1Pos.x)*fYDif > (xUVPos.y-xNode1Pos.y)*fXDif ) 
					{
						iEdgesCrossed++;
					}
				}
				else
				{
					if( (xUVPos.x-xNode1Pos.x)*fYDif < (xUVPos.y-xNode1Pos.y)*fXDif ) 
					{
						iEdgesCrossed++;
					}
				}	
				continue;	
			}
		}
	}

	//inside polygon if an odd number of edges were crossed
	return (iEdgesCrossed%2) == 1;
}


bool Tools_UVEdit::RectangleIntersectsEdge(const Asura_Bounding_Rectangle& xUVRect, const Tools_UVEdit_Node* pxFirstNode) const
{
	//based on Asura_Maths::RayIntersectsBoundingBox() :

	bool bInside = true;

	Asura_Vector_2 xMinCorner(xUVRect.MinX, xUVRect.MinY);
	Asura_Vector_2 xMaxCorner(xUVRect.MaxX, xUVRect.MaxY);

	Asura_Vector_2 xLambda(-1.0f,-1.0f);

	const Asura_Vector_2& xEdgeStart = pxFirstNode->GetUV();
	Asura_Vector_2 xLineDir = pxFirstNode->GetNextNode()->GetUV() - xEdgeStart;

	const float fBetaEpsilon = 1.0e-30f;
	for(int i=0; i<2; i++)
	{
		if (xEdgeStart[i] < xMinCorner[i])
		{
			bInside = false;
			if (xLineDir[i] <= fBetaEpsilon) return false;
		
			xLambda[i] = (xMinCorner[i] - xEdgeStart[i]) / xLineDir[i];
			if(xLambda[i] > 1.0f)
			{
				return false;//intersection is beyond the end of the edge
			}
		}
		else if (xEdgeStart[i] > xMaxCorner[i])
		{
			bInside = false;
			if (xLineDir[i] >= -fBetaEpsilon) return false;
			
			xLambda[i] = (xMaxCorner[i] - xEdgeStart[i]) / xLineDir[i];	
			if(xLambda[i] > 1.0f)
			{
				return false;//intersection is beyond the end of the edge
			}

		}
	}

	// Ray origin inside bounding box? Return as a hit.
	// (NOTE: also a hit if the ray starts on the surface of the BB.)
	if (bInside)
	{
		return true;
	}

	// Find largest lambda value
	int iPlane = 0;
	if (xLambda[1] > xLambda[iPlane]) iPlane = 1;

	Asura_Assert(xLambda[iPlane]>0.0f,"Impossible value");

	int iOtherPlane = 1-iPlane;

   	float fIntersection = xEdgeStart[iOtherPlane] + xLambda[iPlane] * xLineDir[iOtherPlane];
	if (fIntersection < xMinCorner[iOtherPlane] || fIntersection > xMaxCorner[iOtherPlane]) return false;

	/// We have an intersection then.
	return true;
	
}

bool Tools_UVEdit::RectangleIntersectsPolygon(const Asura_Bounding_Rectangle& xUVRect, const Tools_UVEdit_Node* pxFirstNode) const
{
	//get the bounding rectangle of the polygon , and check to see if any of the polygon points
	//lie within the rectangle
	Asura_Bounding_Rectangle xPolyRect;
	xPolyRect.Reset();

	const Tools_UVEdit_Node* pxNode = pxFirstNode;
	do
	{
		if(xUVRect.Contains(pxNode->GetUV())) return true;
	
		xPolyRect.Set(pxNode->GetUV());
		pxNode = pxNode->GetNextNode();
		
	}while(pxNode && pxNode != pxFirstNode);

	//if the rects don't intersect then skip the rest
	if(!(xUVRect.MaxX >= xPolyRect.MinX && xUVRect.MinX <= xPolyRect.MaxX &&
		 xUVRect.MaxY >= xPolyRect.MinY && xUVRect.MinY <= xPolyRect.MaxY))
	{
		return false;
	}
	
	//see if one of the corners of the rectangle lies within the polygon
	Asura_Vector_2 xCorner(xUVRect.MinX,xUVRect.MaxX);
	if(PolygonContainsPoint(xCorner, pxFirstNode)) return true;

	//see if any of the polygon edges intersects the rectangle
	pxNode = pxFirstNode;
	do
	{
		if(RectangleIntersectsEdge(xUVRect,pxNode)) return true;
	
		xPolyRect.Set(pxNode->GetUV());
		pxNode = pxNode->GetNextNode();
		
	}while(pxNode && pxNode != pxFirstNode);

	return false;
}


void Tools_UVEdit::RelaxNodes()
{
	const List<Tools_UVEdit_Node*>* pxSelectedList = GetSelectedNodeList();
	int iNumSelected = pxSelectedList->size();
	if(!iNumSelected) return;

	//allocate some working data for all the selected nodes
	Tools_UVEdit_RelaxNode* pxRelaxNodes = new Tools_UVEdit_RelaxNode[iNumSelected];
	Asura_Mem::Set(pxRelaxNodes, 0, sizeof(Tools_UVEdit_RelaxNode) * iNumSelected);

	//initialise them
	CLIF<Tools_UVEdit_Node*> xSelectedLIF(pxSelectedList);
	for(int i=0; i<iNumSelected; i++, xSelectedLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xSelectedLIF();

		pxRelaxNodes[i].m_pxNode = pxNode;

		//go round the face to find the previous/next nodes
		pxRelaxNodes[i].m_pxNextNode = pxNode->GetNextNode();

		for(Tools_UVEdit_Node* pxFaceNode = pxNode->GetNextNode(); ; pxFaceNode = pxFaceNode->GetNextNode())
		{
			Asura_Assert(pxFaceNode, "");
			if(pxFaceNode->GetNextNode() == pxNode)
			{
				pxRelaxNodes[i].m_pxPrevNode = pxFaceNode;
				break;
			}
		}

		//see if there is a previous equivalent node to this one
		for(int j=i-1; j>=0; j--)
		{
			if(!pxRelaxNodes[j].m_pxNextEquivNode && pxNode->IsWelded(pxRelaxNodes[j].m_pxNode))
			{
				//link them together
				pxRelaxNodes[j].m_pxNextEquivNode = &pxRelaxNodes[i];
				pxRelaxNodes[i].m_pxPrevEquivNode = &pxRelaxNodes[j];
				break;
			}
		}
	}

	if(m_bRelaxNode_LockBoundary)
	{
		//Check each node to see if it is a boundary
		Tools_UVEdit_RelaxNode* pxRelaxNode = pxRelaxNodes;
		for(int i=0; i<iNumSelected; i++, pxRelaxNode++)
		{
			//Only check the first node in each set of equivalent nodes
			if(pxRelaxNode->m_pxPrevEquivNode) continue;

			const Tools_UVEdit_Node* pxCurNode = pxRelaxNode->m_pxNextNode;
			const Tools_UVEdit_Node* pxDestNode = pxRelaxNode->m_pxPrevNode;

			pxRelaxNode->m_bBoundary = true;

			//Search round the vertex to see if there is a chain of triangles with selected verts that completely surround it.
			//If such a chain doens't exist , then consider this to be a boundary
			while(true)
			{
				bool bFoundNextNode = false;
				for(Tools_UVEdit_RelaxNode* pxNode = pxRelaxNode->m_pxNextEquivNode; pxNode; pxNode = pxNode->m_pxNextEquivNode)
				{
					if(pxNode->m_pxPrevNode->IsWelded(pxCurNode) && pxNode->m_pxNextNode->IsSelected())
					{
						pxCurNode = pxNode->m_pxNextNode;
						bFoundNextNode = true;
					}
				}	
				if(!bFoundNextNode) break;

				if(pxCurNode->IsWelded(pxDestNode))
				{
					pxRelaxNode->m_bBoundary = false;
					break;
				}
			}

			//Copy the boundary flag to all equivalent verts
			for(Tools_UVEdit_RelaxNode* pxNode = pxRelaxNode->m_pxNextEquivNode; pxNode; pxNode = pxNode->m_pxNextEquivNode)
			{
				pxNode->m_bBoundary = pxRelaxNode->m_bBoundary;
			}	
		}
	}

	m_pxNodeManager->PrepareUndo();

	Asura_Vector_3 xUVNormal(0.0f,0.0f,1.0f);
	{
		//get the most common normal direction for the UVs
		int iPosCount = 0;
		int iNegCount = 0;

		for(int i=0; i<iNumSelected; i++)
		{
			Asura_Vector_3 xNormal = pxRelaxNodes[i].GetUVNormal();
			if(xNormal.z>0.0f)
			{
				iPosCount++;
			}
			else if(xNormal.z<0.0f)
			{
				iNegCount++;
			}
		}
		if(iNegCount > iPosCount) xUVNormal.z = -1.0f;
		
	}

	Tools_ProgressIndicator xProgress("Relaxing UVs", true);

	//do the requested number of iterations
	for(int iIteration=0; iIteration<m_iRelaxNode_NumIteration; iIteration++)
	{
		xProgress.Update(iIteration, m_iRelaxNode_NumIteration);
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE)) break;
		
		//for each node , determine the desired destination UV for this round
		for(int i=0; i<iNumSelected; i++)
		{
			if(m_bRelaxNode_LockBoundary && pxRelaxNodes[i].m_bBoundary) continue;
			
			
			pxRelaxNodes[i].CalculateDestUV(xUVNormal);
		}
		
		//get the start of each chain of equivalent nodes , and average out the destination UV
		for(int i=0; i<iNumSelected; i++)
		{
			Tools_UVEdit_RelaxNode* pxRelaxNode = &pxRelaxNodes[i];
			if(pxRelaxNode->m_pxPrevEquivNode) continue;
			if(!pxRelaxNode->m_pxNextEquivNode) continue;

			Asura_Vector_2 xAverage = pxRelaxNode->m_xDestUV*pxRelaxNode->m_fWeight;
			float fTotalWeight = pxRelaxNode->m_fWeight;

			for(Tools_UVEdit_RelaxNode* pxEquivNode = pxRelaxNode->m_pxNextEquivNode; pxEquivNode; pxEquivNode = pxEquivNode->m_pxNextEquivNode)
			{
				xAverage += pxEquivNode->m_xDestUV*pxEquivNode->m_fWeight;
				fTotalWeight += pxEquivNode->m_fWeight;
			}
			xAverage /= fTotalWeight;

			//fill in the average for each of these nodes
			for(Tools_UVEdit_RelaxNode* pxEquivNode = pxRelaxNode; pxEquivNode; pxEquivNode = pxEquivNode->m_pxNextEquivNode)
			{
				pxEquivNode->m_xDestUV = xAverage;
			}
		}

		//finally move each node a small amount towards its destination
		for(int i=0; i<iNumSelected; i++)
		{
			Tools_UVEdit_RelaxNode* pxRelaxNode = &pxRelaxNodes[i];
			if(m_bRelaxNode_LockBoundary && pxRelaxNode->m_bBoundary) continue;

			Tools_UVEdit_Node* pxNode = pxRelaxNode->m_pxNode;

			Asura_Vector_2 xOffset = pxRelaxNode->m_xDestUV - pxNode->GetUV();
			Asura_Vector_2 xNewUV = pxNode->GetUV() + xOffset*m_fRelaxNode_Amount;

			pxNode->SetUV(xNewUV);
		}
	}
	xProgress.Finish();

	delete [] pxRelaxNodes;

	m_pxNodeManager->ApplyUV();
	UpdateSelectionRectangle(true);
}


void Tools_UVEdit::WeldNodes()
{
	const List<Tools_UVEdit_Node*>* pxSelectedList = GetSelectedNodeList();
	int iNumSelected = pxSelectedList->size();
	if(iNumSelected < 2) return;

	m_pxNodeManager->PrepareUndo();

	//initialise array for keeping track of which nodes have already been dealt with
	bool* pbDone = new bool[iNumSelected];
	Asura_Mem::Set(pbDone,0,sizeof(bool)*iNumSelected);

	//for each selected node , find all others in the same location , and give them the same UV
	for(int i=0; i<iNumSelected; i++)
	{
		if(pbDone[i]) continue;
		pbDone[i] = true;

		Tools_UVEdit_Node* pxNode = (*pxSelectedList)[i];
		List<Tools_UVEdit_Node*> xEquivList;
		xEquivList.add_entry(pxNode);
		Asura_Vector_2 xAverageUV = pxNode->GetUV();

		//search for other nodes with the same position
		for(int j=i+1; j< iNumSelected; j++)
		{
			if(pbDone[j]) continue;

			Tools_UVEdit_Node* pxOtherNode = (*pxSelectedList)[j];

			float fDistSquared = (pxNode->GetPosition() - pxOtherNode->GetPosition()).MagnitudeSquared();
			if(fDistSquared > 0.001f*0.001f) continue;

			pbDone[j] = true;
			xEquivList.add_entry(pxOtherNode);
			xAverageUV += pxOtherNode->GetUV();
		}

		//apply the average to all the nodes found
		if(xEquivList.size() >= 2)
		{
			xAverageUV /= (float)xEquivList.size();
			for(LIF<Tools_UVEdit_Node*> xEquivLIF(&xEquivList); !xEquivLIF.done(); xEquivLIF.next())
			{
				xEquivLIF()->SetUV(xAverageUV);
			}
		}

	}

	delete [] pbDone;

	//apply the changes
	m_pxNodeManager->ApplyUV();
	UpdateSelectionRectangle(true);

}


void Tools_UVEdit::ShowKeys()
{
	Tools_Help::ShowHelpTopic("UV Edit Overlay");
}


void Tools_UVEdit::OpenScaleOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_UVEDIT_SCALE))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_UVEDIT_SCALE);
		return;
	}

	if(!IsAnySelection())
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"No nodes have been selected.");
		return;
	}

	m_fScaleOverlay_U = 1.0f;
	m_fScaleOverlay_V = 1.0f;
	m_bScaleOverlay_UChanged = false;
	m_bScaleOverlay_VChanged = false;

	//create the scale overlay
	Asura_Overlay* pxOverlay = new Asura_Overlay("UV Scale", false, &s_xScaleOverlayPosition);
	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Numerical_Dialog("U Scale", xOffset, &m_fScaleOverlay_U, 0.0f, 9999.0f, 1.0f, "%.3f", &m_bScaleOverlay_UChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("V Scale", xOffset, &m_fScaleOverlay_V, 0.0f, 9999.0f, 1.0f, "%.3f", &m_bScaleOverlay_VChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Uniform Scale", xOffset, &m_bScaleOverlay_Uniform);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Okay", xOffset, CALLBACK_VOID(ScaleOverlay_Okay));
	pxDialog->SetHelpText("Accept the changes and close this overlay.");
	pxOverlay->AddDialog(pxDialog);
	
	pxDialog = new Asura_Function_Dialog("Cancel", xOffset, CALLBACK_VOID(ScaleOverlay_Cancel));
	pxDialog->SetHelpText("Cancel the changes and close this overlay.");
	pxDialog->SetRelativePositionFlags(NULL, Asura_Vector_2(-1.0f-pxDialog->GetWidth(),0.0f), ASURA_DIALOG_RELATIVE_X_OVERLAY|ASURA_DIALOG_RELATIVE_X_RIGHT);
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_UVEDIT_SCALE;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
	
	//initialise UVs for scaling
	m_xPreScaleSelectionCentreUV = m_xSelectionCentreUV;
	m_pxNodeManager->UpdateInitialUVs();
	m_pxNodeManager->PrepareUndo();
}

void Tools_UVEdit::CloseScaleOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_UVEDIT_SCALE);	
}


void Tools_UVEdit::UpdateScaleOverlay()
{
	if(!Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_UVEDIT_SCALE))
	{
		return;
	}

	bool bChanged = false;
	if(m_bScaleOverlay_Uniform)
	{
		if(m_fScaleOverlay_U != m_fScaleOverlay_V)
		{
			if(m_bScaleOverlay_VChanged)
			{
				m_fScaleOverlay_U = m_fScaleOverlay_V;				
			}
			else
			{
				m_fScaleOverlay_V = m_fScaleOverlay_U;				
			}
			bChanged = true;
		}
	}
	if(m_bScaleOverlay_UChanged || m_bScaleOverlay_VChanged)
	{
		bChanged = true;
		m_bScaleOverlay_UChanged = false;
		m_bScaleOverlay_VChanged = false;
	}

	if(bChanged)
	{
		//apply the scale
		ScaleOverlay_ApplyScale(m_fScaleOverlay_U, m_fScaleOverlay_V);
	}
}

void Tools_UVEdit::ScaleOverlay_Okay()
{
	//UVs will already have bene applied , so just close the overlay
	CloseScaleOverlay();
}

void Tools_UVEdit::ScaleOverlay_Cancel()
{
	//reset the UVs
	if(m_fScaleOverlay_U != 1.0f || m_fScaleOverlay_V != 1.0f)
	{
		ScaleOverlay_ApplyScale(1.0f,1.0f);
	}
	//and close the overlay
	CloseScaleOverlay();
}

void Tools_UVEdit::ScaleOverlay_ApplyScale(float fUScale, float fVScale)
{
	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();

		Asura_Vector_2 xUV = pxNode->GetInitialUV();

		xUV -= m_xPreScaleSelectionCentreUV;
		if(!m_bDragYRestrict) xUV.x *= fUScale;
		if(!m_bDragXRestrict) xUV.y *= fVScale;
		xUV += m_xPreScaleSelectionCentreUV;

		pxNode->SetUV(xUV);

	}

	m_pxNodeManager->ApplyUV();
	UpdateSelectionRectangle(true);
}

void Tools_UVEdit::OpenSettingsOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_UVEDIT_SETTINGS))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_UVEDIT_SETTINGS);
		return;
	}


	//create the settings overlay
	Asura_Overlay* pxOverlay = new Asura_Overlay("UV Edit Settings", false, &s_xSettingsOverlayPosition);
	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;


	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	pxOverlay->PushCurrentParentDialog(pxTabsDialog);

	//Add colours tab
	pxTabsDialog->AddTab("Colours");

	pxDialog = new Tools_ARGBColourPicker_Dialog("Selected Node",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uSelectedNodeColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour used for selected nodes");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Unselected Node",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uUnselectedNodeColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour used for unselected nodes");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Equivalent Node",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uEquivalentNodeColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("This colour is used for any unselected nodes that share the same world position as a selected node.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Primary Edge",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uPrimaryEdgeColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("This colour is used to show connected primary UVs");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Secondary Edge",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uSecondaryEdgeColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("This colour is used to show connected secondary UVs");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Selection Rectangle",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uSelectionRectangleColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("This colour is used for to show the extents of the selection");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Grid",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uGridColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour used for the grid lines");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Grid Axis",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uGridAxisColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour used for the thicker lines representing the main axes of the grid");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Selected Vertex",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uSelectedVertexColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour to use when highighting the vertices that correspond to the selected nodes.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	

	pxDialog = new Tools_ARGBColourPicker_Dialog("Cursor Vertex",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uCursorVertexColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour to use when highighting the vertices that correspond to node under the cursor.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog("Reset", xOffset, CALLBACK_VOID(SetDefaultColours_Confirm));
	pxDialog->SetHelpText("Set colours back to their default values");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Add grid tab
	xOffset.y = fTabsStartY;
	pxTabsDialog->AddTab("Grid");

	pxDialog = new Asura_Numerical_Dialog("Grid Size", xOffset, &m_fGridSize, 1.0f, 9999.0f, 1.0f, "%.0f");
	pxDialog->SetHelpText("The spacing of the grid lines in pixels.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Grid", xOffset, &m_bShowGrid);
	pxDialog->SetHelpText("Determines whether the grid will be shown");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	//Add relax settings
	xOffset.y = fTabsStartY;
	pxTabsDialog->AddTab("Relax");

	pxDialog = new Asura_Numerical_Dialog("Relax Iterations", xOffset, &m_iRelaxNode_NumIteration, 0, 999, 1, "%d");
	pxDialog->SetHelpText("The number of iterations to run each time 'Relax Nodes' is used");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Relax Amount", xOffset, &m_fRelaxNode_Amount, 0.01f, 1.0f, 0.1f, "%.2f");
	pxDialog->SetHelpText("The amount each node should attempt to move towards its desired destination in each iteration of 'Relax Nodes'");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Relax - Lock Boundary", xOffset, &m_bRelaxNode_LockBoundary);
	pxDialog->SetHelpText("Prevents the boundary of the selection from moving when using 'Relax Nodes'");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//vertex tab
	xOffset.y = fTabsStartY;
	pxTabsDialog->AddTab("Vertex");

	pxDialog = new Asura_Tickbox_Dialog("Show Selected Verts", xOffset, &m_bRenderSelectedVerts);
	pxDialog->SetHelpText("Renders a small cube around the vertices of any nodes selected in the UV editor");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Selected Vert Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uSelectedVertexColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour to use when highighting the vertices that correspond to the selected nodes.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	

	pxDialog = new Asura_Tickbox_Dialog("Show Cursor Vertex", xOffset, &m_bRenderSelectedVerts);
	pxDialog->SetHelpText("Renders a small cube around the vertex corresponding to the node under the cursor in the UV editor");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_ARGBColourPicker_Dialog("Cursor Vert Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&m_uCursorVertexColour,NULL, true);
	((Tools_ARGBColourPicker_Dialog*)pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour to use when highighting the vertices that correspond to node under the cursor.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//rotate tab (going to be a bit lonely , but can wait until there is something suitable to group it with)
	xOffset.y = fTabsStartY;
	pxTabsDialog->AddTab("Rotate");

	pxDialog = new Asura_AngleNumerical_Dialog("Rotation Snap", xOffset, &m_fRotationSnap, 0.0f, 360.0f, 1.0f, "%.3f");
	pxDialog->SetHelpText("Angle used when snapping the rotation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	pxOverlay->PopCurrentParentDialog();

	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);


	pxOverlay->m_uID = TOOLS_OVERLAY_UVEDIT_SETTINGS;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

}

void Tools_UVEdit::SetDefaultColours()
{
	m_uSelectedNodeColour = 0xffff0000;
	m_uUnselectedNodeColour = 0xffffffff;
	m_uEquivalentNodeColour = 0xff0000ff;
	m_uPrimaryEdgeColour = 0xff00ff00;
	m_uSecondaryEdgeColour = 0xff00ffff;
	m_uSelectionRectangleColour = 0xffff7f3f;
	m_uGridColour = 0xff0000ff;
	m_uGridAxisColour = 0xff00005f;
	m_uSelectedVertexColour = 0xffff0000;
	m_uCursorVertexColour = 0xffffffff;
}

void Tools_UVEdit::SetDefaultColours_Confirm()
{
	if(ShowYesNoOverlay("Reset all the UV editor colours back to the defaults?"))
	{
		SetDefaultColours();
	}
}


u_int Tools_UVEdit::BlendOverlayAlpha(u_int uColour) const
{
	u_int uOverlayAlpha = Asura_OverlayBrightness>>24;
	u_int uColourAlpha = uColour>>24;
	u_int uBlendedAlpha = (uOverlayAlpha*uColourAlpha)/255;

	return (uBlendedAlpha<<24) | (uColour & 0xffffff);
}

void Tools_UVEdit::SetOverrideTexture()
{
	m_iOverrideTextureID = Tools_PickTexture::PickTexture("Select override texture for UV editor", m_iOverrideTextureID, PICKTEXTURE_ALLTEXTURES);	
	if(m_iOverrideTextureID != ASURA_TEXTURE_NONE)
	{
		m_xOverrideTextureName = *Asura_Texture_Management::GetTextureName(m_iOverrideTextureID);

		m_bUseOverrideTexture = true;
	}
}

void Tools_UVEdit::RenderWorld()
{
	if(!m_pxNodeManager->IsEnvironmentNodeManager()) return;

	if(m_bRenderSelectedVerts)
	{
		for(CLIF<Tools_UVEdit_Node*> xNodeLIF(GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
		{
			const Tools_UVEdit_Node_EnvVert* pxNode = (const Tools_UVEdit_Node_EnvVert*) xNodeLIF();

			int iModule = pxNode->GetModuleIndex();
			int iPoly = pxNode->GetPolygonIndex();
			int iVertInPoly = pxNode->GetIndexInPoly();

			const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);
			int iVertex = pxMesh->PolygonDataPtr[iPoly].VertexID[iVertInPoly];

			Tools_Render::RenderVertexOutline(iModule, iVertex, m_uSelectedVertexColour);

		}
	}

	if(m_bRenderCursorVert && m_pxNodeUnderCursor)
	{
		const Tools_UVEdit_Node_EnvVert* pxNode = (const Tools_UVEdit_Node_EnvVert*) m_pxNodeUnderCursor;

		int iModule = pxNode->GetModuleIndex();
		int iPoly = pxNode->GetPolygonIndex();
		int iVertInPoly = pxNode->GetIndexInPoly();

		const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);
		int iVertex = pxMesh->PolygonDataPtr[iPoly].VertexID[iVertInPoly];

		Tools_Render::RenderVertexOutline(iModule, iVertex, m_uCursorVertexColour);
	}
	m_pxNodeUnderCursor = 0;
}

void Tools_UVEdit::UndoSelection()
{
	if(!m_bLockSelection)
	{
		if(m_pxNodeManager->UndoSelection())
		{
			UpdateSelectionRectangle(true);	
		}
	}
}

void Tools_UVEdit::RedoSelection()
{
	if(!m_bLockSelection)
	{
		if(m_pxNodeManager->RedoSelection())
		{
			UpdateSelectionRectangle(true);	
		}
	}
}


///////////////////////////////////////////////////////////////////////////////////////
Asura_Vector_3 Tools_UVEdit_RelaxNode::GetUVNormal()
{
	const float fEpsilon = 0.0001f;

	Asura_Vector_3 xPrevToNextUV (m_pxNextNode->GetUV().x - m_pxPrevNode->GetUV().x, m_pxNextNode->GetUV().y - m_pxPrevNode->GetUV().y, 0.0f);
	float fPrevToNextUVLength = xPrevToNextUV.Magnitude();
	if(fPrevToNextUVLength < fEpsilon) return Asura_Maths::ZeroVector3;

	Asura_Vector_3 xPrevToThisUV (m_pxNode->GetUV().x - m_pxPrevNode->GetUV().x, m_pxNode->GetUV().y - m_pxPrevNode->GetUV().y, 0.0f);
	float fPrevToThisUVLength =  xPrevToThisUV.Magnitude();
	if(fPrevToThisUVLength < fEpsilon) return Asura_Maths::ZeroVector3;

	return xPrevToNextUV ^ xPrevToThisUV;
}

void Tools_UVEdit_RelaxNode::CalculateDestUV(const Asura_Vector_3& xNormalUV)
{
	const float fEpsilon = 0.0001f;

	//default to the current UV , in case nothing can be worked out
	m_xDestUV = m_pxNode->GetUV();
	m_fWeight = 1.0f;

	//get various lines and distances in world space
	Asura_Vector_3 xPrevToNextPos = m_pxNextNode->GetPosition() - m_pxPrevNode->GetPosition();
	float fPrevToNextPosLength = xPrevToNextPos.Magnitude();
	if(fPrevToNextPosLength < fEpsilon) return;

	Asura_Vector_3 xPrevToThisPos = m_pxNode->GetPosition() - m_pxPrevNode->GetPosition();
	float fPrevToThisPosLength =  xPrevToThisPos.Magnitude();
	if(fPrevToThisPosLength < fEpsilon) return;

	//get a perpendicular to prev->next on the polygon
	Asura_Vector_3 xNormalPos = xPrevToNextPos ^ xPrevToThisPos;
	Asura_Vector_3 xPerpPos = xNormalPos ^ xPrevToNextPos;
	float fPerpPosLength = xPerpPos.Magnitude();
	if(fPerpPosLength < fEpsilon) return;

	
	//do the same for the uvs , treating them as 3d vectors for convenience
	Asura_Vector_3 xPrevToNextUV (m_pxNextNode->GetUV().x - m_pxPrevNode->GetUV().x, m_pxNextNode->GetUV().y - m_pxPrevNode->GetUV().y, 0.0f);
	float fPrevToNextUVLength = xPrevToNextUV.Magnitude();
	if(fPrevToNextUVLength < fEpsilon)
	{
		xPrevToNextUV = Asura_Vector_3(0.01f,0.0f,0.0f);
		fPrevToNextUVLength = 0.01f;

	}

	Asura_Vector_3 xPrevToThisUV (m_pxNode->GetUV().x - m_pxPrevNode->GetUV().x, m_pxNode->GetUV().y - m_pxPrevNode->GetUV().y, 0.0f);
	float fPrevToThisUVLength =  xPrevToThisUV.Magnitude();
	if(fPrevToThisUVLength < fEpsilon)
	{
		xPrevToThisUV = Asura_Vector_3(0.0f,0.01f,0.0f);
		fPrevToThisUVLength = 0.01f;

	}

	//Use the provided UV normal for consistent orientation compared to the other polygons
	Asura_Vector_3 xPerpUV = xNormalUV ^ xPrevToNextUV;
	float fPerpUVLength = xPerpUV.Magnitude();
	if(fPerpUVLength < fEpsilon)
	{
		xPerpUV = Asura_Vector_3(0.0f,0.01f,0.0f);
		fPerpUVLength = 0.01f;
	}

	//get the scaling from world positions to uvs
	float fPosToUVScale = fPrevToNextUVLength / fPrevToNextPosLength;

	//get a couple of unit axis vectors on the world space polygon , and determine the distance of the current
	//node along each axis
	Asura_Vector_3 xPosAxis1 = xPrevToNextPos / fPrevToNextPosLength;
	Asura_Vector_3 xPosAxis2 = xPerpPos / fPerpPosLength;

	float fPosDist1 = xPosAxis1 * xPrevToThisPos;
	float fPosDist2 = xPosAxis2 * xPrevToThisPos;

	//get the same axes in UV space
	Asura_Vector_3 xUVAxis1 = xPrevToNextUV / fPrevToNextUVLength;
	Asura_Vector_3 xUVAxis2 = xPerpUV / fPerpUVLength;

	//scale up the distances , and calculate the equivalent position
	float fUVDist1 = fPosDist1 * fPosToUVScale;
	float fUVDist2 = fPosDist2 * fPosToUVScale;

	Asura_Vector_3 xNewPrevToThisUV = xUVAxis1*fUVDist1 + xUVAxis2*fUVDist2;
	m_xDestUV = m_pxPrevNode->GetUV();
	m_xDestUV.x += xNewPrevToThisUV.x;
	m_xDestUV.y += xNewPrevToThisUV.y;

	//Calculate a weight by taking the highest Pos/UV length across the various lines considered , so that the moves
	//for verts with stretched texturing will get higher priority.
	m_fWeight = Asura_Maths::Max((fPrevToThisPosLength / fPrevToThisUVLength),(fPrevToNextPosLength / fPrevToNextUVLength));
	float fCurUVDist1 = xUVAxis1 * xPrevToThisUV;
	float fCurUVDist2 = xUVAxis2 * xPrevToThisUV;
	if(fCurUVDist1 > fEpsilon)
	{
		m_fWeight = Asura_Maths::Max(m_fWeight, fPosDist1/fCurUVDist1);
	}
	if(fCurUVDist2 > fEpsilon)
	{
		m_fWeight = Asura_Maths::Max(m_fWeight, fPosDist2/fCurUVDist2);
	}

}

///////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_UVEditOverlay **
**---------------------*/

Tools_UVEdit_Overlay::~Tools_UVEdit_Overlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_UVEDIT_SCALE);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_UVEDIT_SETTINGS);
}


///////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_UVEdit_Dialog **
**---------------------*/

Tools_UVEdit_Dialog::Tools_UVEdit_Dialog(Asura_Vector_2& xOffset, Tools_UVEdit* pxUVEdit)
:Asura_Dialog(0, xOffset)
{
	Asura_Gadget* pxGadget = new Tools_UVEdit_Gadget(xOffset, pxUVEdit);
	AddGadget(pxGadget);

	m_xMaxCoord.x = m_xOffset.x + pxGadget->m_fWidth;
	m_xMaxCoord.y = m_xOffset.y + pxGadget->m_fHeight;

	m_pxUVEdit = pxUVEdit;
}

void Tools_UVEdit_Dialog::PerFrameUpdate()
{
	Asura_Dialog::PerFrameUpdate();

	m_pxUVEdit->Update();
}

void Tools_UVEdit_Dialog::StartResize(u_int uResizeFlags)
{
	m_pxUVEdit->StorePreDragState(Asura_Vector_2(0.0f,0.0f));
}

void Tools_UVEdit_Dialog::UpdateResize(u_int uResizeFlags, const Asura_Vector_2& xDelta)
{
	Asura_Vector_2 xPixelDragDelta;
	xPixelDragDelta.x = xDelta.x* ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xPixelDragDelta.y = xDelta.y* ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	m_pxUVEdit->UpdateDrag_ResizeEditArea(uResizeFlags, xPixelDragDelta);

	((Tools_UVEdit_Gadget*) m_pxGadgetList)->CheckForResize();
}

///////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_UVEdit_Gadget **
**---------------------*/

Tools_UVEdit_Gadget::Tools_UVEdit_Gadget(Asura_Vector_2& xOffset,Tools_UVEdit* pxUVEdit)
{
	m_pxUVEdit = pxUVEdit;

	m_xOffset = xOffset;
	m_xOffset.x -= g_fUVEditDragBorder/ASURA_TEXT_OUTPUT_FONT_WIDTH;
	m_xSize = m_pxUVEdit->m_xEditAreaSize;
	m_xTopLeftPos = Asura_Maths::ZeroVector2;

	m_fWidth = (m_pxUVEdit->m_xEditAreaSize.x + 2.0f + 2.0f * g_fUVEditDragBorder) / ASURA_TEXT_OUTPUT_FONT_WIDTH;
	m_fHeight = (m_pxUVEdit->m_xEditAreaSize.y + 2.0f + 2.0f * g_fUVEditDragBorder) / ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	m_iHeldMouseButton = 0;
}

void Tools_UVEdit_Gadget::Update(const Asura_Vector_2& xOrigin)
{
	UpdatePosition(xOrigin);
	
	CheckForResize();

	if(Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_MOUSEWHEELUP))
	{
		m_pxUVEdit->ZoomIn(GetRelativeCursorPos());
	}
	if(Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_MOUSEWHEELDOWN))
	{
		m_pxUVEdit->ZoomOut(GetRelativeCursorPos());
	}

	if(m_bIsMouseOver)
	{
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_SPACE))
		{
			m_pxUVEdit->m_bLockSelection = !m_pxUVEdit->m_bLockSelection;
			Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
		}
	}

	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
	{
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
	{
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		//if(m_bIsMouseOver)
		{
			if( Asura_UserInput::DebouncedKeyPressed( 'z' ) || Asura_UserInput::DebouncedKeyPressed( ',' )) // 'shift+z' or 'shift+<' for selection undo
			{
				m_pxUVEdit->UndoSelection();
				Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
			}
			if ( Asura_UserInput::DebouncedKeyPressed( 'y' ) || Asura_UserInput::DebouncedKeyPressed( '.' )) // 'shift+y' or 'shift+>' for selection redo
			{
				m_pxUVEdit->RedoSelection();
				Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
			}
		}
	}
	else
	{
		if(Asura_UserInput::DebouncedKeyPressed('z'))
		{
			m_pxUVEdit->ZoomExtents();
			Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
		}
	}

	//If the mouse is clicked over the control then take mouse focus
	if(!m_iHeldMouseButton)
	{
		if( m_bIsMouseOver && m_bIsClickable)
		{
 			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
			{
				m_iHeldMouseButton = ASURA_KEYID_MOUSEBUTTON1;
			}
			else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON3))
			{
				m_iHeldMouseButton = ASURA_KEYID_MOUSEBUTTON3;
			}

			if(m_iHeldMouseButton) 			
			{
 				Asura_Overlay_System::TakeFocus(this,OverlayFocusType_Mouse);
			}
		}
	}
	
	//update mouse drag handling and/or cursor
	if( m_bIsMouseOver && m_bIsClickable)
	{
		m_pxUVEdit->HandleMouse(GetRelativeCursorPos(), m_iHeldMouseButton);
	}

	if(m_iHeldMouseButton)
	{
		if(!Asura_UserInput::KeyPressed(m_iHeldMouseButton))
		{
			Asura_Overlay_System::ReleaseFocus(OverlayFocusType_Mouse);
			m_iHeldMouseButton = 0;
		}
	}
 	CheckForResize();
}

void Tools_UVEdit_Gadget::Render(const Asura_Vector_2& xOrigin)
{
	UpdatePosition(xOrigin);

	//get top left in screen coords
	float fX = (float)Asura_Maths::FloatToInt_Near((xOrigin.x + m_xOffset.x) * ASURA_TEXT_OUTPUT_FONT_WIDTH)+g_fUVEditDragBorder;
	float fY = (float)Asura_Maths::FloatToInt_Near((xOrigin.y + m_xOffset.y) * ASURA_TEXT_OUTPUT_FONT_HEIGHT)+g_fUVEditDragBorder;


	//draw outline for rectangle
	u_int uColour = Asura_OverlayBrightness|0xffffff;
	Asura_Render::Render2DLine(fX,fY,fX+m_xSize.x+2.0f,fY,uColour);
	Asura_Render::Render2DLine(fX+m_xSize.x+2.0f,fY,fX+m_xSize.x+2.0f,fY+m_xSize.y+2.0f,uColour);
	Asura_Render::Render2DLine(fX+m_xSize.x+2.0f,fY+m_xSize.y+2.0f,fX,fY+m_xSize.y+2.0f,uColour);
	Asura_Render::Render2DLine(fX,fY+m_xSize.y+2.0f,fX,fY,uColour);
	Asura_Render::FlushDisplay();

	//render the background
	int iTextureID = m_pxUVEdit->GetTextureToDisplay();
	if(iTextureID == ASURA_TEXTURE_NONE)
	{
		RenderChequerBackground(fX+1.0f, fY+1.0f);
	}
	else
	{
		RenderTexture(fX+1.0f, fY+1.0f, iTextureID);
	}

	if(m_pxUVEdit->m_bShowGrid)
	{
		RenderGrid();
	}

	//render lines between the nodes
	RenderEdges();

	//and the nodes themselves
	RenderNodes(fX+1.0f, fY+1.0f);

	//the rectangle around the current selection
	if(m_pxUVEdit->ShouldShowSelectionRectangle())
	{
		RenderSelectionRectangle();
	}

	if(m_pxUVEdit->IsDragSelection())
	{
		RenderDragRectangle();
	}
}

void Tools_UVEdit_Gadget::CheckForResize()
{
	if(m_xSize != m_pxUVEdit->m_xEditAreaSize)
	{
		m_xSize = m_pxUVEdit->m_xEditAreaSize;
		m_fWidth = (m_pxUVEdit->m_xEditAreaSize.x+2.0f+2.0f*g_fUVEditDragBorder)/ASURA_TEXT_OUTPUT_FONT_WIDTH;
		m_fHeight = (m_pxUVEdit->m_xEditAreaSize.y+2.0f+2.0f*g_fUVEditDragBorder)/ASURA_TEXT_OUTPUT_FONT_HEIGHT;

		m_pxDialog->m_xMaxCoord.x = m_pxDialog->m_xOffset.x + m_fWidth;
		m_pxDialog->m_xMaxCoord.y = m_pxDialog->m_xOffset.y + m_fHeight;

		m_pxDialog->m_pxOverlay->FlagForResize();
	}
}

void Tools_UVEdit_Gadget::RenderTexture(float fX, float fY, int iTextureID)
{
	Asura_Vector_2 xTopLeftUV = m_pxUVEdit->ConvertPositionToUV(Asura_Maths::ZeroVector2);
	Asura_Vector_2 xBottomRightUV = m_pxUVEdit->ConvertPositionToUV(m_xSize);

	u_int uBright = Asura_Maths::FloatToInt_Near(m_pxUVEdit->m_fTextureBrightness*255.0f);
	uBright = Asura_Maths::ClampToRange(uBright,0u,255u);

	
	Asura_Render::Overlay.SetRectXY( fX, fY, m_xSize.x+1.0f, m_xSize.y+1.0f );
	Asura_Render::Overlay.Colour = Asura_OverlayBrightness|(uBright<<16)|(uBright<<8)|uBright;		
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.SetRectUV( xTopLeftUV.x, xTopLeftUV.y, xBottomRightUV.x-xTopLeftUV.x, xBottomRightUV.y-xTopLeftUV.y );
	Asura_Render::Overlay.TextureIndex = iTextureID;

	DWORD uMagFilter;
	//disable filtering while rendering the texture
	Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MAGFILTER, &uMagFilter );
	Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_NONE );

	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, uMagFilter );
}

void Tools_UVEdit_Gadget::RenderChequerBackground(float fX, float fY)
{
	//TBD , just replace this with a suitable chequered texture instead
	const float fCheckSize = 8.0f;
	for(u_int uX=0;uX<(m_xSize.x/fCheckSize);uX++)
	{
		for(u_int uY=0;uY<(m_xSize.y/fCheckSize);uY++)
		{
			float fCheckWidth = Asura_Maths::Min(fCheckSize,m_xSize.x-uX*fCheckSize);
			float fCheckHeight = Asura_Maths::Min(fCheckSize,m_xSize.y-uY*fCheckSize);
			
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.Colour = ((uX + uY) % 2) ? 0x00444444 : 0x00888888;
			Asura_Render::Overlay.Colour |= Asura_OverlayBrightness;
			Asura_Render::Overlay.SetRectXY(fX + uX * fCheckSize, fY + uY * fCheckSize, fCheckWidth+1.0f, fCheckHeight+1.0f);
			Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;

			Asura_Render::Overlay.Render();
		}
	}
	Asura_Render::FlushDisplay();

}

void Tools_UVEdit_Gadget::RenderSelectionRectangle()
{
	u_int uColour = m_pxUVEdit->GetSelectionRectangleColour();
	Asura_Bounding_Rectangle xSelRect = m_pxUVEdit->GetSelectionRectangleToRender();

	//render the main rectangle
	RenderClippedRectangle(xSelRect, uColour);

	//and a little edit thing at each corner and centre of each edge
	//corners
	for(int i=0;i<4;i++)
	{
		Asura_Vector_2 xCorner;

		xCorner.x = (i & 1) ?  (xSelRect.MaxX+g_fDragGizmoOffset) : ((xSelRect.MinX-g_fDragGizmoOffset));
		xCorner.y = (i & 2) ?  (xSelRect.MaxY+g_fDragGizmoOffset) : ((xSelRect.MinY-g_fDragGizmoOffset));

		Asura_Bounding_Rectangle xRect;
		xRect.MinX = xCorner.x - g_fDragGizmoHalfWidth;
		xRect.MaxX = xCorner.x + g_fDragGizmoHalfWidth;
		xRect.MinY = xCorner.y - g_fDragGizmoHalfWidth;
		xRect.MaxY = xCorner.y + g_fDragGizmoHalfWidth;

		RenderClippedRectangle(xRect, uColour);

	}

	//edges
	for(int i=0;i<4;i++)
	{
		Asura_Vector_2 xCentre = xSelRect.CalculateCentre();

		switch(i)
		{
			case 0 : xCentre.x = xSelRect.MinX - g_fDragGizmoOffset; break;
			case 1 : xCentre.x = xSelRect.MaxX + g_fDragGizmoOffset; break;
			case 2 : xCentre.y = xSelRect.MinY - g_fDragGizmoOffset; break;
			case 3 : xCentre.y = xSelRect.MaxY + g_fDragGizmoOffset; break;
		}

		Asura_Bounding_Rectangle xRect;
		xRect.MinX = xCentre.x - g_fDragGizmoHalfWidth;
		xRect.MaxX = xCentre.x + g_fDragGizmoHalfWidth;
		xRect.MinY = xCentre.y - g_fDragGizmoHalfWidth;
		xRect.MaxY = xCentre.y + g_fDragGizmoHalfWidth;

		RenderClippedRectangle(xRect, uColour);
	}

	//render cross for the centre
	if(m_pxUVEdit->ShouldShowSelectionCentre())
	{
		Asura_Vector_2 xCentre = m_pxUVEdit->GetSelectionCentre();

		Asura_Vector_2 xTop = xCentre;
		xTop.y -= g_fCentreHalfWidth*2.0f;
		Asura_Vector_2 xBottom = xCentre;
		xBottom.y += g_fCentreHalfWidth*2.0f+1.0f;

		Asura_Vector_2 xLeft = xCentre;
		xLeft.x -= g_fCentreHalfWidth*2.0f;
		Asura_Vector_2 xRight = xCentre;
		xRight.x += g_fCentreHalfWidth*2.0f+1.0f;

		RenderClippedLine(xTop, xBottom, uColour);
		RenderClippedLine(xLeft, xRight, uColour);
	}

	Asura_Render::FlushDisplay();
	
}

void Tools_UVEdit_Gadget::RenderDragRectangle()
{
	//get the rect covered by the drag
	Asura_Bounding_Rectangle xRect = m_pxUVEdit->GetClampedDragRectangle(GetRelativeCursorPos());

	xRect.Displace(m_xTopLeftPos);

	float fBright = Asura_Maths::Sine( Asura_Timers::GetLocalUnscaledTime() * 2.0f );
	int iBright = Asura_Maths::FloatToInt( 255.0f * fBright * fBright );
	unsigned int uColour = Asura_OverlayBrightness | (iBright) | (iBright << 8) | (iBright << 16);

	Asura_Render::Render2DLine(xRect.MinX,xRect.MinY,xRect.MaxX,xRect.MinY,uColour);
	Asura_Render::Render2DLine(xRect.MaxX,xRect.MinY,xRect.MaxX,xRect.MaxY,uColour);
	Asura_Render::Render2DLine(xRect.MaxX,xRect.MaxY,xRect.MinX,xRect.MaxY,uColour);
	Asura_Render::Render2DLine(xRect.MinX,xRect.MaxY,xRect.MinX,xRect.MinY,uColour);
	Asura_Render::FlushDisplay();
	
}

void Tools_UVEdit_Gadget::RenderEdges()
{

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(m_pxUVEdit->GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		const Tools_UVEdit_Node* pxNode1 = xNodeLIF();
		const Tools_UVEdit_Node* pxNode2 = pxNode1->GetNextNode();
		if(!pxNode2) continue;

		u_int uColour = pxNode1->IsPrimaryUV() ? m_pxUVEdit->GetPrimaryEdgeColour() : m_pxUVEdit->GetSecondaryEdgeColour();
		if(!pxNode1->IsSelectable())
		{
			//grey this out by halving all the components
			uColour =	(((uColour>>25) & 0x7f)<<24) | 
						(((uColour>>17) & 0x7f)<<16) | 
						(((uColour>>9) & 0x7f)<<8) |
						(((uColour>>1) & 0x7f));
		}




		Asura_Vector_2 xStart = m_pxUVEdit->ConvertUVToPosition(pxNode1->GetUV(), true);
		Asura_Vector_2 xEnd = m_pxUVEdit->ConvertUVToPosition(pxNode2->GetUV(), true);

		RenderClippedLine(xStart, xEnd, uColour);

		
	}

	Asura_Render::FlushDisplay();
}

void Tools_UVEdit_Gadget::RenderNodes(float fX, float fY)
{
	Asura_Render::Overlay.Colour = m_pxUVEdit->GetUnselectedNodeColour();		
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.SetRectUV( 0.0f, 0.0f, 0.0f, 0.0f );

	m_pxUVEdit->UpdateEquivalentToSelectionFlags();

	//render the unselected nodes first
	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(m_pxUVEdit->GetNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		if(xNodeLIF()->IsSelected()) continue;
		Asura_Vector_2 xPos = m_pxUVEdit->ConvertUVToPosition(xNodeLIF()->GetUV(), true);

		if(xPos.x >= 0.0f && xPos.y >= 0.0f && xPos.x < m_xSize.x && xPos.y < m_xSize.y)
		{
			u_int uColour = 0;
			if(xNodeLIF()->IsEquivalentToASelectedNode())
			{
				uColour = m_pxUVEdit->GetEquivalentNodeColour();
			}
			else
			{
				uColour = m_pxUVEdit->GetUnselectedNodeColour();
			}
			if(!xNodeLIF()->IsSelectable())
			{
				//grey this out by halving all the components
				uColour =	(((uColour>>25) & 0x7f)<<24) | 
							(((uColour>>17) & 0x7f)<<16) | 
							(((uColour>>9) & 0x7f)<<8) |
							(((uColour>>1) & 0x7f));
			}

			Asura_Render::Overlay.Colour = uColour;
			Asura_Render::Overlay.SetRectXY( xPos.x+fX-1.0f, xPos.y+fY-1.0f, 3.0f, 3.0f );
			Asura_Render::Overlay.Render();
		}
	}
	Asura_Render::FlushDisplay();

	//and then the selected nodes
	Asura_Render::Overlay.Colour = m_pxUVEdit->GetSelectedNodeColour();		

	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(m_pxUVEdit->GetSelectedNodeList()); !xNodeLIF.done(); xNodeLIF.next())
	{
		Asura_Vector_2 xPos = m_pxUVEdit->ConvertUVToPosition(xNodeLIF()->GetUV(), true);

		if(xPos.x >= 0.0f && xPos.y >= 0.0f && xPos.x < m_xSize.x && xPos.y < m_xSize.y)
		{
			Asura_Render::Overlay.SetRectXY( xPos.x+fX-1.0f, xPos.y+fY-1.0f, 3.0f, 3.0f );
			Asura_Render::Overlay.Render();
		}
	}
	Asura_Render::FlushDisplay();
}



void Tools_UVEdit_Gadget::UpdatePosition(const Asura_Vector_2& xOrigin)
{
	m_xTopLeftPos.x = (float)Asura_Maths::FloatToInt_Near((xOrigin.x + m_xOffset.x) * ASURA_TEXT_OUTPUT_FONT_WIDTH+1.0f+g_fUVEditDragBorder);
	m_xTopLeftPos.y = (float)Asura_Maths::FloatToInt_Near((xOrigin.y + m_xOffset.y) * ASURA_TEXT_OUTPUT_FONT_HEIGHT+1.0f+g_fUVEditDragBorder);
}

Asura_Vector_2 Tools_UVEdit_Gadget::GetRelativeCursorPos() const
{
	int iX,iY;
	Asura_Cursor::GetScreenSpaceCoords(iX, iY);

	return Asura_Vector_2(iX - m_xTopLeftPos.x, iY - m_xTopLeftPos.y);
}

void Tools_UVEdit_Gadget::RenderClippedLine(const Asura_Vector_2& xPos1, const Asura_Vector_2& xPos2, u_int uColour)
{
#define CLIP_LINE_TO_EDIT_AREA(SIGN,MEMBER,val) \
{\
	if(xStart.MEMBER SIGN val || xEnd.MEMBER SIGN val)\
	{\
		if(xStart.MEMBER SIGN val && xEnd.MEMBER SIGN val) return;\
		Asura_Vector_2 *v1,*v2;\
		if(xStart.MEMBER SIGN xEnd.MEMBER)\
		{\
			v1 = &xStart;\
			v2 = &xEnd;\
		}\
		else\
		{\
			v1 = &xEnd;\
			v2 = &xStart;\
		}\
\
\
		float numerator = val - v1->MEMBER;\
		float denominator = v2->MEMBER - v1->MEMBER;\
		if(denominator == 0) return;\
		float scale = numerator/denominator;\
\
		v1->x += ((v2->x-v1->x)*scale);\
		v1->y += ((v2->y-v1->y)*scale);\
\
	}\
}
	
	
	Asura_Vector_2 xStart = xPos1;
	Asura_Vector_2 xEnd = xPos2;

	//clip the line to the  area on the gadget
	CLIP_LINE_TO_EDIT_AREA(<,x,0.0f)
	CLIP_LINE_TO_EDIT_AREA(>,x,m_xSize.x)
	CLIP_LINE_TO_EDIT_AREA(<,y,0.0f)
	CLIP_LINE_TO_EDIT_AREA(>,y,m_xSize.y)

	//offset to screen coords
	xStart += m_xTopLeftPos;
	xEnd += m_xTopLeftPos;

	Asura_Render::Render2DLine(xStart,xEnd,uColour);

	#undef CLIP_LINE_TO_EDIT_AREA
}

void Tools_UVEdit_Gadget::RenderClippedRectangle(const Asura_Bounding_Rectangle& xRect, u_int uColour)
{
	RenderClippedLine(Asura_Vector_2(xRect.MinX, xRect.MinY), Asura_Vector_2(xRect.MaxX, xRect.MinY), uColour);
	RenderClippedLine(Asura_Vector_2(xRect.MaxX, xRect.MinY), Asura_Vector_2(xRect.MaxX, xRect.MaxY), uColour);
	RenderClippedLine(Asura_Vector_2(xRect.MaxX, xRect.MaxY), Asura_Vector_2(xRect.MinX, xRect.MaxY), uColour);
	RenderClippedLine(Asura_Vector_2(xRect.MinX, xRect.MaxY), Asura_Vector_2(xRect.MinX, xRect.MinY), uColour);
}

void Tools_UVEdit_Gadget::RenderGrid()
{
	const float fGridSizeU = m_pxUVEdit->GetGridSizeU();
	const float fGridSizeV = m_pxUVEdit->GetGridSizeV();

	Asura_Vector_2 xBottomRightUV = m_pxUVEdit->ConvertPositionToUV(m_xSize);

	//only draw the main grid lines if there is going to be a reasonable amount of separation
	if(fGridSizeU*m_pxUVEdit->GetScaleX()>=8.0f && fGridSizeV*m_pxUVEdit->GetScaleY()>=8.0f)
	{
		u_int uColour = m_pxUVEdit->GetGridColour();

		Asura_Vector_2 xTopLeftGridUV = m_pxUVEdit->m_xTopLeftUV;
		xTopLeftGridUV.x = (Asura_Maths::FloatToInt_Near(xTopLeftGridUV.x/fGridSizeU)-1)*fGridSizeU;
		xTopLeftGridUV.y = (Asura_Maths::FloatToInt_Near(xTopLeftGridUV.y/fGridSizeV)-1)*fGridSizeV;
				

		//render horizontal lines
		for(float y=xTopLeftGridUV.y; y<xBottomRightUV.y; y+=fGridSizeV)
		{
			Asura_Vector_2 xLineStart = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(xTopLeftGridUV.x, y), true);
			Asura_Vector_2 xLineEnd = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(xBottomRightUV.x, y), true);

			RenderClippedLine(xLineStart, xLineEnd, uColour);
		}

		//render vertical lines
		for(float x=xTopLeftGridUV.x; x<xBottomRightUV.x; x+=fGridSizeU)
		{
			Asura_Vector_2 xLineStart = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(x, xTopLeftGridUV.y), true);
			Asura_Vector_2 xLineEnd = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(x, xBottomRightUV.y), true);

			RenderClippedLine(xLineStart, xLineEnd, uColour);
		}

		Asura_Render::FlushDisplay();
	}


	//render thicker lines to represent the axis grid lines
	u_int uColour = m_pxUVEdit->GetGridAxisColour();
	//horizontal
	if(xBottomRightUV.x > 0.0f)
	{
		Asura_Vector_2 xLineStart = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(0.0f, 0.0f), true);
		Asura_Vector_2 xLineEnd = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(xBottomRightUV.x, 0.0f), true);

		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.y += 1.0f;
		xLineEnd.y += 1.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.y -= 2.0f;
		xLineEnd.y -= 2.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);
	}

	//vertical
	if(xBottomRightUV.y > 0.0f)
	{
		Asura_Vector_2 xLineStart = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(0.0f, 0.0f), true);
		Asura_Vector_2 xLineEnd = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(0.0f, xBottomRightUV.y), true);

		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.x += 1.0f;
		xLineEnd.x += 1.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.x -= 2.0f;
		xLineEnd.x -= 2.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);

	}
	//and also for the other borders of the first tile of the texture
	{
		Asura_Vector_2 xLineStart = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(0.0f, 1.0f), true);
		Asura_Vector_2 xLineEnd = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(1.0f, 1.0f), true);

		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.y += 1.0f;
		xLineEnd.y += 1.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.y -= 2.0f;
		xLineEnd.y -= 2.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);
	}

	{
		Asura_Vector_2 xLineStart = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(1.0f, 0.0f), true);
		Asura_Vector_2 xLineEnd = m_pxUVEdit->ConvertUVToPosition(Asura_Vector_2(1.0f, 1.0f), true);

		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.x += 1.0f;
		xLineEnd.x += 1.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);
		xLineStart.x -= 2.0f;
		xLineEnd.x -= 2.0f;
		RenderClippedLine(xLineStart, xLineEnd, uColour);

	}


	
	Asura_Render::FlushDisplay();


}


///////////////////////////////////////////////////////////////////////////////////////
/*-------------------**
** Tools_UVEdit_Node **
**-------------------*/

Tools_UVEdit_Node::Tools_UVEdit_Node()
{
	m_xUV = Asura_Maths::ZeroVector2;
	m_xInitialUV = Asura_Maths::ZeroVector2;
	m_pxNext = 0;
	m_bSelected = false;
}

bool Tools_UVEdit_Node::IsSelected(Tools_UVEditSelectMode eSelectMode) const
{
	switch(eSelectMode)
	{
		case TOOLS_UVEDITSELECTMODE_VERTEX :
			return IsSelected();

		case TOOLS_UVEDITSELECTMODE_EDGE :
			return IsSelected() && GetNextNode()->IsSelected();

		case TOOLS_UVEDITSELECTMODE_POLYGON :
		{
			//make sure all the nodes in the polygon are selected
			const Tools_UVEdit_Node* pxNode = this;
			do
			{
				if(!pxNode->IsSelected()) return false;
				pxNode = pxNode->GetNextNode();
				
			}while(pxNode && pxNode != this);
			return true;
		}

		default :
			Asura_Assert(0,"Unhandled selection mode");
	}
	return false;
}


///////////////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_UVEdit_NodeManager **
**--------------------------*/

Tools_UVEdit_NodeManager::Tools_UVEdit_NodeManager(Tools_UVEdit* pxUVEdit)
{
	m_pxUVEdit = pxUVEdit;
	m_bNeedToUpdateEquivalentToSelectionFlags = false;
}

Tools_UVEdit_NodeManager::~Tools_UVEdit_NodeManager()
{
	while(m_xNodeList.size())
	{
		delete m_xNodeList.first_entry();
		m_xNodeList.delete_first_entry();
	}
}

void Tools_UVEdit_NodeManager::Select(Tools_UVEdit_Node* pxNode,bool bAlsoSelectedWeldedNodes)
{
	if(!pxNode->IsSelectable()) return;

	if(!pxNode->m_bSelected)
	{
		pxNode->m_bSelected = true;
		m_xSelectedList.add_entry(pxNode);
		LogSelection(pxNode, true);
		m_bNeedToUpdateEquivalentToSelectionFlags = true;

	}

	if(bAlsoSelectedWeldedNodes)
	{
		//Search for any other nodes at the same spot
		for(LIF<Tools_UVEdit_Node*> xNodeLIF(&m_xNodeList); !xNodeLIF.done(); xNodeLIF.next())
		{
			Tools_UVEdit_Node* pxOtherNode = xNodeLIF();
			if(pxOtherNode->IsSelected()) continue;
			if(!pxOtherNode->IsSelectable()) continue;

			if(pxNode->IsWelded(pxOtherNode))
			{
				pxOtherNode->m_bSelected = true;
				m_xSelectedList.add_entry(pxOtherNode);
				LogSelection(pxNode, true);
				m_bNeedToUpdateEquivalentToSelectionFlags = true;
			}
		}
	}
}

void Tools_UVEdit_NodeManager::Select(Tools_UVEdit_Node* pxFirstNode, Tools_UVEditSelectMode eSelectMode, bool bAlsoSelectedWeldedNodes)
{
	switch(eSelectMode)
	{
		case TOOLS_UVEDITSELECTMODE_VERTEX :
			Select(pxFirstNode, bAlsoSelectedWeldedNodes);
			break;

		case TOOLS_UVEDITSELECTMODE_EDGE :
			Select(pxFirstNode, bAlsoSelectedWeldedNodes);
			Select(pxFirstNode->GetNextNode(), bAlsoSelectedWeldedNodes);
			break;

		case TOOLS_UVEDITSELECTMODE_POLYGON :
		{
			Tools_UVEdit_Node* pxNode = pxFirstNode;
			do
			{
				Select(pxNode, bAlsoSelectedWeldedNodes);
				pxNode = pxNode->GetNextNode();
				
			}while(pxNode && pxNode != pxFirstNode);
			break;
		}

		default :
			Asura_Assert(0,"Unhandled selection mode");
	}
}




void Tools_UVEdit_NodeManager::Deselect(Tools_UVEdit_Node* pxNode,bool bAlsoDeselectedWeldedNodes)
{
	if(pxNode->m_bSelected)
	{
		pxNode->m_bSelected = false;
		m_xSelectedList.delete_entry(pxNode);
		LogSelection(pxNode, false);
		m_bNeedToUpdateEquivalentToSelectionFlags = true;
	}

	if(bAlsoDeselectedWeldedNodes)
	{
		//Search for any other nodes at the same stpo
		for(LIF<Tools_UVEdit_Node*> xNodeLIF(&m_xNodeList); !xNodeLIF.done(); xNodeLIF.next())
		{
			Tools_UVEdit_Node* pxOtherNode = xNodeLIF();
			if(!pxOtherNode->IsSelected()) continue;

			if(pxNode->IsWelded(pxOtherNode))
			{
				pxOtherNode->m_bSelected = false;
				m_xSelectedList.delete_entry(pxOtherNode);
				LogSelection(pxNode, false);
				m_bNeedToUpdateEquivalentToSelectionFlags = true;
			}
		}
	}
}

void Tools_UVEdit_NodeManager::Deselect(Tools_UVEdit_Node* pxFirstNode, Tools_UVEditSelectMode eSelectMode, bool bAlsoDeselectedWeldedNodes)
{
	switch(eSelectMode)
	{
		case TOOLS_UVEDITSELECTMODE_VERTEX :
			Deselect(pxFirstNode, bAlsoDeselectedWeldedNodes);
			break;

		case TOOLS_UVEDITSELECTMODE_EDGE :
			Deselect(pxFirstNode, bAlsoDeselectedWeldedNodes);
			Deselect(pxFirstNode->GetNextNode(), bAlsoDeselectedWeldedNodes);
			break;

		case TOOLS_UVEDITSELECTMODE_POLYGON :
		{
			Tools_UVEdit_Node* pxNode = pxFirstNode;
			do
			{
				Deselect(pxNode, bAlsoDeselectedWeldedNodes);
				pxNode = pxNode->GetNextNode();
				
			}while(pxNode && pxNode != pxFirstNode);
			break;
		}

		default :
			Asura_Assert(0,"Unhandled selection mode");
	}
}



void Tools_UVEdit_NodeManager::ClearSelection()
{
	while(m_xSelectedList.size())
	{
		m_xSelectedList.first_entry()->m_bSelected = false;
		LogSelection(m_xSelectedList.first_entry(), false);
		m_xSelectedList.delete_first_entry();
	}	
	m_bNeedToUpdateEquivalentToSelectionFlags = true;

}

void Tools_UVEdit_NodeManager::UpdateInitialUVs()
{
	for(LIF<Tools_UVEdit_Node*> xNodeLIF(&m_xSelectedList); !xNodeLIF.done(); xNodeLIF.next())
	{
		xNodeLIF()->UpdateInitialUV();	
	}
}

bool Tools_UVEdit_NodeManager::IsOneDistinctNodeSelected() const
{
	if(!m_xSelectedList.size()) return false;

	const Tools_UVEdit_Node* pxNode = m_xSelectedList.first_entry();

	//check to see if the rest of the selection contains equivalent nodes
	for(CLIF<Tools_UVEdit_Node*> xNodeLIF(&m_xSelectedList); !xNodeLIF.done(); xNodeLIF.next())
	{
		const Tools_UVEdit_Node* pxOtherNode = xNodeLIF();
		if(pxOtherNode == pxNode) continue;

		if(!pxNode->IsWelded(pxOtherNode))
		{
			return false;
		}
	}

	return true;
}

void Tools_UVEdit_NodeManager::UpdateEquivalentToSelectionFlags()
{
	if(!m_bNeedToUpdateEquivalentToSelectionFlags) return;
	m_bNeedToUpdateEquivalentToSelectionFlags = false;

	//Check each node and compare against the selection to see if it is equivalent to any of them
	for(LIF<Tools_UVEdit_Node*> xNodeLIF(&m_xNodeList); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node* pxNode = xNodeLIF();
		pxNode->m_bEquivalentToASelectedNode = false;
		
		if(pxNode->IsSelected())
		{
			pxNode->m_bEquivalentToASelectedNode = true;
		}
		else
		{
			for(LIF<Tools_UVEdit_Node*> xSelectedLIF(&m_xSelectedList); !xSelectedLIF.done(); xSelectedLIF.next())		
			{
				if(pxNode->IsEquivalent(xSelectedLIF()))
				{
					pxNode->m_bEquivalentToASelectedNode = true;
					break;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_UVEdit_Node_EnvVert **
**---------------------------*/

Tools_UVEdit_Node_EnvVert::Tools_UVEdit_Node_EnvVert(const Asura_Mesh* pxMesh, int iModuleIndex, int iPolygonIndex, int iVertInPoly, int iChannel, bool bSelectable)
:m_iModuleIndex(iModuleIndex)
,m_iPolygonIndex(iPolygonIndex)
,m_iVertInPoly(iVertInPoly)
,m_iChannel(iChannel)
{
	m_bSelectable = bSelectable;
	const Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[iPolygonIndex];

	m_xUV.x = pxTex->U[iChannel][iVertInPoly];
	m_xUV.y = pxTex->V[iChannel][iVertInPoly];

	//store the world position of the vertex
	int iVert = pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[iVertInPoly];
	m_xPos = Asura_Module_System::GetPositionOfModule(iModuleIndex) + pxMesh->VertexDataPtr[iVert].Position;
}

bool Tools_UVEdit_Node_EnvVert::Matches( const Tools_UVEdit_Node_EnvVert* pxNode) const
{
	return m_iModuleIndex == pxNode->m_iModuleIndex &&
			m_iPolygonIndex == pxNode->m_iPolygonIndex &&
			m_iVertInPoly == pxNode->m_iVertInPoly &&
			m_iChannel == pxNode->m_iChannel;

}

bool Tools_UVEdit_Node_EnvVert::IsEquivalent(const Tools_UVEdit_Node* _pxNode) const
{
	const Tools_UVEdit_Node_EnvVert* pxNode = (const Tools_UVEdit_Node_EnvVert*) _pxNode;

	if(pxNode->m_iChannel != m_iChannel) return false;

	//check the world positions are approximately the same
	float fDistSquared = (m_xPos - pxNode->m_xPos).MagnitudeSquared();
	if(fDistSquared > 0.001f*0.001f) return false;

	return true;
}

bool Tools_UVEdit_Node_EnvVert::IsWelded(const Tools_UVEdit_Node* _pxNode) const
{
	const Tools_UVEdit_Node_EnvVert* pxNode = (const Tools_UVEdit_Node_EnvVert*) _pxNode;

	if(pxNode->m_iChannel != m_iChannel) return false;

	//check the UVs are approximately the same
	float fUVDistSquared = (GetUV()- pxNode->GetUV()).MagnitudeSquared();
	if(fUVDistSquared > 0.0001f*0.0001f) return false;

	//check the world positions are approximately the same
	float fDistSquared = (m_xPos - pxNode->m_xPos).MagnitudeSquared();
	if(fDistSquared > 0.001f*0.001f) return false;

	return true;
}

void Tools_UVEdit_Node_EnvVert::ApplyUV()
{
	if(m_iModuleIndex>=Asura_Module_System::GetNumberOfModules()) return;
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(m_iModuleIndex);

	if(m_iPolygonIndex>=pxMesh->NumberOfPolygons) return;
	if(m_iVertInPoly>=pxMesh->PolygonDataPtr[m_iPolygonIndex].NumberOfVertices()) return;
	if(!pxMesh->TextureDataPtr) return;

	Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[m_iPolygonIndex];
	pxTex->U[m_iChannel][m_iVertInPoly] = m_xUV.x;
	pxTex->V[m_iChannel][m_iVertInPoly] = m_xUV.y;

	Tools_UpdateTexturingOnSurfaces(pxMesh,false);
}

void Tools_UVEdit_Node_EnvVert::UpdateFromMesh()
{
	if(m_iModuleIndex>=Asura_Module_System::GetNumberOfModules()) return;
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(m_iModuleIndex);

	if(m_iPolygonIndex>=pxMesh->NumberOfPolygons) return;
	if(m_iVertInPoly>=pxMesh->PolygonDataPtr[m_iPolygonIndex].NumberOfVertices()) return;
	if(!pxMesh->TextureDataPtr) return;

	Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[m_iPolygonIndex];
	m_xUV.x = pxTex->U[m_iChannel][m_iVertInPoly];
	m_xUV.y = pxTex->V[m_iChannel][m_iVertInPoly];

	m_xInitialUV = m_xUV;
}


///////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_UVEdit_NodeManager_Environment **
**--------------------------------------*/

Tools_UVEdit_NodeManager_Environment::Tools_UVEdit_NodeManager_Environment(Tools_UVEdit* pxUVEdit)
:Tools_UVEdit_NodeManager(pxUVEdit)
{
	m_pxUndo = new Tools_UVEdit_SelectionUndo_EnvVert(this);
	m_pxUndo->Initialise();
	m_bDisableUndoLogging = false;
}

Tools_UVEdit_NodeManager_Environment::~Tools_UVEdit_NodeManager_Environment()
{
	delete m_pxUndo;
}

void Tools_UVEdit_NodeManager_Environment::Update()
{
	m_pxUndo->MarkUndoStartIfEndOfBatch();
}

void Tools_UVEdit_NodeManager_Environment::UpdateNodes()
{
	m_bDisableUndoLogging = true;
	//delete all the non-selected nodes and keep a backup of the selected list
	while(m_xNodeList.size())
	{
		Tools_UVEdit_Node* pxNode = m_xNodeList.first_entry();
		m_xNodeList.delete_first_entry();

		if(!pxNode->IsSelected())
		{
			delete pxNode;
		}
	}

	List<Tools_UVEdit_Node*> xSelectedList = m_xSelectedList;
	m_xSelectedList.empty();

	int iEditedChannels = m_pxUVEdit->GetEditedChannels();
	bool bShowAll = m_pxUVEdit->GetShowAllChannels();


	for(int iChannel=0; iChannel<ASURA_MATERIAL_MAXTEXTURES; iChannel++)
	{
		const bool bSelectable = ((1<<iChannel) & iEditedChannels) ? true : false;
		
		if(bShowAll || bSelectable)
		{
			//create new nodes for all the selected polygons
			PolygonID xPolyID;
			Tools_Polygon_Selection::StartIteration();

			while(Tools_Polygon_Selection::GetNextSelected(&xPolyID))
			{
	
				int iModuleIndex = xPolyID.ModuleIndex;
				int iPolygonIndex = xPolyID.PolyIndex;

				const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);
				Asura_Assert(pxMesh->TextureDataPtr,"");

				int iNumVerts = pxMesh->PolygonDataPtr[iPolygonIndex].NumberOfVertices();

				//create a node for each vert in the poly
				Tools_UVEdit_Node_EnvVert* apxPolyNodes[4];

				for(int i=0;i<iNumVerts; i++)
				{
					apxPolyNodes[i] = new Tools_UVEdit_Node_EnvVert(pxMesh, iModuleIndex, iPolygonIndex, i, iChannel, bSelectable);
					m_xNodeList.add_entry(apxPolyNodes[i]);

					//does this match any of the old selected nodes?
					for(LIF<Tools_UVEdit_Node*> xSelectedLIF(&xSelectedList); !xSelectedLIF.done(); xSelectedLIF.next())
					{
						Tools_UVEdit_Node_EnvVert* pxSelNode = (Tools_UVEdit_Node_EnvVert*) xSelectedLIF();

						if(pxSelNode->Matches(apxPolyNodes[i]))
						{
							if(bSelectable)
							{
								Select(apxPolyNodes[i],false);
							}

							delete pxSelNode;
							xSelectedLIF.delete_current();
							break;
						}
					}

				}

				//link them together
				for(int i=0;i<iNumVerts; i++)
				{
					apxPolyNodes[i]->SetNextNode(apxPolyNodes[(i+1)%iNumVerts] );
				}
			}
		}
	}

	//delete any leftover old nodes
	while(xSelectedList.size())
	{
		delete xSelectedList.first_entry();
		xSelectedList.delete_first_entry();
	}

	m_bNeedToUpdateEquivalentToSelectionFlags = true;
	m_bDisableUndoLogging = false;

}

void Tools_UVEdit_NodeManager_Environment::ApplyUV()
{
	for(CLIF<Tools_UVEdit_Node*> xSelLIF(GetSelectedNodeList()); !xSelLIF.done(); xSelLIF.next())
	{
		Tools_UVEdit_Node_EnvVert* pxNode = (Tools_UVEdit_Node_EnvVert*) xSelLIF();

		pxNode->ApplyUV();
	}
}

void Tools_UVEdit_NodeManager_Environment::UpdateNodesFromSource()
{
	for(CLIF<Tools_UVEdit_Node*> xSelLIF(GetNodeList()); !xSelLIF.done(); xSelLIF.next())
	{
		Tools_UVEdit_Node_EnvVert* pxNode = (Tools_UVEdit_Node_EnvVert*) xSelLIF();

		pxNode->UpdateFromMesh();
	}
}

void Tools_UVEdit_NodeManager_Environment::PrepareUndo()
{
	int iPrevModuleIndex = -1;
	int iPrevPolyIndex = -1;

	for(CLIF<Tools_UVEdit_Node*> xSelLIF(GetSelectedNodeList()); !xSelLIF.done(); xSelLIF.next())
	{
		Tools_UVEdit_Node_EnvVert* pxNode = (Tools_UVEdit_Node_EnvVert*) xSelLIF();

		//undo is done a polygon at a time, so don't bother if this is part of the same polygon as done before
		if(pxNode->GetModuleIndex() == iPrevModuleIndex && pxNode->GetPolygonIndex() == iPrevPolyIndex) continue;

		PolygonID xPolyID;
		xPolyID.ModuleIndex = pxNode->GetModuleIndex();
		xPolyID.PolyIndex = pxNode->GetPolygonIndex();
		
		Tools_Texturing_UndoData_TextureInfo::Create(xPolyID);
		if(iPrevModuleIndex != xPolyID.ModuleIndex)
		{
			Tools_SaveAsura::SetModuleModified(xPolyID.ModuleIndex);
		}
		
		iPrevModuleIndex = xPolyID.ModuleIndex;
		iPrevPolyIndex = xPolyID.PolyIndex;
	}
}

Tools_UVEdit_Node_EnvVert* Tools_UVEdit_NodeManager_Environment::FindNode(int iModuleIndex, int iPolygonIndex, int iVertInPoly, int iChannel)
{
	for(LIF<Tools_UVEdit_Node*> xNodeLIF(&m_xNodeList); !xNodeLIF.done(); xNodeLIF.next())
	{
		Tools_UVEdit_Node_EnvVert* pxNode = (Tools_UVEdit_Node_EnvVert*) xNodeLIF();

		if(pxNode->GetModuleIndex() == iModuleIndex &&
			pxNode->GetPolygonIndex() == iPolygonIndex &&
			pxNode->GetIndexInPoly() == iVertInPoly &&
			pxNode->GetChannel() == iChannel)
		{
			return pxNode;
		}
	}
	return 0;
}

bool Tools_UVEdit_NodeManager_Environment::UndoSelection()
{
	return m_pxUndo->Undo();
}

bool Tools_UVEdit_NodeManager_Environment::RedoSelection()
{
	return m_pxUndo->Redo();
}


void Tools_UVEdit_NodeManager_Environment::LogSelection(const Tools_UVEdit_Node* pxNode, bool bSelect)
{
	if(!m_bDisableUndoLogging)
	{
		m_pxUndo->LogNodeSelected((const Tools_UVEdit_Node_EnvVert*) pxNode, bSelect);
	}
}


///////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------------**
** Tools_UVEdit_SelectionUndoData_EnvVert **
**----------------------------------------*/

void Tools_UVEdit_SelectionUndo_EnvVert::ProcessUndoData(void* pUndo)
{
	Tools_UVEdit_SelectionUndoData_EnvVert* pxUndo = (Tools_UVEdit_SelectionUndoData_EnvVert*) pUndo;

	//undo the selection/deselection
	Tools_UVEdit_Node_EnvVert* pxNode = m_pxNodeManager->FindNode(pxUndo->m_usModuleIndex, pxUndo->m_iPolygonIndex, pxUndo->m_ucVertInPoly, pxUndo->m_ucChannel);
	if(pxNode)
	{
		if(pxUndo->m_eUndoType == Tools_UVEdit_SelectionUndoData_EnvVert::UndoType_Select)
			m_pxNodeManager->Deselect(pxNode, false);
		else
			m_pxNodeManager->Select(pxNode, false);
	}
	//convert the undo
	if(pxUndo->m_eUndoType == Tools_UVEdit_SelectionUndoData_EnvVert::UndoType_Select)
		pxUndo->m_eUndoType = Tools_UVEdit_SelectionUndoData_EnvVert::UndoType_Deselect;
	else
		pxUndo->m_eUndoType = Tools_UVEdit_SelectionUndoData_EnvVert::UndoType_Select;

	
}

void Tools_UVEdit_SelectionUndo_EnvVert::LogNodeSelected(const Tools_UVEdit_Node_EnvVert* pxNode, bool bSelect)
{
	Asura_Assert(pxNode, "");

	Tools_UVEdit_SelectionUndoData_EnvVert* pxUndo = (Tools_UVEdit_SelectionUndoData_EnvVert*) AllocateNextUndoData();
	if(!pxUndo) return;

	pxUndo->m_eUndoType = bSelect ? Tools_UVEdit_SelectionUndoData_EnvVert::UndoType_Select : Tools_UVEdit_SelectionUndoData_EnvVert::UndoType_Deselect;
	pxUndo->m_usModuleIndex = (u_short)pxNode->GetModuleIndex(); 
	pxUndo->m_iPolygonIndex = pxNode->GetPolygonIndex(); 
	pxUndo->m_ucVertInPoly = (u_char)pxNode->GetIndexInPoly(); 
	pxUndo->m_ucChannel = (u_char)pxNode->GetChannel(); 

}



