//
// Tools_Visibility_System.h
//
// Created by KJFL - 01 June 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_VISIBILITY_SYSTEM_H__
#define __TOOLS_VISIBILITY_SYSTEM_H__

#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM

class Asura_Bounding_Box;
class Tools_Environment_LOS_Data;
class Tools_VisibilitySegment;
class Tools_Visibility_SplitHintQuad;

class Tools_Visibility_System
{

public:

    typedef void(*START_VIS_CALCS_CALLBACK)( void );
    typedef void(*END_VIS_CALCS_CALLBACK)( void );

    typedef bool(*RAY_HIT_FILTER_CALLBACK)( const Asura_Vector_3& xRayStart, const Asura_Vector_3& xRayDir, const Asura_Vector_3& xHitPoint, u_int uHitModuleIdx, u_int uHitPolyIdx );
    typedef bool(*RAY_MODULE_TEST_FILTER_CALLBACK)( Asura_Hash_ID uZoneHash, u_int uModuleIdx, const void* pvUserData, const Tools_Environment_LOS_Data& xLOSData );

    static void Initialise( void );
    static void Reset( void );
    static void Shutdown( void );

    static void ToggleOverlay( void );
    static bool IsOverlayOpen( void );

    static u_int GetNumberOfVisibilityFeatures( void );
    static u_int GetSkyBoxFeatureID( void );

    static bool IsExternalFeature( u_int uFeatureID );

    static void RefineVisibility( const Asura_Vector_3& xSamplePoint, const Asura_Vector_3& xDirection );

    static void DebugRender( void );

    static void StartProgressIndicator( const char* pszCurrentTaskString );
    static void UpdateProgressIndicator( u_int uCurrent = 0, u_int uMax = 0, bool bForceUpdate = false );

    static void SetStartVisCalcsCallback( START_VIS_CALCS_CALLBACK pfnStartVisCalcsCallback );
    static void SetEndVisCalcsCallback( END_VIS_CALCS_CALLBACK pfnEndVisCalcsCallback );

    // NB: these callbacks must be thread-safe.
    static void SetRayHitFilterCallback( RAY_HIT_FILTER_CALLBACK pfnHitFilterCallback );
    static void SetRayModuleTestFilterCallback( RAY_MODULE_TEST_FILTER_CALLBACK pfnModuleTestFilterCallback, const void* pvUserData );

    static bool FireSingleRay( const Asura_Vector_3& xStart, const Asura_Vector_3& xDirection, Asura_Vector_3& xHitPoint, bool& bFrontFacing, u_int& uFeatureID, Asura_Vector_3& xNormal, bool& bHitVisBlocker );

    static bool ShouldHighlightNonVisibleGeometry( void );

private:

    static void SetAsModified( void );

    static void RecalculateBaseEnvironmentData( void );

    static void InitialiseAsuraData( void );

    static void CreateOverlay( void );
    static void CloseOverlay( void );

    static void InitialiseProgressIndicator( void );
    static void ShutdownProgressIndicator( void );

    static void CreateViewCells( int iSetting );

    static bool CreateVisibilitySegments( void );
    static void CreateVisibilitySegments_MT_Callback( const void* pxTaskDataPacket );
    static bool CreateVisibilitySegments_MT( void );
    static void ClearStoredVisibilitySegments( void );

    static bool Split( void );
    static void SplitMT_Callback( const void* pData );
    static bool SplitMT( void );
    static void FindSplitHintQuads( void );
    static void CalculateSplitHintQuadAABB( Tools_Visibility_SplitHintQuad* pxSplitHintQuad, int iInitialModule, int iInitialPoly );
    static void RenderSplitHintQuads( void );

    static bool CreateLeafToLeafVisibilties( void );

    static void RemoveLinks( void );
    static void DoIncrementalAutoRefinement( void );
    static void AutoRefine( void );
    static void AutoRefine_MT( void );
    static void AutoRefine_MT_Callback( const void* pxTaskDataPacket );

    // get a random sample point - either in a random visibility feature, or in the world as a whole
    // (MP - 17.03.2009) Now returns true if it succeeded in finding a
    // suitable point.
    static bool GetRandomPoint( Asura_Vector_3& xPoint, bool bInAVisibilityFeature = false ); 
    static bool GetRandomPointInAABB( Asura_Vector_3& xPoint, const Asura_Bounding_Box& xBB );
    static bool IsVisSegmentSamplePointValid( const Asura_Vector_3& xSamplePoint );

    static void GetRandomDirection( Asura_Vector_3& xDirection );

    static bool FireRay_Internal( const Asura_Vector_3& xStart, const Asura_Vector_3& xDirection, Asura_Vector_3& xHitPoint, bool& bFrontFacing, u_int& uFeatureID, Asura_Vector_3& xNormal, bool& bHitVisBlocker );
    static u_int FindVisNodeUnderCursor( void );

    static bool ShouldRenderVisibleNodeBBs( void );
    static void RenderVisibleNodeBBs( void );

    static void ManuallySelectCameraNode( void );
    static bool ManuallySelectCameraNode_PickCallback( bool bClicked );
    static void ManuallySelectCameraNode_RenderCallback( u_int uSuggestedARGB );

    static void RenderVisSegments( void );
    static bool RenderVisSegments_ShouldRender( void );
    static bool RenderVisSegments_ShouldBeAllowedToSelectTargetNode( void );
    static void RenderVisSegments_RenderSingleSegment( const Tools_VisibilitySegment* pxSegment );
    static void RenderVisSegments_GenerateFilteredList( void );
    static bool RenderVisSegments_Filter( const Tools_VisibilitySegment* pxSegment );
    static void RenderVisSegments_SelectTargetNode( void );
    static bool RenderVisSegments_SelectTargetNode_PickCallback( bool bClicked );
    static void RenderVisSegments_SelectTargetNode_RenderCallback( u_int uSuggestedARGB );
    static void RenderVisSegments_ClearTargetNode( void );
    static void RenderVisSegments_ClearFilteredList( void );

};

#endif //ASURA_USE_NEW_VISIBILITY_SYSTEM

#endif//__TOOLS_VISIBILITY_SYSTEM_H__

