//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Editing.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_WAYPOINT_EDITING_H__
#define __TOOLS_WAYPOINT_EDITING_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Tabs_Dialog;
class Asura_Vector_2;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Editing
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Waypoint_Editing
{
public:
    Tools_Waypoint_Editing();
    virtual ~Tools_Waypoint_Editing();

    virtual void Shutdown();

    virtual void AddOverlayItemsToTabsDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );

    // Adds a link from one -> two and two -> one, if possible.
    virtual void AddLink( const int iWaypointOne, const int iWaypointTwo, const bool bRecalcDependentData, const bool bTwoWay = true );
    // Deletes any links from one -> two and two -> one.
    virtual void DeleteLink( const int iWaypointOne, const int iWaypointTwo, const bool bRecalcDependentData, const bool bTwoWay = true );
    // Deletes the waypoint and all links to and from it.
    virtual void DeleteWaypointAndLinks( const int iWaypointIndex, const bool bRecalcDependentData );

protected:
	virtual void CreateWaypoint();
    virtual void AdjustSelectedWaypointRadius();
    virtual void StopAdjustingWaypointRadius();
    void AddLinkBetweenSelectedWaypoints( int iUserData );
    void DeleteLinkBetweenSelectedWaypoints( int iUserData );
    void DeleteSelectedWaypointsAndLinks();

    static void InsertLinkIntoWaypointInfo( const int iWaypointOne, const int iWaypointTwo, const unsigned int uFlags, const float fDistance );
    static void RemoveLinkFromWaypointInfo( const int iWaypointOne, const int iWaypointTwo );
    static void RemoveWaypointFromWaypointInfo( const int iWaypoint );
    static void UpdateWaypointLinkBlocks();

	static void CreateWaypointStatic()										{ s_pxThis->CreateWaypoint();									}
    static void AdjustSelectedWaypointRadiusStatic()						{ s_pxThis->AdjustSelectedWaypointRadius();						}
    static void StopAdjustingWaypointRadiusStatic()							{ s_pxThis->StopAdjustingWaypointRadius();						}
    static void AddLinkBetweenSelectedWaypointsStatic( int iUserData )		{ s_pxThis->AddLinkBetweenSelectedWaypoints( iUserData );		}
    static void DeleteLinkBetweenSelectedWaypointsStatic( int iUserData )	{ s_pxThis->DeleteLinkBetweenSelectedWaypoints( iUserData );	}
    static void DeleteSelectedWaypointsAndLinksStatic()						{ s_pxThis->DeleteSelectedWaypointsAndLinks();					}

	static bool AllowedToCreateWaypoint();
    static bool AllowedToAdjustWaypointRadius();
    static bool AllowedToEditLinking();
    static bool AllowedToDeleteWaypoints();

protected:
    bool           m_bAdjustingWaypointRadius;
    Asura_Vector_2 m_xRadiusAdjustOverlayPosition;

    static Tools_Waypoint_Editing* s_pxThis;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_WAYPOINT_EDITING_H__
