//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Object.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Nick Davis
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Waypointing/Tools_Waypoint_Object.h"

// asura
#include "Mesh/Asura_Mesh_System.h"

// tools
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Object
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< Tools_Waypoint_Object > Tools_Waypoint_Object::s_xList;

Tools_Waypoint_Object::Tools_Waypoint_Object( Asura_AI_Navigation_System::UserDefinedWaypoint* pxUDW )
: Tools_Object( TOT_UserDefinedWaypointPosition )
, m_uGuid( pxUDW->GetID() )
, m_xName()
, m_xUDW( *pxUDW )
{
    const Asura_String xMeshName = "ToolsShape_CutsceneTrackPoint";
    SetMesh( Asura_Mesh_System::GetMesh( xMeshName ) );

	Asura_String xIDString;
	Asura_StringUtil::ToString( m_xUDW.GetID(), xIDString );
	m_xName += "User Defined Waypoint " + xIDString;

    Position = m_xUDW.m_xPosition;
    m_xUDW.m_xOrientQ.ToMatrix( Orientation );

	s_xList.AddToList( this );
}

Tools_Waypoint_Object::~Tools_Waypoint_Object()
{ 
	s_xList.RemoveFromList( this );
}

void Tools_Waypoint_Object::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
    PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );

	m_xUDW.m_xPosition = Position;
	m_xUDW.m_xOrientQ.FromMatrix( Orientation );

	Tools_SaveAsura::SetWaypointingModified();
}

void Tools_Waypoint_Object::Destroy()
{
    // Delete our UDW in the Asura_AI_Navigation_System data.
    Asura_AI_Navigation_System::DeleteUserDefinedWaypoint( &m_xUDW );
    Tools_SaveAsura::SetWaypointingModified();

    PARENT::Destroy();
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
