//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Object.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Nick Davis
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_WAYPOINT_OBJECT_H___
#define ___TOOLS_WAYPOINT_OBJECT_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// tools
#include "Objects/Tools_Object.h"

// Asura
#include "AI/Asura_AI_Navigation.h"
#include "Core/Asura_HashedList.h"
#include "Core/Asura_String.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Object
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Waypoint_Object : public Tools_Object
{
    typedef Tools_Object PARENT;
	friend class Tools_Mode_Waypointing;

public:
    Tools_Waypoint_Object( Asura_AI_Navigation_System::UserDefinedWaypoint* pxUDW );
    virtual ~Tools_Waypoint_Object();

    virtual void UpdatePositionInAsuraData( bool bUpdateLinkedObjects );
    virtual const char* GetName()            { return m_xName.GetString(); }
	virtual const char* GetTypeDescription() { return "User Defined Waypoint"; }
    // Called when this is explicitly deleted by the user.
    virtual void Destroy();
	// We have a separate waypointing verify so these don't get verified in objects mode.
	virtual bool WaypointingVerify() { return PARENT::Verify( false ); }
	virtual Asura_Guid GetGuid() const { return m_uGuid; }
	bool IsDestroyed() { return false; }

	const Asura_Vector_3& GetPosition() const { return m_xUDW.m_xPosition; }
	const Asura_Quat&     GetOrientQ() const { return m_xUDW.m_xOrientQ; }
	u_int                 GetFlags() const { return m_xUDW.m_uFlags; }
	const Asura_AI_Navigation_System::UserDefinedWaypoint& GetUDW() const { return m_xUDW; }

	static Asura_Hashed_List< Tools_Waypoint_Object >* GetList() { return &s_xList; }

protected:
	Asura_AI_Navigation_System::UserDefinedWaypoint& GetUDW() { return m_xUDW; }
	void SetName( const Asura_String& xName ) { m_xName = xName; }

private:
	const Asura_Guid m_uGuid;
	Asura_String m_xName;
	Asura_AI_Navigation_System::UserDefinedWaypoint& m_xUDW;

	static Asura_Hashed_List< Tools_Waypoint_Object > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif
