//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Selection.h
//
// Provides functionality to select waypoints using different methods, and
// remembers which waypoints are currently selected and the order in which
// they were selected.
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_WAYPOINT_SELECTION_H__
#define __TOOLS_WAYPOINT_SELECTION_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "AI/Asura_AI_Navigation.h"
#include "Core/Asura_Maths.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Tabs_Dialog;

class Tools_ModeDataFile;

//////////////////////////////////////////////////////////////////////////////
//
// Helper Classes
//
//////////////////////////////////////////////////////////////////////////////

class Tools_AStar
{
public:
    float        m_fCostToHere;
    float        m_fCostToGoal;
    float        m_fTotalCost;
    int          m_iIndex;
    bool         m_bOpen;
    bool         m_bClosed;
    Tools_AStar* m_pxParent;
};

class Tools_Waypoint_Selection_AStar
{
public:
    static void Initialise();
    static void Shutdown();
    static void DoAStar();

    static float *s_pfMinDistances;
    static bool *s_pbFoundThisNode;
private:
    static Tools_AStar* GetOpenListNode();
    static Tools_AStar* s_pxAStarList;
    static Asura_AStar_OpenList_Elem* s_pxAStarOpenList;
    static int s_iNumElemsInOpenList;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Waypoint_Selection
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Waypoint_Selection
{
    friend class Tools_Waypoint_Selection_AStar;

public:
    static void Activate();
    static void Deactivate();
	static void OnWaypointDataChanged();
    static void TransferToolsModeData( const int iMode, Tools_ModeDataFile& xFile );
    static bool IsEnabled() { return s_bEnabled; }

    static void AddOverlayItemsToTabsDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );

    static void Update();
    static void Render();

    static void RenderHighlitWaypoint(int iIndex);
    static void RenderSelectedWaypoint(int iIndex);

    struct SelectionData
    {
        SelectionData() : m_bIsSelected( false ), m_iSelectionIndex( -1 ) {}
        bool m_bIsSelected;
        int  m_iSelectionIndex;
    };
	static void			  ClearSelection();
    static int            GetNumSelectedWaypoints() { return s_iNumSelected;          }
    static int            GetSelectionArrayLength() { return s_iSelectionArrayLength; }
    static SelectionData* GetSelectionArray()       { return s_pxSelectionArray;      }
    static int            GetHighlitWaypointIndex() { return s_iHighlitWaypoint;      }
    // Supplies the index of the only selected waypoint. Returns whether it found a selected waypoint or not.
    static bool           GetSelectedWaypoint( int& iWaypointOne );
    // This will supply the first two selected waypoint indices, in the order in which they were selected by the user. Returns whether it found two waypoints or not.
    static bool           GetFirstTwoSelectedWaypoints( int& iWaypointOne, int& iWaypointTwo );

private:
    static void InvertSelection();
    static void SelectByIndex();
    static void SelectByCoverpointIndex();
    static void SelectByZoneID();
    static void SelectAll();
    static void SelectAllLinked();
    static void SelectAllUnlinked();
    static void HandleMouseSelection();
    static void RenderLinks( int iWPIndex );
    static bool GetWaypointUnderMouse();

private:
	static bool           s_bActive;
    static bool           s_bEnabled;
    // The length of the selection array. Here for safety - should always be the same as the number of waypoints.
    static int            s_iSelectionArrayLength;
    // Array of the selected waypoints.
    static SelectionData* s_pxSelectionArray;
    static int            s_iNumSelected;
    // Waypoint the cursor is currently over.
    static int            s_iHighlitWaypoint;
    static bool           s_bButtonDownForSelection;

    enum { CURRENT_MODE_DATA_VERSION = 0 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_WAYPOINT_SELECTION_H__
