//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_Extended_Particle_Rain_Shelter.cpp
//
//  CREATED     :  Adam A. Brown  (19.6.2003)
//
//  Copyright (C) 2003 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////


#include "Core/Asura.h"

// This file's header.
#include "Weather/Tools_Extended_Particle_Rain_Shelter.h"

#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "SpecialFX/Asura_Extended_Particle_Rain_System.h"

#include "Tools_Overlays.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeEdit_Object.h"



//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//
// Tools_Extended_Particle_Rain_Shelter
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------

////////////////////////////////////////
// Static data.
////////////////////////////////////////
Asura_Vector_2 Tools_Extended_Particle_Rain_Shelter::s_xOverlayPosition = Asura_Vector_2(10.f, 10.f);
bool Tools_Extended_Particle_Rain_Shelter::s_bOverlayOn = false;
Tools_Extended_Particle_Rain_Shelter* Tools_Extended_Particle_Rain_Shelter::s_pxThis = NULL;


////////////////////////////////////////////////////////////////////////////////
//
// Public statics
//
////////////////////////////////////////////////////////////////////////////////
Tools_Object* Tools_Extended_Particle_Rain_Shelter::Create(void)
{
    // resize the weather system buffer and copy the data from the old buffer
    Asura_Extended_Particle_Rain_Shelter* pxShelter = Asura_Extended_Particle_Rain_System::NewRainShelter();

    // Create the tools object
    Tools_Object* pxToolsObject = new Tools_Extended_Particle_Rain_Shelter(pxShelter);
    Tools_Object_System::SelectObject(pxToolsObject);

	pxShelter->m_xOBB.CentreOn(pxToolsObject->Position);

    return pxToolsObject;
};


void Tools_Extended_Particle_Rain_Shelter::EditVolume()
{
	Asura_Extended_Particle_Rain_Shelter *pxShelter = s_pxThis->m_pxShelter;

	if(!s_pxThis->VolumeObject)
	{	
		//Center volume on tools object.
		//pxShelter->m_xOBB.CentreOn(s_pxThis->Position);

		//create the volume object
		char szVolumeName[200];
		sprintf(szVolumeName, "%s - Volume", s_pxThis->GetName());
		new Tools_Oriented_VolumeEdit_Object(szVolumeName, &pxShelter->m_xOBB, &s_pxThis->VolumeObject);
		s_pxThis->VolumeObject->SetOwner(s_pxThis);
	}

	//select the volume object
	Tools_Object_System::SelectObject(s_pxThis->VolumeObject);
}



////////////////////////////////////////////////////////////////////////////////
//
// Constructor / Destructor
//
////////////////////////////////////////////////////////////////////////////////
Tools_Extended_Particle_Rain_Shelter::Tools_Extended_Particle_Rain_Shelter(Asura_Extended_Particle_Rain_Shelter* pxShelter) 
: Tools_Object(TOT_Extended_Rain_Shelter)
, VolumeObject(NULL)
{
    if(pxShelter)
    {   
        this->m_pxShelter = pxShelter;
        
        Asura_String mesh_name = "ToolsShape_CutsceneEvent";
        Mesh = Asura_Mesh_System::GetMesh(mesh_name);

        m_bChanged = false;
        char szName[100];

        sprintf( szName, "Rain Shelter XXXX");//, m_uShelterID);
        m_xName = szName;

        Orientation.Identity();

        if(pxShelter)
        {
            Position = pxShelter->m_xOBB.CalculateCentre();
            m_xPrevPosition = Position;           
        }
    }
}


Tools_Extended_Particle_Rain_Shelter::~Tools_Extended_Particle_Rain_Shelter()
{

}


////////////////////////////////////////////////////////////////////////////////
//
// OnSelection / OnDeselection
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Extended_Particle_Rain_Shelter::OnSelection()
{
    CreateOverlay();

	s_pxThis = this;
}

void Tools_Extended_Particle_Rain_Shelter::OnDeselection()
{
    DeleteOverlay();
}


////////////////////////////////////////////////////////////////////////////////
//
// Overlay functions
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Extended_Particle_Rain_Shelter::CreateOverlay()
{
    Asura_Overlay* pxNewOverlay;
    pxNewOverlay = new Asura_Overlay(GetTypeDescription(), false, &s_xOverlayPosition);
    
    Asura_Dialog* pxNewDialog;
    Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxNewDialog = new Asura_Function_Dialog("Edit Volume", xOffset, EditVolume);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += 1.5f;
    
    pxNewOverlay->ID = TOOLS_OVERLAY_SELECTEDOBJECT;

    pxNewOverlay->Origin = s_xOverlayPosition;
    Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay, false);
}


void Tools_Extended_Particle_Rain_Shelter::DeleteOverlay()
{
    Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}


////////////////////////////////////////////////////////////////////////////////
//
// UpdateWhileSelected
//
////////////////////////////////////////////////////////////////////////////////
void Tools_Extended_Particle_Rain_Shelter::UpdateWhileSelected()
{
    if( !(Position == m_xPrevPosition) )
    {
        m_bChanged = true;
    }
    
    if(m_bChanged)
    {
        //1: Update position of shelter.
		if(m_pxShelter)
		{
			m_pxShelter->m_xOBB.Displace(Position - m_xPrevPosition);
		}

		//2: Dirty mode

		m_xPrevPosition = Position;
		m_bChanged = false;
    }

    Tools_Object::UpdateWhileSelected();
}
