/*************************************************************
 *
 * Description   : Tools files for the weather mode.
 *
 * File Name     : Tools_Mode_Weather.h
 *
 * Author        : Richard May
 *
 *************************************************************/

#ifndef _TOOLS_MODE_WEATHER_H_ /* GATE. */
#define _TOOLS_MODE_WEATHER_H_

enum TOOLS_WEATHER_SUBMODES
{
	TOOLS_WEATHER_SUBMODE_NONE = 0,
};

/************/
/* Includes */
/************/

#include "Tools_Mode.h"

/************/
/* Classes. */
/************/

template<class T> class List;
template<class T> class Asura_Graph;
class Asura_Tabs_Dialog;
class Tools_CSMDirector_Object;

class Tools_Mode_Weather : public Tools_Mode
{
public:
	Tools_Mode_Weather()
	{
		s_pxThis = this;
	}
	
	virtual ~Tools_Mode_Weather()
	{
		s_pxThis = NULL;
	}

	virtual void InitialiseMode();
	virtual void Update();
	virtual void RenderWorld_PostToneMap();
	virtual void ShutdownMode();
	virtual void QuickSave();
 	virtual bool IsModified();

	// Logs current state of selection for undo
	static void PrepareUndo();
	static void Undo();
	static void Redo();

	static void TransferModeData(int mode);

	static void EnterSubMode( const TOOLS_WEATHER_SUBMODES eSubMode );
	static void ExitMode( void );

	static const TOOLS_WEATHER_SUBMODES GetSubMode(){ return s_eSubMode; };

	#ifdef ASURA_USE_WATER
	static bool IsPreviewingWater() { return s_bWaterPreview; }
	#endif //ASURA_USE_WATER

protected:
	virtual void AddWeatherOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	void RefreshWeatherOverlay();
	
	static bool s_bMadeChanges;
	static Tools_Mode_Weather* s_pxThis;
	static Asura_Tabs_Dialog* s_pxTabsDialog;

private:
	static void HandleKeyboard();
	static void ToggleWeatherOverlay();
	static bool WeatherOverlayIsOn();
	static void CreateWeatherOverlay();
    static void MoveSunToCamPos( void );

	#ifdef ASURA_USE_PSP_FILTER
	static bool IsEnabled_PSPFilter();
	#endif

	#ifdef ASURA_USE_PSP_BLUR
	static bool IsEnabled_PSPBlur();
	#endif


	#ifdef ASURA_USE_WATER
	static void ToggleWaterOverlay();
	static bool WaterOverlayIsOn();
	static void CreateWaterOverlay();

	static void UpdateWaterOverlay();

	static void SetWaterSunDirection();
	static void SelectWaterTexture(int iIndex);
	static void ClearWaterTexture(int iIndex);

	static void StartWaterPreview();
	static void StopWaterPreview();

	static void SelectWaterTemplate();
	static void ResetWaterToDefaults();
	#endif //ASURA_USE_WATER

	enum DialogID
	{
		DialogID_Water_SelectTemplate,
		DialogID_Water_Texture0,
		DialogID_Water_Texture1,
		DialogID_Water_Texture2,

		DialogID_Water_WaterCycleScale1,
		DialogID_Water_WaterCycleScale2,
		DialogID_Water_WaterCycleScale3,
		DialogID_Water_XScale1,
		DialogID_Water_XScale2,
		DialogID_Water_XScale3,
		DialogID_Water_YScale1,
		DialogID_Water_YScale2,
		DialogID_Water_YScale3,
		DialogID_Water_ZScale1,
		DialogID_Water_ZScale2,
		DialogID_Water_ZScale3,
		DialogID_Water_Falloff1,
		DialogID_Water_Falloff2,
		DialogID_Water_Falloff3,
		DialogID_Water_Threshold,
		DialogID_Water_LightOffset,
		DialogID_Water_LightScale,
		DialogID_Water_AlphaLevel,
		DialogID_Water_Reflection,
		DialogID_Water_SunTextureOffsetU,
		DialogID_Water_SunTextureOffsetV,
		DialogID_Water_SunTextureScaleU,
		DialogID_Water_SunTextureScaleV,
		DialogID_Water_SunYScale,
		DialogID_Water_HorizonVScale,
		DialogID_Water_HorizonVOffset,
		DialogID_Water_HorizonYScale,
		DialogID_Water_HorizonBrightness,
		DialogID_Water_HorizonFresnelScale,
		DialogID_Water_Transparent,
		DialogID_Water_HorizonReflect,
		DialogID_Water_SunReflect,
		DialogID_Water_BlendHorizon,
		DialogID_Water_SubdivisionYScale,
		DialogID_Water_SunlightFalloffNear,
		DialogID_Water_SunlightFalloffFar,
        DialogID_Water_UScrollSpeed,
        DialogID_Water_VScrollSpeed,

		DialogID_Water_Preview_Start,
		DialogID_Water_Preview_Stop,
    };

	static Asura_Handle s_iMenuHandle;
	static Asura_Vector_2 s_xWeatherOverlayPosition;
	static bool	s_bWeatherOverlayOpen;
	static TOOLS_WEATHER_SUBMODES	s_eSubMode;
	static bool s_bChangedFogOverlay;
	static Asura_Graph<float>* s_pxFogDensityGraph;
	static bool s_bSunGlareEnabled;
	static bool s_bLensFlareEnabled;
	static bool s_bHorizonGlareEnabled;
	static bool s_bHideSunGlareIcon;
	static bool s_bFogEnabled;
	static bool s_bGraphFogEnabled;
	static float s_fSkyboxFogBlend;
	static float s_fFogNearPlane;
	static float s_fFogFarPlane;
	static float s_fFogValueAtFarPlane;
	static float s_fFogRed;
	static float s_fFogGreen;
	static float s_fFogBlue;
	static float s_fGlowRed;
	static float s_fGlowGreen;
	static float s_fGlowBlue;
	static float s_fGlowLevel;
	static bool s_bChangedMinFog;
	static bool s_bChangedMaxFog;

	//for simplicity with ini file , have water overlay position available regardless of whether water is allowed
	static bool s_bWaterOverlayOpen;
	static Asura_Vector_2 s_xWaterOverlayPosition;
#ifdef ASURA_USE_WATER
	static bool s_bWaterModified;
	static bool s_bWaterPreview;
#endif // ASURA_USE_WATER

#ifdef ASURA_USE_CSM
	static Tools_CSMDirector_Object* s_pxCSMDirector;
#endif
};

/****************/
/* End of file. */
/****************/

#endif /* GATE */
