//
// Tools_WaterTemplate.cpp
//
// Created by Richard Rice - 11/01/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Weather/Tools_WaterTemplate.h"

#include "Misc/list_tem.hpp"
#include "Misc/Tools_IniFile.h"
#include "Rendering/Asura_Water.h"



List<Tools_WaterTemplate*> Tools_WaterTemplate_System::s_xTemplateList;

void Tools_WaterTemplate_System::Initialise()
{
	LoadTemplates();
}

void Tools_WaterTemplate_System::Shutdown()
{
	while(s_xTemplateList.size())
	{
		delete s_xTemplateList.first_entry();
		s_xTemplateList.delete_first_entry();
	}
}

const Tools_WaterTemplate* Tools_WaterTemplate_System::GetTemplate(const char* pszName)
{
	for(LIF<Tools_WaterTemplate*> xTempLIF(&s_xTemplateList);!xTempLIF.done();xTempLIF.next())
	{
		if(!stricmp(pszName,xTempLIF()->m_xTemplateName.GetString()))
		{
			return  xTempLIF();
		}
	}
	return 0;
}

void Tools_WaterTemplate_System::UpdateFromTemplate()
{
	Asura_Water_System::Env_Water_Controls& xControl = Asura_Water_System::GetEnvController();
	if(xControl.m_xTemplateName.IsBlank()) return;

	//find the template
	const Tools_WaterTemplate* pxTemplate = GetTemplate(xControl.m_xTemplateName.GetString());
	if(!pxTemplate) return;

	//update all the entries that are valid in the template
	if(pxTemplate->m_bWaterCycleScale1_Valid) xControl.m_fWaterCycleScale1 = pxTemplate->m_fWaterCycleScale1;
	if(pxTemplate->m_bWaterCycleScale2_Valid) xControl.m_fWaterCycleScale2 = pxTemplate->m_fWaterCycleScale2;
	if(pxTemplate->m_bWaterCycleScale3_Valid) xControl.m_fWaterCycleScale3 = pxTemplate->m_fWaterCycleScale3;

	if(pxTemplate->m_bXScale1_Valid) xControl.m_fXScale1 = pxTemplate->m_fXScale1;
	if(pxTemplate->m_bXScale2_Valid) xControl.m_fXScale2 = pxTemplate->m_fXScale2;
	if(pxTemplate->m_bXScale3_Valid) xControl.m_fXScale3 = pxTemplate->m_fXScale3;

	if(pxTemplate->m_bYScale1_Valid) xControl.m_fYScale1 = pxTemplate->m_fYScale1;
	if(pxTemplate->m_bYScale2_Valid) xControl.m_fYScale2 = pxTemplate->m_fYScale2;
	if(pxTemplate->m_bYScale3_Valid) xControl.m_fYScale3 = pxTemplate->m_fYScale3;

	if(pxTemplate->m_bZScale1_Valid) xControl.m_fZScale1 = pxTemplate->m_fZScale1;
	if(pxTemplate->m_bZScale2_Valid) xControl.m_fZScale2 = pxTemplate->m_fZScale2;
	if(pxTemplate->m_bZScale3_Valid) xControl.m_fZScale3 = pxTemplate->m_fZScale3;

	if(pxTemplate->m_bFalloff1_Valid) xControl.m_fFalloff1 = pxTemplate->m_fFalloff1;
	if(pxTemplate->m_bFalloff2_Valid) xControl.m_fFalloff2 = pxTemplate->m_fFalloff2;
	if(pxTemplate->m_bFalloff3_Valid) xControl.m_fFalloff3 = pxTemplate->m_fFalloff3;

	if(pxTemplate->m_bThreshold_Valid) xControl.m_fThreshold = pxTemplate->m_fThreshold;
	if(pxTemplate->m_bLightOffset_Valid) xControl.m_fLightOffset = pxTemplate->m_fLightOffset;
	if(pxTemplate->m_bLightScale_Valid) xControl.m_fLightScale = pxTemplate->m_fLightScale;

	if(pxTemplate->m_bAlphaLevel_Valid) xControl.m_fAlphaLevel = pxTemplate->m_fAlphaLevel;
	if(pxTemplate->m_bReflection_Valid) xControl.m_fSunBrightness = pxTemplate->m_fReflection;

	if (pxTemplate->m_bSunTextureOffsetU_Valid) xControl.m_afTexOffset[0] = pxTemplate->m_fSunTextureOffsetU;
	if (pxTemplate->m_bSunTextureOffsetV_Valid) xControl.m_afTexOffset[1] = pxTemplate->m_fSunTextureOffsetV;
	if (pxTemplate->m_bSunTextureScaleU_Valid) xControl.m_afTexScale[0] = pxTemplate->m_fSunTextureScaleU;
	if (pxTemplate->m_bSunTextureScaleV_Valid) xControl.m_afTexScale[1] = pxTemplate->m_fSunTextureScaleV;

	if(pxTemplate->m_bHorizonVScale_Valid) xControl.m_fHorizonVScale = pxTemplate->m_fHorizonVScale;
	if(pxTemplate->m_bHorizonVOffset_Valid) xControl.m_fHorizonVOffset = pxTemplate->m_fHorizonVOffset;
	if(pxTemplate->m_bHorizonYScale_Valid) xControl.m_fHorizonYScale = pxTemplate->m_fHorizonYScale;
	if(pxTemplate->m_bHorizonBrightness_Valid) xControl.m_fHorizonBrightness = pxTemplate->m_fHorizonBrightness;

	if(pxTemplate->m_bSubdivisionYScale_Valid) xControl.m_fSubdivisionYScale = pxTemplate->m_fSubdivisionYScale;
	if(pxTemplate->m_bSunlightFalloffNear_Valid) xControl.m_fSunlightFalloffNear = pxTemplate->m_fSunlightFalloffNear;
	if(pxTemplate->m_bSunlightFalloffFar_Valid) xControl.m_fSunlightFalloffFar = pxTemplate->m_fSunlightFalloffFar;

	if(pxTemplate->m_bTransparent_Valid)
	{
		if(pxTemplate->m_bTransparent)
			xControl.m_uFlags |= AsuraWaterFlag_Transparent;
		else
			xControl.m_uFlags &=~ AsuraWaterFlag_Transparent;

	}

	if(pxTemplate->m_bHorizonReflect_Valid)
	{
		if(pxTemplate->m_bHorizonReflect)
			xControl.m_uFlags |= AsuraWaterFlag_HorizonReflect;
		else
			xControl.m_uFlags &=~ AsuraWaterFlag_HorizonReflect;

	}

	if(pxTemplate->m_bSunReflect_Valid)
	{
		if(pxTemplate->m_bSunReflect)
			xControl.m_uFlags |= AsuraWaterFlag_SunReflect;
		else
			xControl.m_uFlags &=~ AsuraWaterFlag_SunReflect;

	}

	if(pxTemplate->m_bBlendHorizon_Valid)
	{
		if(pxTemplate->m_bBlendHorizon)
			xControl.m_uFlags |= AsuraWaterFlag_BlendHorizon;
		else
			xControl.m_uFlags &=~ AsuraWaterFlag_BlendHorizon;

	}

    if( pxTemplate->m_bUScrollSpeed_Valid ) { xControl.m_afUVSpeeds[0] = pxTemplate->m_fUScrollSpeed; }
    if( pxTemplate->m_bVScrollSpeed_Valid ) { xControl.m_afUVSpeeds[1] = pxTemplate->m_fVScrollSpeed; }
}


void Tools_WaterTemplate_System::LoadTemplates()
{
	Tools_IniFile xIni("ToolsIniFiles/WaterTemplates.ini");
	if(!xIni.LoadIniFile()) return;

	//each section in the ini file should be a different template
	for(int i=0;i<xIni.GetNumberOfSections();i++)
	{
		if(!xIni.GetIniTextSectionByIndex(i)) continue;
		if(!xIni.GetCurrentSectionName()) continue;

		//allocate the new template
		Tools_WaterTemplate* pxTemplate = new Tools_WaterTemplate;
		pxTemplate->m_xTemplateName = xIni.GetCurrentSectionName();
		s_xTemplateList.add_entry(pxTemplate);

		//read in the values
		pxTemplate->m_bWaterCycleScale1_Valid = xIni.GetFloatInCurrentSection("WaterCycleScale1",pxTemplate->m_fWaterCycleScale1);
		pxTemplate->m_bWaterCycleScale2_Valid = xIni.GetFloatInCurrentSection("WaterCycleScale2",pxTemplate->m_fWaterCycleScale2);
		pxTemplate->m_bWaterCycleScale3_Valid = xIni.GetFloatInCurrentSection("WaterCycleScale3",pxTemplate->m_fWaterCycleScale3);

		pxTemplate->m_bXScale1_Valid = xIni.GetFloatInCurrentSection("XScale1",pxTemplate->m_fXScale1);
		pxTemplate->m_bXScale2_Valid = xIni.GetFloatInCurrentSection("XScale2",pxTemplate->m_fXScale2);
		pxTemplate->m_bXScale3_Valid = xIni.GetFloatInCurrentSection("XScale3",pxTemplate->m_fXScale3);

		pxTemplate->m_bYScale1_Valid = xIni.GetFloatInCurrentSection("YScale1",pxTemplate->m_fYScale1);
		pxTemplate->m_bYScale2_Valid = xIni.GetFloatInCurrentSection("YScale2",pxTemplate->m_fYScale2);
		pxTemplate->m_bYScale3_Valid = xIni.GetFloatInCurrentSection("YScale3",pxTemplate->m_fYScale3);
	
		pxTemplate->m_bZScale1_Valid = xIni.GetFloatInCurrentSection("ZScale1",pxTemplate->m_fZScale1);
		pxTemplate->m_bZScale2_Valid = xIni.GetFloatInCurrentSection("ZScale2",pxTemplate->m_fZScale2);
		pxTemplate->m_bZScale3_Valid = xIni.GetFloatInCurrentSection("ZScale3",pxTemplate->m_fZScale3);

		pxTemplate->m_bFalloff1_Valid = xIni.GetFloatInCurrentSection("Falloff1",pxTemplate->m_fFalloff1);
		pxTemplate->m_bFalloff2_Valid = xIni.GetFloatInCurrentSection("Falloff2",pxTemplate->m_fFalloff2);
		pxTemplate->m_bFalloff3_Valid = xIni.GetFloatInCurrentSection("Falloff3",pxTemplate->m_fFalloff3);

		pxTemplate->m_bThreshold_Valid = xIni.GetFloatInCurrentSection("Threshold",pxTemplate->m_fThreshold);
		pxTemplate->m_bLightOffset_Valid = xIni.GetFloatInCurrentSection("LightOffset",pxTemplate->m_fLightOffset);
		pxTemplate->m_bLightScale_Valid = xIni.GetFloatInCurrentSection("LightScale",pxTemplate->m_fLightScale);
	
		pxTemplate->m_bAlphaLevel_Valid = xIni.GetFloatInCurrentSection("AlphaLevel",pxTemplate->m_fAlphaLevel);
		pxTemplate->m_bReflection_Valid = xIni.GetFloatInCurrentSection("Reflection",pxTemplate->m_fReflection);

		pxTemplate->m_bSunTextureOffsetU_Valid = xIni.GetFloatInCurrentSection("SunTextureOffsetU", pxTemplate->m_fSunTextureOffsetU);
		pxTemplate->m_bSunTextureOffsetV_Valid = xIni.GetFloatInCurrentSection("SunTextureOffsetV", pxTemplate->m_fSunTextureOffsetV);
		pxTemplate->m_bSunTextureScaleU_Valid = xIni.GetFloatInCurrentSection("SunTextureScaleU", pxTemplate->m_fSunTextureScaleU);
		pxTemplate->m_bSunTextureScaleV_Valid = xIni.GetFloatInCurrentSection("SunTextureScaleV", pxTemplate->m_fSunTextureScaleV);

		pxTemplate->m_bHorizonVScale_Valid = xIni.GetFloatInCurrentSection("HorizonVScale",pxTemplate->m_fHorizonVScale);
		pxTemplate->m_bHorizonVOffset_Valid = xIni.GetFloatInCurrentSection("HorizonVOffset",pxTemplate->m_fHorizonVOffset);
		pxTemplate->m_bHorizonYScale_Valid = xIni.GetFloatInCurrentSection("HorizonYScale",pxTemplate->m_fHorizonYScale);
		pxTemplate->m_bHorizonBrightness_Valid = xIni.GetFloatInCurrentSection("HorizonBrightness",pxTemplate->m_fHorizonBrightness);

		pxTemplate->m_bSubdivisionYScale_Valid = xIni.GetFloatInCurrentSection("SubdivisionYScale",pxTemplate->m_fSubdivisionYScale);
		pxTemplate->m_bSunlightFalloffNear_Valid = xIni.GetFloatInCurrentSection("SunlightFalloffNear",pxTemplate->m_fSunlightFalloffNear);
		pxTemplate->m_bSunlightFalloffFar_Valid = xIni.GetFloatInCurrentSection("SunlightFalloffFar",pxTemplate->m_fSunlightFalloffFar);

		pxTemplate->m_bTransparent_Valid = xIni.GetBoolInCurrentSection("Transparent",pxTemplate->m_bTransparent);
		pxTemplate->m_bHorizonReflect_Valid = xIni.GetBoolInCurrentSection("HorizonReflect",pxTemplate->m_bHorizonReflect);
		pxTemplate->m_bSunReflect_Valid = xIni.GetBoolInCurrentSection("SunReflect",pxTemplate->m_bSunReflect);
		pxTemplate->m_bBlendHorizon_Valid = xIni.GetBoolInCurrentSection("BlendHorizon",pxTemplate->m_bBlendHorizon);

		pxTemplate->m_bUScrollSpeed_Valid = xIni.GetFloatInCurrentSection( "UScrollSpeed", pxTemplate->m_fUScrollSpeed );
		pxTemplate->m_bVScrollSpeed_Valid = xIni.GetFloatInCurrentSection( "VScrollSpeed", pxTemplate->m_fVScrollSpeed );
	}
}
