//
// Tools_WaterTemplate.h
//
// Created by Richard Rice - 11/01/2005
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_WATERTEMPLATE_H__
#define __TOOLS_WATERTEMPLATE_H__

#include "Core/Asura_String.h"

struct Tools_WaterTemplate;
template<class T> class List;


class Tools_WaterTemplate_System
{
public :
	//initialise system , loading the templates
	static void Initialise();
	//shutdown and delete the templates
	static void Shutdown();

	//get the named template
	static const Tools_WaterTemplate* GetTemplate(const char* pszName);

	//update the environment water control in Asura_Water_System from its template
	static void UpdateFromTemplate();

	static const List<Tools_WaterTemplate*>* GetTemplateList(){return &s_xTemplateList;}

private :
	static void LoadTemplates();

	static List<Tools_WaterTemplate*> s_xTemplateList;
};


struct Tools_WaterTemplate
{
	Asura_String m_xTemplateName;
		
	//bitfield indicating valid entries :
	u_int m_bWaterCycleScale1_Valid : 1;
	u_int m_bWaterCycleScale2_Valid : 1;
	u_int m_bWaterCycleScale3_Valid : 1;

	u_int m_bXScale1_Valid : 1;
	u_int m_bXScale2_Valid : 1;
	u_int m_bXScale3_Valid : 1;

	u_int m_bZScale1_Valid : 1;
	u_int m_bZScale2_Valid : 1;
	u_int m_bZScale3_Valid : 1;

	u_int m_bYScale1_Valid : 1;
	u_int m_bYScale2_Valid : 1;
	u_int m_bYScale3_Valid : 1;

	u_int m_bFalloff1_Valid : 1;
	u_int m_bFalloff2_Valid : 1;
	u_int m_bFalloff3_Valid : 1;

	u_int m_bThreshold_Valid : 1;

	u_int m_bLightOffset_Valid : 1;
	u_int m_bLightScale_Valid : 1;

	u_int m_bAlphaLevel_Valid : 1;
	u_int m_bReflection_Valid : 1;

	u_int m_bSunTextureOffsetU_Valid : 1;
	u_int m_bSunTextureOffsetV_Valid : 1;
	u_int m_bSunTextureScaleU_Valid : 1;
	u_int m_bSunTextureScaleV_Valid : 1;

	u_int m_bHorizonVScale_Valid : 1;
	u_int m_bHorizonVOffset_Valid : 1;
	u_int m_bHorizonYScale_Valid : 1;
	u_int m_bHorizonBrightness_Valid : 1;

	u_int m_bTransparent_Valid : 1;
	u_int m_bHorizonReflect_Valid : 1;
	u_int m_bSunReflect_Valid : 1;
	u_int m_bBlendHorizon_Valid : 1;

	u_int m_bSubdivisionYScale_Valid : 1;
	u_int m_bSunlightFalloffNear_Valid : 1;
	u_int m_bSunlightFalloffFar_Valid : 1;

    u_int m_bUScrollSpeed_Valid : 1;
    u_int m_bVScrollSpeed_Valid : 1;

	//entries from Asura_Water_System::Water_Controls
	float m_fWaterCycleScale1;
	float m_fWaterCycleScale2;
	float m_fWaterCycleScale3;	// in case we decide we want a third wave

	float m_fXScale1;
	float m_fXScale2;
	float m_fXScale3;

	float m_fZScale1;
	float m_fZScale2;
	float m_fZScale3;

	float m_fYScale1;
	float m_fYScale2;
	float m_fYScale3;

	float m_fFalloff1;
	float m_fFalloff2;
	float m_fFalloff3;

	float m_fThreshold;

	float m_fLightOffset;
	float m_fLightScale;

	float m_fAlphaLevel;
	float m_fReflection;

	float m_fSunTextureOffsetU;
	float m_fSunTextureOffsetV;
	float m_fSunTextureScaleU;
	float m_fSunTextureScaleV;

	bool m_bTransparent;
	bool m_bHorizonReflect;
	bool m_bSunReflect;
	bool m_bBlendHorizon;

	float m_fHorizonVScale;		// controls V texture co-ordinate scale for horizon reflection
	float m_fHorizonVOffset;	// ditto for offset
	float m_fHorizonYScale;		// controls horizon effect normal scaling for flatter waves
	float m_fHorizonBrightness;	// controls how much less bright the horizon reflection is compared to the sun

	float m_fSubdivisionYScale;
	float m_fSunlightFalloffNear;
	float m_fSunlightFalloffFar;

    float m_fUScrollSpeed;
    float m_fVScrollSpeed;

};



#endif //__TOOLS_WATERTEMPLATE_H__
