//
// Tools_Wii_Export.h
//
// Created by KOR - 09 January 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_WII_EXPORT_H__
#define __TOOLS_WII_EXPORT_H__

#include "../Wii/ChunkSystem/Asura_Platform_Chunk_ResourceFile.h"

class Asura_Chunk_Stream;
class Asura_Hierarchy_Skin;
class Asura_Mesh;
class Tools_HierarchyProcessor_MultiMaterial_Vertex;
class Tools_Wii_DisplayListFactory;

typedef u_int (*ENVIRONMENT_ENCODE_COLOUR_FN)(u_int uRGBA);
typedef u_int (*HIERARCHY_SET_COLOUR_FN) (const Tools_HierarchyProcessor_MultiMaterial_Vertex* pxVertex);
typedef u_int (*HIERARCHY_DISPLAYLIST_FN)(u_int uNumStrips, void **pvDisplayLists, u_int *puDisplayListSizes);

struct Tools_Wii_Export_Processed_Module
{
	// Array of pointers to display list data per strip
	char** m_ppcDisplayListData;
	// Array of sizes of display list data per strip
	u_int* m_puDisplayListSizes;
	// Number of tex-coords
	u_int m_uNumTexcoords;
};

class Tools_Wii_Export
{
public:
	enum
	{
		EXPORT_FLAG_USE_DISPLAY_LISTS	= 0x00000001,
		EXPORT_FLAG_NORMALS				= 0x00000002,
		EXPORT_FLAG_VERTEXCOLOURRGBA6	= 0x00000004,
	};

	static void SetHierarchyExportNormals(bool bNormals)			{ SetHierarchyExportFlag(bNormals, ASURA_WII_HIERARCHY_CHUNK_FLAG_NORMALS); }
	static void SetHierarchyExportColours(bool bColours)			{ SetHierarchyExportFlag(bColours, ASURA_WII_HIERARCHY_CHUNK_FLAG_COLOURS); }
	static void SetHierarchyExportUV1(bool bUVs)					{ SetHierarchyExportFlag(bUVs, ASURA_WII_HIERARCHY_CHUNK_FLAG_UV1); }
	static void SetHierarchyExportDisplayList(bool bUseDisplayLists){ SetHierarchyExportFlag(bUseDisplayLists, ASURA_WII_HIERARCHY_CHUNK_FLAG_DISPLAYLIST);	}
	static void SetHierarchyExportAlpha(bool bAlpha)				{ SetHierarchyExportFlag(bAlpha, ASURA_WII_HIERARCHY_CHUNK_FLAG_ALPHA);	}
	static void SetHierarchyExportFlag(bool bFlag, u_int uFlag)		{ SetExportFlag(s_uHierarchyExportFlags, bFlag, uFlag);	}
	static void SetHierarchyExportColourFunc(HIERARCHY_SET_COLOUR_FN ColourFunc = NULL) { s_HierarchyColorFunc = ColourFunc; }
	static void SetHierarchyExportDisplayListGenerator(Tools_Wii_DisplayListFactory *pxFactory = NULL) { s_pxDisplayListGenerator = pxFactory; }

	static bool GetHierarchyExportNormals()		{ return GetHierarchyExportFlag(ASURA_WII_HIERARCHY_CHUNK_FLAG_NORMALS); }
	static bool GetHierarchyExportColours()		{ return GetHierarchyExportFlag(ASURA_WII_HIERARCHY_CHUNK_FLAG_COLOURS); }
	static bool GetHierarchyExportUV1()			{ return GetHierarchyExportFlag(ASURA_WII_HIERARCHY_CHUNK_FLAG_UV1); }
	static bool GetHierarchyExportDisplayList()	{ return GetHierarchyExportFlag(ASURA_WII_HIERARCHY_CHUNK_FLAG_DISPLAYLIST);	}
	static bool GetHierarchyExportAlpha()		{ return GetHierarchyExportFlag(ASURA_WII_HIERARCHY_CHUNK_FLAG_ALPHA); }
	static bool GetHierarchyExportFlag(u_int uFlag)	{ return GetExportFlag(s_uHierarchyExportFlags, uFlag); }
	static u_int GetHierarchyExportFlags() { return s_uHierarchyExportFlags; }

	// To allow projects to over-ride colour encoding.
	// Used by Simpsons for line tagging data.
	static u_int ARGB8888ToRGBA6(u_int uARGB);
	static void SetEnvEncodeFunction(ENVIRONMENT_ENCODE_COLOUR_FN pfnEncodeFunction) { s_pfnEnvironmentEncodeFunction = pfnEncodeFunction; }
	static ENVIRONMENT_ENCODE_COLOUR_FN GetEnvEncodeFunction() { return s_pfnEnvironmentEncodeFunction; }
	static void SetEnvUseNormals(bool bUseNormals)				{ SetExportFlag(s_uEnvironmentExportFlags, bUseNormals, EXPORT_FLAG_NORMALS); }
	static void SetEnvUseDisplayLists(bool bUseDisplayLists)	{ SetExportFlag(s_uEnvironmentExportFlags, bUseDisplayLists, EXPORT_FLAG_USE_DISPLAY_LISTS); }
	static bool GetEnvironmentExportFlag(u_int uFlag) { return GetExportFlag(s_uEnvironmentExportFlags, uFlag); }
	static u_char GetClosestNormalIndex(const Asura_Vector_3& xNormal);

	// pxSkin not a const pointer as it may calculate the normals at this point.
	static bool ExportSmoothSkin(HANDLE hOutput, class Asura_Hierarchy_Skin* pxSkin);
	static bool ExportHierarchy(HANDLE hOutput, class Asura_Hierarchy_Skin* pxSkin);
	static void ExportEnvironment(HANDLE hOutput, bool bSubdivide = false);
	static bool ExportShape(HANDLE hOutput, Asura_Mesh* pxMesh, Asura_Hash_ID uHashID, int iMaterialIndex);

private:
	static void SetExportFlag(u_int& uFlags, bool bSet, u_int uFlag)
	{
		if(bSet)
		{
			uFlags |= uFlag;
		}
		else
		{
			uFlags &= ~uFlag;
		}
	}
	static bool GetExportFlag(u_int& uFlags, u_int uFlag)
	{
		return (uFlags & uFlag) == static_cast<u_int>(uFlag);
	}
	static void CalculateEnvironmentDisplayLists(Tools_Wii_Export_Processed_Module* pxProcessedModules, u_int &uTotalDisplayListData, bool bIncludeNormals);
	static void WriteBoundingBox(Asura_Chunk_Stream& xStream, const Asura_Vector_3 *pPositions, u_int uNumPositions);

	static HIERARCHY_SET_COLOUR_FN s_HierarchyColorFunc;
	static u_int s_uHierarchyExportFlags;
	static Tools_Wii_DisplayListFactory *s_pxDisplayListGenerator;

	static ENVIRONMENT_ENCODE_COLOUR_FN s_pfnEnvironmentEncodeFunction;
	static u_int s_uEnvironmentExportFlags;
	// Normal table
	static unsigned int s_uNormalTableNumNormals;
	static Asura_Vector_3 s_axNormalTable[252];
};

#endif//__TOOLS_WII_EXPORT_H__