//
// Tools_Wii_Textures.cpp
//
// Created by KOR - 18/09/2003 16:01:47
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//


// Asura includes
#include "Core/Asura.h"

// This file's header
#include "Wii/Tools_Wii_Textures.h"

#ifdef ASURA_MAIN

#include "Core/Asura_NameTree.h"
#include "Core/Asura_TextureFlags.h"
#include "Core/Asura_MaterialFlags.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Bmp.h"
#include "SpecialFX/Asura_Skybox.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_Chunk_Macros.h"
#endif
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "../Wii/ChunkSystem/Asura_Platform_Chunk_ResourceFile.h"

// Tools includes
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_TextureConversionData.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Wii/TPLOptions.h"
#include "Wii/TPLWriter.h"

// Static members
Asura_NameTree			Tools_Wii_Textures::s_xSizeBufferTree;
Asura_RGB_Buffer		Tools_Wii_Textures::s_axRGBBuffers[ASURA_MAX_TEXTURE_SIZES];
bool					Tools_Wii_Textures::s_bInitialised = false;
Asura_String			Tools_Wii_Textures::s_xExportName;
Asura_Wii_Alpha_Mapping	Tools_Wii_Textures::s_axMappings[ASURA_MAX_WII_EXPORT_TEXTURES];
u_int					Tools_Wii_Textures::s_uNumMappings;
bool					Tools_Wii_Textures::s_bMergeAlphaTextures = false;

// For saving windows bitmaps
typedef VOID (*PutWord_F)(WORD, FILE *);
typedef VOID (*PutDword_F)(DWORD, FILE *);
extern VOID PutByte(BYTE v, FILE *fp);
extern VOID PutLittleWord(WORD v, FILE *fp);
extern VOID PutLittleDword(DWORD v, FILE *fp);

static const char szExportDir[] = "\\graphics\\Wii_Alpha_Textures";

void Tools_Wii_Textures::BeginExport(const char* szExportName)
{
	Asura_Assert(!s_bInitialised, "Already begun export for this file");
	s_bInitialised = true;
	Asura_String xTempStr = szExportName;
	xTempStr.SplitFilename(0, &s_xExportName, 0);
	s_xSizeBufferTree.Reset();
	for(u_int u = 0; u < ASURA_MAX_TEXTURE_SIZES; ++u)
	{
		s_axRGBBuffers[u].eCurrentChannel = Asura_Channel_Red;
		s_axRGBBuffers[u].pucBuffer = 0;
		s_axRGBBuffers[u].uExportNumber = 0;
	}
	s_uNumMappings = 0;
}

void Tools_Wii_Textures::EndExport(HANDLE hSaveHandle)
{
	if(!s_uNumMappings)
	{
		//Skip this for files that haven't added any textures. RAR 12/06/2007
		s_bInitialised = false;
		return;
	}
	
	char szBMPname[256];
	char szTPLname[256];
	for(u_int u = 0; u < ASURA_MAX_TEXTURE_SIZES; ++u)
	{
		if(s_axRGBBuffers[u].eCurrentChannel != Asura_Channel_Red)
		{
			char szBMPname[256];
			GetAlphaTextureBMPFilename(s_axRGBBuffers + u, szBMPname);
			SaveWindowsBitmap(s_axRGBBuffers + u, szBMPname);
		}
	}
	// Now convert all bitmaps to TPLs and add them to the export file
	for(u_int u = 0; u < ASURA_MAX_TEXTURE_SIZES; ++u)
	{
		u_int uMaxFileNo = s_axRGBBuffers[u].uExportNumber;
		TPLOptions xOpts;
		memset(&xOpts, 0, sizeof(xOpts));
		xOpts.m_bUseS3TC = 1;
		xOpts.m_bUseWrap = 1;
		xOpts.m_uAlphaBits = 0;
		xOpts.m_uBaseLOD = 0;
		xOpts.m_uMinLOD = s_axRGBBuffers[u].uMipsToDrop;
		xOpts.m_uMaxLOD = s_axRGBBuffers[u].uMipsToDrop + 1;
		for(s_axRGBBuffers[u].uExportNumber = 0; s_axRGBBuffers[u].uExportNumber < uMaxFileNo; ++s_axRGBBuffers[u].uExportNumber)
		{
			GetAlphaTextureBMPFilename(s_axRGBBuffers + u, szBMPname);
			GetAlphaTextureTPLFilename(s_axRGBBuffers + u, szTPLname);
			Asura_String in = szBMPname;
			Asura_String out = szTPLname;
			Tools_Wii_Textures::ConvertTexture(in, out, xOpts);
			// And export the resourcefile chunk with the TPL in

			{
				HANDLE file = CreateFile(szTPLname,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
				if(file==INVALID_HANDLE_VALUE)
				{
					Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Failed to open %s",szTPLname);
					return;
				}
				//allocate a buffer , and read in the file
				int trueFileSize = GetFileSize(file,0);
				int fileSize = ((trueFileSize+3)/4)*4;
				char* buffer = new char[fileSize];
				unsigned long bytesRead;
				ReadFile(file,buffer,trueFileSize,&bytesRead,0);
				CloseHandle(file);
				Asura_String xBaseFilenameForResource = szBMPname;

				const int totalSize = sizeof(Asura_Chunk_ResourceFile) + (xBaseFilenameForResource.GetSizeInInts()*4) + (fileSize);

				unsigned long bytesWritten;
				//create the chunk header
				Asura_Chunk_ResourceFile chunk;
				chunk.Create(totalSize, 0, ASURA_RESOURCEFILE_TYPE_TEXTURE, 0, fileSize);
				
				WriteFile(hSaveHandle,&chunk,sizeof(chunk),&bytesWritten,0);

				// add archivename (padded to int)
				WriteFile(hSaveHandle,xBaseFilenameForResource.GetString(),xBaseFilenameForResource.GetSizeInInts()*4,&bytesWritten,0);

				// add contents of file
				WriteFile(hSaveHandle,buffer,fileSize,&bytesWritten,0);

				delete [] buffer;
			}
		}
	}
	Asura_Chunk_ResourceFile xResourceFile;
	Asura_String xResourceFilename("alphamaps");
	int iFileSize;
	int iChunkSize;
	int iTotalBytesWritten = 0;
	unsigned long uBytesWritten;
	// Now we somehow need to save out the mapping from original textures to alpha textures
	// Calculate "file" size
	iFileSize = sizeof(int);
	for(u_int u = 0; u < s_uNumMappings; u++)
	{
		iFileSize += 4 * (s_axMappings[u].xSourceTexture.GetSizeInInts() + s_axMappings[u].xAlphaTexture.GetSizeInInts());
	}
	iFileSize += s_uNumMappings * sizeof(Asura_RGB_Channel);
	// Calculate chunk size
	iChunkSize = sizeof(Asura_Chunk_ResourceFile) + (xResourceFilename.GetSizeInInts()*4) + iFileSize;
	xResourceFile.Create(iChunkSize, 0, ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC, ASURA_RESOURCEFILE_TYPE_WII_ALPHAMAPS, iFileSize);
	WriteFile(hSaveHandle, &xResourceFile, sizeof(Asura_Chunk_ResourceFile), &uBytesWritten, 0);
	iTotalBytesWritten += uBytesWritten;
	WriteFile(hSaveHandle, xResourceFilename.GetString(), xResourceFilename.GetSizeInInts()*4, &uBytesWritten, 0);
	iTotalBytesWritten += uBytesWritten;
	ASURA_CHUNK_VERIFY(s_uNumMappings, int);
	WriteFile(hSaveHandle, &s_uNumMappings, 4, &uBytesWritten, 0);
	iTotalBytesWritten += uBytesWritten;
	ASURA_CHUNK_VERIFY(s_uNumMappings, int);

	for(u_int u = 0; u < s_uNumMappings; u++)
	{
		WriteFile(hSaveHandle, s_axMappings[u].xSourceTexture.GetString(), s_axMappings[u].xSourceTexture.GetSizeInInts()*4, &uBytesWritten, 0);
		iTotalBytesWritten += uBytesWritten;
		WriteFile(hSaveHandle, s_axMappings[u].xAlphaTexture.GetString(), s_axMappings[u].xAlphaTexture.GetSizeInInts()*4, &uBytesWritten, 0);
		iTotalBytesWritten += uBytesWritten;
		ASURA_CHUNK_VERIFY(s_axMappings[u].eChannel, Asura_RGB_Channel);
		WriteFile(hSaveHandle, &s_axMappings[u].eChannel, 4, &uBytesWritten, 0);
		iTotalBytesWritten += uBytesWritten;
	}
	Asura_Assert(iTotalBytesWritten == iChunkSize, "Size mismatch.");
	s_bInitialised = false;
}

void Tools_Wii_Textures::GetAlphaTextureBMPFilename(Asura_RGB_Buffer* pxRGBBuffer, char szFilename[])
{
	sprintf(szFilename, "%s\\%s_%ux%ud%u_%u.bmp", szExportDir, s_xExportName.GetString(), pxRGBBuffer->uWidth, pxRGBBuffer->uHeight, pxRGBBuffer->uMipsToDrop, pxRGBBuffer->uExportNumber);
}

void Tools_Wii_Textures::GetAlphaTextureTPLFilename(Asura_RGB_Buffer* pxRGBBuffer, char szFilename[])
{
	sprintf(szFilename, "%s\\%s_%ux%ud%u_%u.tpl", szExportDir, s_xExportName.GetString(), pxRGBBuffer->uWidth, pxRGBBuffer->uHeight, pxRGBBuffer->uMipsToDrop, pxRGBBuffer->uExportNumber);
}

bool Tools_Wii_Textures::ExportTexture(const char* inputName,const char* outputName, const TextureConversionData& xConversionData, u_int uMinTexRes, u_int uMaxMipDrops)
{
	Asura_Assert(s_bInitialised, "Gamecube texture exporter not initialised");
	if(!s_bInitialised)
	{
		return false;
	}
	if(xConversionData.m_uMaximumTextureSize)
	{
		uMinTexRes = Asura_Maths::Min(uMinTexRes, xConversionData.m_uMaximumTextureSize);
	}

	Asura_String in = inputName;
	Asura_String out = outputName;
	Asura_String filename;
	TPLOptions xOpts;
	bool bSuccess = false;

	// Use DirectX to load the texture and check its dimensions
	Direct3D_Texture*	dxTexture = 0;
	Direct3D_Surface*	dxSurface = 0;
	D3DSURFACE_DESC		dxSurfaceDesc;
	D3DXIMAGE_INFO		imageInfo;
	D3DLOCKED_RECT		rect;
	unsigned char*		pucBufferPtr;
	int					iCurrentByte;

	//attempt to load in the texture to an 8888 format
	HRESULT hres = D3DXCreateTextureFromFileExA(
			Asura_Direct3D::D3DDevice,
			inputName,
			D3DX_DEFAULT,//width
			D3DX_DEFAULT,//height
			1,//MipLevels
			0,//Usage
			D3DFMT_A8R8G8B8,//format
			D3DPOOL_SYSTEMMEM,//Pool
			D3DX_FILTER_NONE,//Filter
			D3DX_FILTER_NONE,//MipFilter
			0,//ColorKey
			&imageInfo,//pSourceInfo
			0,//pPalette
			&dxTexture
		);	

	if(FAILED(hres))
	{
		Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Error %x loading %s",hres,inputName);
		return false;
	}

	//get the surface
	hres = dxTexture->GetSurfaceLevel(0,&dxSurface);
	if(FAILED(hres))
	{
		Asura_Assert(0,"Failed to get texture surface");
		goto Clearup;
	}

	//get the surface description
	hres = dxSurface->GetDesc(&dxSurfaceDesc);
	if(FAILED(hres))
	{
		Asura_Assert(0,"Failed to get texture surface description");
		goto Clearup;
	}

	//check its in the format requested
	if(dxSurfaceDesc.Format != D3DFMT_A8R8G8B8)
	{
		Asura_Assert(0,"Failed to load texture as D3DFMT_A8R8G8B8");
		goto Clearup;
	}

	//warn if width/height have changed
	if(imageInfo.Width!=dxSurfaceDesc.Width || imageInfo.Height!=dxSurfaceDesc.Height)
	{
		Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Invalid dimensions (non power of 2?): %dx%d",imageInfo.Width,imageInfo.Height);
		goto Clearup;
	}

	((Asura_String) inputName).SplitFilename(0,&filename,0);
	bool isSkybox;
	isSkybox = false;
	for(int i=0;i<7;i++)
	{
		Asura_String fullTextureName = Asura_Skybox::GetTexture(i);
		if(fullTextureName.IsBlank()) continue;
		Asura_String splitTextureName;
		fullTextureName.SplitFilename(0,&splitTextureName,0);
		if(splitTextureName == filename)
		{
			isSkybox = true;
			break;
		}
	}
	// Zero options in case we miss any members
	memset(&xOpts, 0, sizeof(TPLOptions));
	if(isSkybox)
	{
		xOpts.m_uMinLOD = 0;
		xOpts.m_uMaxLOD = 0;
		xOpts.m_uBaseLOD = 0;
		xOpts.m_bUseWrap = 0;
		xOpts.m_uAlphaBits = 0;
		xOpts.m_uMinAlphaLOD = 0;
	}
	else
	{
		xOpts.m_uMinLOD = 0;
		u_int uExportRes = dxSurfaceDesc.Width;
		while(uExportRes > uMinTexRes)
		{
			xOpts.m_uMinLOD++;
			uExportRes /= 2;
		}
		xOpts.m_uMinLOD = Asura_Maths::Min(xOpts.m_uMinLOD, uMaxMipDrops);
		if(xConversionData.m_iTextureFlags & (AsuraTextureFlag_NoMipMaps|AsuraTextureFlag_WIIOnly_NoMipMaps))
		{
			xOpts.m_uMaxLOD = xOpts.m_uMinLOD;
		}
		else
		{
			xOpts.m_uMaxLOD = Asura_Maths::ClampToRange(xOpts.m_uMinLOD + 8, 0u, 10u);
		}
		xOpts.m_uBaseLOD = 0;
		xOpts.m_bUseWrap = 1;
		xOpts.m_uMinAlphaLOD = 0;

		const u_int uMaterialFlagsRequiringAlpha = (AsuraMaterialFlag_SelfIlluminate |
													AsuraMaterialFlag_Transparent |
													AsuraMaterialFlag_Reflective |
													AsuraMaterialFlag_Specular_Soft |
													AsuraMaterialFlag_Specular_Sharp );
		if(xConversionData.m_uMaterialFlags & uMaterialFlagsRequiringAlpha || xConversionData.m_iTextureFlags & AsuraTextureFlag_8BitAlpha)
		{
			xOpts.m_uAlphaBits = 4;
			// KOR - 16/04/2008 - 1024 textures are too big with 4-bit alpha.
			// Make the alpha layer half the resolution of the colour channel.
			if(uExportRes > 512)
			{
				xOpts.m_uMinAlphaLOD = 1;
			}
		}
		else
		{ 
			xOpts.m_uAlphaBits = 0;
		}
	}
	if(xOpts.m_uAlphaBits > 1)
	{
		if (s_bMergeAlphaTextures)
		{
			Asura_Assert(s_uNumMappings < ASURA_MAX_WII_EXPORT_TEXTURES, "Too many alpha textures to export");
			if(s_uNumMappings == ASURA_MAX_WII_EXPORT_TEXTURES)
			{
				return false;
			}
			// Right, we need to create our separate alpha texture
			char szSizeName[32];
			sprintf(szSizeName, "%ux%ud%u", dxSurfaceDesc.Width, dxSurfaceDesc.Height, xOpts.m_uMinLOD);
			int iIndex;
			s_xSizeBufferTree.FindNodeIndexByName(szSizeName, true, iIndex);
			if(s_axRGBBuffers[iIndex].eCurrentChannel == Asura_Channel_Red)
			{
				// Start new buffer
				Asura_Assert(!s_axRGBBuffers[iIndex].pucBuffer, "Buffer already exists");
				s_axRGBBuffers[iIndex].uHeight = dxSurfaceDesc.Height;
				s_axRGBBuffers[iIndex].uWidth = dxSurfaceDesc.Width;
				s_axRGBBuffers[iIndex].uMipsToDrop = xOpts.m_uMinLOD;
				s_axRGBBuffers[iIndex].pucBuffer = new unsigned char[dxSurfaceDesc.Height * dxSurfaceDesc.Width * 3];
				memset(s_axRGBBuffers[iIndex].pucBuffer, 0, dxSurfaceDesc.Height * dxSurfaceDesc.Width * 3);
			}
			else
			{
				Asura_Assert(s_axRGBBuffers[iIndex].uHeight == dxSurfaceDesc.Height, "Size mismatch.");
				Asura_Assert(s_axRGBBuffers[iIndex].uWidth == dxSurfaceDesc.Width, "Size mismatch.");
				Asura_Assert(s_axRGBBuffers[iIndex].pucBuffer, "No buffer");
			}

			//hres = dxTexture->LockRect(xOpts.uMinLOD, &rect,NULL,D3DLOCK_READONLY);
			hres = dxTexture->LockRect(0, &rect,NULL,D3DLOCK_READONLY);
			if(FAILED(hres))
			{
				Asura_Assert(0,"Failed to lock texture for reading");
			}
			else
			{	
				pucBufferPtr = (unsigned char *)rect.pBits+rect.Pitch*(dxSurfaceDesc.Height - 1);

				// Set correct offset for desired channel
				iCurrentByte = s_axRGBBuffers[iIndex].eCurrentChannel;
				bool bWhiteTexture;
				bWhiteTexture = true;
				for (int i = dxSurfaceDesc.Height - 1; i >= 0; --i)
				{
					for (unsigned int j = 0; j < dxSurfaceDesc.Width; ++j)
					{
						unsigned int colour = *(unsigned int*)&pucBufferPtr[j*4];
						s_axRGBBuffers[iIndex].pucBuffer[iCurrentByte] = (unsigned char)((colour&0xff000000)>>24);
						if((unsigned char)((colour&0xff000000)>>24) != 0xff)
						{
							bWhiteTexture = false;
						}
						iCurrentByte += 3;
					}
					// pad to 4 byte boundary
					pucBufferPtr -= rect.Pitch;
				}

				dxTexture->UnlockRect(0);

				// No point saving an entirely white alpha channel.
				if(bWhiteTexture)
				{
					// Remove buffer if red channel
					if(s_axRGBBuffers[iIndex].eCurrentChannel == Asura_Channel_Red)
					{
						delete [] s_axRGBBuffers[iIndex].pucBuffer;
						s_axRGBBuffers[iIndex].pucBuffer = 0;
					}
				}
				else
				{
					char szBMPname[256];
					GetAlphaTextureBMPFilename(s_axRGBBuffers + iIndex, szBMPname);
					s_axMappings[s_uNumMappings].xSourceTexture = strstr(inputName, "graphics\\");
					if(s_axMappings[s_uNumMappings].xSourceTexture[0] != '\\')
					{
						// Prepend slash
						Asura_String xTemp = "\\";
						xTemp += s_axMappings[s_uNumMappings].xSourceTexture;
						s_axMappings[s_uNumMappings].xSourceTexture = xTemp;
					}
					s_axMappings[s_uNumMappings].xAlphaTexture = szBMPname;
					if(s_axMappings[s_uNumMappings].xAlphaTexture[0] != '\\')
					{
						// Prepend slash
						Asura_String xTemp = "\\";
						xTemp += s_axMappings[s_uNumMappings].xAlphaTexture;
						s_axMappings[s_uNumMappings].xAlphaTexture = xTemp;
					}
					s_axMappings[s_uNumMappings].eChannel = s_axRGBBuffers[iIndex].eCurrentChannel;
					s_uNumMappings++;
					if(s_axRGBBuffers[iIndex].eCurrentChannel == Asura_Channel_Blue)
					{
						// We're done.
						SaveWindowsBitmap(s_axRGBBuffers + iIndex, szBMPname);
					}
					else
					{
						s_axRGBBuffers[iIndex].eCurrentChannel=(Asura_RGB_Channel)(s_axRGBBuffers[iIndex].eCurrentChannel+1);
					}
				}
			}
			// Now we have separate alpha texture, don't put alpha in the current export.
			xOpts.m_uAlphaBits = 0;
		} 
	}
	TPLPaletteOptions xPaletteOptions;
	if(xConversionData.m_bUseFixedPalette)
	{
		TextureConversionDataBase::GetFixedPalette(xPaletteOptions.m_uNumPaletteEntries,xPaletteOptions.m_puPalette);
		xOpts.m_pxPaletteOptions = &xPaletteOptions;
		if(xConversionData.m_bWritePaletteAsRedScaleIndices)
		{
			// TODO
		}
	}
	else
	{
		xOpts.m_bUseS3TC = 1;
	}
	
	bSuccess = ConvertTexture(in, out, xOpts);

Clearup :
		
	if(dxSurface) dxSurface->Release();
	if(dxTexture)
	{
		dxTexture->Release();
	}

	return bSuccess;
}

#endif

//#define USE_DEBUG_TPLWRITER_DLL (1)

bool Tools_Wii_Textures::ConvertTexture(Asura_String xIn, Asura_String xOut, TPLOptions xOpts)
{
	Tools_FileSystem::EnsureDirectoryExists(xOut.GetString());
#ifdef USE_DEBUG_TPLWRITER_DLL
	HINSTANCE xDLLHandle = LoadLibrary("TPLWriterDLL_Debug.dll");
#else
	HINSTANCE xDLLHandle = LoadLibrary("TPLWriterDLL.dll");
#endif
	if(!xDLLHandle)
	{
		Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Could not open TPLWriterDLL.dll");
		return false;
	}
	bool bSuccess = false;
	CREATETPL_PTR pfCreateTPL = (CREATETPL_PTR)GetProcAddress(xDLLHandle, "CreateTPL");
	GETERRORMSG_PTR pfGetErrorMsg = (GETERRORMSG_PTR)GetProcAddress(xDLLHandle, "GetErrorMessage");
	if(pfCreateTPL && pfGetErrorMsg)
	{
		if((pfCreateTPL)(xIn.GetString(), xOut.GetString(), &xOpts))
		{
			const char* szError = (pfGetErrorMsg)();
			if(szError[0])
			{
			    Tools_Level_Export::AddMessage(Tools_Level_Export::MT_WARNING, szError);
			}
			bSuccess = true;
		}
		else
		{
			const char* szError = (pfGetErrorMsg)();
		    Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR, szError);
		}
	}
	else
	{
	    Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Failed to get CreateTPL function from TPLWriterDLL.dll");
	}

	return bSuccess;
}

#ifdef ASURA_MAIN

bool Tools_Wii_Textures::SaveWindowsBitmap(Asura_RGB_Buffer* pxRGBBuffer, const char* szFilename)
{
	Tools_FileSystem::EnsureDirectoryExists(szFilename);
	FILE * fp = fopen(szFilename,"wb");
	Asura_Assert(fp, "Could not open file for writing.");
	if (!fp)
	{
		return false;
	}
	BMPHEADER2 h;

	// fill out header
	
	h.Header.Type      = 'B'+'M'*256;
	h.Header.Reserved1 = 0;
	h.Header.Reserved2 = 0;
	h.Header.Offset    = 14+40+0;

	/*
	** The type of information found in a BMP structure is indicated by
	** the Size (Information Headere Size) field with a non-zero value.
	*/
	h.PmInfo.Size   = 0;
	h.Pm2Info.Size  = 0;

	h.WinInfo.Size          = 40;
	h.WinInfo.Width         = pxRGBBuffer->uWidth;
	h.WinInfo.Height        = pxRGBBuffer->uHeight;
	h.WinInfo.Planes        = 1;
	h.WinInfo.BitCount      = 24;
	h.WinInfo.Compression   = 0;
	h.WinInfo.SizeImage     = h.WinInfo.Width*h.WinInfo.Height*3;
	h.WinInfo.XPelsPerMeter = h.WinInfo.Width;
	h.WinInfo.YPelsPerMeter = h.WinInfo.Height;
	h.WinInfo.ClrUsed       = 0;
	h.WinInfo.ClrImportant  = 0;

	h.Header.FileSize  = h.WinInfo.SizeImage + h.Header.Offset + 8;

	// write header

	PutWord_F PutWord = PutLittleWord;
	PutDword_F PutDword = PutLittleDword;

    PutWord(h.Header.Type, fp);
    PutDword(h.Header.FileSize, fp);
    PutWord(h.Header.Reserved1, fp);
    PutWord(h.Header.Reserved2, fp);
    PutDword(h.Header.Offset, fp);

	PutDword(h.WinInfo.Size, fp);

    PutDword(h.WinInfo.Width, fp);
    PutDword(h.WinInfo.Height, fp);
    PutWord(h.WinInfo.Planes, fp);
    PutWord(h.WinInfo.BitCount, fp);
    PutDword(h.WinInfo.Compression, fp);
    PutDword(h.WinInfo.SizeImage, fp);
    PutDword(h.WinInfo.XPelsPerMeter, fp);
    PutDword(h.WinInfo.YPelsPerMeter, fp);
    PutDword(h.WinInfo.ClrUsed, fp);
    PutDword(h.WinInfo.ClrImportant, fp);

	// write 24 bit image
	fwrite( pxRGBBuffer->pucBuffer, 1, h.WinInfo.SizeImage, fp );

	fclose(fp);

	// Now tidy up the buffer
	pxRGBBuffer->eCurrentChannel = Asura_Channel_Red;
	delete [] pxRGBBuffer->pucBuffer;
	pxRGBBuffer->pucBuffer = 0;
	pxRGBBuffer->uExportNumber++;
	return true;
}

#endif
