//
// Tools_Zone_Editor.h
//
// Created by Richard Rice - 25/01/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifndef __TOOLS_ZONE_EDITOR_H__
#define __TOOLS_ZONE_EDITOR_H__

#include "Core/Asura_Maths.h"


class Asura_Zone;
class Asura_Zone_Controller;
class Tools_Zone_VolumeEdit_Controller;

class Tools_Zone_Editor
{
public :
	Tools_Zone_Editor(Asura_Zone_Controller* pxZoneController);
	virtual ~Tools_Zone_Editor();

	///per frame update of editing
	virtual void Update();

	void RenderWorld();
	void RenderUI();

	void StartEditing();
	void EndEditing();


protected :
	void CreateOverlay();
	void DeleteOverlay();
	
	//get heading for main overlay
	virtual const char* GetTitle(){return "Zone Editing";}

	//add main zone editing controls
	virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);

	//delete overlays/volumes for previous selection , and create for new one
	void HandleSelectionChange();

	//create overlay for selected zone
	void CreateZoneOverlay();
	//delete same
	void DeleteZoneOverlay();
	//add controls to said overlay
	virtual void AddZoneOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset){};
	
	//set the appropriate file as modified
	virtual void SetModified(){}

    // specifies whether the system wants default zones to be used
    // currently just permits/restricts creation of the default zone
    virtual bool AllowDefaultZones() { return true; }

	//get the currently selected zone
	Asura_Zone* GetSelectedZone(){return m_pxSelectedZone;}

	void TransferModeData(int iMode,const char* szFilename);


private :
	void BuildZoneNameArray();

	static void CreateZone(int iThis){((Tools_Zone_Editor*)iThis)->CreateZone(false);}
	static void CreateDefaultZone(int iThis) {((Tools_Zone_Editor*)iThis)->CreateZone(true);}
	void CreateZone(bool bDefaultZone);

	static void DeleteZone(int iThis){((Tools_Zone_Editor*)iThis)->DeleteZone();}
	void DeleteZone();

	static void RenameZone(int iThis){((Tools_Zone_Editor*)iThis)->RenameZone();}
	void RenameZone();

	static bool IsZoneSelected();
	static bool DoesntHaveDefaultZone();

	void SortZones();
	static int SortZoneCallback(const void*,const void*);

	//the zone controller to be edited
	Asura_Zone_Controller* m_pxZoneController;

	int m_iSelectedZone;
	Asura_Zone* m_pxSelectedZone;
	bool m_bSelectedZoneChanged;

	//volume editor for current selection
	Tools_Zone_VolumeEdit_Controller* m_pxVolumeEdit;

	int m_iOverlayID;
	Asura_Vector_2 m_xOverlayPos;
	bool m_bOverlayMinimised;

	int m_iZoneOverlayID;
	Asura_Vector_2 m_xZoneOverlayPos;
	bool m_bZoneOverlayMinimised;

	int m_iNumZoneNames;
	const char** m_ppszZoneNames;

	bool m_bShowVolumes;

protected :
	bool m_bChanged;
	bool m_bActive;//currently editing


	static Tools_Zone_Editor* s_pxThis;
};




#endif //__TOOLS_ZONE_EDITOR_H__
