//
// Tools_Zone_VolumeEdit_Controller.cpp
//
// Created by Richard Rice - 25/01/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Zone/Tools_Zone_VolumeEdit_Controller.h"

#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"


Asura_Vector_2 Tools_Zone_VolumeEdit_Controller::s_xOverlayPosition;

Tools_Zone_VolumeEdit_Controller::Tools_Zone_VolumeEdit_Controller(Asura_Zone* pxZone,bool* pbChanged)
:Tools_VolumeEdit_Controller(pbChanged)
{
	m_pxZone = pxZone;
	m_pbChanged = pbChanged;

	static bool s_bOverlayPositionInit = false;
	if(!s_bOverlayPositionInit)
	{
		s_xOverlayPosition.x = (Asura_Viewport::GetWidth()/ASURA_TEXT_OUTPUT_FONT_WIDTH) - 15.0f;
		s_xOverlayPosition.y = 8.0f;

		s_bOverlayPositionInit = true;
	}
}

void Tools_Zone_VolumeEdit_Controller::StartVolumeEditing()
{
	Tools_VolumeEdit_Controller::StartVolumeEditing();
	
	//create a volume if none exists yet
	if(!GetNumberOfVolumes()) CreateVolume();
}



void Tools_Zone_VolumeEdit_Controller::CreateVolumeMain()
{
	//allocate a new array of volumes , and copy the old ones across
	Asura_OrientedVolume* pxVolumes = new Asura_OrientedVolume[m_pxZone->m_iNumberOfVolumes+1];

	for(int i=0;i<m_pxZone->m_iNumberOfVolumes;i++)
	{
		pxVolumes[i] = m_pxZone->m_pxVolumes[i];
	}

	//initialise the new volume
	pxVolumes[m_pxZone->m_iNumberOfVolumes].m_xBox.Reset();
	pxVolumes[m_pxZone->m_iNumberOfVolumes].m_xBox.Set(*Asura_Camera::GetPosition());
	pxVolumes[m_pxZone->m_iNumberOfVolumes].m_xBox.Grow(1.5f);
	pxVolumes[m_pxZone->m_iNumberOfVolumes].m_xOrient = Asura_Maths::IdentityQuat;

	//replace the old array
	delete [] m_pxZone->m_pxVolumes;
	m_pxZone->m_pxVolumes = pxVolumes;
	m_pxZone->m_iNumberOfVolumes++;

}

void Tools_Zone_VolumeEdit_Controller::DeleteVolumeMain(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<m_pxZone->m_iNumberOfVolumes,"");

	//shuffle down any volumes after the one being deleted
	for(int i=iIndex;i<m_pxZone->m_iNumberOfVolumes-1;i++)
	{
		m_pxZone->m_pxVolumes[i] = m_pxZone->m_pxVolumes[i+1];
	}
	m_pxZone->m_iNumberOfVolumes--;
}

Asura_Vector_3 Tools_Zone_VolumeEdit_Controller::GetDefaultPosition()
{
	return *Asura_Camera::GetPosition();
}
