#!/bin/make

.SUFFIXES:
.SUFFIXES: .c .cpp .cxx .cc .s .asm

DEVKIT		:=/usr/local/psp/devkit
PSPGCC		:=/usr/local/psp/gcc-1.9.2

################################################################

PROJ		:=UnitTest
ARCH		:=psp
COMP		:=GCC
BUILD		?=rel
SFX		:=.o

# The target directory for the executable must be in
# MS Windows, not Cygwin, form.
EXED		:=z:/MiamiVice/PSP_Release
EXENAME		:=$(ARCH)_$(COMP)_$(BUILD)_$(PROJ)

OBJD		:=$(ARCH)_$(COMP)_$(BUILD)

LIBD		:=$(DEVKIT)/lib

################################################################

ASURA		:=../../../../Asura
ASURAI		:=$(ASURA)/Independent
ASURAA		:=$(ASURA)/PSP

BRANCH		:=../../Asura_Branch
BRANCHI		:=$(BRANCH)/Independent
BRANCHA		:=$(BRANCH)/psp

GAME		:=../..
GAMEI		:=$(ASURAI)/Independent
GAMEA		:=$(GAME)/Psp

INCP		:=\
		  -I"$(GAMEA)"\
		  -I"$(GAMEI)"\
		  -I"$(GAME)/UnitTesting"\
		  -I"$(ASURAA)"\
		  -I"$(ASURAI)"\
		  -I"$(DEVKIT)/include"\
		  -I.
#		  -I"$(BRANCHA)"\
#		  -I"$(BRANCHI)"\

LIBP		:=-L"$(LIBD)"
LIBS		:="-lstdc++" \
		  "-lsupc++" \
		  "-lwave"\
		  "-lsas_weak"\
		  "-lgu"\
		  "-lgum"\
		  "-lkernel"\
		  "$(LIBD)/pspnet_stub_weak.a"\
		  "$(LIBD)/pspnet_adhoc_stub_weak.a"\
		  "$(LIBD)/pspnet_adhocctl_stub_weak.a"\
		  "$(LIBD)/pspnet_adhoc_matching_stub_weak.a"\
		  "$(LIBD)/utility_stub.a"\
		  "$(LIBD)/wlan_stub.a"\
		  "$(LIBD)/mpeg_stub_weak.a"\
		  "$(LIBD)/mediaman_stub.a"\
		  "$(LIBD)/libatrac3plus_stub_weak.a"\
		  "$(LIBD)/power_stub.a"\
		  "$(LIBD)/rtc_stub.a"\
		  "$(LIBD)/ctrl_stub.a"\
		  "$(LIBD)/display_stub.a"\
		  "$(LIBD)/ge_user_stub.a"\
		  "-lm"

################################################################

CCDEFS_any	:=-DPG_DEBUG \
		  -DSN_TARGET_PSP_PRX \
		  -UASURA_SUPPORT_NETMSG_SYSTEM \
		  -UASURA_SUPPORT_EXTERNAL_NETWORK \
		  -DASURA_PSP_MODULES_NOT_PRELOADED \
		  -DASURA_VECTOR_4_IN_PLATFORM_CODE
CCDEFS_rel	:=-DNDEBUG -U_DEBUG
CCDEFS_dbg	:=-UNDEBUG -D_DEBUG -UASURA_MEMORYMANAGEMENT_ENABLED

#CC_SNC		:=pspsnc
#CCOPTS_SNC	:=-Xwchart=short -Xvfpuscalar=8 -Xvfpureserve=64:127
#CCOPTS_SNC_rel	:=-O3 -Xxopt=4 
#CCOPTS_SNC_dbg	:=-O0

#CXX_SNC	:=pspsnc
#CXX_SNC	:=

CC_GCC		:=psp-gcc
CCOPTS_GCC	:=-MMD -w -fshort-wchar \
		  -msingle-float -fsingle-precision-constant \
		  -menable-vfpu=m0-m7
CCOPTS_GCC_rel	:=-O3 \
		  -ffast-math -mfast-vfpu-math \
		  -fno-align-functions -fno-align-jumps -fno-align-loops
CCOPTS_GCC_dbg	:=-O1

CXX_GCC		:=psp-g++
CXXOPTS_GCC	:=-fno-common -fno-exceptions -fno-rtti

LD_GCC		:=psp-g++
LDOPTS_GCC	:=-gdwarf-1 -G4 \
		  -fno-common -fno-exceptions -fno-rtti
#LDOPTS_GCC_rel	:=-Wl,-M,-t
#LDOPTS_GCC_dbg	:=-Wl,-M,-t

################################################################

CC		:=$(CC_$(COMP))
CCDEFS		:=$(CCDEFS_any) $(CCDEFS_$(COMP)) \
		  $(CCDEFS_$(BUILD)) $(CCDEFS_$(COMP)_$(BUILD))
CCOPTS		:=$(CCOPTS_any) $(CCOPTS_$(COMP)) \
		  $(CCOPTS_$(BUILD)) $(CCOPTS_$(COMP)_$(BUILD))

CCFLAGS		:=-c -DPROGRAMMER_INITIALS='"'"$$INITIALS"'"' \
		  -DASURA_BUILD_APP='"$(PROJ)"' \
		  -DASURA_BUILD_TYPE='"$(BUILD)"' \
		  -DASURA_BUILD_TIME='"$(shell date)"' \
		  -DASURA_BUILD_DEFS='"$(CCDEFS)"' \
		  -DASURA_BUILD_OPTS='"$(CCOPTS)"' \
		  $(INCP) $(CCDEFS) $(CCOPTS)

CXX		:=$(CXX_$(COMP))
CXXFLAGS	:=$(CCFLAGS) $(CXXFLAGS_$(COMP))

LD		:=$(LD_$(COMP))
LDOPTS		:=$(LDOPTS_$(COMP)) $(LDOPTS_$(BUILD)) \
		  $(LDOPTS_$(COMP)_$(BUILD))
LDFLAGS		:=$(LIBP) $(LDOPTS)

all:	$(EXED)/$(EXENAME).prx
#all:	$(EXED)/$(EXENAME).elf

.PHONY:		always

$(OBJD)/Asura_PSP_Core$(SFX): always

################################################################

LD_OBJS		:=\
		  $(OBJD)/main$(SFX)\
		  \
		  $(OBJD)/Asura_SpeedTest$(SFX)\
		  $(OBJD)/Asura_SpeedTest_MatrixVector3$(SFX)\
		  $(OBJD)/Asura_SpeedTest_MatrixVector4$(SFX)\
		  $(OBJD)/Asura_SpeedTest_Trig$(SFX)\
		  \
		  $(OBJD)/Asura_UnitTest$(SFX)\
		  $(OBJD)/Asura_UnitTest_Maths$(SFX)\
		  $(OBJD)/Asura_UnitTest_Maths_SoA$(SFX)\
		  $(OBJD)/Asura_UnitTest_Maths_Trig$(SFX)\
		  $(OBJD)/Asura_UnitTest_Maths_Vector4$(SFX)\
		  \
		  $(OBJD)/Asura_Maths$(SFX)\
		  $(OBJD)/Asura_Platform_Maths$(SFX)\
		  \
		  $(OBJD)/Asura_Timers$(SFX)\
		  $(OBJD)/Asura_Platform_Timers$(SFX)\
		  $(OBJD)/Asura_String$(SFX)\
		  $(OBJD)/Asura_Assert$(SFX)\
		  $(OBJD)/Asura_PlatformLogfile$(SFX)\
		  $(OBJD)/Asura_PlatformBasics$(SFX)

$(OBJD)/Asura_Maths$(SFX): $(ASURAI)/Core/Asura_Maths.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_String$(SFX): $(ASURAI)/Core/Asura_String.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_Timers$(SFX): $(ASURAI)/Core/Asura_Timers.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_SpeedTest$(SFX): $(ASURAI)/UnitTest/Asura_SpeedTest.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_SpeedTest_MatrixVector3$(SFX): $(ASURAI)/UnitTest/Asura_SpeedTest_MatrixVector3.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_SpeedTest_MatrixVector4$(SFX): $(ASURAI)/UnitTest/Asura_SpeedTest_MatrixVector4.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_SpeedTest_Trig$(SFX): $(ASURAI)/UnitTest/Asura_SpeedTest_Trig.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_UnitTest$(SFX): $(ASURAI)/UnitTest/Asura_UnitTest.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_UnitTest_Maths$(SFX): $(ASURAI)/UnitTest/Asura_UnitTest_Maths.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_UnitTest_Maths_SoA$(SFX): $(ASURAI)/UnitTest/Asura_UnitTest_Maths_SoA.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_UnitTest_Maths_Trig$(SFX): $(ASURAI)/UnitTest/Asura_UnitTest_Maths_Trig.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_UnitTest_Maths_Vector4$(SFX): $(ASURAI)/UnitTest/Asura_UnitTest_Maths_Vector4.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_PlatformBasics$(SFX): $(ASURAA)/Core/Asura_PlatformBasics.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_Platform_Maths$(SFX): $(ASURAA)/Core/Asura_Platform_Maths.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_Platform_Timers$(SFX): $(ASURAA)/Core/Asura_Platform_Timers.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_Assert$(SFX): $(ASURAA)/Debugging/Asura_Assert.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/Asura_PlatformLogfile$(SFX): $(ASURAA)/Debugging/Asura_PlatformLogfile.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

$(OBJD)/main$(SFX): $(GAMEA)/main.cpp
	$(CC) $(CXXFLAGS) "$<" -o "$@"

################################################################

-include $(OBJD)/*.d

ALLOBJS		:=$(OBJD)/$(EXENAME).ld \
		  -Wl,--start-group $(LIBS) -Wl,--end-group

clean:
	rm -f $(EXED)/$(EXENAME).prx $(EXED)/$(EXENAME).elf
	rm -f $(LD_OBJS) $(OBJD)/*.d

$(OBJD):
	mkdir -p "$@"

$(EXED):
	mkdir -p "$@"

$(OBJD)/$(EXENAME).ld: $(OBJD) psp_GCC.mak
	rm -f "$@"; for F in $(LD_OBJS); do echo "INPUT($$F)" >> "$@"; done

$(EXED)/$(EXENAME).prx : $(EXED) $(OBJD) $(LD_OBJS) $(OBJD)/$(EXENAME).ld
	$(LD) -startfiles $(LDFLAGS) $(ALLOBJS) -o "$@"

$(EXED)/$(EXENAME).elf : $(EXED) $(OBJD) $(LD_OBJS) $(OBJD)/$(EXENAME).ld
	$(LD) -startfiles $(LDFLAGS) $(ALLOBJS) --zgenelf -o "$@"

# ;;; Local Variables:		***
# ;;; indent-tabs-mode: t	***
# ;;; tab-width: 8		***
# ;;; tab-stop: 4		***
# ;;; End:			***
# vim:noet:ts=8:sw=4:nowrap
