//
// Asura_Task_Array.h
//
// Created by KJFL - 23 January 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_TASK_ARRAY_H__
#define __ASURA_TASK_ARRAY_H__

#ifdef ASURA_USE_TASK_SYSTEM

#include "Core/Threads/Asura_Task_System.h"

#include "Core/Asura_Multithread.h" // for mutex

// KJFL - 23 January 2008 - Asura_Task_Array. You supply an array size, and the number of array elements you'd
// like processed per Task function call. Then the worker threads will process a section of the array at a time,
// passing a start index and number of items to process to the Task function (you may end up with a "remainder"
// number of items to process at the end of the array. What work you actually do per "array item" is up to the 
// caller: the first use for this class by Tools_Visibility_System involves doing ray-tests vs the environment
// and storing hit data in an array which can be millions of entries long. 

class Asura_Task_Array : public Asura_Task
{
	public:	
		Asura_Task_Array( u_int uTotalArraySize, u_int uSuggestedNumberPerTask, Asura_TaskFunction pfnTask );
		~Asura_Task_Array();

		// KJFL - 24 January 2008 - I'm not entirely sold on the mechanism I've got here
		// for reusing data packets, but it's got to be better than newing and deleting
		// packets on the fly, which is what I did originally in the proof of concept.
		//
		// Task fns get a pointer to one of these packets which they can extra data from
		// and then call the SignalFinishedWithData() method to "return" the packet back
		// for future reuse. We only need as many packets as there are threads.
		class TaskDataPacket
		{
			public:
		
				// two accessors to get the data a worker thread needs
				u_int GetArrayStartIndex( void ) const { return m_uArrayStartIndex; }
				u_int GetNumberToProcess( void ) const { return m_uNumberToProcess; }
				
				// method to call when you've accessed the packet and no longer need it
				void  SignalFinishedWithData( void ) const { m_bFinishedWithData = true; }

			private: friend class Asura_Task_Array;

				// private constructor to keep people honest
				TaskDataPacket()
				: m_uArrayStartIndex(0)
				, m_uNumberToProcess(0)
				, m_bFinishedWithData(true)
				{}

				u_int			m_uArrayStartIndex;
				u_int			m_uNumberToProcess;
				mutable bool	m_bFinishedWithData;
		};

		// Get a pointer to a TaskDataPacket - called by worker threads. This will be a pointer
		// to a TaskDataPacket, as defined above. 
		void* GetTaskData( void );
	
		// Get the current array index - this does not do any locks to guarantee thread-safe 
		// ordering. and is designed to be used for simple things like progress bars etc.
		u_int GetCurrentIndex( void ) const { return m_uArrayIndex; }

		// Indicates when all the tasks have been assigned - not that all the tasks have finished
		// (the destructor of this class waits for all tasks to finish processing though).
		bool HasFinished( void ) const;

	private:

		u_int				m_uArraySize;
		u_int				m_uArrayIndex;
		u_int				m_uElementsPerTask;
		bool				m_bTerminateTasks;

		TaskDataPacket*		m_pxDataPackets;
		u_int				m_uNumberOfDataPackets;

		// TODO: may be able to get rid of this mutex
		Asura_Mutex			m_xAccessMutex;



};

#endif //#ifdef ASURA_USE_TASK_SYSTEM

#endif//__ASURA_TASK_ARRAY_H__

